#include "luaxml.h"
#include <string>
#include <sstream>
#include "babel.h"

const char* XMLATTR_TYPE = "t";
const char* XMLATTR_KEYTYPE = "kt";

//---------------------------------------------------------------------------
int luavar2xml(lua_State *L, int index, TiXmlElement *e)
{
    int luatype = lua_type(L, index);

    if(luatype == LUA_TNIL){
        e->SetAttribute(XMLATTR_TYPE, LUA_TNIL);
    }
    else if(luatype == LUA_TNUMBER){
        e->SetAttribute(XMLATTR_TYPE, LUA_TNUMBER);
        std::ostringstream oss;
        oss << lua_tonumber(L, index);
        TiXmlText text(oss.str().c_str());
        e->InsertEndChild(text);
    }
    else if(luatype == LUA_TBOOLEAN){
        e->SetAttribute(XMLATTR_TYPE, LUA_TBOOLEAN);
        std::ostringstream oss;
        oss << lua_toboolean(L, index);
        TiXmlText text(oss.str().c_str());
        e->InsertEndChild(text);
    }
    else if(luatype == LUA_TSTRING){
        e->SetAttribute(XMLATTR_TYPE, LUA_TSTRING);
        std::string sjis(lua_tostring(L, index));
        std::string utf8 = babel::sjis_to_utf8(sjis);
        TiXmlText text(utf8.c_str());
//        text.SetCDATA(true);
        e->InsertEndChild(text);
    }
    else if(luatype == LUA_TTABLE){
        e->SetAttribute("t", LUA_TTABLE);
        // e[uB
        lua_pushnil(L);
        while (lua_next(L, index) != 0) {
            // `L[L  -2A`lL  -1 ̈ʒuɂB
            // ɕϊ邽߂̒lX^bNŃRs[B
            lua_pushvalue(L, -2);
            // Gge[ũL[l̖Oō쐬B
            std::string key("_");
            key += lua_tostring(L, -1);
            TiXmlElement ne(key.c_str());
            // ɕϊꂽL[l̂ĂB
            lua_pop(L, 1);
            // L[̌^oĂB
            if(lua_isnumber(L, -2)){
                ne.SetAttribute(XMLATTR_KEYTYPE, LUA_TNUMBER);
            }else{
                ne.SetAttribute(XMLATTR_KEYTYPE, LUA_TSTRING);
            }
            // lċAŒHB
            luavar2xml(L, lua_gettop(L), &ne);
            e->InsertEndChild(ne);
            // `lLA`L[L ͎̌JԂ̂߂ɎcB
            lua_pop(L, 1);
        }
    }
    return 0;
}
//---------------------------------------------------------------------------
// index Ŏw肳ꂽʒũe[uɁAXML̃L[ƒl̃yAi[B
int xml2luavar(lua_State *L, int index, const TiXmlElement *e)
{
    int vt, kvt;
    // ϐ^Cv̓ǂݍ݁B
    if(e->QueryIntAttribute(XMLATTR_TYPE, &vt) != TIXML_SUCCESS){
        return -1;
    }
    // L[̕ϐ^Cv̓ǂݍ݁B
    if(e->QueryIntAttribute(XMLATTR_KEYTYPE, &kvt) != TIXML_SUCCESS){
        return -1;
    }
    const char *key = e->Value() + 1;
    if(kvt == LUA_TNUMBER){
        // L[l̏ꍇB
        int ki;
        if(sscanf(key, "%d", &ki) == 1){
            lua_pushnumber(L, ki);
        }else{
            return -1;
        }
    }
    else{
        // L[̏ꍇB
        lua_pushstring(L, key);
    }
    // nil ̏B
    if(vt == LUA_TNIL){
        lua_pushnil(L);
        lua_settable(L, index);
        return 0;
    }
    // ȍ~ŏϐ^CvɂĂ͎qƈӖȂB
    if(e->FirstChild() == NULL){
        lua_pop(L, 1);
        return 0;
    }
    if(e->FirstChild()->Type() == TiXmlNode::TEXT){
        // XJ[^ϐ̏B
        const TiXmlText *text = e->FirstChild()->ToText();
        if(vt == LUA_TNUMBER){
            double d;
            if(sscanf(text->Value(), "%lf", &d) == 1){
                lua_pushnumber(L, d);
                lua_settable(L, index);
            }
            else{
                // lǂݍ݂̎sBL[jB
                lua_pop(L, 1);
            }
        }
        else if(vt == LUA_TBOOLEAN){
            int i;
            if(sscanf(text->Value(), "%d", &i) == 1){
                lua_pushboolean(L, i);
                lua_settable(L, index);
            }
            else{
                // lǂݍ݂̎sBL[jB
                lua_pop(L, 1);
            }
        }
        else if(vt == LUA_TSTRING){
            std::string utf8(text->Value());
            std::string sjis = babel::utf8_to_sjis(utf8);
            lua_pushstring(L, sjis.c_str());
            lua_settable(L, index);
        }
        else{
            // lǂݍ݂̎sBL[jB
            lua_pop(L, 1);
        }
    }
    else if(vt == LUA_TTABLE){
        // e[u͍ċAB
        // Ve[uǉB
        lua_newtable(L);
        int tableindex = lua_gettop(L);
        for(const TiXmlElement *child = e->FirstChildElement();
            child != NULL; child = child->NextSiblingElement()){
            xml2luavar(L, tableindex, child);
        }
        lua_settable(L, index);
    }
    else{
        // \Ȃ^B͂BL[jB
        lua_pop(L, 1);
    }
    return 0;
}
