#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SDL.h"
#include "SDL_image.h"
#include "charbmp.h"
#include "jpstr.h"
#if defined(WIN32)
#include "chr_win32.h"
#endif

struct TCHR_FONT_BMP{
    SDL_Surface *surface;       /* W̃T[tFCX */
    SDL_Surface *surface_tmp;   /* `pꎞT[tFCX */
    int code_table[65536];      /* R[hƃp^[ԍ̕ϊe[u */
    int chr_w;                  /* iSṕj */
    int chr_h;                  /* c */
    int chr_xcount;             /* ɕł镶 */
};

/*<---------------------------------------------------------------------->*/
CHR_FONT_BMP* charbmp_alloc(void)
{
    CHR_FONT_BMP *font_bmp;

    font_bmp = (CHR_FONT_BMP*)malloc(sizeof(CHR_FONT_BMP));
    font_bmp->surface = NULL;
    font_bmp->surface_tmp = NULL;
    return font_bmp;
}
/*<---------------------------------------------------------------------->*/
/* charbmp_load()ŊmۂT[tFCX΁AB */
void charbmp_free(CHR_FONT_BMP *font_bmp)
{
    if(font_bmp->surface != NULL)
        SDL_FreeSurface(font_bmp->surface);
    if(font_bmp->surface_tmp != NULL)
        SDL_FreeSurface(font_bmp->surface_tmp);
    free(font_bmp);
}
/*<---------------------------------------------------------------------->*/
/* chrtbl.c ō쐬R[h\ƕ摜ǂݍށB
 * 摜t@C file_bmp ƃtHg font_name ͂ǂ炩w肷B
 * iw肷Ɖ摜t@CDj
 * tHgɂ镶摜͍̂ƂWindowsB
 */
int charbmp_load(CHR_FONT_BMP *font_bmp, const char *file_code, const char *file_bmp,
    int chr_w, int chr_h, const char *font_name)
{
    SDL_Surface *surface;
    Uint32 flag,key;
    FILE *fp;
    unsigned char c[2];
    int code, code_count=0;
    int *code_array;
    int i, j;

    fp = fopen(file_code, "rb");
    if(fp != NULL){
        code_array = (int *)malloc(sizeof(int)*65536);
        memset(font_bmp->code_table, 0, sizeof font_bmp->code_table);
        while(fread(c, sizeof(char), 2, fp) == sizeof(char)*2){
            code = (((int)c[0])<<8) | ((int)c[1]);
            font_bmp->code_table[code] = code_count;
            code_array[code_count] = code;
            code_count++;
        }
        fclose(fp);
    }
    else{
#if defined(WIN32)
        code_array = (int *)malloc(sizeof(int)*65536);
        /* pȂ */
//        for(i=0x20;i<=0x7E;i++){
//            code_array[code_count] = i;
//            code_count++;
//        }
#if defined(USE_UNICODE)
        for(i=0x0;i<=0xFFFF;i++){
            code_array[code_count] = i;
            code_count++;
        }
/*
        for(i=0x3040;i<=0x309F;i++){
            code_array[code_count] = i;
            code_count++;
        }
        for(i=0x30A0;i<=0x30FF;i++){
            code_array[code_count] = i;
            code_count++;
        }
        for(i=0x4E00;i<=0x9FFF;i++){
            code_array[code_count] = i;
            code_count++;
        }
*/
#else
        /* pJi */
        for(i=0xA1;i<=0xDF;i++){
            code_array[code_count] = i;
            code_count++;
        }
        for(i=0x81;i<=0x9F;i++){
            for(j=0x40;j<=0x7E;j++){
                code_array[code_count] = (i<<8)+j;
                code_count++;
            }
            for(j=0x80;j<=0xFC;j++){
                code_array[code_count] = (i<<8)+j;
                code_count++;
            }
        }
        for(i=0xE0;i<=0xFC;i++){
            for(j=0x40;j<=0x7E;j++){
                code_array[code_count] = (i<<8)+j;
                code_count++;
            }
            for(j=0x80;j<=0xFC;j++){
                code_array[code_count] = (i<<8)+j;
                code_count++;
            }
        }
        for(i=0; i<code_count; i++){
            font_bmp->code_table[code_array[i]] = i;
        }
#endif
#else
        return -1;
#endif
    }

    if(file_bmp != NULL){
        surface = IMG_Load(file_bmp);
    }
    else{
    /* t@Cw肳ĂȂAWindowsȂWindowspR[hŕ摜B */
#if defined(WIN32)
        /* ɑSp64ׂB */
        surface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCALPHA, chr_w*64, chr_h*((code_count+63)/64),
                32, 0x00ff0000, 0x0000ff00, 0x000000ff, 0x00000000);
        if(surface == NULL){
            fprintf(stderr, "%s:failed to create surface.\n", __FILE__);
            free(code_array);
            return -1;
        }
        if(GetStringSurface(surface, code_array, code_count, chr_h, font_name) == -1){
            SDL_FreeSurface(surface);
            free(code_array);
            return -1;
        }
#else
        free(code_array);
        return -1;
#endif
    }
    free(code_array);
    if(surface == NULL){
        return -1;
    }
    /* 𓧖FƂB */
    flag = SDL_SRCCOLORKEY;
    key = SDL_MapRGB(surface->format, 0, 0, 0);
    SDL_SetColorKey(surface, flag, key);
    font_bmp->surface = surface;
    font_bmp->chr_w = chr_w;
    font_bmp->chr_h = chr_h;
    font_bmp->chr_xcount = surface->w/chr_w;

    /* ꎞT[tFCX𓧖FƂB */
    /* J[}XNŒȂ͎̂蔲B */
    font_bmp->surface_tmp = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCALPHA, chr_w, chr_h,
            32, 0x00ff0000, 0x0000ff00, 0x000000ff, 0x00000000);
    SDL_SetColorKey(font_bmp->surface_tmp, flag, key);
    return 0;
}
/*<---------------------------------------------------------------------->*/
/* Ft̕`Brgba = 0xRRGGBBAA */
int charbmp_draw_char_rgba(CHR_FONT_BMP *font_bmp, SDL_Surface *dst, int x, int y, int code, Uint32 rgba)
{
    int index, srcx, srcy, bltw;
    Uint8 a_old;
    int ret;
    SDL_Rect srcrect, dstrect;
    SDL_Rect tmprect;

    if(font_bmp->surface == NULL)
        return -1;
    index = font_bmp->code_table[code];
    srcx = index%font_bmp->chr_xcount;
    srcy = index/font_bmp->chr_xcount;
    if(code <= 0xDF){
        bltw = font_bmp->chr_w/2;
    }else{
        bltw = font_bmp->chr_w;
    }
    srcrect.x = srcx*font_bmp->chr_w;
    srcrect.y = srcy*font_bmp->chr_h;
    srcrect.w = bltw;
    srcrect.h = font_bmp->chr_h;
    dstrect.x = x;
    dstrect.y = y;
    if(rgba == 0xffffffff){
        /* sȂ炻̂܂ܕ`B */
        return SDL_BlitSurface(font_bmp->surface, &srcrect, dst, &dstrect);
    }
    else if((rgba & 0xffffff00) == 0xffffff00){
        /* ̕`Ȃ烿lݒB */
        a_old = font_bmp->surface->format->alpha;
        SDL_SetAlpha(font_bmp->surface, SDL_SRCALPHA, rgba & 0xff);
        ret = SDL_BlitSurface(font_bmp->surface, &srcrect, dst, &dstrect);
        SDL_SetAlpha(font_bmp->surface, SDL_SRCALPHA, a_old);
        return ret;
    }
    else{
        /* 蔲FFt`悷邽߂ɁAꎞT[tFCXblitĂ܂blit */
        /* ꎞIɓFƂB */
        SDL_SetColorKey(font_bmp->surface, SDL_SRCCOLORKEY,
                SDL_MapRGB(font_bmp->surface->format, 255, 255, 255));
        /* ꎞT[tFCXwFœhԂB */
        tmprect.x = 0;
        tmprect.y = 0;
        tmprect.w = bltw;
        tmprect.h = font_bmp->chr_h;
        SDL_FillRect(font_bmp->surface_tmp, &tmprect,
                SDL_MapRGB(font_bmp->surface_tmp->format,
                           (rgba&0xff000000)>>24,
                           (rgba&0x00ff0000)>>16,
                           (rgba&0x0000ff00)>>8));
        /* ꎞT[tFCXɕ`āA{̕`ɂ܂`B */
        SDL_BlitSurface(font_bmp->surface, &srcrect, font_bmp->surface_tmp, &tmprect);
        SDL_SetColorKey(font_bmp->surface, SDL_SRCCOLORKEY,
                SDL_MapRGB(font_bmp->surface->format, 0, 0, 0));
        /* lݒB */
        SDL_SetAlpha(font_bmp->surface_tmp, SDL_SRCALPHA, rgba & 0xff);
        return SDL_BlitSurface(font_bmp->surface_tmp, &tmprect, dst, &dstrect);
    }
}
/*<---------------------------------------------------------------------->*/
/* ̒P`B */
int charbmp_draw_char(CHR_FONT_BMP *font_bmp, SDL_Surface *dst, int x, int y, int code)
{
    return charbmp_draw_char_rgba(font_bmp, dst, x, y, code, 0xffffffff);
}
/*<---------------------------------------------------------------------->*/
/* Ft̕`Brgba = 0xRRGGBBAA */
int charbmp_draw_string_rgba(CHR_FONT_BMP *font_bmp, SDL_Surface *dst, int x, int y, const char *str, Uint32 rgba)
{
    int index=0, chr1, chr2, code, dstx, dstw;

    dstx = x;
    for(;;){
        chr1 = (int)(unsigned char)str[index];
        if(chr1 == '\0'){
            break;
        }
#if defined(USE_UNICODE)
        index++;
        chr2 = (int)(unsigned char)str[index];
        //if(chr2 == '\0')
        //    break;
        code = (chr2<<8) + chr1;
        if(code < 0x80){
            dstw = font_bmp->chr_w/2;
        }else{
            dstw = font_bmp->chr_w;
        }
        index++;
#else
        if(is_sjis_2byte_1st(chr1)){
            index++;
            chr2 = (int)(unsigned char)str[index];
            if(chr2 == '\0')
                break;
            index++;
            code = (chr1<<8) | chr2;
            dstw = font_bmp->chr_w;
        }
        else{
            index++;
            code = chr1;
            dstw = font_bmp->chr_w/2;
        }
#endif

        if(charbmp_draw_char_rgba(font_bmp, dst, dstx, y, code, rgba) == -1)
            return -1;
        dstx += dstw;
    }
    return 0;
}
/*<---------------------------------------------------------------------->*/
/* ̒P`B */
int charbmp_draw_string(CHR_FONT_BMP *font_bmp, SDL_Surface *dst, int x, int y, const char *str)
{
    return charbmp_draw_string_rgba(font_bmp, dst, x, y, str, 0xffffffff);
}
/*<---------------------------------------------------------------------->*/
void charbmp_get_charsize(const CHR_FONT_BMP *font_bmp, int *w, int *h)
{
	*w = font_bmp->chr_w;
	*h = font_bmp->chr_h;
}
/*<---------------------------------------------------------------------->*/
