#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <exception>

#include <boost/scoped_array.hpp>
#include "gcnchar.h"
#include "jpstr.h"
#include "gcnopenglimagelite.h"

#if defined(WIN32)
#include "chr_win32.h"
#endif

//---------------------------------------------------------------------------
// array65536̂ƂB
bool LoadCharCode(const char *file, int *array, int *count)
{
    FILE *fp = fopen(file, "r");
    if(fp == NULL){
        return false;
    }

    unsigned char buf[32];
    while(fgets(reinterpret_cast<char*>(buf), sizeof buf, fp) != NULL){
        if(buf[0] < 0x80){
            array[*count] = buf[0];
        }
        else{
            array[*count] = ((static_cast<int>(buf[1]))<<8) + buf[0];
        }
        (*count)++;
    }
    fclose(fp);
    return true;
}
//---------------------------------------------------------------------------
TGcnCharDrawer::TGcnCharDrawer() :
    m_SDLSurface(NULL),
    m_chr_w(0),
    m_chr_h(0),
    m_SDLCharXCount(0),
    m_OpenGLCharCount(0),
    m_OpenGLCharXCount(0)
{
}
//---------------------------------------------------------------------------
TGcnCharDrawer::~TGcnCharDrawer()
{
    for(unsigned u=0; u<m_SurfaceList.size(); ++u){
        delete m_SurfaceList[u];
    }
    if(m_SDLSurface != NULL){
        SDL_FreeSurface(m_SDLSurface);
    }
}
//---------------------------------------------------------------------------
/* chrtbl.c ō쐬R[h\ƕ摜ǂݍށB
 * 摜t@C file_bmp ƃtHg font_name ͂ǂ炩w肷B
 * iw肷Ɖ摜t@CDj
 * tHgɂ镶摜͍̂ƂWindowsB
 */
bool TGcnCharDrawer::Load(const char *file_code, const char *file_bmp, int chr_w, int chr_h, const char *fontname)
{
    // d[hsB
    if(m_SDLSurface != NULL){
        m_LastError = "already loaded.";
        return false;
    }

    Uint32 flag,key;
    FILE *fp;
    int code, code_count=0;

    if(file_code == NULL){
        file_code = "";
    }

    char pngfile[256];
    if(file_bmp != NULL){
        strcpy(pngfile, file_bmp);
    }
    else{
        sprintf(pngfile, "char%d.png", chr_h);
    }

    boost::scoped_array<int> code_array(new int[65536]);

    fp = fopen(file_code, "r");
    if(fp != NULL){
        char buf[32];
        while(fgets(buf, sizeof buf, fp) != NULL){
            int c1 = static_cast<unsigned char>(buf[0]);
            int c2 = static_cast<unsigned char>(buf[1]);
            if(c1 < 0x80){
                code_array[code_count++] = c1;
            }
            else{
                code_array[code_count++] = (c1<<8) + c2;
            }
        }
        fclose(fp);
        for(int i=0; i<code_count; ++i){
            m_code_table[code_array[i]] = i;
        }
    }
    else{
#if defined(WIN32)
        int i, j;
        // pȂ
        for(i=0x20;i<=0x7E;i++){
            code_array[code_count] = i;
            code_count++;
        }
        // pJi
        for(i=0xA1;i<=0xDF;i++){
            code_array[code_count] = i;
            code_count++;
        }
        for(i=0x81;i<=0x9F;i++){
            for(j=0x40;j<=0x7E;j++){
                code_array[code_count] = (i<<8)+j;
                code_count++;
            }
            for(j=0x80;j<=0xFC;j++){
                code_array[code_count] = (i<<8)+j;
                code_count++;
            }
        }
        for(i=0xE0;i<=0xFC;i++){
            for(j=0x40;j<=0x7E;j++){
                code_array[code_count] = (i<<8)+j;
                code_count++;
            }
            for(j=0x80;j<=0xFC;j++){
                code_array[code_count] = (i<<8)+j;
                code_count++;
            }
        }
        for(i=0; i<code_count; i++){
            m_code_table[code_array[i]] = i;
        }
#else
        return false;
#endif
    }

    bool regenerate = false;
    m_SDLSurface = IMG_Load(pngfile);
    if(m_SDLSurface == NULL){
        regenerate = true;
    }
    int needw = chr_w*64;
    int needh = chr_h*((code_count+63)/64);
    if(!regenerate && (needw != m_SDLSurface->w || needh != m_SDLSurface->h)){
        SDL_FreeSurface(m_SDLSurface);
        regenerate = true;
    }
    if(regenerate){
        // [hs܂̓TCYႤAWindowsȂWindowspR[hŕ摜B
#if defined(WIN32)
        if(GenerateChrPng(code_array.get(), code_count, needw, needh, chr_h, fontname, pngfile) == -1){
            m_LastError = "failed to generate font image file(png).";
            return false;
        }
        m_SDLSurface = IMG_Load(pngfile);
        if(m_SDLSurface == NULL){
            m_LastError = "failed to IMG_Load";
            return false;
        }
#else
        return false;
#endif
    }
    // SDL_SurfaceɂẴTCYi[B
    m_chr_w = chr_w;
    m_chr_h = chr_h;
    m_SDLCharXCount = m_SDLSurface->w / chr_w;

    // őeNX`TCY擾B
    int max_texture_size;
    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &max_texture_size);
    // 悭mȂ̂܂܍őTCYł͂Ȃۂ͔̏炢炵B
    max_texture_size /= 2;

    // T[tFCXꖇ̏cɕׂ鐔BiőTCY̕傫Ɖj
    int xcount = max_texture_size / chr_w;
    int ycount = max_texture_size / chr_h;
    // T[tFCXꖇɊi[\ȕB
    int surface_chars = xcount * ycount;
    // KvȃT[tFCXB
    int surface_count = (code_count + surface_chars - 1) / surface_chars;
    m_SurfaceList.reserve(surface_count);

    int pixelbytes = m_SDLSurface->format->BytesPerPixel;
    unsigned pixeltmp[2] = {0, 0xffffffff};
    int charindex = 0;
    // OpenGLT[tFCX1̃sNZzB
    int glsw = max_texture_size;
    int glsh = max_texture_size;
    boost::scoped_array<unsigned> pixels(new unsigned[glsw * glsh]);
    for(int si=0; si<surface_count; ++si){
        for(int ci=0; ci<surface_chars; ++ci){
            if(code_count <= charindex){
                // ŌOpenGLT[tFCX̌̕ɃS~邾낤͋CɂȂi`悳邱Ƃ͖jB
                continue;
            }
            // ]SDL_Surfacë̕ʒuB
            int srcx = (charindex % m_SDLCharXCount) * chr_w;
            int srcy = (charindex / m_SDLCharXCount) * chr_h;
            // ]OpenGLT[tFCẌ̕ʒuB
            int dstx = (ci % xcount) * chr_w;
            int dsty = (ci / xcount) * chr_h;
            for(int py=0; py<chr_h; ++py){
                unsigned char *psrc = (unsigned char*)m_SDLSurface->pixels + m_SDLSurface->pitch*(srcy+py) + srcx*pixelbytes;
                unsigned *pdst = pixels.get() + dsty * glsw + dstx;
                for(int px=0; px<chr_w; ++px){
                    *pdst = pixeltmp[psrc[0] != 0];
                    ++pdst;
                    psrc += pixelbytes;
                }
                ++dsty;
            }
            ++charindex;
        }
        gcn::OpenGLImage *newimg = NULL;
        try{
            newimg = new gcn::OpenGLImage(pixels.get(), glsw, glsh);
        }catch(gcn::Exception& e){
        }
        if(newimg == NULL){
            m_LastError = "failed to new gcn::OpenGLImage";
            return false;
        }
        //newimg->convertToDisplayFormat();
        m_SurfaceList.push_back(newimg);
    }
    // OpenGLT[tFCXɂẴTCYi[B
    m_OpenGLCharCount = surface_chars;
    m_OpenGLCharXCount = xcount;
    return true;
}
//---------------------------------------------------------------------------
void TGcnCharDrawer::DrawChar(gcn::Graphics *g, int x, int y, int code, const gcn::Color& c)
{
    //DrawCharARGB(g, x, y, code, c.a<<24 | c.r<<16 | c.g<<8 | c.b);
    // ۂɂABGRɂȂĂB
    DrawCharARGB(g, x, y, code, c.a<<24 | c.b<<16 | c.g<<8 | c.r);
}
//---------------------------------------------------------------------------
// Ft̕`Bargb = 0xAABBGGRR
void TGcnCharDrawer::DrawCharARGB(gcn::Graphics *g, int x, int y, int code, Uint32 argb)
{
    if(m_SDLSurface == NULL){
        return;
    }

    int index = m_code_table[code];

    gcn::OpenGLImage *surface = m_SurfaceList[index / m_OpenGLCharCount];
    index %= m_OpenGLCharCount;
    int srcx = (index % m_OpenGLCharXCount) * m_chr_w;
    int srcy = (index / m_OpenGLCharXCount) * m_chr_h;
    if(argb == 0xffffffff){
        GLfloat blend[4] = {1.0, 1.0, 1.0, 1.0};
        DrawOpenGLTexture(reinterpret_cast<gcn::OpenGLGraphics*>(g), surface, srcx, srcy, x, y, m_chr_w, m_chr_h, blend);
    }
    else{
        GLfloat blend[4];
        blend[0] = static_cast<GLfloat>(argb & 0x000000ff) / 255;
        blend[1] = static_cast<GLfloat>((argb & 0x0000ff00)>>8) / 255;
        blend[2] = static_cast<GLfloat>((argb & 0x00ff0000)>>16) / 255;
        blend[3] = static_cast<GLfloat>((argb & 0xff000000)>>24) / 255;
        DrawOpenGLTexture(reinterpret_cast<gcn::OpenGLGraphics*>(g), surface, srcx, srcy, x, y, m_chr_w, m_chr_h, blend);
    }
}
//---------------------------------------------------------------------------
bool TGcnCharDrawer::DrawString(gcn::Graphics *g, int x, int y, const char *str, const gcn::Color& c)
{
    //return DrawStringARGB(g, x, y, str, c.a<<24 | c.r<<16 | c.g<<8 | c.b);
    // ۂɂABGRɂȂĂB
    return DrawStringARGB(g, x, y, str, c.a<<24 | c.b<<16 | c.g<<8 | c.r);
}
//---------------------------------------------------------------------------
// Ft̕`Bargb = 0xAARRGGBB
bool TGcnCharDrawer::DrawStringARGB(gcn::Graphics *g, int x, int y, const char *str, Uint32 argb)
{
    if(str[0] == '\0'){
        return true;
    }

    int index=0, chr1, chr2, code, dstx, dsty, dstw;
    int dsttotalw = strlen(str) * (m_chr_w / 2);

    dstx = x;
    dsty = y;
    for(;;){
        chr1 = (int)(unsigned char)str[index];
        if(chr1 == '\0'){
            break;
        }
        if(is_sjis_2byte_1st(chr1)){
            ++index;
            chr2 = (int)(unsigned char)str[index];
            if(chr2 == '\0')
                break;
            ++index;
            code = (chr1<<8) | chr2;
            dstw = m_chr_w;
        }
        else{
            ++index;
            code = chr1;
            dstw = m_chr_w/2;
        }

        DrawCharARGB(g, dstx, dsty, code, argb);
        dstx += dstw;
    }
    static const GLfloat blend[4] = {1.0, 1.0, 1.0, 1.0};
    glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, blend);
    return true;
}
//---------------------------------------------------------------------------
void TGcnCharDrawer::DrawStringOnRect(gcn::Graphics *g, int x, int y, const gcn::Rectangle& r, int yspacing, const char *str, const gcn::Color& c)
{
    //return DrawStringOnRectARGB(g, x, y, r, yspacing, str, c.a<<24 | c.r<<16 | c.g<<8 | c.b);
    // ۂɂABGRɂȂĂB
    return DrawStringOnRectARGB(g, x, y, r, yspacing, str, c.a<<24 | c.b<<16 | c.g<<8 | c.r);
}
//---------------------------------------------------------------------------
void TGcnCharDrawer::DrawStringOnRectARGB(gcn::Graphics *g, int x, int y, const gcn::Rectangle& r, int yspacing, const char *str, Uint32 argb)
{
    //int index=0, chr1, chr2, code, dstx, dsty, dstw;
    int index=0, chr1, chr2, dstx, dsty, dstw;
    std::string s;

    dstx = x;
    dsty = y;
    bool loopflag = true;
    while(loopflag){
        chr1 = (int)(unsigned char)str[index];
        bool flush = false;
        if(chr1 == '\0'){
            flush = true;
            loopflag = false;
        }
        else if(chr1 == '\n'){
            ++index;
            flush = true;
        }
        else{
            int oldindex = index;
            if(is_sjis_2byte_1st(chr1)){
                dstw = m_chr_w;
                if(dstx + dstw <= x + r.width){
                    ++index;
                    chr2 = (int)(unsigned char)str[index];
                    if(chr2 == '\0'){
                        // ُn
                        break;
                    }
                    ++index;
                    s += chr1;
                    s += chr2;
                    dstx += dstw;
                }
                else{
                    flush = true;
                }
            }
            else{
                dstw = m_chr_w/2;
                if(dstx + dstw <= x + r.width){
                    ++index;
                    s += chr1;
                    dstx += dstw;
                }
                else{
                    flush = true;
                }
            }
        }

        if(flush){
            DrawStringARGB(g, x, dsty, s.c_str(), argb);
            dstx = x;
            dsty += m_chr_h + yspacing;
            s.clear();
        }
    }
}
//---------------------------------------------------------------------------
void TGcnCharDrawer::GetCharSize(int *w, int *h) const
{
    *w = m_chr_w;
    *h = m_chr_h;
}
//---------------------------------------------------------------------------
void TGcnCharDrawer::BltCharARGB(int code, unsigned *dstpixels, int dstx, int dsty, int dstw, Uint32 argb)
{
    int index = m_code_table[code];
    int srcx = (index%m_SDLCharXCount)*m_chr_w;
    int srcy = (index/m_SDLCharXCount)*m_chr_h;
    int srcw = m_chr_w;
    int srch = m_chr_h;

    int pixelbytes = m_SDLSurface->format->BytesPerPixel;
    for(int py=0; py<srch; ++py){
        int i = (dsty + py) * dstw + dstx;
        unsigned char *psrc = (unsigned char*)m_SDLSurface->pixels + m_SDLSurface->pitch*(srcy+py) + srcx*pixelbytes;
        for(int px=0; px<srcw; ++px){
            if(psrc[0]){
                dstpixels[i] = argb;
            }
            ++i;
            psrc += pixelbytes;
        }
    }
}
//---------------------------------------------------------------------------
// Bybold͖Ƃ1ŒBborderw,borderh0-1̎B
gcn::Image* TGcnCharDrawer::GenerateRichFontImage(const char *str, int xbold, int ybold, Uint32 argb, int borderw, int borderh, Uint32 argb_border)
{
    if(m_SDLSurface ==NULL){
        return NULL;
    }
    int chrs = strlen(str);
    if(!chrs){
        return NULL;
    }
    int chrw = m_chr_w + xbold + borderw*2;
    int chrh = m_chr_h + ybold + borderh*2;
    int imgw = chrw * chrs;

    boost::scoped_array<unsigned> pixels(new unsigned[imgw * chrh]);
    memset(pixels.get(), 0, imgw * chrh * sizeof(unsigned));

    int dstx = 0;
    int dsty = 0;
    int index = 0;
    int chr1, chr2, dstw, code;
    for(;;){
        chr1 = (int)(unsigned char)str[index];
        if(chr1 == '\0' || chr1 == '\n'){
            break;
        }
        if(is_sjis_2byte_1st(chr1)){
            ++index;
            chr2 = (int)(unsigned char)str[index];
            if(chr2 == '\0')
                break;
            ++index;
            code = (chr1<<8) | chr2;
            dstw = m_chr_w;
        }
        else{
            ++index;
            code = chr1;
            dstw = m_chr_w/2;
        }

        for(int bw = 0; bw < borderw; ++bw){
            BltCharARGB(code, pixels.get(), dstx + bw, borderh, imgw, argb_border);
            BltCharARGB(code, pixels.get(), dstx + borderw + xbold + bw, borderh, imgw, argb_border);
        }
        for(int bh = 0; bh < borderh; ++bh){
            BltCharARGB(code, pixels.get(), dstx + borderw, bh, imgw, argb_border);
            BltCharARGB(code, pixels.get(), dstx + borderw, borderh + ybold + bh, imgw, argb_border);
        }
        for(int x = 0; x < xbold; ++x){
            BltCharARGB(code, pixels.get(), dstx + borderw + x, borderh, imgw, argb);
        }

        dstx += dstw;
    }
    gcn::OpenGLImage *newimg = new gcn::OpenGLImage(pixels.get(), imgw, chrh);
    return newimg;
}
//---------------------------------------------------------------------------
void TGcnCharDrawer::DrawOpenGLTexture(gcn::OpenGLGraphics *g, const gcn::OpenGLImage* srcImage, int srcX, int srcY, int dstX, int dstY, int width, int height, const GLfloat *blend)
{
    const gcn::ClipRectangle& top = g->getCurrentClipArea();
    bool alpha = (g->getColor().a != 255); // gcn::OpenGLGraphics ̃\[X̒gɈˑB܂ɘjĕςȂ낤B

    dstX += top.xOffset;
    dstY += top.yOffset;

    // Find OpenGL texture coordinates
    float texX1 = srcX / (float)srcImage->getTextureWidth();
    float texY1 = srcY / (float)srcImage->getTextureHeight();
    float texX2 = (srcX+width) / (float)srcImage->getTextureWidth();
    float texY2 = (srcY+height) / (float)srcImage->getTextureHeight();

    glBindTexture(GL_TEXTURE_2D, srcImage->getTextureHandle());

    glEnable(GL_TEXTURE_2D);

    if(!alpha){
        glEnable(GL_BLEND);
    }

    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
    glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, blend);

    // Draw a textured quad -- the image
    glBegin(GL_QUADS);
    glTexCoord2f(texX1, texY1);
    glVertex3i(dstX, dstY, 0);

    glTexCoord2f(texX1, texY2);
    glVertex3i(dstX, dstY + height, 0);

    glTexCoord2f(texX2, texY2);
    glVertex3i(dstX + width, dstY + height, 0);

    glTexCoord2f(texX2, texY1);
    glVertex3i(dstX + width, dstY, 0);
    glEnd();

    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
    glDisable(GL_TEXTURE_2D);

    if(!alpha){
        glDisable(GL_BLEND);
    }
}
