#include "gcnopenglimagelite.h"
#include <guichan/exception.hpp>
#include <sstream>

TGcnOpenGLImageLite::TGcnOpenGLImageLite(const unsigned int* pixels, int width, int height, bool convertToDisplayFormat)
    : gcn::OpenGLImage(GLuint(0), width, height, true)
{
    bool tmpbuf = false;
    if(mTextureWidth == mWidth && mTextureHeight && mHeight){
        mPixels = const_cast<unsigned*>(pixels);
    }
    else{
        tmpbuf = true;
        mPixels = new unsigned[mTextureWidth * mTextureHeight];
		int x, y;
		for (y = 0; y < mTextureHeight; y++)
		{
			for (x = 0; x < mTextureWidth; x++)
			{
				if (x < mWidth && y < mHeight)
				{
					mPixels[x + y * mTextureWidth] = pixels[x + y * mWidth];
				}
				else
				{
					mPixels[x + y * mTextureWidth] = 0x00000000;
				}
			}
		}
    }
    try{
        TGcnOpenGLImageLite::convertToDisplayFormat();
    }catch(const gcn::Exception& e){
        if(tmpbuf){
            delete[] mPixels;
        }
        mPixels = NULL;
        throw e;
    }

    if(tmpbuf){
        delete[] mPixels;
    }
    mPixels = NULL;
}

TGcnOpenGLImageLite::~TGcnOpenGLImageLite()
{
}

gcn::Color TGcnOpenGLImageLite::getPixel(int x, int y)
{
    return gcn::Color(0, 0, 0, 0);
}

void TGcnOpenGLImageLite::putPixel(int x, int y, const gcn::Color& color)
{
}

void TGcnOpenGLImageLite::convertToDisplayFormat()
{
    if(mPixels == NULL)
    {
        throw GCN_EXCEPTION("Image has already been converted to display format");
    }

    glGetError();
    glGenTextures(1, &mTextureHandle);
    GLenum error_tex = glGetError();
    if(error_tex){
        std::stringstream ss;
        ss << "falied to glGenTextures. glGetError = " << error_tex;
        throw GCN_EXCEPTION(ss.str());
    }
    glBindTexture(GL_TEXTURE_2D, mTextureHandle);

    glTexImage2D(GL_TEXTURE_2D,
                 0,
                 4,
                 //GL_RGBA,
                 mTextureWidth,
                 mTextureHeight,
                 0,
                 GL_RGBA,
                 GL_UNSIGNED_BYTE,
                 mPixels);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

    GLenum error = glGetError();
    if (error)
    {
        std::string errmsg;
        switch (error)
        {
          case GL_INVALID_ENUM:
              errmsg = "GL_INVALID_ENUM";
              break;

          case GL_INVALID_VALUE:
              errmsg = "GL_INVALID_VALUE";
              break;

          case GL_INVALID_OPERATION:
              errmsg = "GL_INVALID_OPERATION";
              break;

          case GL_STACK_OVERFLOW:
              errmsg = "GL_STACK_OVERFLOW";
              break;

          case GL_STACK_UNDERFLOW:
              errmsg = "GL_STACK_UNDERFLOW";
              break;

          case GL_OUT_OF_MEMORY:
              errmsg = "GL_OUT_OF_MEMORY";
              break;
        }

        throw GCN_EXCEPTION(std::string("Unable to convert to OpenGL display format, glGetError said: ") + errmsg);
    }
}
