#include <vcl.h>
#include <io.h>
#include <cstdarg>
#include <ctime>
#include <exception>
#include <boost/filesystem/path.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/scoped_ptr.hpp>
#include <babel.h>
#include <imgctl.h>
#include "cnv.h"
#include "farentxt.h"
#include "scriptcnv.h"
#include "convertpicture.h"
#include "imgctlutil.h"
#include "FarencnvForm.h"   // vOXo[XV邽߁B

const int MAPGNDTYPE_MAX =16;
const int MAPOBJTYPE_MAX = 5;
const int MOVETYPE_MAX  = 10;
const int ATTRIBUTE_MAX = 18;           // IWiɂ鑮̐B
const int ATTRIBUTE_FULLMAX = ATTRIBUTE_MAX + 22;  // ǉ̐B
const int MAGICGROUP_MAX = 6;
const int MAGIC_MAX = 8;
const int MAGICRANK_MAX = 5;
const int ATTRRES_MAX   = 4;
const int SCREENCENTERANIM_WIDTH = 128; // NCGƂ̒ɕ\Aj̃TCYB
const char* XMLCHARCODE = "utf-8";
const char* NECROMANCY_SPECIAL = "앜";
const char* REVIVE_NECROMANCY = "Necromancy";
const char* REVIVE_ME = "ReviveMe";
const char* MASTER_REVIVAL = "MasterRevival";
const char* MASTER_INHERIT = "MasterInherit";

const char *mapgnd_name[MAPGNDTYPE_MAX] =
{
"gp1", "n", "", "", "", "", "C", "[C", "rn", "ጴ",
"", "Rn", "Ό", "n", "X", "gp2"
};

const char *mapobj_name[MAPOBJTYPE_MAX] = {"gp", "X", "", "H", ""};

const char *movetype_name[MOVETYPE_MAX] = {"^Cv", "ݑ^Cv", "C^Cv", "^Cv", "n^Cv", "Rn^Cv", "Xу^Cv", "ጴ^Cv", "Rn^Cv", "s^Cv"};

const char *magicgroup_name[MAGICGROUP_MAX] = {"", "", "", "y", "", ""};

const char *attr_name[ATTRIBUTE_FULLMAX] =
{
"Ȃ", "ʏU",
"ōU", "ΉU", "ჍU", "U", "zU", "U", "zU", "_U",
"Ζ@", "@", "@", "y@", "@", "Ŗ@",
"HP", "MP",     // ܂Ō̑̕тƓB
// ͒ǉŁAo͂BIWiɂ͑Ή鑮͖B
"ǔj", "UZ", "hZ", "ZʉZ", "xZ", "͉Z", "RZ",
"HP񕜉Z", "MP񕜉Z", "U񐔉Z", "U񐔌",
"Ŏ", "Ή", "჎","", "z", "",
"RύX", "ړ^CvύX", "", "", "ϐ"
};

const char *attr_shortname[ATTRIBUTE_FULLMAX] =
{
"Ȃ", "ʏ",
"", "Ή", "", "", "z", "", "z", "_",
"", "", "", "y", "", "",
"HP", "MP",     // ܂Ō̑̕тƓB
"ǔj", "U", "h", "Z", "", "", "",
"HP", "MP", "U", "U",
"Ŏ", "Ύ", "","", "", "",
"", "ڕ", "", "", "ϖ"
};

enum TAttributeType{
    ATTRTYPE_NONE,
    ATTRTYPE_ATTACK,        // GɓZ
    ATTRTYPE_BOOST,         // ɓZ
    ATTRTYPE_CRASHWALL,     // ǂɓZ
    ATTRTYPE_SUMMON,
    ATTRTYPE_REVIVE,        // gȂi݂邾ĵ̑^CvłȂƂӖŁB
    ATTRTYPE_MAX
};
const char *attr_type_name[ATTRTYPE_MAX] = {"None", "Attack", "Boost", "CrashWall", "Summon", "Revive"};

const int attr_type[ATTRIBUTE_FULLMAX] =
{
ATTRTYPE_NONE, ATTRTYPE_ATTACK,
ATTRTYPE_ATTACK, ATTRTYPE_ATTACK, ATTRTYPE_ATTACK, ATTRTYPE_ATTACK, ATTRTYPE_ATTACK, ATTRTYPE_ATTACK, ATTRTYPE_ATTACK,
ATTRTYPE_ATTACK, ATTRTYPE_ATTACK, ATTRTYPE_ATTACK, ATTRTYPE_ATTACK, ATTRTYPE_ATTACK, ATTRTYPE_ATTACK, ATTRTYPE_ATTACK,
ATTRTYPE_BOOST, ATTRTYPE_BOOST,
ATTRTYPE_CRASHWALL, ATTRTYPE_BOOST, ATTRTYPE_BOOST, ATTRTYPE_BOOST, ATTRTYPE_BOOST, ATTRTYPE_BOOST, ATTRTYPE_BOOST,
ATTRTYPE_BOOST, ATTRTYPE_BOOST, ATTRTYPE_BOOST, ATTRTYPE_ATTACK,
ATTRTYPE_BOOST, ATTRTYPE_BOOST, ATTRTYPE_BOOST, ATTRTYPE_BOOST, ATTRTYPE_BOOST, ATTRTYPE_BOOST,
ATTRTYPE_BOOST, ATTRTYPE_BOOST, ATTRTYPE_SUMMON, ATTRTYPE_REVIVE, ATTRTYPE_ATTACK
};

const char *attr_id[ATTRIBUTE_FULLMAX] =
{
"None", "Physical",
"Poison", "Stone", "Paralyze", "Sleep", "Dream", "Death", "Absorb",
"Holy", "Fire", "Water", "Wind", "Earth", "Light", "Darkness",
"HPRecover", "MPRecover",
"CrashWall", "PlusPhysicalAttack", "PlusPhysicalDefense", "PlusSkill", "PlusSpeed", "PlusMagicAttack", "PlusMagicDefense",
"PlusHPRecover", "PlusMPRecover", "PlusAttackCount", "MinusAttackCount",
"CurePoison", "CureStone", "CurePralyze", "CureSleep", "CureDream", "CureAll",
"ChangeResistance", "ChangeMoveType", "Summon", "Revive", "CantGuard"
};

// ̒萔BRo[^Ŏĝł{̂Ƃ͊֌WȂi܂ܓтjB
// @̑n[hR[fBOŏo͂̂ɎgB
enum TAttributeIndex
{
    ATTRIBUTE_NONE,
    ATTRIBUTE_PHYSICAL,
    ATTRIBUTE_POISON,
    ATTRIBUTE_STONE,
    ATTRIBUTE_PARALYZE,
    ATTRIBUTE_SLEEP,
    ATTRIBUTE_DREAM,
    ATTRIBUTE_DEATH,
    ATTRIBUTE_ABSORB,
    ATTRIBUTE_HOLY,
    ATTRIBUTE_FIRE,
    ATTRIBUTE_WATER,
    ATTRIBUTE_WIND,
    ATTRIBUTE_EARTH,
    ATTRIBUTE_LIGHT,
    ATTRIBUTE_DARK,
    ATTRIBUTE_HPRECOVER,
    ATTRIBUTE_MPRECOVER,
    ATTRIBUTE_CRASHWALL,    // ͒ǉB
    ATTRIBUTE_PLUS_PA,
    ATTRIBUTE_PLUS_PD,
    ATTRIBUTE_PLUS_SKILL,
    ATTRIBUTE_PLUS_SPEED,
    ATTRIBUTE_PLUS_MA,
    ATTRIBUTE_PLUS_MD,
    ATTRIBUTE_PLUS_HPRECOVER,
    ATTRIBUTE_PLUS_MPRECOVER,
    ATTRIBUTE_PLUS_ATTACKCOUNT,
    ATTRIBUTE_MINUS_ATTACKCOUNT,
    ATTRIBUTE_CURE_POISON,
    ATTRIBUTE_CURE_STONE,
    ATTRIBUTE_CURE_PARALYZE,
    ATTRIBUTE_CURE_SLEEP,
    ATTRIBUTE_CURE_DREAM,
    ATTRIBUTE_CURE_ALL,
    ATTRIBUTE_CHANGE_RESISTANCE,
    ATTRIBUTE_CHANGE_MOVETYPE,
    ATTRIBUTE_SUMMON,
    ATTRIBUTE_REVIVE
};

enum TRace
{
    RACE_HUMAN,
    RACE_LIZARDMAN,
    RACE_GOBLIN,
    RACE_ELF,
    RACE_DWARF,
    RACE_DEVIL,
    RACE_ICEMAN,
    RACE_MONSTER,
    RACE_MAX
};
//const char *race_id[RACE_MAX] = {"human", "lizardman", "goblin", "elf", "dwarf", "devil", "iceman", "monster"};
const char *race_name[RACE_MAX] = {"l", "U[h}", "Su", "Gt", "h[t", "", "ACX}", "X^["};

const char *attrres_id[ATTRRES_MAX] = {"normal", "weak", "strong", "absorb"};

const char *TFarenCnv::AnimationTypeID[ANIMATION_MAX] = {"", "Chip", "Center", "Stream", "Shot", "ScreenCenter"};

const char *TFarenCnv::RangeTypeID[RANGE_MAX] = {"", "Normal", "Me", "Stream",
    "AllMySide", "AllEnemy", "AllWall", "LastDeadMySide", "LastDeadEnemy", "AllDeadMySide", "AllDeadEnemy", "LowerThan"};

const char *TFarenCnv::PowerTypeID[POWERTYPE_MAX] = {"None", "PhysicalAttack", "PhysicalDefense", "MagicAttack", "MagicDefense", "Skill"};

const char *TFarenCnv::FormulaID[FORMULA_MAX] = {"None", "FTAttack", "FTSpecial", "FTMagic", "Anytime", "FTAttackHit", "FTSpecialHit", "FTMagicHit"};

const char *TFarenCnv::HitJudgeTypeID[FORMULA_MAX] = {"None", "SingleJudge", "DoubleJudge", "SeparateJudge"};

const int UsableMagicLevel[MAGIC_MAX][MAGICRANK_MAX] =
{
    {1,2,3,3,3}, {1,2,3,3,3}, {0,1,1,2,2}, {0,1,1,2,2}, {0,0,1,1,2}, {0,0,1,1,2}, {0,0,0,1,1}, {0,0,0,0,1}
};

//const char *CNVSRCDIR = "cnvsrc";

FILE *fp_log;

//---------------------------------------------------------------------------
bool operator==(const TConvertedChara& lhs, const TConvertedChara& rhs){
    return lhs.id == rhs.id;
}
//---------------------------------------------------------------------------
void InsertTextChild(TiXmlElement& e, const char* str)
{
    e.InsertEndChild(TiXmlText(babel::sjis_to_utf8(str).c_str()));
}
//---------------------------------------------------------------------------
void SetTextAttribute(TiXmlElement& e, const char* name, const char* str)
{
    e.SetAttribute(name, babel::sjis_to_utf8(str).c_str());
}
//---------------------------------------------------------------------------
// ̗eSpXy[X菜B
std::string CutZenSpace(const std::string& src)
{
    if(src.empty()){
        return "";
    }
    unsigned sz = src.size();
    unsigned u1, u2;
    for(u1=0; u1<sz-1; ++u1){
        if(static_cast<unsigned char>(src[u1]) == 0x81 && src[u1 + 1] == 0x40){
            ++u1;
        }
        else{
            break;
        }
    }
    for(u2=sz-1; u2>0; --u2){
        if(static_cast<unsigned char>(src[u2 - 1]) == 0x81 && src[u2] == 0x40){
            --u2;
        }
        else{
            break;
        }
    }
    std::string dst;
    for(unsigned u=u1; u<=u2; ++u){
        dst.push_back(src[u]);
    }
    return dst;
}
//---------------------------------------------------------------------------
void OutputLog(char *fmt, ...)
{
    using namespace std;
    va_list argp;
    time_t t;
    char tm_str[32];

    t = time(NULL);
    strftime(tm_str, sizeof(tm_str), "%y/%m/%d %H:%M:%S", localtime(&t));
    fprintf(fp_log, "%s  ", tm_str);

    va_start(argp, fmt);
    vfprintf(fp_log, fmt, argp);
    va_end(argp);
    fprintf(fp_log, "\n");
    fflush(fp_log);
}
//---------------------------------------------------------------------------
HDIB CreateAllZeroDIB(int width, int height)
{
    BITMAPINFO bi;
    ZeroMemory(&bi, sizeof bi);
    bi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    bi.bmiHeader.biWidth = width;
    bi.bmiHeader.biHeight = height;
    bi.bmiHeader.biPlanes = 1;
    bi.bmiHeader.biBitCount = 24;
    bi.bmiHeader.biCompression = BI_RGB;
    bi.bmiHeader.biSizeImage = (bi.bmiHeader.biWidth + 7) / 8 * bi.bmiHeader.biHeight * bi.bmiHeader.biBitCount;
    // f[^I[0DIB쐬
    return CreateDIB(&bi, NULL);
}
//---------------------------------------------------------------------------
TFarenCnv::TFarenCnv() :
    m_LogFileName("log.txt")
{
}
//---------------------------------------------------------------------------
bool TFarenCnv::ConvertFarenScenario(const char *srcpath, const char *dstoutpath, bool cnvScript, bool noDefault)
{
    bool result = true;
    boost::filesystem::path fspath_src(srcpath);
    AnsiString dstpath;
    dstpath.sprintf("%s\\%s", dstoutpath, fspath_src.leaf().c_str());
#if 0
    if(!_access(dstpath.c_str(), 0)){
        AnsiString str;
        str.sprintf("fBNg %s ɑ݂܂B㏑܂H", dstpath.c_str());
        if(MessageDlg(str, mtConfirmation,TMsgDlgButtons() << mbYes << mbNo, 0) == mrNo){
            return false;
        }
    }
#endif
    // ɂꍇ̃G[͖
    CreateDirectory(dstpath.c_str(), NULL);

    // Ro[gDefaulttH_̃pXB
    AnsiString srcbasepath = ExtractFilePath(srcpath);
    AnsiString defscesrcpath;
    AnsiString defscedstpath;
    if(!noDefault){
        defscesrcpath.sprintf("%sDefault", srcbasepath.c_str());
        defscedstpath.sprintf("%s\\Default", dstoutpath);
    }

    fp_log = fopen(m_LogFileName.c_str(), "w");
    if(fp_log == NULL){
        ShowMessage("Ot@C쐬s");
        return false;
    }

    try{
        if(!ConvertConfig(srcpath, dstpath.c_str())){
            OutputLog("Configt@Cϊs");
            throw std::exception();
        }

        AnsiString datapath;
        datapath.sprintf("%s\\Data", dstpath);
        if(_access(datapath.c_str(), 0)){
            if(!CreateDirectory(datapath.c_str(), NULL)){
                OutputLog("%s ̍쐬Ɏs܂B", datapath.c_str());
                throw std::exception();
            }
        }
        Form1->SetProgressBar(5);

        // ֐Ԃ̈ˑ֌W
        // ConvertPicture  ConvertMagicTable 

        if(!LoadCallAble(srcpath)){
            OutputLog("CallAblet@CǍs");
            throw std::exception();
        }
        Form1->SetProgressBar(10);

        if(!ConvertAreaData(srcpath, dstpath.c_str())){
            OutputLog("AreaDatat@Cϊs");
            throw std::exception();
        }
        Form1->SetProgressBar(15);

        if(!ConvertPicture(srcpath, dstpath.c_str(), ConvertedImageMap)){
            OutputLog("Picture摜t@Cϊs");
            throw std::exception();
        }
        Form1->SetProgressBar(20);

        if(!ConvertMagicTable(srcpath, dstpath.c_str(), defscesrcpath, defscedstpath)){
            OutputLog("MagicTablet@Cϊs");
            throw std::exception();
        }
        Form1->SetProgressBar(25);

        if(!ConvertMasterData(srcpath, dstpath.c_str(), noDefault)){
            OutputLog("MasterDatat@Cϊs");
            throw std::exception();
        }
        Form1->SetProgressBar(30);

        if(!ConvertMoveAndSkill(srcpath, dstpath.c_str(), defscesrcpath)){
            OutputLog("Move&Skillt@Cϊs");
            throw std::exception();
        }
        Form1->SetProgressBar(35);

        if(!OutputMapChipType(srcpath, dstpath.c_str())){
            OutputLog("n`o͎s");
            throw std::exception();
        }
        Form1->SetProgressBar(40);

        if(!OutputMagicGroup(srcpath, dstpath.c_str())){
            OutputLog("@O[vo͎s");
            throw std::exception();
        }
        Form1->SetProgressBar(45);

        if(!OutputAttribute(srcpath, dstpath.c_str())){
            OutputLog("o͎s");
            throw std::exception();
        }
        if(!OutputFieldMap(srcpath, dstpath.c_str(), noDefault)){
            OutputLog("tB[h}bvo͎s");
            throw std::exception();
        }
        if(!OutputRace(srcpath, dstpath.c_str())){
            OutputLog("푰o͎s");
            throw std::exception();
        }
        if(!OutputRevive(srcpath, dstpath.c_str())){
            OutputLog("o͎s");
            throw std::exception();
        }
        if(!OutputLua(dstpath.c_str())){
            OutputLog("Luao͎s");
            throw std::exception();
        }
        Form1->SetProgressBar(50);

        for(int i=1; i<=5; i++){
            if(!ConvertScenarioX(srcpath, dstpath.c_str(), i)){
                OutputLog("Scenario%dt@Cϊs", i);
                throw std::exception();
            }
            Form1->SetProgressBar(55 + i*2);
        }
        if(!ConvertPowerMap(srcpath, dstpath.c_str(), defscesrcpath, defscedstpath)){
            OutputLog("PowerMapt@Cϊs");
            throw std::exception();
        }
        Form1->SetProgressBar(70);

        if(!ConvertBMap(srcpath, dstpath.c_str(), defscesrcpath)){
            OutputLog("BMapt@Cϊs");
            throw std::exception();
        }
        Form1->SetProgressBar(75);

        if(!ConvertMapChipTxt(srcpath, dstpath.c_str(), defscesrcpath)){
            OutputLog("TikeiTypet@CRs[s");
            throw std::exception();
        }

        if(!OutputMapObjTxt(dstpath.c_str())){
            OutputLog("㕨t@Co͎s");
            throw std::exception();
        }
        Form1->SetProgressBar(80);

        if(cnvScript){
            boost::scoped_ptr<TFarenScriptCnv> scrcnv(new TFarenScriptCnv(CharaIDArray, AreaIDArray));
            if(!scrcnv->ConvertScripts(srcpath, dstpath.c_str())){
                OutputLog("Scriptt@Cϊs");
                throw std::exception();
            }
        }
        else{
            TFarenScriptCnv::OutputDummyScript(dstpath.c_str());
        }
        Form1->SetProgressBar(85);

        if(!CopyMusicSound(srcpath, dstpath.c_str())){
            OutputLog("Music&&Soundt@CRs[s");
            throw std::exception();
        }
        Form1->SetProgressBar(90);

        if(!CopyCnvSrc(dstpath.c_str())){
            OutputLog("cnvsrct@CRs[s");
            throw std::exception();
        }
        Form1->SetProgressBar(100);
    }
    catch(std::exception&){
        result = false;
    }
    if(result){
        OutputLog("ϊ܂B");
    }
    fclose(fp_log);

    return result;
}
//---------------------------------------------------------------------------
void TFarenCnv::SetLogFileName(const char *path)
{
    m_LogFileName = path;
}
//---------------------------------------------------------------------------
void TFarenCnv::SetCnvSrcPath(const char *path)
{
    m_CnvSrcPath = path;
}
//---------------------------------------------------------------------------
// ConfigScenario.xml
bool TFarenCnv::ConvertConfig(const char *srcpath, const char *dstpath)
{
    AnsiString src, dst;
    src.sprintf("%s\\Config", srcpath);
    dst.sprintf("%s\\Scenario.xml", dstpath);

    TFarenTxt ftxt;
    if(!ftxt.LoadFromFile(src.c_str())){
        OutputLog("failed to open %s", src.c_str());
        return false;
    }
    int index, count;
    if(!ftxt.GetCategory("Name", &index, &count) || !count){
        OutputLog("%s [Name]܂łB");
        return false;
    }

    TiXmlDocument doc(dst.c_str());
    TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
    doc.InsertEndChild(dec);
    TiXmlElement e("Scenario");
    TiXmlElement e_title("Title");
    InsertTextChild(e_title, ftxt.GetString(index).c_str());
    e.InsertEndChild(e_title);
    doc.InsertEndChild(e);

    if(!doc.SaveFile()){
        return false;
    }

    return true;
}
//---------------------------------------------------------------------------
bool TFarenCnv::LoadCallAble(const char *srcpath)
{
    AnsiString src;
    src.sprintf("%s\\Data\\CallAble", srcpath);

    TFarenTxt ftxt;
    if(!ftxt.LoadFromFile(src.c_str())){
        OutputLog("failed to open %s", src.c_str());
        return false;
    }

    for(int typecount=0; ; typecount++){
        AnsiString str;
        str.sprintf("Init%d", typecount);
        int index, count;
        if(!ftxt.GetCategory(str.c_str(), &index, &count)){
            // 0n܂ĂȂViÎ100܂ł͍ĎsĂ݂B
            if(typecount > 100){
                break;
            }
            // LqCfbNXɂĂ͋̃f[^ƂB
            TiXmlElement e("AreaCharacter");
            CallableList.push_back(e);
            continue;
        }
        TiXmlElement e("AreaCharacter");
        // nтɑ݂LN^[
        TiXmlElement e_inhabit("Inhabit");
        for(int i=0; i<count;i++){
            if(i == count - 1 && !strcmpi(ftxt.GetString(index+i).c_str(), "end")){
                // Ō̍s"end"Ȃ疳B
                break;
            }
            TiXmlElement e_chara("Character");
            SetTextAttribute(e_chara, "id", ftxt.GetString(index+i).c_str());
            e_inhabit.InsertEndChild(e_chara);
        }
        e.InsertEndChild(e_inhabit);

        // قLN^[
        TiXmlElement e_employ("Employable");
        str.sprintf("Call%d", typecount);
        if(!ftxt.GetCategory(str.c_str(), &index, &count)){
            // ĂAreaCharacter͓ĂB
            CallableList.push_back(e);
            break;
        }
        for(int i=0; i<count; i++){
            if(i == count - 1 && !strcmpi(ftxt.GetString(index+i).c_str(), "end")){
                // Ō̍s"end"Ȃ疳B
                break;
            }
            TiXmlElement e_chara("Character");
            SetTextAttribute(e_chara, "id", ftxt.GetString(index+i).c_str());
            e_employ.InsertEndChild(e_chara);
        }
        e.InsertEndChild(e_employ);
        CallableList.push_back(e);
    }
    return true;
}
//---------------------------------------------------------------------------
// AreaDataAreaData.xml
bool TFarenCnv::ConvertAreaData(const char *srcpath, const char *dstpath)
{
    AnsiString src, dst;
    src.sprintf("%s\\Data\\AreaData", srcpath);
    dst.sprintf("%s\\Data\\AreaData.xml", dstpath);
    //std::vector<std::string> idlist;

    FILE *fp = fopen(src.c_str(), "r");
    if(fp == NULL){
        OutputLog("failed to open %s", src.c_str());
        return false;
    }

    std::map<std::string, int> idCount; // dGAIDɘAԂt邽߂̃}bvB
    // IDǂ
    for(int count = 0; ; count++){
        char buf[1024];
        if(fgets(buf, sizeof buf, fp) == NULL){
            break;
        }
        std::vector<std::string> v;
        boost::algorithm::split(v, buf, boost::is_space());
        // 炩ɗvfȂI
        if(v.size() < 19){
            break;
        }
        // 󕶎łȂvfz̑OɎĂ
        std::remove(v.begin(), v.end(), "");

        // dGAID邩B
        std::map<std::string, int>::iterator itr = idCount.find(v[0]);
        if(itr == idCount.end()){
            AreaIDArray.push_back(v[0]);
            idCount[v[0]] = 1;  // IDo^
        }
        else{
            AnsiString areaid;
            areaid.sprintf("%s%d", v[0].c_str(), itr->second);
            AreaIDArray.push_back(areaid.c_str());
            ++itr->second;      // IDZ
        }
    }
    AreaIDNeighbor.resize(AreaIDArray.size());
    for(unsigned u=0; u<AreaIDNeighbor.size(); u++){
        AreaIDNeighbor[u].resize(8);
    }

    fseek(fp, 0, SEEK_SET);
    TiXmlDocument doc(dst.c_str());
    TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
    doc.InsertEndChild(dec);
    TiXmlElement e_areadata("AreaList");
    for(int count = 0; ; count++){
        char buf[1024];
        if(fgets(buf, sizeof buf, fp) == NULL){
            break;
        }
        std::vector<std::string> v;
        boost::algorithm::split(v, buf, boost::is_space());
        // 炩ɗvfȂI
        if(v.size() < 19){
            break;
        }
        // 󕶎łȂvfz̑OɎĂ
        std::remove(v.begin(), v.end(), "");

        TiXmlElement e_area("Area");
        SetTextAttribute(e_area, "id", AreaIDArray[count].c_str());
        // GA
        TiXmlElement e_name("Name");
        InsertTextChild(e_name, v[0].c_str());
        e_area.InsertEndChild(e_name);
        // oEٗp\L͒ڃGȀɂB
        unsigned callable;
        if(sscanf(v[1].c_str(), "%u", &callable) == 1 && callable < CallableList.size()){
            e_area.InsertEndChild(CallableList[callable]);
        }
        // b
        TiXmlElement e_income("BaseIncome");
        InsertTextChild(e_income, v[2].c_str());
        e_area.InsertEndChild(e_income);
        // X
        TiXmlElement e_city("City");
        SetTextAttribute(e_city, "max", v[3].c_str());
        e_area.InsertEndChild(e_city);
        // MAXя[
        TiXmlElement e_wall("Wall");
        SetTextAttribute(e_wall, "max", v[4].c_str());
        SetTextAttribute(e_wall, "ini", v[7].c_str());
        e_area.InsertEndChild(e_wall);
        // H
        TiXmlElement e_road("Road");
        SetTextAttribute(e_road, "max", v[5].c_str());
        e_area.InsertEndChild(e_road);
        // ͌ٗpp
        TiXmlElement e_inhcost("InhabitCost");
        SetTextAttribute(e_inhcost, "value", v[6].c_str());
        e_area.InsertEndChild(e_inhcost);
        // W
        TiXmlElement e_xy("Location");
        e_xy.SetAttribute("x", v[8].c_str());
        e_xy.SetAttribute("y", v[9].c_str());
        e_area.InsertEndChild(e_xy);
        // אڃGA
        for(int i=0; i<8; i++){
            // łȂ00LڂĂאڃGA͗vfo͂ȂB
            int n;
            if(sscanf(v[10+i].c_str(), "%d", &n) != 1 || !n || static_cast<unsigned>(n-1) >= AreaIDArray.size()){
                continue;
            }
            TiXmlElement e_neighbor("NeighborArea");
            e_neighbor.SetAttribute("index", i);
            //InsertTextChild(e_neighbor, v[10+i].c_str());
            if(static_cast<unsigned>(n-1) >= AreaIDArray.size()){
                fclose(fp);
                OutputLog("אڃGÃCfbNXɎs");
                return false;
            }
            // ߖT̃GAIDLB
            AreaIDNeighbor[count][i] = AreaIDArray[n-1];
            InsertTextChild(e_neighbor, AreaIDArray[n-1].c_str());
            e_area.InsertEndChild(e_neighbor);
        }
        // BGM
        TiXmlElement e_music("Music");
        AnsiString str;
        str.sprintf("%s.mid", v[18].c_str());
        InsertTextChild(e_music, str.c_str());
        e_area.InsertEndChild(e_music);

        TiXmlElement e_bmap("BattleMapFile");
        str.sprintf("BMap_area%02d.xml", count+1);
        InsertTextChild(e_bmap, str.c_str());
        e_area.InsertEndChild(e_bmap);

        e_areadata.InsertEndChild(e_area);
    }
    fclose(fp);
    doc.InsertEndChild(e_areadata);

    if(!doc.SaveFile()){
        return false;
    }

    return true;
}
//---------------------------------------------------------------------------
// CharacterDataCharacterData.xml
// isFirstDatáAŏ̃f[^̂ݐ^w肷Biŏ̃f[^NKCΏۂ̃}X^[Ƃ邽߁j
bool TFarenCnv::ConvertCharacterData(const char *srcpath, const char *dstpath, bool noDefault, bool isDefaultScenario, bool isFirstData, int specialOffset, int specialOffsetEnd)
{
    AnsiString src, dst;
    src.sprintf("%s\\CharacterData", srcpath);
    dst.sprintf("%s\\CharacterData.xml", dstpath);

    FILE *fp = fopen(src.c_str(), "rb");
    if(fp == NULL){
        OutputLog("failed to open %s", src.c_str());
        return false;
    }
    if(fseek(fp, 1024, SEEK_SET)){
        OutputLog("%s ŏ1024BytẽV[Ns", src.c_str());
        fclose(fp);
        return false;
    }

    TiXmlDocument doc(dst.c_str());
    TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
    doc.InsertEndChild(dec);
    TiXmlElement e_top("CharacterList");

    std::vector<TCharacterData> chrvec;
    int count;
    for(count=0; ; count++)
    {
        TCharacterData chrdata;
        if(fread(&chrdata, 1, sizeof chrdata, fp) < sizeof chrdata){
            break;
        }
        chrvec.push_back(chrdata);
    }
    fclose(fp);

    for(int chr=1; chr<count; chr++){
        const TCharacterData& chrdata = chrvec[chr];
        /*int x = -1;
        for(x=0;x<sizeof chrdata.dummy6; ++x){
            if(chrdata.dummy6[x])
                break;
        }
        if(x != -1){
            fprintf(fp_log, "%3d:%16s %d%d%d%d%d%d%d%d %d %d\n", chr, chrdata.ID,
                (chrdata.HeroChar>>7) & 1,
                (chrdata.HeroChar>>6) & 1,
                (chrdata.HeroChar>>5) & 1,
                (chrdata.HeroChar>>4) & 1,
                (chrdata.HeroChar>>3) & 1,
                (chrdata.HeroChar>>2) & 1,
                (chrdata.HeroChar>>1) & 1,
                (chrdata.HeroChar & 1),
                chrdata.dummy6[x],
                x
                );
        }*/
        TiXmlElement e("Character");
        SetTextAttribute(e, "id", chrdata.ID);
        // O
        TiXmlElement e_name("Name");
        InsertTextChild(e_name, chrdata.Name);
        e.InsertEndChild(e_name);
        /*if(chrdata.HeroChar & 1){
            TiXmlElement e_stuff("Stuff");
            e_stuff.SetAttribute("flag", "true");
            e.InsertEndChild(e_stuff);
        }*/
        // @
        for(int i=0; i<MAGICGROUP_MAX; i++){
            TiXmlElement e_magic("Magic");
            AnsiString str;
            str.sprintf("magicgroup%d", i);
            e_magic.SetAttribute("id", str.c_str());
            e_magic.SetAttribute("rank", chrdata.Magic[i]);
            e.InsertEndChild(e_magic);
        }
        // 
        for(int i=0; i<15; i++){
            TiXmlElement e_res("Resistance");
            e_res.SetAttribute("id", attr_id[i+1]);
            if(chrdata.Merit[i] < 0 || chrdata.Merit[i] >= ATTRRES_MAX){
                continue;
            }
            e_res.SetAttribute("value", attrres_id[chrdata.Merit[i]]);
            e.InsertEndChild(e_res);
        }
        // l
        if(chrdata.HeroFlag[0]){
            TiXmlElement e_hero("Hero");
            e_hero.SetAttribute("flag", "true");
            e.InsertEndChild(e_hero);
        }
        // ȃLƂĈ
        if(chrdata.HeroFlag[2]){
            TiXmlElement e_deadmsg("DeadMessage");
            e_deadmsg.SetAttribute("flag", "true");
            e.InsertEndChild(e_deadmsg);
        }
        // QȂtO
        if(chrdata.HeroFlag[1]){
            TiXmlElement e_nowander("NoWander");
            e_nowander.SetAttribute("flag", "true");
            e.InsertEndChild(e_nowander);
        }
        // l
        TiXmlElement e_race("Race");
        if(chrdata.Race < 0 || chrdata.Race >= RACE_MAX){
            OutputLog("%s ̎푰CfbNX͈͂𒴂Ă܂B", chrdata.Name);
            return false;
        }
        SetTextAttribute(e_race, "value", race_name[chrdata.Race]);
        e.InsertEndChild(e_race);
        // ێƏp
        if(chrdata.HeroFlag[0]){
            // l
            TiXmlElement e_ecost("EmployCost");
            e_ecost.SetAttribute("value", chrdata.Cost * 5);
            e.InsertEndChild(e_ecost);
            TiXmlElement e_tcost("TurnCost");
            e_tcost.SetAttribute("value", chrdata.Cost);
            e.InsertEndChild(e_tcost);
        }
        else{
            // 
            TiXmlElement e_ecost("EmployCost");
            e_ecost.SetAttribute("value", chrdata.Cost);
            e.InsertEndChild(e_ecost);
            TiXmlElement e_tcost("TurnCost");
            e_tcost.SetAttribute("value", 0);
            e.InsertEndChild(e_tcost);
        }
        // ړ^Cv
        TiXmlElement e_mvt("MoveType");
        if(chrdata.MoveType < 0 || chrdata.MoveType >= MOVETYPE_MAX){
            OutputLog("%s ̈ړ^CvCfbNX͈͂𒴂Ă܂B", chrdata.Name);
            return false;
        }
        SetTextAttribute(e_mvt, "value", movetype_name[chrdata.MoveType]);
        e.InsertEndChild(e_mvt);
        // ړ
        TiXmlElement e_mv("Movable");
        e_mv.SetAttribute("value", chrdata.Movable);
        e.InsertEndChild(e_mv);
        // HP
        TiXmlElement e_hp("HP");
        e_hp.SetAttribute("value", chrdata.HP);
        e.InsertEndChild(e_hp);
        // MP
        TiXmlElement e_mp("MP");
        e_mp.SetAttribute("value", chrdata.MP);
        e.InsertEndChild(e_mp);
        // U
        TiXmlElement e_pa("PhysicalAttack");
        e_pa.SetAttribute("value", chrdata.PhysicalAttack);
        e.InsertEndChild(e_pa);
        // h
        TiXmlElement e_pd("PhysicalDefense");
        e_pd.SetAttribute("value", chrdata.PhysicalDefense);
        e.InsertEndChild(e_pd);
        // Z
        TiXmlElement e_skill("Skill");
        e_skill.SetAttribute("value", chrdata.Skill);
        e.InsertEndChild(e_skill);
        // 
        TiXmlElement e_speed("Speed");
        e_speed.SetAttribute("value", chrdata.Speed);
        e.InsertEndChild(e_speed);
        // 
        TiXmlElement e_ma("MagicAttack");
        e_ma.SetAttribute("value", chrdata.MagicAttack);
        e.InsertEndChild(e_ma);
        // R
        TiXmlElement e_md("MagicDefense");
        e_md.SetAttribute("value", chrdata.MagicDefense);
        e.InsertEndChild(e_md);
        // mC
        TiXmlElement e_morale("Morale");
        e_morale.SetAttribute("value", chrdata.Morale);
        e.InsertEndChild(e_morale);
        // E
        TiXmlElement e_brave("Aggressive");
        e_brave.SetAttribute("value", chrdata.Aggressive);
        e.InsertEndChild(e_brave);
        // HP
        TiXmlElement e_hprec("HPRecover");
        e_hprec.SetAttribute("value", chrdata.HPRecover);
        e.InsertEndChild(e_hprec);
        // MP
        TiXmlElement e_mprec("MPRecover");
        e_mprec.SetAttribute("value", chrdata.MPRecover);
        e.InsertEndChild(e_mprec);
        // KEZ
        //OutputLog("Chara:%s spe:%d", chrdata.ID, chrdata.SpecialNo);
        if(chrdata.SpecialNo >= 1){
            unsigned speindex;
            if(chrdata.OriginalSpecialData){
                speindex = specialOffset + chrdata.SpecialNo - 1;
            }
            else{
                if(noDefault){
                    OutputLog("Error:Defaultf[^gpȂIvVw肳܂AL %s Default̓ZĂ܂B", chrdata.ID);
                    return false;
                }
                speindex = chrdata.SpecialNo - 1;
            }
            //OutputLog("%s %d id=%s spe=%d -> %d (%d)", src.c_str(), chr, chrdata.ID, chrdata.SpecialNo, speindex, chrdata.OriginalSpecialData);
            // LtH_[J̓Z̓CfbNXLtH_SpecialData`FbNB
            if(!chrdata.OriginalSpecialData || speindex < static_cast<unsigned>(specialOffsetEnd)){
                if(speindex < m_SpecialID.size()){
                    if(!m_SpecialID[speindex].empty()){
                        TiXmlElement e_special("Special");
                        SetTextAttribute(e_special, "id", m_SpecialID[speindex].c_str());
                        e_special.SetAttribute("count", chrdata.SpecialCount);
                        e.InsertEndChild(e_special);
                    }
                    else{
                        OutputLog("Warning:Chara=%s ̕KEZ͉摜t@CߖłB̃L̕KEZ𖳂Ƃ܂B", chrdata.ID);
                    }
                }
                else{
                    OutputLog("Warning:Chara=%s ̕KEZCfbNX0ȊO(%d)łA͈͊Ôߖ܂B", chrdata.ID, chrdata.SpecialNo);
                }
            }
        }
        // 앜
        if(std::find(m_GhostList.begin(), m_GhostList.end(), chr) != m_GhostList.end()){
            TiXmlElement e_rev("AutoSpecial");
            SetTextAttribute(e_rev, "id", NECROMANCY_SPECIAL);
            SetTextAttribute(e_rev, "event", "EnemyDead");    // MeDead, MySideDead, EnemyDead
            e.InsertEndChild(e_rev);
        }
        if(isFirstData){
            // RNKC
            for(unsigned u=0; u<MasterRevivalList.size(); ++u){
                if(MasterRevivalList[u].master_from == chr){
                    TiXmlElement e_rev("AutoSpecial");
                    AnsiString speid;
                    speid.sprintf("%s%u", MASTER_REVIVAL, u);
                    SetTextAttribute(e_rev, "id", speid.c_str());
                    SetTextAttribute(e_rev, "event", "MeDead");
                    e.InsertEndChild(e_rev);
                    break;
                }
            }
            // zXp
            for(unsigned u=0; u<MasterInheritList.size(); ++u){
                if(MasterInheritList[u].master_from == chr){
                    TiXmlElement e_rev("AutoScript");
                    AnsiString scfunc;
                    scfunc.sprintf("%s%u", MASTER_INHERIT, u);
                    SetTextAttribute(e_rev, "function", scfunc.c_str());
                    SetTextAttribute(e_rev, "event", "MeDead");
                    e.InsertEndChild(e_rev);
                    break;
                }
            }
        }
        // NX`FW
        if(chrdata.ClassChange >= 1 && chrdata.ClassChange < count){
            TiXmlElement e_cc("ClassChange");
            SetTextAttribute(e_cc, "next", chrvec[chrdata.ClassChange].ID);
            e.InsertEndChild(e_cc);
        }
        // ol
        TiXmlElement e_exp("Experience");
        e_exp.SetAttribute("value", chrdata.Experience);
        e.InsertEndChild(e_exp);
        // ͎w
        TiXmlElement e_power("Power");
        e_power.SetAttribute("value", chrdata.Experience);
        e.InsertEndChild(e_power);
        // o
        TiXmlElement e_app("Appearance");
        e_app.SetAttribute("TurnMin", chrdata.TurnMin);
        if(chrdata.TurnMax < 9999){
            e_app.SetAttribute("TurnMax", chrdata.TurnMax);
        }
        e.InsertEndChild(e_app);
        // U
        for(int i=0; i<8; i++){
            TiXmlElement e_attack("Attack");
            // UIDłȂCfbNXB
            e_attack.SetAttribute("index", i);
            if(chrdata.AttackAttribute[i] < 0 || chrdata.AttackAttribute[i] >= ATTRIBUTE_MAX){
                continue;
            }
            e_attack.SetAttribute("Power", chrdata.AttackPower[i]);
            if(!chrdata.AttackAttribute[i]){
                e_attack.SetAttribute("specialid", "None");
            }
            else{
                // ConvertSpecialTableŏo͂ĂUIDƓB
                AnsiString str;
                str.sprintf("Attack%s", attr_id[chrdata.AttackAttribute[i]]);
                e_attack.SetAttribute("specialid", str.c_str());
            }
            e.InsertEndChild(e_attack);
        }
        // ٗp\
        for(int i=0; i<8; i++){
            // o^Ȃ͂Ƃ΂B
            if(!chrdata.Employable[i]){
                continue;
            }
            TiXmlElement e_employ("Employable");
            // ٗp\IDłȂCfbNXB
            e_employ.SetAttribute("index", i);
            if(chrdata.Employable[i] < 0 || chrdata.Employable[i] > count-1){
                continue;
            }
            InsertTextChild(e_employ, chrvec[chrdata.Employable[i]].ID);
            e.InsertEndChild(e_employ);
        }
        // NCGΏہBĂۂɒT猩B
        if((chrdata.dummy6[10])){
            TiXmlElement e_target("Target");
            e_target.SetAttribute("id", "undead");
            e.InsertEndChild(e_target);
        }
        //OutputLog("%s:Default = %d BeastTamer = %d", chrdata.ID, chrdata.DefaultSpecialData, chrdata.BeastTamer);
        // r[XgeC}[
        if(chrdata.BeastTamer){
            TiXmlElement e_tamer("BeastTamer");
            TiXmlElement e_com("Command");
            InsertTextChild(e_com, "X^[");
            e_tamer.InsertEndChild(e_com);
            e.InsertEndChild(e_tamer);
        }
        // 摜
        TiXmlElement e_image("BattleChip");
        AnsiString chipfile;
        if(chrdata.OriginalChipImage){
            chipfile.sprintf("Char%d.png", chrdata.ImageNo);
        }
        else{
            if(isDefaultScenario){
                chipfile.sprintf("Char%d.png", chrdata.ImageNo);
            }
            else{
                if(noDefault){
                    OutputLog("Error:Defaultf[^gpȂIvVw肳܂AL %s Default̃`bv摜ݒ肳Ă܂B", chrdata.ID);
                    return false;
                }
                chipfile.sprintf("../../Default/DefChar/Char%d.png", chrdata.ImageNo);
            }
        }
        InsertTextChild(e_image, chipfile.c_str());
        e.InsertEndChild(e_image);
        // i0͖j
        if(chrdata.FaceNo){
            TiXmlElement e_face("Face");
            AnsiString facefile;
            if(chrdata.OriginalFaceImage){
                facefile.sprintf("Face%d.png", chrdata.FaceNo);
            }
            else{
                if(isDefaultScenario){
                    facefile.sprintf("Face%d.png", chrdata.FaceNo);
                }
                else{
                    if(noDefault){
                        OutputLog("Error:Defaultf[^gpȂIvVw肳܂AL %s Default̊摜ݒ肳Ă܂B", chrdata.ID);
                        return false;
                    }
                    facefile.sprintf("../../Default/DefChar/Face%d.png", chrdata.FaceNo);
                }
            }
            InsertTextChild(e_face, facefile.c_str());
            e.InsertEndChild(e_face);
        }

        e_top.InsertEndChild(e);
        // XNvgϊ̂߂ɊoĂB
        CharaIDArray.push_back(chrdata.ID);
        // ŃLIDƂÔŊoĂB
        TConvertedChara cc;
        cc.id = chrdata.ID;
        cc.name = chrdata.Name;
        ConvertedCharaList.push_back(cc);
        // ԍꍇA摜t@C𐶐ăLIDŋLB
        if(chrdata.MasterFlagNo){
            AnsiString flagfile;
            if(chrdata.OriginalFlagImage){
                flagfile.sprintf("Flag%d.png", chrdata.MasterFlagNo);
            }
            else{
                if(isDefaultScenario){
                    flagfile.sprintf("Flag%d.png", chrdata.MasterFlagNo);
                }
                else{
                    if(noDefault){
                        OutputLog("Error:Defaultf[^gpȂIvVw肳܂AL %s Default̊摜ݒ肳Ă܂B", chrdata.ID);
                        return false;
                    }
                    flagfile.sprintf("../../Default/DefChar/Flag%d.png", chrdata.MasterFlagNo);
                }
            }
            MasterFlagFileMap[chrdata.ID] = flagfile.c_str();
        }
    }

    doc.InsertEndChild(e_top);
    if(!doc.SaveFile()){
        return false;
    }

    AnsiString bmpfiles;
    bmpfiles.sprintf("%s\\*.bmp", srcpath);
    WIN32_FIND_DATA findData;
    ZeroMemory(&findData, sizeof findData);
    HANDLE hFind = FindFirstFile(bmpfiles.c_str(), &findData);
    if(hFind != INVALID_HANDLE_VALUE){
        do{
            AnsiString filename = ExtractFileName(findData.cFileName);
            AnsiString bmpfile;
            bmpfile.sprintf("%s\\%s", srcpath, filename.c_str());
            //OutputLog("%s loading...", bmpfile.c_str());
            HDIB hdib = BMPtoDIB(bmpfile.c_str());
            if(hdib == NULL){
                OutputLog("摜 %s ̓ǂݍ݂Ɏs܂B", bmpfile.c_str());
                continue;
            }
            //OutputLog("%s loading OK", bmpfile.c_str());
            AnsiString noext = filename.SubString(0, filename.Length() - 4);
            AnsiString pngfile;
            pngfile.sprintf("%s\\%s.png", dstpath, noext.c_str());
            if(BreathImageFiles.find(noext.c_str()) != BreathImageFiles.end()){
                // uX̉摜t@C͉HĕۑB
                const chipxcount = 7;
                const chipw = 32;
                // bmp̕擾B
                int width, height;
                // uX摜̉ɂĂ224Ɖ肵BłȂ݂Ȃ̂Œ~B
                //if(!GetHDIBSize(hdib, &width, &height) || width != chipxcount * chipw){
                //    OutputLog("uX摜 %s ̉224Ȃ", bmpfile.c_str());
                if(!GetHDIBSize(hdib, &width, &height)){
                    OutputLog("uX摜 %s ̃TCY擾ɎsB", bmpfile.c_str());
                    DeleteDIB(hdib);
                    return false;
                }
                int realxcount = width / chipw;
                int realycount = height / chipw;
                int realcount = realxcount * realycount;
                // c`bvTCYŊAj[VB3ŒȂ̂ȂB`bv7ŊɕύXB
                //int animcount = height / chipw;
                int animcount = (realcount + 6) / 7;
                // f[^I[0DIB쐬
                HDIB breathdib = CreateAllZeroDIB(chipw * 3 * animcount, chipw * 3);
                // H̉摜ɓ\tWB
                const int paste_to_x[chipxcount] = {chipw, chipw/2, chipw/2 + chipw, 0,     chipw*2, chipw/2, chipw/2 + chipw};
                const int paste_to_y[chipxcount] = {chipw,       0,               0, chipw,   chipw, chipw*2, chipw*2};
                int index = 0;
                for(int anim=0; anim<animcount; ++anim){
                    for(int chip=0; chip<chipxcount; ++chip){
                        //BOOL b = PasteDIB(breathdib, paste_to_x[chip] + anim*chipw*3, paste_to_y[chip], chipw, chipw, hdib, chipw*chip, chipw*anim, NULL, 0);
                        // H̃`bvȂꍇAPɉ\ȂB
                        if(index >= realcount){
                            break;
                        }
                        int srcx = (index % realxcount) * chipw;
                        int srcy = (index / realxcount) * chipw;
                        BOOL b = PasteDIB(breathdib, paste_to_x[chip] + anim*chipw*3, paste_to_y[chip], chipw, chipw, hdib, srcx, srcy, NULL, 0);
                        if(!b){
                            ShowMessage("PasteDIB Error:sȂ͂Ȃ񂾂B");
                        }
                        ++index;
                    }
                }
                DeleteDIB(hdib);
                hdib = breathdib;
            }
            else if(ShotImageFiles.find(noext.c_str()) != ShotImageFiles.end()){
                // ˍỦ摜t@C͉HĕۑB
                const direction = 8;
                const chipw = 32;
                int realchipcount = direction;
                // bmp̕擾B
                int width, height;
                // ˍU摜̉ɂĂ256Ɖ肵B̂256ȂViÎœKɑΉB
                //if(!GetHDIBSize(hdib, &width, &height) || width != direction * chipw){
                if(!GetHDIBSize(hdib, &width, &height)){
                    OutputLog("ˍU摜 %s ̃TCY擾ɎsB", bmpfile.c_str());
                    DeleteDIB(hdib);
                    return false;
                }
                if(width != direction * chipw){
                    realchipcount = width / chipw;
                }
                HDIB shotdib = CreateAllZeroDIB(chipw, chipw * direction);
                for(int chip=0; chip<direction; ++chip){
                    int srcx = chipw*chip;
                    if(srcx + chipw > width){
                        srcx = chipw * (chip % realchipcount);
                    }
                    BOOL b = PasteDIB(shotdib, 0, chipw*chip, chipw, chipw, hdib, srcx, 0, NULL, 0);
                    if(!b){
                        ShowMessage("PasteDIB Error:sȂ͂Ȃ񂾂B");
                    }
                }
                DeleteDIB(hdib);
                hdib = shotdib;
            }
            else if(KokuryuImageFiles.find(noext.c_str()) != KokuryuImageFiles.end()){
                TKokuryuData koku = KokuryuImageFiles.find(noext.c_str())->second;
                // ̉摜t@C͉HĕۑB
                const int defchipw = 96;
                // bmp̕擾B
                int width, height;
                if(!GetHDIBSize(hdib, &width, &height)){
                    OutputLog("^Cv摜 %s ̃TCY擾ɎsB", bmpfile.c_str());
                    DeleteDIB(hdib);
                    return false;
                }
                if(koku.width > width){
                    OutputLog("Warning:^Cv摜 %s ɂă`bvTCY%dw肳܂A摜̉%dłB`bvTCY%dƂ݂Ȃĕϊ܂B", bmpfile.c_str(), koku.width, width, width);
                    koku.width = width;
                }
                if(koku.width > height){
                    OutputLog("Warning:^Cv摜 %s ɂă`bvTCY%dw肳܂A摜̏c%dłB`bvTCY%dƂ݂Ȃĕϊ܂B", bmpfile.c_str(), koku.width, height, height);
                    koku.width = height;
                }
                int imganicount = (width / koku.width) * (height / koku.width);
                if(imganicount > koku.anicount){
                    OutputLog("Warning:^Cv摜 %s ɂăAj[V%dw肳܂A摜%d̑傫ɂȂĂ܂B摜̗]ȕ͖܂B", bmpfile.c_str(), koku.anicount, imganicount);
                }
                if(koku.width > defchipw){
                    OutputLog("^Cv摜 %s ɂ1Aj[ṼTCY%dw肳܂A96hbg𒴂Ă邽ߏł܂B", bmpfile.c_str(), koku.width);
                    DeleteDIB(hdib);
                    return false;
                }
                int cnvcount = std::min(imganicount, koku.anicount);
                HDIB kokudib = CreateAllZeroDIB(defchipw * cnvcount, defchipw);
                int srcx = 0, srcy = 0;
                int dstx = 0;
                for(int chip=0; chip<cnvcount; ++chip){
                    if(srcx + koku.width > width){
                        srcx = 0;
                        srcy += koku.width;
                    }
                    BOOL b = PasteDIB(kokudib, dstx, 0, koku.width, koku.width, hdib, srcx, srcy, NULL, 0);
                    if(!b){
                        ShowMessage("PasteDIB Error:sȂ͂Ȃ񂾂B");
                    }
                    srcx += koku.width;
                    dstx += defchipw;
                }
                DeleteDIB(hdib);
                hdib = kokudib;
            }
            PNGOPT pngopt;
            ZeroMemory(&pngopt, sizeof pngopt);
            pngopt.dwFlag = POF_TRNSCOLOR | POF_BACKCOLOR;
            pngopt.wCompLevel = 5;
            pngopt.clrTrans = RGB(0, 0, 0);
            pngopt.clrBack =  RGB(0, 0, 0); // w肵ȂViXœߕŕ\Ă܂B
            BOOL succeed = DIBtoPNGex(pngfile.c_str(), hdib, &pngopt);
            DeleteDIB(hdib);
            if(!succeed){
                return false;
            }
            //OutputLog("%s convert OK", bmpfile.c_str());
        }while(FindNextFile(hFind, &findData));
        FindClose(hFind);
    }

    return true;
}
//---------------------------------------------------------------------------
// MagicTableMagicTable.xml
bool TFarenCnv::ConvertMagicTable(const char *srcpath, const char *dstpath, const AnsiString& defscesrcpath, const AnsiString& defscedstpath)
{
    AnsiString src, dst;
    AnsiString src_sound;
    src.sprintf("%s\\Data\\MagicTable", srcpath);
    src_sound.sprintf("%s\\Data\\SoundFileName", srcpath);
    dst.sprintf("%s\\Data\\MagicTable.xml", dstpath);

    FILE *fp = fopen(src.c_str(), "r");
    if(fp == NULL){
        if(!defscesrcpath.IsEmpty()){
            src.sprintf("%s\\Data\\MagicTable", defscesrcpath.c_str());
            fp = fopen(src.c_str(), "r");
        }
    }
    if(fp == NULL){
        OutputLog("failed to open %s", src.c_str());
        return false;
    }
    FILE *fp_sound = fopen(src_sound.c_str(), "r");
    if(fp_sound == NULL){
        if(!defscesrcpath.IsEmpty()){
            src_sound.sprintf("%s\\Data\\SoundFileName", defscesrcpath.c_str());
            fp_sound = fopen(src_sound.c_str(), "r");
        }
    }
    if(fp_sound == NULL){
        fclose(fp);
        OutputLog("failed to open %s", src_sound.c_str());
        return false;
    }
    TiXmlDocument doc(dst.c_str());
    TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
    doc.InsertEndChild(dec);

    TiXmlElement e_magiclist("MagicList");
    for(int count = 0; ; count++){
        char buf[1024];
        if(fgets(buf, sizeof buf, fp) == NULL){
            break;
        }
        // ʉt@C͓ǂ߂ȂĂCɂȂ
        char buf_sound[1024] = "";
        if(fgets(buf_sound, sizeof buf_sound, fp_sound) != NULL){
            char *p = strchr(buf_sound, '\n');
            if(p != NULL){
                *p = '\0';
            }
        }
        std::vector<std::string> v;
        boost::algorithm::split(v, buf, boost::is_space());
        // 炩ɗvfȂI
        if(v.size() < 15){
            break;
        }
        // 󕶎łȂvfz̑OɎĂ
        std::remove(v.begin(), v.end(), "");

        TiXmlElement e_magic("Magic");
        SetTextAttribute(e_magic, "id", v[0].c_str());
        // @JeS
        AnsiString groupid;
        groupid.sprintf("magicgroup%d", count/8);
        e_magic.SetAttribute("groupid", groupid.c_str());
        // gp񐔐
        if(count%8 > 5){
            e_magic.SetAttribute("limit", 1);
        }
        // @
        TiXmlElement e_name("Name");
        InsertTextChild(e_name, v[0].c_str());
        e_magic.InsertEndChild(e_name);
        // ʁi0: 1:⏕ 2:Uj
        int mtype;
        if(sscanf(v[1].c_str(), "%d", &mtype) != 1 || mtype < 0 || mtype > 2){
            continue;
        }
        //e_magic.SetAttribute("type", typestr[mtype]);
        for(int level=1; level<=3; ++level){
            TiXmlElement e_entry("Entry");
            e_entry.SetAttribute("level", level);
            // KvȖ@x
            int needrank = -1;
            for(int i=0; i<MAGICRANK_MAX; ++i){
                if(UsableMagicLevel[count%MAGIC_MAX][i] >= level){
                    needrank = i + 1;
                    break;
                }
            }
            e_entry.SetAttribute("rank", needrank);
            int levelindex = 2 + (level-1)*4;
            // MP
            int mp = 0;
            sscanf(v[levelindex].c_str(), "%d", &mp);
/*
            if(!mp || mp > 998){
                // MP܂͑傫Ȓl͓o^ȂƂ݂ȂB
                // vf48*3ɂ邽߂MP0œo^B
                magic.mp = 0;
                MagicEntryList.push_back(magic);
                continue;
            }
*/
            TMagicEntry magic;
            magic.isdeluge = false;
            const char *levelstr[3] = {"P", "Q", "R"};
            AnsiString str;
            str.sprintf("%s_%s%s", magicgroup_name[count/8], v[0].c_str(), levelstr[level-1]);
            magic.id = str.c_str();
            str.sprintf("%s%s", v[0].c_str(), levelstr[level-1]);
            magic.name = str.c_str();
            magic.consumemp = mp;
            magic.magictype = mtype;
            magic.powertype = POWERTYPE_MA;
            magic.guardtype = POWERTYPE_MD;
            magic.hittype = POWERTYPE_MA;
            magic.avoidtype = POWERTYPE_MD;
            magic.powerformula = FORMULA_FTMAGIC;
            magic.hitformula = FORMULA_ANYTIME;
            // ˒
            magic.range = 0;
            sscanf(v[levelindex+1].c_str(), "%d", &magic.range);
            // La
            magic.radius = 0;
            sscanf(v[levelindex+2].c_str(), "%d", &magic.radius);
            // З
            magic.power = 0;
            sscanf(v[levelindex+3].c_str(), "%d", &magic.power);
            // 
            AnsiString summonchara;
            switch(count/8){
            case 0:
                // 
                switch(count%8){
                case 7:
                    // HP[m
                    magic.script_after = "volcano";
                    // breakȂ
                case 0:
                case 6:
                    // t@CAAeIXgCNAHP[m
                    magic.attribute = ATTRIBUTE_FIRE;
                    break;
                case 1:
                    // LA|CY
                    magic.attribute = ATTRIBUTE_CURE_POISON;
                    break;
                case 2:
                    // A^bN
                    magic.attribute = ATTRIBUTE_PLUS_PA;
                    break;
                case 3:
                    // EH[^K[h
                    magic.attribute = ATTRIBUTE_CHANGE_RESISTANCE;
                    magic.restarget = attr_id[ATTRIBUTE_WATER];
                    magic.restype = attrres_id[2];  // 2:
                    magic.addstr = "h";
                    break;
                case 4:
                    // Gg
                    magic.attribute = ATTRIBUTE_SUMMON;
                    summonchara.sprintf("FireElement%d", level);
                    magic.summonchara = summonchara.c_str();
                    break;
                case 5:
                    // Jo[
                    magic.attribute = ATTRIBUTE_PLUS_HPRECOVER;
                    break;
                }
                break;
            case 1:
                // 
                switch(count%8){
                case 7:
                    // f[W
                    magic.script_after = "deluge";
                    magic.isdeluge = true;
                    // breakȂ
                case 0:
                case 6:
                    // t[YAuU[hAf[W
                    magic.attribute = ATTRIBUTE_WATER;
                    break;
                case 1:
                    // LAXg[
                    magic.attribute = ATTRIBUTE_CURE_STONE;
                    break;
                case 2:
                    // XL
                    magic.attribute = ATTRIBUTE_PLUS_SKILL;
                    break;
                case 3:
                    // tCK[h
                    magic.attribute = ATTRIBUTE_CHANGE_RESISTANCE;
                    magic.restarget = attr_id[ATTRIBUTE_FIRE];
                    magic.restype = attrres_id[2];  // 2:
                    magic.addstr = "Ζh";
                    break;
                case 4:
                    // Gg
                    magic.attribute = ATTRIBUTE_SUMMON;
                    summonchara.sprintf("WaterElement%d", level);
                    magic.summonchara = summonchara.c_str();
                    break;
                case 5:
                    // }WbNJo[
                    magic.attribute = ATTRIBUTE_PLUS_MPRECOVER;
                    break;
                }
                break;
            case 2:
                // 
                switch(count%8){
                case 0:
                case 6:
                case 7:
                    // GAJb^[ECgjOEgl[h
                    magic.attribute = ATTRIBUTE_WIND;
                    break;
                case 1:
                    // LApCY
                    magic.attribute = ATTRIBUTE_CURE_PARALYZE;
                    break;
                case 2:
                    // Xs[h
                    magic.attribute = ATTRIBUTE_PLUS_SPEED;
                    break;
                case 3:
                    // tC
                    magic.attribute = ATTRIBUTE_CHANGE_MOVETYPE;
                    magic.movetype = movetype_name[9];  // 9:s^CvID
                    break;
                case 4:
                    // Gg
                    magic.attribute = ATTRIBUTE_SUMMON;
                    summonchara.sprintf("WindElement%d", level);
                    magic.summonchara = summonchara.c_str();
                    break;
                case 5:
                    // AQ
                    magic.attribute = ATTRIBUTE_PLUS_ATTACKCOUNT;
                    magic.power = 1;
                    break;
                }
                break;
            case 3:
                // y
                switch(count%8){
                case 7:
                    // EH[X}bV
                    magic.magictype = 3;    // ꂾʂɏǔjwB
                    magic.attribute = ATTRIBUTE_CRASHWALL;
                    break;
                case 6:
                    magic.excludemt = movetype_name[9]; // s^Cv
                    // FALLTHROUGH
                case 0:
                    // AVbhNEhEA[XNEFCN
                    magic.attribute = ATTRIBUTE_EARTH;
                    break;
                case 1:
                    // LAX[v
                    magic.attribute = ATTRIBUTE_CURE_SLEEP;
                    break;
                case 2:
                    // A[}[
                    magic.attribute = ATTRIBUTE_PLUS_PD;
                    break;
                case 3:
                    // OreCg
                    magic.attribute = ATTRIBUTE_MINUS_ATTACKCOUNT;
                    magic.power = -1;
                    break;
                case 4:
                    // Gg
                    magic.attribute = ATTRIBUTE_SUMMON;
                    summonchara.sprintf("SoilElement%d", level);
                    magic.summonchara = summonchara.c_str();
                    break;
                case 5:
                    // Xg[GbW
                    magic.attribute = ATTRIBUTE_STONE;
                    break;
                }
                break;
            case 4:
                // 
                switch(count%8){
                case 0:
                case 7:
                    // VCjOAm@
                    magic.attribute = ATTRIBUTE_LIGHT;
                    break;
                case 1:
                case 4:
                    // q[AI[q[
                    magic.attribute = ATTRIBUTE_HPRECOVER;
                    break;
                case 2:
                    // WXg
                    magic.attribute = ATTRIBUTE_PLUS_MD;
                    break;
                case 3:
                    // LAI[
                    magic.attribute = ATTRIBUTE_CURE_ALL;
                    break;
                case 5:
                    // Gg
                    magic.attribute = ATTRIBUTE_SUMMON;
                    summonchara.sprintf("LightElement%d", level);
                    magic.summonchara = summonchara.c_str();
                    break;
                case 6:
                    // NCG
                    magic.attribute = ATTRIBUTE_LIGHT;
                    magic.target = "undead";
                    break;
                }
                break;
            case 5:
                // 
                switch(count%8){
                case 0:
                    // ubhTbN
                    magic.attribute = ATTRIBUTE_ABSORB;
                    break;
                case 1:
                    // |CYNbV
                    magic.attribute = ATTRIBUTE_POISON;
                    break;
                case 2:
                    // }WbN
                    magic.attribute = ATTRIBUTE_PLUS_MA;
                    break;
                case 3:
                    // pCY
                    magic.attribute = ATTRIBUTE_PARALYZE;
                    break;
                case 4:
                    // Gg
                    magic.attribute = ATTRIBUTE_SUMMON;
                    summonchara.sprintf("DarkElement%d", level);
                    magic.summonchara = summonchara.c_str();
                    break;
                case 5:
                    // C[W
                    magic.attribute = ATTRIBUTE_DREAM;
                    break;
                case 6:
                    // fX
                    magic.attribute = ATTRIBUTE_DEATH;
                    break;
                case 7:
                    // GNXv[W
                    magic.attribute = ATTRIBUTE_DARK;
                    break;
                }
                break;
            }
            // ˒^Cv
            if(mp && mp < 999){
                magic.rangetype = GetMagicRangeType(magic.magictype, magic.range, magic.isdeluge);
                if(!magic.rangetype){
                    OutputLog("@ %s ̎˒^Cv %d ˒ %d ̔Ɏs܂B", magic.id.c_str(), magic.magictype, magic.range);
                    return false;
                }
            }
            // ʉt@C
            str.sprintf("%s.wav", buf_sound);
            magic.sound = str.c_str();
            // 摜t@C
            str.sprintf("Magic%d.png", count);
            magic.imgfile = str.c_str();
            // 摜񂩂Aj[V߂B

            TImageData imgdata;
            TConvertedImageMap::const_iterator itr = ConvertedImageMap.find(magic.imgfile);
            if(itr == ConvertedImageMap.end()){
                // PicturezɖꍇADefault/PictureTB
                if(!defscedstpath.IsEmpty()){
                    AnsiString deffile;
                    deffile.sprintf("%s\\Picture\\%s", defscedstpath.c_str(), magic.imgfile.c_str());
                    if(!_access(deffile.c_str(), 0)){
                        // Default/PictureɂꍇwB
                        HDIB hdib = PNGtoDIB(deffile.c_str());
                        if(hdib == NULL){
                            OutputLog("摜t@C %s ݂͑܂摜̎擾̂߂̃[hɎs܂B", deffile.c_str());
                            return false;
                        }
                        if(!GetHDIBSize(hdib, &imgdata.width, &imgdata.height)){
                            OutputLog("摜t@C %s ݂͑܂摜̎擾Ɏs܂B", deffile.c_str());
                            DeleteDIB(hdib);
                            return false;
                        }
                        DeleteDIB(hdib);
                        str.sprintf("../../Default/Picture/Magic%d.png", count);
                        magic.imgfile = str.c_str();
                    }
                    else{
                        // Default/Pictureɂ΃G[B
                        OutputLog("PicturetH_ɖ摜t@C %s w肳܂B\n"
                                    "Ro[gΏۂDefaultViIȊȌꍇA%s ݂Ă΂̃G[ɂ͂Ȃ܂B\n"
                                    "DefaultViIɃRo[gΉłꍇ܂B",
                                    magic.imgfile.c_str(), deffile.c_str());
                        return false;
                    }
                }
                else{
                    OutputLog("PicturetH_ɖ摜t@C %s w肳܂B", magic.imgfile.c_str());
                    return false;
                }
            }
            else{
                imgdata = itr->second;
            }
            //int animwidth;
            if(count%8 < 6){
                magic.animwidth = 32;
                magic.animtype = ANIMATION_CHIP;
            }else{
                magic.animwidth = 128;
                magic.animtype = ANIMATION_SCREENCENTER;
            }
            magic.animcount = (imgdata.width / magic.animwidth) * (imgdata.height / magic.animwidth);

            if(!mp || mp > 998){
                // MP܂͑傫Ȓl͓o^ȂƂ݂ȂB
                // vf48*3ɂ邽߂MP-1œo^B
                magic.consumemp = -1;
            }
            else{
                // SpecialIDݒ
                SetTextAttribute(e_entry, "specialid", magic.id.c_str());
                e_magic.InsertEndChild(e_entry);
            }

            // mapɋL^
            MagicEntryList.push_back(magic);
        }
        // 
        TiXmlElement e_man("Guidance");
        InsertTextChild(e_man, v[14].c_str());
        e_magic.InsertEndChild(e_man);
        e_magiclist.InsertEndChild(e_magic);
    }
    doc.InsertEndChild(e_magiclist);
    fclose(fp);
    fclose(fp_sound);

    if(!doc.SaveFile()){
        return false;
    }

    return true;
}
//---------------------------------------------------------------------------
// SpecialTableSpecialTable.xml
// isFirstDatáAŏ̃f[^̂ݐ^w肷Biŏ̃f[^Default̃f[^邽߁j
// forceOutput = truȅꍇ\[XĂo͂B
bool TFarenCnv::ConvertSpecialTable(const char *srcpath, const char *dstpath, bool noDefault, bool isDefaultScenario, bool isFirstData, bool forceOutput, bool& exists, bool &defout)
{
    AnsiString src, dst;
    src.sprintf("%s\\SpecialTable", srcpath);
    dst.sprintf("%s\\SpecialTable.xml", dstpath);

    if(!forceOutput){
        // ̃t@C͖Ζł
        if(_access(src.c_str(), 0)){
            exists = false;
            return true;
        }
    }
    exists = true;

    TiXmlDocument doc(dst.c_str());
    TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
    doc.InsertEndChild(dec);

    std::vector<std::string> strlist;
    // DefaultSpecialTableɓǂށBDefaultł͓RȂB
    if(!noDefault && isFirstData && !isDefaultScenario){
        if(!LoadDefaultSpecialTable(srcpath, strlist)){
            return false;
        }
        defout = true;
    }
    else{
        defout = false;
    }

    // ǉsoĂB
    int etclinecount = strlist.size();
    if(!forceOutput){
        FILE *fp = fopen(src.c_str(), "r");
        if(fp == NULL){
            return false;
        }

        bool necro = false;
        for(;;){
            char buf[1024];
            if(fgets(buf, sizeof buf, fp) == NULL){
                break;
            }
            // ŌɃf[^ł͂ȂsViI邽߁A󔒂̗LŃf[^sƔ肷B
            if(strchr(buf, ' ') != NULL){
                strlist.push_back(buf);
                if(!isDefaultScenario && noDefault && !strncmp(buf, "앜", 8)){
                    necro = true;
                }
            }
        }
        // IWiViIŎ앜ΒǉB
        if(noDefault && !necro){
            strlist.push_back("앜            6 0 000  0 1 000 0 0  0 lo     32  7\n");
        }
        fclose(fp);
    }

    int linenum = 0;
    TiXmlElement e_top("SpecialList");
    for(unsigned u=0; u<strlist.size(); ++u){
        ++linenum;
        const char *line = strlist[u].c_str();
        std::vector<std::string> v;
        boost::algorithm::split(v, line, boost::is_space());
        // 炩ɗvfȂI
        if(v.size() < 13){
            break;
        }
        // 󕶎łȂvfz̑OɎĂ
        std::remove(v.begin(), v.end(), "");

        // dID邩B
        std::map<std::string, int>::iterator itr = m_SpecialIDCount.find(v[0]);
        if(itr == m_SpecialIDCount.end()){
            m_SpecialID.push_back(v[0]); // IDL^B
            m_SpecialIDCount[v[0]] = 1;  // IDo^
        }
        else{
            AnsiString id;
            id.sprintf("%s%d", v[0].c_str(), itr->second);
            m_SpecialID.push_back(id.c_str());
            ++itr->second;      // IDZ
        }

        // 摜̑݃`FbNB
        bool imgok = false;
        for(unsigned u=0; u<m_CharaDirList.size(); ++u){
            AnsiString imgchk;
            imgchk.sprintf("%s/../%s/%s.bmp", srcpath, m_CharaDirList[u].c_str(), v[0].c_str());
            if(!_access(imgchk.c_str(), 0)){
                imgok = true;
                break;
            }
        }
        if(!imgok){
            AnsiString imgchk2;
            imgchk2.sprintf("%s/../../Default/DefChar/%s.bmp", srcpath, v[0].c_str());
            if(_access(imgchk2.c_str(), 0)){
                OutputLog("Warning:Z %s ̉摜t@C %s.bmp Ȃ߁A̓Z𖳌ɂ܂B", v[0].c_str(), v[0].c_str());
                m_SpecialID.back().clear();
                continue;
            }
        }

        TMagicEntry specialEntry;

        specialEntry.id = m_SpecialID.back();
        // ˒
        sscanf(v[4].c_str(), "%d", &specialEntry.range);
        // 
        //int magicattr = -1;  // ނɕ̒lw肳ꂽꍇA͎ނŌ߂10ڂ̒l͖B
        int sp;
        if(sscanf(v[1].c_str(), "%d", &sp) != 1){
            // SpXy[XčĎsB
            std::string v1 = CutZenSpace(v[1]);
            // łʖڂȂ疳B
            if(sscanf(v1.c_str(), "%d", &sp) != 1){
                OutputLog("SpecialTable%ds(%s)2Ԗڂ̗̒l̎擾Ɏs߂̋Z𖳎܂B", linenum - etclinecount, v[0].c_str());
                continue;
            }
        }
        specialEntry.attribute = -1;
        specialEntry.animtype = ANIMATION_NONE;
        if(sp <= -1 && sp >= -48){
            int magicindex = MagicEntryList.size() + sp * 3;
            const TMagicEntry& magic = MagicEntryList[magicindex];
            OutputLog("Information:SpecialTable%ds(%s)2Ԗڂ̗̒l͈͊OłA-1`-48̒l͖@ʂƉ߂A@ %s Ɠ̋ZƂ܂B", linenum - etclinecount, v[0].c_str(), magic.id.c_str());
            // ܂邲ƃRs[B
            specialEntry = magic;
            // IDɖ߂B
            specialEntry.id = m_SpecialID.back();
            // @̑wB
            //specialEntry.attribute = magic.attribute;
            // @̎˒ނwB
            specialEntry.rangetype = GetMagicRangeType(magic.magictype, specialEntry.range, magic.isdeluge);
            if(!specialEntry.rangetype){
                OutputLog("Z %s ̎˒^Cv̔Ɏs܂B", specialEntry.id.c_str());
                return false;
            }
            // @̃Aj[VނwB
            //specialEntry.animtype = magic.animtype;
        }
        else{
            if(sp < 1 || sp > 8){
                OutputLog("SpecialTable%ds(%s)2Ԗڂ̗̒l͈͊Oi1`8̒lɑΉjB", linenum - etclinecount, v[0].c_str());
                return false;
            }
            // KEZ̃^CvɊ蓖Ă˒ށB
            static const TRangeType ranges[8] = {RANGE_NORMAL, RANGE_NORMAL, RANGE_ME, RANGE_NORMAL, RANGE_ME, RANGE_LASTDEADENEMY, RANGE_STREAM, RANGE_ME};
            specialEntry.rangetype = ranges[sp - 1];
        }

        // O
        specialEntry.name = v[0];
        // ˍU(1), uX(2)͉摜t@Cϊ̂߂ɖOo^B
        if(sp == 1){
            ShotImageFiles.insert(v[0]);
        }
        else if(sp == 2){
            BreathImageFiles.insert(v[0]);
        }
        // MP
        sscanf(v[3].c_str(), "%d", &specialEntry.consumemp);
        // La
        sscanf(v[5].c_str(), "%d", &specialEntry.radius);
        // З
        sscanf(v[6].c_str(), "%d", &specialEntry.power);
        // gp\͒li{̂ł͑SĂ̔\͒lT|[g邪IWi͂ꂾj
        int powertype = 0;
        sscanf(v[7].c_str(), "%d", &powertype);
        if(powertype < 0 || powertype > 2){
            OutputLog("SpecialTable%ds(%s)8Ԗڂ̗̒l͈͊Oi0`2̒lɑΉĂ܂j", linenum - etclinecount, v[0].c_str());
            return false;
        }
        static const TPowerType powerstatus[3] = {POWERTYPE_NONE, POWERTYPE_PA, POWERTYPE_MA};
        specialEntry.powertype = powerstatus[powertype];
        // gp\͒li󂯑j
        int guardtype = 0;
        sscanf(v[8].c_str(), "%d", &guardtype);
        if(guardtype == 3){
            OutputLog("SpecialTable%ds(%s)9Ԗڂ̗̒l͈͊O(3)łB0̊ԈႢƌȂ܂B", linenum - etclinecount, v[0].c_str());
            guardtype = 0;
        }
        else if(guardtype < 0 || guardtype > 2){
            OutputLog("SpecialTable%ds(%s)9Ԗڂ̗̒l͈͊Oi0`2̒lɑΉĂ܂j", linenum - etclinecount, v[0].c_str());
            return false;
        }
        static const TPowerType guardstatus[3] = {POWERTYPE_NONE, POWERTYPE_PD, POWERTYPE_MD};
        specialEntry.guardtype = guardstatus[guardtype];
        // gp\͒liZo̍Uߑj
        specialEntry.hittype = POWERTYPE_SKILL;
        // gp\͒liZo̎󂯑j
        specialEntry.avoidtype = POWERTYPE_SKILL;
        // 
        if(specialEntry.attribute == -1){
            int attr = 0;
            if(sscanf(v[9].c_str(), "%d", &attr) != 1){
                OutputLog("SpecialTable%ds(%s)10Ԗڂ̗̒l̎擾Ɏs܂B", linenum - etclinecount, v[0].c_str());
                return false;
            }
            if(attr > 17){
                OutputLog("SpecialTable%ds(%s)10Ԗڂ̗̒l͈͊Oi0`17̒lɑΉĂ܂j", linenum - etclinecount, v[0].c_str());
                return false;
            }
            if(attr > 0){
                specialEntry.attribute = attr;
            }
            else if(attr < 0){
                // ̒l͑=ϐB
                specialEntry.attribute = ATTRIBUTE_MAX + 21;
            }
            else{
                if(sp == 5){
                    // ǔj(5)
                    specialEntry.attribute = ATTRIBUTE_CRASHWALL;
                }
                else{
                    // 앜(6)
                    specialEntry.attribute = ATTRIBUTE_REVIVE;
                }
            }
        }
        if(sp == 6){
            // 앜(6)
            specialEntry.necromancy = true;
        }
        // _[WvZ
        specialEntry.powerformula = FORMULA_FTSPECIAL;
        // vZ
        if(guardtype == 1){
            specialEntry.hitformula = FORMULA_FTSPECIALHIT; // ΂ƂB
        }
        else{
            specialEntry.hitformula = FORMULA_ANYTIME;      // uXƂB
        }
        // ǉʖvZ
        if(specialEntry.attribute <= ATTRIBUTE_DEATH){
            specialEntry.addformula = FORMULA_FTSPECIALHIT;
        }
        else{
            specialEntry.addformula = FORMULA_ANYTIME;
        }
        // ƒǉʖ̘A֌WFɘAAǉAƗ3B
        specialEntry.hitjudge = HITJUDGE_SEPARATE;
        // 摜t@C
        AnsiString imgfile;
        imgfile.sprintf("%s.png", v[0].c_str());
        specialEntry.imgfile = imgfile.c_str();
        // 摜t@ĈPAj[ṼTCY
        int width;
        if(sscanf(v[11].c_str(), "%d", &width) != 1){
            OutputLog("SpecialTable%ds(%s)12Ԗڂ̗̒lǂ߂܂łB", linenum - etclinecount, v[0].c_str());
            return false;
        }
        // Aj[V
        int anicount;
        if(sscanf(v[12].c_str(), "%d", &anicount) != 1){
            OutputLog("SpecialTable%ds(%s)13Ԗڂ̗̒lǂ߂܂łB", linenum - etclinecount, v[0].c_str());
            return false;
        }
        if(sp == 1){
            // |n̓Aj[VȂ̂ŏCB
            anicount = 1;
        }
        else if(sp == 2){
            // uX͏Ă鐔Ȃ̂ŏCB
            width *= 3;
            anicount /= 7;
        }
        else if(sp == 8){
            // ̓Aj[VoĂB
            TKokuryuData koku;
            koku.width = width;
            koku.anicount = anicount;
            KokuryuImageFiles[v[0]] = koku;
        }
        const TAnimationType anitype[8] = {ANIMATION_SHOT, ANIMATION_CENTER, ANIMATION_CHIP, ANIMATION_CHIP, ANIMATION_CHIP, ANIMATION_CHIP, ANIMATION_STREAM, ANIMATION_CENTER};
        specialEntry.animwidth = width;
        specialEntry.animcount = anicount;
        if(specialEntry.animtype == ANIMATION_NONE){
            specialEntry.animtype = anitype[sp - 1];
        }
        // ʉt@C
        AnsiString soundfile;
        soundfile.sprintf("%s.wav", v[10].c_str());
        specialEntry.sound = soundfile.c_str();

        if(!OutputMagicEntry(e_top, specialEntry)){
            return false;
        }
    }

    if(!isFirstData){
        doc.InsertEndChild(e_top);
    
        if(!doc.SaveFile()){
            return false;
        }
    
        return true;
    }

/*
    for(unsigned u=0; u<strlist.size(); ++u){
        ++linenum;
        const char *line = strlist[u].c_str();
        std::vector<std::string> v;
        boost::algorithm::split(v, line, boost::is_space());
        // 炩ɗvfȂI
        if(v.size() < 13){
            break;
        }
        // 󕶎łȂvfz̑OɎĂ
        std::remove(v.begin(), v.end(), "");

        // dID邩B
        std::map<std::string, int>::iterator itr = m_SpecialIDCount.find(v[0]);
        if(itr == m_SpecialIDCount.end()){
            m_SpecialID.push_back(v[0]); // IDL^B
            m_SpecialIDCount[v[0]] = 1;  // IDo^
        }
        else{
            AnsiString id;
            id.sprintf("%s%d", v[0].c_str(), itr->second);
            m_SpecialID.push_back(id.c_str());
            ++itr->second;      // IDZ
        }

        // 摜̑݃`FbNB
        bool imgok = false;
        for(unsigned u=0; u<m_CharaDirList.size(); ++u){
            AnsiString imgchk;
            imgchk.sprintf("%s/../%s/%s.bmp", srcpath, m_CharaDirList[u].c_str(), v[0].c_str());
            if(!_access(imgchk.c_str(), 0)){
                imgok = true;
                break;
            }
        }
        if(!imgok){
            AnsiString imgchk2;
            imgchk2.sprintf("%s/../../Default/DefChar/%s.bmp", srcpath, v[0].c_str());
            if(_access(imgchk2.c_str(), 0)){
                OutputLog("Warning:Z %s ̉摜t@C %s.bmp Ȃ߁A̓Z𖳌ɂ܂B", v[0].c_str(), v[0].c_str());
                m_SpecialID.back().clear();
                continue;
            }
        }

        TiXmlElement e_entry("Special");
        AnsiString str;
        SetTextAttribute(e_entry, "id", m_SpecialID.back().c_str());
        // 
        int sp = 0;
        int magicattr = -1;  // ނɕ̒lw肳ꂽꍇA͎ނŌ߂10ڂ̒l͖B
        sscanf(v[1].c_str(), "%d", &sp);
        if(!sp){
            // SpXy[XčĎsB
            std::string v1 = CutZenSpace(v[1]);
            sscanf(v1.c_str(), "%d", &sp);
            // łʖڂȂ疳B
            if(!sp){
                //m_SpecialID.push_back(v[0]); // IDL^i炳Ȃ߁jB
                OutputLog("SpecialTable%ds(%s)2Ԗڂ̗̒l̎擾Ɏs߂̋Z𖳎܂B", linenum - etclinecount, v[0].c_str());
                continue;
            }
        }
        if(sp <= -1 && sp >= -48){
            int magicindex = MagicEntryList.size() + sp * 3;
            const TMagicEntry& magic = MagicEntryList[magicindex];
            OutputLog("Information:SpecialTable%ds(%s)2Ԗڂ̗̒l͈͊OłA-1`-48̒l͖@ʂƉ߂A@ %s Ɠ̋ZƂ܂B", linenum - etclinecount, v[0].c_str(), magic.id.c_str());
            // @̑wB
            magicattr = magic.attribute;
            // @̎˒ނwB
            std::string rangestr;
            if(!GetMagicRangeType(magic, rangestr)){
                return false;
            }
            e_entry.SetAttribute("rangetype", rangestr.c_str());
        }
        else{
            if(sp < 1 || sp > 8){
                OutputLog("SpecialTable%ds(%s)2Ԗڂ̗̒l͈͊Oi1`8̒lɑΉjB", linenum - etclinecount, v[0].c_str());
                return false;
            }
            // KEZ̃^CvɊ蓖Ă˒ށB
            static const char* rangestr[8] = {"Normal", "Normal", "Me", "Normal", "Me", "LastDeadEnemy", "Stream", "Me"};
            e_entry.SetAttribute("rangetype", rangestr[sp - 1]);
        }

        // O
        TiXmlElement e_name("Name");
        InsertTextChild(e_name, v[0].c_str());
        e_entry.InsertEndChild(e_name);
        // ˍU(1), uX(2)͉摜t@Cϊ̂߂ɖOo^B
        if(sp == 1){
            ShotImageFiles.insert(v[0].c_str());
        }
        else if(sp == 2){
            BreathImageFiles.insert(v[0].c_str());
        }
        // MP
        int mp = 0;
        sscanf(v[3].c_str(), "%d", &mp);
        TiXmlElement e_mp("ConsumeMP");
        e_mp.SetAttribute("value", mp);
        e_entry.InsertEndChild(e_mp);
        // ˒
        int range = 0;
        sscanf(v[4].c_str(), "%d", &range);
        TiXmlElement e_range("MaxRange");
        e_range.SetAttribute("value", range);
        e_entry.InsertEndChild(e_range);
        // La
        int radius = 0;
        sscanf(v[5].c_str(), "%d", &radius);
        TiXmlElement e_radius("MaxRadius");
        --radius;
        e_radius.SetAttribute("value", radius);
        e_entry.InsertEndChild(e_radius);
        // З
        int power = 0;
        sscanf(v[6].c_str(), "%d", &power);
        TiXmlElement e_power("Power");
        e_power.SetAttribute("value", power);
        e_entry.InsertEndChild(e_power);
        // gp\͒li{̂ł͑SĂ̔\͒lT|[g邪IWi͂ꂾj
        static const char* powerstr[3] = {"None", "PhysicalAttack", "MagicAttack"};
        int powertype = 0;
        sscanf(v[7].c_str(), "%d", &powertype);
        if(powertype < 0 || powertype > 2){
            OutputLog("SpecialTable%ds(%s)8Ԗڂ̗̒l͈͊Oi0`2̒lɑΉĂ܂j", linenum - etclinecount, v[0].c_str());
            return false;
        }
        TiXmlElement e_powertype("PowerType");
        InsertTextChild(e_powertype, powerstr[powertype]);
        e_entry.InsertEndChild(e_powertype);
        // gp\͒li󂯑j
        static const char* guardstr[3] = {"None", "PhysicalDefense", "MagicDefense"};
        int guardtype = 0;
        sscanf(v[8].c_str(), "%d", &guardtype);
        if(guardtype < 0 || guardtype > 2){
            OutputLog("SpecialTable%ds(%s)9Ԗڂ̗̒l͈͊Oi0`2̒lɑΉĂ܂j", linenum - etclinecount, v[0].c_str());
            return false;
        }
        TiXmlElement e_guardtype("GuardType");
        InsertTextChild(e_guardtype, guardstr[guardtype]);
        e_entry.InsertEndChild(e_guardtype);
        // gp\͒liZo̍Uߑj
        TiXmlElement e_hittype("HitType");
        InsertTextChild(e_hittype, "Skill");
        e_entry.InsertEndChild(e_hittype);
        // gp\͒liZo̎󂯑j
        TiXmlElement e_avoidtype("AvoidType");
        InsertTextChild(e_avoidtype, "Skill");
        e_entry.InsertEndChild(e_avoidtype);
        // ̎
        bool anytime;
        // 
        TiXmlElement e_attr("Attribute");
        if(magicattr == -1){
            int attr = -1;
            sscanf(v[9].c_str(), "%d", &attr);
            if(attr < 0 || attr > 17){
                OutputLog("SpecialTable%ds(%s)10Ԗڂ̗̒l͈͊Oi0`17̒lɑΉĂ܂j", linenum - etclinecount, v[0].c_str());
                return false;
            }
            if(attr){
                InsertTextChild(e_attr, attr_id[attr]);
            }
            else{
                if(sp == 5){
                    // ǔj(5)
                    InsertTextChild(e_attr, attr_id[ATTRIBUTE_CRASHWALL]);
                }
                else{
                    // 앜(6)
                    InsertTextChild(e_attr, attr_id[ATTRIBUTE_REVIVE]);
                }
            }
            anytime = (attr > ATTRIBUTE_DEATH);    // ܂ł͖̑vZwB
        }
        else{
            if(magicattr < 0 || magicattr > ATTRIBUTE_FULLMAX - 1){
                OutputLog("SpecialTable%ds(%s)2Ԗڂ̗̒lɕ̒lw肳ꂽ߁A10Ԗڂ̗̒lgpɑ肵悤Ƃ܂s܂(%d)B", linenum - etclinecount, v[0].c_str(), magicattr);
                return false;
            }
            InsertTextChild(e_attr, attr_id[magicattr]);
            anytime = (magicattr > ATTRIBUTE_DEATH);    // ܂ł͖̑vZwB
        }
        e_entry.InsertEndChild(e_attr);
        if(sp == 6){
            // 앜(6)
            TiXmlElement e_rev("Revive");
            SetTextAttribute(e_rev, "id", REVIVE_NECROMANCY);
            e_entry.InsertEndChild(e_rev);
        }
        // _[WvZ
        TiXmlElement e_formula("PowerFormula");
        InsertTextChild(e_formula, "FTSpecial");
        e_entry.InsertEndChild(e_formula);
        // vZ
        TiXmlElement e_hit("HitFormula");
        if(guardtype == 1){
            InsertTextChild(e_hit, "FTSpecialHit");     // ΂ƂB
        }
        else{
            InsertTextChild(e_hit, "Anytime");          // uXƂB
        }
        e_entry.InsertEndChild(e_hit);
        // ǉʖvZ
        TiXmlElement e_addhit("AdditionalHitFormula");
        if(anytime){
            InsertTextChild(e_addhit, "Anytime");
        }
        else{
            InsertTextChild(e_addhit, "FTSpecialHit");
        }
        e_entry.InsertEndChild(e_addhit);
        // ƒǉʖ̘A֌WFɘAAǉAƗ3B
        TiXmlElement e_judge("HitJudgeType");
        InsertTextChild(e_judge, "SeparateJudge");
        e_entry.InsertEndChild(e_judge);
        // 摜t@C
        AnsiString imgfile;
        imgfile.sprintf("%s.png", v[0].c_str());
        TiXmlElement e_image("Image");
        InsertTextChild(e_image, imgfile.c_str());
        // 摜t@ĈPAj[ṼTCY
        int width;
        if(sscanf(v[11].c_str(), "%d", &width) != 1){
            OutputLog("SpecialTable%ds(%s)12Ԗڂ̗̒lǂ߂܂łB", linenum - etclinecount, v[0].c_str());
            return false;
        }
        // Aj[V
        int anicount;
        if(sscanf(v[12].c_str(), "%d", &anicount) != 1){
            OutputLog("SpecialTable%ds(%s)13Ԗڂ̗̒lǂ߂܂łB", linenum - etclinecount, v[0].c_str());
            return false;
        }
        if(sp == 1){
            // |n̓Aj[VȂ̂ŏCB
            anicount = 1;
        }
        else if(sp == 2){
            // uX͏Ă鐔Ȃ̂ŏCB
            width *= 3;
            anicount /= 7;
        }
        else if(sp == 8){
            // ̓Aj[VoĂB
            TKokuryuData koku;
            koku.width = width;
            koku.anicount = anicount;
            KokuryuImageFiles[v[0]] = koku;
        }
        const char *anitype[8] = {"Shot", "Center", "Chip", "Chip", "Chip", "Chip", "Stream", "Center"};
        e_image.SetAttribute("width", width);
        e_image.SetAttribute("height", width);
        e_image.SetAttribute("animation", anicount);
        e_image.SetAttribute("type", anitype[sp-1]);
        e_entry.InsertEndChild(e_image);
        // ʉt@C
        AnsiString soundfile;
        soundfile.sprintf("%s.wav", v[10].c_str());
        TiXmlElement e_sound("Sound");
        InsertTextChild(e_sound, soundfile.c_str());
        e_entry.InsertEndChild(e_sound);

        e_top.InsertEndChild(e_entry);
    }
*/

    // eUB@ȂyȂ̂Ŗ@XgɓB
    for(int atkindex = ATTRIBUTE_PHYSICAL; atkindex < ATTRIBUTE_DARK; ++atkindex){
        TMagicEntry atk;
        AnsiString str;
        str.sprintf("Attack%s", attr_id[atkindex]);
        atk.id = str.c_str();
        str.sprintf("%sU", attr_shortname[atkindex]);
        atk.name = str.c_str();
        atk.consumemp = 0;
        atk.magictype = 2;  // 2:U
        atk.rangetype = RANGE_NORMAL;
        atk.range = 1;      // gȂꉞB
        atk.radius = 0;     // B
        atk.power = 0;      // gȂB
        atk.attribute = atkindex;
        atk.animwidth = 0;
        atk.animcount = 0;
        atk.animtype = ANIMATION_NONE;
        atk.powertype = POWERTYPE_PA;
        atk.guardtype = POWERTYPE_PD;
        atk.hittype = POWERTYPE_SKILL;
        atk.avoidtype = POWERTYPE_SKILL;
        atk.powerformula = FORMULA_FTATTACK;
        atk.hitformula = FORMULA_FTATTACKHIT;
        //atk.screencenter = false;
        atk.isdeluge = false;
        atk.sound = "U.wav";
        MagicEntryList.push_back(atk);
    }

    // UɂǔjB
    {
        TMagicEntry atk;
        atk.id = "AttackWall";
        atk.name = "ǔj";
        atk.consumemp = 0;
        atk.magictype = 2;  // 2:U
        atk.rangetype = RANGE_NORMAL;
        atk.range = 1;      // gȂꉞB
        atk.radius = 1;
        atk.power = 0;      // gȂB
        atk.attribute = ATTRIBUTE_CRASHWALL;
        atk.animwidth = 0;
        atk.animcount = 0;
        atk.animtype = ANIMATION_NONE;
        atk.powertype = POWERTYPE_PA;
        atk.guardtype = POWERTYPE_PD;
        atk.hittype = POWERTYPE_SKILL;
        atk.avoidtype = POWERTYPE_SKILL;
        atk.powerformula = FORMULA_FTATTACK;
        atk.hitformula = FORMULA_ANYTIME;
        //atk.screencenter = false;
        atk.isdeluge = false;
        atk.sound = "U.wav";
        MagicEntryList.push_back(atk);
    }

    // RNKCB
    for(unsigned u=0; u<MasterRevivalList.size(); ++u){
        TMagicEntry revival;
        AnsiString speid;
        speid.sprintf("%s%u", MASTER_REVIVAL, u);
        revival.id = speid.c_str();
        AnsiString revname;
        revname.sprintf("%u", u);
        revival.name = revname.c_str();
        revival.consumemp = 0;
        revival.magictype = 4;  // 4:}X^[
        revival.rangetype = RANGE_ME;
        revival.range = 0;      // gȂꉞB
        revival.radius = 1;     // B
        revival.power = 0;      // gȂB
        revival.attribute = ATTRIBUTE_REVIVE;
        AnsiString revid;
        revid.sprintf("%s%u", REVIVE_ME, u);
        revival.reviveid = revid.c_str();
        revival.script_before = speid.c_str();
        revival.animwidth = 0;
        revival.animcount = 0;
        revival.animtype = ANIMATION_NONE;
        revival.powertype = POWERTYPE_PA;
        revival.guardtype = POWERTYPE_PD;
        revival.hittype = POWERTYPE_SKILL;
        revival.avoidtype = POWERTYPE_SKILL;
        revival.powerformula = FORMULA_FTATTACK;
        revival.hitformula = FORMULA_ANYTIME;
        //revival.screencenter = false;
        revival.isdeluge = false;
        MagicEntryList.push_back(revival);
    }

    // @f[^oB
    for(unsigned u=0; u<MagicEntryList.size(); ++u){
        const TMagicEntry& magic = MagicEntryList[u];
        if(magic.consumemp != -1 && !OutputMagicEntry(e_top, magic)){
            return false;
        }
/*
        TiXmlElement e_entry("Special");
        AnsiString str;
        SetTextAttribute(e_entry, "id", magic.id.c_str());
        switch(magic.magictype){
        case 0: // 
            if(magic.range){
                e_entry.SetAttribute("rangetype", "Normal");
            }
            else{
                OutputLog("@ %s ͏^Cvł˒0ɂȂĂ܂B", magic.id.c_str());
                return false;
            }
            break;
        case 1: // ⏕
            if(magic.range){
                e_entry.SetAttribute("rangetype", "Normal");
            }
            else{
                e_entry.SetAttribute("rangetype", "AllMySide");
            }
            break;
        case 2: // U
            if(magic.range){
                e_entry.SetAttribute("rangetype", "Normal");
            }
            else{
                if(magic.isdeluge){
                    e_entry.SetAttribute("rangetype", "LowerThan");
                }
                else{
                    e_entry.SetAttribute("rangetype", "AllEnemy");
                }
            }
            break;
        case 3: // ǔj
            if(magic.range){
                e_entry.SetAttribute("rangetype", "Normal");
            }
            else{
                e_entry.SetAttribute("rangetype", "AllWall");
            }
            break;
        case 4: // }X^[
            e_entry.SetAttribute("rangetype", "Me");
            break;
        default:
            OutputLog("@ %s ɖΉ^Cv %d w肳܂B", magic.id.c_str(), magic.magictype);
            return false;   // ɗ͂͂ȂB
        }
        // f[W͊xwB
        if(magic.isdeluge){
            e_entry.SetAttribute("rangeheight", "3");
        }
        // iKEZƕтႤo͂l͋ʂɂȂj
        static const char* spstr2[5] = {"Summon", "Boost", "Attack", "CrashWall", "Auto"};
        e_entry.SetAttribute("effecttype", spstr2[magic.magictype]);

        // O
        TiXmlElement e_name("Name");
        InsertTextChild(e_name, magic.name.c_str());
        e_entry.InsertEndChild(e_name);
        // MP
        TiXmlElement e_mp("ConsumeMP");
        e_mp.SetAttribute("value", magic.consumemp);
        e_entry.InsertEndChild(e_mp);
        // ˒
        TiXmlElement e_range("MaxRange");
        e_range.SetAttribute("value", magic.range);
        e_entry.InsertEndChild(e_range);
        // La
        TiXmlElement e_radius("MaxRadius");
        e_radius.SetAttribute("value", magic.radius - 1);
        e_entry.InsertEndChild(e_radius);
        // \ǉiΖhAhj
        if(!magic.addstr.empty()){
            TiXmlElement e_addstr("AdditionalString");
            SetTextAttribute(e_addstr, "value", magic.addstr.c_str());
            e_entry.InsertEndChild(e_addstr);
        }
        // g^[QbgiNCGj
        if(!magic.target.empty()){
            TiXmlElement e_target("Target");
            e_target.SetAttribute("id", magic.target.c_str());
            e_entry.InsertEndChild(e_target);
        }
        // Oړ^Cv
        if(!magic.excludemt.empty()){
            TiXmlElement e_exmt("ExcludeMoveType");
            SetTextAttribute(e_exmt, "id", magic.excludemt.c_str());
            e_entry.InsertEndChild(e_exmt);
        }
        // З
        TiXmlElement e_power("Power");
        e_power.SetAttribute("value", magic.power);
        e_entry.InsertEndChild(e_power);
        // gp\͒ligpEŒj
        TiXmlElement e_powertype("PowerType");
        if(magic.animwidth){
            InsertTextChild(e_powertype, "MagicAttack");
        }
        else{
            InsertTextChild(e_powertype, "PhysicalAttack");
        }
        e_entry.InsertEndChild(e_powertype);
        // gp\͒li󂯑EŒj
        TiXmlElement e_guardtype("GuardType");
        if(magic.animwidth){
            InsertTextChild(e_guardtype, "MagicDefense");
        }
        else{
            InsertTextChild(e_guardtype, "PhysicalDefense");
        }
        e_entry.InsertEndChild(e_guardtype);
        // gp\͒liZo̍Uߑj
        TiXmlElement e_hittype("HitType");
        if(magic.animwidth){
            InsertTextChild(e_hittype, "MagicAttack");
        }
        else{
            InsertTextChild(e_hittype, "Skill");
        }
        e_entry.InsertEndChild(e_hittype);
        // gp\͒liZo̎󂯑j
        TiXmlElement e_avoidtype("AvoidType");
        if(magic.animwidth){
            InsertTextChild(e_avoidtype, "MagicDefense");
        }
        else{
            InsertTextChild(e_avoidtype, "Skill");
        }
        e_entry.InsertEndChild(e_avoidtype);
        // 
        TiXmlElement e_attr("Attribute");
        InsertTextChild(e_attr, attr_id[magic.attribute]);
        e_entry.InsertEndChild(e_attr);
        // ΖhEh
        if(!magic.restarget.empty()){
            TiXmlElement e_res("Resistance");
            e_res.SetAttribute("attribute", magic.restarget.c_str());
            e_res.SetAttribute("resistance", magic.restype.c_str());
            e_entry.InsertEndChild(e_res);
        }
        // tC
        if(!magic.movetype.empty()){
            TiXmlElement e_mt("MoveType");
            SetTextAttribute(e_mt, "id", magic.movetype.c_str());
            e_entry.InsertEndChild(e_mt);
        }
        // Gg
        if(!magic.summonchara.empty()){
            TiXmlElement e_summon("SummonCharacter");
            e_summon.SetAttribute("id", magic.summonchara.c_str());
            e_entry.InsertEndChild(e_summon);
        }
        // XNvg֐
        if(!magic.script_before.empty()){
            TiXmlElement e_script("Script");
            e_script.SetAttribute("timing", "before");
            e_script.SetAttribute("function", magic.script_before.c_str());
            e_entry.InsertEndChild(e_script);
        }
        if(!magic.script_after.empty()){
            TiXmlElement e_script("Script");
            e_script.SetAttribute("timing", "after");
            e_script.SetAttribute("function", magic.script_after.c_str());
            e_entry.InsertEndChild(e_script);
        }
        if(magic.magictype == 4){
            // RNKC
            TiXmlElement e_rev("Revive");
            SetTextAttribute(e_rev, "id", magic.reviveid.c_str());
            e_entry.InsertEndChild(e_rev);
        }
        // _[WvZ
        TiXmlElement e_formula("PowerFormula");
        if(magic.animwidth){
            InsertTextChild(e_formula, "FTMagic");
        }
        else{
            InsertTextChild(e_formula, "FTAttack");
        }
        e_entry.InsertEndChild(e_formula);
        // vZ
        TiXmlElement e_hit("HitFormula");
        if(magic.animwidth){
            InsertTextChild(e_hit, "Anytime");
        }
        else{
            InsertTextChild(e_hit, "FTAttackHit");
        }
        e_entry.InsertEndChild(e_hit);
        // ǉʖvZ
        TiXmlElement e_addhit("AdditionalHitFormula");
        if(magic.magictype == 2){
            if(magic.animwidth){
                if(magic.attribute != ATTRIBUTE_ABSORB){
                    InsertTextChild(e_addhit, "FTMagicHit");
                }
                else{
                    InsertTextChild(e_addhit, "Anytime");
                }
            }
            else{
                InsertTextChild(e_addhit, "FTAttackHit");
            }
        }
        else{
            InsertTextChild(e_addhit, "Anytime");
        }
        e_entry.InsertEndChild(e_addhit);
        // ƒǉʖ̘A֌WFɘAAǉAƗ3B
        TiXmlElement e_judge("HitJudgeType");
        InsertTextChild(e_judge, "DoubleJudge");
        e_entry.InsertEndChild(e_judge);
        if(magic.animwidth){
            // 摜t@C
            TiXmlElement e_image("Image");
            InsertTextChild(e_image, magic.imgfile.c_str());
            // 摜t@ĈPAj[ṼTCY
            e_image.SetAttribute("width", magic.animwidth);
            e_image.SetAttribute("height", magic.animwidth);
            e_image.SetAttribute("animation", magic.animcount);
            if(!magic.screencenter){
                e_image.SetAttribute("type", "Chip");
            }
            else{
                e_image.SetAttribute("type", "ScreenCenter");
            }
            e_entry.InsertEndChild(e_image);
        }
        // ʉt@C
        TiXmlElement e_sound("Sound");
        InsertTextChild(e_sound, magic.sound.c_str());
        e_entry.InsertEndChild(e_sound);
        e_top.InsertEndChild(e_entry);
*/
        m_SpecialID.push_back(magic.id); // IDL^B
    }
    doc.InsertEndChild(e_top);

    if(!doc.SaveFile()){
        return false;
    }

    return true;
}
//---------------------------------------------------------------------------
bool TFarenCnv::OutputMagicEntry(TiXmlElement& e, const TMagicEntry& magic) const
{
    TiXmlElement e_entry("Special");
    AnsiString str;
    SetTextAttribute(e_entry, "id", magic.id.c_str());
/*
    switch(magic.magictype){
    case 0: // 
        if(magic.range){
            e_entry.SetAttribute("rangetype", "Normal");
        }
        else{
            OutputLog("@ %s ͏^Cvł˒0ɂȂĂ܂B", magic.id.c_str());
            return false;
        }
        break;
    case 1: // ⏕
        if(magic.range){
            e_entry.SetAttribute("rangetype", "Normal");
        }
        else{
            e_entry.SetAttribute("rangetype", "AllMySide");
        }
        break;
    case 2: // U
        if(magic.range){
            e_entry.SetAttribute("rangetype", "Normal");
        }
        else{
            if(magic.isdeluge){
                e_entry.SetAttribute("rangetype", "LowerThan");
            }
            else{
                e_entry.SetAttribute("rangetype", "AllEnemy");
            }
        }
        break;
    case 3: // ǔj
        if(magic.range){
            e_entry.SetAttribute("rangetype", "Normal");
        }
        else{
            e_entry.SetAttribute("rangetype", "AllWall");
        }
        break;
    case 4: // }X^[
        e_entry.SetAttribute("rangetype", "Me");
        break;
    default:
        OutputLog("@ %s ɖΉ^Cv %d w肳܂B", magic.id.c_str(), magic.magictype);
        return false;   // ɗ͂͂ȂB
    }
*/
/*
    TRangeType rangetype = GetMagicRangeType(magic.magictype, magic.range);
    if(!rangetype){
        OutputLog("@ %s ̎˒^Cv %d ˒ %d ̔Ɏs܂B", magic.id.c_str(), magic.magictype, magic.range);
        return false;
    }
*/
    e_entry.SetAttribute("rangetype", RangeTypeID[magic.rangetype]);
    // f[W͊xwB
    if(magic.isdeluge){
        e_entry.SetAttribute("rangeheight", "3");
    }
    // iKEZƕтႤo͂l͋ʂɂȂj
/*    static const char* spstr2[5] = {"Summon", "Boost", "Attack", "CrashWall", "Auto"};
    e_entry.SetAttribute("effecttype", spstr2[magic.magictype]);*/

    // O
    TiXmlElement e_name("Name");
    InsertTextChild(e_name, magic.name.c_str());
    e_entry.InsertEndChild(e_name);
    // MP
    TiXmlElement e_mp("ConsumeMP");
    e_mp.SetAttribute("value", magic.consumemp);
    e_entry.InsertEndChild(e_mp);
    // ˒
    TiXmlElement e_range("MaxRange");
    e_range.SetAttribute("value", magic.range);
    e_entry.InsertEndChild(e_range);
    // La
    TiXmlElement e_radius("MaxRadius");
    e_radius.SetAttribute("value", magic.radius - 1);
    e_entry.InsertEndChild(e_radius);
    // \ǉiΖhAhj
    if(!magic.addstr.empty()){
        TiXmlElement e_addstr("AdditionalString");
        SetTextAttribute(e_addstr, "value", magic.addstr.c_str());
        e_entry.InsertEndChild(e_addstr);
    }
    // g^[QbgiNCGj
    if(!magic.target.empty()){
        TiXmlElement e_target("Target");
        e_target.SetAttribute("id", magic.target.c_str());
        e_entry.InsertEndChild(e_target);
    }
    // Oړ^Cv
    if(!magic.excludemt.empty()){
        TiXmlElement e_exmt("ExcludeMoveType");
        SetTextAttribute(e_exmt, "id", magic.excludemt.c_str());
        e_entry.InsertEndChild(e_exmt);
    }
    // З
    TiXmlElement e_power("Power");
    e_power.SetAttribute("value", magic.power);
    e_entry.InsertEndChild(e_power);
    // gp\͒ligpEŒj
    TiXmlElement e_powertype("PowerType");
    InsertTextChild(e_powertype, PowerTypeID[magic.powertype]);
//    if(magic.animwidth){
//        InsertTextChild(e_powertype, "MagicAttack");
//    }
//    else{
//        InsertTextChild(e_powertype, "PhysicalAttack");
//    }
    e_entry.InsertEndChild(e_powertype);
    // gp\͒li󂯑EŒj
    TiXmlElement e_guardtype("GuardType");
    InsertTextChild(e_guardtype, PowerTypeID[magic.guardtype]);
//    if(magic.animwidth){
//        InsertTextChild(e_guardtype, "MagicDefense");
//    }
//    else{
//        InsertTextChild(e_guardtype, "PhysicalDefense");
//    }
    e_entry.InsertEndChild(e_guardtype);
    // gp\͒liZo̍Uߑj
    TiXmlElement e_hittype("HitType");
    InsertTextChild(e_hittype, PowerTypeID[magic.hittype]);
//    if(magic.animwidth){
//        InsertTextChild(e_hittype, "MagicAttack");
//    }
//    else{
//        InsertTextChild(e_hittype, "Skill");
//    }
    e_entry.InsertEndChild(e_hittype);
    // gp\͒liZo̎󂯑j
    TiXmlElement e_avoidtype("AvoidType");
    InsertTextChild(e_avoidtype, PowerTypeID[magic.avoidtype]);
//    if(magic.animwidth){
//        InsertTextChild(e_avoidtype, "MagicDefense");
//    }
//    else{
//        InsertTextChild(e_avoidtype, "Skill");
//    }
    e_entry.InsertEndChild(e_avoidtype);
    // 
    TiXmlElement e_attr("Attribute");
    InsertTextChild(e_attr, attr_id[magic.attribute]);
    e_entry.InsertEndChild(e_attr);
    // ΖhEh
    if(!magic.restarget.empty()){
        TiXmlElement e_res("Resistance");
        e_res.SetAttribute("attribute", magic.restarget.c_str());
        e_res.SetAttribute("resistance", magic.restype.c_str());
        e_entry.InsertEndChild(e_res);
    }
    // tC
    if(!magic.movetype.empty()){
        TiXmlElement e_mt("MoveType");
        SetTextAttribute(e_mt, "id", magic.movetype.c_str());
        e_entry.InsertEndChild(e_mt);
    }
    // Gg
    if(!magic.summonchara.empty()){
        TiXmlElement e_summon("SummonCharacter");
        e_summon.SetAttribute("id", magic.summonchara.c_str());
        e_entry.InsertEndChild(e_summon);
    }
    // XNvg֐
    if(!magic.script_before.empty()){
        TiXmlElement e_script("Script");
        e_script.SetAttribute("timing", "before");
        e_script.SetAttribute("function", magic.script_before.c_str());
        e_entry.InsertEndChild(e_script);
    }
    if(!magic.script_after.empty()){
        TiXmlElement e_script("Script");
        e_script.SetAttribute("timing", "after");
        e_script.SetAttribute("function", magic.script_after.c_str());
        e_entry.InsertEndChild(e_script);
    }
    if(magic.magictype == 4){
        // RNKC
        TiXmlElement e_rev("Revive");
        SetTextAttribute(e_rev, "id", magic.reviveid.c_str());
        e_entry.InsertEndChild(e_rev);
    }
    // 앜
    if(magic.necromancy){
        TiXmlElement e_rev("Revive");
        SetTextAttribute(e_rev, "id", REVIVE_NECROMANCY);
        e_entry.InsertEndChild(e_rev);
    }
    // _[WvZ
    TiXmlElement e_formula("PowerFormula");
    InsertTextChild(e_formula, FormulaID[magic.powerformula]);
//    if(magic.animwidth){
//        InsertTextChild(e_formula, "FTMagic");
//    }
//    else{
//        InsertTextChild(e_formula, "FTAttack");
//    }
    e_entry.InsertEndChild(e_formula);
    // vZ
    TiXmlElement e_hit("HitFormula");
    InsertTextChild(e_hit, FormulaID[magic.hitformula]);
//    if(magic.animwidth){
//        InsertTextChild(e_hit, "Anytime");
//    }
//    else{
//        InsertTextChild(e_hit, "FTAttackHit");
//    }
    e_entry.InsertEndChild(e_hit);
    // ǉʖvZ
    TiXmlElement e_addhit("AdditionalHitFormula");
    if(magic.addformula == FORMULA_NONE){
        if(magic.magictype == 2){
            if(magic.animwidth){
                if(magic.attribute != ATTRIBUTE_ABSORB){
                    InsertTextChild(e_addhit, "FTMagicHit");
                }
                else{
                    InsertTextChild(e_addhit, "Anytime");
                }
            }
            else{
                InsertTextChild(e_addhit, "FTAttackHit");
            }
        }
        else{
            InsertTextChild(e_addhit, "Anytime");
        }
    }
    else{
        InsertTextChild(e_addhit, FormulaID[magic.addformula]);
    }
    e_entry.InsertEndChild(e_addhit);
    // ƒǉʖ̘A֌WFɘAAǉAƗ3B
    TiXmlElement e_judge("HitJudgeType");
    //InsertTextChild(e_judge, "DoubleJudge");
    InsertTextChild(e_judge, HitJudgeTypeID[magic.hitjudge]);
    e_entry.InsertEndChild(e_judge);
    if(magic.animwidth){
        // 摜t@C
        TiXmlElement e_image("Image");
        InsertTextChild(e_image, magic.imgfile.c_str());
        // 摜t@ĈPAj[ṼTCY
        e_image.SetAttribute("width", magic.animwidth);
        e_image.SetAttribute("height", magic.animwidth);
        e_image.SetAttribute("animation", magic.animcount);
        e_image.SetAttribute("type", AnimationTypeID[magic.animtype]);
//        if(!magic.screencenter){
//            e_image.SetAttribute("type", "Chip");
//        }
//        else{
//            e_image.SetAttribute("type", "ScreenCenter");
//        }
        e_entry.InsertEndChild(e_image);
    }
    // ʉt@C
    TiXmlElement e_sound("Sound");
    InsertTextChild(e_sound, magic.sound.c_str());
    e_entry.InsertEndChild(e_sound);
    e.InsertEndChild(e_entry);
    return true;
}
//---------------------------------------------------------------------------
TRangeType TFarenCnv::GetMagicRangeType(int magictype, int range, bool isdeluge) const
{
    switch(magictype){
    case 0: // 
        if(range){
            return RANGE_NORMAL;
        }
        else{
            OutputLog("Error:^Cv@ɑ΂Ď˒0w肳܂B");
            return RANGE_INVALID;
        }
    case 1: // ⏕
        if(range){
            return RANGE_NORMAL;
        }
        else{
            return RANGE_ALLMYSIDE;
        }
    case 2: // U
        if(range){
            return RANGE_NORMAL;
        }
        else{
            if(isdeluge){
                return RANGE_LOWERTHAN;
            }
            else{
                return RANGE_ALLENEMY;
            }
        }
    case 3: // ǔj
        if(range){
            return RANGE_NORMAL;
        }
        else{
            return RANGE_ALLWALL;
        }
    case 4: // }X^[
        return RANGE_ME;
    }
    return RANGE_INVALID;   // ɗ͂͂ȂB
}
/*bool TFarenCnv::GetMagicRangeType(const TMagicEntry& magic, std::string& rangestr) const
{
    switch(magic.magictype){
    case 0: // 
        if(magic.range){
            rangestr = "Normal";
        }
        else{
            OutputLog("Error:@ %s ͏^Cvł˒0ɂȂĂ܂B", magic.id.c_str());
            return false;
        }
        break;
    case 1: // ⏕
        if(magic.range){
            rangestr = "Normal";
        }
        else{
            rangestr = "AllMySide";
        }
        break;
    case 2: // U
        if(magic.range){
            rangestr = "Normal";
        }
        else{
            if(magic.isdeluge){
                rangestr = "LowerThan";
            }
            else{
                rangestr = "AllEnemy";
            }
        }
        break;
    case 3: // ǔj
        if(magic.range){
            rangestr = "Normal";
        }
        else{
            rangestr = "AllWall";
        }
        break;
    case 4: // }X^[
        rangestr = "Me";
        break;
    default:
        OutputLog("@ %s ɖΉ^Cv %d w肳܂B", magic.id.c_str(), magic.magictype);
        return false;   // ɗ͂͂ȂB
    }
    return true;
}*/
//---------------------------------------------------------------------------
// DefaultSpecialTable𕶎zɓǂݍށB
bool TFarenCnv::LoadDefaultSpecialTable(const char *srcpath, std::vector<std::string>& strlist) const
{
    AnsiString srcdef;
    srcdef.sprintf("%s/../../Default/DefChar/SpecialTable", srcpath);
    FILE *fp_def = fopen(srcdef.c_str(), "r");
    if(fp_def == NULL){
        OutputLog("ViItH_ƓʒuDefaultViIHߎs܂B%s", srcdef.c_str());
        return false;
    }
    for(;;){
        char buf[1024];
        if(fgets(buf, sizeof buf, fp_def) == NULL){
            break;
        }
        if(strchr(buf, ' ') != NULL){
            strlist.push_back(buf);
        }
    }
    fclose(fp_def);
    return true;
}
//---------------------------------------------------------------------------
// MasterDataMasterData.xml
bool TFarenCnv::ConvertMasterData(const char *srcpath, const char *dstpath, bool noDefault)
{
    const int POWERMAP_MAX = 14;
    const unsigned PowerMapColor[POWERMAP_MAX][3] =
        {{50, 50, 255}, {255, 0, 0}, {208, 0, 208}, {235, 221, 14},
         {153, 217, 153}, {255, 170, 255}, {248, 209, 146}, {0, 120, 0},
         {153, 97, 36}, {255, 255, 255}, {200, 200, 255}, {178, 122, 56},
         {255, 255, 170}, {208, 0, 208}};

    AnsiString src, dst;
    src.sprintf("%s\\Data\\MasterData", srcpath);
    dst.sprintf("%s\\Data\\MasterData.xml", dstpath);

    TFarenTxt ftxt;
    if(!ftxt.LoadFromFile(src.c_str())){
        return false;
    }

    TiXmlDocument doc(dst.c_str());
    TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
    doc.InsertEndChild(dec);
    TiXmlElement e_top("MasterList");

    int index, count;
    if(ftxt.GetCategory("Ghost", &index, &count)){
        for(int i=0; i<count; ++i){
            int ghost;
            if(sscanf(ftxt.GetString(index + i).c_str(), "%d", &ghost) == 1){
                m_GhostList.push_back(ghost);
            }
        }
    }

    if(ftxt.GetCategory("MasterRevival", &index, &count)){
        for(int i=0; i<count; ++i){
            std::string line = ftxt.GetString(index + i);
            const char *linep = line.c_str();   // xW8030ôŁB
            std::vector<std::string> v;
            boost::algorithm::split(v, linep, boost::is_space());
            // 炩ɗvfȂ珈Ȃ
            if(v.size() < 4){
                continue;
            }
            // 󕶎łȂvfz̑OɎĂ
            std::remove(v.begin(), v.end(), "");

            TMasterRevival rev;
            if(sscanf(v[0].c_str(), "%d", &rev.master_from) != 1){
                continue;
            }
            if(sscanf(v[1].c_str(), "%d", &rev.master_to) != 1){
                continue;
            }
            // 3ڂ͉Ȃ̂œǂݍ܂ȂB
            // bZ[W
            rev.message = v[3];
            MasterRevivalList.push_back(rev);
        }
    }

    if(ftxt.GetCategory("MasterInherit", &index, &count)){
        for(int i=0; i<count; ++i){
            std::string line = ftxt.GetString(index + i);
            const char *linep = line.c_str();   // xW8030ôŁB
            std::vector<std::string> v;
            boost::algorithm::split(v, linep, boost::is_space());
            // 炩ɗvfȂ珈Ȃ
            if(v.size() < 5){
                continue;
            }
            // 󕶎łȂvfz̑OɎĂ
            std::remove(v.begin(), v.end(), "");

            TMasterInherit inh;
            if(sscanf(v[0].c_str(), "%d", &inh.master_from) != 1){
                continue;
            }
            if(sscanf(v[2].c_str(), "%d", &inh.master_to) != 1){
                continue;
            }
            inh.chara = v[1];
            // 4ڂ͉Ȃ̂œǂݍ܂ȂB
            // bZ[W
            inh.message = v[4];
            MasterInheritList.push_back(inh);
        }
    }

    // Ō7"Default"ȂDefaultViIƂB
    AnsiString srcpart = srcpath;
    srcpart = srcpart.SubString(srcpart.Length() - 6, 7);
    bool isDefaultScenario = !srcpart.AnsiCompare("Default");
    if(!ftxt.GetCategory("Character", &index, &count) || !count){
        OutputLog("MasterData[Character]܂łB");
        return false;
    }
    // [Character]̃fBNgXgɓB
    for(int i=0; i<count; ++i){
        m_CharaDirList.push_back(ftxt.GetString(index+i));
    }

    // ʉy
    TiXmlElement e_commonmusic("CommonMusic");
    const int commonmusiccount = 2;
    const char *commonmusicname[commonmusiccount] = {"EventMusic", "DecisiveMusic"};
    for(int i=0; i<commonmusiccount; ++i){
        int linecount, lineindex;
        if(ftxt.GetCategory(commonmusicname[i], &lineindex, &linecount) && linecount == 1){
            std::string musicfile = ftxt.GetString(lineindex) + ".mid";
            TiXmlElement e(commonmusicname[i]);
            InsertTextChild(e, musicfile.c_str());
            e_commonmusic.InsertEndChild(e);
        }
        else{
            OutputLog("MasterData [%s]̋LqȂ̂ŁA`ƌȂ܂B", commonmusicname[i]);
        }
    }
    e_top.InsertEndChild(e_commonmusic);

    // }X^[ŗLy
    const int mastermusiccount = 2;
    std::map<int, std::string> battleMusicMap, masterBattleMusicMap;
    std::map<int, std::string> *maps[mastermusiccount] = {&battleMusicMap, &masterBattleMusicMap};
    const char *mastermusicname[mastermusiccount] = {"BattleMusic", "MasterBattleMusic"};
    for(int i=0; i<mastermusiccount; ++i){
        int linecount, lineindex;
        if(ftxt.GetCategory(mastermusicname[i], &lineindex, &linecount) && linecount){
            for(int m=0; m<linecount; ++m){
                // "}X^[ԍ yt@C"ǂݍށB
                const char *p = ftxt.GetString(lineindex + m).c_str();
                std::vector<std::string> v;
                boost::algorithm::split(v, p, boost::is_space());
                while(!v.empty() && v.front().empty()){
                    v.erase(v.begin());
                }
                if(v.size() < 2){
                    OutputLog("MasterData [%s]%dsڂ\"%s\"̉߂Ɏs܂ivf2łȂHjB", mastermusicname[i], lineindex + m, p);
                    continue;
                }
                int masterindex;
                if(sscanf(v[0].c_str(), "%d", &masterindex) != 1){
                    OutputLog("MasterData [%s]%dsڂ\"%s\"̉߂Ɏs܂i}X^[ԍ擾sjB", mastermusicname[i], lineindex + m, p);
                    continue;
                }
                maps[i]->insert(std::pair<int, std::string>(masterindex, v[1] + ".mid"));
            }
        }
    }

    bool forceOutput;
    if(isDefaultScenario){
        forceOutput = false;
    }
    else{
        // IWiViȈꍇASpecialDataŒ1邩ǂ肷B
        forceOutput = true;
        for(unsigned u=0; u<m_CharaDirList.size(); ++u){
            AnsiString sppath;
            sppath.sprintf("%s\\%s\\SpecialTable", srcpath, m_CharaDirList[u].c_str());
            if(!_access(sppath.c_str(), 0)){
                forceOutput = false;
                break;
            }
        }
        if(forceOutput){
            OutputLog("Information:IWiSpecialTableȂ߁Aŏ̃LN^[tH_SpecialTable.xmlo͂܂B");
        }
    }
    if(noDefault && forceOutput){
        OutputLog("Error:Defaultf[^gpȂIvVw肳܂ASpecialTable܂łB");
        return false;
    }
    std::vector<int> specialOffsetList(count);
    std::vector<std::string> chrSrcPathList(count);
    std::vector<std::string> chrDstPathList(count);
    bool specialFirstDataGood = false;
    TiXmlElement e_chara("Character");
    for(int i=0; i<count; ++i){
        TiXmlElement e("Directory");
        std::string dirname = ftxt.GetString(index+i);
        InsertTextChild(e, dirname.c_str());
        e_chara.InsertEndChild(e);
        AnsiString chrsrcpath, chrdstpath;
        chrsrcpath.sprintf("%s\\%s", srcpath, dirname.c_str());
        chrdstpath.sprintf("%s\\%s", dstpath, dirname.c_str());
        if(_access(chrdstpath.c_str(), 0)){
            if(!CreateDirectory(chrdstpath.c_str(), NULL)){
                OutputLog("%s ̍쐬Ɏs܂B", chrdstpath.c_str());
                return false;
            }
        }
        bool specialExists;
        bool defout;
        specialOffsetList[i] = m_SpecialID.size();
        // LtH_SpecialTableꍇ͕KEZ̊CfbNXCharacterDataƓLtH_ɂB
        if(!ConvertSpecialTable(chrsrcpath.c_str(), chrdstpath.c_str(), noDefault, isDefaultScenario, !specialFirstDataGood, forceOutput, specialExists, defout)){
            OutputLog("SpecialTablet@Cϊs");
            return false;
        }
        // o͂狭o̓tOB
        forceOutput = false;
        if(specialExists){
            // ŏSpecialTableo͍ς݃tO𗧂ĂB
            specialFirstDataGood = true;
            if(defout){
                // 19DefaultSpecialData̐B
                specialOffsetList[i] = 19;
            }
        }
        else{
            specialOffsetList[i] = 0;
        }
        chrSrcPathList[i] = chrsrcpath.c_str();
        chrDstPathList[i] = chrdstpath.c_str();
    }
    for(int i=0; i<count; ++i){
        int offsetend;
        if(i < count - 1){
            offsetend = specialOffsetList[i-1];
        }
        else{
            offsetend = m_SpecialID.size();
        }
        if(!ConvertCharacterData(chrSrcPathList[i].c_str(), chrDstPathList[i].c_str(), noDefault, isDefaultScenario, !i, specialOffsetList[i], offsetend)){
            OutputLog("CharacterDatat@Cϊs");
            return false;
        }
    }
    e_top.InsertEndChild(e_chara);

    for(int mcount=1; ; mcount++){
        AnsiString str;
        str.sprintf("Master%d", mcount);
        if(!ftxt.GetCategory(str.c_str(), &index, &count) || count < 3){
            break;
        }
        // Ń}X^[IDƂĎgĂ镶̂܂ܐwcIDƂĎgpB
        std::string facecharaid = ftxt.GetString(index);
        if(std::find(CharaIDArray.begin(), CharaIDArray.end(), facecharaid) != CharaIDArray.end()){
            // }X^[L
            TiXmlElement e("Master");
            SetTextAttribute(e, "id", facecharaid.c_str());
            // }X^[
            TiXmlElement e_name("Name");
            InsertTextChild(e_name, ftxt.GetString(index+1).c_str());
            e.InsertEndChild(e_name);
            // ͒n}̐F
            TiXmlElement e_pcolor("PowerMapColor");
            AnsiString pcolor;
            if(mcount - 1 < POWERMAP_MAX){
                pcolor.sprintf("ff%02x%02x%02x", PowerMapColor[mcount - 1][0], PowerMapColor[mcount - 1][1], PowerMapColor[mcount - 1][2]);
            }
            else{
                pcolor = "ff808080";
            }
            InsertTextChild(e_pcolor, pcolor.c_str());
            e.InsertEndChild(e_pcolor);
            // }X^[yBw肳ĂȂꍇicount==3ĵł̏ꍇ͋󕶎ɂB
            TiXmlElement e_music("Music");
            AnsiString music;
            if(count > 3){
                music.sprintf("%s.mid", ftxt.GetString(index+3).c_str());
            }
            InsertTextChild(e_music, music.c_str());
            e.InsertEndChild(e_music);
            // }X^[ŗLŷPB
            std::map<int, std::string>::const_iterator itr_battle = battleMusicMap.find(mcount);
            if(itr_battle != battleMusicMap.end()){
                TiXmlElement e_battle("BattleMusic");
                InsertTextChild(e_battle, itr_battle->second.c_str());
                e.InsertEndChild(e_battle);
            }
            // }X^[ŗLŷQB
            std::map<int, std::string>::const_iterator itr_masterbattle = masterBattleMusicMap.find(mcount);
            if(itr_masterbattle != masterBattleMusicMap.end()){
                TiXmlElement e_masterbattle("MasterBattleMusic");
                InsertTextChild(e_masterbattle, itr_masterbattle->second.c_str());
                e.InsertEndChild(e_masterbattle);
            }
            // tOt@C
            std::map<std::string, std::string>::const_iterator itr = MasterFlagFileMap.find(ftxt.GetString(index));
            if(itr != MasterFlagFileMap.end()){
                TiXmlElement e_flag("FlagFile");
                InsertTextChild(e_flag, itr->second.c_str());
                e.InsertEndChild(e_flag);
            }
            e_top.InsertEndChild(e);
            // ̎̂߂ɊoĂB
            SideIDArray.push_back(ftxt.GetString(index).c_str());
        }
        else{
            OutputLog("Warning:MasterDataŃ}X^[LƂ %s w肳܂ACharacterDataɈvID̃L܂łB̃}X^[𖳎܂B", facecharaid.c_str());
            // ȂƐwcIDXgĂ܂̂œB
            SideIDArray.push_back("");
        }
        // ̎̂߂ɊoĂBWarning[gɓꍇł͂ȂƐwcIDXgĂ܂B
        //SideIDArray.push_back(ftxt.GetString(index).c_str());
    }

    doc.InsertEndChild(e_top);

    if(!doc.SaveFile()){
        return false;
    }

    return true;
}
//---------------------------------------------------------------------------
// Move&SkillMoveAndSkill.xml
bool TFarenCnv::ConvertMoveAndSkill(const char *srcpath, const char *dstpath, const AnsiString& defscesrcpath)
{
    AnsiString src, dst;
    src.sprintf("%s\\Data\\Move&Skill", srcpath);
    dst.sprintf("%s\\Data\\MoveAndSkill.xml", dstpath);

    FILE *fp = fopen(src.c_str(), "r");
    if(fp == NULL){
        if(!defscesrcpath.IsEmpty()){
            src.sprintf("%s\\Data\\Move&Skill", defscesrcpath.c_str());
            fp = fopen(src.c_str(), "r");
        }
    }
    if(fp == NULL){
        return false;
    }

    TiXmlDocument doc(dst.c_str());
    TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
    doc.InsertEndChild(dec);

    TiXmlElement e_top("MoveTypeList");
    for(int movetype=0; movetype<MOVETYPE_MAX; ){
        // 2s1n`^CvɑΉĂ̂2sǂłB
        char buf[1024];
        if(fgets(buf, sizeof buf, fp) == NULL){
            break;
        }
        // sꍇ͂Ƃ΂B
        if(buf[0] == '\n'){
            continue;
        }
        // xĂ݂B
        std::vector<std::string> v;
        boost::algorithm::split(v, buf, boost::is_space());
        // ȂΎ1sBDefault2sgȂ̂1sŏĂViÎŁB
        if(v.size() < 40){
            std::string line;
            line += buf;
            if(fgets(buf, sizeof buf, fp) == NULL){
                break;
            }
            line += buf;
            const char *p = line.c_str();
            boost::algorithm::split(v, p, boost::is_space());
            // 炩ɗvfȂI
            if(v.size() < 40){
                break;
            }
        }
        // 󕶎łȂvfz̑OɎĂ
        std::remove(v.begin(), v.end(), "");

        // ړ^CvIDłȂCfbNXňBςIDɂĂ݂B
        TiXmlElement e_move("MoveType");
        //e_move.SetAttribute("index", movetype);
        SetTextAttribute(e_move, "id", movetype_name[movetype]);
        TiXmlElement e_movename("Name");
        InsertTextChild(e_movename, movetype_name[movetype]);
        e_move.InsertEndChild(e_movename);

        int gndcount = 0;
        int objcount = 0;
        for(int count = 0; count < MAPGNDTYPE_MAX+MAPOBJTYPE_MAX; count++){
            // r܂ł͒n`A̕͏㕨Ƃďo͂B
            int index;
            bool existsData;
            TiXmlElement e("");
            if(count < MAPGNDTYPE_MAX){
                e.SetValue("MapGround");
                // n`IDłȂCfbNXňB
                e.SetAttribute("index", gndcount);
                gndcount++;
                index = count*2;
                existsData = true;
            }else{
                e.SetValue("MapObject");
                // 㕨IDłȂCfbNXňB
                e.SetAttribute("index", objcount);
                existsData = objcount != 0;
                objcount++;
                index = (count-1)*2;
            }
            // ړ
            int cost = 1;
            if(existsData){
                sscanf(v[index].c_str(), "%d", &cost);
            }
            if(!cost){
                OutputLog("ړ^Cv %s  %d Ԃ̈ړRXg0ݒ肳Ă܂A0͈Ȃ߂Ƃ肠99Ɖ߂܂iIWĩt@[gD[KňړRXg0ǂӖɂȂ̂sȂ̂ŁjB",
                            movetype_name[movetype], count);
                cost = 99;
            }
            e.SetAttribute("MoveCost", cost);
            // ZʏC
            int skill = 0;
            if(existsData){
                sscanf(v[index+1].c_str(), "%d", &skill);
            }
            e.SetAttribute("Skill", skill);
            e_move.InsertEndChild(e);
        }
        e_top.InsertEndChild(e_move);
        ++movetype;
    }
    fclose(fp);
    doc.InsertEndChild(e_top);

    if(!doc.SaveFile()){
        return false;
    }

    return true;
}
//---------------------------------------------------------------------------
// Œ̒n`o͂B
bool TFarenCnv::OutputMapChipType(const char *srcpath, const char *dstpath)
{
    AnsiString src, dst;
    dst.sprintf("%s\\Data\\MapChipType.xml", dstpath);

    TiXmlDocument doc(dst.c_str());
    TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
    doc.InsertEndChild(dec);

    TiXmlElement e_top("GroundList");
    TiXmlElement e_gndlist("MapGroundList");
    for(int gndcount = 0; gndcount < MAPGNDTYPE_MAX; gndcount++){
        // n`IDłȂCfbNXňB
        TiXmlElement e("MapGround");
        e.SetAttribute("index", gndcount);
        TiXmlElement e_name("Name");
        InsertTextChild(e_name, mapgnd_name[gndcount]);
        e.InsertEndChild(e_name);
        e_gndlist.InsertEndChild(e);
    }
    e_top.InsertEndChild(e_gndlist);

    TiXmlElement e_objlist("MapObjectList");
    for(int objcount = 0; objcount < MAPOBJTYPE_MAX; objcount++){
        // 㕨IDłȂCfbNXňB
        TiXmlElement e("MapObject");
        e.SetAttribute("index", objcount);
        TiXmlElement e_name("Name");
        InsertTextChild(e_name, mapobj_name[objcount]);
        e.InsertEndChild(e_name);
        e_objlist.InsertEndChild(e);
    }
    e_top.InsertEndChild(e_objlist);
    doc.InsertEndChild(e_top);

    if(!doc.SaveFile()){
        return false;
    }

    return true;
}
//---------------------------------------------------------------------------
// Œ̖@O[vo͂B
bool TFarenCnv::OutputMagicGroup(const char *srcpath, const char *dstpath)
{
    static const unsigned groupcolor[MAGICGROUP_MAX] = {0xffff0000, 0xff0040ff, 0xff00ff00, 0xffff8080, 0xffffff00, 0xff000040};
    AnsiString src, dst;
    dst.sprintf("%s\\Data\\MagicGroup.xml", dstpath);

    TiXmlDocument doc(dst.c_str());
    TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
    doc.InsertEndChild(dec);

    TiXmlElement e_list("MagicGroupList");
    for(int count = 0; count < MAGICGROUP_MAX; count++){
        TiXmlElement e("MagicGroup");
        // ID
        AnsiString groupid;
        groupid.sprintf("magicgroup%d", count);
        e.SetAttribute("id", groupid.c_str());
        // O
        TiXmlElement e_name("Name");
        InsertTextChild(e_name, magicgroup_name[count]);
        e.InsertEndChild(e_name);
        // F
        TiXmlElement e_color("Color");
        AnsiString colorstr;
        colorstr.sprintf("%08x", groupcolor[count]);
        InsertTextChild(e_color, colorstr.c_str());
        e.InsertEndChild(e_color);
        e_list.InsertEndChild(e);
    }
    doc.InsertEndChild(e_list);

    if(!doc.SaveFile()){
        return false;
    }

    return true;
}
//---------------------------------------------------------------------------
// Œ̑o͂B
bool TFarenCnv::OutputAttribute(const char *srcpath, const char *dstpath)
{
    AnsiString dst;
    dst.sprintf("%s\\Data\\Attribute.xml", dstpath);

    TiXmlDocument doc(dst.c_str());
    TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
    doc.InsertEndChild(dec);

    TiXmlElement e_list("AttributeList");
    for(int i = 0; i < ATTRIBUTE_FULLMAX; i++){
        TiXmlElement e("Attribute");
        e.SetAttribute("id", attr_id[i]);
        e.SetAttribute("type", attr_type_name[attr_type[i]]);
        TiXmlElement e_name("Name");
        InsertTextChild(e_name, attr_name[i]);
        e.InsertEndChild(e_name);
        TiXmlElement e_sname("ShortName");
        InsertTextChild(e_sname, attr_shortname[i]);
        e.InsertEndChild(e_sname);
        // (2)`(8) HP(16)MP(17)ёϐȊO̒ǉ̓VXe`̌ʂo͂B
        if((i >= 2 && i <= 8) || i==16 || i==17 || (i >= ATTRIBUTE_MAX && i != ATTRIBUTE_MAX + 21)){
            // VXe`̌ʂID͊{Iɂ͑IDƓB
            TiXmlElement e_se("SystemEffect");
            if(i == ATTRIBUTE_MINUS_ATTACKCOUNT){
                // AU͍UƃVXe`IDB
                e_se.SetAttribute("id", attr_id[ATTRIBUTE_PLUS_ATTACKCOUNT]);
            }
            else{
                e_se.SetAttribute("id", attr_id[i]);
            }
            e.InsertEndChild(e_se);
        }
        e_list.InsertEndChild(e);
    }
    doc.InsertEndChild(e_list);

    if(!doc.SaveFile()){
        return false;
    }

    return true;
}
//---------------------------------------------------------------------------
bool TFarenCnv::OutputFieldMap(const char *srcpath, const char *dstpath, bool noDefault)
{
    // FieldMap.bmp̃TCY擾B
    int fmwidth, fmheight;
    AnsiString bmpfile;
    AnsiString pngfile;
    bmpfile.sprintf("%s\\Picture\\FieldMap.bmp", srcpath);
    HDIB hdib = BMPtoDIB(bmpfile.c_str());
    if(hdib != NULL){
        if(!GetHDIBSize(hdib, &fmwidth, &fmheight)){
            OutputLog("failed to GetHDIBSize()");
            DeleteDIB(hdib);
            return false;
        }
        DeleteDIB(hdib);
        pngfile = "FieldMap.png";
    }
    else{
        if(noDefault){
            OutputLog("Error:Defaultf[^gpȂIvVw肳܂A%s ܂łB", bmpfile.c_str());
            return false;
        }
        else{
            // ꍇDefault̂QƂŁB
            fmwidth = 1280;
            fmheight = 960;
            pngfile = "../../Default/Picture/FieldMap.png";
        }
    }

    AnsiString dst;
    dst.sprintf("%s\\Data\\FieldMap.xml", dstpath);
    TiXmlDocument doc(dst.c_str());
    TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
    doc.InsertEndChild(dec);

    TiXmlElement e_top("FieldMap");
    e_top.SetAttribute("Width", fmwidth);
    e_top.SetAttribute("Height", fmheight);
    TiXmlElement e_image("Image");
    TiXmlElement e_file("File");
    InsertTextChild(e_file, pngfile.c_str());
    e_image.InsertEndChild(e_file);
    e_top.InsertEndChild(e_image);
    doc.InsertEndChild(e_top);
    if(!doc.SaveFile()){
        return false;
    }
    return true;
}
//---------------------------------------------------------------------------
// Œ̎푰o͂B
bool TFarenCnv::OutputRace(const char *srcpath, const char *dstpath)
{
    // OEɔzŕ\B
    //            l
    //     Gt      ACX}
    // Su             h[t
    //   X^[    U[h}
    //            
    const int AffinityRing[RACE_MAX] = {RACE_HUMAN, RACE_ICEMAN, RACE_DWARF, RACE_LIZARDMAN, RACE_DEVIL, RACE_MONSTER, RACE_GOBLIN, RACE_ELF};

    AnsiString dst;
    dst.sprintf("%s\\Data\\Race.xml", dstpath);

    TiXmlDocument doc(dst.c_str());
    TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
    doc.InsertEndChild(dec);

    TiXmlElement e_list("RaceList");
    for(int race = RACE_HUMAN; race < RACE_MAX; ++race){
        TiXmlElement e("Race");
        SetTextAttribute(e, "id", race_name[race]);
        TiXmlElement e_name("Name");
        InsertTextChild(e_name, race_name[race]);
        e.InsertEndChild(e_name);

        // 푰ɑ΂鑊iقǑǂjo͂B
        TiXmlElement e_aff("Affinity");
        int ringindex = std::find(AffinityRing, AffinityRing + RACE_MAX, race) - AffinityRing;
        for(int other = RACE_HUMAN; other < RACE_MAX; ++other){
            if(race == other){
                continue;
            }
            int otherindex = std::find(AffinityRing, AffinityRing + RACE_MAX, other) - AffinityRing;
            int dist = abs(ringindex - otherindex);
            if(dist > RACE_MAX / 2){
                dist = RACE_MAX - dist;
            }
            TiXmlElement e_other("Race");
            SetTextAttribute(e_other, "id", race_name[other]);
            e_other.SetAttribute("distance", dist);
            e_aff.InsertEndChild(e_other);
        }
        e.InsertEndChild(e_aff);

        e_list.InsertEndChild(e);
    }
    doc.InsertEndChild(e_list);

    if(!doc.SaveFile()){
        return false;
    }

    return true;
}
//---------------------------------------------------------------------------
// Œ̕o͂B
bool TFarenCnv::OutputRevive(const char *srcpath, const char *dstpath)
{
    AnsiString dst;
    dst.sprintf("%s\\Data\\Revive.xml", dstpath);

    TiXmlDocument doc(dst.c_str());
    TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
    doc.InsertEndChild(dec);

    TiXmlElement e_list("ReviveList");
    {
        // 앜
        TiXmlElement e("Revive");
        SetTextAttribute(e, "id", REVIVE_NECROMANCY);
        TiXmlElement e_method("SelectMethod");
        SetTextAttribute(e_method, "value", "Employable");  // Employable, FromList
        e.InsertEndChild(e_method);
        e_list.InsertEndChild(e);
    }

    for(unsigned u=0; u<MasterRevivalList.size(); ++u){
        // RNKC
        TiXmlElement e("Revive");
        AnsiString revid;
        revid.sprintf("%s%u", REVIVE_ME, u);
        SetTextAttribute(e, "id", revid.c_str());
        TiXmlElement e_method("SelectMethod");
        SetTextAttribute(e_method, "value", "FromList");  // Employable, FromList
        TiXmlElement e_charalist("CharacterList");
        TiXmlElement e_chara("Character");
        SetTextAttribute(e_chara, "id", ConvertedCharaList[MasterRevivalList[u].master_to - 1].id.c_str());
        e_charalist.InsertEndChild(e_chara);
        e_method.InsertEndChild(e_charalist);
        e.InsertEndChild(e_method);
        e_list.InsertEndChild(e);
    }

    doc.InsertEndChild(e_list);

    if(!doc.SaveFile()){
        return false;
    }

    return true;
}
//---------------------------------------------------------------------------
bool TFarenCnv::OutputLua(const char *dstpath)
{
    AnsiString dstdir;
    dstdir.sprintf("%s\\Script", dstpath);
    if(_access(dstdir.c_str(), 0)){
        if(!CreateDirectory(dstdir.c_str(), NULL)){
            OutputLog("%s ̍쐬Ɏs܂B", dstdir.c_str());
            return false;
        }
    }

    // 퓬XNvg
    AnsiString dst;
    dst.sprintf("%s\\battle.lua", dstdir.c_str());
    FILE *fp = fopen(dst.c_str(), "w");
    if(fp == NULL){
        OutputLog("%s ̍쐬Ɏs܂B", dst.c_str());
        return false;
    }

    //fprintf(fp, "require('battle_util.lua')\n");

    for(unsigned u=0; u<MasterRevivalList.size(); ++u){
        const TMasterRevival& rev = MasterRevivalList[u];
        fprintf(fp, "function MasterRevival%u()\n", u);
        fprintf(fp, "    api_dialog_show('', '%s', BUTTON_OK, true)\n", rev.message.c_str());
        fprintf(fp, "    while api_dialog_is_visible() do\n");
        fprintf(fp, "        coroutine.yield()\n");
        fprintf(fp, "    end\n");
        fprintf(fp, "    api_change_master_type('%s', '%s', '%s')\n", CharaIDArray[rev.master_from-1].c_str(), CharaIDArray[rev.master_to-1].c_str(), CharaIDArray[rev.master_from-1].c_str());
        fprintf(fp, "    return true\n");
        fprintf(fp, "end\n");
        fprintf(fp, "\n");
    }

    for(unsigned u=0; u<MasterInheritList.size(); ++u){
        const TMasterInherit& inh = MasterInheritList[u];
        fprintf(fp, "function MasterInherit%u()\n", u);
        fprintf(fp, "    if not api_exists_hero('%s', '%s') then\n", CharaIDArray[inh.master_from-1].c_str(), inh.chara.c_str());
        fprintf(fp, "        return false\n");
        fprintf(fp, "    end\n");
        fprintf(fp, "    api_dialog_show('', '%s', BUTTON_OK, true)\n", inh.message.c_str());
        fprintf(fp, "    while api_dialog_is_visible() do\n");
        fprintf(fp, "        coroutine.yield()\n");
        fprintf(fp, "    end\n");
        fprintf(fp, "    api_change_master_type('%s', '%s', '%s')\n", CharaIDArray[inh.master_from-1].c_str(), CharaIDArray[inh.master_to-1].c_str(), inh.chara.c_str());
        fprintf(fp, "    return true\n");
        fprintf(fp, "end\n");
        fprintf(fp, "\n");
    }

    fprintf(fp, "function deluge()\n");
    fprintf(fp, "    local w, h = api_battle_get_map_size()\n");
    fprintf(fp, "    for y = 0, h-1, 1 do\n");
    fprintf(fp, "        for x = 0, w-1, 1 do\n");
    fprintf(fp, "            local mh = api_battle_get_map_height(x, y)\n");
    fprintf(fp, "            if mh <= 2 then\n");
    fprintf(fp, "                api_battle_set_map_chip(x, y, 16)\n");
    fprintf(fp, "                api_battle_rm_map_obj(x, y, MAPOBJ_CITY)\n");
    fprintf(fp, "                api_battle_rm_map_obj(x, y, MAPOBJ_WALL)\n");
    fprintf(fp, "                api_battle_rm_map_obj(x, y, MAPOBJ_ROAD)\n");
    fprintf(fp, "            end\n");
    fprintf(fp, "        end\n");
    fprintf(fp, "    end\n");
    fprintf(fp, "end\n");
    fprintf(fp, "\n");

    fprintf(fp, "function volcano()\n");
    fprintf(fp, "    -- 117`bv\n");
    fprintf(fp, "    local chips = {          12,  3,  3,  3,  3, 13,\n");
    fprintf(fp, "                           19, 32, 23, 23, 23, 33, 18,\n");
    fprintf(fp, "                         19, 39, 32, 23, 23, 33, 38, 18,\n");
    fprintf(fp, "                       19, 39, 39, 32, 23, 33, 38, 38, 18,\n");
    fprintf(fp, "                     19, 39, 39, 39,  1,  1, 38, 38, 38, 18,\n");
    fprintf(fp, "                    4, 24, 24, 24,  1,  1,  1, 25, 25, 25,  5,\n");
    fprintf(fp, "                     16, 36, 36, 36,  1,  1, 37, 37, 37, 17,\n");
    fprintf(fp, "                       16, 36, 36, 34, 22, 35, 37, 37, 17,\n");
    fprintf(fp, "                         16, 36, 34, 22, 22, 35, 37, 17,\n");
    fprintf(fp, "                           16, 34, 22, 22, 22, 35, 17,\n");
    fprintf(fp, "                             14,  2,  2,  2,  2, 15}\n");
    fprintf(fp, "    local x, y = api_battle_get_last_target()\n");
    fprintf(fp, "    local rad = api_battle_get_neighbor_sorted(x, y, 5)\n");
    fprintf(fp, "    for i, p in ipairs(rad) do\n");
    fprintf(fp, "        api_battle_set_map_chip(p.x, p.y, 140 + chips[i])\n");
    fprintf(fp, "        api_battle_rm_map_obj(p.x, p.y)\n");
    fprintf(fp, "    end\n");
    fprintf(fp, "end\n");
    fprintf(fp, "\n");

    fclose(fp);

    // }X^[SEQ[I[o[XNvg
    AnsiString srcscript;
    AnsiString dstscript;
    srcscript.sprintf("%s\\world.lua", m_CnvSrcPath.c_str());
    dstscript.sprintf("%s\\world.lua", dstdir.c_str());
    if(!CopyFile(srcscript.c_str(), dstscript.c_str(), FALSE)){
        OutputLog("cnvsrc -> ScriptԂ̃t@CRs[Ɏs܂B");
        return false;
    }

    srcscript.sprintf("%s\\ofutil.lua", m_CnvSrcPath.c_str());
    dstscript.sprintf("%s\\ofutil.lua", dstdir.c_str());
    if(!CopyFile(srcscript.c_str(), dstscript.c_str(), FALSE)){
        OutputLog("cnvsrc -> ScriptԂ̃t@CRs[Ɏs܂B");
        return false;
    }

    return true;
}
//---------------------------------------------------------------------------
// ScenarioXScenarioX.xml
bool TFarenCnv::ConvertScenarioX(const char *srcpath, const char *dstpath, int num)
{
    AnsiString src, dst;
    src.sprintf("%s\\Data\\Scenario%d", srcpath, num);
    dst.sprintf("%s\\Data\\Scenario%d.xml", dstpath, num);

    if(_access(src.c_str(), 0)){
        OutputLog("%s t@C܂łBϊXLbv܂B", src.c_str());
        return true;
    }

    TFarenTxt ftxt;
    if(!ftxt.LoadFromFile(src.c_str())){
        OutputLog("%s t@CI[vł܂łB", src.c_str());
        return false;
    }

    TiXmlDocument doc(dst.c_str());
    TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
    doc.InsertEndChild(dec);

    TiXmlElement e_chap("Chapter");
    int index, count;
    if(!ftxt.GetCategory("Name", &index, &count)){
        OutputLog("%s [Name]܂łB", src.c_str());
        return false;
    }
    TiXmlElement e_name("Name");
    InsertTextChild(e_name, ftxt.GetString(index).c_str());
    e_chap.InsertEndChild(e_name);

    std::string expstr;
    if(ftxt.GetCategory("Explanation", &index, &count)){
        for(int i=0; i<count; i++){
            if(i>2){
                expstr += "\n";
            }
            expstr += ftxt.GetString(index+i);
        }
    }
    else{
        OutputLog("Warning:%s [Explanation]܂łBViI̐͋ɂȂ܂B", src.c_str());
    }
    TiXmlElement e_exp("Explanation");
    InsertTextChild(e_exp, expstr.c_str());
    e_chap.InsertEndChild(e_exp);

    if(!ftxt.GetCategory("Turn", &index, &count)){
        OutputLog("%s [Turn]܂łB", src.c_str());
        return false;
    }
    TiXmlElement e_turn("Turn");
    InsertTextChild(e_turn, ftxt.GetString(index).c_str());
    e_chap.InsertEndChild(e_turn);

    // vC[IׂȂ}X^[ꗗB
    std::set<int> TakeOff;
    if(ftxt.GetCategory("TakeOff", &index, &count)){
        int masternum;
        for(int i=0; i<count; ++i){
            if(sscanf(ftxt.GetString(index+i).c_str(), "%d", &masternum) == 1){
                TakeOff.insert(masternum);
            }
        }
    }

    // [Area]̏L}X^[ԍPɂĐzɕB
    if(!ftxt.GetCategory("Area", &index, &count)){
        OutputLog("%s [Area]܂łB", src.c_str());
        return false;
    }
    std::string areastr;
    for(int i=0; i<count; i++){
        areastr += ftxt.GetString(index+i);
    }
    std::vector<std::string> v;
    const char *p = areastr.c_str();
    boost::algorithm::split(v, p, boost::is_space());
    // 󕶎łȂvfz̑OɎĂ
    std::remove(v.begin(), v.end(), "");

    for(unsigned master=0; ; ++master){
        // rƂł邱Ƃ̂œKɒli߂ČB
        int i;
        for(i=0; i<100; ++i){
            AnsiString key;
            key.sprintf("Master%d", master + 1);
            if(ftxt.GetCategory(key.c_str(), &index, &count)){
                break;
            }
            ++master;
        }
        if(i == 100){
            break;
        }
        if(count < 5){
            OutputLog("%s [Master]̌̍s܂B", src.c_str());
            return false;
        }
        if(master >= SideIDArray.size()){
            OutputLog("Warning:%s [Master]͈̐͂𒴂Ă܂(%d)Bȍ~[Master]𖳎܂B", src.c_str(), master);
            break;
        }
        TiXmlElement e_master("Master");
        SetTextAttribute(e_master, "id", SideIDArray[master].c_str());
        // }X^[ʒu
        AnsiString key;
        key.sprintf("%d", master + 1);
        std::vector<std::string>::const_iterator itr = std::find(v.begin(), v.end(), key.c_str());
        if(itr != v.end()){
            int ai = itr - v.begin();
            if(static_cast<unsigned>(ai-1) < AreaIDArray.size()){
                SetTextAttribute(e_master, "area", AreaIDArray[ai-1].c_str());
            }
        }

        // vC[IׂȂ}X^[B
        if(TakeOff.find(master + 1) != TakeOff.end()){
            SetTextAttribute(e_master, "notplayer", "true");
        }

        TiXmlElement e_handle("Handle");
        InsertTextChild(e_handle, ftxt.GetString(index).c_str());
        e_master.InsertEndChild(e_handle);

        TiXmlElement e_diff("Diffculty");
        InsertTextChild(e_diff, ftxt.GetString(index+1).c_str());
        e_master.InsertEndChild(e_diff);

        TiXmlElement e_mexp("Explanation");
        std::string expstr;
        for(int i=2; i<count; i++){
            if(i>2){
                expstr += "\n";
            }
            expstr += ftxt.GetString(index+i);
        }
        InsertTextChild(e_mexp, expstr.c_str());
        e_master.InsertEndChild(e_mexp);
        e_chap.InsertEndChild(e_master);
    }

    // GA̎xz
    if(ftxt.GetCategory("Area", &index, &count)){
        int areacount = 0;
        for(int i=0; i<count; i++){
            std::vector<std::string> v_area;
            const char *p = ftxt.GetString(index+i).c_str();
            boost::algorithm::split(v_area, p, boost::is_space());
            // 󕶎łȂvfz̑OɎĂ(remove)BłɎۂɍ폜(erase)B
            v_area.erase(std::remove(v_area.begin(), v_area.end(), ""), v_area.end());
            for(unsigned u=0; u<v_area.size(); u++){
                if(static_cast<unsigned>(areacount) >= AreaIDArray.size()){
                    OutputLog("%s [Area]Ɏw肳Ă}X^[ԍ̐ߏłBFGA%uȂ̂ŉߏ蕪͖܂B", src.c_str(), AreaIDArray.size());
                    break;
                }
                int n;
                if(sscanf(v_area[u].c_str(), "%d", &n) == 1){
                    TiXmlElement e_area("Area");
                    SetTextAttribute(e_area, "id", AreaIDArray[areacount].c_str());
                    if(n && static_cast<unsigned>(n-1) < SideIDArray.size()){
                        SetTextAttribute(e_area, "master", SideIDArray[n-1].c_str());
                        e_chap.InsertEndChild(e_area);
                    }
                    else if(n == -1){
                        SetTextAttribute(e_area, "ignore", "true");
                        e_chap.InsertEndChild(e_area);
                    }
                    else if(n){
                        OutputLog("%s [Area]́i0琔āj%dԖڂɎw肳ꂽ%d0ł-1łGACfbNX͈͓̔ɂ܂B̋Lq𖳎܂B", src.c_str(), areacount, n);
                    }
                }
                ++areacount;
            }
        }
    }

    if(ftxt.GetCategory("League", &index, &count)){
        for(int i=0; i<count; i++){
            std::vector<std::string> v_league;
            const char *p = ftxt.GetString(index+i).c_str();
            boost::algorithm::split(v_league, p, boost::is_space());
            if(v_league.size() < 3){
                OutputLog("%s [League]Ɏw肳ꂽ3ɂȂĂȂs܂B", src.c_str());
                return false;
            }
            // 󕶎łȂvfz̑OɎĂ
            std::remove(v_league.begin(), v_league.end(), "");
            TiXmlElement e_a("Alliance");
            int id1, id2, turn;
            if(sscanf(v_league[0].c_str(), "%d", &id1) != 1 ||
               sscanf(v_league[1].c_str(), "%d", &id2) != 1 ||
               sscanf(v_league[2].c_str(), "%d", &turn) != 1){
                OutputLog("%s [League]Ɏw肳ꂽołȂs܂B", src.c_str());
                return false;
            }
            --id1;
            --id2;
            if(id1 < 0 || static_cast<unsigned>(id1) >= SideIDArray.size() ||
               id2 < 0 || static_cast<unsigned>(id2) >= SideIDArray.size()){
                OutputLog("Warning:%s [League]Ɏw肳ꂽ}X^[ԍ %d ܂ %d ͈ [1, %d] 𒴂Ă܂B̍s[%s]𖳎܂B", src.c_str(), id1+1, id2+1, SideIDArray.size(), p);
                continue;
            }
            if(SideIDArray[id1].empty() || SideIDArray[id2].empty()){
                OutputLog("Warning:%s [League]Ɏw肳ꂽ}X^[ԍ %d ܂ %d MasterDataŖȃLw肳Ă閳ȃ}X^[łB̍s[%s]𖳎܂B", src.c_str(), id1+1, id2+1, p);
                continue;
            }
            SetTextAttribute(e_a, "id1", SideIDArray[id1].c_str());
            SetTextAttribute(e_a, "id2", SideIDArray[id2].c_str());
            if(turn != 99){
                e_a.SetAttribute("turn", turn);
            }
            else{
                e_a.SetAttribute("turn", "infinite");
            }
            e_chap.InsertEndChild(e_a);
        }
    }

    if(ftxt.GetCategory("Locate", &index, &count)){
        for(int i=0; i<count; i++){
            const char *linestr = ftxt.GetString(index+i).c_str();
            const char *p = linestr;
            // ŏɐlID邪AoCĝőoCgΉ֐Ő؂oB
            size_t sz = FindChar(p, ' ');
            if(sz == -1){
                OutputLog("%s [Locate]Ɏw肳ꂽlIĎ̋󔒂𐳂ołȂs܂B[%s]", src.c_str(), linestr);
                return false;
            }
            std::string charaid = p;
            charaid = charaid.substr(0, sz);
            p = p + sz;
            std::vector<std::string> v_loc;
            boost::algorithm::split(v_loc, p, boost::is_space());
            if(v_loc.size() < 2){
                OutputLog("%s [Locate]̃GAԍ܂̓N𐳂ołȂs܂B[%s]", src.c_str(), linestr);
                return false;
            }
            // 󕶎łȂvfz̑OɎĂ
            std::remove(v_loc.begin(), v_loc.end(), "");
            TiXmlElement e_chara("Character");
            int areaid, rank;
            if(sscanf(v_loc[0].c_str(), "%d", &areaid) != 1 ||
               sscanf(v_loc[1].c_str(), "%d", &rank) != 1){
                // SpXy[XĂViIiAt^[}nL2ĵŏčĎsĂ݂B
                std::string v_loc1 = CutZenSpace(v_loc[0]);
                std::string v_loc2 = CutZenSpace(v_loc[1]);
                if(sscanf(v_loc1.c_str(), "%d", &areaid) != 1 ||
                   sscanf(v_loc2.c_str(), "%d", &rank) != 1){
                    OutputLog("%s [Locate]̃GAԍ܂̓N𐳂ołȂs܂iltH[}bgG[jB[%s]", src.c_str(), linestr);
                    return false;
                }
            }
            SetTextAttribute(e_chara, "id", charaid.c_str());
            //AnsiString area;
            //area.sprintf("area%d", areaid);
            //e_chara.SetAttribute("area", area.c_str());
            if(static_cast<unsigned>(areaid-1) < AreaIDArray.size()){
                SetTextAttribute(e_chara, "area", AreaIDArray[areaid-1].c_str());
            }
            else{
                OutputLog("%s [Locate]ɖȃGAԍ܂܂Ăs܂B[%s]", src.c_str(), linestr);
                return false;
                //SetTextAttribute(e_chara, "area", "ȃGAID");
            }
            e_chara.SetAttribute("rank", rank);
            e_chap.InsertEndChild(e_chara);
        }
    }
    doc.InsertEndChild(e_chap);

    if(!doc.SaveFile()){
        return false;
    }

    return true;
}
//---------------------------------------------------------------------------
// PowerMapPowerMap.xml
bool TFarenCnv::ConvertPowerMap(const char *srcpath, const char *dstpath, const AnsiString& defscesrcpath, const AnsiString& defscedstpath)
{
    AnsiString src, dst;
    src.sprintf("%s\\Data\\PowerMap", srcpath);
    dst.sprintf("%s\\Data\\PowerMap.xml", dstpath);

    FILE *fp = fopen(src.c_str(), "r");
    if(fp == NULL){
        if(!defscesrcpath.IsEmpty()){
            src.sprintf("%s\\Data\\PowerMap", defscesrcpath.c_str());
            fp = fopen(src.c_str(), "r");
        }
    }
    if(fp == NULL){
        OutputLog("PowerMap܂łB");
        return false;
    }

    TiXmlDocument doc(dst.c_str());
    TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
    doc.InsertEndChild(dec);
    TiXmlElement e_top("PowerMap");

    AnsiString imgfile, imgstr;
    imgfile.sprintf("%s\\Picture\\PowerMap.bmp", srcpath);
    if(!access(imgfile.c_str(), 0)){
        imgstr = "PowerMap.png";
    }
    else{
        if(!defscedstpath.IsEmpty()){
            imgstr.sprintf("../../Default/Picture/PowerMap.png");
        }
        else{
            OutputLog("Error:Defaultf[^gpȂIvVw肳܂APowerMap.bmp܂łB");
            return false;
        }
    }
    TiXmlElement e_image("Image");
    TiXmlElement e_file("File");
    InsertTextChild(e_file, imgstr.c_str());
    e_image.InsertEndChild(e_file);
    e_top.InsertEndChild(e_image);

    TiXmlElement e_list("AreaList");
    for(unsigned areacount=0; ; ++areacount){
        char buf[1024];
        if(fgets(buf, sizeof buf, fp) == NULL){
            break;
        }
        // GA0Ԃ̃f[^Ă邪svȂ̂łƂ΂B
        if(!areacount){
            continue;
        }
        std::vector<std::string> v;
        boost::algorithm::split(v, buf, boost::is_space());
        // 炩ɗvfȂI
        if(v.empty()){
            break;
        }
        // 󕶎łȂvfz̑OɎĂ
        std::remove(v.begin(), v.end(), "");

        TiXmlElement e("Area");
        if(areacount - 1 >= AreaIDArray.size()){
            OutputLog("Warning:PowerMapAreaData(%d)f[^Ă܂BPowerMap%dsڈȍ~𖳎܂B", AreaIDArray.size(), areacount + 1);
            break;
        }
        SetTextAttribute(e, "id", AreaIDArray[areacount - 1].c_str());
        unsigned nums = v.size();
        bool notempty = false;
        for(unsigned u=0; ;++u){
            // vf2ȂIB
            if(u*2 + 1 >= nums){
                break;
            }
            // s̍Ō̓[1ŏI邪A͏̗vfbreakĂƂ΂B
            TiXmlElement e_paint("PaintPoint");
            int x = 0;
            if(sscanf(v[u*2].c_str(), "%d", &x) != 1){
                break;
            }
            e_paint.SetAttribute("x", x);
            int y = 0;
            if(sscanf(v[u*2+1].c_str(), "%d", &y) != 1){
                break;
            }
            e_paint.SetAttribute("y", y);
            e.InsertEndChild(e_paint);
            notempty = true;
        }
        if(notempty){
            e_list.InsertEndChild(e);
        }
    }
    fclose(fp);
    e_top.InsertEndChild(e_list);
    doc.InsertEndChild(e_top);

    if(!doc.SaveFile()){
        return false;
    }

    return true;
}
//---------------------------------------------------------------------------
// BMap_NoXXBMap_NoXX.xml
bool TFarenCnv::ConvertBMap(const char *srcpath, const char *dstpath, const AnsiString& defscesrcpath)
{
    AnsiString src, dst;
    src.sprintf("%s\\Data", srcpath);
    dst.sprintf("%s\\Data", dstpath);

    typedef struct{
        unsigned char Char[32][32];     // n``bvԍij
        unsigned char ObjChar[32][32];  // 㕨̃`bvԍij
        unsigned char ObjCharNo[32][32]; // XEǁE̔ԍ
        unsigned char Height[32][32];   // ij
        struct{
            unsigned char x,y;
        }Escape[8][20];                 // אڒnƔzuij
        int CityKz,WallKz,RoadKz;
    }TBMap;

    for(int mapnum=0; ; mapnum++)
    {
        AnsiString bmappath;
        bmappath.sprintf("%s\\BMap_No%d", src.c_str(), mapnum + 1);
        FILE *fp = fopen(bmappath.c_str(), "rb");
        if(fp == NULL){
            if(static_cast<unsigned>(mapnum) < AreaIDArray.size() - 1 && !defscesrcpath.IsEmpty()){
                OutputLog("Warning:BMap_No%02dt@C܂łBDefaultViI̕⊮݂܂B", mapnum + 1);
                bmappath.sprintf("%s\\Data\\BMap_No%d", defscesrcpath.c_str(), mapnum + 1);
                fp = fopen(bmappath.c_str(), "rb");
            }
        }
        if(fp == NULL){
            break;
        }

        if(static_cast<unsigned>(mapnum) >= AreaIDArray.size()){
            OutputLog("Warning:BMap_No%02dt@C̓GAf[^͈̔͂𒴂Ă܂B̃t@Cȍ~BMapt@Cϊ܂B", mapnum + 1);
            fclose(fp);
            break;
        }

        TBMap bmap;
        if(fread(&bmap, 1, sizeof bmap, fp) < sizeof bmap){
            fclose(fp);
            return false;
        }
        fclose(fp);

        AnsiString xmlpath;
        xmlpath.sprintf("%s\\BMap_area%02d.xml", dst.c_str(), mapnum + 1);
        TiXmlDocument doc(xmlpath.c_str());
        TiXmlDeclaration dec("1.0", XMLCHARCODE, "yes");
        doc.InsertEndChild(dec);

        TiXmlElement e_bmap("BattleMap");
        e_bmap.SetAttribute("Width", 32);
        e_bmap.SetAttribute("Height", 32);
        e_bmap.SetAttribute("City", bmap.CityKz);
        e_bmap.SetAttribute("Wall", bmap.WallKz);
        e_bmap.SetAttribute("Road", bmap.RoadKz);
        TiXmlElement e_chip("MapChip");
        TiXmlElement e_obj("MapObject");
        TiXmlElement e_objnum("MapObjectNo");
        TiXmlElement e_height("MapHeight");
        for(int y=0; y<32; y++){
            AnsiString str_chip, str_obj, str_objnum, str_height;
            for(int x=0; x<32; x++){
                // n``bvԍ
                AnsiString val_chip;
                if(x){
                    str_chip += ",";
                }
                val_chip.sprintf("%d", bmap.Char[x][y]);
                str_chip += val_chip;
                // 㕨̃`bvԍ
                AnsiString val_obj;
                if(x){
                    str_obj += ",";
                }
                val_obj.sprintf("%d", bmap.ObjChar[x][y]);
                str_obj += val_obj;
                // XEǁE̔ԍ
                AnsiString val_objnum;
                if(x){
                    str_objnum += ",";
                }
                val_objnum.sprintf("%d", bmap.ObjCharNo[x][y]);
                str_objnum += val_objnum;
                // 
                AnsiString val_height;
                if(x){
                    str_height += ",";
                }
                val_height.sprintf("%d", bmap.Height[x][y]);
                str_height += val_height;
            }
            AnsiString str_y;
            str_y.sprintf("L%d", y);

            TiXmlElement e_chipline(str_y.c_str());
            InsertTextChild(e_chipline, str_chip.c_str());
            e_chip.InsertEndChild(e_chipline);

            TiXmlElement e_objline(str_y.c_str());
            InsertTextChild(e_objline, str_obj.c_str());
            e_obj.InsertEndChild(e_objline);

            TiXmlElement e_objnumline(str_y.c_str());
            InsertTextChild(e_objnumline, str_objnum.c_str());
            e_objnum.InsertEndChild(e_objnumline);

            TiXmlElement e_heightline(str_y.c_str());
            InsertTextChild(e_heightline, str_height.c_str());
            e_height.InsertEndChild(e_heightline);
        }
        e_bmap.InsertEndChild(e_chip);
        e_bmap.InsertEndChild(e_obj);
        e_bmap.InsertEndChild(e_objnum);
        e_bmap.InsertEndChild(e_height);

        TiXmlElement e_neighborarea("NeighborArea");
        for(int dir=0; dir<8; dir++){
            TiXmlElement e_narea("Area");
            //e_narea.SetAttribute("id", dir);
            const std::string& nid = AreaIDNeighbor[mapnum][dir];
            if(!nid.empty()){
                SetTextAttribute(e_narea, "id", nid.c_str());
                for(int i=0; i<20; i++){
                    AnsiString str_i;
                    str_i.sprintf("Point%d", i);
                    TiXmlElement e_i(str_i.c_str());
                    e_i.SetAttribute("x", bmap.Escape[dir][i].x);
                    e_i.SetAttribute("y", bmap.Escape[dir][i].y);
                    e_narea.InsertEndChild(e_i);
                }
                e_neighborarea.InsertEndChild(e_narea);
            }
            //else{
            //    std::vector<char> zero(20*2);
            //    if(memcmp(&zero[0], &bmap.Escape[dir], 20*2)){
            //        OutputLog("%s ŗאڃGAw肳ĂȂɂւ炸̏zuWf[^݁B̃f[^̓Ro[gꂸj܂iȂHjB", bmappath.c_str());
            //    }
            //}
        }
        e_bmap.InsertEndChild(e_neighborarea);

        doc.InsertEndChild(e_bmap);
        if(!doc.SaveFile()){
            return false;
        }
    }

    return true;
}
//---------------------------------------------------------------------------
// n`Rs[B
bool TFarenCnv::ConvertMapChipTxt(const char *srcpath, const char *dstpath, const AnsiString& defscesrcpath)
{
    AnsiString src, dst;
    src.sprintf("%s\\Data\\TikeiType", srcpath);
    dst.sprintf("%s\\Data\\MapChip.txt", dstpath);
    FILE *fp_r = fopen(src.c_str(), "r");
    if(fp_r == NULL){
        if(!defscesrcpath.IsEmpty()){
            src.sprintf("%s\\Data\\TikeiType", defscesrcpath.c_str());
            fp_r = fopen(src.c_str(), "r");
        }
    }
    if(fp_r == NULL){
        return false;
    }
    FILE *fp_w = fopen(dst.c_str(), "w");
    if(fp_w == NULL){
        fclose(fp_r);
        return false;
    }
    char buf[256];
    while(fgets(buf, sizeof buf, fp_r) != NULL){
        std::vector<std::string> v;
        boost::algorithm::split(v, buf, boost::is_space());
        int count = std::remove(v.begin(), v.end(), "") - v.begin();
        for(int i=0; i<count; i++){
            if(i){
                fputc(' ', fp_w);
            }
            int val;
            if(sscanf(v[i].c_str(), "%x", &val) == 1){
                fprintf(fp_w, "%2d", val);
            }
        }
        fputc('\n', fp_w);
    }
    fclose(fp_r);
    fclose(fp_w);
    return true;
}
//---------------------------------------------------------------------------
// Œ̏㕨o͂B
bool TFarenCnv::OutputMapObjTxt(const char *dstpath)
{
    AnsiString dst;
    dst.sprintf("%s\\Data\\MapObj.txt", dstpath);

    FILE *fp = fopen(dst.c_str(), "w");
    if(fp == NULL){
        return false;
    }

    fprintf(fp, "0 1 1 1 1 1 1 4 4 4 4 4 4 4 4 4 2 2 2 2\n");
    fprintf(fp, "2 2 2 2 2 3 3 3 3 3 3 3 3 3 3 3 3 3 3 3\n");
    fclose(fp);
    return true;
}
//---------------------------------------------------------------------------
bool TFarenCnv::CopyFiles(const char *srcdir, const char *dstdir)
{
    // ɂꍇ̃G[͖
    CreateDirectory(dstdir, NULL);

    AnsiString filefilter;
    filefilter.sprintf("%s\\*.*", srcdir);
    WIN32_FIND_DATA findData;
    ZeroMemory(&findData, sizeof findData);
    HANDLE hFind = FindFirstFile(filefilter.c_str(), &findData);
    if(hFind != INVALID_HANDLE_VALUE){
        do{
            if(!(findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)){
                AnsiString srcfile;
                AnsiString dstfile;
                srcfile.sprintf("%s\\%s", srcdir, findData.cFileName);
                dstfile.sprintf("%s\\%s", dstdir, findData.cFileName);
                if(!CopyFile(srcfile.c_str(), dstfile.c_str(), FALSE)){
                    OutputLog("CopyFile Error : %s -> %s", srcfile.c_str(), dstfile.c_str());
                    return false;
                }
            }
        }while(FindNextFile(hFind, &findData));
        FindClose(hFind);
    }
    return true;
}
//---------------------------------------------------------------------------
bool TFarenCnv::CopyCnvSrc(const char *dstpath)
{
    AnsiString src, dst;
    src.sprintf("%s\\System", m_CnvSrcPath.c_str());
    dst.sprintf("%s\\System", dstpath);
    return CopyFiles(src.c_str(), dst.c_str());
}
//---------------------------------------------------------------------------
bool TFarenCnv::CopyMusicSound(const char *srcpath, const char *dstpath)
{
    AnsiString src, dst;
    src.sprintf("%s\\Music&Sound", srcpath);
    dst.sprintf("%s\\MusicAndSound", dstpath);
    return CopyFiles(src.c_str(), dst.c_str());
}
//---------------------------------------------------------------------------
