#include <vcl.h>
#include <stdio.h>
#include <vector>
#include <map>
#include <boost/algorithm/string.hpp>
#include "scriptcnv.h"
#include "sharedata.h"
#include "commonfunc.h"

namespace
{
    const char* TypeName[] = {"ɂȂ", "l", "}X^[ID", "GAID", "lID"};

    bool is_comma(unsigned char c){
        return c == ',';
    }

    bool is_space(unsigned char c){
        return c == ' ' || c == '\t' || c == '\n';
    }

    // chr SJIS2byte1byteڂ̒l
    bool is_sjis_2byte_1st(unsigned char chr)
    {
        return (chr >= 0x81 && chr <= 0x9F) || (chr >= 0xE0 && chr <= 0xFC);
    }

    // chr SJIS2byte2byteڂ̒l
    bool is_sjis_2byte_2nd(unsigned char chr)
    {
        return (chr >= 0x40 && chr <= 0x7E) || (chr >= 0x80 && chr <= 0xFC);
    }
}
//---------------------------------------------------------------------------
size_t FindChar(const std::string& str, unsigned char chr)
{
    size_t len = str.length();
    for(size_t i=0; i<len; ++i){
        if(str.at(i) == chr){
            return i;
        }
        if(is_sjis_2byte_1st(str.at(i))){
            ++i;
            if(!is_sjis_2byte_2nd(str.at(i))){
                OutputLog("[%d]ςȕ܂BiSJIS̑oCg1oCgڂ2oCgڂHj", i);
                return -1;
            }
        }
    }
    return -1;
}
//---------------------------------------------------------------------------
TFarenScriptCnv::TFarenScriptCnv(const std::vector<std::string>& chrids, const std::vector<std::string>& areaids) :
    CharaIDArray(chrids),
    AreaIDArray(areaids)
{
}
//---------------------------------------------------------------------------
bool TFarenScriptCnv::ConvertScripts(const char *srcpath, const char *dstpath)
{
	TDestValue v;
    v.MyType = ET_MASTERID;
    v.IndexType = ET_AREAID;
    v.Name = "api_scenario_get_area_master";
    TableNameMap["AreaKuni"] = v;

    v.MyType = ET_NONE;
    v.IndexType = ET_AREAID;
    v.Name = "api_scenario_get_area_chara_count";
    TableNameMap["AreaButaiKz"] = v;

    v.MyType = ET_MASTERID;
    v.IndexType = ET_CHARAID;
    v.Name = "api_get_chara_master";
    TableNameMap["HeroKuni"] = v;

    v.MyType = ET_AREAID;
    v.IndexType = ET_MASTERID;
    v.Name = "api_scenario_get_master_exists_area";
    TableNameMap["BaseArea"] = v;

    v.MyType = ET_BOOL;
    v.IndexType = ET_MASTERID;
    v.Name = "api_master_is_exists";
    TableNameMap["KuniFlag"] = v;

    v.MyType = ET_BOOL;
    v.IndexType = ET_MASTERID;
    v.Name = "api_master_is_player";
    //v.Name = "ofutil_master_event";
    TableNameMap["KuniPlayer"] = v;

    v.MyType = ET_AREAID;
    //v.IndexType = ET_CHARAID;
    v.IndexType = ET_CHARAID_HEROFLAG;
    v.Name = "api_scenario_get_chara_exists_area";
    TableNameMap["HeroFlag"] = v;

    v.MyType = ET_CHARAID;
    //v.IndexType = ET_MASTERID;
    v.IndexType = ET_PLANEINT;
    v.Name = "api_scenario_get_champion";
    TableNameMap["MostWin"] = v;

    v.MyType = ET_NONE;
    v.IndexType = ET_CHARAID;
    v.Name = "api_scenario_get_wincount";
    TableNameMap["WinKz"] = v;

    v.MyType = ET_NONE;
    v.IndexType = ET_MASTERID;
    v.Name = "api_master_get_ley";
    TableNameMap["Ley"] = v;

    TSimpleCmd cmd;
    cmd.ArgType = ET_AREAID;
    cmd.Name = "api_scenario_focus_area";
    SimpleCmdMap["AreaFocus"] = cmd;

    cmd.ArgType = ET_CHARAID;
    cmd.Name = "api_scenario_remove_chara";
    SimpleCmdMap["CharDelete"] = cmd;

    AnsiString src, dst;

    src.sprintf("%s\\Data\\EventScript", srcpath);
    dst.sprintf("%s\\Script\\EventScript.lua", dstpath);
    if(!ConvertScript(src.c_str(), dst.c_str())){
        return false;
    }

    return true;
}
//---------------------------------------------------------------------------
bool TFarenScriptCnv::OutputDummyScript(const char *dstpath)
{
    AnsiString dst;
    dst.sprintf("%s\\Script\\EventScript.lua", dstpath);
    FILE *fp = fopen(dst.c_str(), "w");
    if(fp == NULL){
        return false;
    }
    fprintf(fp, "function on_system_start()\n");
    fprintf(fp, "end\n");
    fprintf(fp, "\n");
    fprintf(fp, "function global_script()\n");
    fprintf(fp, "end\n");
    fclose(fp);
    return true;
}
//---------------------------------------------------------------------------
// rӒlƉZqƉEӒlɕB󔒂͖ƂB
void TFarenScriptCnv::SplitComparation(const std::string& src, std::string& lval, std::string& ope, std::string& rval)
{
    size_t sz;
    sz = src.find("==");
    if(sz != std::string::npos){
        lval = src.substr(0, sz);
        rval = src.substr(sz + 2);
        ope = "==";
        return;
    }
    sz = src.find("!=");
    if(sz != std::string::npos){
        lval = src.substr(0, sz);
        rval = src.substr(sz + 2);
        ope = "~=";
        return;
    }
    sz = src.find("<=");
    if(sz != std::string::npos){
        lval = src.substr(0, sz);
        rval = src.substr(sz + 2);
        ope = "<=";
        return;
    }
    sz = src.find(">=");
    if(sz != std::string::npos){
        lval = src.substr(0, sz);
        rval = src.substr(sz + 2);
        ope = ">=";
        return;
    }
    sz = src.find('<');
    if(sz != std::string::npos){
        lval = src.substr(0, sz);
        rval = src.substr(sz + 1);
        ope = "<";
        return;
    }
    sz = src.find('>');
    if(sz != std::string::npos){
        lval = src.substr(0, sz);
        rval = src.substr(sz + 1);
        ope = ">";
        return;
    }
    lval = src;
    rval = "false";
    ope = "~=";
}
//---------------------------------------------------------------------------
int TFarenScriptCnv::GetArrayIndex(const char* src)
{
    const char* p = src;
    while(isalpha(*p)){
        p++;
    }
    if(*p != '['){
        return -1;
    }
    p++;
    char buf[32];
    int i = 0;
    while(isdigit(*p)){
        if(i >= (sizeof buf)-1){
            return -1;
        }
        buf[i] = *p;
        i++;
        p++;
    }
    if(*p != ']'){
        return -1;
    }
    buf[i] = '\0';
    int val;
    if(sscanf(buf, "%d", &val) != 1){
        return -1;
    }
    return val;
}
//---------------------------------------------------------------------------
void TFarenScriptCnv::PrintTab(FILE *fp, int tabs)
{
    for(int i=0; i<tabs; i++){
        fputc('\t', fp);
    }
}
//---------------------------------------------------------------------------
void TFarenScriptCnv::RegistVariable(TVarTable& VarTable, std::vector<TDstLine>& DstLines, const std::string& varname, TExpectedType et)
{
    TVariable var;
    var.VarType = et;
    // ɐ錾ς݂̕ϐ̌^ŕύXꂽꍇA錾̕CB
    TVarTable::const_iterator itr = VarTable.find(varname);
    if(itr != VarTable.end()){
        if(itr->second.LastDeclaredLine != -1){
            if(et != ET_PLANEINT){
                // ɂϐŁA^łȂAɍŌ̎g̍ӂ̏ꍇ͉EӂnilɕύXB
                AnsiString chkstr;
                chkstr.sprintf("%s = ", varname.c_str());
                if(!strncmp(DstLines[itr->second.LastDeclaredLine].Value.c_str(), chkstr.c_str(), chkstr.Length())){
                    AnsiString str;
                    str.sprintf("%s = nil\n", varname.c_str());
                    DstLines[itr->second.LastDeclaredLine].Value = str.c_str();
                }
            }
        }
    }
    // ^ll̏ꍇ͌nilɕύX邩Ȃ̂ōŌ̐錾soĂB
    // ̏̂͐ll̂݁B
    if(et == ET_PLANEINT){
        var.LastDeclaredLine = DstLines.size();
    }
    else{
        // ^llłȂꍇ͑O̒lpŌ̐錾sȂƂB
        if(itr != VarTable.end()){
            var.LastDeclaredLine = itr->second.LastDeclaredLine;
        }
        else{
            var.LastDeclaredLine = -1;
        }
    }
    VarTable[varname] = var;
}
//---------------------------------------------------------------------------
// realtype:ۂɌoꂽ^iʂł͍Ӓl̉ߌʂɎgj
// exptype:҂^iʂł͉EӒl̊҂^Ɏgj
// VarTableDstLinesRegistVariableĂяô߂ɂB
std::string TFarenScriptCnv::ConvertVariable(TVarTable& VarTable, std::vector<TDstLine>& DstLines, AnsiString& errstr, const std::string src, TExpectedType& realtype, TExpectedType exptype)
{
    realtype = ET_NONE;
    if(src.empty()){
        return "";
    }

    if(!src.compare("true")){
        return "true";
    }
    else if(!src.compare("false")){
        return "false";
    }
    else if(src.at(0) == '%'){
        std::string dst = "\"";
        int len = src.length();
        for(int i=1; i<len; i++){
            unsigned char c = src.at(i);
            if(isalnum(c)){
                dst += c;
            }
            else if(i+1 < len){
                if(!is_sjis_2byte_1st(c)){
                    break;
                }
                unsigned char c2 = src.at(i+1);
                if(!is_sjis_2byte_2nd(c2)){
                    break;
                }
                dst += c;
                dst += c2;
                i++;
            }
            else{
                break;
            }
        }
        dst += "\"";
        realtype = ET_MASTERID;
        return dst;
    }
    else if(!src.compare(0, 5, "Flag[")){
        /*int index = GetArrayIndex(src.c_str());
        if(index == -1){
            errstr.sprintf("Flag̓Y̌oɎs܂B");
            return "";
        }*/
        if(src.length() < 6){
            errstr.sprintf("Flag̓Y̌oɎs܂B");
            return "";
        }
        std::string strindex = src.substr(5, src.length() - 6);
        int index;
        sscanf(strindex.c_str(), "%d", &index);
        if(sscanf(strindex.c_str(), "%d", &index) == 1){
            if(index > 400){
                TExpectedType et;
                std::string master_id = ConvertVariable(VarTable, DstLines, errstr, strindex, et, ET_MASTERID);
                if(!errstr.IsEmpty()){
                    return "";
                }
                if(master_id.empty()){
                    errstr.sprintf("Flag[4xx]̓Ỹ}X^[IDւ̕ϊɎs܂B");
                    return "";
                }
                AnsiString dst;
                dst.sprintf("api_scenario_is_ending(%s)", master_id.c_str());
                realtype = ET_BOOL;
                return dst.c_str();
            }
            else if(!index){
                // Flag[0]́uGfBOiׂājv̔ŎgĂB
                realtype = ET_BOOL;
                return "api_scenario_is_ending()";
            }
            else{
                // ʏz
                return src;
            }
        }
        else{
            errstr.sprintf("Flag̓Ył͂Ȃ悤łBzFlag̎głB");
            return "";
        }
    }
    else{
        size_t sz = FindChar(src, '[');
        if(sz != -1 && sz > 0 && sz < src.length()){
            std::string key = src.substr(0, sz);
            TTableNameMap::const_iterator itr = TableNameMap.find(key);
            if(itr != TableNameMap.end()){
                std::string rest = src.substr(sz + 1);
                size_t restlen = rest.length();
                // ʂG[
                if(!restlen || rest.at(restlen - 1) != ']'){
                    errstr.sprintf("z`̋Lqłʂoł܂łB");
                    return "";
                }
                rest = rest.substr(0, restlen - 1);
                TExpectedType et;
                std::string strindex = ConvertVariable(VarTable, DstLines, errstr, rest, et, itr->second.IndexType);
                if(!errstr.IsEmpty()){
                    return "";
                }
                if(strindex.empty()){
                    errstr.sprintf("%s̓Y̕ϊɎs܂BYɊ҂ꂽ^%słB", key.c_str(), TypeName[itr->second.IndexType]);
                    return "";
                }
                realtype = itr->second.MyType;
                return itr->second.Name + "(" + strindex + ")";
            }
            else{
                // ʏz"Flag"ȊOɖ̂ŃG[
                errstr.sprintf("m̔z\"%s\"̕ϊɎs܂B", key.c_str());
                return "";
            }
        }
    }

    if(!src.compare("Scenario")){
        return "api_get_chapter_number()";
    }
    if(!src.compare("StartTurn")){
        return "api_get_progressed_turn()";
    }
    if(!src.compare("Turn")){
        return "api_get_turn()";
    }
    if(!src.compare(0, 6, "Random")){
        if(src.length() < 8 || src.at(6) != '(' || src.at(src.length()-1) != ')'){
            errstr.sprintf("Random̈̎擾Ɏs܂B");
            return "";
        }
        std::string arg = src.substr(7, src.length() - 8);
        AnsiString dst;
        dst.sprintf("(math.random(%s)-1)", arg.c_str());
        return dst.c_str();
    }

    // lƂĉ߂Ă݂B߂łȂ琔lȊO̕ƂB
    int val;
    if(sscanf(src.c_str(), "%d", &val) != 1){
        TVarTable::const_iterator itr = VarTable.find(src);
        if(itr != VarTable.end()){
            // ɂϐŁA̌^҂Ăꍇ͌^o^B
            if(exptype != ET_NONE){
                RegistVariable(VarTable, DstLines, src, exptype);
            }
            // ϐ̌^ԂB
            realtype = itr->second.VarType;
            return src;
        }
        else{
            // 񑦒lB
            realtype = ET_PLANESTR;
            AnsiString str;
            str.sprintf("\"%s\"", src.c_str());
            return str.c_str();
            //errstr.sprintf("\"%s\"ϐƉ߂܂A^̌oɎs܂B", src.c_str());
            //return "";
        }
    }

    // ID҂ĂƂɐl萔oĂϊB
    if(exptype == ET_MASTERID || exptype == ET_CHARAID || exptype == ET_CHARAID_HEROFLAG){
        if(val > 400){
            // GfBOtȌꍇ
            val -= 401;
        }
        else{
            if(exptype == ET_CHARAID_HEROFLAG){
                // DefaultViI2564sڂł̃[gʂB
                val -= 21;
            }
            else{
                val -= 1;
            }
        }
        if(val < 0 || static_cast<unsigned>(val) >= CharaIDArray.size()){
            errstr.sprintf("\"%s\"LIDƉ߂܂AϊɎs܂B", src.c_str());
            realtype = ET_PLANEINT; // ^͌ołĂB
            return "";
        }
        realtype = ET_MASTERID;
        AnsiString str;
        str.sprintf("\"%s\"", CharaIDArray[val].c_str());
        return str.c_str();
    }
    else if(exptype == ET_AREAID){
        realtype = ET_AREAID;
        //AnsiString str;
        //str.sprintf("\"area%d\"", val);
        //return str.c_str();
        val -= 1;
        if(static_cast<unsigned>(val) < AreaIDArray.size()){
            AnsiString str;
            str.sprintf("\"%s\"", AreaIDArray[val].c_str());
            return str.c_str();
        }
        else{
            //return "ȃGAID";
            return "nil";
        }
    }
    // llB
    realtype = ET_PLANEINT;
    return src;
}
//---------------------------------------------------------------------------
bool TFarenScriptCnv::ConvertScript(const char *srcpath, const char *dstpath)
{
    FILE *fp = fopen(srcpath, "r");
    if(fp == NULL){
        AnsiString str;
        str.printf("%s ̓ǍɎs܂B", srcpath);
        ShowMessage(str);
        return false;
    }
    FILE *fp_out = fopen(dstpath, "w");
    if(fp_out == NULL){
        AnsiString str;
        str.printf("%s ̍쐬Ɏs܂B", dstpath);
        ShowMessage(str);
        return false;
    }
    FILE *fp_err = fopen("error.txt", "w");
    if(fp_err == NULL){
        return false;
    }

    std::vector<TDstLine> DstLines;
    {
        TDstLine dl;
        dl.Value = "Flag = {}\n";
        dl.TabLevel = 0;
        DstLines.push_back(dl);

        dl.Value = "function global_script()\n";
        dl.TabLevel = 0;
        DstLines.push_back(dl);
    }
    AnsiString errstr;
    int srcnum = 0;
    int tablevel = 1;   // Ŝ֐ƌȂB
    int lastifnum = -1;
    int laststartnum = -1;
    int lastgotonum = -1;
    int lastgototablevel = 0;
    // oĂKvs̈B
    int lastopenedwindow = -1;
    int lastwindowon = 0;
    bool isConversation = false;
    for(;;){
        char buf[1024];
        if(fgets(buf, sizeof buf, fp) == NULL){
            break;
        }
        srcnum++;
        // ̂Z~RĂsiftHViɈӏĵŎ菜B
        char *semicolon = strchr(buf, ';');
        if(semicolon != NULL){
            *semicolon = '\0';
        }
        const char *bufhead = buf;
        while(*bufhead != '\0' && is_space(*bufhead)){
            bufhead++;
        }
        if(!strncmp(bufhead, "//", 2)){
            AnsiString str;
            str.sprintf("-- %s", &bufhead[2]);
            TDstLine dl;
            dl.Value = str.c_str();
            dl.TabLevel = tablevel;
            DstLines.push_back(dl);
            continue;
        }

        std::vector<std::string> v;
        //boost::algorithm::split(v, buf, boost::is_space());
        boost::algorithm::split(v, bufhead, is_space);
        // 炩ɗvfȂƂ΂
        if(v.empty()){
            continue;
        }
        // 󕶎̗vfžɂǂ
        std::remove(v.begin(), v.end(), "");
        //if(srcnum==101){
        //    for(unsigned u=0;u<v.size();u++)
        //        ShowMessage(v[u].c_str());
        //}

        if(v[0].empty()){
            // s͂̂܂܏óB
            TDstLine dl;
            dl.Value = "\n";
            dl.TabLevel = 0;
            DstLines.push_back(dl);
        }
        else if(!v[0].compare("Check")){
            // PɖB
        }
        else if(!v[0].compare("if")){
            if(v.size() < 2){
                errstr.sprintf("if̌ɕ񂪌܂łB");
                break;
            }
            if(v[1].at(0) != '('){
                errstr.sprintf("if̌'('܂łB");
                break;
            }
            //AnsiString dstline = "if ";
            AnsiString dstline;
            // '('
            v[1] = v[1].substr(1);
            // 胋[v
            for(int i=1; ; i++){
                //if(srcnum==101){
                    //ShowMessage(buf);
                    //ShowMessage(v[i].c_str());
                //}
                // Ō̕񂩂ǂ肷Bʂƕ̊Ԃ͖̂ł悵B
                bool is_end = false;
                size_t sz = v[i].length();
                if(v[i].at(sz-1) == ')'){
                    is_end = true;
                    v[i] = v[i].substr(0, sz-1);
                }

                // r
                std::string lval, rval, ope;
                SplitComparation(v[i], lval, ope, rval);

                TExpectedType et_l, et_r;
                std::string lval2 = ConvertVariable(VarTable, DstLines, errstr, lval, et_l);
                if(!errstr.IsEmpty()){
                    break;
                }
                if(lval2.empty()){
                    errstr.sprintf("if̏̍\"%s\"̕ϊɎs܂B", lval.c_str());
                    break;
                }
                // Eӂɂ͍ӂƓ^҂B
                std::string rval2 = ConvertVariable(VarTable, DstLines, errstr, rval, et_r, et_l);
                if(et_l == ET_MASTERID || et_l == ET_CHARAID || et_l == ET_AREAID){
                    if(ope.compare("==") && ope.compare("~=")){
                        // sȊOBȂ킿召rB
                        // TODO:{͑召r̓GAID̔r̗ɕϊׂA
                        // DefaultViIɂ͂̕Kvӏ̂ŖB
                        int n;
                        if(!strncmp(lval.c_str(), "HeroFlag", 8) && (!ope.compare(">") || !ope.compare(">=")) &&
                           sscanf(rval.c_str(), "%d", &n) == 1 && n - 1 >= static_cast<signed>(AreaIDArray.size())){
                            // DefaultViI HeroFlag[%AeB}Cg]>62 ĂƂB
                            errstr = "";
                            lval2 = "";
                            ope = "";
                            rval2 = "false";
                        }
                        else{
                            // 召rsĂƂ͗L̔ƂB
                            errstr = "";
                            rval2 = "nil";
                            ope = "~=";
                        }
                    }
                    else if(!errstr.IsEmpty()){
                        // LID̔sĂƎv鎮͐ƂB
                        errstr = "";
                        rval2 = "nil";
                        if(ope.compare("==")){
                            ope = "~=";
                        }
                    }
                }
                if(et_l != et_r){
                    if(et_l == ET_BOOL && et_r == ET_PLANEINT){
                        // ^UlrĂ鐔ll͐^UllɕϊB
                        rval2 = rval.compare("0") ? "true" : "false";
                        if(ope.compare("==")){
                            ope = "~=";
                        }
                    }
                }
                if(!errstr.IsEmpty()){
                    break;
                }
                if(rval2.empty()){
                    errstr.sprintf("if̏̉E\"%s\"̕ϊɎs܂B", rval.c_str());
                    break;
                }
                //fprintf(fp_out, "%s%s%s ", lval2.c_str(), ope.c_str(), rval2.c_str());
                dstline = "(" + dstline;    // EΉ
                dstline += lval2.c_str();
                dstline += ope.c_str();
                dstline += rval2.c_str();
                dstline += ")";             // EΉ
                dstline += " ";

                if(is_end){
                    dstline = "if " + dstline;
                    dstline += "then\n";
                    TDstLine dl;
                    dl.Value = dstline.c_str();
                    dl.TabLevel = tablevel;
                    lastifnum = DstLines.size();
                    DstLines.push_back(dl);
                    tablevel++;
                    break;
                }
                else{
                    i++;
                    if(v.size() <= static_cast<unsigned>(i)){
                        errstr.sprintf("if̘̏_Zq܂͏I[̌oɎs܂B");
                        break;
                    }
                    // ̕
                    if(!v[i].compare("&&")){
                        dstline += "and ";
                    }
                    else if(!v[i].compare("||")){
                        dstline += "or ";
                    }
                    else{
                        errstr.sprintf("if̘̏_Zq̌oɎs܂B");
                        break;
                    }
                }
            }
            if(!errstr.IsEmpty()){
                break;
            }
        }
        else if(!v[0].compare("{")){
            // Ȃ
        }
        else if(!v[0].compare("}")){
            tablevel--;
            if(tablevel == 1){
                isConversation = false;
            }
            //PrintTab(fp_out, tablevel);
            //fprintf(fp_out, "end\n");
            TDstLine dl;
            dl.TabLevel = tablevel;
            dl.Value = "end\n";
            DstLines.push_back(dl);
        }
        else if(!v[0].compare("Start")){
            AnsiString dstline;
            dstline.sprintf("while true do\n");
            TDstLine dl;
            dl.TabLevel = tablevel;
            dl.Value = dstline.c_str();
            laststartnum = DstLines.size();
            DstLines.push_back(dl);
            tablevel++;
        }
        else if(!v[0].compare("GoTo")){
            if(laststartnum == -1){
                errstr.sprintf("GoToɑΉStarťoɎs܂B");
                break;
            }
            if(lastifnum == -1 || lastifnum <= laststartnum){
                errstr.sprintf("GoToɑΉif̌oɎs܂BsƖ[vɂȂ鋰ꂪ܂B");
                break;
            }
            AnsiString dstline;
            dstline.sprintf("else\n");
            TDstLine dl;
            dl.TabLevel = tablevel - 1;
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);
            dstline.sprintf("break\n");
            dl.TabLevel = tablevel;
            dl.Value = dstline.c_str();
            lastgotonum = DstLines.size();
            lastgototablevel = tablevel;
            DstLines.push_back(dl);
            //tablevel--;
            //dstline.sprintf("end\n");
            //dl.TabLevel = tablevel;
            //dl.Value = dstline.c_str();
            //DstLines.push_back(dl);
        }
        else if(!v[0].compare("End")){
            TDstLine dl;
            AnsiString dstline;
            if(lastgotonum != -1){
                // ŌɏoĂGoTo̒whileɑΉendB
                dstline.sprintf("end\n");
                dl.TabLevel = lastgototablevel - 1;
                dl.Value = dstline.c_str();
                std::vector<TDstLine>::iterator itr_ins = DstLines.insert(DstLines.begin() + lastgotonum + 1, dl);
                // ̌̎͂PB
                for(std::vector<TDstLine>::iterator itr = itr_ins+1; itr != DstLines.end(); itr++){
                    itr->TabLevel--;
                }
                // whileŏオ̎B
                tablevel--;
            }
            lastgotonum = -1;
            dstline.sprintf("return\n");
            dl.TabLevel = tablevel;
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);
        }
        /*else if(!v[0].compare("AreaFocus")){
            TExpectedType et;
            std::string index = ConvertVariable(VarTable, DstLines, errstr, v[1], et, ET_AREAID);
            if(!errstr.IsEmpty()){
                break;
            }
            AnsiString dstline;
            dstline.sprintf("api_area_focus(%s);\n", index.c_str());
            TDstLine dl;
            dl.TabLevel = tablevel;
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);
        }*/
        else if(!v[0].compare("Set")){
            std::vector<std::string> args;
            const char* p = v[1].c_str();
            boost::algorithm::split(args, p, is_comma);
            if(args.size() < 2){
                errstr.sprintf("Seẗ\"%s\"̉߂Ɏs܂B", v[1].c_str());
                break;
            }
            AnsiString dstline;
            if(!args[1].compare(0, 5, "YesNo")){
                if(args[1].length() < 7){
                    errstr.sprintf("Seẗ\"%s\"̉߂Ɏs܂B", args[1].c_str());
                    break;
                }
                std::string msg = args[1].substr(6, args[1].length() - 7);
                dstline.sprintf("%s = ofutil_dialog_yesno(\"%s\")\n", args[0].c_str(), msg.c_str());
                // ϐ̌^L^B
                RegistVariable(VarTable, DstLines, args[0], ET_BOOL);
            }
            else{
                TExpectedType realtype;
                std::string rval = ConvertVariable(VarTable, DstLines, errstr, args[1], realtype);
                if(!errstr.IsEmpty()){
                    break;
                }
                bool output = false;
                if(!args[0].compare(0, 5, "Flag[")){
                    // Flag[401`]ւSet͓ʂɉ߂B
                    std::string strindex = args[0].substr(5, args[0].length() - 6);
                    int index;
                    if(sscanf(strindex.c_str(), "%d", &index) == 1 && index > 400){
                        TExpectedType et;
                        std::string master_id = ConvertVariable(VarTable, DstLines, errstr, strindex, et, ET_MASTERID);
                        if(!errstr.IsEmpty()){
                            return "";
                        }
                        if(master_id.empty()){
                            errstr.sprintf("Flag[4xx]̓Ỹ}X^[IDւ̕ϊɎs܂B");
                            return "";
                        }
                        dstline.sprintf("api_scenario_set_ending_enabled(%s, %s)\n", master_id.c_str(), rval.c_str());
                        output = true;
                    }
                }
                if(!output){
                    // ʏ̑B
                    dstline.sprintf("%s = %s\n", args[0].c_str(), rval.c_str());
                    // ϐ̌^L^B
                    RegistVariable(VarTable, DstLines, args[0], realtype);
                }
            }
            TDstLine dl;
            dl.TabLevel = tablevel;
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);
        }
        else if(!v[0].compare("Music")){
            if(v.size() < 2 || v[1].empty()){
                errstr.sprintf("Music̈̎擾Ɏs܂B");
                break;
            }
            //PrintTab(fp_out, tablevel);
            TExpectedType et;
            std::string index = ConvertVariable(VarTable, DstLines, errstr, v[1], et);
            if(!errstr.IsEmpty()){
                break;
            }
            AnsiString dstline;
            dstline.sprintf("api_music_set(%s)\n", index.c_str());
            TDstLine dl;
            dl.TabLevel = tablevel;
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);
            //fprintf(fp_out, "api_music_set(%s)\n", index.c_str());
        }
        else if(!v[0].compare("Window")){
            if(v.size() < 2 || v[1].empty()){
                errstr.sprintf("Windoẅ̎擾Ɏs܂B");
                break;
            }
            if(sscanf(v[1].c_str(), "%d", &lastopenedwindow) != 1){
                errstr.sprintf("Windoẅ̉߂Ɏs܂B");
                break;
            }
            AnsiString dstline;
            //dstline.sprintf("api_window_set_visible(window_handle%d, true)\n", lastopenedwindow);
            dstline.sprintf("ofutil_window_show(window_handle%d)\n", lastopenedwindow);
            TDstLine dl;
            dl.TabLevel = tablevel;
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);

            //dl.Value = "line = 0\n";
            dl.Value = "ofutil_reset_text()\n";
            DstLines.push_back(dl);
        }
        else if(!v[0].compare("Conversation") || !v[0].compare("FaceOut")){
            if(v.size() < 2 || v[1].empty()){
                errstr.sprintf("Conversation̈̎擾Ɏs܂B");
                break;
            }
            isConversation = true;
            TExpectedType et;
            std::string arg = ConvertVariable(VarTable, DstLines, errstr, v[1], et, ET_CHARAID);
            if(!errstr.IsEmpty()){
                break;
            }
            AnsiString dstline;
            TDstLine dl;
            dl.TabLevel = tablevel;
            dstline.sprintf("api_window_clear_image(window_handle%d, LAYER_FACEORCHIP)\n", lastopenedwindow);
            dstline.sprintf("api_window_clear_text(window_handle%d, LAYER_FACEORCHIP)\n", lastopenedwindow);
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);

            //dstline.sprintf("api_window_set_faceorchip(window_handle%d, FACE_X, FACE_Y, LAYER_FACEORCHIP, %s)\n", lastopenedwindow, arg.c_str());
            dstline.sprintf("ofutil_face(window_handle%d, LAYER_FACEORCHIP, %s)\n", lastopenedwindow, arg.c_str());
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);
        }
        else if(!v[0].compare("Print")){
            if(v.size() < 2 || v[1].empty()){
                errstr.sprintf("Prinẗ̎擾Ɏs܂B");
                break;
            }
            // ͉̈߂KvȂH
            //TExpectedType et;
            //std::string arg = ConvertVariable(VarTable, DstLines, errstr, v[1], et);
            //if(!errstr.IsEmpty()){
            //    break;
            //}
            //const char *textx;
            //textx = isConversation ? "TEXT_X" : "-1";
            AnsiString dstline;
            //dstline.sprintf("api_window_set_text(window_handle%d, %s, TEXT_Y + line * LINE_SKIP, LAYER_TEXT, '%s')\n", lastopenedwindow, textx, v[1].c_str());
            if(isConversation){
                dstline.sprintf("ofutil_talk(window_handle%d, '%s')\n", lastopenedwindow, v[1].c_str());
            }
            else{
                dstline.sprintf("ofutil_narration(window_handle%d, '%s')\n", lastopenedwindow, v[1].c_str());
            }
            TDstLine dl;
            dl.TabLevel = tablevel;
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);

            //dl.Value = "line = line + 1\n";
            //DstLines.push_back(dl);
        }
        else if(!v[0].compare("PrintKz")){
            if(v.size() < 2 || v[1].empty()){
                errstr.sprintf("PrintKz̈̎擾Ɏs܂B");
                break;
            }
            TExpectedType et;
            std::string arg = ConvertVariable(VarTable, DstLines, errstr, v[1], et);
            if(!errstr.IsEmpty()){
                break;
            }
            AnsiString dstline;
            //dstline.sprintf("api_window_draw_number(%s)\n", arg.c_str());
            //dstline.sprintf("api_window_set_text(window_handle%d, %s)\n", lastopenedwindow, arg.c_str());
            dstline.sprintf("ofutil_talk(window_handle%d, string.format('%%d', %s))\n", lastopenedwindow, arg.c_str());
            TDstLine dl;
            dl.TabLevel = tablevel;
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);
        }
        else if(!v[0].compare("WindowOn")){
            if(v.size() < 2 || v[1].empty()){
                errstr.sprintf("WindowOn̈̎擾Ɏs܂B");
                break;
            }
            TExpectedType et;
            std::string arg = ConvertVariable(VarTable, DstLines, errstr, v[1], et);
            if(!errstr.IsEmpty()){
                break;
            }
            // sJԂĂWindowOn1ɂ܂Ƃ߂B
            if(DstLines.size() > 0 && !strncmp(DstLines.back().Value.c_str(), "show_window_period", 18)){
                // sJԂ2ڈȍ~WindowOn
                int windowon = 0;
                sscanf(arg.c_str(), "%d", &windowon);
                lastwindowon += windowon;
                AnsiString dstline;
                dstline.sprintf("show_window_period(window_handle%d, %d)\n", lastopenedwindow, lastwindowon);
                DstLines.back().Value = dstline.c_str();
            }
            else{
                // 1sA͕sJԂ̍ŏWindowOn
                lastwindowon = 0;
                sscanf(arg.c_str(), "%d", &lastwindowon);
                AnsiString dstline;
                dstline.sprintf("show_window_period(window_handle%d, %d)\n", lastopenedwindow, lastwindowon);
                TDstLine dl;
                dl.TabLevel = tablevel;
                dl.Value = dstline.c_str();
                DstLines.push_back(dl);
            }
        }
        else if(!v[0].compare("Clear")){
            AnsiString dstline;
            TDstLine dl;
            dl.TabLevel = tablevel;
            dstline.sprintf("api_window_clear_text(window_handle%d, LAYER_TEXT)\n", lastopenedwindow);
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);

//            dstline.sprintf("api_window_clear_image(window_handle%d)\n", lastopenedwindow);
//            dl.Value = dstline.c_str();
//            DstLines.push_back(dl);

            //dl.Value = "line = 0\n";
            dl.Value = "ofutil_reset_text()\n";
            DstLines.push_back(dl);
        }
        else if(!v[0].compare("WindowOff")){
            for(int i=0; i<2; ++i){
                AnsiString dstline;
                dstline.sprintf("api_window_set_visible(window_handle%d, false)\n", i);
                TDstLine dl;
                dl.TabLevel = tablevel;
                dl.Value = dstline.c_str();
                DstLines.push_back(dl);
            }
        }
        else if(!v[0].compare("Size")){
            if(v.size() < 2 || v[1].empty()){
                errstr.sprintf("Sizë̎擾Ɏs܂B");
                break;
            }
            // ,ŋ؂ꂽ̊Ԃɋ󔒂͂Ȃ̂ƂB
            TExpectedType et;
            std::string arg1 = ConvertVariable(VarTable, DstLines, errstr, v[1], et);
            if(!errstr.IsEmpty()){
                break;
            }
            AnsiString dstline;
            dstline.sprintf("api_window_set_size(window_handle%d, %s)\n", lastopenedwindow, arg1.c_str());
            TDstLine dl;
            dl.TabLevel = tablevel;
            dl.Value = dstline.c_str();
            //DstLines.push_back(dl);   // TODO:Ƃ肠OFF
        }
        else if(!v[0].compare("CharSet")){
            if(v.size() < 2 || v[1].empty()){
                errstr.sprintf("CharSeẗ̎擾Ɏs܂B");
                break;
            }
            std::vector<std::string> args;
            const char* p = v[1].c_str();
            boost::algorithm::split(args, p, is_comma);
            if(args.size() < 2){
                errstr.sprintf("CharSeẗ\"%s\"̉߂Ɏs܂B", v[1].c_str());
                break;
            }
            TExpectedType et;
            std::string lval = ConvertVariable(VarTable, DstLines, errstr, args[0], et, ET_AREAID);
            if(!errstr.IsEmpty()){
                break;
            }
            std::string rval = ConvertVariable(VarTable, DstLines, errstr, args[1], et, ET_CHARAID);
            if(!errstr.IsEmpty()){
                break;
            }

            AnsiString dstline;
            dstline.sprintf("api_scenario_insert_area_chara(%s, %s)\n", lval.c_str(), rval.c_str());
            TDstLine dl;
            dl.TabLevel = tablevel;
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);
        }
        else if(!v[0].compare("Plus")){
            std::vector<std::string> args;
            const char* p = v[1].c_str();
            boost::algorithm::split(args, p, is_comma);
            if(args.size() < 2){
                errstr.sprintf("Plus̈\"%s\"̉߂Ɏs܂B", v[1].c_str());
                break;
            }
            AnsiString dstline;
            if(!args[0].compare(0, 3, "Ley")){
                if(args[0].length() < 5){
                    errstr.sprintf("Plus Leÿ\"%s\"̉߂Ɏs܂B", args[0].c_str());
                    break;
                }
                std::string keystr = args[0].substr(4, args[0].length() - 5);
                TExpectedType et;
                std::string key = ConvertVariable(VarTable, DstLines, errstr, keystr, et, ET_MASTERID);
                dstline.sprintf("api_master_plus_ley(%s, %s)\n", key.c_str(), args[1].c_str());
            }
            else if(!args[0].compare(0, 8, "AreaCity") ||
               !args[0].compare(0, 8, "AreaWall") ||
               !args[0].compare(0, 8, "AreaRoad"))
            {
                if(args[0].length() < 10){
                    errstr.sprintf("Plus̈\"%s\"̉߂Ɏs܂B", args[0].c_str());
                    break;
                }
                std::string keystr = args[0].substr(9, args[0].length() - 10);
                TExpectedType et;
                std::string key = ConvertVariable(VarTable, DstLines, errstr, keystr, et, ET_AREAID);
                // ł܂r̂͂ƂËB
                if(!args[0].compare(0, 8, "AreaCity")){
                    dstline.sprintf("api_scenario_plus_area_city(%s, %s)\n", key.c_str(), args[1].c_str());
                }
                else if(!args[0].compare(0, 8, "AreaWall")){
                    dstline.sprintf("api_scenario_plus_area_wall(%s, %s)\n", key.c_str(), args[1].c_str());
                }
                else{
                    dstline.sprintf("api_scenario_plus_area_road(%s, %s)\n", key.c_str(), args[1].c_str());
                }
            }
            else{
                TExpectedType et_l, et_r;
                std::string lval = ConvertVariable(VarTable, DstLines, errstr, args[0], et_l);
                std::string rval = ConvertVariable(VarTable, DstLines, errstr, args[1], et_r);
                if(et_l == ET_MASTERID){
                    // ftHgViÍuGfBOiׂājvł̉ZgĂƊႢA
                    // ۂɂ͈ӖB}X^[ID̉Z͏ȂƂftHgViIɂ݂͑Ȃ̂ŁA
                    // iet_l == ET_MASTERID̏ꍇj̏ubN͕svH
                    if(et_r == ET_PLANEINT && !rval.compare("1")){
                        dstline.sprintf("%s = api_master_get_next(%s)\n", lval.c_str(), lval.c_str());
                    }
                    else{
                        errstr.sprintf("Plus̈̕ϐ\"%s\"̓}X^[IDƉ߂܂ARo[^͂̉Z̕ϊT|[gĂ܂B", lval.c_str());
                        break;
                    }
                }
                else if(et_l == ET_MASTERID){
                    if(et_r == ET_PLANEINT && !rval.compare("1")){
                        dstline.sprintf("%s = api_area_get_next(%s)\n", lval.c_str(), lval.c_str());
                    }
                    else{
                        errstr.sprintf("Plus̈̕ϐ\"%s\"̓GAIDƉ߂܂ARo[^͂̉Z̕ϊT|[gĂ܂B", lval.c_str());
                        break;
                    }
                }
                else{
                    // ʏϐ
                    dstline.sprintf("%s = %s + %s\n", lval.c_str(), lval.c_str(), rval.c_str());
                }
            }
            TDstLine dl;
            dl.TabLevel = tablevel;
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);
        }
        else if(!v[0].compare("Picture")){
            std::vector<std::string> args;
            const char* p = v[1].c_str();
            boost::algorithm::split(args, p, is_comma);
            if(args.size() < 3){
                errstr.sprintf("Picturë\"%s\"̉߂Ɏs܂B", v[1].c_str());
                break;
            }
            AnsiString dstline;
            dstline.sprintf("api_window_set_image(window_handle%d, %s, %s, 0, IMAGE_EVENT, api_image_load(\"%s.bmp\"))\n", lastopenedwindow, args[1].c_str(), args[2].c_str(), args[0].c_str());
            TDstLine dl;
            dl.TabLevel = tablevel;
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);
        }
        else if(!v[0].compare("OutPrint")){
            std::vector<std::string> args;
            const char* p = v[1].c_str();
            boost::algorithm::split(args, p, is_comma);
            if(args.size() < 3){
                errstr.sprintf("OutPrinẗ\"%s\"̉߂Ɏs܂B", v[1].c_str());
                break;
            }
            AnsiString dstline;
            dstline.sprintf("api_window_set_text(window_handle%d, %s, %s, 0, '%s')\n", lastopenedwindow, args[0].c_str(), args[1].c_str(), args[2].c_str());
            TDstLine dl;
            dl.TabLevel = tablevel;
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);
        }
        else{
            TSimpleCmdMap::const_iterator itr = SimpleCmdMap.find(v[0]);
            if(itr == SimpleCmdMap.end()){
                errstr.sprintf("m̎s \"%s\" ܂B", v[0].c_str());
                break;
            }
            if(v.size() < 2 || v[1].empty()){
                errstr.sprintf("%s̈̎擾Ɏs܂B", v[0].c_str());
                break;
            }
            TExpectedType et;
            std::string arg = ConvertVariable(VarTable, DstLines, errstr, v[1], et, itr->second.ArgType);
            if(!errstr.IsEmpty()){
                break;
            }
            AnsiString dstline;
            dstline.sprintf("%s(%s)\n", itr->second.Name.c_str(), arg.c_str());
            TDstLine dl;
            dl.TabLevel = tablevel;
            dl.Value = dstline.c_str();
            DstLines.push_back(dl);
        }
        /*else{
            errstr.sprintf("m̎s \"%s\" ܂B", v[0].c_str());
            break;
        }*/
    }

    {
        // function global_script() ̏IB
        TDstLine dl;
        dl.Value = "end\n";
        dl.TabLevel = 0;
        DstLines.push_back(dl);
    }

    fprintf(fp_out, "require('ofutil')\n");
    //fprintf(fp_out, "local FACE_X = 8\n");
    //fprintf(fp_out, "local FACE_Y = 8\n");
    //fprintf(fp_out, "TEXT_X = 128\n");
    //fprintf(fp_out, "TEXT_Y = 8\n");
    fprintf(fp_out, "LINE_SKIP = 18\n");
    fprintf(fp_out, "local LAYER_FACEORCHIP = 1\n");
    fprintf(fp_out, "LAYER_TEXT = 2\n");
    fprintf(fp_out, "\n");
    fprintf(fp_out, "function on_system_start()\n");
    fprintf(fp_out, "    -- XNvg[hɃEBhEB\n");
    fprintf(fp_out, "    window_handle0 = api_window_new()\n");
    fprintf(fp_out, "    window_handle1 = api_window_new()\n");
    fprintf(fp_out, "    arrange_talk_window(window_handle0, 0)\n");
    fprintf(fp_out, "    arrange_talk_window(window_handle1, 1)\n");
    fprintf(fp_out, "end\n");
    fprintf(fp_out, "\n");
    for(std::vector<TDstLine>::const_iterator itr = DstLines.begin(); itr != DstLines.end(); itr++){
        //fprintf(fp_out, "%d", itr->TabLevel);
        PrintTab(fp_out, itr->TabLevel);
        fprintf(fp_out, "%s", itr->Value.c_str());
    }

    if(!errstr.IsEmpty()){
        fprintf(fp_err, "LINE %d : %s\n", srcnum, errstr.c_str());
    }
    fclose(fp_err);

    fclose(fp_out);
    fclose(fp);
    return errstr.IsEmpty();
}
//---------------------------------------------------------------------------
