//---------------------------------------------------------------------------
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/shared_ptr.hpp>
#include "masterselect.h"
#include "worldmap.h"
#include "msgdlg.h"
#include "ofimage.h"
#include "gui.h"
#include "ofini.h"
#include "gamemaster.h"
#include "gcnskinmanager.h"
#include "drawutil.h"

#include "boostformat.h"
#include "logger.h"

#define CHARA_WIDTH    32
#define CHARA_HEIGHT   32
#define CHARA_MARGIN   8
//---------------------------------------------------------------------------
TAreaCharaListPane::TAreaCharaListPane() :
    m_XCount(5),
    m_SelectedArea(-1),
    m_Selected(0),
    m_Browsed(-1),
    m_SelectLocked(false)
{
    m_drawer = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_SMALL);
    g_GameMaster.GetSkinManager()->ApplyDarkPaneSkin(this);
    setSize((CHARA_WIDTH + CHARA_MARGIN)*5, (CHARA_HEIGHT + CHARA_MARGIN)*4);
    addMouseListener(this);
}
//---------------------------------------------------------------------------
// TCharaListPaneƋ@\ԂĂ̂Ōp\B
void TAreaCharaListPane::draw(gcn::Graphics *g)
{
    g->setColor(getBackgroundColor());
    gcn::Rectangle r(0, 0, getWidth(), getHeight());
    g->fillRectangle(r);

    if(m_SelectedArea == -1){
        return;
    }

    const TOpenFarenData* data = g_GameMaster.GetData();
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TAreaInstance& area = game->GetAreaInstance(m_SelectedArea);
    //g_Logger.DebugLog((boost::format("%d") % area.m_charalist.size()).str());
    for(int i=0; i<area.m_charalist.size(); i++){
        int x = (i%m_XCount)*(CHARA_WIDTH + CHARA_MARGIN);
        int y = (i/m_XCount)*(CHARA_HEIGHT + CHARA_MARGIN);
        //g_Logger.DebugLog((boost::format("%d") % area.m_charalist[i].m_type).str());
        //g_Logger.DebugLog((boost::format("%d") % data->GetCharacter(area.m_charalist[i].m_type).m_chipindex).str());
        //g_Logger.DebugLog(data->GetCharacter(area.m_charalist[i].m_type).m_chipfile);
        g_GameMaster.GetImage()->DrawCharaChip(g, x, y, data->GetCharacter(area.m_charalist[i].m_type).m_chipindex);
        if(area.m_charalist[i].m_actioned){
            m_drawer->DrawString(g, x+11, y+14, "End", gcn::Color(0, 0, 0));
            m_drawer->DrawString(g, x+10, y+14, "End", gcn::Color(255, 255, 255));
        }
        if(i == m_Selected){
            g->setColor(gcn::Color(255, 0, 0, 128));
            g_DrawUtil.DrawRectangle(g, gcn::Rectangle(x,   y,   CHARA_WIDTH-1, CHARA_HEIGHT-1));
            g_DrawUtil.DrawRectangle(g, gcn::Rectangle(x+1, y+1, CHARA_WIDTH-3, CHARA_HEIGHT-3));
            g_DrawUtil.DrawRectangle(g, gcn::Rectangle(x+2, y+2, CHARA_WIDTH-5, CHARA_HEIGHT-5));
            g_DrawUtil.DrawRectangle(g, gcn::Rectangle(x+3, y+3, CHARA_WIDTH-7, CHARA_HEIGHT-7));
        }
    }
}
//---------------------------------------------------------------------------
void TAreaCharaListPane::SetSelectedArea(int areaindex)
{
    m_SelectedArea = areaindex;
    m_Selected = -1;
    m_Browsed = -1;
    NotifyStateChenged();
}
//---------------------------------------------------------------------------
void TAreaCharaListPane::ClearState()
{
    m_Selected = -1;
    m_Browsed = -1;
    NotifyStateChenged();
}
//---------------------------------------------------------------------------
// ̌TWorldMapLogicłCxgi_uNbNŏڍו\ĵconsume()͂ȂB
void TAreaCharaListPane::mousePressed(gcn::MouseEvent &mouseEvent)
{
    if(m_SelectedArea == -1){
        return;
    }
    if(m_SelectLocked){
        return;
    }

    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TAreaInstance& area = game->GetAreaInstance(m_SelectedArea);
    if(mouseEvent.getButton() == gcn::MouseEvent::LEFT){
        int cx = (mouseEvent.getX() / (CHARA_WIDTH + CHARA_MARGIN));
        int cy = (mouseEvent.getY() / (CHARA_HEIGHT + CHARA_MARGIN));
        if(cx + cy*m_XCount < area.m_charalist.size()){
            m_Selected = cx + cy*m_XCount;
            NotifyStateChenged();
        }
    }
}
//---------------------------------------------------------------------------
// ̌TWorldMapLogicłCxĝconsume()͂ȂB
void TAreaCharaListPane::mouseMoved(gcn::MouseEvent &mouseEvent)
{
    if(m_SelectedArea == -1){
        return;
    }

    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TAreaInstance& area = game->GetAreaInstance(m_SelectedArea);
    int cx = (mouseEvent.getX() / (CHARA_WIDTH + CHARA_MARGIN));
    int cy = (mouseEvent.getY() / (CHARA_HEIGHT + CHARA_MARGIN));
    if(cx + cy*m_XCount < area.m_charalist.size()){
        m_Browsed = cx + cy*m_XCount;
    }
}
//---------------------------------------------------------------------------
// addActionListenerĂ郊XiɏԕωʒmB
void TAreaCharaListPane::NotifyStateChenged()
{
    for(ActionListenerIterator itr = mActionListeners.begin(); itr != mActionListeners.end(); ++itr){
        (*itr)->action(gcn::ActionEvent(this, ""));
    }
}
//---------------------------------------------------------------------------
TAreaInfoPane::TAreaInfoPane()
    : m_SelectedArea(-1)
{
    m_drawer = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_SMALL);
    g_GameMaster.GetSkinManager()->ApplyDarkPaneSkin(this);
    setForegroundColor(gcn::Color(255,255,255));
    //setBackgroundColor(g_GameMaster.GetSkinManager()->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE));
    setWidth(TGuiContainerList::FONT_SIZE_SMALL * 7);
    setHeight(160);
}
//---------------------------------------------------------------------------
void TAreaInfoPane::SetSelectedArea(int areaindex)
{
    m_SelectedArea = areaindex;
}
//---------------------------------------------------------------------------
void TAreaInfoPane::draw(gcn::Graphics *g)
{
    g->setColor(getBackgroundColor());
    gcn::Rectangle r(0, 0, getWidth(), getHeight());
    g->fillRectangle(r);

    if(m_SelectedArea == -1){
        return;
    }

    const int fontsize = TGuiContainerList::FONT_SIZE_SMALL;
    r.y = fontsize / 2;
    r.width -= fontsize / 4;
    r.height = fontsize * 7;
    gcn::Color c = getForegroundColor();
    DrawFrame(g, r, "GA", m_drawer);
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TAreaInstance& area = game->GetAreaInstance(m_SelectedArea);
    std::string income = (boost::format("@%d") % game->GetAreaIncome(m_SelectedArea)).str();
    std::string city = (boost::format(" X  %3d/%3d") % area.m_city % area.m_citymax).str();
    std::string wall = (boost::format(" %3d/%3d") % area.m_wall % area.m_wallmax).str();
    std::string road = (boost::format("H %3d/%3d") % area.m_road % area.m_roadmax).str();
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize+1, income.c_str(), c);
    if(area.m_road == area.m_roadmax){
        m_drawer->DrawString(g, r.x + 8, r.y + fontsize*2+4, "ʁ@", c);
    }
    else{
        m_drawer->DrawString(g, r.x + 8, r.y + fontsize*2+4, "ʁ@~", c);
    }
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize*3+5, city.c_str(), c);
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize*4+6, wall.c_str(), c);
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize*5+7, road.c_str(), c);
    if(area.m_master != -1){
        // }X^[L
        const TOpenFarenData* data = g_GameMaster.GetData();
        const TBattleCharacter& charadata = data->GetCharacter(data->GetMaster(game->GetMasterInstance(area.m_master).m_type).m_facecharaindex);
        unsigned sz = area.m_charalist.size();
        int sameracecount = 0;
        for(unsigned u=0; u<sz; ++u){
            if(charadata.m_race == data->GetCharacter(area.m_charalist[u].m_type).m_race){
                sameracecount++;
            }
        }
        std::string samerace = (boost::format("푰@%2dl") % sameracecount).str();
        m_drawer->DrawString(g, r.x + 8, r.y + fontsize*8-4, samerace.c_str(), c);
    }
    int power = 0;
    for(unsigned u=0; u<area.m_charalist.size(); ++u){
        power += area.m_charalist[u].m_power;
    }
    power /= 1000;
    std::string powerstr = (boost::format("́@%5d") % power).str();
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize*9-3, powerstr.c_str(), c);
}
//---------------------------------------------------------------------------
TAreaOpeListModel::TAreaOpeListModel()
{
}
//---------------------------------------------------------------------------
std::string TAreaOpeListModel::getElementAt(int i)
{
    //static const char *elm[AREAOPE_MAX] = {"P", "lޑ{", "X", "ǌ", "H"};
    //if(i < 0 || i >= AREAOPE_MAX){
    //    return "";
    //}
    //return elm[i];
    return TOpenFarenGame::AreaOperationName[i];
}
//---------------------------------------------------------------------------
int TAreaOpeListModel::getNumberOfElements()
{
    return AREAOPE_MAX;
}
//---------------------------------------------------------------------------
TAreaOpePane::TAreaOpePane()
    : m_Shown(false)
{
    m_drawer = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_SMALL);
    add(&m_ButtonMove);
    add(&m_ButtonEmploy);
    add(&m_ButtonDismiss);
    add(&m_AreaOpeDropDown);
    m_AreaOpeDropDown.setListModel(&m_AreaOpeListModel);
}
//---------------------------------------------------------------------------
void TAreaOpePane::draw(gcn::Graphics *g)
{
    static const gcn::Color white(255, 255, 255);

    gcn::Container::draw(g);
    g->setColor(white);
    g->drawLine(0, 0, getWidth()-1, 0);
    if(m_ShowControl){
        m_drawer->DrawString(g, 4, 4, "ҋ@̍s", white);
    }
}
//---------------------------------------------------------------------------
void TAreaOpePane::Show(bool showcontrol)
{
    if(!m_Shown){
        int buttony = 8;
        m_ButtonMove.setCaption("ړ");
        m_ButtonMove.setSize(56, 24);
        m_ButtonMove.setPosition(224, buttony);
        g_GameMaster.GetSkinManager()->ApplyButtonEnabled(&m_ButtonMove);

        buttony += 28;
        m_ButtonEmploy.setCaption("ٗp");
        m_ButtonEmploy.setSize(56, 24);
        m_ButtonEmploy.setPosition(224, buttony);

        buttony += 28;
        m_ButtonDismiss.setCaption("");
        m_ButtonDismiss.setSize(56, 24);
        m_ButtonDismiss.setPosition(224, buttony);

        m_AreaOpeDropDown.setPosition(4, 20);

        //setBackgroundColor(g_Ini.GetPaneColor());
        //setBaseColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_DARKPANE));
        g_GameMaster.GetSkinManager()->ApplyDarkPaneSkin(this);
        setSize(284, 128);
        m_Shown = true;
    }

    m_ButtonMove.setVisible(showcontrol);
    m_ButtonEmploy.setVisible(showcontrol);
    m_ButtonDismiss.setVisible(showcontrol);
    m_AreaOpeDropDown.setVisible(showcontrol);
    m_ShowControl = showcontrol;
}
//---------------------------------------------------------------------------
void TAreaOpePane::SetSelectedArea(int areaindex)
{
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TAreaInstance& area = game->GetAreaInstance(areaindex);
    m_AreaOpeDropDown.setSelected(area.m_operation);
}
//---------------------------------------------------------------------------
TCharaListWindow::TCharaListWindow()
//    : m_Shown(false)
{
    setTitleBarHeight(20);
    setOpaque(false);

    m_CharaListPane.setPosition(0, 0);
    //m_CharaListPane.setBackgroundColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_DARKPANE));
    m_CharaListPane.addActionListener(this);
    add(&m_CharaListPane);

    m_AreaInfoPane.setPosition(m_CharaListPane.getWidth(), 0);
    //m_AreaInfoPane.setBackgroundColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_DARKPANE));
    add(&m_AreaInfoPane);

    m_AreaOpePane.setPosition(0, m_AreaInfoPane.getHeight());
    add(&m_AreaOpePane);

    //setBaseColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_WINDOW));
    g_GameMaster.GetSkinManager()->ApplyWindowSkin(this);
}
//---------------------------------------------------------------------------
void TCharaListWindow::Show(bool showbutton)
{
    m_AreaOpePane.Show(showbutton);
    resizeToContent();
    setVisible(true);
}
//---------------------------------------------------------------------------
void TCharaListWindow::SetSelectedArea(int areaindex)
{
    m_CharaListPane.SetSelectedArea(areaindex);
    m_AreaInfoPane.SetSelectedArea(areaindex);
    m_AreaOpePane.SetSelectedArea(areaindex);
}
//---------------------------------------------------------------------------
void TCharaListWindow::draw(gcn::Graphics *g)
{
    TGcnWindow::draw(g);
    if(m_CharaListPane.GetSelectedArea() != -1){
        g->setColor(gcn::Color(255,255,255));
        const TOpenFarenGame* game = g_GameMaster.GetGame();
        const TAreaInstance& area = game->GetAreaInstance(m_CharaListPane.GetSelectedArea());
        g->drawText(area.m_name.c_str(), 4, 2);
        g->drawText(area.m_name.c_str(), 5, 2);
    }
}
//---------------------------------------------------------------------------
void TCharaListWindow::action(const gcn::ActionEvent& actionEvent)
{
    if(actionEvent.getSource() == &m_CharaListPane){
        int ci = m_CharaListPane.GetSelected();
        bool enable = false;
        if(ci != -1){
            const TOpenFarenGame* game = g_GameMaster.GetGame();
            const TAreaInstance& area = game->GetAreaInstance(m_CharaListPane.GetSelectedArea());
            enable = !area.m_charalist[ci].m_actioned;
        }
        if(enable){
            g_GameMaster.GetSkinManager()->ApplyButtonEnabled(&m_AreaOpePane.GetButtonEmploy());
            g_GameMaster.GetSkinManager()->ApplyButtonEnabled(&m_AreaOpePane.GetButtonDismiss());
        }
        else{
            g_GameMaster.GetSkinManager()->ApplyButtonDisabled(&m_AreaOpePane.GetButtonEmploy());
            g_GameMaster.GetSkinManager()->ApplyButtonDisabled(&m_AreaOpePane.GetButtonDismiss());
        }
    }
}
//---------------------------------------------------------------------------
TMasterInfoPane::TMasterInfoPane()
    : m_MasterInstance(-1), m_Tick(0)
{
    m_drawer = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_SMALL);
    m_drawerbig = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_NORMAL);
    g_GameMaster.GetSkinManager()->ApplyDarkPaneSkin(this);
    setForegroundColor(gcn::Color(255,255,255));
    //setBackgroundColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_DARKPANE));
    setSize(200, 128);
}
//---------------------------------------------------------------------------
void TMasterInfoPane::draw(gcn::Graphics *g)
{
    const int fps = g_Ini.GetIntValue(TOpenFarenIni::WINDOW_FPS);

    g->setColor(getBackgroundColor());
    gcn::Rectangle r(0, 0, getWidth(), getHeight());
    g->fillRectangle(r);

    if(m_MasterInstance == -1){
        return;
    }

    const TOpenFarenData* data = g_GameMaster.GetData();
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TMasterInstance& master = game->GetMasterInstance(m_MasterInstance);
    const TMaster& masterdata = data->GetMaster(master.m_type);
    const TBattleCharacter& charadata = data->GetCharacter(masterdata.m_facecharaindex);
    gcn::Color c = getForegroundColor();
    m_drawerbig->DrawString(g, 0, 0, charadata.m_name.c_str(), c);
    m_drawerbig->DrawString(g, 1, 0, charadata.m_name.c_str(), c);
    const int fontsize = TGuiContainerList::FONT_SIZE_NORMAL;
    int infoy = TGuiContainerList::FONT_SIZE_NORMAL + 8;
    g_GameMaster.GetImage()->DrawCharaFaceOrChip(g, 0, getHeight() - 96, charadata.m_faceindex, charadata.m_chipindex);
    if(masterdata.m_flagindex != -1){
        g_GameMaster.GetImage()->DrawFlag(g, 136, 0, masterdata.m_flagindex, m_Tick/(30*fps/60));
    }

    const int rx = 104;
    m_drawer->DrawString(g, rx,     infoy, "", c);
    m_drawer->DrawString(g, rx + 1, infoy, "", c);
    std::string income = (boost::format("%3d Ley") % master.m_income).str();
    m_drawer->DrawString(g, rx + fontsize, infoy + fontsize + 2, income.c_str(), c);

    m_drawer->DrawString(g, rx,     infoy + fontsize*2 + 4, "lޔ", c);
    m_drawer->DrawString(g, rx + 1, infoy + fontsize*2 + 4, "lޔ", c);
    std::string outgo = (boost::format("%3d Ley") % master.m_outgo).str();
    m_drawer->DrawString(g, rx + fontsize, infoy + fontsize*3 + 6, outgo.c_str(), c);

    std::string money = (boost::format("%3d Ley") % master.m_money).str();
    m_drawer->DrawString(g, rx,     infoy + fontsize*4 + 8, "R", c);
    m_drawer->DrawString(g, rx + 1, infoy + fontsize*4 + 8, "R", c);
    m_drawer->DrawString(g, rx + fontsize, infoy + fontsize*5 + 10, money.c_str(), c);
}
//---------------------------------------------------------------------------
TMasterOpePane::TMasterOpePane()
{
    const TGcnSkinManager *skinmng = g_GameMaster.GetSkinManager();

    m_ButtonAreaList.setCaption("GA");
    m_ButtonAreaList.setSize(88, 24);
    m_ButtonAreaList.setPosition(8, 8);
    skinmng->ApplyButtonEnabled(&m_ButtonAreaList);
    add(&m_ButtonAreaList);

    m_ButtonMasterList.setCaption("͏");
    m_ButtonMasterList.setSize(88, 24);
    m_ButtonMasterList.setPosition(8, 36);
    skinmng->ApplyButtonEnabled(&m_ButtonMasterList);
    add(&m_ButtonMasterList);

    m_ButtonHeroList.setCaption("lޏ");
    m_ButtonHeroList.setSize(88, 24);
    m_ButtonHeroList.setPosition(8, 64);
    skinmng->ApplyButtonEnabled(&m_ButtonHeroList);
    add(&m_ButtonHeroList);

    m_ButtonPowerMap.setCaption("͒n}");
    m_ButtonPowerMap.setSize(88, 24);
    m_ButtonPowerMap.setPosition(8, 92);
    skinmng->ApplyButtonEnabled(&m_ButtonPowerMap);
    add(&m_ButtonPowerMap);

    m_ButtonAlliance.setCaption("");
    m_ButtonAlliance.setSize(88, 24);
    m_ButtonAlliance.setPosition(104, 8);
    skinmng->ApplyButtonEnabled(&m_ButtonAlliance);
    add(&m_ButtonAlliance);

    m_ButtonWar.setCaption("푈");
    m_ButtonWar.setSize(88, 24);
    m_ButtonWar.setPosition(104, 36);
    skinmng->ApplyButtonEnabled(&m_ButtonWar);
    add(&m_ButtonWar);

    m_ButtonTurnEnd.setCaption("^[I");
    m_ButtonTurnEnd.setSize(88, 24);
    m_ButtonTurnEnd.setPosition(104, 64);
    skinmng->ApplyButtonEnabled(&m_ButtonTurnEnd);
    add(&m_ButtonTurnEnd);

    //setBaseColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_DARKPANE));
    skinmng->ApplyDarkPaneSkin(this);
    setSize(200, 120);
}
//---------------------------------------------------------------------------
TMasterWindow::TMasterWindow()
    : m_Shown(false)
{
    m_MasterInfoPane.setPosition(0, 0);
    add(&m_MasterInfoPane);

    m_MasterOpePane.setPosition(0, m_MasterInfoPane.getHeight());
    add(&m_MasterOpePane);

    setOpaque(false);
    setTitleBarHeight(20);
}
//---------------------------------------------------------------------------
bool TMasterWindow::Show()
{
    if(!m_Shown){
//        m_MasterInfoPane.setPosition(0, 0);
//        add(&m_MasterInfoPane);
//
//        int buttony = m_MasterInfoPane.getHeight() + 8;
//        m_ButtonAlliance.setCaption("");
//        m_ButtonAlliance.setWidth(56);
//        m_ButtonAlliance.setHeight(24);
//        m_ButtonAlliance.setPosition(8, buttony);
//        g_Ini.ApplyButtonEnabledColor(m_ButtonAlliance);
//        add(&m_ButtonAlliance);
//
//        m_ButtonWar.setCaption("푈");
//        m_ButtonWar.setWidth(56);
//        m_ButtonWar.setHeight(24);
//        m_ButtonWar.setPosition(72, buttony);
//        g_Ini.ApplyButtonEnabledColor(m_ButtonWar);
//        add(&m_ButtonWar);
//
//        m_ButtonTurnEnd.setCaption("I");
//        m_ButtonTurnEnd.setWidth(56);
//        m_ButtonTurnEnd.setHeight(24);
//        m_ButtonTurnEnd.setPosition(136, buttony);
//        g_Ini.ApplyButtonEnabledColor(m_ButtonTurnEnd);
//        add(&m_ButtonTurnEnd);

	    //setBaseColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_WINDOW));
        g_GameMaster.GetSkinManager()->ApplyWindowSkin(this);
        m_Shown = true;
    }

    resizeToContent();
    //setWidth(m_ButtonTurnEnd.getX() + m_ButtonTurnEnd.getWidth() + 8);
    //setHeight(getTitleBarHeight() + m_ButtonTurnEnd.getY() + m_ButtonTurnEnd.getHeight() + 16);
    setVisible(true);
    return true;
}
//---------------------------------------------------------------------------
void TMasterWindow::draw(gcn::Graphics *g)
{
    TGcnWindow::draw(g);
    if(m_MasterInfoPane.GetMasterInstance() != -1){
        g->setColor(gcn::Color(255,255,255));
        const TOpenFarenData* data = g_GameMaster.GetData();
        const TOpenFarenGame* game = g_GameMaster.GetGame();
        const TMaster& masterdata = data->GetMaster(game->GetMasterInstance(m_MasterInfoPane.GetMasterInstance()).m_type);
        g->drawText(masterdata.m_name.c_str(), 4, 2);
        g->drawText(masterdata.m_name.c_str(), 5, 2);
    }
}
//---------------------------------------------------------------------------
std::string TEmployListModel::getElementAt(int i)
{
    const TBattleCharacter& charadata = g_GameMaster.GetData()->GetCharacter(m_TypeList[i]);
    std::string str = (boost::format("%s@ip%dLeyj") % charadata.m_name.c_str() % charadata.m_employcost).str();
    if(charadata.m_race == m_Race){
        str = "" + str;
    }
    return str;
}
//---------------------------------------------------------------------------
int TEmployListModel::getNumberOfElements()
{
    return m_TypeList.size();
}
//---------------------------------------------------------------------------
int TEmployListModel::GetEmployAt(int i) const
{
    return m_TypeList[i];
}
//---------------------------------------------------------------------------
TEmployWindow::TEmployWindow()
{
    setTitleBarHeight(20);
    add(&m_EmployListBox);
    add(&m_ButtonOk);
    add(&m_ButtonCancel);
}
//---------------------------------------------------------------------------
bool TEmployWindow::Show(const std::vector<int>& typelist, int race, int budget)
{
    const TGcnSkinManager *skinmng = g_GameMaster.GetSkinManager();

    m_EmployListModel.SetEmployList(typelist);
    m_EmployListModel.SetRace(race);
    m_Budget = budget;

    m_EmployListBox.setPosition(0, 0);
    m_EmployListBox.setWidth(320);
    m_EmployListBox.setHeight(160);
    m_EmployListBox.setListModel(&m_EmployListModel);
    m_EmployListBox.addActionListener(this);
    //g_Ini.ApplyListBoxColor(m_EmployListBox);
    skinmng->ApplyListBoxSkin(&m_EmployListBox);
    //add(&m_EmployListBox);

    m_ButtonOk.setCaption("ق");
    //m_ButtonOk.setCaption("߂");
    m_ButtonOk.setWidth(64);
    m_ButtonOk.setHeight(24);
    m_ButtonOk.setPosition(328, 40);
    skinmng->ApplyButtonDisabled(&m_ButtonOk);
    //add(&m_ButtonOk);

    m_ButtonCancel.setCaption("߂");
    m_ButtonCancel.setWidth(64);
    m_ButtonCancel.setHeight(24);
    m_ButtonCancel.setPosition(328, 80);
    skinmng->ApplyButtonEnabled(&m_ButtonCancel);
    //add(&m_ButtonCancel);

    //setBaseColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_WINDOW));
    skinmng->ApplyWindowSkin(this);
    resizeToContent();
    setVisible(true);
    return true;
}
//---------------------------------------------------------------------------
int TEmployWindow::GetSelectedEmployType()
{
    int i = m_EmployListBox.getSelected();  // ̊֐constłȂB
    if(i < 0){
        return -1;
    }
    return m_EmployListModel.GetEmployAt(i);
}
//---------------------------------------------------------------------------
void TEmployWindow::draw(gcn::Graphics *g)
{
    TGcnWindow::draw(g);
    g->setColor(gcn::Color(255,255,255));
    g->drawText("ٗp", 4, 2);
    g->drawText("ٗp", 5, 2);
}
//---------------------------------------------------------------------------
void TEmployWindow::action(const gcn::ActionEvent& actionEvent)
{
    if(actionEvent.getSource() == &m_EmployListBox){
        int i = m_EmployListBox.getSelected();
        if(i < 0){
            return;
        }
        const TBattleCharacter& charadata = g_GameMaster.GetData()->GetCharacter(m_EmployListModel.GetEmployAt(i));
        if(charadata.m_employcost <= m_Budget){
            g_GameMaster.GetSkinManager()->ApplyButtonEnabled(&m_ButtonOk);
        }
        else{
            g_GameMaster.GetSkinManager()->ApplyButtonDisabled(&m_ButtonOk);
        }
    }
}
//---------------------------------------------------------------------------
TWarCharaListPane::TWarCharaListPane() :
    m_XCount(5),
    m_Browsed(false),
    m_MouseHandler(NULL)
{
    m_drawer = g_GuiContainerList.GetDrawer(m_FontSize);
    setWidth((m_CharaWidth + m_CharaMargin)*m_XCount);
    addMouseListener(this);
}
//---------------------------------------------------------------------------
void TWarCharaListPane::draw(gcn::Graphics *g)
{
    g->setColor(getBackgroundColor());
    gcn::Rectangle r(0, 0, getWidth(), getHeight());
    g->fillRectangle(r);

    // `揈͂܂Yɂ܂Ƃ܂ȂB
    // UpdateHeight()GetCharaIndexAt()ɂl̃[vƂD܂ȂԁB
    gcn::Color c(255,255,255);
    if(!m_WarCharaList.empty()){
        const TOpenFarenData* data = g_GameMaster.GetData();
        const TOpenFarenGame* game = g_GameMaster.GetGame();
        int xcount = 0;
        int x = 0;
        int y = m_AreaLabelHeight;
        int lastarea = m_WarCharaList[0].first;
        // ŏ̃L̃GA`B
        const TAreaInstance& area1 = game->GetAreaInstance(lastarea);
        m_drawer->DrawString(g, 0, 2, area1.m_name.c_str(), c);
        m_drawer->DrawString(g, 1, 2, area1.m_name.c_str(), c);
        for(unsigned u=0; u<m_WarCharaList.size(); ++u){
            int ai = m_WarCharaList[u].first;
            const TAreaInstance& area = game->GetAreaInstance(ai);
            if(lastarea != ai){
                // L݃GAς̓GA`ĉsHB
                xcount = 0;
                x = 0;
                y += m_CharaHeight + m_CharaMargin;
                m_drawer->DrawString(g, 0, y+2, area.m_name.c_str(), c);
                m_drawer->DrawString(g, 1, y+2, area.m_name.c_str(), c);
                y += m_AreaLabelHeight;
                lastarea = ai;
            }
            // L`bv̕`B
            const TCharacterInstance& chara = area.m_charalist[m_WarCharaList[u].second];
            g_GameMaster.GetImage()->DrawCharaChip(g, x, y, data->GetCharacter(chara.m_type).m_chipindex);
            // ̃LGAȂWXVB
            if(u + 1 < m_WarCharaList.size()){
                if(m_WarCharaList[u + 1].first == lastarea){
                    if(xcount < m_XCount - 1){
                        x += m_CharaWidth + m_CharaMargin;
                        xcount++;
                    }
                    else{
                        x = 0;
                        xcount = 0;
                        y += m_CharaHeight + m_CharaMargin;
                    }
                }
            }
        }
    }
}
//---------------------------------------------------------------------------
void TWarCharaListPane::UpdateHeight()
{
    // KvȃTCY߂Bdraw()Ɠ悤ȃ[v̂͂ǂȂ̂B
    if(!m_WarCharaList.empty()){
        int xcount = 0;
        int y = m_AreaLabelHeight;
        int lastarea = m_WarCharaList[0].first;
        int height = 0;
        for(unsigned u=0; u<m_WarCharaList.size(); ++u){
            int ai = m_WarCharaList[u].first;
            if(lastarea != ai){
                xcount = 0;
                y += m_CharaHeight + m_CharaMargin + m_AreaLabelHeight;
                lastarea = ai;
            }
            height = y + m_CharaHeight + m_CharaMargin;
            // ̃LGAȂWXVB
            if(u + 1 < m_WarCharaList.size()){
                if(m_WarCharaList[u + 1].first == lastarea){
                    if(xcount < m_XCount - 1){
                        xcount++;
                    }
                    else{
                        xcount = 0;
                        y += m_CharaHeight + m_CharaMargin;
                    }
                }
            }
        }
        setHeight(height);
    }
    else{
        setHeight(m_CharaHeight + m_CharaMargin + m_AreaLabelHeight);
    }
}
//---------------------------------------------------------------------------
struct TCharaIndexLess
{
    bool operator()(const std::pair<int, int>& lhs, const std::pair<int, int>& rhs){
        if(lhs.first != rhs.first){
            return lhs.first < rhs.first;
        }
        return lhs.second < rhs.second;
    }
};

void TWarCharaListPane::AddChara(int areaindex, int charaindex)
{
    m_WarCharaList.push_back(std::pair<int, int>(areaindex, charaindex));
    std::sort(m_WarCharaList.begin(), m_WarCharaList.end(), TCharaIndexLess());
    UpdateHeight();
}
//---------------------------------------------------------------------------
void TWarCharaListPane::RemoveChara(int areaindex, int charaindex)
{
    m_WarCharaList.erase(std::remove(m_WarCharaList.begin(), m_WarCharaList.end(), std::pair<int, int>(areaindex, charaindex)), m_WarCharaList.end());
    UpdateHeight();
}
//---------------------------------------------------------------------------
void TWarCharaListPane::ClearChara()
{
    m_WarCharaList.clear();
    UpdateHeight();
}
//---------------------------------------------------------------------------
bool TWarCharaListPane::GetCharaIndexAt(int px, int py, int& areaindex, int& charaindex) const
{
    // draw()̃[vRs[ďR[hŋ߂B
    if(m_WarCharaList.empty()){
        return false;
    }
    int xcount = 0;
    int x = 0;
    int y = m_AreaLabelHeight;
    int lastarea = m_WarCharaList[0].first;
    for(unsigned u=0; u<m_WarCharaList.size(); ++u){
        int ai = m_WarCharaList[u].first;
        if(lastarea != ai){
            x = 0;
            xcount = 0;
            y += m_CharaHeight + m_CharaMargin + m_AreaLabelHeight;
            lastarea = ai;
        }
        gcn::Rectangle r(x, y, m_CharaWidth, m_CharaHeight);
        if(r.isPointInRect(px, py)){
            areaindex = ai;
            charaindex = m_WarCharaList[u].second;
            return true;
        }
        // ̃LGAȂWXVB
        if(u + 1 < m_WarCharaList.size()){
            if(m_WarCharaList[u + 1].first == lastarea){
                if(xcount < m_XCount - 1){
                    x += m_CharaWidth + m_CharaMargin;
                    xcount++;
                }
                else{
                    x = 0;
                    xcount = 0;
                    y += m_CharaHeight + m_CharaMargin;
                }
            }
        }
    }
    return false;
}
//---------------------------------------------------------------------------
const TCharacterInstance* TWarCharaListPane::GetBrowsedCharacter() const
{
    if(m_Browsed){
        return &m_BrowsedCharacter;
    }
    else{
        return NULL;
    }
}
//---------------------------------------------------------------------------
void TWarCharaListPane::mousePressed(gcn::MouseEvent &mouseEvent)
{
    if(m_MouseHandler != NULL){
        m_MouseHandler->HandleMousePressed(mouseEvent);
    }
}
//---------------------------------------------------------------------------
void TWarCharaListPane::mouseMoved(gcn::MouseEvent &mouseEvent)
{
    int ai, ci;
    if(GetCharaIndexAt(mouseEvent.getX(), mouseEvent.getY(), ai, ci)){
        m_Browsed = true;
        const TOpenFarenGame* game = g_GameMaster.GetGame();
        m_BrowsedCharacter = game->GetAreaInstance(ai).m_charalist[ci];
    }
    if(m_MouseHandler != NULL){
        m_MouseHandler->HandleMouseMoved(mouseEvent);
    }
}
//---------------------------------------------------------------------------
TWarCharaWindow::TWarCharaWindow() :
    m_LeftAreaIndex(-1),
    m_RightAreaIndex(-1),
    m_Shown(false),
    m_Browsed(false)
{
    setTitleBarHeight(20);
}
//---------------------------------------------------------------------------
void TWarCharaWindow::SetLeftArea(int index)
{
    m_LeftAreaIndex = index;
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TAreaInstance& area = game->GetAreaInstance(m_LeftAreaIndex);
    m_LeftPane.SetCharaList(&area.m_charalist, &m_LeftToRightFlag[m_LeftAreaIndex]);
}
//---------------------------------------------------------------------------
void TWarCharaWindow::Show(int targetarea)
{
    m_LeftAreaIndex = -1;
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    // SGÃL̕ړtOmۂB蔲B
    m_LeftToRightFlag.clear();
    m_LeftToRightFlag.resize(game->GetAreaInstanceCount());
    for(unsigned u=0; u<m_LeftToRightFlag.size(); u++){
        m_LeftToRightFlag[u].resize(game->GetAreaInstance(u).m_charalist.size());
    }
    m_LeftPane.SetCharaList(NULL, NULL);

    m_RightPane.ClearChara();
    m_RightAreaIndex = targetarea;

    if(!m_Shown){
        const TGcnSkinManager *skinmng = g_GameMaster.GetSkinManager();

        m_LeftPane.setPosition(LR_MARGIN, 20);
        skinmng->ApplyDarkPaneSkin(&m_LeftPane);
        m_LeftPane.SetMouseHandler(this);
        add(&m_LeftPane);

        m_RightPane.setPosition(LR_MARGIN + m_LeftPane.getWidth() + 16, m_LeftPane.getY());
        skinmng->ApplyDarkPaneSkin(&m_RightPane);
        m_RightPane.SetMouseHandler(this);
        add(&m_RightPane);

        int buttony = m_LeftPane.getY() + m_LeftPane.getHeight() + 4;
        m_ButtonAllRight.setCaption("SĉE");
        m_ButtonAllRight.setSize(80, 24);
        m_ButtonAllRight.setPosition(LR_MARGIN, buttony);
        m_ButtonAllRight.addActionListener(this);
        skinmng->ApplyButtonEnabled(&m_ButtonAllRight);
        add(&m_ButtonAllRight);

        m_ButtonAllLeft.setCaption("Sč");
        m_ButtonAllLeft.setSize(80, 24);
        m_ButtonAllLeft.setPosition(m_RightPane.getX() + m_RightPane.getWidth() - m_ButtonAllLeft.getWidth() - 4, buttony);
        m_ButtonAllLeft.addActionListener(this);
        skinmng->ApplyButtonEnabled(&m_ButtonAllLeft);
        add(&m_ButtonAllLeft);

        m_ButtonOk.setCaption("m");
        m_ButtonOk.setSize(80, 24);
        m_ButtonOk.setPosition(m_LeftPane.getX() + m_LeftPane.getWidth() - m_ButtonOk.getWidth() - 4, buttony);
        skinmng->ApplyButtonDisabled(&m_ButtonOk);
        add(&m_ButtonOk);

        m_ButtonCancel.setCaption("~");
        m_ButtonCancel.setSize(80, 24);
        m_ButtonCancel.setPosition(m_RightPane.getX() + 4, buttony);
        skinmng->ApplyButtonEnabled(&m_ButtonCancel);
        add(&m_ButtonCancel);

        skinmng->ApplyWindowSkin(this);
        setBaseColor(skinmng->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_GRAYPANE));
        //resizeToContent();
        UpdateSize();
        m_Shown = true;
    }
    m_Browsed = false;
    setVisible(true);
}
//---------------------------------------------------------------------------
void TWarCharaWindow::Hide()
{
    m_Browsed = false;
    m_LeftPane.SetCharaList(NULL, NULL);
    m_RightPane.ClearChara();
    setVisible(false);
}
//---------------------------------------------------------------------------
void TWarCharaWindow::GetLeftToRight(std::vector<std::pair<int, int> >& indice) const
{
    indice.clear();
    for(unsigned u=0; u<m_LeftToRightFlag.size(); u++){
        for(unsigned chr = 0; chr < m_LeftToRightFlag[u].size(); ++chr){
            if(m_LeftToRightFlag[u][chr]){
                indice.push_back(std::pair<int, int>(u, chr));
            }
        }
    }
}
//---------------------------------------------------------------------------
void TWarCharaWindow::UpdateButtonEnabled()
{
    const TGcnSkinManager *skinmng = g_GameMaster.GetSkinManager();
    if(m_RightPane.GetCharaIndexCount()){
        skinmng->ApplyButtonEnabled(&m_ButtonOk);
    }
    else{
        skinmng->ApplyButtonDisabled(&m_ButtonOk);
    }
}
//---------------------------------------------------------------------------
void TWarCharaWindow::draw(gcn::Graphics *g)
{
    TGcnWindow::draw(g);
    gcn::Rectangle r = getChildrenArea();
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    g->setColor(gcn::Color(255,255,255));
    if(m_LeftAreaIndex != -1){
        const TAreaInstance& larea = game->GetAreaInstance(m_LeftAreaIndex);
        g->drawText(larea.m_name.c_str(), r.x + m_LeftPane.getX(),     r.y + 2);
        g->drawText(larea.m_name.c_str(), r.x + m_LeftPane.getX() + 1, r.y + 2);
    }
    if(m_RightAreaIndex != -1){
        const TAreaInstance& rarea = game->GetAreaInstance(m_RightAreaIndex);
        std::string target = (boost::format("URi%sցj") % rarea.m_name.c_str()).str();
        g->drawText(target.c_str(), r.x + m_RightPane.getX(),     r.y + 2);
        g->drawText(target.c_str(), r.x + m_RightPane.getX() + 1, r.y + 2);
    }
    g->drawText("푈", 4, 2);
    g->drawText("푈", 5, 2);

    int rest = TOpenFarenGame::AREACHARA_MAX - m_RightPane.GetCharaIndexCount();
    std::string reststr = (boost::format("o\c %2d") % rest).str();
    DrawBoldText(g, reststr.c_str(), m_RightPane.getX(), getTitleBarHeight() + m_ButtonCancel.getY() - 20);
}
//---------------------------------------------------------------------------
void TWarCharaWindow::HandleMousePressed(gcn::MouseEvent &mouseEvent)
{
    if(mouseEvent.getSource() == &m_LeftPane){
        // ̃LXgŃ}EXƂ̏
        mouseEvent.consume();
        if(m_LeftAreaIndex == -1){
            return;
        }
        const TOpenFarenGame* game = g_GameMaster.GetGame();
        const TAreaInstance& area = game->GetAreaInstance(m_LeftAreaIndex);
        int index = m_LeftPane.GetCharaIndexAt(mouseEvent.getX(), mouseEvent.getY());
        if(index >= area.m_charalist.size()){
            return;
        }
        if(area.m_charalist[index].m_actioned){
            return;
        }
        // ɑIς
        if(m_LeftToRightFlag[m_LeftAreaIndex][index]){
            return;
        }
        if(m_RightPane.GetCharaIndexCount() >= TOpenFarenGame::AREACHARA_MAX){
            return;
        }
        m_LeftToRightFlag[m_LeftAreaIndex][index] = 1;
        m_RightPane.AddChara(m_LeftAreaIndex, index);
        UpdateSize();
        UpdateButtonEnabled();
    }
    else if(mouseEvent.getSource() == &m_RightPane){
        // ẼLXgŃ}EXƂ̏
        mouseEvent.consume();
        int ai, ci;
        if(!m_RightPane.GetCharaIndexAt(mouseEvent.getX(), mouseEvent.getY(), ai, ci)){
            return;
        }
        m_LeftToRightFlag[ai][ci] = 0;
        m_RightPane.RemoveChara(ai, ci);
        UpdateSize();
        UpdateButtonEnabled();
    }
}
//---------------------------------------------------------------------------
void TWarCharaWindow::HandleMouseMoved(gcn::MouseEvent &mouseEvent)
{
    // }EXɂL̃CX^XRs[B̃R[h̏悤͂Ȃ̂ƂCB
    if(mouseEvent.getSource() == &m_LeftPane){
        if(m_LeftAreaIndex != -1){
            int i = m_LeftPane.GetBrowsed();
            if(i != -1){
                const TOpenFarenGame* game = g_GameMaster.GetGame();
                const TAreaInstance& area = game->GetAreaInstance(m_LeftAreaIndex);
                m_BrowsedCharacter = area.m_charalist[i];
                m_Browsed = true;
            }
        }
    }
    else if(mouseEvent.getSource() == &m_RightPane){
        int ai, ci;
        if(!m_RightPane.GetCharaIndexAt(mouseEvent.getX(), mouseEvent.getY(), ai, ci)){
            return;
        }
        const TOpenFarenGame* game = g_GameMaster.GetGame();
        const TAreaInstance& area = game->GetAreaInstance(ai);
        m_BrowsedCharacter = area.m_charalist[ci];
        m_Browsed = true;
    }
}
//---------------------------------------------------------------------------
void TWarCharaWindow::action(const gcn::ActionEvent& actionEvent)
{
    if(actionEvent.getSource() == &m_ButtonAllRight){
        if(m_LeftAreaIndex == -1){
            return;
        }
        // SĂ̍LoEɈړ
        const TOpenFarenGame* game = g_GameMaster.GetGame();
        const TAreaInstance& area = game->GetAreaInstance(m_LeftAreaIndex);
        for(unsigned u = 0; u<area.m_charalist.size(); u++){
            if(m_RightPane.GetCharaIndexCount() >= TOpenFarenGame::AREACHARA_MAX){
                break;
            }
            if(!m_LeftToRightFlag[m_LeftAreaIndex][u] && !area.m_charalist[u].m_actioned){
                m_LeftToRightFlag[m_LeftAreaIndex][u] = 1;
                m_RightPane.AddChara(m_LeftAreaIndex, u);
            }
        }
        UpdateSize();
        UpdateButtonEnabled();
    }
    else if(actionEvent.getSource() == &m_ButtonAllLeft){
        // ËړSLZ
        for(unsigned u=0; u<m_LeftToRightFlag.size(); u++){ // f[^\Ɏ𔲂ƂƂɂȂĂ킯B
            m_LeftToRightFlag[u].assign(m_LeftToRightFlag[u].size(), 0);
        }
        m_RightPane.ClearChara();
        UpdateSize();
        UpdateButtonEnabled();
    }
}
//---------------------------------------------------------------------------
void TWarCharaWindow::UpdateSize()
{
    int ly = m_LeftPane.getY() + m_LeftPane.getHeight();
    int ry = m_RightPane.getY() + m_RightPane.getHeight();
    int y = ly > ry ? ly : ry;
    y += 24;
    m_ButtonAllRight.setY(y);
    m_ButtonAllLeft.setY(y);
    m_ButtonOk.setY(y);
    m_ButtonCancel.setY(y);
    //resizeToContent();
    setSize(getPadding() * 2 + m_RightPane.getX() + m_RightPane.getWidth() + LR_MARGIN * 2, getTitleBarHeight() + y + m_ButtonCancel.getHeight() + 8);
}
//---------------------------------------------------------------------------
TEmployHeroPane::TEmployHeroPane() :
    m_Area(-1),
    m_Hero(-1)
{
    m_drawer = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_SMALL);
    m_drawerbig = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_NORMAL);

    m_ButtonDetail.setCaption("ڍ");
    m_ButtonDetail.setSize(80, 24);
    m_ButtonDetail.setPosition(112, 96);
    add(&m_ButtonDetail);

    m_ButtonEmploy.setCaption("ق");
    m_ButtonEmploy.setSize(80, 24);
    m_ButtonEmploy.setPosition(4, 132);
    add(&m_ButtonEmploy);

    m_ButtonCancel.setCaption("قȂ");
    m_ButtonCancel.setSize(80, 24);
    m_ButtonCancel.setPosition(112, 132);
    add(&m_ButtonCancel);

    //setBaseColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_DARKPANE));
    g_GameMaster.GetSkinManager()->ApplyDarkPaneSkin(this);
    setSize(196, 160);
}
//---------------------------------------------------------------------------
void TEmployHeroPane::draw(gcn::Graphics *g)
{
    static const gcn::Color white(255, 255, 255);

    gcn::Container::draw(g);
    if(m_Area == -1){
        return;
    }
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    std::string line1 = (boost::format("%s") % (game->GetAreaInstance(m_Area).m_name)).str();
    m_drawer->DrawString(g, 4, 4, line1.c_str(), white);
    m_drawer->DrawString(g, 4, 18, "lނ𔭌", white);

    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(m_Hero);
    m_drawerbig->DrawString(g, 16, 40, charadata.m_name.c_str(), white);
    m_drawerbig->DrawString(g, 17, 40, charadata.m_name.c_str(), white);

    const TOpenFarenImage* image = g_GameMaster.GetImage();
    image->DrawCharaChip(g, 160, 8, charadata.m_chipindex);

    std::string line2 = (boost::format("ٗp %3dLey  ێ %3dLey") % charadata.m_employcost % charadata.m_turncost).str();
    m_drawer->DrawString(g, 4, 64, line2.c_str(), white);

    std::string line3 = (boost::format("R %4dLey") % m_Money).str();
    m_drawer->DrawString(g, 4, 88, line3.c_str(), white);

    m_drawer->DrawString(g, 4, 104, "ق܂H", white);
}
//---------------------------------------------------------------------------
void TEmployHeroPane::Show(int area, int hero, int money)
{
    m_Area = area;
    m_Hero = hero;
    m_Money = money;
}
//---------------------------------------------------------------------------
TEmployHeroWindow::TEmployHeroWindow()
{
    setOpaque(false);
    add(&m_EmployHeroPane);
}
//---------------------------------------------------------------------------
void TEmployHeroWindow::Show(int area, int hero, int money)
{
    m_EmployHeroPane.Show(area, hero, money);
    resizeToContent();
    setVisible(true);
    g_GuiContainerList.GetGui()->getTop()->moveToTop(this);
    g_GuiContainerList.MoveWidgetToCenter(this);
}
//---------------------------------------------------------------------------
TPowerMapPane::TPowerMapPane()
{
    m_drawer = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_SMALL);
    add(&m_ImagePane);
    m_ImagePane.setPosition(0, 0);
    g_GameMaster.GetSkinManager()->ApplyDarkPaneSkin(this);
    setOpaque(false);
}
//---------------------------------------------------------------------------
void TPowerMapPane::draw(gcn::Graphics *g)
{
    static const gcn::Color white(255, 255, 255);

    //g->setColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_DARKPANE));
    g->setColor(getBackgroundColor());
    gcn::Rectangle r = getDimension();
    int imgw = m_ImagePane.getWidth();
    gcn::Rectangle fr(imgw, 0, r.width - imgw, r.height);
    g->fillRectangle(fr);
    int imgh = m_ImagePane.getHeight();
    int h = getHeight();
    if(imgh < h){
        gcn::Rectangle fr2(0, imgh, getWidth(), h - imgh);
        g->fillRectangle(fr2);
    }
    // }X^[Ɛ͐F`B
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TOpenFarenData* data = g_GameMaster.GetData();
    gcn::Rectangle cr(imgw + 2, 2, 12, 12);
    int y = 0;
    int mastercount = game->GetMasterInstanceCount();
    for(int mi=0; mi<mastercount; ++mi){
        const TMasterInstance& masterins = game->GetMasterInstance(mi);
        if(!masterins.m_exists){
            continue;
        }
        const TMaster& master = data->GetMaster(masterins.m_type);
        g->setColor(IntToGcnColor(master.m_powermapcolor));
        g->fillRectangle(cr);
        m_drawer->DrawString(g, imgw + 16, y + 2, master.m_name.c_str(), white);
        y += LINE_HEIGHT;
        cr.y += LINE_HEIGHT;
    }
    gcn::Container::draw(g);
}
//---------------------------------------------------------------------------
void TPowerMapPane::Show()
{
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TOpenFarenData* data = g_GameMaster.GetData();
    // ͒n}摜B
    TOpenFarenImage* image = g_GameMaster.GetImage();
    image->InitPowerMap();
    int areacount = game->GetAreaInstanceCount();
    for(int ai=0; ai<areacount; ++ai){
        const TAreaInstance& area = game->GetAreaInstance(ai);
        if(area.m_ignore || area.m_master == -1){
            continue;
        }
        const TMasterInstance& master = game->GetMasterInstance(area.m_master);
        const TMaster& masterdata = data->GetMaster(master.m_type);
        image->SetPowerMapPoints(masterdata.m_powermapcolor, area.m_powermappoint);
    }
    // ͒n}摜\B
    m_ImagePane.SetImage(image->GetPowerMap());

    int needw = 0;
    int needh = 0;
    int mastercount = game->GetMasterInstanceCount();
    for(int mi=0; mi<mastercount; ++mi){
        const TMasterInstance& masterins = game->GetMasterInstance(mi);
        if(!masterins.m_exists){
            continue;
        }
        const TMaster& master = data->GetMaster(masterins.m_type);
        int w = m_drawer->GetStringWidth(master.m_name.c_str()) + 20;
        if(needw < w){
            needw = w;
        }
        needh += LINE_HEIGHT;
    }
    // Kɏ݂B
    if(needw > 256){
        needw = 256;
    }
    if(needh < m_ImagePane.getHeight()){
        needh = m_ImagePane.getHeight();
    }
    setSize(m_ImagePane.getWidth() + needw, needh);
}
//---------------------------------------------------------------------------
// n}摜̃}EXCxgo邽߁B
// qWidget̃CxgeWidgetɓ߂vpeB΂̂A悤Ȃ̂ł̂悤ɁB
void TPowerMapPane::SetMouseListener(gcn::MouseListener *mouseListener)
{
    addMouseListener(mouseListener);
    m_ImagePane.addMouseListener(mouseListener);
}
//---------------------------------------------------------------------------
TPowerMapWindow::TPowerMapWindow()
{
    setOpaque(false);
    m_PowerMapPane.SetMouseListener(this);
    add(&m_PowerMapPane);
}
//---------------------------------------------------------------------------
void TPowerMapWindow::mousePressed(gcn::MouseEvent &mouseEvent)
{
    if(mouseEvent.getSource() != this){
        setVisible(false);
        mouseEvent.consume();
    }
    gcn::Window::mousePressed(mouseEvent);
}
//---------------------------------------------------------------------------
void TPowerMapWindow::Show()
{
    m_PowerMapPane.Show();
    resizeToContent();
    setVisible(true);
    g_GuiContainerList.GetGui()->getTop()->moveToTop(this);
    g_GuiContainerList.MoveWidgetToCenter(this);
}
//---------------------------------------------------------------------------
