#include "battlecharalist.h"
#include "ofdata.h"
#include "ofimage.h"
#include "boostformat.h"
#include "ofini.h"
#include "gui.h"
#include "gamemaster.h"
#include "gcnskinmanager.h"
#include "logger.h"

#include "battlemap.h"

//---------------------------------------------------------------------------
TBattleCharaListPane::TBattleCharaListPane()
{
    m_Drawer = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_SMALL);
    m_DrawerBig = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_NORMAL);
    m_MasterType.resize(2);
    m_CharaList.resize(2);
    setSize(464, TOPMARGIN + 360);
}
//---------------------------------------------------------------------------
TBattleCharaListPane::~TBattleCharaListPane()
{
}
//---------------------------------------------------------------------------
void TBattleCharaListPane::draw(gcn::Graphics *g)
{
    static const gcn::Color white(255, 255, 255);
    static const int basey = 40;
    int listy = 0;

    gcn::Rectangle r = getDimension();
    // ̃LXg̔wihB
    r.x = 0;
    if(m_ShowGraph){
        r.y = TOPMARGIN;
        r.height -= TOPMARGIN;
        listy = TOPMARGIN;
    }
    g->setColor(g_GameMaster.GetSkinManager()->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_BATTLECHARALISTPANE));
    g->fillRectangle(r);
    // 㕔̃Ot̔wihB
    if(m_ShowGraph){
        r.y = 0;
        r.height = TOPMARGIN;
        g->setColor(gcn::Color(32, 32, 32, 192));
        g->fillRectangle(r);
        // Ot`
        DrawForceBrave(g, getWidth()/2, 0, m_LeftForce, m_LeftBrave, m_RightForce, m_RightBrave, m_Drawer);
    }

    const TOpenFarenData* data = g_GameMaster.GetData();
    const TOpenFarenImage* image = g_GameMaster.GetImage();
    unsigned sz = m_CharaList.size();
    for(unsigned u=0; u<sz; ++u){
        std::string mastername;
        if(m_MasterType[u] != -1){
            mastername = data->GetMaster(m_MasterType[u]).m_name + "R";
        }
        else{
            mastername = "R";
        }
        int masterx, x, addx;
        if(u){
            masterx = 0;
            x = 0;
            addx = 104;
        }
        else{
            int w = getWidth();
            masterx = w - mastername.size() * (TGuiContainerList::FONT_SIZE_NORMAL / 2) - 1;
            x = w - 104;
            addx = -104;
        }
        m_DrawerBig->DrawString(g, masterx, listy + 4, mastername.c_str(), white);
        m_DrawerBig->DrawString(g, masterx+1, listy + 4, mastername.c_str(), white);

        int y = listy + basey;
        int ycount = 0;
        unsigned characount = m_CharaList[u]->size();
        for(unsigned cu=0; cu<characount; ++cu){
            const TCharacterInstance& chara = (*m_CharaList[u])[cu];
            if(!chara.m_hp || chara.m_runaway){
                continue;
            }
            const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
            image->DrawCharaChip(g, x, y, charadata.m_chipindex);

            std::string hp = (boost::format("HP %3d/%3d") % chara.m_hp % chara.m_hpmax_rank).str();
            std::string mp = (boost::format("MP %3d/%3d") % chara.m_mp % chara.m_mpmax_rank).str();
            m_Drawer->DrawString(g, x+36, y+2, hp.c_str(), white);
            m_Drawer->DrawString(g, x+36, y+16, mp.c_str(), white);

            y += TOpenFarenImage::MAPCHIP_HEIGHT;
            ++ycount;
            if(ycount == CHARAYCOUNT){
                x += addx;
                y = listy + basey;
                ycount = 0; // 2̏ꍇӖȂǁB
            }
        }
    }
}
//---------------------------------------------------------------------------
//void TBattleCharaListPane::SetCharaList(int index, int mastertype, const std::vector<const TCharacterInstance*>& charalist)
void TBattleCharaListPane::SetCharaList(int index, int mastertype, const TCharacterInstanceList* charalist)
{
    m_MasterType[index] = mastertype;
    m_CharaList[index] = charalist;
}
//---------------------------------------------------------------------------
void TBattleCharaListPane::ClearCharaList()
{
//    unsigned sz = m_CharaList.size();
//    for(unsigned u=0; u<sz; ++u){
//        m_CharaList[u].clear();
//    }
}
//---------------------------------------------------------------------------
void TBattleCharaListPane::SetProperty(int lforce, int lbrave, int rforce, int rbrave, int turn)
{
    m_LeftForce = lforce;
    m_LeftBrave = lbrave;
    m_RightForce = rforce;
    m_RightBrave = rbrave;
    m_Turn = turn;
}
//---------------------------------------------------------------------------
TBattleCharaListWindow::TBattleCharaListWindow() :
    m_CanHide(false)
{
    //setBaseColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_WINDOW));
    g_GameMaster.GetSkinManager()->ApplyWindowSkin(this);
    setOpaque(false);
    add(&m_BattleCharaListPane);
    resizeToContent();
}
//---------------------------------------------------------------------------
TBattleCharaListWindow::~TBattleCharaListWindow()
{
}
//---------------------------------------------------------------------------
void TBattleCharaListWindow::mousePressed(gcn::MouseEvent &mouseEvent)
{
    if(m_CanHide && getChildrenArea().isPointInRect(mouseEvent.getX(), mouseEvent.getY())){
        setVisible(false);
        mouseEvent.consume();
    }
    else{
        gcn::Window::mousePressed(mouseEvent);
    }
}
//---------------------------------------------------------------------------
