//---------------------------------------------------------------------------
#include "battlecommand.h"
#include "gamemaster.h"
#include "gcnskinmanager.h"
#include "ofini.h"
#include "gui.h"
#include "logger.h"
//---------------------------------------------------------------------------
void TBattleCommandListener::mouseMoved(gcn::MouseEvent &mouseEvent)
{
    gcn::Rectangle cr = m_Window->getChildrenArea();
    int x = mouseEvent.getX() - cr.x;
    int y = mouseEvent.getY() - cr.y;
    if(x >= 0 && x < TBattleCommandWindow::BUTTON_WIDTH * TOpenFarenImage::BUTTONIMAGE_MAX &&
       y >= 0 && y < TBattleCommandWindow::BUTTON_HEIGHT){
        m_Window->SetCommandStringIndex(x / TBattleCommandWindow::BUTTON_WIDTH);
    }
    else{
        m_Window->SetCommandStringIndex(-1);
    }
}
//---------------------------------------------------------------------------
TBattleCommandWindow::TBattleCommandWindow() :
    m_Shown(false)
{
    g_GameMaster.GetSkinManager()->ApplyWindowSkin(this);
    setOpaque(false);
}
//---------------------------------------------------------------------------
void TBattleCommandWindow::Show()
{
    if(!m_Shown){
        TOpenFarenImage* image = g_GameMaster.GetImage();
        for(int i = TOpenFarenImage::BUTTONIMAGE_WAIT; i < TOpenFarenImage::BUTTONIMAGE_MAX; ++i){
            m_ButtonList[i].setImage(image->GetButtonImage(static_cast<TOpenFarenImage::TButtonImageIndex>(i)));
            m_ButtonList[i].setSize(BUTTON_WIDTH, BUTTON_HEIGHT);
            m_ButtonList[i].setPosition(i*BUTTON_WIDTH, 0);
            add(&m_ButtonList[i]);
        }
        //resizeToContent();
        int pad = getPadding() * 2;
        int w = TOpenFarenImage::BUTTONIMAGE_MAX * BUTTON_WIDTH + pad;
        int h = getTitleBarHeight() + BUTTON_HEIGHT + COMSTR_HEIGHT + pad;
        setSize(w, h);
        m_Listener.SetWindow(this);
        addMouseListener(&m_Listener);
        m_Shown = true;
    }
    m_CommandStringIndex = -1;
    setVisible(true);
    g_GuiContainerList.GetGui()->getTop()->moveToTop(this);
}
//---------------------------------------------------------------------------
void TBattleCommandWindow::draw(gcn::Graphics *g)
{
    TGcnWindow::draw(g);
    gcn::Rectangle cr = getChildrenArea();
    for(int i = TOpenFarenImage::BUTTONIMAGE_WAIT; i < TOpenFarenImage::BUTTONIMAGE_MAX; ++i){
        if(!m_ButtonList[i].isEnabled()){
            g->setColor(gcn::Color(0, 0, 0, 128));
            gcn::Rectangle r = m_ButtonList[i].getDimension();
            r.x += cr.x;
            r.y += cr.y;
            g->fillRectangle(r);
        }
    }
    gcn::Rectangle r = cr;
    r.y += BUTTON_HEIGHT;
    r.height -= BUTTON_HEIGHT;
    g->setColor(g_GameMaster.GetSkinManager()->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE));
    g->fillRectangle(r);
    if(m_CommandStringIndex != -1 && !m_CommandStrings.empty()){
        int fontsize = TGuiContainerList::FONT_SIZE_NORMAL;
        int x = r.x + (r.width - m_CommandStrings[m_CommandStringIndex].size() * fontsize / 2) / 2;
        int y = r.y + (COMSTR_HEIGHT - fontsize) / 2;
        g->setColor(gcn::Color(255,255,255));
        g->drawText(m_CommandStrings[m_CommandStringIndex].c_str(), x, y);
        g->drawText(m_CommandStrings[m_CommandStringIndex].c_str(), x+1, y);
        //g_Logger.DebugLog(m_CommandStrings[m_CommandStringIndex]);
    }
}
//---------------------------------------------------------------------------
void TBattleCommandWindow::SetCommandStrings(const std::vector<std::string>& strs)
{
    m_CommandStrings = strs;
}
//---------------------------------------------------------------------------
void TBattleCommandWindow::SetCommandStringIndex(int index)
{
    m_CommandStringIndex = index;
}
//---------------------------------------------------------------------------
