//---------------------------------------------------------------------------
#ifndef battleifH
#define battleifH

#include <vector>
#include "ofdata.h"
#include "ofgame.h"
#include "bmap.h"
#include "valuelayer.h"
//---------------------------------------------------------------------------
struct TEvalResult
{
    point_t location;
    int masterindex;
    int charaindex;
    //int hitratio;     // 
    int value;
    int resattr;                // ϐύX
    TResistance restype;        // ϐύX
    int movetype;               // ړ^CvύX
    int revivecharatype;
    int summoncharatype;
    TSystemEffect systemeffect;
};
typedef std::vector<TEvalResult> TEvalResultList;
//---------------------------------------------------------------------------
enum TActionType{
    ACTION_WAIT,
    ACTION_ATTACK,
    ACTION_CHARGE,
    ACTION_SPECIAL,
    ACTION_RUNAWAY,
    ACTION_MAX
};
//---------------------------------------------------------------------------
struct TAction
{
    TActionType acttype;
    point_t location;
    int special;
};
typedef std::vector<TAction> TActionList;
//---------------------------------------------------------------------------
class TBattleInterface
{
public:
    virtual bool IsBattle() const = 0;
    virtual int GetTurn() const = 0;
    virtual void GetMapSize(int& w, int& h) const = 0;
    virtual int GetMapHeight(int x, int y) const = 0;
    virtual const TBattleMap* GetBattleMap() const = 0;
    virtual int GetMoveTypeCost(int movetype) const = 0;
    virtual int GetMoveTypeSkill(int movetype) const = 0;
    virtual void GetMovableLayer(TValueLayer<int>& layer) const = 0;
    virtual void GetMovableLayerFrom(int master, const TCharacterInstance& chara, int x, int y, int val, TValueLayer<int>& movelayer, TValueLayer<point_t>& movefromlayer, bool loose) const = 0;
    virtual int FindMasterIndex(int mastertype) const = 0;
    virtual void GetActiveChara(int& master, int& charaindex) const = 0;
    virtual int GetCharaCount(int master) const = 0;
    virtual bool GetCharaOn(int x, int y, int& master, int& charaindex) const = 0;
    virtual bool GetChara(int master, int charatype, int& charaindex) const = 0;
    virtual void GetCharaStatus(int master, int charaindex, TCharacterInstance& chara) const = 0;
    virtual TCharacterInstance CreateCharaInstance(int charatype, int rank) const = 0;
    virtual void GetActionList(int movex, int movey, TActionList& actlist, std::vector<point_t>& targetlist) const = 0;
    virtual bool GetLastTarget(int& x, int& y) const = 0;
    virtual bool EvalAction(int movex, int movey, TActionType acttype, int special, int targetx, int targety, TEvalResultList& evallist) const = 0;
    virtual bool SetAction(int movex, int movey, TActionType acttype, int special, int targetx, int targety) = 0;
    virtual bool GetAutoRunAwayPoint(point_t& loc) const = 0;
    //virtual bool ChangeMasterType(int master, int mastertype) = 0;
    virtual void SetCharaBrave(int master, int charaindex, int brave) = 0;
    virtual bool ChangeCharaType(int master, int charaindex, int charatype) = 0;
    virtual void SetMapChip(int x, int y, int chip) = 0;
    virtual void RemoveMapObj(int x, int y) = 0;
    virtual void RemoveMapObjSpecify(int x, int y, TMapObject objtype) = 0;
//    virtual void ShowMessageTimer(const std::string& title, const std::string& msg, bool timeout) = 0;
};
//---------------------------------------------------------------------------
#endif
