//---------------------------------------------------------------------------
#ifndef battlemapH
#define battlemapH

#include <vector>
//#include <boost/shared_ptr.hpp>
#include <guichan.hpp>
#include "imagepane.h"
#include "animationpane.h"
#include "areaope.h"
#include "charastatus.h"
#include "valuelayer.h"
#include "battleif.h"
#include "oflua.h"
#include "gcnwindow.h"

void DrawForceBrave(gcn::Graphics *g, int basecenterx, int basey, int leftforce, int leftbrave, int rightforce, int rightbrave, TGcnCharDrawer *chardrawersmall);

class TBattleMap;
//---------------------------------------------------------------------------
struct TShowingChara
{
    int m_type;
    int m_markchip;     // ^Ũ}[NBTOpenFarenImage::TChipImageIndex̒lB
    point_t m_location1;
    point_t m_location2;
    int m_step;             // 0-256 0̎m_location1ɁA256̎m_location2ɕ\B
};
//---------------------------------------------------------------------------
struct TDamageString
{
    std::string m_caption;
    point_t m_location;
    gcn::Color m_color;
    int m_tick;
};
//---------------------------------------------------------------------------
struct TRadiusChip
{
    point_t m_location;
    gcn::Color m_color;
};
//---------------------------------------------------------------------------
struct TEffectImage
{
    point_t m_location;
    int m_index;
    int m_anim;
    int m_direction;
    int m_width;
    int m_height;
};
//---------------------------------------------------------------------------
class TBattleMapPane : public gcn::Widget
{
private:
    TBattleMap *m_map;
    int m_Tick;
    std::vector<TShowingChara> m_ShowingCharaList;
    std::vector<gcn::Rectangle> m_ChipFrameList;
    std::vector<TDamageString> m_DamageStringList;
    std::vector<TRadiusChip> m_RadiusChipList;
    TValueLayer<int> *m_ShadeLayer;
    std::vector<TEffectImage> m_EffectImageList;
    point_t m_BrightChip;
    int m_BrightStep;
    //static const int BRIGHTSTEP_MAX = 16;
    //static const int m_FontSize = 16;
    static const int m_ChipWidth = 32;
    static const int m_ChipHeight = 32;
    static const int DAMAGETICK_MAX = 9;
    int m_AnimationSpeed;                   // 傫قǂAj[VB
public:
    TBattleMapPane();
    virtual ~TBattleMapPane();
    void SetMap(TBattleMap *bmap);
    int AddShowingChara(const TShowingChara& c);
    void SetShowingChara(int i, const TShowingChara& c) { m_ShowingCharaList[i] = c; }
    void RemoveShowingChara(int i) { m_ShowingCharaList.erase(m_ShowingCharaList.begin() + i); }
    void ClearShowingChara() { m_ShowingCharaList.clear(); }
    void AddChipFrame(int x, int y);
    //void XY_PhyToMap(int PhyX, int PhyY, int *MapX, int *MapY) const;
    void ClearChipFrame() { m_ChipFrameList.clear(); }
    void SetShadeLayer(TValueLayer<int> *l) { m_ShadeLayer = l; }
    void SetBrightChip(int x, int y);
    void ClearBrightChip();
    void AddDamageString(int chipx, int chipy, const std::string& str, const gcn::Color& c);
    void AddRadiusChip(int chipx, int chipy, const gcn::Color& c);
    void ClearRadiusChip();
    void AddEffectImage(int x, int y, int index, int anim, int dir, int width, int height);
    void ClearEffectImage() { m_EffectImageList.clear(); }
    void SetAnimationSpeed(int speed) { m_AnimationSpeed = speed; }
    virtual void logic();
    virtual void draw(gcn::Graphics *g);
};
//---------------------------------------------------------------------------
class TBattleTopPane : public gcn::Widget
{
public:
    TBattleTopPane();
    virtual ~TBattleTopPane();
    void SetString(const std::string& areaname, const std::string& lstr, const std::string& rstr);
    void SetProperty(int lforce, int lbrave, int rforce, int rbrave, int turn);
    virtual void draw(gcn::Graphics *g);
private:
    std::string m_AreaName;
    std::string m_LeftName;
    std::string m_RightName;
    bool m_Watching;
    int m_LeftForce;
    int m_LeftBrave;
    int m_RightForce;
    int m_RightBrave;
    int m_Turn;
    TGcnCharDrawer *m_CharDrawerSmall;
};
//---------------------------------------------------------------------------
class TBattleBottomPane : public gcn::Widget
{
public:
    TBattleBottomPane();
    virtual ~TBattleBottomPane();
    void SetMessage(const std::string& msg) { m_Message = msg; }
    void ClearMessage() { m_Message.clear(); }
    void SetPointedChip(int x, int y, int movetype);
    void ClearPointedChip();
    void SetOperatingChara(const TCharacterInstance& chara, int mark);
    void ClearOperatingChara();
    void SetMap(TBattleMap* p) { m_Map = p; }
    virtual void draw(gcn::Graphics *g);
private:
    std::string m_Message;
    TBattleMap *m_Map;
    point_t m_PointedChip;
    int m_PointedMoveType;
    TCharacterInstance m_OperatingChara;
    int m_OperatingCharaMark;
    TGcnCharDrawer *m_CharDrawer;

    static const int m_FontSize = 12;
};
//---------------------------------------------------------------------------
class TBattleMapLogic;
class TBattleMapWindow : public TGcnWindow
{
public:
    TBattleMapWindow();
    virtual ~TBattleMapWindow();
    bool Show();
    TBattleMapPane& GetBattleMapPane() { return m_BattleMapPane; }
    TBattleTopPane& GetBattleTopPane() { return m_BattleTopPane; }
    TBattleBottomPane& GetBattleBottomPane() { return m_BattleBottomPane; }
    void SetBattleMapLogic(TBattleMapLogic *l){ m_Logic = l; }
    void GetScrollAmount(int& x, int& y);
    void SetScrollAmount(int x, int y);
    void SetScrollCenter(int x, int y);
    void SetScrollCenterOfChip(int chipx, int chipy);
    void SetScrollCenterOfChips(const std::vector<point_t>& plist);
    void SetGuidance(const std::string& str) { m_Guidance = str; }
    void ClearGuidance() { m_Guidance.clear(); }
    //void SetBottomMessage(const std::string& msg) { m_BattleBottomPane.SetMessage(msg); }
    //void ClearBottomMessage() { m_BattleBottomPane.ClearMessage(); }
    virtual void draw(gcn::Graphics *g);
    virtual void logic();
private:
    gcn::ScrollArea m_ScrollArea;
    TBattleMapPane m_BattleMapPane;
    TBattleTopPane m_BattleTopPane;
    TBattleBottomPane m_BattleBottomPane;
    TBattleMapLogic *m_Logic;
    std::string m_Guidance;
    static const int m_FontSize = 16;
};
//---------------------------------------------------------------------------
struct TAutoSpecialEntry
{
    TSpecialEventType m_event;
    int m_master;
    int m_charaindex;
    std::string m_function;     // XNvg̎ɒlBKEZ̎͋󕶎B
};
typedef std::vector<TAutoSpecialEntry> TAutoSpecialEntryList;
//---------------------------------------------------------------------------
struct TEffectResult
{
    point_t m_location;
    bool m_hit;
    int m_value;
    int m_special;
    int m_revivecharamaster;
    int m_revivecharaindex;
    int m_revivecharatype;
    int m_summoncharatype;
    TSystemEffect m_systemeffect;
};
typedef std::vector<TEffectResult> TEffectResultList;
//---------------------------------------------------------------------------
class TGcnWidgetSet;
class TCharaPlaceWindow;
class TBattleCommandWindow;
class TMagicWindow;
class TBattleCharaListWindow;
class TBattleMapLogic : public gcn::MouseListener, public gcn::KeyListener, public gcn::ActionListener, public TBattleInterface
{
public:
    TBattleMapLogic(TGcnWidgetSet *wset);
    virtual ~TBattleMapLogic() {}
    void SetExternalWindow(TCharaStatusWindow *csw, TCharaStatusFullWindow *csfw);
    void SetScriptAPI(TScriptAPI* p) { m_ScriptAPI = p; }
    bool Start(TBattleMap *bmap, const std::vector<TCharacterInstanceList*>& charalist, const std::vector<int>& masterindex, int areaindex, const std::vector<std::vector<int> >& neighborarea);
    void DoLogic();
    virtual void mousePressed(gcn::MouseEvent &mouseEvent);
    virtual void mouseMoved(gcn::MouseEvent &mouseEvent);
    virtual void keyPressed(gcn::KeyEvent &keyEvent);
    virtual void action(const gcn::ActionEvent& actionEvent);
    bool IsEnd() const { return m_LogicStatus == LOGIC_END; }
    int GetWinMaster() const { return m_WinMaster; }
    bool CanNoWatch() const;
    void SetNoWatch();
    void ApplyConfig();
    // TBattleInterface̊֐
    virtual bool IsBattle() const { return !IsEnd(); }
    virtual int GetTurn() const { return m_Turn; }
    virtual void GetMapSize(int& w, int& h) const;
    virtual int GetMapHeight(int x, int y) const;
    virtual const TBattleMap* GetBattleMap() const;
    virtual int GetMoveTypeCost(int movetype) const;
    virtual int GetMoveTypeSkill(int movetype) const;
    virtual void GetMovableLayer(TValueLayer<int>& layer) const;
    virtual void GetMovableLayerFrom(int master, const TCharacterInstance& chara, int x, int y, int val, TValueLayer<int>& movelayer, TValueLayer<point_t>& movefromlayer, bool loose) const;
    virtual int FindMasterIndex(int mastertype) const;
    virtual int GetMyMaster() const;
    virtual void GetActiveChara(int& master, int& charaindex) const;
    virtual int GetCharaCount(int master) const { return m_CharaList[master]->size(); }
    virtual bool GetCharaOn(int x, int y, int& master, int& charaindex) const;
    virtual bool GetChara(int master, int charatype, int& charaindex) const;
    virtual void GetCharaStatus(int master, int charaindex, TCharacterInstance& chara) const;
    virtual TCharacterInstance CreateCharaInstance(int charatype, int rank) const;
    virtual void GetActionList(int movex, int movey, TActionList& actlist, std::vector<point_t>& targetlist) const;
    virtual bool GetLastTarget(int& x, int& y) const;
    virtual bool EvalAction(int movex, int movey, TActionType acttype, int special, int targetx, int targety, TEvalResultList& evallist) const;
    virtual bool SetAction(int movex, int movey, TActionType acttype, int special, int targetx, int targety);
    virtual bool GetAutoRunAwayPoint(point_t& loc) const;
    virtual void SetCharaBrave(int master, int charaindex, int brave);
    //virtual bool ChangeMasterType(int master, int mastertype);
    virtual bool ChangeCharaType(int master, int charaindex, int charatype);
    virtual void SetMapChip(int x, int y, int chip);
    virtual void RemoveMapObj(int x, int y);
    virtual void RemoveMapObjSpecify(int x, int y, TMapObject objtype);
private:
    TGcnWidgetSet *m_WidgetSet;
    TBattleMapWindow *m_BattleMapWindow;
    TBattleCommandWindow *m_BattleCommandWindow;
    TMagicWindow *m_MagicWindow;
    TAnimationPane *m_AnimationPane;
    TBattleCharaListWindow *m_BattleCharaListWindow;
    TCharaPlaceWindow *m_CharaPlaceWindow;
    // ł͐|C^ĖႤEBhEB
    TCharaStatusWindow *m_CharaStatusWindow;
    TCharaStatusFullWindow *m_CharaStatusFullWindow;
    TBattleMap *m_Map;
    //TStrategyInterface *m_StrategyInterface;
    TValueLayer<int> m_ShadeLayer;
    TValueLayer<int> m_MasterLayer;
    TValueLayer<int> m_CharaLayer;
    TValueLayer<int> m_MoveLayer;
    TValueLayer<int> m_DummyLayer;
    TValueLayer<point_t> m_MoveFromLayer;

    enum TLogicStatus{
        LOGIC_PLAYERTACTICS = 1,
        LOGIC_PLAYERPLACE,
        LOGIC_PLACE,
        LOGIC_PLACECOMPLETE,
        LOGIC_NEXTCHARA,
        LOGIC_PROGRESSTURN,
        LOGIC_MOVING,
        LOGIC_ATTACKING,
        LOGIC_CHARGING,
        //LOGIC_CRASHWALL,
        //LOGIC_EFFECT_BEFORE,
        LOGIC_EFFECT,
        //LOGIC_EFFECT_AFTER,
        LOGIC_WAITBEFORENEXTCHARA,
        LOGIC_PLAYERCOMMAND_BEFOREMOVE,
        LOGIC_PLAYERCOMMAND_AFTERMOVE,
        LOGIC_PLAYERMOVE,
        LOGIC_PLAYERATTACK,
        LOGIC_PLAYERCHARGE,
        LOGIC_PLAYERSPECIAL,
        LOGIC_PLAYERMAGIC,
        LOGIC_COMTHINK,
        LOGIC_SCRIPT,
        LOGIC_HERODEAD_MSG,
        LOGIC_TIMEOUT_MSG,
        LOGIC_TIMEOUT_RETREAT,
        LOGIC_BATTLEEND_MSG,
        LOGIC_END,
        LOGIC_ERROR     // vÕoO܂̓f[^G[B
    }m_LogicStatus;
    //bool m_BeforeLogicIsSpecial;
    bool m_NoWatch;
    int m_SelectedSpecial;
    int m_SpecialCount;

    enum TScriptStatus{
        SCRIPT_BEFORE_NEXTCHARA = 1,
        SCRIPT_BEFORE_SPECIAL,
        SCRIPT_AFTER_SPECIAL
    }m_ScriptStatus;

    static const int SAVESIDE_PLACEMAX = 128;   // ̊XEǂ܂ŏzuWƂ݂ȂB
    static const int SAVESIDE_PLACEMIN = 20;    // ŒKvȏzuWB
    static const int ACTIONPOINT_OK = 500;      // s|Cg܂sł邩B
    static const int TURNLOOP_MAX = 5;          // 1^[s|Cg𑝂₷B
    static const int CHARGE_MAX = 31;           // IWi܂łȂ̂mȂǂꂮ炢H
    static const int BRAVE_MAX = 100;           // BravȅB
    static const int BADSTATUS_TICKMAX = 3;     // obhXe[^XɑԁB

    // ȉ̃Xg͌ł͗vfK2ɂȂ邪AUm_NeighborArea̒lŔ肷邱ƁB
    // LXgB}X^[U}X^[̏ŕԁB
    std::vector<TCharacterInstanceList*> m_CharaList;

    // }X^[U}X^[̏Ń}X^[ԁB
    std::vector<int> m_MasterIndex;

    // owGACfbNX̃XgB}X^[U}X^[̏ŃLԁB
    // ɂĂ-1B
    std::vector<std::vector<int> > m_NeighborArea;
    // ̃GACfbNXB
    int m_AreaIndex;

    // zuW̃XgB}X^[U}X^[̏ŕԁB
    std::vector<point_t> m_PlaceXYList;

    // ߉\Ȏ푰tOB}X^[U}X^[̏Ń}X^[ԁB
    std::vector<std::vector<int> > m_CommandableRace;

    // wc̖@̎gp񐔁B񐔐pB
    std::vector<std::vector<int> > m_MagicUseCount;
    // TSpecial̃CfbNXTMagicDatãCfbNXt}bvB@̎gp񐔏pB
    // COMgp񐔐@gTSpecial̃CfbNXȂ߁B蔲B
    std::map<int, int> m_SpecialMagicMap;

    // zupB
    std::vector<int> m_PlacedFlag;
    int m_PlaceMasterIndex;
    int m_PlayerPlacedCount;
    TCharacterInstanceList m_PlaceCharaList;

    // pBɓDœłB
    std::vector<std::pair<int, int> > m_RunAwayList;
    int m_SaveRunAwayCount; // ̓\B퓬JnɈ񋁂߂B

    // pB
    std::vector<std::vector<int> > m_DeadList;

    // KEZXgB
    TAutoSpecialEntryList m_AutoSpecialList;
    // KEZɒlBSꍇŨXgGg폜̂ŏ̃Xg̃CfbNXɂ͂łȂB
    //std::vector<std::pair<int, int> > m_ActivatedAutoSpecialList;
    std::vector<TAutoSpecialEntry> m_ActivatedAutoSpecialList;

    // XNvgXgB
    //TAutoScriptEntryList m_AutoScriptEntryList;

    int m_Turn;
    int m_TurnLoop;     // s|Cg
    int m_ActiveMaster;
    int m_ActiveChara;
    bool m_ActiveCharaIsPlayer;
    int m_WinMaster;    // }X^[̃CfbNX
    // ẼXgBsǏolvZɎgBz͈ꉞUł邽߃|C^B
    std::vector<const TCharacterInstance*> m_KillList;

    // ڍׂ\ĂLB
    int m_ShownCharaMaster;
    int m_ShownChara;

    // ړAj[Vp
    int m_MoveFramePerChip;
    int m_MoveFrame;
    int m_MoveFrameMax;
    point_t m_MoveFrom;     // ړOW
    point_t m_MoveTo;       // ړW
    bool m_Moved;

    // UAj[Vp
    int m_AttackIndex;
    int m_AttackFrame;
    int m_AttackFrameMax;
    point_t m_AttackTo;
    // Up
    int m_DefenseMaster;
    int m_DefenseChara;
    TEffectResultList m_LastAttackResultList;

    std::vector<point_t> m_EffectTo;
    point_t m_EffectTarget;
    int m_EffectFrame;
    int m_EffectCount;              // sʂ̉񐔂AƂ肠͕KEZ̏ꍇ̂ݎgpB
    int m_WaitBeforeNextCharaFrame;

    int m_TurnProgressFrame;
    int m_TurnProgressFrameMax;
    enum TTurnProgressStage{
        TURNPROGRESS_BADSTATUS,
        TURNPROGRESS_HPRECOVER,
        TURNPROGRESS_MPRECOVER,
        TURNPROGRESS_POISON,
        TURNPROGRESS_RUNAWAY,
        TURNPROGRESS_MAX
    }m_TurnProgressStage;

    // ⏕p
    // eLŌɎs@/KEZBL[m_CharaList̃CfbNXAlSpecialIndex
    std::vector<std::map<int, int> > m_LastSpecialTable;
    // wcōŌɎsꂽKEZB
    std::vector<int> m_LastSpecial;

    // CxgXNvg
    TOpenFarenLuaManager m_LuaBattleEvent;
    // OTScriptAPICX^XQƂB
    TScriptAPI *m_ScriptAPI;

    // AIp
    std::vector<TOpenFarenLuaManager> m_LuaBattleCom;
    TActionType m_ComActionType;
    int m_ComSpecial;
    point_t m_ComTarget;

    // AIvlp
    std::vector<int> m_MoveTypeCost;
    std::vector<int> m_MoveTypeSkill;

    //int EffectFrameMax;
    int m_AnimationSpeed;                   // 傫قǂAj[VB
    int m_AnimationPerChip;
    static const int MOVECOST_FACTOR = 3;     // ړ͂ƃRXǧW

    void DoSelectedWaitLogic();
    void DoSelectedMoveLogic(int x, int y);
    bool DoSelectedAttackLogic(int x, int y, bool isCharge);
    void DoSelectedSpecialLogic(int specialindex, bool isBeforeScript);
    void DoSelectedRunAwayLogic();
    bool DoAttackingLogic();
    void DoEffectLogic();
    bool DoJudgeGameLogic();
    bool DoJudgeHeroLogic();
    bool DoAutoSpecialLogic();
    void DoMoveEndLogic();
    void TransitEffectLogic(int targetx, int targety);
    int FindNextAttackIndex(const TCharacterInstance& atkchara, const TCharacterInstance& defchara, int totalindex) const;
    void GetPlaceList(const std::vector<int>& neighbor, const TCharacterInstanceList& charalist, int offset, std::vector<point_t>& placelist, TCharacterInstanceList& target);
    void PlayerNextPlace();
    void ActivateNextChara(bool& battleend, bool& progressturn);
    bool ProgressTurn();
    bool IsCommandableChara(int master, int chara);
    void UpdateCommandableRace();
    void UpdateAutoSpecialList();
    void ActivateAutoSpecial(int master, int charaindex);
    bool IsPlayerMaster(int master) const;
    void GetMovable(int master, const TCharacterInstance& chara);
    void GetMovableFrom(int master, const TCharacterInstance& chara, int x, int y, int val, TValueLayer<int>& movelayer, TValueLayer<point_t>& movefromlayer, bool loose) const;
    bool InEnemyZoc(int masterindex, int x, int y) const;
    void GetEnemyNextList(int masterindex, int x, int y, std::vector<point_t>& plist) const;
    void GetMySideNextList(int masterindex, int charaindex, int x, int y, std::vector<point_t>& plist) const;
    bool HasWallNext(int x, int y) const;
    bool ExistsEnemyNext(int masterindex, int x, int y, bool checkEnemyAttack) const;
    bool ExistsEnemyInRadius(int masterindex, int x, int y, int radius) const;
    void EvalAttackDamage(const TCharacterInstance& atkchara, int atkindex, const TCharacterInstance& defchara, std::vector<TEffectResult>& resultlist) const;
    void GetAttackList(const TCharacterInstance& atkchara, std::vector<TAttack>& atklist) const;
    bool CanAttack(const TCharacterInstance& atkchara) const;
    void EvalAttackList(int atkmaster, int atkcharaindex, int defmaster, int defcharaindex, TEvalResultList& evallist, bool isCharge) const;
    void EvalSpecial(int master, const TCharacterInstance& atkchara, const std::vector<point_t>& targetlist, int special, std::vector<TEffectResult>& resultlist, int power, bool usepower) const;
    void EvalRevive(int master, const TBattleCharacter& atkcharadata, const std::vector<point_t>& targetlist, int special, std::vector<TEffectResult>& resultlist) const;
    void ShowSpecial(const TCharacterInstance& atkchara, const std::vector<point_t>& targetlist, int special, int frame, std::deque<bool>& evaltiming, bool& end) const;
    void UpdateActionCommandWindow(bool moved);
    void UpdateShowingChara(int masterindex, int charaindex);
    void UpdateShowingChara();
    void UpdateForceBrave();
    void UpdateSideName();
    void HighlightCircle(int x, int y);
    void ShowCharaDetail(int masterindex, int charaindex);
    void HideCharaDetail();
    void UpdateCharaDetail();
    void ShowBattleCharaList(bool canHide);
    void HideBattleCharaList();
    bool GetRange(TValueLayer<int>& layer, int x, int y, int rangetype, int range, int radius, int reverse) const;
    void GetNormalRange(TValueLayer<int>& layer, int x, int y, int range, int reverse) const;
    void GetStreamRange(TValueLayer<int>& layer, int x, int y, int radius, int reverse) const;
    int GetStreamDirection(int fromx, int fromy, int x, int y) const;
    void GetWallRange(TValueLayer<int>& layer, int reverse);
    void GenerateEffectPointList(point_t from, point_t to, int specialindex);
    void GetEffectPointList(point_t from, point_t to, int radius, TRangeType rangetype, int rangeheight, std::vector<point_t>& plist) const;
    void GetNormalPointList(int x, int y, int radius, std::vector<point_t>& plist) const;
    void GetStreamPointList(int x, int y, int radius, int dir, std::vector<point_t>& plist) const;
    void GetAllMySidePointList(int master, std::vector<point_t>& plist) const;
    void GetAllEnemyPointList(int master, std::vector<point_t>& plist) const;
    void GetWallPointList(std::vector<point_t>& plist) const;
    void GetLastDeadMySidePointList(int master, std::vector<point_t>& plist, std::vector<std::pair<int, int> >& indexlist) const;
    void GetLastDeadEnemyPointList(int master, std::vector<point_t>& plist, std::vector<std::pair<int, int> >& indexlist) const;
    void GetAllDeadMySidePointList(int master, std::vector<point_t>& plist, std::vector<std::pair<int, int> >& indexlist) const;
    void GetAllDeadEnemyPointList(int master, std::vector<point_t>& plist, std::vector<std::pair<int, int> >& indexlist) const;
    void GetLowerThanPointList(int rangeheight, std::vector<point_t>& plist) const;
    void GetEffectTargetList(const std::vector<point_t>& src, const std::deque<bool>& evaltiming, int masterindex, int special, std::vector<point_t>& plist) const;
    bool CanEffect(point_t from, point_t to, int masterindex, int specialindex) const;
    int GetAnimationDistance(point_t from, point_t to) const;
    int GetAnimationDistanceMax(point_t from, const std::vector<point_t>& tolist) const;
    int GetAnimationDirection(point_t from, point_t to) const;
    void SetRadiusChip(int x, int y, int radius, const gcn::Color& c);
    void SetStreamChip(int x, int y, int radius, int dir, const gcn::Color& c);
    void SetEffectString(const std::vector<TEffectResult>& results);
    void ApplyEffectResult(TCharacterInstance* atkchara, const std::vector<TEffectResult>& results, bool& existsDeadChara);
    void ApplySummon(const TEffectResult& result, TCharacterInstance* atkchara);
    void ApplyRevive(const TEffectResult& result);
    bool ApplyHPChange(int masterindex, int charaindex, int dhp);
    void AddExperience(TCharacterInstance& c, bool actioned, std::vector<const TCharacterInstance*> killlist);
    void UpdateOnCharaChange();
    void UpdateMapObjStepped();
    bool JudgeGame(int& winmaster) const;
    bool CanRunAway(int master, int charaindex, int movex, int movey) const;
    void CallBattleProgress();
    void UpdateBattleSpeed();
    void ShowBattleCommand();
    void HideBattleCommand();
    void PlayBattleMusic();
    void InitMoveTypeCostSkill();
    void UpdateMoveTypeCostSkill(int x, int y, int oldtype);
    bool IsAllowedRedoAction(bool moved, TActionType acttype, int special) const;
    bool IsValidXY(int x, int y) const;
};
//---------------------------------------------------------------------------
#endif
