#include <cmath>
#include "bmap.h"
#include "ofxml.h"
#include "boostformat.h"
#include "boostlexicalcast.h"
#include "logger.h"
#include "ofdata.h"
#include "neighbor.h"
#include "gamemaster.h"

namespace{
    int GetIntValues(const char *str, int *values, int valuemax)
    {
        const char *p = str;
        int i;
        for(i=0; i<valuemax; i++){
            if(i){
                p = strchr(p, ',');
                if(p == NULL){
                    return i;
                }
                p++;
            }
            /*try{
                values[i] = boost::lexical_cast(p);
            }
            catch(boost::bad_lexical_cast&){
                break;
            }*/
            if(sscanf(p, "%d", &values[i]) != 1){
                return i;
            }
        }
        return i;
    }

    void IntValuesToString(std::string& dst, const int *values, int valuemax)
    {
        for(int i=0; i<valuemax; i++){
            if(i){
                dst += ",";
            }
            dst += boost::lexical_cast<std::string>(values[i]);
        }
    }

    const char* XMLELM_MAPOBJINS = "MapObjectInstance";
}
//---------------------------------------------------------------------------
bool TBattleMap::LoadFromElement(const TiXmlElement *e_root, const char *locstr, const TOpenFarenData *data)
{
    int width, height, city, wall, road;
    if(e_root->QueryIntAttribute("Width", &width) != TIXML_SUCCESS){
        g_Logger.ErrorLog((boost::format("%s : Width not found.") % locstr).str());
        return false;
    }
    if(e_root->QueryIntAttribute("Height", &height) != TIXML_SUCCESS){
        g_Logger.ErrorLog((boost::format("%s : Height not found.") % locstr).str());
        return false;
    }
    if(e_root->QueryIntAttribute("City", &city) != TIXML_SUCCESS){
        g_Logger.ErrorLog((boost::format("%s : City not found.") % locstr).str());
        return false;
    }
    if(e_root->QueryIntAttribute("Wall", &wall) != TIXML_SUCCESS){
        g_Logger.ErrorLog((boost::format("%s : Wall not found.") % locstr).str());
        return false;
    }
    if(e_root->QueryIntAttribute("Road", &road) != TIXML_SUCCESS){
        g_Logger.ErrorLog((boost::format("%s : Road not found.") % locstr).str());
        return false;
    }

    m_MapChipLayer.Resize(width, height);
    m_MapObjLayer.Resize(width, height);
    m_MapObjNoLayer.Resize(width, height);
    m_MapHeightLayer.Resize(width, height);

    const TiXmlElement *e_chip = e_root->FirstChildElement("MapChip");
    if(e_chip == NULL){
        g_Logger.ErrorLog((boost::format("%s : MapChip not found.") % locstr).str());
        return false;
    }
    const TiXmlElement *e_obj = e_root->FirstChildElement("MapObject");
    if(e_obj == NULL){
        g_Logger.ErrorLog((boost::format("%s : MapObject not found.") % locstr).str());
        return false;
    }
    const TiXmlElement *e_objno = e_root->FirstChildElement("MapObjectNo");
    if(e_objno == NULL){
        g_Logger.ErrorLog((boost::format("%s : MapObjectNo not found.") % locstr).str());
        return false;
    }
    const TiXmlElement *e_height = e_root->FirstChildElement("MapHeight");
    if(e_height == NULL){
        g_Logger.ErrorLog((boost::format("%s : MapHeight not found.") % locstr).str());
        return false;
    }

    for(int y=0; y<height; ++y){
        std::string stry = (boost::format("L%d") % y).str();
        // n`
        const TiXmlElement *e_line = e_chip->FirstChildElement(stry.c_str());
        if(e_line == NULL){
            g_Logger.ErrorLog((boost::format("%s : MapChip Format Error.") % locstr).str());
            return false;
        }
        const TiXmlText *text = e_line->FirstChild()->ToText();
        if(GetIntValues(text->Value(), &m_MapChipLayer.GetValue(0, y), width) != width){
            g_Logger.ErrorLog((boost::format("%s : MapChip < Width.") % locstr).str());
            return false;
        }
        // 㕨
        e_line = e_obj->FirstChildElement(stry.c_str());
        if(e_line == NULL){
            g_Logger.ErrorLog((boost::format("%s : MapObject Format Error.") % locstr).str());
            return false;
        }
        text = e_line->FirstChild()->ToText();
        if(GetIntValues(text->Value(), &m_MapObjLayer.GetValue(0, y), width) != width){
            g_Logger.ErrorLog((boost::format("%s : MapObject < Width.") % locstr).str());
            return false;
        }
        // 㕨̔zuԍ
        e_line = e_objno->FirstChildElement(stry.c_str());
        if(e_line == NULL){
            g_Logger.ErrorLog((boost::format("%s : MapObjectNo Format Error.") % locstr).str());
            return false;
        }
        text = e_line->FirstChild()->ToText();
        if(GetIntValues(text->Value(), &m_MapObjNoLayer.GetValue(0, y), width) != width){
            g_Logger.ErrorLog((boost::format("%s : MapObjectNo < Width.") % locstr).str());
            return false;
        }
        // 
        e_line = e_height->FirstChildElement(stry.c_str());
        if(e_line == NULL){
            g_Logger.ErrorLog((boost::format("%s : MapHeight Format Error.") % locstr).str());
            return false;
        }
        text = e_line->FirstChild()->ToText();
        if(GetIntValues(text->Value(), &m_MapHeightLayer.GetValue(0, y), width) != width){
            g_Logger.ErrorLog((boost::format("%s : MapHeight < Width.") % locstr).str());
            return false;
        }
    }

    // אڃGȀzuWǂݍށB
    const TiXmlElement *e_neighbor = e_root->FirstChildElement("NeighborArea");
    if(e_neighbor == NULL){
        g_Logger.ErrorLog((boost::format("%s : Neighbor not found.") % locstr).str());
        return false;
    }
    m_NeighborAreaID.clear();
    m_NeighborArea.clear();
    for(const TiXmlElement *child = e_neighbor->FirstChildElement("Area"); child != NULL; child = child->NextSiblingElement("Area")){
        m_NeighborAreaID.push_back(GetElementAttribute(*child, "id"));
        std::vector<point_t> plist;
        for(;;){
            std::string key = (boost::format("Point%d") % plist.size()).str();
            const TiXmlElement *e = child->FirstChildElement(key.c_str());
            if(e == NULL){
                break;
            }
            point_t p;
            if(e->QueryIntAttribute("x", &p.x) != TIXML_SUCCESS ||
               e->QueryIntAttribute("y", &p.y) != TIXML_SUCCESS){
                g_Logger.ErrorLog((boost::format("%s : zuW̓ǍɎs܂B") % locstr).str());
                return false;
            }
            plist.push_back(p);
        }
        if(plist.size() < NEED_NEIGHBOR_POINT_SIZE){
            const int psize = NEED_NEIGHBOR_POINT_SIZE;
            g_Logger.ErrorLog((boost::format("%s : zuW̐܂(%d)BŒ%dKvłB") % locstr % plist.size() % psize).str());
            return false;
        }
        m_NeighborArea.push_back(std::pair<int, std::vector<point_t> >(-1, plist));
    }

    m_MapChipInstanceLayer = m_MapChipLayer;
    m_MapHeightInstanceLayer = m_MapHeightLayer;
    m_MapObjInstanceLayer.Resize(width, height);
    m_CityXY.clear();
    m_WallXY.clear();
    m_RoadXY.clear();

    int radius = std::max(width / 2, height / 2);
    int centerx = width / 2;
    int centery = height / 2;
    int from = TNeighbor::GetNeighborIndex(0);
    int to = TNeighbor::GetNeighborIndex(radius + 1);
    // Sɋ߂TB΂𒆐SWɂB
    int mode = centery%2;
    for(int i=from; i<to; ++i){
        point_t p = TNeighbor::GetNeighbor(mode, i);
        int x = centerx + p.x;
        int y = centery + p.y;
        if(x >= 0 && x < width && y >= 0 && y < height){
            int obj = m_MapObjLayer.GetValue(x, y);
            int objtype;
            try{
                objtype = data->GetMapObjType(obj);
            }
            catch(std::out_of_range&){
                g_Logger.ErrorLog((boost::format("%s : 㕨Cɖ`̃CfbNX܂B") % locstr).str());
                return false;
            }
            if(objtype == MAPOBJ_CASTLE){
                centerx = x;
                centery = y;
                break;
            }
        }
    }
    m_CenterXY.x = centerx;
    m_CenterXY.y = centery;

    // 㕨̍W𒊏oċLB
    mode = centery%2;
    for(int i=from; i<to; ++i){
        point_t p = TNeighbor::GetNeighbor(mode, i);
        int x = centerx + p.x;
        int y = centery + p.y;
        if(x >= 0 && x < width && y >= 0 && y < height){
            int obj = m_MapObjLayer.GetValue(x, y);
            int objtype;
            try{
                objtype = data->GetMapObjType(obj);
            }
            catch(std::out_of_range&){
                g_Logger.ErrorLog((boost::format("%s : 㕨Cɖ`̃CfbNX܂B") % locstr).str());
                return false;
            }
            switch(objtype){
            case MAPOBJ_CITY:
                m_CityXY.push_back(point_t(x, y));
                break;
            case MAPOBJ_WALL:
                m_WallXY.push_back(point_t(x, y));
                break;
            case MAPOBJ_ROAD:
                m_RoadXY.push_back(point_t(x, y));
                break;
            case MAPOBJ_CASTLE:
                // ͂̂܂܃Rs[B
                m_MapObjInstanceLayer.GetValue(x, y) = obj;
                break;
            default:
                break;
            }
        }
    }
    return true;
}
//---------------------------------------------------------------------------
// [hς݂̃}bvɒ񉻑Ώۂ̃f[^ǂݍŏ㏑B
bool TBattleMap::LoadFromElementSerialize(const TiXmlElement *e_root)
{
    const TiXmlElement *e_obj = e_root->FirstChildElement(XMLELM_MAPOBJINS);
    if(e_obj == NULL){
        g_Logger.ErrorLog((boost::format("%s not found.") % XMLELM_MAPOBJINS).str());
        return false;
    }
    int width = m_MapObjInstanceLayer.GetWidth();
    int height = m_MapObjInstanceLayer.GetHeight();
    for(int y=0; y<height; ++y){
        std::string stry = (boost::format("L%d") % y).str();
        const TiXmlElement *e_line = e_obj->FirstChildElement(stry.c_str());
        if(e_line == NULL){
            g_Logger.ErrorLog((boost::format("%s format error.") % XMLELM_MAPOBJINS).str());
            return false;
        }
        const TiXmlText *text = e_line->FirstChild()->ToText();
        if(GetIntValues(text->Value(), &m_MapObjInstanceLayer.GetValue(0, y), width) != width){
            g_Logger.ErrorLog((boost::format("%s elements < Width.") % XMLELM_MAPOBJINS).str());
            return false;
        }
    }
    return true;
}
//---------------------------------------------------------------------------
bool TBattleMap::SaveToElementSerialize(TiXmlElement *e_root) const
{
    OFWRNLOG("call");
    TiXmlElement e_obj(XMLELM_MAPOBJINS);
    int width = m_MapObjInstanceLayer.GetWidth();
    int height = m_MapObjInstanceLayer.GetHeight();
    for(int y=0; y<height; ++y){
        std::string line;
        IntValuesToString(line, &m_MapObjInstanceLayer.GetValue(0, y), width);
        std::string stry = (boost::format("L%d") % y).str();
        TiXmlElement e_line(stry.c_str());
        InsertTextChild(e_line, line.c_str());
        e_obj.InsertEndChild(e_line);
    }
    e_root->InsertEndChild(e_obj);
    return true;
}
//---------------------------------------------------------------------------
void TBattleMap::SetCity(int count)
{
    SetObj(m_CityXY, count);
}
//---------------------------------------------------------------------------
void TBattleMap::SetWall(int count)
{
    SetObj(m_WallXY, count);
}
//---------------------------------------------------------------------------
void TBattleMap::SetRoad(int count)
{
    SetObj(m_RoadXY, count);
}
//---------------------------------------------------------------------------
int TBattleMap::GetMapObjType(int x, int y) const
{
    int mtype = GetMapType(x, y);
    const TOpenFarenData* data = g_GameMaster.GetData();
    const int gcount = data->GetMapGroundCount();
    if(mtype >= gcount){
        return mtype - gcount;
    }
    else{
        return 0;
    }
}
//---------------------------------------------------------------------------
bool TBattleMap::GetNeighborPlace(int areaindex, std::vector<point_t>& plist) const
{
    int i;
    for(i=0; i<m_NeighborArea.size(); ++i){
        if(m_NeighborArea[i].first == areaindex){
            break;
        }
    }
    if(i == m_NeighborArea.size()){
        return false;
    }
//g_Logger.DebugLog((boost::format("%d neighbor=%d") % i % m_NeighborArea[i].first).str());
    plist = m_NeighborArea[i].second;
    return true;
}
//---------------------------------------------------------------------------
void TBattleMap::RemoveMapObj(const TOpenFarenData *data, int x, int y)
{
    int maptype = GetMapType(x, y);
    if(maptype == MAPOBJ_CITY){
        RemoveMapObjXY(m_CityXY, x, y);
    }
    else if(maptype == MAPOBJ_WALL){
        RemoveMapObjXY(m_WallXY, x, y);
    }
    else if(maptype == MAPOBJ_ROAD){
        RemoveMapObjXY(m_RoadXY, x, y);
    }
    m_MapObjInstanceLayer.GetValue(x, y) = 0;
    UpdateMapType(data, x, y);
}
//---------------------------------------------------------------------------
void TBattleMap::RemoveMapObjSpecify(const TOpenFarenData *data, int x, int y, TMapObject objtype)
{
    int maptype = GetMapObjType(x, y);
    if(maptype == objtype){
        RemoveMapObj(data, x, y);
    }
}
//---------------------------------------------------------------------------
void TBattleMap::SetMapChip(const TOpenFarenData *data, int x, int y, int chip)
{
    m_MapChipInstanceLayer.GetValue(x, y) = chip;
    UpdateMapType(data, x, y);
}
//---------------------------------------------------------------------------
// w肳ꂽ㕨̐߂B
int TBattleMap::CountObj(TMapObject objtype) const
{
    int count = 0;
    for(TValueLayer<int>::ConstIterator itr = m_MapObjInstanceLayer.begin(); itr != m_MapObjInstanceLayer.end(); ++itr){
        count += *itr == objtype;
    }
    return count;
}
//---------------------------------------------------------------------------
// f[W⃔HP[mŉꂽn`񕜂B
void TBattleMap::RestoreAfterBattle()
{
    m_MapHeightInstanceLayer = m_MapHeightLayer;
    m_MapChipInstanceLayer = m_MapChipLayer;
}
//---------------------------------------------------------------------------
// 퓬̂ݎgf[^𐶐B
bool TBattleMap::GenerateTempData(const TOpenFarenData *data)
{
    int width = GetWidth();
    int height = GetHeight();
    m_MapTypeLayer.Resize(width, height);
    for(int y=0; y<height; ++y){
        for(int x=0; x<width; ++x){
            if(!UpdateMapType(data, x, y)){
                g_Logger.ErrorLog("퓬JñGA}bvɎs܂B");
                return false;
            }
        }
    }
    return true;
}
//---------------------------------------------------------------------------
// 퓬̂ݎgf[^jB
void TBattleMap::ClearTempData()
{
    m_MapTypeLayer.Resize(0, 0);
}
//---------------------------------------------------------------------------
void TBattleMap::RemoveMapObjXY(std::vector<point_t>& xylist, int x, int y)
{
    std::vector<point_t>::iterator itr = std::find(xylist.begin(), xylist.end(), point_t(x, y));
    if(itr == xylist.end()){
        g_Logger.ErrorLog("TBattleMap::RemoveMapObjXY() logic error");  // 肦Ȃ͂B
        return;
    }
    xylist.erase(itr);
}
//---------------------------------------------------------------------------
bool TBattleMap::UpdateMapType(const TOpenFarenData *data, int x, int y)
{
    const int gcount = data->GetMapGroundCount();
    int obj = m_MapObjInstanceLayer.GetValue(x, y);
    if(obj){
        // 㕨ꍇBstd::out_of_range̓[hɃ`FbNĂ̂ŋNȂ͂B
        m_MapTypeLayer.GetValue(x, y) = gcount + data->GetMapObjType(obj);
    }
    else{
        // 㕨ꍇB̓[hɃ`FbNĂȂ̂Ń`FbNB
        int chip = m_MapChipInstanceLayer.GetValue(x, y);
        try{
            m_MapTypeLayer.GetValue(x, y) = data->GetMapChipType(chip);
        }
        catch(std::out_of_range&){
            g_Logger.ErrorLog((boost::format("TBattleMap::UpdateMapType() ErrorFn`Cɖ`̃CfbNX%d܂B") % chip).str());
            return false;
        }
    }
    return true;
}
//---------------------------------------------------------------------------
// }bv̏㕨w肳ꂽɂB
void TBattleMap::SetObj(const std::vector<point_t>& xylist, int count)
{
    for(unsigned u=0; u<xylist.size() && count > 0; ++u){
        const point_t& p = xylist[u];
        if(m_MapObjInstanceLayer.GetValue(p.x, p.y) != m_MapObjLayer.GetValue(p.x, p.y)){
            m_MapObjInstanceLayer.GetValue(p.x, p.y) = m_MapObjLayer.GetValue(p.x, p.y);
        }
        --count;
    }
}
//---------------------------------------------------------------------------
