//---------------------------------------------------------------------------
#ifndef bmapH
#define bmapH

#include <vector>
#include <string>
#include <tinyxml.h>
#include "oftypes.h"
#include "valuelayer.h"
#include <algorithm>

//---------------------------------------------------------------------------
enum TMapObject
{
    MAPOBJ_CITY = 1,
    MAPOBJ_WALL,
    MAPOBJ_ROAD,
    MAPOBJ_CASTLE,
    MAPOBJ_MAX
};
//---------------------------------------------------------------------------
class TOpenFarenData;
class TBattleMap
{
private:
    // XMLǂݍ񂾐ÓIȃf[^BQ[ɈB
    TValueLayer<int> m_MapChipLayer;
    TValueLayer<int> m_MapObjLayer;
    TValueLayer<int> m_MapObjNoLayer;
    TValueLayer<int> m_MapHeightLayer;
    // Iɐn`E㕨EBn`ƍ̓f[W^HP[mgȂÓIf[^ƓB
    TValueLayer<int> m_MapChipInstanceLayer;
    TValueLayer<int> m_MapObjInstanceLayer;
    TValueLayer<int> m_MapHeightInstanceLayer;
    // אڃGAID̃XgB
    std::vector<std::string> m_NeighborAreaID;
    // iGAԍAzuWXgj̃XgB
    std::vector<std::pair<int, std::vector<point_t> > > m_NeighborArea;
    // SW
    point_t m_CenterXY;
    // 㕨̍WXgi}bvŜ𑖍̂邽߃[hɐjB
    std::vector<point_t> m_CityXY;
    std::vector<point_t> m_WallXY;
    std::vector<point_t> m_RoadXY;

    // ȉ͐퓬̂ݐf[^B
    TValueLayer<int> m_MapTypeLayer;    // TMoveTypëړRXg^Zʕ␳̃CfbNXB

    static const int NEED_NEIGHBOR_POINT_SIZE = 20; // TOpenFarenGame::AREA_CHARAMAXƓlB
public:
    TBattleMap(int w = 0, int h = 0) : m_MapChipLayer(w, h), m_MapObjLayer(w, h), m_MapObjNoLayer(w, h), m_MapHeightLayer(w, h),
        m_MapChipInstanceLayer(0, 0), m_MapObjInstanceLayer(0, 0), m_MapHeightInstanceLayer(0, 0),
        m_MapTypeLayer(0, 0) {}
    bool LoadFromElement(const TiXmlElement *e_root, const char *locstr, const TOpenFarenData *data);
    bool LoadFromElementSerialize(const TiXmlElement *e_root);
    bool SaveToElementSerialize(TiXmlElement *e_root) const;
    int GetWidth() const { return m_MapChipLayer.GetWidth(); }
    int GetHeight() const { return m_MapChipLayer.GetHeight(); }
    void SetCity(int count);
    void SetWall(int count);
    void SetRoad(int count);
    int GetMapChip(int x, int y) const { return m_MapChipInstanceLayer.GetValue(x, y); }
    int GetMapObj(int x, int y) const { return m_MapObjInstanceLayer.GetValue(x, y); }
    int GetMapHeight(int x, int y) const { return m_MapHeightInstanceLayer.GetValue(x, y); }
    int GetMapType(int x, int y) const { return m_MapTypeLayer.GetValue(x, y); }
    int GetMapObjType(int x, int y) const;
    int GetCityCount() const { return m_CityXY.size(); }
    int GetWallCount() const { return m_WallXY.size(); }
    point_t GetCityXY(int i) const { return m_CityXY[i]; }
    point_t GetWallXY(int i) const { return m_WallXY[i]; }
    point_t GetCenterXY() const { return m_CenterXY; }
    // אڃGÃCfbNX邽߂̃C^[tF[XB
    int GetNeighborCount() const { return m_NeighborAreaID.size(); }
    const std::string& GetNeighborID(int i) const { return m_NeighborAreaID[i]; }
    void SetNeighborIndex(int i, int areaindex) { m_NeighborArea[i].first = areaindex; }
    //int GetNeighborIndex(int i) const { return m_NeighborArea[i].first; } // ̎ɕKvH
    bool GetNeighborPlace(int areaindex, std::vector<point_t>& plist) const;
    void RemoveMapObj(const TOpenFarenData *data, int x, int y);
    void RemoveMapObjSpecify(const TOpenFarenData *data, int x, int y, TMapObject objtype);
    void SetMapChip(const TOpenFarenData *data, int x, int y, int chip);
    int CountObj(TMapObject objtype) const;
    void RestoreAfterBattle();
    bool GenerateTempData(const TOpenFarenData *data);
    void ClearTempData();
private:
    void RemoveMapObjXY(std::vector<point_t>& xylist, int x, int y);
    bool UpdateMapType(const TOpenFarenData *data, int x, int y);
    void SetObj(const std::vector<point_t>& xylist, int count);
};

#endif
