//---------------------------------------------------------------------------
#include "charalistpane.h"
#include "ofimage.h"
#include "ofini.h"
#include "gui.h"
#include "gamemaster.h"
#include "gcnskinmanager.h"
#include "drawutil.h"

#include "boostformat.h"
#include "logger.h"

#define FONT_SIZE           12

//---------------------------------------------------------------------------
TCharaListPane::TCharaListPane() :
    m_XCount(5),
    m_YCount(4),
    m_CharaList(NULL),
    m_Browsed(-1),
    m_Selected(-1),
    m_MouseHandler(NULL)
{
    m_drawer = g_GuiContainerList.GetDrawer(FONT_SIZE);
    setWidth((m_CharaWidth + m_CharaMargin)*m_XCount);
    setHeight((m_CharaHeight + m_CharaMargin)*m_YCount);
    addMouseListener(this);
}
//---------------------------------------------------------------------------
void TCharaListPane::SetYCount(int count)
{
    m_YCount = count;
    setHeight((m_CharaHeight + m_CharaMargin)*m_YCount);
}
//---------------------------------------------------------------------------
void TCharaListPane::draw(gcn::Graphics *g)
{
    g->setColor(getBackgroundColor());
    gcn::Rectangle r(0, 0, getWidth(), getHeight());
    g->fillRectangle(r);

    if(m_CharaList == NULL){
        return;
    }

    const TOpenFarenData* data = g_GameMaster.GetData();
    //const TOpenFarenGame* game = g_GameMaster.GetGame();
    for(int i = 0; i < m_CharaList->size(); i++){
        if(m_CharaHidden == NULL || !(*m_CharaHidden)[i]){
            int x = (i%m_XCount)*(m_CharaWidth + m_CharaMargin);
            int y = (i/m_XCount)*(m_CharaHeight + m_CharaMargin);
            //g_Logger.DebugLog((boost::format("%d") % area.m_charalist[i].m_type).str());
            //g_Logger.DebugLog((boost::format("%d") % data->GetCharacter(area.m_charalist[i].m_type).m_chipindex).str());
            //g_Logger.DebugLog(data->GetCharacter(area.m_charalist[i].m_type).m_chipfile);
            const TCharacterInstance& chara = (*m_CharaList)[i];
            g_GameMaster.GetImage()->DrawCharaChip(g, x, y, data->GetCharacter(chara.m_type).m_chipindex);
            if(chara.m_actioned){
                m_drawer->DrawString(g, x+11, y+14, "End", gcn::Color(0, 0, 0));
                m_drawer->DrawString(g, x+10, y+14, "End", gcn::Color(255, 255, 255));
            }
        }
    }
    if(m_Selected != -1){
        int x = (m_Selected%m_XCount)*(m_CharaWidth + m_CharaMargin);
        int y = (m_Selected/m_XCount)*(m_CharaHeight + m_CharaMargin);
        g->setColor(g_GameMaster.GetSkinManager()->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_SELFRAMECOLOR));
        g_DrawUtil.DrawRectangle(g, gcn::Rectangle(x,   y,   m_CharaWidth-1, m_CharaHeight-1));
        g_DrawUtil.DrawRectangle(g, gcn::Rectangle(x+1, y+1, m_CharaWidth-3, m_CharaHeight-3));
        g_DrawUtil.DrawRectangle(g, gcn::Rectangle(x+2, y+2, m_CharaWidth-5, m_CharaHeight-5));
        g_DrawUtil.DrawRectangle(g, gcn::Rectangle(x+3, y+3, m_CharaWidth-7, m_CharaHeight-7));
    }
}
//---------------------------------------------------------------------------
void TCharaListPane::SetCharaList(const TCharacterInstanceList* charalist, const std::vector<int>* hidden)
{
    m_CharaList = charalist;
    m_CharaHidden = hidden;
    m_Browsed = -1;
}
//---------------------------------------------------------------------------
int TCharaListPane::GetCharaIndexAt(int x, int y) const
{
    int cx = x / (m_CharaWidth + m_CharaMargin);
    int cy = y / (m_CharaHeight + m_CharaMargin);
    return cx + cy*m_XCount;
}
//---------------------------------------------------------------------------
void TCharaListPane::mousePressed(gcn::MouseEvent &mouseEvent)
{
    if(m_MouseHandler != NULL){
        m_MouseHandler->HandleMousePressed(mouseEvent);
    }
}
//---------------------------------------------------------------------------
void TCharaListPane::mouseMoved(gcn::MouseEvent &mouseEvent)
{
    int cx = (mouseEvent.getX() / (m_CharaWidth + m_CharaMargin));
    int cy = (mouseEvent.getY() / (m_CharaHeight + m_CharaMargin));
    if(m_CharaList == NULL){
        return;
    }
    if(cx + cy*m_XCount < m_CharaList->size()){
        m_Browsed = cx + cy*m_XCount;
    }
    if(m_MouseHandler != NULL){
        m_MouseHandler->HandleMouseMoved(mouseEvent);
    }
}
//---------------------------------------------------------------------------
