//---------------------------------------------------------------------------
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/shared_ptr.hpp>
#include "charastatus.h"
#include "ofimage.h"
#include "ofini.h"
#include "gui.h"
#include "gamemaster.h"
#include "gcnskinmanager.h"
#include "drawutil.h"

#include "boostformat.h"
#include "logger.h"

#define CORNER_MARGIN 8
//#define FONT_SIZE           12
//#define FONT_SIZE_DEFAULT   16  // GUIigĂtHgƓTCYɂ邱ƁiȂj
#define BOOST_COLOR (gcn::Color(128,255,255))

namespace
{
    void DrawRank(gcn::Graphics *g, gcn::Color c, int x, int y, int rank, TGcnCharDrawer *drawer)
    {
        if(rank < 0 || rank > 5){
            g_Logger.ErrorLog((boost::format("DrawRank(%d, %d, %d) invalid parameter rank.") % x % y % rank).str());
            return;
        }
        const char rankchr[6] = {'E', 'D', 'C', 'B', 'A', 'S'};
        drawer->DrawString(g, x, y, "Rank", c);
        drawer->DrawChar(g, x + (TGuiContainerList::FONT_SIZE_SMALL * 5)/2, y, rankchr[rank], c);
    }

    void DrawExp(gcn::Graphics *g, gcn::Color c, int x, int y, int exp, TGcnCharDrawer *drawer)
    {
        std::string expstr = (boost::format("Exp %3d") % exp).str();
        drawer->DrawString(g, x, y, expstr.c_str(), c);
    }
}
//---------------------------------------------------------------------------
TGcnCharaStatusPane::TGcnCharaStatusPane()
{
    m_drawer = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_SMALL);
    setSize(160, 96);
    setForegroundColor(gcn::Color(255,255,255));
    setBackgroundColor(g_GameMaster.GetSkinManager()->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE));
}
//---------------------------------------------------------------------------
void TGcnCharaStatusPane::draw(gcn::Graphics *g)
{
    static const gcn::Color boost_color = BOOST_COLOR;
    static const int left = 2;
    static const int fontsize = TGuiContainerList::FONT_SIZE_SMALL;

    gcn::Rectangle r = getDimension();
    r.x = r.y = 0;
    g->setColor(getBackgroundColor());
    g->fillRectangle(r);
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(m_chara.m_type);
    gcn::Color c = getForegroundColor();
    m_drawer->DrawString(g, left, 0, charadata.m_name.c_str(), c);
    m_drawer->DrawString(g, left+1, 0, charadata.m_name.c_str(), c);
    m_drawer->DrawString(g, left, fontsize+1, data->GetRace(charadata.m_race).m_name.c_str(), c);
    DrawRank(g, c, left, fontsize*2+2, m_chara.m_rank, m_drawer);
    DrawExp(g, c, left + fontsize*4, fontsize*2+2, m_chara.m_experience, m_drawer);

    // 
    int sesx = left;
    int sesy = fontsize*3+3;
    for(TSystemEffectStatusList::const_iterator itr = m_chara.m_seslist.begin(); itr != m_chara.m_seslist.end(); ++itr){
        const std::string& sesname = data->GetSystemEffectName(itr->m_type);
        m_drawer->DrawString(g, sesx, sesy, sesname.c_str(), c);
        sesx += fontsize / 2 * (sesname.size() + 2);
    }
    for(std::vector<int>::const_iterator itr = m_chara.m_addstrspe.begin(); itr != m_chara.m_addstrspe.end(); ++itr){
        const TSpecial& spe = data->GetSpecial(*itr);
        m_drawer->DrawString(g, sesx, sesy, spe.m_addstr.c_str(), c);
        sesx += fontsize / 2 * (spe.m_addstr.size() + 1);
    }
    if(m_chara.m_attackcount_plus > 0){
        m_drawer->DrawString(g, sesx, sesy, "U", c);
        sesx += fontsize / 2 * 7;
    }
    else if(m_chara.m_attackcount_plus < 0){
        m_drawer->DrawString(g, sesx, sesy, "U", c);
        sesx += fontsize / 2 * 7;
    }
    if(m_chara.m_try_runaway){
        m_drawer->DrawString(g, sesx, sesy, "", c);
    }

    std::string hpmp = (boost::format("HP %3d/%3d   MP %3d/%3d") % m_chara.m_hp % m_chara.m_hpmax_rank % m_chara.m_mp % m_chara.m_mpmax_rank).str();
    m_drawer->DrawString(g, left, fontsize*4+4, hpmp.c_str(), c);

    std::string para1 = (boost::format("U %3d") % (m_chara.m_pa_rank + m_chara.m_pa_plus)).str();
    std::string para2 = (boost::format("h %3d") % (m_chara.m_pd_rank + m_chara.m_pd_plus)).str();
    std::string para3 = (boost::format("Z %3d") % (m_chara.m_skill_rank + m_chara.m_skill_plus)).str();
    std::string para4 = (boost::format(" %3d") % (m_chara.m_speed_rank + m_chara.m_speed_plus)).str();
    std::string para5 = (boost::format(" %3d") % (m_chara.m_ma_rank + m_chara.m_ma_plus)).str();
    std::string para6 = (boost::format("R %3d") % (m_chara.m_md_rank + m_chara.m_md_plus)).str();
    m_drawer->DrawString(g, left, fontsize*5+5, para1.c_str(), m_chara.m_pa_plus ? boost_color : c);
    m_drawer->DrawString(g, left + fontsize*4+4, fontsize*5+5, para2.c_str(), m_chara.m_pd_plus ? boost_color : c);
    m_drawer->DrawString(g, left + fontsize*8+8, fontsize*5+5, para3.c_str(), m_chara.m_skill_plus ? boost_color : c);
    m_drawer->DrawString(g, left, fontsize*6+6, para4.c_str(), m_chara.m_speed_plus ? boost_color : c);
    m_drawer->DrawString(g, left + fontsize*4+4, fontsize*6+6, para5.c_str(), m_chara.m_ma_plus ? boost_color : c);
    m_drawer->DrawString(g, left + fontsize*8+8, fontsize*6+6, para6.c_str(), m_chara.m_md_plus ? boost_color : c);

    g_GameMaster.GetImage()->DrawCharaChip(g, left + fontsize*10, 0, charadata.m_chipindex);
}
//---------------------------------------------------------------------------
TCharaStatusWindow::TCharaStatusWindow()
    : m_Shown(false)
{
    setOpaque(false);
}
//---------------------------------------------------------------------------
void TCharaStatusWindow::Show(const TCharacterInstance& chara)
{
    m_CharaStatusPane.SetCharaStatus(chara);

    if(!m_Shown){
        m_CharaStatusPane.setPosition(0, 0);
        add(&m_CharaStatusPane);

        setTitleBarHeight(2);
        //setBaseColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_WINDOW));
        g_GameMaster.GetSkinManager()->ApplyWindowSkin(this);
        resizeToContent();
        MoveToCorner();
        m_Shown = true;
    }
    setVisible(true);
}
//---------------------------------------------------------------------------
void TCharaStatusWindow::MoveToCorner()
{
    gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
    int topw = top->getWidth();
    int toph = top->getHeight();
    if(getX() != CORNER_MARGIN){
        setX(CORNER_MARGIN);
    }
    else{
        setX(topw - getWidth() - CORNER_MARGIN);
    }
    setY(toph - getHeight() - CORNER_MARGIN);
}
//---------------------------------------------------------------------------
void TCharaStatusWindow::mouseMoved(gcn::MouseEvent &mouseEvent)
{
    gcn::Window::mouseMoved(mouseEvent);
    MoveToCorner();
}
//---------------------------------------------------------------------------
TGcnCharaStatusFullPane::TGcnCharaStatusFullPane()
{
    m_drawer = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_SMALL);
    m_drawerBig = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_NORMAL);
    setSize(FULLWINDOW_NOFACEWIDTH, FULLWINDOW_NOHEROHEIGHT);
    setForegroundColor(gcn::Color(255,255,255));
    setBackgroundColor(g_GameMaster.GetSkinManager()->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE));
}
//---------------------------------------------------------------------------
void TGcnCharaStatusFullPane::SetCharaStatus(const TCharacterInstance& chara)
{
    m_chara = chara;
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(m_chara.m_type);
    if(charadata.m_faceindex != -1){
        setWidth(FULLWINDOW_NOFACEWIDTH + 96);
    }
    else{
        setWidth(FULLWINDOW_NOFACEWIDTH);
    }
    if(charadata.m_hero || charadata.m_master){
        setHeight(FULLWINDOW_NOHEROHEIGHT + 24);
    }
    else{
        setHeight(FULLWINDOW_NOHEROHEIGHT);
    }
}
//---------------------------------------------------------------------------
void TGcnCharaStatusFullPane::draw(gcn::Graphics *g)
{
    //if(m_drawer == NULL){
    //    return;
    //}
    gcn::Rectangle r = getDimension();
    r.x = r.y = 0;
    g->setColor(getBackgroundColor());
    g->fillRectangle(r);
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(m_chara.m_type);
    g_GameMaster.GetImage()->DrawCharaChip(g, 0, 0, charadata.m_chipindex);
    if(charadata.m_faceindex != -1){
        g_GameMaster.GetImage()->DrawFace(g, FULLWINDOW_NOFACEWIDTH, 0, charadata.m_faceindex);
    }
    gcn::Color c = getForegroundColor();
    const int fontsize = TGuiContainerList::FONT_SIZE_SMALL;
    m_drawerBig->DrawString(g, 40, 0, charadata.m_name.c_str(), c);
    m_drawerBig->DrawString(g, 41, 0, charadata.m_name.c_str(), c);
    m_drawer->DrawString(g, 40, 18, data->GetRace(charadata.m_race).m_name.c_str(), c);
    //DrawRank(g, c, fontsize*10, 18, m_chara.m_rank, m_drawer);
    //DrawExp(g, c, fontsize*14, 18, m_chara.m_experience, m_drawer);
    DrawRank(g, c, 152, 18, m_chara.m_rank, m_drawer);
    DrawExp(g, c, 200, 18, m_chara.m_experience, m_drawer);
    if(charadata.m_master){
        m_drawer->DrawString(g, 198, 4, "}X^[", c);
    }
    r.x = 4;
    r.y = 40;
    r.width = fontsize*8;
    r.height = fontsize*10 + 24;
    DrawFrame(g, r, "Xe[^X", m_drawer);

    std::string para1 = (boost::format("HP     %3d/%3d") % m_chara.m_hp % m_chara.m_hpmax_rank).str();
    std::string para2 = (boost::format("MP     %3d/%3d") % m_chara.m_mp % m_chara.m_mpmax_rank).str();
    std::string para3 = (boost::format("U %3d/%3d") % (m_chara.m_pa_rank + m_chara.m_pa_plus) % m_chara.m_pa_rank).str();
    std::string para4 = (boost::format("h %3d/%3d") % (m_chara.m_pd_rank + m_chara.m_pd_plus) % m_chara.m_pd_rank).str();
    std::string para5 = (boost::format("Z   %3d/%3d") % (m_chara.m_skill_rank + m_chara.m_skill_plus) % m_chara.m_skill_rank).str();
    std::string para6 = (boost::format("f %3d/%3d") % (m_chara.m_speed_rank + m_chara.m_speed_plus) % m_chara.m_speed_rank).str();
    std::string para7 = (boost::format("   %3d/%3d") % (m_chara.m_ma_rank + m_chara.m_ma_plus) % m_chara.m_ma_rank).str();
    std::string para8 = (boost::format("R %3d/%3d") % (m_chara.m_md_rank + m_chara.m_md_plus) % m_chara.m_md_rank).str();
    std::string para9 = (boost::format("HPĐ %3d/%3d") % (charadata.m_hprecover + m_chara.m_hprecover_plus) % charadata.m_hprecover).str();
    std::string para10 = (boost::format("MPĐ %3d/%3d") % (charadata.m_mprecover + m_chara.m_mprecover_plus) % charadata.m_mprecover).str();
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize+1, para1.c_str(), c);
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize*2+2, para2.c_str(), c);
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize*3+3, para3.c_str(), c);
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize*4+4, para4.c_str(), c);
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize*5+5, para5.c_str(), c);
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize*6+6, para6.c_str(), c);
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize*7+7, para7.c_str(), c);
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize*8+8, para8.c_str(), c);
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize*9+9, para9.c_str(), c);
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize*10+10, para10.c_str(), c);

    // @x`
    r.x = 4;
    r.y += r.height + 12;
    r.height = fontsize*2 + 20;
    DrawFrame(g, r, "@", m_drawer);
    //m_drawer->DrawString(g, r.x + 8, r.y + FONT_SIZE, "   y  ", c);
    for(int i=0; i<data->GetMagicGroupCount(); i++){
        m_drawer->DrawString(g, r.x + 8 + i*14, r.y + fontsize, data->GetMagicGroup(i).m_name.c_str(), c);
        static const char *mlvstr[6] = {"~", "c", "b", "a", "`", "r"};
        int mlv = charadata.m_magicrank[i];
        if(mlv >= 0 && mlv <= 5){
            m_drawer->DrawString(g, r.x + 8 + i*14, r.y + fontsize*2, mlvstr[mlv], c);
        }
    }

    // UƕKEZ`
    r.x += r.width + 8;
    r.y = fontsize*7;
    r.width = 140;
    r.height = fontsize*7;
    DrawFrame(g, r, "U", m_drawer);
    std::string atklist = "(";
    int atkcount = 0;
    for(unsigned u=0; u<charadata.m_attacklist.size(); u++){
        if(charadata.m_attacklist[u].m_special != -1){
            const TSpecial& spe = data->GetSpecial(charadata.m_attacklist[u].m_special);
            const TAttribute& attr = data->GetAttribute(spe.m_attribute);
            if(atkcount){
                if(!(atkcount%3)){
                    atklist += "\n";
                }
                atklist += " ";
            }
            atklist += attr.m_shortname;
            atkcount++;
        }
    }
    std::string atk = "U~";
    static const char *atkstr[10] = {"O", "P", "Q", "R", "S", "T", "U", "V", "W", "X"};
    if(atkcount + m_chara.m_attackcount_plus >= 0 && atkcount + m_chara.m_attackcount_plus <= 9){
        atk += atkstr[atkcount + m_chara.m_attackcount_plus];
    }
    gcn::Color atkcolor;
    if(m_chara.m_attackcount_plus > 0){
        atkcolor = BOOST_COLOR;
    }
    else if(m_chara.m_attackcount_plus < 0){
        atkcolor = gcn::Color(255, 0, 0);
    }
    else{
        atkcolor = gcn::Color(255, 255, 255);
    }
    m_drawer->DrawString(g, r.x + 8, r.y + fontsize+1, atk.c_str(), atkcolor);
    if(atkcount){
        atklist += ")";
        gcn::Rectangle r_atk(r.x + 8, r.y + fontsize*2, fontsize*10, fontsize*2 + 2);
        m_drawer->DrawStringOnRect(g, r.x + 8, r.y + fontsize*2+2, r_atk, 1, atklist.c_str(), c);
    }
    if(charadata.m_specialindex != -1){
        const TSpecial& special = data->GetSpecial(charadata.m_specialindex);
        std::string spestr = special.m_name + "~";
        if(charadata.m_specialcount <= 9){
            spestr += atkstr[charadata.m_specialcount];
        }
        m_drawer->DrawString(g, r.x + 8, r.y + fontsize*5+7, spestr.c_str(), c);
    }

    // ړ^Cv`
    if(charadata.m_movetype >= 0 && charadata.m_movetype < data->GetMoveTypeCount()){
        std::string movetypestr = (boost::format("ړ^Cv: %s") % data->GetMoveType(m_chara.m_movetype).m_name.c_str()).str();
        m_drawer->DrawString(g, r.x, fontsize*4, movetypestr.c_str(), c);
    }
    else{
        g_Logger.WarningLog((boost::format("TGcnCharaStatusFullPane::draw(): invalid movetype %d (id:%s)") % charadata.m_movetype % charadata.m_id.c_str()).str());
    }
    std::string movestr = (boost::format("ړ: %d") % charadata.m_movable).str();
    m_drawer->DrawString(g, r.x + fontsize*2, fontsize*5 + 1, movestr.c_str(), c);

    // `ɋEア̕`
    r.y += r.height + fontsize;
    r.height = fontsize*16 + 16;
    DrawFrame(g, r, "", m_drawer);
    int attry = r.y + fontsize + 1;
    for(int i=0; i<data->GetAttributeCount(); i++){
        int reg = m_chara.m_resistance[i];
        const TAttribute& attr = data->GetAttribute(i);
        std::string str = attr.m_name;
        bool show = true;
        if(reg == RES_WEAK){
            str += "Ɏア";
        }
        else if(reg == RES_STRONG){
            str += "ɋ";
        }
        else if(reg == RES_ABSORB){
            str += "z";
        }
        else{
            show = false;
        }
        if(show){
            m_drawer->DrawString(g, r.x + 8, attry, str.c_str(), c);
            attry += fontsize + 1;
        }
    }

    if(charadata.m_hero || charadata.m_master){
        std::string win = (boost::format("j: %4d") % m_chara.m_wincount).str();
        m_drawer->DrawString(g, r.x, FULLWINDOW_NOHEROHEIGHT - 4, win.c_str(), c);
    }
    if(charadata.m_hero){
        std::string cost = (boost::format("lޔ: %4d") % charadata.m_turncost).str();
        m_drawer->DrawString(g, r.x, FULLWINDOW_NOHEROHEIGHT + 10, cost.c_str(), c);
    }
}
//---------------------------------------------------------------------------
TCharaStatusFullWindow::TCharaStatusFullWindow()
    : m_Shown(false)
{
    setOpaque(false);
}
//---------------------------------------------------------------------------
bool TCharaStatusFullWindow::Show(const TCharacterInstance& chara)
{

    m_CharaStatusFullPane.SetCharaStatus(chara);

    if(!m_Shown){
        m_CharaStatusFullPane.setPosition(0, 0);
        add(&m_CharaStatusFullPane);

        //m_ButtonOk.setCaption("OK");
        //m_ButtonOk.setWidth(48);
        //m_ButtonOk.setHeight(24);
        //m_ButtonOk.setPosition(m_CharaStatusFullPane.getWidth()-m_ButtonOk.getWidth(), m_CharaStatusFullPane.getHeight());
        //g_Ini.ApplyButtonEnabledColor(m_ButtonOk);
        //m_ButtonOk.addActionListener(this);
        //add(&m_ButtonOk);

        setTitleBarHeight(2);
	    //setBaseColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_WINDOW));
        g_GameMaster.GetSkinManager()->ApplyWindowSkin(this);
        m_Shown = true;
    }
    resizeToContent();
    setVisible(true);
    return true;
}
//---------------------------------------------------------------------------
/*void TCharaStatusFullWindow::action(const gcn::ActionEvent& actionEvent)
{
    if(actionEvent.getSource() == &m_ButtonOk){
        setVisible(false);
    }
}*/
//---------------------------------------------------------------------------
void TCharaStatusFullWindow::mousePressed(gcn::MouseEvent &mouseEvent)
{
    setVisible(false);
    mouseEvent.consume();
}
//---------------------------------------------------------------------------
TCharaMoveWindow::TCharaMoveWindow() :
    m_Shown(false),
    m_Browsed(false),
    m_LeftAreaIndex(-1),
    m_RightAreaIndex(-1)
{
    setTitleBarHeight(20);
}
//---------------------------------------------------------------------------
void TCharaMoveWindow::Show(int larea, const TCharacterInstanceList& lcharalist, int rarea, const TCharacterInstanceList& rcharalist)
{
    m_LeftAreaIndex = larea;
    m_LeftTopCharaList = lcharalist;
    m_LeftToRightFlag.clear();
    m_LeftToRightFlag.resize(lcharalist.size());
    m_LeftBottomCharaList.clear();
    m_LeftTopPane.SetCharaList(&m_LeftTopCharaList, &m_LeftToRightFlag);
    m_LeftBottomPane.SetCharaList(&m_LeftBottomCharaList, NULL);

    m_RightAreaIndex = rarea;
    m_RightTopCharaList.clear();
    m_RightBottomCharaList = rcharalist;
    m_RightToLeftFlag.clear();
    m_RightToLeftFlag.resize(rcharalist.size());
    m_RightTopPane.SetCharaList(&m_RightTopCharaList, NULL);
    m_RightBottomPane.SetCharaList(&m_RightBottomCharaList, &m_RightToLeftFlag);

    if(!m_Shown){
        const TGcnSkinManager *skinmng = g_GameMaster.GetSkinManager();
        gcn::Color dark = skinmng->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE);

        m_LeftTopPane.setPosition(0, 20);
        m_LeftTopPane.setBackgroundColor(dark);
        m_LeftTopPane.SetMouseHandler(this);
        add(&m_LeftTopPane);

        m_LeftBottomPane.setPosition(0, m_LeftTopPane.getY() + m_LeftTopPane.getHeight() + 4);
        m_LeftBottomPane.setBackgroundColor(dark);
        m_LeftBottomPane.SetMouseHandler(this);
        add(&m_LeftBottomPane);

        m_RightTopPane.setPosition(m_LeftTopPane.getWidth() + 16, m_LeftTopPane.getY());
        m_RightTopPane.setBackgroundColor(dark);
        m_RightTopPane.SetMouseHandler(this);
        add(&m_RightTopPane);

        m_RightBottomPane.setPosition(m_LeftBottomPane.getWidth() + 16, m_LeftBottomPane.getY());
        m_RightBottomPane.setBackgroundColor(dark);
        m_RightBottomPane.SetMouseHandler(this);
        add(&m_RightBottomPane);

        int buttony = m_LeftBottomPane.getY() + m_LeftBottomPane.getHeight() + 4;
        m_ButtonAllRight.setCaption("SĉE");
        m_ButtonAllRight.setSize(80, 24);
        m_ButtonAllRight.setPosition(4, buttony);
        m_ButtonAllRight.addActionListener(this);
        skinmng->ApplyButtonEnabled(&m_ButtonAllRight);
        add(&m_ButtonAllRight);

        m_ButtonAllLeft.setCaption("Sč");
        m_ButtonAllLeft.setSize(80, 24);
        m_ButtonAllLeft.setPosition(m_RightBottomPane.getX() + m_RightBottomPane.getWidth() - m_ButtonAllLeft.getWidth() - 4, buttony);
        m_ButtonAllLeft.addActionListener(this);
        skinmng->ApplyButtonEnabled(&m_ButtonAllLeft);
        add(&m_ButtonAllLeft);

        m_ButtonOk.setCaption("m");
        m_ButtonOk.setSize(80, 24);
        m_ButtonOk.setPosition(m_LeftBottomPane.getX() + m_LeftBottomPane.getWidth() - m_ButtonOk.getWidth() - 4, buttony);
        skinmng->ApplyButtonEnabled(&m_ButtonOk);
        add(&m_ButtonOk);

        m_ButtonCancel.setCaption("~");
        m_ButtonCancel.setSize(80, 24);
        m_ButtonCancel.setPosition(m_RightBottomPane.getX() + 4, buttony);
        skinmng->ApplyButtonEnabled(&m_ButtonCancel);
        add(&m_ButtonCancel);

        //setBackgroundColor(g_Ini.GetCharacterMoveWindowChildrenAreaColor());
        skinmng->ApplyWindowSkin(this);
        resizeToContent();
        m_Shown = true;
    }
    m_Browsed = false;
    setVisible(true);
}
//---------------------------------------------------------------------------
void TCharaMoveWindow::GetLeftToRight(std::vector<int>& indice) const
{
    indice.clear();
    for(unsigned u=0; u<m_LeftToRightFlag.size(); u++){
        if(m_LeftToRightFlag[u]){
            indice.push_back(u);
        }
    }
}
//---------------------------------------------------------------------------
void TCharaMoveWindow::GetRightToLeft(std::vector<int>& indice) const
{
    indice.clear();
    for(unsigned u=0; u<m_RightToLeftFlag.size(); u++){
        if(m_RightToLeftFlag[u]){
            indice.push_back(u);
        }
    }
}
//---------------------------------------------------------------------------
const TCharacterInstance* TCharaMoveWindow::GetBrowsedCharacter() const
{
    if(m_Browsed){
        return &m_BrowsedCharacter;
    }
    else{
        return NULL;
    }
}
//---------------------------------------------------------------------------
void TCharaMoveWindow::draw(gcn::Graphics *g)
{
    TGcnWindow::draw(g);
    //g->setColor(getBackgroundColor());
    gcn::Rectangle r = getChildrenArea();
    //g->fillRectangle(r);
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    g->setColor(gcn::Color(255,255,255));
    if(m_LeftAreaIndex != -1){
        const TAreaInstance& larea = game->GetAreaInstance(m_LeftAreaIndex);
        g->drawText(larea.m_name.c_str(), r.x + m_LeftTopPane.getX(),     r.y + 2);
        g->drawText(larea.m_name.c_str(), r.x + m_LeftTopPane.getX() + 1, r.y + 2);
    }
    if(m_RightAreaIndex != -1){
        const TAreaInstance& rarea = game->GetAreaInstance(m_RightAreaIndex);
        g->drawText(rarea.m_name.c_str(), r.x + m_RightTopPane.getX(),     r.y + 2);
        g->drawText(rarea.m_name.c_str(), r.x + m_RightTopPane.getX() + 1, r.y + 2);
    }
    g->drawText("ړ", 4, 2);
    g->drawText("ړ", 5, 2);
}
//---------------------------------------------------------------------------
void TCharaMoveWindow::HandleMousePressed(gcn::MouseEvent &mouseEvent)
{
    if(mouseEvent.getSource() == &m_LeftTopPane){
        mouseEvent.consume();
        int index = m_LeftTopPane.GetCharaIndexAt(mouseEvent.getX(), mouseEvent.getY());
        if(index >= m_LeftTopCharaList.size()){
            return;
        }
        if(m_LeftTopCharaList[index].m_actioned){
            return;
        }
        if(m_RightTopCharaList.size() + m_RightBottomCharaList.size() >= TOpenFarenGame::AREACHARA_MAX){
            return;
        }
        if(m_LeftToRightFlag[index]){
            return;
        }
        m_LeftToRightFlag[index] = 1;
        m_RightTopCharaList.push_back(m_LeftTopCharaList[index]);
    }
    else if(mouseEvent.getSource() == &m_LeftBottomPane){
        mouseEvent.consume();
        int index = m_LeftBottomPane.GetCharaIndexAt(mouseEvent.getX(), mouseEvent.getY());
        if(index >= m_LeftBottomCharaList.size()){
            return;
        }
        int count = 0;
        int flagindex = -1;
        for(unsigned u=0; u<m_RightToLeftFlag.size(); u++){
            if(m_RightToLeftFlag[u]){
                if(count == index){
                    flagindex = u;
                    break;
                }
                count++;
            }
        }
        if(flagindex == -1){
            g_Logger.ErrorLog("TCharaMoveWindow::mousePressed():vOɃoO");
            return;
        }
        m_RightToLeftFlag[flagindex] = 0;
        m_LeftBottomCharaList.erase(m_LeftBottomCharaList.begin() + index);
    }
    else if(mouseEvent.getSource() == &m_RightTopPane){
        mouseEvent.consume();
        int index = m_RightTopPane.GetCharaIndexAt(mouseEvent.getX(), mouseEvent.getY());
        if(index >= m_RightTopCharaList.size()){
            return;
        }
        int count = 0;
        int flagindex = -1;
        for(unsigned u=0; u<m_LeftToRightFlag.size(); u++){
            if(m_LeftToRightFlag[u]){
                if(count == index){
                    flagindex = u;
                    break;
                }
                count++;
            }
        }
        if(flagindex == -1){
            g_Logger.ErrorLog("TCharaMoveWindow::mousePressed():vOɃoO");
            return;
        }
        m_LeftToRightFlag[flagindex] = 0;
        m_RightTopCharaList.erase(m_RightTopCharaList.begin() + index);
    }
    else if(mouseEvent.getSource() == &m_RightBottomPane){
        mouseEvent.consume();
        int index = m_RightBottomPane.GetCharaIndexAt(mouseEvent.getX(), mouseEvent.getY());
        if(index >= m_RightBottomCharaList.size()){
            return;
        }
        if(m_RightBottomCharaList[index].m_actioned){
            return;
        }
        if(m_LeftTopCharaList.size() + m_LeftBottomCharaList.size()  >= TOpenFarenGame::AREACHARA_MAX){
            return;
        }
        if(m_RightToLeftFlag[index]){
            return;
        }
        m_RightToLeftFlag[index] = 1;
        m_LeftBottomCharaList.push_back(m_RightBottomCharaList[index]);
    }
}
//---------------------------------------------------------------------------
void TCharaMoveWindow::HandleMouseMoved(gcn::MouseEvent &mouseEvent)
{
    // }EXɂL̃CX^XRs[B̃R[h̏悤͂Ȃ̂ƂCB
    if(mouseEvent.getSource() == &m_LeftTopPane){
        int i = m_LeftTopPane.GetBrowsed();
        if(i != -1 && i < m_LeftTopCharaList.size()){
            m_BrowsedCharacter = m_LeftTopCharaList[i];
            m_Browsed = true;
        }
    }
    else if(mouseEvent.getSource() == &m_LeftBottomPane){
        int i = m_LeftBottomPane.GetBrowsed();
        if(i != -1 && i < m_LeftBottomCharaList.size()){
            m_BrowsedCharacter = m_LeftBottomCharaList[i];
            m_Browsed = true;
        }
    }
    else if(mouseEvent.getSource() == &m_RightTopPane){
        int i = m_RightTopPane.GetBrowsed();
        if(i != -1 && i < m_RightTopCharaList.size()){
            m_BrowsedCharacter = m_RightTopCharaList[i];
            m_Browsed = true;
        }
    }
    else if(mouseEvent.getSource() == &m_RightBottomPane){
        int i = m_RightBottomPane.GetBrowsed();
        if(i != -1 && i < m_RightBottomCharaList.size()){
            m_BrowsedCharacter = m_RightBottomCharaList[i];
            m_Browsed = true;
        }
    }
}
//---------------------------------------------------------------------------
void TCharaMoveWindow::action(const gcn::ActionEvent& actionEvent)
{
    if(actionEvent.getSource() == &m_ButtonAllRight){
        // E獶̈ړSLZ
        m_RightToLeftFlag.assign(m_RightToLeftFlag.size(), 0);
        m_LeftBottomCharaList.clear();
        // SĂ̍LoEɈړ
        for(unsigned u = 0; u<m_LeftTopCharaList.size(); u++){
            if(m_RightTopCharaList.size() + m_RightBottomCharaList.size() >= TOpenFarenGame::AREACHARA_MAX){
                break;
            }
            if(!m_LeftToRightFlag[u] && !m_LeftTopCharaList[u].m_actioned){
                m_LeftToRightFlag[u] = 1;
                m_RightTopCharaList.push_back(m_LeftTopCharaList[u]);
            }
        }
    }
    else if(actionEvent.getSource() == &m_ButtonAllLeft){
        // ËړSLZ
        m_LeftToRightFlag.assign(m_LeftToRightFlag.size(), 0);
        m_RightTopCharaList.clear();
        // SẲELo荶Ɉړ
        for(unsigned u = 0; u<m_RightBottomCharaList.size(); u++){
            if(m_LeftTopCharaList.size() + m_LeftBottomCharaList.size()  >= TOpenFarenGame::AREACHARA_MAX){
                break;
            }
            if(!m_RightToLeftFlag[u] && !m_RightBottomCharaList[u].m_actioned){
                m_RightToLeftFlag[u] = 1;
                m_LeftBottomCharaList.push_back(m_RightBottomCharaList[u]);
            }
        }
    }
}
//---------------------------------------------------------------------------
// 퓬ʂ̉Eɕ\B
void DrawSimpleCharaStatus(gcn::Graphics *g, int x, int y, const TCharacterInstance* chara)
{
    TGcnCharDrawer *drawer = g_GuiContainerList.GetDrawer(12);
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(chara->m_type);
    gcn::Color c(255, 255, 255);
    drawer->DrawString(g, x+2, y+2, charadata.m_name.c_str(), c);
    drawer->DrawString(g, x+3, y+2, charadata.m_name.c_str(), c);
    DrawRank(g, c, x+2, y+16, chara->m_rank, drawer);
    DrawExp(g, c, x+2, y+30, chara->m_experience, drawer);
    std::string hp = (boost::format("HP %3d/%3d") % chara->m_hp % chara->m_hpmax_rank).str();
    drawer->DrawString(g, x+52, y+16, hp.c_str(), c);
    std::string mp = (boost::format("MP %3d/%3d") % chara->m_mp % chara->m_mpmax_rank).str();
    drawer->DrawString(g, x+52, y+30, mp.c_str(), c);
}
//---------------------------------------------------------------------------
