//---------------------------------------------------------------------------
#include "formula.h"
#include "boostformat.h"
#include "mtrand.h"
#include "bmap.h"
#include "logger.h"
#include "gamemaster.h"

namespace{
    const int TCharacterInstance::*PowerPtr[POWER_MAX] = {
        &TCharacterInstance::m_dummy,
        &TCharacterInstance::m_hp, &TCharacterInstance::m_mp,
        &TCharacterInstance::m_pa_rank, &TCharacterInstance::m_pd_rank,
        &TCharacterInstance::m_skill_rank, &TCharacterInstance::m_speed_rank,
        &TCharacterInstance::m_ma_rank, &TCharacterInstance::m_md_rank};
    const int TCharacterInstance::*PlusPowerPtr[POWER_MAX] = {
        &TCharacterInstance::m_dummy,
        &TCharacterInstance::m_hp, &TCharacterInstance::m_mp,
        &TCharacterInstance::m_pa_plus, &TCharacterInstance::m_pd_plus,
        &TCharacterInstance::m_skill_plus, &TCharacterInstance::m_speed_plus,
        &TCharacterInstance::m_ma_plus, &TCharacterInstance::m_md_plus};
}
//---------------------------------------------------------------------------
// _[W▽̎ZoɎgpXe[^Xl𓾂B
void GetEffectPower(const TCharacterInstance& atkchara, const TCharacterInstance& defchara, const TSpecial& spe, int *atk, int *def, int *hit, int *avoid)
{
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TAttribute& attr = data->GetAttribute(spe.m_attribute);
    *atk = atkchara.*PowerPtr[spe.m_powertype];
    if(spe.m_powertype >= POWER_PA){
        *atk += atkchara.*PlusPowerPtr[spe.m_powertype];
    }
    if(attr.m_systemeffect <= SYSTEM_ABSORB){
        // atkȊO͍Un݂̂ŐݒB
        *def = defchara.*PowerPtr[spe.m_guardtype];
        *hit = defchara.*PowerPtr[spe.m_hittype];
        *avoid = defchara.*PowerPtr[spe.m_avoidtype];
        // HP,MPȊO͉ZltB
        if(spe.m_powertype >= POWER_PA){
            *def += defchara.*PlusPowerPtr[spe.m_guardtype];
            *hit += defchara.*PlusPowerPtr[spe.m_hittype];
            *avoid += defchara.*PlusPowerPtr[spe.m_avoidtype];
        }
    }
    else{
        // 񕜌n͏30
        *def = 30;
        // {svȂ̂ꉞlĂB_œ⏕@ɂȂ邩H
        *hit = 100;
        *avoid = 20;
    }
}
//---------------------------------------------------------------------------
int FormulaFTAttack(const TBattleMap& btmap, const TCharacterInstance& atkchara, const TCharacterInstance& defchara, const TSpecial& spe, int power, bool isCure)
{
    int atk, def, hit, avoid;
    GetEffectPower(atkchara, defchara, spe, &atk, &def, &hit, &avoid);
//OFDBGLOG((boost::format("atk=%d def=%d hit=%d avd=%d")%atk%def%hit%avoid).str());
    // Attack=(U+30+random(8))*Ul/80
    double atk2 = (atk + 30 + mtrand_n(8)) * power / 80;
    // Defenceh+30
    double def2 = def + 30;
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& defcharadata = data->GetCharacter(defchara.m_type);
    // ɁuvLꍇ͂Defence*2Auアv̏ꍇAttack*2B
    if(defchara.m_resistance[spe.m_attribute] == RES_STRONG){
        def2 *= 2;
    }
    else if(defchara.m_resistance[spe.m_attribute] == RES_WEAK){
        atk2 *= 2;
    }
//OFDBGLOG((boost::format("end")).str());
    // _[W = Attack^0.7*Attack/(Defense+10+random(10))*(20+random(5))/35
    return std::pow(atk2, 0.7) * (atk2 / (def2 + 10 + mtrand_n(10))) * (20 + mtrand_n(5)) / 35;
}
//---------------------------------------------------------------------------
int FormulaFTSpecial(const TBattleMap& btmap, const TCharacterInstance& atkchara, const TCharacterInstance& defchara, const TSpecial& spe, int power, bool isCure)
{
    int atk, def, hit, avoid;
    GetEffectPower(atkchara, defchara, spe, &atk, &def, &hit, &avoid);
    // Attack=iAttack+random(8)+30)*ZЗ/100
    double atk2 = (atk + mtrand_n(8) + 30) * power / 100;
    // Defence=Defence+30
    // AMP񕜁AHP񕜂̋Z͏Defence30ŔB
    double def2;
    if(isCure){
        def2 = 30;
    }
    else{
        def2 = def + 30;
    }
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& defcharadata = data->GetCharacter(defchara.m_type);
    // ɁuvLꍇ͂Defence*3Auアv̏ꍇAttack*2B
    if(defchara.m_resistance[spe.m_attribute] == RES_STRONG){
        def2 *= 3;
    }
    else if(defchara.m_resistance[spe.m_attribute] == RES_WEAK){
        atk2 *= 2;
    }
    // _[W=Attack^0.7*(Attack/(Defense+10+random(10)+0.5)*(20+random(5))/35
    return std::pow(atk2, 0.7) * (atk2 / (def2 + 10 + mtrand_n(10)) + 0.5) * (20 + mtrand_n(5)) / 35;
}
//---------------------------------------------------------------------------
int FormulaFTMagic(const TBattleMap& btmap, const TCharacterInstance& atkchara, const TCharacterInstance& defchara, const TSpecial& spe, int power, bool isCure)
{
    int atk, def, hit, avoid;
    GetEffectPower(atkchara, defchara, spe, &atk, &def, &hit, &avoid);
    // Attack=((*0.8{Random(8){60)*(@З+30))/40
    double atk2 = (atk * 0.8 + mtrand_n(8) + 60) * (power + 30) / 40;
    // Defence=R*0.8+70
    // AMP񕜁AHP񕜂̋Z͏Defence30ŔB
    double def2;
    if(isCure){
        def2 = 30;
    }
    else{
        def2 = def * 0.8 + 70;
    }
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& defcharadata = data->GetCharacter(defchara.m_type);
    // ɁuvLꍇ͂Defence*3Auアv̏ꍇAttack*2B
    if(defchara.m_resistance[spe.m_attribute] == RES_STRONG){
        def2 *= 2;
    }
    else if(defchara.m_resistance[spe.m_attribute] == RES_WEAK){
        atk2 *= 2;
    }
    // _[W=(Attack^0.4)*(Attack/(Defense+random(10))+.6)*(30+random(5))/35
    return std::pow(atk2, 0.4) * (atk2 / (def2 + mtrand_n(10)) + 0.6) * (30 + mtrand_n(5)) / 35;
}
//---------------------------------------------------------------------------
int FormulaFTAttackHit(const TBattleMap& btmap, const TCharacterInstance& atkchara, const TCharacterInstance& defchara, const TSpecial& spe, int power, bool isCure)
{
    int atk, def, hit, avoid;
    GetEffectPower(atkchara, defchara, spe, &atk, &def, &hit, &avoid);

    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& atkcharadata = data->GetCharacter(atkchara.m_type);
    const TBattleCharacter& defcharadata = data->GetCharacter(defchara.m_type);
    // UZ+(Z*n`C)+20
    int atkgnd = btmap.GetMapType(atkchara.m_location.x, atkchara.m_location.y);
    const TMoveType& atkmt = data->GetMoveType(atkchara.m_movetype);
    double atkskill2 = hit + (hit * atkmt.m_skill[atkgnd] / 100) + 20;

    // h䑤Z+(Z*n`C)+20
    int defgnd = btmap.GetMapType(defchara.m_location.x, defchara.m_location.y);
    const TMoveType& defmt = data->GetMoveType(defchara.m_movetype);
    double defskill2 = avoid + (avoid * defmt.m_skill[defgnd] / 100) + 20;

    // ɁuvLꍇ͂Defence*2Auアv̏ꍇ͕KB
    if(defchara.m_resistance[spe.m_attribute] == RES_STRONG){
        defskill2 *= 2;
    }
    else if(defchara.m_resistance[spe.m_attribute] == RES_WEAK){
        return 100;
    }
    // =(/)*80 (%)
    return 80 * atkskill2 / defskill2;
}
//---------------------------------------------------------------------------
int FormulaFTMagicHit(const TBattleMap& btmap, const TCharacterInstance& atkchara, const TCharacterInstance& defchara, const TSpecial& spe, int power, bool isCure)
{
    int atk, def, hit, avoid;
    GetEffectPower(atkchara, defchara, spe, &atk, &def, &hit, &avoid);
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& defcharadata = data->GetCharacter(defchara.m_type);
    // l@U
    double dhit = hit;
    // l@h䑤R+(R*n`C)+20
    int maptype = btmap.GetMapType(defchara.m_location.x, defchara.m_location.y);
    const TMoveType& mt = data->GetMoveType(defchara.m_movetype);
    double davoid = avoid * (100. + mt.m_skill[maptype])/100 + 20;

//g_Logger.DebugLog((boost::format("dhit=%f davoid=%f") % dhit % davoid).str());
    // ɁuvLꍇ͂Defence*2Auアv̏ꍇ͕KB
    if(defchara.m_resistance[spe.m_attribute] == RES_STRONG){
        davoid *= 2;
    }
    else if(defchara.m_resistance[spe.m_attribute] == RES_WEAK){
        return 100;
    }
    // @=/*80
    return dhit * 80 / davoid;
}
//---------------------------------------------------------------------------
int EvalExperience(const TCharacterInstance& c, bool actioned, std::vector<const TCharacterInstance*> killlist)
{
    const TOpenFarenData* data = g_GameMaster.GetData();
    // |ĽolvB
    int killexp = 0;
    for(unsigned u=0; u<killlist.size(); ++u){
        const TBattleCharacter& enemydata = data->GetCharacter(killlist[u]->m_type);
        killexp += enemydata.m_experience;
    }
    // ҋ@                      擾olʁrandom(5) 
    // @A퓬Ȃǂsꍇ  擾olʁ10+random(5) 
    // GEƂ              擾olʁ(GŗLol+10)*25/(ŗLol+20) + 10+random(5)
    int exp = mtrand_n(5);
    if(actioned){
        exp += 10;
    }
    const TBattleCharacter& charadata = data->GetCharacter(c.m_type);
    exp += (killexp + 10) * 25 / (charadata.m_experience + 20);
    // 擾o=擾ol*200/(100+ŗLol)
    exp = exp * 200 / (100 + charadata.m_experience);
    if(exp < 0){
        exp = 1;
    }
    else if(exp > TOpenFarenGame::EXPERIENCE_MAX){
        exp = TOpenFarenGame::EXPERIENCE_MAX;
    }
    return exp;
}
//---------------------------------------------------------------------------
// PłLȍUĂ邩B
bool HasAttack(const TBattleCharacter& charadata)
{
    for(std::vector<TAttack>::const_iterator itr = charadata.m_attacklist.begin(); itr != charadata.m_attacklist.end(); ++itr){
        if(itr->m_special != -1){
            return true;
        }
    }
    return false;
}
//---------------------------------------------------------------------------
void AddSystemEffect(TCharacterInstance& c, TSystemEffect addeffect)
{
    TSystemEffectStatus ses;
    ses.m_type = addeffect;
    ses.m_tick = 0;
    TSystemEffectStatusList::iterator itr = std::lower_bound(c.m_seslist.begin(), c.m_seslist.end(), ses, TSystemEffectLess());
    if(itr != c.m_seslist.end() && itr->m_type == addeffect){
        // Ɏw肳ꂽԈُɂȂĂJE^B
        itr->m_tick = 0;
    }
    else{
        // w肳ꂽԈُłȂΕt^B
        c.m_seslist.insert(itr, ses);
    }
}
//---------------------------------------------------------------------------
void RemoveSystemEffect(TCharacterInstance& c, TSystemEffect addeffect)
{
    TSystemEffectStatus ses;
    ses.m_type = addeffect;
    ses.m_tick = 0;
    TSystemEffectStatusList::iterator itr = std::lower_bound(c.m_seslist.begin(), c.m_seslist.end(), ses, TSystemEffectLess());
    if(itr != c.m_seslist.end() && itr->m_type == addeffect){
        c.m_seslist.erase(itr);
    }
}
//---------------------------------------------------------------------------
bool HasSystemEffect(const TCharacterInstance& c, TSystemEffect addeffect)
{
    TSystemEffectStatus ses;
    ses.m_type = addeffect;
    ses.m_tick = 0;
    TSystemEffectStatusList::const_iterator itr = std::lower_bound(c.m_seslist.begin(), c.m_seslist.end(), ses, TSystemEffectLess());
    return (itr != c.m_seslist.end() && itr->m_type == addeffect);
}
//---------------------------------------------------------------------------
bool CanAction(const TCharacterInstance& c)
{
    for(unsigned u=0; u<c.m_seslist.size(); ++u){
        if(c.m_seslist[u].m_type == SYSTEM_STONE ||
           c.m_seslist[u].m_type == SYSTEM_SLEEP ||
           c.m_seslist[u].m_type == SYSTEM_PARALYZE){
            return false;
        }
    }
    return true;
}
//---------------------------------------------------------------------------
