//---------------------------------------------------------------------------
#include <tinyxml.h>
#include <babel.h>
#include <cstdio>
#include <boost/filesystem/operations.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include "boostformat.h"
#include "ofini.h"
#include "ofxml.h"
#include "saveloaddlg.h"
#include "scenarioselect.h"
#include "masterselect.h"
#include "neighbor.h"
#include "gamemaster.h"
#include "ofdata.h"
#include "ofimage.h"
#include "ofmixer.h"
#include "ofgame.h"
#include "worldmap.h"
#include "widgetset.h"
#include "drawutil.h"
#include "gcnskinmanager.h"

TFarenGameMaster g_GameMaster;
#include "gui.h"
#include "logger.h"

namespace
{
    const char* SAVE_DIR = "Save";
    const char* SCE_DIR = "Scenario";
    const char* DEFSCE_DIR = "Default";
    const char* PICTURE_DIR = "Picture";
    const char* MUSICANDSOUND_DIR = "MusicAndSound";
    const char* SYSTEM_DIR = "System";

    const char* XMLELM_SAVETOP = "Save";
    const char* XMLELM_CHAPTER = "Chapter";
    const char* XMLELM_SCEDIR = "Scenario";
    const char* XMLELM_SAVEDATE = "Date";
    const char* XMLELM_SAVEDIGEST = "Digest";
    const char* MSG_CRASHDATA = "݊Ajf[^";
}
const char *TFarenGameMaster::INIFILE = "openfaren.ini";
//---------------------------------------------------------------------------
TFarenGameMaster::TFarenGameMaster()
    : m_Data(),
    m_Image(),
    m_Game(),
    m_WorldMapLogic(),
    m_GameStarted(false),
    m_MixerEnabled(false),
    m_KeyQueue(16)
{
}
//---------------------------------------------------------------------------
void TFarenGameMaster::Init()
{
    // wNXߖTB
    TNeighbor::Init();

    // `撲NXB
    g_DrawUtil.SetSlide(g_Ini.GetIntValue(TOpenFarenIni::DRAWRECT_SLIDEL),
                        g_Ini.GetIntValue(TOpenFarenIni::DRAWRECT_SLIDER),
                        g_Ini.GetIntValue(TOpenFarenIni::DRAWRECT_SLIDET),
                        g_Ini.GetIntValue(TOpenFarenIni::DRAWRECT_SLIDEB));

    boost::shared_ptr<TGcnSkinManager> pskin(new TGcnSkinManager());
    if(!pskin->LoadSkinImage(".")){
        OFWRNLOG("ǂݍ߂ȂXL摜");
    }
    m_SkinManager = pskin;

    boost::shared_ptr<TStartMenuWindow> pw(new TStartMenuWindow());
    g_GuiContainerList.Add(pw);
    m_StartMenuWindow = pw.get();

    boost::shared_ptr<TSysMenuWindow> psmw(new TSysMenuWindow());
    g_GuiContainerList.Add(psmw);
    m_SysMenuWindow = psmw.get();

    boost::shared_ptr<TConfigWindow> pcw(new TConfigWindow());
    g_GuiContainerList.Add(pcw);
    m_ConfigWindow = pcw.get();
    m_ConfigWindow->setVisible(false);

    boost::shared_ptr<TSaveLoadWindow> pslw(new TSaveLoadWindow());
    g_GuiContainerList.Add(pslw);
    m_SaveLoadWindow = pslw.get();
    m_SaveLoadWindow->setVisible(false);

    boost::shared_ptr<TSceSelectWindow> pssw(new TSceSelectWindow());
    g_GuiContainerList.Add(pssw);
    m_SceSelectWindow = pssw.get();
    m_SceSelectWindow->Init();
    m_SceSelectWindow->setVisible(false);

    boost::shared_ptr<TMasterSelectWindow> pmsw(new TMasterSelectWindow());
    g_GuiContainerList.Add(pmsw);
    m_MasterSelectWindow = pmsw.get();
    m_MasterSelectWindow->setVisible(false);

    gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
    top->add(m_StartMenuWindow);
    top->add(m_SysMenuWindow);
    top->add(m_ConfigWindow);
    top->add(m_SaveLoadWindow);
    top->add(m_SceSelectWindow);
    top->add(m_MasterSelectWindow);

    // ʒu߁B
    g_GuiContainerList.MoveWidgetToCenter(m_StartMenuWindow);
    m_SysMenuWindow->setPosition(0, top->getHeight() - m_SysMenuWindow->getHeight());
    m_SysMenuWindow->setVisible(false);
    g_GuiContainerList.MoveWidgetToCenter(m_SceSelectWindow);

    g_Logger.DebugLog("global window ok.");

    SetStartMenuVisible(true);
}
//---------------------------------------------------------------------------
void TFarenGameMaster::Destroy()
{
    m_WorldMapLogic.reset();
    m_Game.reset();
    m_Data.reset();
    m_Image.reset();
    m_Mixer.reset();
    m_WidgetSet.reset();
}
//---------------------------------------------------------------------------
// uGetSaveFileNamevWindowsł͊̃}NƏՓ˂悤B
std::string TFarenGameMaster::GenerateSaveFileName(const std::string& savdir, int num) const
{
    return (boost::format("%s/save%02d.xml") % savdir % num).str();
}
//---------------------------------------------------------------------------
// MasterData.xml Ŏw肳ꂽLN^[fBNgt@CTBViItH_ɖDefaultTB
// chrdirno != -1̏ꍇ͂̃LtH_ŏɒTB
bool TFarenGameMaster::FindCharacterFile(boost::shared_ptr<TOpenFarenData> data, const std::string& scepath, const std::string& filename, int chrdirno, std::string& dirpath) const
{
    if(chrdirno != -1){
        std::string chrdir = scepath + "/" + data->GetCharacterDir(chrdirno);
        std::string chkpath = chrdir + "/" + filename;
        if(boost::filesystem::exists(chkpath)){
            dirpath = chrdir;
            return true;
        }
    }
    for(int dirno = 0; dirno < data->GetCharacterDirCount(); ++dirno){
        if(chrdirno != -1 && dirno == chrdirno){
            continue;
        }
        std::string chrdir = scepath + "/" + data->GetCharacterDir(dirno);
        std::string chkpath = chrdir + "/" + filename;
        if(boost::filesystem::exists(chkpath)){
            dirpath = chrdir;
            return true;
        }
    }
    // "../Default"܂܂ĂDefault͒TȂB
    if(strstr(filename.c_str(), "../Default") != NULL){
        return false;
    }
    // DefaultTB
    std::string defpath = m_BasePath + "/" + SCE_DIR + "/" + DEFSCE_DIR;
    const char *defdir[2] = {"/DefChar", "/Element"};
    for(int dirno = 0; dirno < 2; ++dirno){
        std::string chkpath = defpath + defdir[dirno] + "/" + filename;
        if(boost::filesystem::exists(chkpath)){
            dirpath = defpath + defdir[dirno];
            return true;
        }
    }
    return false;
}
//---------------------------------------------------------------------------
// ViItH_Default̉t@CTBdirname"Picture"ȂǁB
bool TFarenGameMaster::FindPlacedFile(boost::shared_ptr<TOpenFarenData> data, const std::string& scepath, const std::string& dirname, const std::string& filename, std::string& dirpath) const
{
    std::string chkpath = scepath + "/" + dirname + "/" + filename;
//g_Logger.DebugLog(chkpath);
    if(boost::filesystem::exists(chkpath)){
        dirpath = scepath + "/" + dirname;
        return true;
    }
    // "../Default"܂܂ĂDefault͒TȂB
    if(strstr(filename.c_str(), "../Default") != NULL){
        return false;
    }
    // DefaultTB
    std::string defpath = m_BasePath + "/" + SCE_DIR + "/" + DEFSCE_DIR;
    chkpath = defpath + "/" + dirname + "/" + filename;
//g_Logger.DebugLog(chkpath);
    if(boost::filesystem::exists(chkpath)){
        dirpath = defpath + "/" + dirname;
        return true;
    }
    return false;
}
//---------------------------------------------------------------------------
int TFarenGameMaster::LoadEffectImage(boost::shared_ptr<TOpenFarenData> data, boost::shared_ptr<TOpenFarenImage> image, const std::string& scepath, const std::string& imgfile)
{
    std::string chrdir;
    if(FindPlacedFile(data, scepath, PICTURE_DIR, imgfile, chrdir)){
        return image->LoadEffect(chrdir + "/" + imgfile);
    }
    if(FindCharacterFile(data, scepath, imgfile, -1, chrdir)){
        return image->LoadEffect(chrdir + "/" + imgfile);
    }
    return -1;
}
//---------------------------------------------------------------------------
bool TFarenGameMaster::LoadPermanentImage(boost::shared_ptr<TOpenFarenData> data, boost::shared_ptr<TOpenFarenImage> image, boost::shared_ptr<TOpenFarenGame> game, const std::string& scepath)
{
    image->SetDirectory(scepath);
    image->SetSystemImageDirectory(m_BasePath + "/" + "System");

    if(!image->LoadButtonImage()){
        m_LastError = "{^摜̓ǂݍ݂Ɏs";
        return false;
    }
    if(!image->LoadChipImage()){
        m_LastError = "ʃ`bv摜̓ǂݍ݂Ɏsio[WႤFarencnv.exẽf[^Hj";
        return false;
    }

    // GA}bvwi摜[hB
    std::string fieldmapfile = data->GetFieldMapFile();
    if(!fieldmapfile.empty()){
        std::string imgdir;
        if(FindPlacedFile(data, scepath, PICTURE_DIR, fieldmapfile, imgdir)){
            image->LoadFieldMapImage(imgdir + "/" + fieldmapfile);
        }
        else{
            OFWRNLOG((boost::format("%s not found.") % fieldmapfile).str());
        }
    }

    // ̂CharacterDataƈႤfBNgɉ摜L̂őSẴLfBNgǍsB
    for(int i = 0; i < data->GetCharacterCount(); ++i){
        const TBattleCharacter& charadata = data->GetCharacter(i);
        // 摜擾B
        int faceindex = -1;
        if(!charadata.m_facefile.empty()){
            std::string chrdir;
            if(FindCharacterFile(data, scepath, charadata.m_facefile, charadata.m_dirno, chrdir)){
                faceindex = image->LoadFace(chrdir, charadata.m_facefile);
            }
            if(faceindex == -1){
                OFWRNLOG((boost::format("摜 %s ̓ǂݍ݂Ɏs܂BL %s ̊𖳂ɂ܂B") % charadata.m_facefile % charadata.m_id).str());
            }
        }
        // `bv摜擾B
        int chipindex = -1;
        std::string chrdir;
        if(FindCharacterFile(data, scepath, charadata.m_chipfile, charadata.m_dirno, chrdir)){
            chipindex = image->LoadCharaChip(chrdir, charadata.m_chipfile);
        }
        if(chipindex == -1){
            OFERRLOG((boost::format("L`bv摜 %s ̓ǂݍ݂Ɏs") % charadata.m_chipfile).str());
            m_LastError = "L`bv摜 " + charadata.m_chipfile + " ̓ǂݍ݂Ɏs";
            return false;
        }
        data->SetCharacterImage(i, faceindex, chipindex);
/*
        TBattleCharacter& charadata = data->GetCharacter(i);
        if(!charadata.m_facefile.empty()){
            std::string chrdir;
            if(FindCharacterFile(data, scepath, charadata.m_facefile, charadata.m_dirno, chrdir)){
                charadata.m_faceindex = image->LoadFace(chrdir, charadata.m_facefile);
            }
            if(charadata.m_faceindex == -1){
                OFWRNLOG((boost::format("摜 %s ̓ǂݍ݂Ɏs܂BL %s ̊𖳂ɂ܂B") % charadata.m_facefile % charadata.m_id).str());
            }
        }
        std::string chrdir;
        if(FindCharacterFile(data, scepath, charadata.m_chipfile, charadata.m_dirno, chrdir)){
            charadata.m_chipindex = image->LoadCharaChip(chrdir, charadata.m_chipfile);
        }
        if(charadata.m_chipindex == -1){
            m_LastError = "L`bv摜 " + charadata.m_chipfile + " ̓ǂݍ݂Ɏs";
            return false;
        }
*/
    }

    const char *bmapcharfile = "BMAPCHAR.png";
    std::string bmapchardir;
    if(!FindPlacedFile(data, scepath, PICTURE_DIR, bmapcharfile, bmapchardir) ||
       !image->LoadMapChip(bmapchardir + "/" + bmapcharfile)){
        m_LastError = "BMAPCHAR.png ̓ǂݍ݂Ɏs";
        OFERRLOG(m_LastError);
        return false;
    }

    const char *bmapobjfile = "BMAPOBJ.png";
    std::string bmapobjdir;
    if(!FindPlacedFile(data, scepath, PICTURE_DIR, bmapobjfile, bmapobjdir) ||
       !image->LoadMapObj(bmapobjdir + "/" + bmapobjfile)){
        m_LastError = "BMAPOBJ.png ̓ǂݍ݂Ɏs";
        OFERRLOG(m_LastError);
        return false;
    }

    std::string powermapdir;
    if(!FindPlacedFile(data, scepath, PICTURE_DIR, data->GetPowerMapFile(), powermapdir) ||
       !image->LoadPowerMap(powermapdir + "/" + data->GetPowerMapFile())){
        m_LastError = (boost::format("%s ̓ǂݍ݂Ɏs") % data->GetPowerMapFile()).str();
        OFERRLOG(m_LastError);
        return false;
    }

    const char *flag0file = "Flag0.png";
    int nomasterflag = -1;
    std::string flag0dir;
    if(FindPlacedFile(data, scepath, PICTURE_DIR, flag0file, flag0dir)){
        nomasterflag = image->LoadFlag(flag0dir, flag0file);
    }
    if(nomasterflag == -1){
        m_LastError = "摜 Flag0.png ̓ǂݍ݂Ɏs";
        OFERRLOG(m_LastError);
        return false;
    }
    data->SetNoMasterFlagIndex(nomasterflag);

    for(int i=0; i<data->GetMasterCount(); ++i){
        TMaster& master = data->GetMaster(i);
        //TBattleCharacter& charadata = data->GetCharacter(master.m_facecharaindex);
        if(!master.m_flagfile.empty()){
            std::string chrdir;
            if(FindCharacterFile(data, scepath, master.m_flagfile, -1, chrdir)){
                master.m_flagindex = image->LoadFlag(chrdir, master.m_flagfile);
            }
            if(FindPlacedFile(data, scepath, PICTURE_DIR, master.m_flagfile, chrdir)){
                master.m_flagindex = image->LoadFlag(chrdir, master.m_flagfile);
            }
        }
    }

    int specount = data->GetSpecialCount();
    for(int i=0; i<specount; ++i){
        const TSpecial& spe = data->GetSpecial(i);
        // @EKEZɉ摜t@Cw肳ĂȂꍇ͖@EKEZ̃[hɃ`FbNĂ̂ŖȂB
        if(!spe.m_imagefile.empty()){
            int img = LoadEffectImage(data, image, scepath, spe.m_imagefile);
            if(img == -1){
                bool usedspecial = false;
                for(unsigned u=0; u<data->GetCharacterCount(); ++u){
                    if(data->GetCharacter(u).m_specialindex == i){
                        usedspecial = true;
                        break;
                    }
                }
                if(usedspecial){
                    m_LastError = "Special " + spe.m_id + " ̉摜 " + spe.m_imagefile + " [hł܂łB";
                    OFERRLOG(m_LastError);
                    return false;
                }
                else{
                    g_Logger.WarningLog((boost::format("Special %s ̉摜 %s ܂łSpecial͎gĂȂ̂ŃX[܂B") % spe.m_id % spe.m_imagefile).str());
                }
            }
            data->SetSpecialImage(i, img);
        }
    }
    return true;
}
//---------------------------------------------------------------------------
bool TFarenGameMaster::LoadNonPermanentImage(TOpenFarenImage *image, TOpenFarenGame *game)
{
    for(int i=0; i<game->GetAreaInstanceCount(); ++i){
        const TAreaInstance& area = game->GetAreaInstance(i);
        if(!area.m_ignore){
            int imgnum = image->GenerateAreaName(area.m_name);
            if(imgnum == -1){
                m_LastError = "GA " + area.m_id + " ̖O摜𐶐ł܂łB";
                return false;
            }
            game->SetAreaNameImage(i, imgnum);
        }
    }
    return true;
}
//---------------------------------------------------------------------------
void TFarenGameMaster::ClearNonPermanentImage(TOpenFarenImage *image)
{
    image->ClearAreaName();
}
//---------------------------------------------------------------------------
// TOpenFarenImagezꍇɃGAO摜̂܂܎g悤ȂgBtrueB
bool TFarenGameMaster::CopyNonPermanentImage(const TOpenFarenGame *game_src, TOpenFarenGame *game_dst)
{
    for(int i=0; i<game_dst->GetAreaInstanceCount(); ++i){
        const TAreaInstance& area_src = game_src->GetAreaInstance(i);
        const TAreaInstance& area_dst = game_dst->GetAreaInstance(i);
        if(!area_dst.m_ignore){
            if(!area_dst.m_id.compare(area_src.m_id)){
                game_dst->SetAreaNameImage(i, area_src.m_nameimage);
            }
            else{
                return false;
            }
        }
    }
    return true;
}
//---------------------------------------------------------------------------
bool TFarenGameMaster::LoadMusicAndSound(boost::shared_ptr<TOpenFarenData> data, boost::shared_ptr<TOpenFarenGame> game, boost::shared_ptr<TOpenFarenMixer> mixer, const std::string& scepath)
{
    if(g_Ini.GetBoolValue(TOpenFarenIni::SOUND_ENABLED)){
        const char *chunkfiles[TOpenFarenMixer::CHUNK_MAX] = {"Ȃ񂾁H.wav", "U.wav", "SI.wav"};

        for(int i=0; i<TOpenFarenMixer::CHUNK_MAX; ++i){
#if defined(WIN32)
            const char *chunkfile = chunkfiles[i];
#else
            std::string chunkfiletmp = chunkfiles[i];
            g_Ini.EncodeFileNameFromSJIS(chunkfiletmp, chunkfiletmp);
            const char *chunkfile = chunkfiletmp.c_str();
#endif
            std::string dirpath;
            if(!FindPlacedFile(data, scepath, MUSICANDSOUND_DIR, chunkfile, dirpath)){
                g_Logger.ErrorLog((boost::format("ʌʉ %s ܂łB") % chunkfile).str());
                return false;
            }
            if(!mixer->LoadPermanentChunk(static_cast<TOpenFarenMixer::TPermanentChunk>(i), dirpath + "/" + chunkfile)){
                g_Logger.ErrorLog((boost::format("ʌʉ %s ̓ǂݍ݂Ɏs܂B") % chunkfile).str());
                return false;
            }
        }

        int specount = data->GetSpecialCount();
        for(int i=0; i<specount; ++i){
            const TSpecial& spe = data->GetSpecial(i);
            if(spe.m_soundfile.empty()){
                continue;
            }

            std::string dirpath;
            if(!FindPlacedFile(data, scepath, MUSICANDSOUND_DIR, spe.m_soundfile, dirpath)){
                g_Logger.WarningLog((boost::format("%s ̌ʉ %s ܂łB̌ʉ𖳉ɂ܂B") % spe.m_id % spe.m_soundfile).str());
                continue;
            }
            int chunk = mixer->LoadChunk(dirpath + "/" + spe.m_soundfile);
            if(chunk == -1){
                g_Logger.WarningLog((boost::format("%s ̌ʉ %s ̓ǂݍ݂Ɏs܂B̌ʉ𖳉ɂ܂B") % spe.m_id % spe.m_soundfile).str());
                continue;
            }
            data->SetSpecialChunk(i, chunk);
        }
    }

    if(g_Ini.GetBoolValue(TOpenFarenIni::MUSIC_ENABLED)){
        // ʉy
        for(int i=0; i<TOpenFarenMixer::MUSIC_MAX; ++i){
#if defined(WIN32)
            const char *musicfile = data->GetCommonMusicFile(i).c_str();
#else
            std::string musicfiletmp = data->GetCommonMusicFile(i);
            g_Ini.EncodeFileNameFromSJIS(musicfiletmp, musicfiletmp);
            const char *musicfile = musicfiletmp.c_str();
#endif
            if(musicfile[0] == '\0'){
                continue;
            }
            std::string dirpath;
            if(!FindPlacedFile(data, scepath, MUSICANDSOUND_DIR, musicfile, dirpath)){
                g_Logger.WarningLog((boost::format("ʉy %s ܂łB") % musicfile).str());
                continue;
            }
            if(!mixer->LoadPermanentMusic(static_cast<TOpenFarenMixer::TPermanentMusic>(i), dirpath + "/" + musicfile)){
                g_Logger.WarningLog((boost::format("ʉy %s ̓ǂݍ݂Ɏs܂B") % musicfile).str());
                continue;
            }
        }

        // }X^[̉y
        int mastercount = data->GetMasterCount();
        for(int i=0; i<mastercount; ++i){
            const TMaster& master = data->GetMaster(i);
            const int musics = 3;
            int music[musics] = {-1, -1, -1};
            const std::string TMaster::*musicfilelist[musics] = {&TMaster::m_musicfile, &TMaster::m_battlemusicfile, &TMaster::m_masterbattlemusicfile};
            for(int m=0; m<musics; ++m){
                const std::string& musicfile = master.*musicfilelist[m];
                if(musicfile.empty()){
                    continue;
                }
                std::string dirpath;
                if(!FindPlacedFile(data, scepath, MUSICANDSOUND_DIR, musicfile, dirpath)){
                    g_Logger.WarningLog((boost::format("%s ̉y %s ܂łB̉y𖳉ɂ܂B") % master.m_id % musicfile).str());
                    continue;
                }
                music[m] = mixer->LoadMusic(dirpath + "/" + musicfile);
                if(music[m] == -1){
                    g_Logger.WarningLog((boost::format("%s ̉y %s ̓ǂݍ݂Ɏs܂B̉y𖳉ɂ܂B") % master.m_id % musicfile).str());
                    continue;
                }
            }
            data->SetMasterMusic(i, music[0], music[1], music[2]);
        }

        // GẢy
        int areacount = game->GetAreaInstanceCount();
        for(int i=0; i<areacount; ++i){
            const TAreaInstance& area = game->GetAreaInstance(i);
            const std::string& musicfile = area.m_musicfile;
            if(musicfile.empty()){
                continue;
            }
            std::string dirpath;
            if(!FindPlacedFile(data, scepath, MUSICANDSOUND_DIR, musicfile, dirpath)){
                g_Logger.WarningLog((boost::format("%s ̉y %s ܂łB̉y𖳉ɂ܂B") % area.m_id % musicfile).str());
                continue;
            }
            int music = mixer->LoadMusic(dirpath + "/" + musicfile);
            if(music == -1){
                g_Logger.WarningLog((boost::format("%s ̉y %s ̓ǂݍ݂Ɏs܂B̉y𖳉ɂ܂B") % area.m_id % musicfile).str());
                continue;
            }
            game->SetAreaMusic(i, music);
        }
    }
    return true;
}
//---------------------------------------------------------------------------
bool TFarenGameMaster::LoadNewGame(const std::string& dir, int chapindex)
{
    boost::shared_ptr<TGcnWidgetSet> widgetset(new TGcnWidgetSet);
    boost::shared_ptr<TOpenFarenData> data(new TOpenFarenData());
    boost::shared_ptr<TOpenFarenImage> image(new TOpenFarenImage());
    boost::shared_ptr<TOpenFarenMixer> mixer(new TOpenFarenMixer(m_MixerEnabled));
    boost::shared_ptr<TOpenFarenGame> game(new TOpenFarenGame());

    std::string scepath = m_BasePath + "/" + SCE_DIR + "/" + dir;

    //m_ScenarioDir = dir;
    //m_ScenarioPath = m_BasePath + "/Scenario/" + dir;
    if(!data->Load(scepath, chapindex)){
        m_LastError = "ViIf[^ " + scepath + " ̓ǂݍ݂Ɏs";
        return false;
    }

    if(!game->LoadNew(*data, scepath, chapindex, m_SceSelectWindow->GetSelectedLevel())){
        m_LastError = "Q[f[^̓ǂݍ݂܂͍\zɎs";
        return false;
    }

    if(!LoadMusicAndSound(data, game, mixer, scepath)){
        m_LastError = "f[^̓ǂݍ݂Ɏs";
        return false;
    }
    if(!LoadPermanentImage(data, image, game, scepath)){
        m_LastError = "摜f[^̓ǂݍ݂Ɏs(p^[1)";
        return false;
    }
    if(!LoadNonPermanentImage(image.get(), game.get())){
        m_LastError = "摜f[^̓ǂݍ݂Ɏs(p^[2)";
        return false;
    }

    m_WidgetSet = widgetset;
    m_ScenarioDir = dir;
    m_ScenarioPath = scepath;
    m_Data = data;
    m_Image = image;
    m_Mixer = mixer;
    m_Game = game;
    boost::shared_ptr<TWorldMapLogic> logic(new TWorldMapLogic(widgetset.get()));
    m_WorldMapLogic = logic;
    return true;
}
//---------------------------------------------------------------------------
bool TFarenGameMaster::StartNewGame(const std::vector<int>& playerindice)
{
    m_MasterSelectWindow->setVisible(false);
    // I΂ꂽ}X^[̃vC[tO𗧂ĂB
    for(unsigned u=0; u<playerindice.size(); ++u){
        m_Game->SetMasterPlayer(playerindice[u], true);
    }
    // vC[tO𗧂ĂĂłȂƏ_zuL̃NȂȂB
    m_Game->PlaceAllAreaCharaRandom();
    m_WorldMapLogic->Start(true, false, NULL);
    m_GameStarted = true;
    return true;
}
//---------------------------------------------------------------------------
bool TFarenGameMaster::SaveGame(int num)
{
    TiXmlElement e(XMLELM_SAVETOP);

    TiXmlElement e_sce(XMLELM_SCEDIR);
    InsertTextChild(e_sce, m_ScenarioDir.c_str());
    e.InsertEndChild(e_sce);

    using namespace boost::posix_time;
    TiXmlElement e_date(XMLELM_SAVEDATE);
    //InsertTextChild(e_date, to_simple_string(microsec_clock::local_time()).c_str());
    InsertTextChild(e_date, GetNowDateStr().c_str());
    e.InsertEndChild(e_date);

    TiXmlElement e_digest(XMLELM_SAVEDIGEST);
    std::string digest;
    m_Game->GetSaveTitleString(digest);
    InsertTextChild(e_digest, digest.c_str());
    e.InsertEndChild(e_digest);

    TiXmlElement e_chap(XMLELM_CHAPTER);
    e_chap.SetAttribute("index", m_Game->GetChapterNumber());
    e.InsertEndChild(e_chap);

    m_Game->Save(e);
    m_WorldMapLogic->Save(e);

    std::string savdir = m_BasePath + "/" + SAVE_DIR;
    std::string filename = GenerateSaveFileName(savdir, num);
    TiXmlDocument doc(filename.c_str());
    doc.SetTabSize(2);
    doc.InsertEndChild(e);
    if(!doc.SaveFile()){
        return false;
    }
    return true;
}
//---------------------------------------------------------------------------
bool TFarenGameMaster::LoadGame(int num)
{
    //LoadGame2(num);

    std::string savdir = m_BasePath + "/" + SAVE_DIR;
    std::string filename = GenerateSaveFileName(savdir, num);
    TiXmlDocument doc(filename.c_str());
    if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog((boost::format("%s XML parse error") % filename).str());
        return false;
    }
    const TiXmlElement *e_top = doc.FirstChildElement(XMLELM_SAVETOP);
    if(e_top == NULL){
        g_Logger.ErrorLog((boost::format("%s %s s") % filename % XMLELM_SAVETOP).str());
        return false;
    }

    int chapindex;
    if(!FindIntAttribute(e_top, XMLELM_CHAPTER, "index", &chapindex, (boost::format("%s index not found.") % XMLELM_CHAPTER).str())){
        return false;
    }

    const TiXmlElement *e_sce = e_top->FirstChildElement(XMLELM_SCEDIR);
    if(e_sce == NULL){
        g_Logger.ErrorLog((boost::format("%s %s s") % filename % XMLELM_SCEDIR).str());
        return false;
    }
#if defined(WIN32)
    std::string scedir = GetElementText(*e_sce);
#else
    std::string tmp = GetElementTextEncoding(*e_sce, ENCODING_UTF8);
    std::string scedir;
    g_Ini.EncodeFileNameFromUTF8(tmp, scedir);
#endif
    std::string scepath = m_BasePath + "/" + SCE_DIR + "/" + scedir;

    boost::shared_ptr<TGcnWidgetSet> widgetset(new TGcnWidgetSet);

OFDBGLOG("2");
    if(!m_ScenarioDir.compare(scedir)){
        boost::shared_ptr<TOpenFarenGame> game(new TOpenFarenGame());
        if(!game->LoadSaved(*m_Data, scepath, chapindex, *e_top)){
            return false;
        }

        boost::shared_ptr<TWorldMapLogic> logic(new TWorldMapLogic(widgetset.get()));

        // GAC[W̃Rs[܂͐B{Iɂ̓Rs[ɂȂ͂B
        if(!CopyNonPermanentImage(m_Game.get(), game.get())){
            // TODO:̏s͂ps\BvCBAreaNameʃNXɂH
            ClearNonPermanentImage(m_Image.get());
            if(!LoadNonPermanentImage(m_Image.get(), game.get())){
                return false;
            }
        }

        m_Game.reset();
        m_WorldMapLogic.reset();

        m_Game = game;
        m_WorldMapLogic = logic;
    }
    else{
OFDBGLOG("3");
        boost::shared_ptr<TOpenFarenData> data(new TOpenFarenData());
        if(!data->Load(scepath, chapindex)){
            m_LastError = "ViIf[^ " + scepath + " ̓ǂݍ݂Ɏs";
            return false;
        }

OFDBGLOG("4");
        boost::shared_ptr<TOpenFarenGame> game(new TOpenFarenGame());
        if(!game->LoadSaved(*data, scepath, chapindex, *e_top)){
            return false;
        }

        boost::shared_ptr<TOpenFarenMixer> mixer(new TOpenFarenMixer(m_MixerEnabled));
        if(!LoadMusicAndSound(data, game, mixer, scepath)){
            return false;
        }

OFDBGLOG("5");
        boost::shared_ptr<TOpenFarenImage> image(new TOpenFarenImage());
        if(!LoadPermanentImage(data, image, game, scepath)){
            return false;
        }
        if(!LoadNonPermanentImage(image.get(), game.get())){
            return false;
        }

OFDBGLOG("6");
        boost::shared_ptr<TWorldMapLogic> logic(new TWorldMapLogic(widgetset.get()));
OFDBGLOG("7");
        m_Data.reset();
        m_Image.reset();
        m_Game.reset();
        m_WorldMapLogic.reset();

        m_ScenarioDir = scedir;
        m_ScenarioPath = scepath;
        m_Data = data;
        m_Image = image;
        m_Mixer = mixer;
        m_Game = game;
        m_WorldMapLogic = logic;
    }
OFDBGLOG("9");
    //m_WidgetSet.reset();
    m_WidgetSet = widgetset;
    m_WorldMapLogic->Start(false, false, e_top);
    m_GameStarted = true;
OFDBGLOG("10");
    return true;
}
//---------------------------------------------------------------------------
void TFarenGameMaster::GetSaveDigest(const std::string& savdir, std::vector<std::string>& savlist, int maxsize) const
{
    savlist.clear();
    for(int i=0; i<maxsize; ++i){
        std::string filename = GenerateSaveFileName(savdir, i);
        if(boost::filesystem::exists(filename)){
            TiXmlDocument doc(filename.c_str());
            if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
                g_Logger.ErrorLog((boost::format("%s XML parse error") % filename).str());
                savlist.push_back(MSG_CRASHDATA);
                continue;
            }
            const TiXmlElement *e_top = doc.FirstChildElement(XMLELM_SAVETOP);
            if(e_top == NULL){
                g_Logger.ErrorLog((boost::format("%s XML %s not found.") % filename % XMLELM_SAVETOP).str());
                savlist.push_back(MSG_CRASHDATA);
                continue;
            }
            const TiXmlElement *e_date = e_top->FirstChildElement(XMLELM_SAVEDATE);
            if(e_date == NULL){
                g_Logger.ErrorLog((boost::format("%s XML %s not found.") % filename % XMLELM_SAVEDATE).str());
                savlist.push_back(MSG_CRASHDATA);
                continue;
            }
            std::string date = GetElementText(*e_date);

            const TiXmlElement *e_digest = e_top->FirstChildElement(XMLELM_SAVEDIGEST);
            if(e_digest == NULL){
                g_Logger.ErrorLog((boost::format("%s XML %s not found.") % filename % XMLELM_SAVEDIGEST).str());
                savlist.push_back(MSG_CRASHDATA);
                continue;
            }
            std::string digest = GetElementText(*e_digest);

            savlist.push_back(date.substr(0, 19) + " " + digest);
        }
        else{
            savlist.push_back("f[^Ȃ");
        }
    }
}
//---------------------------------------------------------------------------
bool TFarenGameMaster::FindScriptPath(const std::string& file, std::string& path) const
{
    std::string str = m_ScenarioPath + "/Script/" + file;
    if(boost::filesystem::exists(str)){
        path = str;
        return true;
    }
    str = m_BasePath + "/Script/" + file;
    if(boost::filesystem::exists(str)){
        path = str;
        return true;
    }
    return false;
}
//---------------------------------------------------------------------------
bool TFarenGameMaster::FindEventImagePath(const std::string& imgfile, std::string& path) const
{
    std::string chrdir;
    if(FindPlacedFile(m_Data, m_ScenarioPath, PICTURE_DIR, imgfile, chrdir)){
        path = chrdir + "/" + imgfile;
        return true;
    }
    if(FindCharacterFile(m_Data, m_ScenarioPath, imgfile, -1, chrdir)){
        path = chrdir + "/" + imgfile;
        return true;
    }
    return false;
}
//---------------------------------------------------------------------------
//bool TFarenGameMaster::FindSystemFilePath(const std::string& filename, std::string& path) const
//{
//    std::string chkpath = m_ScenarioPath + "/" + SYSTEM_DIR + "/" + filename;
//    if(boost::filesystem::exists(chkpath)){
//        path = chkpath + "/" + filename;
//        return true;
//    }
//    chkpath = m_BasePath + "/" + SYSTEM_DIR + "/" + filename;
//    if(boost::filesystem::exists(chkpath)){
//        path = chkpath + "/" + filename;
//        return true;
//    }
//    return false;
//}
//---------------------------------------------------------------------------
void TFarenGameMaster::SetStartMenuVisible(bool b)
{
    m_StartMenuWindow->setVisible(b);
//    if(b){
//        gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
//        top->moveToTop(m_StartMenuWindow);
//    }
}
//---------------------------------------------------------------------------
void TFarenGameMaster::SetSysMenuVisible(bool b)
{
    m_SysMenuWindow->setVisible(b);
    if(b){
        gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
        top->moveToTop(m_SysMenuWindow);
    }
}
//---------------------------------------------------------------------------
void TFarenGameMaster::SetSysMenuMode(TSysMenuMode mode)
{
    m_SysMenuWindow->SetMode(mode);
}
//---------------------------------------------------------------------------
// ŏ̉ʂƃViIIʂ؂ւ鎞ɎgB
void TFarenGameMaster::SetSceSelectVisible(bool b)
{
    m_SceSelectWindow->setVisible(b);
    if(!b){
        // ŏ̃j[\B
        SetStartMenuVisible(true);
    }
}
//---------------------------------------------------------------------------
// }X^[IʂƃViIIʂ؂ւ鎞ɎgB
void TFarenGameMaster::SetMasterSelectVisible(bool b)
{
    if(b){
        m_SceSelectWindow->setVisible(false);
        m_MasterSelectWindow->GetPane().StartOperation();
        g_GuiContainerList.MoveWidgetToCenter(m_MasterSelectWindow);
    }
    m_MasterSelectWindow->setVisible(b);
    if(!b){
        m_MasterSelectWindow->GetPane().EndOperation();
        // ViIf[^jB
        Destroy();
        // ViIIj[\B
        m_SceSelectWindow->setVisible(true);
    }
}
//---------------------------------------------------------------------------
void TFarenGameMaster::ShowSaveLoad(bool isSave)
{
    m_SaveLoadWindow->setVisible(true);
    std::string savdir = m_BasePath + "/" + SAVE_DIR;
    if(!boost::filesystem::exists(savdir)){
        boost::filesystem::create_directory(savdir);
    }
    std::vector<std::string> savlist;
    GetSaveDigest(savdir, savlist, 10);
    m_SaveLoadWindow->SetMode(isSave, savlist);
    g_GuiContainerList.MoveWidgetToCenter(m_SaveLoadWindow);
    gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
    top->moveToTop(m_SaveLoadWindow);
}
//---------------------------------------------------------------------------
void TFarenGameMaster::HideSaveLoad()
{
    m_SaveLoadWindow->setVisible(false);
    if(m_GameStarted){
        // Q[ȂVXej[\B
        SetSysMenuVisible(true);
    }
    else{
        // Q[JnOȂŏ̃j[\B
        SetStartMenuVisible(true);
    }
}
//---------------------------------------------------------------------------
void TFarenGameMaster::ShowConfig()
{
    m_ConfigWindow->InitUI();
    m_ConfigWindow->setVisible(true);
    g_GuiContainerList.MoveWidgetToCenter(m_ConfigWindow);
    gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
    top->moveToTop(m_ConfigWindow);
}
//---------------------------------------------------------------------------
void TFarenGameMaster::HideConfig(bool ok)
{
    if(ok){
        // `撲NXB
        g_DrawUtil.SetSlide(g_Ini.GetIntValue(TOpenFarenIni::DRAWRECT_SLIDEL),
                            g_Ini.GetIntValue(TOpenFarenIni::DRAWRECT_SLIDER),
                            g_Ini.GetIntValue(TOpenFarenIni::DRAWRECT_SLIDET),
                            g_Ini.GetIntValue(TOpenFarenIni::DRAWRECT_SLIDEB));
        if(m_WorldMapLogic.get() != NULL){
            m_WorldMapLogic->ApplyConfig();
        }
    }
    m_ConfigWindow->setVisible(false);
    if(m_GameStarted){
        // Q[ȂVXej[\B
        SetSysMenuVisible(true);
    }
    else{
        // Q[JnOȂŏ̃j[\B
        SetStartMenuVisible(true);
    }
}
//---------------------------------------------------------------------------
//void TFarenGameMaster::SetKeyQueueEnabled(bool enabled)
//{
//    m_KeyQueueEnabled = enabled;
//    if(!enabled){
//        m_KeyQueue.clear();
//    }
//}
//---------------------------------------------------------------------------
bool TFarenGameMaster::GetKeyQueue(SDLKey& key)
{
    if(m_KeyQueue.empty()){
        return false;
    }
    key = m_KeyQueue.front();
    return true;
}
//---------------------------------------------------------------------------
void TFarenGameMaster::PopKeyQueue()
{
    if(!m_KeyQueue.empty()){
        m_KeyQueue.pop_front();
    }
}
//---------------------------------------------------------------------------
void TFarenGameMaster::PushKeyQueue(SDLKey key)
{
    if(m_KeyQueue.size() < m_KeyQueue.capacity()){
        m_KeyQueue.push_back(key);
    }
}
//---------------------------------------------------------------------------
void TFarenGameMaster::ClearKeyQueue()
{
    m_KeyQueue.clear();
}
//---------------------------------------------------------------------------
bool TFarenGameMaster::FindPlacedMusicSoundFile(const std::string& filename, std::string& dirpath) const
{
    return FindPlacedFile(m_Data, m_ScenarioPath, MUSICANDSOUND_DIR, filename, dirpath);
}
//---------------------------------------------------------------------------
