//---------------------------------------------------------------------------
#ifndef gamemasterH
#define gamemasterH

#include <boost/shared_ptr.hpp>
#include <boost/circular_buffer.hpp>
#include <string>
#include <vector>
#include <SDL.h>
#include "sysmenu.h"
//#include "gcnwindow.h"

class TGcnWidgetSet;
class TGcnSkinManager;
class TOpenFarenData;
class TOpenFarenImage;
class TOpenFarenMixer;
class TOpenFarenGame;
class TWorldMapLogic;
//class TStartMenuWindow;
//class TSysMenuWindow;
class TSaveLoadWindow;
class TSceSelectWindow;
class TMasterSelectWindow;
//---------------------------------------------------------------------------
class TFarenGameMaster
{
private:
    boost::shared_ptr<TGcnWidgetSet> m_WidgetSet;
    boost::shared_ptr<TGcnSkinManager> m_SkinManager;
    boost::shared_ptr<TOpenFarenData> m_Data;
    boost::shared_ptr<TOpenFarenImage> m_Image;
    boost::shared_ptr<TOpenFarenMixer> m_Mixer;
    boost::shared_ptr<TOpenFarenGame> m_Game;
    boost::shared_ptr<TWorldMapLogic> m_WorldMapLogic;
    TStartMenuWindow* m_StartMenuWindow;
    TSysMenuWindow* m_SysMenuWindow;
    TConfigWindow* m_ConfigWindow;
    TSaveLoadWindow* m_SaveLoadWindow;
    TSceSelectWindow* m_SceSelectWindow;
    TMasterSelectWindow* m_MasterSelectWindow;
    std::string m_BasePath;     // OpenFareñCXg[fBNgB
    std::string m_ScenarioPath; // ViIfBNg̃tpXBႦDefaultȂm_BasePath + "/Scenario/Default"
    std::string m_ScenarioDir;  // ViIfBNgBႦDefaultȂ"Default"
    std::string m_LastError;
    bool m_GameStarted;
    bool m_MixerEnabled;
    boost::circular_buffer<SDLKey> m_KeyQueue;

    std::string GenerateSaveFileName(const std::string& savdir, int num) const;
    bool FindCharacterFile(boost::shared_ptr<TOpenFarenData> data, const std::string& scepath, const std::string& filename, int chrdirno, std::string& dirpath) const;
    bool FindPlacedFile(boost::shared_ptr<TOpenFarenData> data, const std::string& scepath, const std::string& dirname, const std::string& filename, std::string& dirpath) const;
    int LoadEffectImage(boost::shared_ptr<TOpenFarenData> data, boost::shared_ptr<TOpenFarenImage> image, const std::string& scepath, const std::string& imgfile);
    bool LoadPermanentImage(boost::shared_ptr<TOpenFarenData> data, boost::shared_ptr<TOpenFarenImage> image, boost::shared_ptr<TOpenFarenGame> game, const std::string& scepath);
    bool LoadNonPermanentImage(TOpenFarenImage *image, TOpenFarenGame *game);
    void ClearNonPermanentImage(TOpenFarenImage *image);
    bool CopyNonPermanentImage(const TOpenFarenGame *game_src, TOpenFarenGame *game_dst);
    bool LoadMusicAndSound(boost::shared_ptr<TOpenFarenData> data, boost::shared_ptr<TOpenFarenGame> game, boost::shared_ptr<TOpenFarenMixer> mixer, const std::string& scepath);
public:
    static const int GAMELEVEL_MAX = 5;
    enum TBattleSpeed{
        BATTLESPEED_MIN = 1,
        BATTLESPEED_MAX = 5
    };
    static const char *INIFILE;

    TFarenGameMaster();
    void Init();
    void Destroy();
    void SetBasePath(const std::string& path) { m_BasePath = path; }
    const std::string& GetBasePath() const { return m_BasePath; }
    const std::string& GetScenarioPath() const { return m_ScenarioPath; }
    bool LoadNewGame(const std::string& dir, int chapindex);
    bool StartNewGame(const std::vector<int>& playerindice);
    bool SaveGame(int num);
    bool LoadGame(int num);
    void GetSaveDigest(const std::string& savdir, std::vector<std::string>& savlist, int maxsize) const;
    bool FindScriptPath(const std::string& file, std::string& path) const;
    bool FindEventImagePath(const std::string& imgfile, std::string& path) const;
    //bool FindSystemFilePath(const std::string& filename, std::string& path) const;
    void SetStartMenuVisible(bool b);
    void SetSysMenuVisible(bool b);
    void SetSysMenuMode(TSysMenuMode mode);
    void SetSceSelectVisible(bool b);
    void SetMasterSelectVisible(bool b);
    //void SetSaveLoadVisible(bool b, bool isSave);
    void ShowSaveLoad(bool isSave);
    void HideSaveLoad();
    void ShowConfig();
    void HideConfig(bool ok);
    bool IsShowingConfig() const { return m_ConfigWindow->isVisible(); }
    bool GetKeyQueue(SDLKey& key);
    void PopKeyQueue();
    void PushKeyQueue(SDLKey key);
    void ClearKeyQueue();
    void SetMixerEnabled() { m_MixerEnabled = true; }
    bool FindPlacedMusicSoundFile(const std::string& filename, std::string& dirpath) const;
    TGcnSkinManager* GetSkinManager() { return m_SkinManager.get(); }
    const TOpenFarenData* GetData() { return m_Data.get(); }
    TOpenFarenImage* GetImage() { return m_Image.get(); }
    TOpenFarenMixer* GetMixer() { return m_Mixer.get(); }
    TOpenFarenGame* GetGame() { return m_Game.get(); }
    const std::string& GetLastError() const { return m_LastError; }
};
//---------------------------------------------------------------------------
extern TFarenGameMaster g_GameMaster;
//---------------------------------------------------------------------------
#endif
