//---------------------------------------------------------------------------
#include "gcnskinmanager.h"
#include "gcnwindow.h"
#include "ofini.h"
#include "boostformat.h"
#include "logger.h"
//---------------------------------------------------------------------------
TGcnSkinManager::TGcnSkinManager()
{
    m_Skin = g_Ini.GetDefaultSkin();
    for(int i = SKINIMAGE_WINDOW; i < SKINIMAGE_MAX; ++i){
        m_SkinImage[i] = NULL;
    }
}
//---------------------------------------------------------------------------
TGcnSkinManager::~TGcnSkinManager()
{
    for(int i = SKINIMAGE_WINDOW; i < SKINIMAGE_MAX; ++i){
        if(m_SkinImage[i] != NULL){
            delete m_SkinImage[i];
        }
    }
}
//---------------------------------------------------------------------------
// ̓pXBOpenFareñCXg[fBNg܂̓ViIfBNgB
bool TGcnSkinManager::LoadSkinImage(const std::string& baseDir)
{
    static const char *filenames[SKINIMAGE_MAX] = {
        "window.png"};
    for(int i = SKINIMAGE_WINDOW; i < SKINIMAGE_MAX; ++i){
        m_SkinImage[i] = NULL;
    }
    for(int i = SKINIMAGE_WINDOW; i < SKINIMAGE_MAX; ++i){
        m_SkinImage[i] = LoadImage(baseDir + "/System/" + filenames[i]);
        if(m_SkinImage[i] == NULL){
            return false;
        }
    }
    return true;
}
//---------------------------------------------------------------------------
void TGcnSkinManager::ApplyWindowSkin(TGcnWindow *win) const
{
    win->setBaseColor(m_Skin->GetColorValue(TOpenFarenSkin::SKINCOLOR_WINDOW));
    win->setBackgroundColor(gcn::Color(0, 0, 0, 192));
    win->SetSkinImage(m_SkinImage[SKINIMAGE_WINDOW]);
}
//---------------------------------------------------------------------------
void TGcnSkinManager::ApplyGrayPaneSkin(gcn::Widget *widget) const
{
    //widget->setBaseColor(m_Skin->GetColorValue(TOpenFarenSkin::SKINCOLOR_WINDOW));
    widget->setBackgroundColor(m_Skin->GetColorValue(TOpenFarenSkin::SKINCOLOR_WINDOW));
}
//---------------------------------------------------------------------------
void TGcnSkinManager::ApplyDarkPaneSkin(gcn::Widget *widget) const
{
    //widget->setBaseColor(m_Skin->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE));
    widget->setBackgroundColor(m_Skin->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE));
}
//---------------------------------------------------------------------------
void TGcnSkinManager::ApplyRadioButtonSkin(gcn::RadioButton *rb) const
{
    rb->setBackgroundColor(m_Skin->GetColorValue(TOpenFarenSkin::SKINCOLOR_RADIOBUTTONBG));
    rb->setForegroundColor(g_Ini.GetFontColor());
}
//---------------------------------------------------------------------------
void TGcnSkinManager::ApplyButtonEnabled(gcn::Button *b) const
{
    b->setForegroundColor(g_Ini.GetFontColor());
    b->setBaseColor(m_Skin->GetColorValue(TOpenFarenSkin::SKINCOLOR_BUTTONENABLED));
    b->setEnabled(true);
}
//---------------------------------------------------------------------------
void TGcnSkinManager::ApplyButtonDisabled(gcn::Button *b) const
{
    b->setForegroundColor(g_Ini.GetFontDisabledColor());
    b->setBaseColor(m_Skin->GetColorValue(TOpenFarenSkin::SKINCOLOR_BUTTONDISABLED));
    b->setEnabled(false);
}
//---------------------------------------------------------------------------
void TGcnSkinManager::ApplyCheckBoxEnabled(gcn::CheckBox *cb) const
{
    cb->setForegroundColor(g_Ini.GetFontColor());
    cb->setBackgroundColor(gcn::Color(0, 0, 0));
    cb->setEnabled(true);
}
//---------------------------------------------------------------------------
void TGcnSkinManager::ApplyCheckBoxDisabled(gcn::CheckBox *cb) const
{
    cb->setForegroundColor(gcn::Color(192, 192, 192)); // GetFontDisabledColor()ƂقƂǌȂȂB
    cb->setBackgroundColor(gcn::Color(128, 128, 128));
    cb->setEnabled(false);
}
//---------------------------------------------------------------------------
void TGcnSkinManager::ApplyListBoxSkin(gcn::ListBox *lb) const
{
    lb->setForegroundColor(g_Ini.GetFontColor());
    lb->setBackgroundColor(m_Skin->GetColorValue(TOpenFarenSkin::SKINCOLOR_LISTBOXBGCOLOR));
    lb->setSelectionColor(m_Skin->GetColorValue(TOpenFarenSkin::SKINCOLOR_LISTBOXSELCOLOR));
}
//---------------------------------------------------------------------------
void TGcnSkinManager::ApplyScrollAreaSkin(gcn::ScrollArea *sa) const
{
    sa->setBackgroundColor(m_Skin->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE));
    sa->setScrollbarWidth(g_Ini.GetIntValue(TOpenFarenIni::SCROLLBAR_WIDTH));
}
//---------------------------------------------------------------------------
gcn::Image* TGcnSkinManager::LoadImage(const std::string& filename)
{
    gcn::Image* img;
    try{
        img = gcn::Image::load(filename);
    }
    catch(gcn::Exception){
        OFERRLOG((boost::format("Failed to load image %s") % filename.c_str()).str());
        return NULL;
    }
    return img;
}
//---------------------------------------------------------------------------
