//---------------------------------------------------------------------------
#include "gridimpl.h"
#include "gui.h"
#include "gamemaster.h"
#include "gcnskinmanager.h"
#include "ofgame.h"
#include "ofini.h"
#include "logger.h"
#include "boostformat.h"
#include "boostlexicalcast.h"
#include "porting.h"
//---------------------------------------------------------------------------
namespace{
    // RXgN^Ŏw肳ꂽnԖڂvectorvfŔrs֐IuWFNgB
    // ~̐؂ւ͑ɕ@邩Ȃǂ킩Ȃ̂ŕEEE~̑gݍ킹SB
    struct TVecStrCompA{
        int m_n;
        TVecStrCompA(int n) : m_n(n) {}
        int operator()(const std::vector<std::string>& lhs, const std::vector<std::string>& rhs){
            return lhs[m_n] < rhs[m_n];
        }
    };

    struct TVecStrCompD{
        int m_n;
        TVecStrCompD(int n) : m_n(n) {}
        int operator()(const std::vector<std::string>& lhs, const std::vector<std::string>& rhs){
            return lhs[m_n] > rhs[m_n];
        }
    };

    struct TVecIntCompA{
        int m_n;
        TVecIntCompA(int n) : m_n(n) {}
        int operator()(const std::vector<std::string>& lhs, const std::vector<std::string>& rhs){
            return boost::lexical_cast<int>(lhs[m_n]) < boost::lexical_cast<int>(rhs[m_n]);
        }
    };

    struct TVecIntCompD{
        int m_n;
        TVecIntCompD(int n) : m_n(n) {}
        int operator()(const std::vector<std::string>& lhs, const std::vector<std::string>& rhs){
            return boost::lexical_cast<int>(lhs[m_n]) > boost::lexical_cast<int>(rhs[m_n]);
        }
    };

    void SortVec(std::vector<std::vector<std::string> >& vecs, int col, bool isstr, bool desc)
    {
        if(isstr){
            if(desc){
                std::stable_sort(vecs.begin(), vecs.end(), TVecStrCompD(col));
            }
            else{
                std::stable_sort(vecs.begin(), vecs.end(), TVecStrCompA(col));
            }
        }
        else{
            if(desc){
                std::stable_sort(vecs.begin(), vecs.end(), TVecIntCompD(col));
            }
            else{
                std::stable_sort(vecs.begin(), vecs.end(), TVecIntCompA(col));
            }
        }
    }
}
//---------------------------------------------------------------------------
int TAreaGridModel::GetNumberOfColumns()
{
    return COLS;
}
//---------------------------------------------------------------------------
int TAreaGridModel::GetNumberOfRows()
{
    return m_StringValues.size() + 1;
}
//---------------------------------------------------------------------------
std::string TAreaGridModel::GetElementAt(int col, int row)
{
    static const char *title[COLS] = {"GA", "xz", "", "X", "", "H", "", "͎w", "ҋ@s"};
    if(!row){
        return title[col];
    }
    return m_StringValues[row - 1][col];
}
//---------------------------------------------------------------------------
void TAreaGridModel::OnSort(int col)
{
    static const bool isstr[COLS] = {true, true, false, false, false, false, false, false, true};
    bool desc = m_Desc[col];
    SortVec(m_StringValues, col, isstr[col], desc);
    m_Desc.assign(m_Desc.size(), false);
    m_Desc[col] = !desc;
}
//---------------------------------------------------------------------------
int TAreaGridModel::GetColumnWidth(int col)
{
    return m_ColumnWidth[col];
}
//---------------------------------------------------------------------------
int TAreaGridModel::GetRowHeight()
{
    return ROW_HEIGHT;
}
//---------------------------------------------------------------------------
bool TAreaGridModel::GetColumnAlignment(int col)
{
    static const bool al[COLS] = {false, false, true, true, true, true, true, true};
    return al[col];
}
//---------------------------------------------------------------------------
void TAreaGridModel::Init()
{
    // ۂ̂ƂStructString()ƃGAIDŕԂ킯A
    // [UɂƂĂ͒mƂł͂Ȃ̂ŃGA\[gƂB
    m_Desc.clear();
    m_Desc.resize(COLS);
    StructString();
    AdjustColumnWidth();
    OnSort(0);
}
//---------------------------------------------------------------------------
int TAreaGridModel::GetWidth() const
{
    int w = 1;
    for(int i=0; i<m_ColumnWidth.size(); ++i){
        w += m_ColumnWidth[i];
    }
    return w;
}
//---------------------------------------------------------------------------
int TAreaGridModel::GetHeight() const
{
    return (m_StringValues.size() + 1) * ROW_HEIGHT + 1;
}
//---------------------------------------------------------------------------
// ʓ|Ȃ̂ŃHǕz\zB
void TAreaGridModel::StructString()
{
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TOpenFarenData* data = g_GameMaster.GetData();
    int rows = game->GetAreaInstanceCount();
    m_StringValues.clear();
    for(int i=0; i<rows; ++i){
        const TAreaInstance& area = game->GetAreaInstance(i);
        if(area.m_ignore){
            continue;
        }
        std::vector<std::string> vstr(COLS);
        vstr[0] = area.m_name;
        if(area.m_master != -1){
            vstr[1] = data->GetMaster(game->GetMasterInstance(area.m_master).m_type).m_name;
            vstr[8] = TOpenFarenGame::AreaOperationName[area.m_operation];
        }
        else{
            vstr[1] = "n";
            vstr[8] = "||||";
        }
        vstr[2] = (boost::format("%d") % game->GetAreaIncome(i)).str();
        vstr[3] = (boost::format("%d") % area.m_city).str();
        vstr[4] = (boost::format("%d") % area.m_wall).str();
        vstr[5] = (boost::format("%d") % area.m_road).str();
        vstr[6] = (boost::format("%d") % (area.m_charalist.size())).str();
        vstr[7] = (boost::format("%d") % (game->GetAreaPower(i) / 1000)).str();
        m_StringValues.push_back(vstr);
    }
}
//---------------------------------------------------------------------------
TAreaGridWindow::TAreaGridWindow()
{
    //const int gridareaw = 608;
    //const int gridareah = 640;

    m_ButtonClose.setCaption("");
    m_ButtonClose.setSize(56, 24);
    //m_ButtonClose.setPosition(gridareaw - 64, 4);
    g_GameMaster.GetSkinManager()->ApplyButtonEnabled(&m_ButtonClose);
    add(&m_ButtonClose);

    const int scbarw = g_Ini.GetIntValue(TOpenFarenIni::SCROLLBAR_WIDTH);
    m_Grid.SetGridModel(&m_AreaGridModel);
    m_Grid.setPosition(0, 0);
    //m_Grid.setSize(gridareaw - scbarw, gridareah);
    m_Grid.setBackgroundColor(g_GameMaster.GetSkinManager()->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE));

    m_GridScrollArea.setScrollbarWidth(scbarw);
    m_GridScrollArea.setVerticalScrollPolicy(gcn::ScrollArea::SHOW_ALWAYS);
    m_GridScrollArea.setContent(&m_Grid);
    m_GridScrollArea.setPosition(0, 32);
    //m_GridScrollArea.setSize(gridareaw, gridareah);
    m_GridScrollArea.setBackgroundColor(m_Grid.getBackgroundColor());
    add(&m_GridScrollArea);

    g_GameMaster.GetSkinManager()->ApplyWindowSkin(this);
    resizeToContent();
}
//---------------------------------------------------------------------------
void TAreaGridWindow::Show()
{
    const int scmaxh = 512;

    const int scbarw = g_Ini.GetIntValue(TOpenFarenIni::SCROLLBAR_WIDTH);
    m_AreaGridModel.Init();
    int gridw = m_AreaGridModel.GetWidth();
    int gridh = m_AreaGridModel.GetHeight();
    m_Grid.setSize(gridw, gridh);
    m_GridScrollArea.setSize(gridw + scbarw, min(gridh, scmaxh));
    m_ButtonClose.setPosition(gridw + scbarw - 64, 4);
    resizeToContent();
    setVisible(true);
    g_GuiContainerList.GetGui()->getTop()->moveToTop(this);
    g_GuiContainerList.MoveWidgetToCenter(this);
}
//---------------------------------------------------------------------------
void TAreaGridModel::AdjustColumnWidth()
{
    static const int minw[COLS] = {128, 128, 40, 40, 40, 40, 40, 64, 80};
    static const int maxw[COLS] = {256, 256, 40, 40, 40, 40, 40, 64, 80};
    m_ColumnWidth.resize(COLS);
    m_ColumnWidth.assign(minw, minw + COLS);
    int rows = m_StringValues.size();
    for(int i=0; i<2; ++i){
        for(int row=0; row<rows; ++row){
            int w = m_StringValues[row][i].size() * TGuiContainerList::FONT_SIZE_SMALL / 2 + 4;
            if(m_ColumnWidth[i] < w){
                m_ColumnWidth[i] = w;
            }
        }
    }
    for(int i=0; i<COLS; ++i){
        if(m_ColumnWidth[i] > maxw[i]){
            m_ColumnWidth[i] = maxw[i];
        }
    }
}
//---------------------------------------------------------------------------
int TMasterGridModel::GetNumberOfColumns()
{
    return COLS;
}
//---------------------------------------------------------------------------
int TMasterGridModel::GetNumberOfRows()
{
    return m_StringValues.size() + 1;
}
//---------------------------------------------------------------------------
std::string TMasterGridModel::GetElementAt(int col, int row)
{
    static const char *title[COLS] = {"", "{n", "̒n", "R", "", "lސ", "͎w", ""};
    if(!row){
        return title[col];
    }
    return m_StringValues[row - 1][col];
}
//---------------------------------------------------------------------------
void TMasterGridModel::OnSort(int col)
{
    static const bool isstr[COLS] = {true, true, false, false, false, false, false, true};
    bool desc = m_Desc[col];
    SortVec(m_StringValues, col, isstr[col], desc);
    m_Desc.assign(m_Desc.size(), false);
    m_Desc[col] = !desc;
}
//---------------------------------------------------------------------------
int TMasterGridModel::GetColumnWidth(int col)
{
    return m_ColumnWidth[col];
}
//---------------------------------------------------------------------------
int TMasterGridModel::GetRowHeight()
{
    return ROW_HEIGHT;
}
//---------------------------------------------------------------------------
bool TMasterGridModel::GetColumnAlignment(int col)
{
    static const bool al[COLS] = {false, false, true, true, true, true, true, true};
    return al[col];
}
//---------------------------------------------------------------------------
void TMasterGridModel::Init()
{
    m_Desc.clear();
    m_Desc.resize(COLS);
    StructString();
    AdjustColumnWidth();
    OnSort(0);
}
//---------------------------------------------------------------------------
int TMasterGridModel::GetWidth() const
{
    int w = 1;
    for(int i=0; i<m_ColumnWidth.size(); ++i){
        w += m_ColumnWidth[i];
    }
    return w;
}
//---------------------------------------------------------------------------
int TMasterGridModel::GetHeight() const
{
    return (m_StringValues.size() + 1) * ROW_HEIGHT + 1;
}
//---------------------------------------------------------------------------
void TMasterGridModel::StructString()
{
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TOpenFarenData* data = g_GameMaster.GetData();
    int curmaster = game->GetCurrentMasterIndex();
    int rows = game->GetMasterInstanceCount();
    m_StringValues.clear();
    for(int i=0; i<rows; ++i){
        const TMasterInstance& masterins = game->GetMasterInstance(i);
        const TMaster& master = data->GetMaster(masterins.m_type);
        if(!masterins.m_exists){
            continue;
        }
        std::vector<std::string> vstr(COLS);
        // }X^[
        vstr[0] = master.m_name;
        // {n
        int basearea = game->GetMasterBaseArea(i);
        vstr[1] = game->GetAreaInstance(basearea).m_name;
        // ̒n
        vstr[2] = (boost::format("%d") % game->GetMasterAreaCount(i)).str();
        // R
        vstr[3] = (boost::format("%d") % masterins.m_money).str();
        // 
        vstr[4] = (boost::format("%d") % masterins.m_income).str();
        // lސ
        vstr[5] = (boost::format("%d") % game->GetMasterHeroCount(i)).str();
        // ͎w
        vstr[6] = (boost::format("%d") % (game->GetMasterPower(i)/1000)).str();
        // 
        int alturn = 0;
        if(curmaster != i){
            int alindex = game->FindAlliance(curmaster, i);
            if(alindex != -1){
                const TAlliance& al = game->GetAlliance(alindex);
                alturn = al.turn;
            }
        }
        if(alturn){
            if(alturn == -1){
                vstr[7] = "";
            }
            else{
                vstr[7] = (boost::format("%d") % alturn).str();
            }
        }
        else{
            vstr[7] = "-";
        }
        m_StringValues.push_back(vstr);
    }
}
//---------------------------------------------------------------------------
void TMasterGridModel::AdjustColumnWidth()
{
    static const int minw[COLS] = {128, 128, 40, 40, 40, 40, 64, 40};
    static const int maxw[COLS] = {256, 256, 40, 40, 40, 40, 64, 40};
    m_ColumnWidth.resize(COLS);
    m_ColumnWidth.assign(minw, minw + COLS);
    int rows = m_StringValues.size();
    for(int i=0; i<2; ++i){
        for(int row=0; row<rows; ++row){
            int w = m_StringValues[row][i].size() * TGuiContainerList::FONT_SIZE_SMALL / 2 + 4;
            if(m_ColumnWidth[i] < w){
                m_ColumnWidth[i] = w;
            }
        }
    }
    for(int i=0; i<COLS; ++i){
        if(m_ColumnWidth[i] > maxw[i]){
            m_ColumnWidth[i] = maxw[i];
        }
    }
}
//---------------------------------------------------------------------------
TMasterGridWindow::TMasterGridWindow()
{
    m_ButtonClose.setCaption("");
    m_ButtonClose.setSize(56, 24);
    g_GameMaster.GetSkinManager()->ApplyButtonEnabled(&m_ButtonClose);
    add(&m_ButtonClose);

    const int scbarw = g_Ini.GetIntValue(TOpenFarenIni::SCROLLBAR_WIDTH);
    m_Grid.SetGridModel(&m_MasterGridModel);
    m_Grid.setPosition(0, 0);
    m_Grid.setBackgroundColor(g_GameMaster.GetSkinManager()->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE));

    m_GridScrollArea.setScrollbarWidth(scbarw);
    m_GridScrollArea.setVerticalScrollPolicy(gcn::ScrollArea::SHOW_ALWAYS);
    m_GridScrollArea.setContent(&m_Grid);
    m_GridScrollArea.setPosition(0, 32);
    m_GridScrollArea.setBackgroundColor(m_Grid.getBackgroundColor());
    add(&m_GridScrollArea);

    g_GameMaster.GetSkinManager()->ApplyWindowSkin(this);
    resizeToContent();
}
//---------------------------------------------------------------------------
void TMasterGridWindow::Show()
{
    const int scmaxh = 512;

    const int scbarw = g_Ini.GetIntValue(TOpenFarenIni::SCROLLBAR_WIDTH);
    m_MasterGridModel.Init();
    int gridw = m_MasterGridModel.GetWidth();
    int gridh = m_MasterGridModel.GetHeight();
    m_Grid.setSize(gridw, gridh);
    m_GridScrollArea.setSize(gridw + scbarw, min(gridh, scmaxh));
    m_ButtonClose.setPosition(gridw + scbarw - 64, 4);
    resizeToContent();
    setVisible(true);
    g_GuiContainerList.GetGui()->getTop()->moveToTop(this);
    g_GuiContainerList.MoveWidgetToCenter(this);
}
//---------------------------------------------------------------------------
int THeroGridModel::GetNumberOfColumns()
{
    return COLS;
}
//---------------------------------------------------------------------------
int THeroGridModel::GetNumberOfRows()
{
    return m_StringValues.size() + 1;
}
//---------------------------------------------------------------------------
std::string THeroGridModel::GetElementAt(int col, int row)
{
    static const char *title[COLS] = {"lޖ", "؍݃GA", "xz", "j"};
    if(!row){
        return title[col];
    }
    return m_StringValues[row - 1][col];
}
//---------------------------------------------------------------------------
void THeroGridModel::OnSort(int col)
{
    static const bool isstr[COLS] = {true, true, true, false};
    bool desc = m_Desc[col];
    SortVec(m_StringValues, col, isstr[col], desc);
    m_Desc.assign(m_Desc.size(), false);
    m_Desc[col] = !desc;
}
//---------------------------------------------------------------------------
int THeroGridModel::GetColumnWidth(int col)
{
    return m_ColumnWidth[col];
}
//---------------------------------------------------------------------------
int THeroGridModel::GetRowHeight()
{
    return ROW_HEIGHT;
}
//---------------------------------------------------------------------------
bool THeroGridModel::GetColumnAlignment(int col)
{
    static const bool al[COLS] = {false, false, false, true};
    return al[col];
}
//---------------------------------------------------------------------------
void THeroGridModel::Init()
{
    m_Desc.clear();
    m_Desc.resize(COLS);
    StructString();
    AdjustColumnWidth();
    OnSort(0);
}
//---------------------------------------------------------------------------
int THeroGridModel::GetWidth() const
{
    int w = 1;
    for(int i=0; i<m_ColumnWidth.size(); ++i){
        w += m_ColumnWidth[i];
    }
    return w;
}
//---------------------------------------------------------------------------
int THeroGridModel::GetHeight() const
{
    return (m_StringValues.size() + 1) * ROW_HEIGHT + 1;
}
//---------------------------------------------------------------------------
void THeroGridModel::StructString()
{
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TOpenFarenData* data = g_GameMaster.GetData();
    m_StringValues.clear();
    int areacount = game->GetAreaInstanceCount();
    for(int ai=0; ai<areacount; ++ai){
        const TAreaInstance& area = game->GetAreaInstance(ai);
        if(area.m_ignore){
            continue;
        }
        int characount = area.m_charalist.size();
        for(int i=0; i<characount; ++i){
            const TCharacterInstance& chara = area.m_charalist[i];
            const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
            if(charadata.m_hero || charadata.m_master){
                std::vector<std::string> vstr(COLS);
                // lޖ
                vstr[0] = charadata.m_name;
                // ؍݃GA
                vstr[1] = area.m_name;
                // xz
                if(area.m_master != -1){
                    const TMasterInstance& masterins = game->GetMasterInstance(area.m_master);
                    const TMaster& master = data->GetMaster(masterins.m_type);
                    vstr[2] = master.m_name;
                }
                else{
                    vstr[2] = "";
                }
                // j
                vstr[3] = (boost::format("%d") % chara.m_wincount).str();
                m_StringValues.push_back(vstr);
            }
        }
    }
    int deadcount = game->GetDeadCharaCount();
    for(int i=0; i<deadcount; ++i){
        const TCharacterInstance& chara = game->GetDeadChara(i);
        const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
        std::vector<std::string> vstr(COLS);
        // lޖ
        vstr[0] = charadata.m_name;
        // ؍݃GA
        vstr[1] = "S";
        // xz
        vstr[2] = "||||";
        // j
        vstr[3] = (boost::format("%d") % chara.m_wincount).str();
        m_StringValues.push_back(vstr);
    }
}
//---------------------------------------------------------------------------
void THeroGridModel::AdjustColumnWidth()
{
    static const int minw[COLS] = {128, 128, 128, 48};
    static const int maxw[COLS] = {256, 256, 256, 48};
    m_ColumnWidth.resize(COLS);
    m_ColumnWidth.assign(minw, minw + COLS);
    int rows = m_StringValues.size();
    for(int i=0; i<3; ++i){
        for(int row=0; row<rows; ++row){
            int w = m_StringValues[row][i].size() * TGuiContainerList::FONT_SIZE_SMALL / 2 + 4;
            if(m_ColumnWidth[i] < w){
                m_ColumnWidth[i] = w;
            }
        }
    }
    for(int i=0; i<COLS; ++i){
        if(m_ColumnWidth[i] > maxw[i]){
            m_ColumnWidth[i] = maxw[i];
        }
    }
}
//---------------------------------------------------------------------------
THeroGridWindow::THeroGridWindow()
{
    //const int gridareaw = 512;  // łgȂB
    //const int gridareah = 480;

    m_ButtonClose.setCaption("");
    m_ButtonClose.setSize(56, 24);
    //m_ButtonClose.setPosition(gridareaw - 64, 4);
    g_GameMaster.GetSkinManager()->ApplyButtonEnabled(&m_ButtonClose);
    add(&m_ButtonClose);

    const int scbarw = g_Ini.GetIntValue(TOpenFarenIni::SCROLLBAR_WIDTH);
    m_Grid.SetGridModel(&m_HeroGridModel);
    m_Grid.setPosition(0, 0);
    //m_Grid.setSize(gridareaw - scbarw, gridareah);
    m_Grid.setBackgroundColor(g_GameMaster.GetSkinManager()->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE));

    m_GridScrollArea.setScrollbarWidth(scbarw);
    m_GridScrollArea.setVerticalScrollPolicy(gcn::ScrollArea::SHOW_ALWAYS);
    m_GridScrollArea.setContent(&m_Grid);
    m_GridScrollArea.setPosition(0, 32);
    //m_GridScrollArea.setSize(gridareaw, gridareah);
    //m_GridScrollArea.setHeight(gridareah);
    m_GridScrollArea.setBackgroundColor(m_Grid.getBackgroundColor());
    add(&m_GridScrollArea);

    g_GameMaster.GetSkinManager()->ApplyWindowSkin(this);
    //resizeToContent();
}
//---------------------------------------------------------------------------
void THeroGridWindow::Show()
{
    const int scmaxh = 512;

    const int scbarw = g_Ini.GetIntValue(TOpenFarenIni::SCROLLBAR_WIDTH);
    m_HeroGridModel.Init();
    int gridw = m_HeroGridModel.GetWidth();
    int gridh = m_HeroGridModel.GetHeight();
    m_Grid.setSize(gridw, gridh);
    m_GridScrollArea.setSize(gridw + scbarw, min(gridh, scmaxh));
    m_ButtonClose.setPosition(gridw + scbarw - 64, 4);
    resizeToContent();
    setVisible(true);
    g_GuiContainerList.GetGui()->getTop()->moveToTop(this);
    g_GuiContainerList.MoveWidgetToCenter(this);
}
//---------------------------------------------------------------------------
