//---------------------------------------------------------------------------
#ifndef gridimplH
#define gridimplH

#include <string>
#include <vector>
#include <guichan.hpp>
#include "grid.h"
#include "gcnwindow.h"
//---------------------------------------------------------------------------
class TAreaGridModel : public TGcnGridModel
{
public:
    virtual ~TAreaGridModel() {}
    virtual int GetNumberOfColumns();
    virtual int GetNumberOfRows();
    virtual std::string GetElementAt(int col, int row);
    virtual void OnSort(int col);
    virtual int GetColumnWidth(int col);
    virtual int GetRowHeight();
    virtual bool GetColumnAlignment(int col);
    void Init();
    int GetWidth() const;
    int GetHeight() const;
private:
    static const int COLS = 9;
    static const int ROW_HEIGHT = 14;
    std::vector<std::vector<std::string> > m_StringValues;
    std::vector<int> m_ColumnWidth;
    std::deque<bool> m_Desc;

    void StructString();
    void AdjustColumnWidth();
};
//---------------------------------------------------------------------------
class TAreaGridWindow : public TGcnWindow
{
public:
    TAreaGridWindow();
    virtual ~TAreaGridWindow() {}
    void Show();
    gcn::Button& GetButtonClose() { return m_ButtonClose; }
private:
    gcn::Button m_ButtonClose;
    gcn::ScrollArea m_GridScrollArea;
    TGcnGrid m_Grid;
    TAreaGridModel m_AreaGridModel;
};
//---------------------------------------------------------------------------
class TMasterGridModel : public TGcnGridModel
{
public:
    virtual ~TMasterGridModel() {}
    virtual int GetNumberOfColumns();
    virtual int GetNumberOfRows();
    virtual std::string GetElementAt(int col, int row);
    virtual void OnSort(int col);
    virtual int GetColumnWidth(int col);
    virtual int GetRowHeight();
    virtual bool GetColumnAlignment(int col);
    void Init();
    int GetWidth() const;
    int GetHeight() const;
private:
    static const int COLS = 8;
    static const int ROW_HEIGHT = 14;
    std::vector<std::vector<std::string> > m_StringValues;
    std::vector<int> m_ColumnWidth;
    std::deque<bool> m_Desc;

    void StructString();
    void AdjustColumnWidth();
};
//---------------------------------------------------------------------------
class TMasterGridWindow : public TGcnWindow
{
public:
    TMasterGridWindow();
    virtual ~TMasterGridWindow() {}
    void Show();
    gcn::Button& GetButtonClose() { return m_ButtonClose; }
private:
    gcn::Button m_ButtonClose;
    gcn::ScrollArea m_GridScrollArea;
    TGcnGrid m_Grid;
    TMasterGridModel m_MasterGridModel;
};
//---------------------------------------------------------------------------
class THeroGridModel : public TGcnGridModel
{
public:
    virtual ~THeroGridModel() {}
    virtual int GetNumberOfColumns();
    virtual int GetNumberOfRows();
    virtual std::string GetElementAt(int col, int row);
    virtual void OnSort(int col);
    virtual int GetColumnWidth(int col);
    virtual int GetRowHeight();
    virtual bool GetColumnAlignment(int col);
    void Init();
    int GetWidth() const;
    int GetHeight() const;
private:
    static const int COLS = 4;
    static const int ROW_HEIGHT = 14;
    std::vector<std::vector<std::string> > m_StringValues;
    std::vector<int> m_ColumnWidth;
    std::deque<bool> m_Desc;

    void StructString();
    void AdjustColumnWidth();
};
//---------------------------------------------------------------------------
class THeroGridWindow : public TGcnWindow
{
public:
    THeroGridWindow();
    virtual ~THeroGridWindow() {}
    void Show();
    gcn::Button& GetButtonClose() { return m_ButtonClose; }
private:
    gcn::Button m_ButtonClose;
    gcn::ScrollArea m_GridScrollArea;
    TGcnGrid m_Grid;
    THeroGridModel m_HeroGridModel;
};
//---------------------------------------------------------------------------
#endif
