#include "guichanjp.h"
#include "gui.h"
#include "logger.h"
//---------------------------------------------------------------------------
TGuichanFontSJIS::TGuichanFontSJIS()
{
#if (USE_OPENGL)
#else
    m_Surface = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCALPHA, 640, 640,
                32, 0x00ff0000, 0x0000ff00, 0x000000ff, 0x00000000);
//                32, 0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff);
    m_Image = new gcn::SDLImage(m_Surface, false);
#endif
}
//---------------------------------------------------------------------------
TGuichanFontSJIS::~TGuichanFontSJIS()
{
#if (USE_OPENGL)
#else
    SDL_FreeSurface(m_Surface);
    delete m_Image;
#endif
}
//---------------------------------------------------------------------------
#if (USE_OPENGL)
bool TGuichanFontSJIS::Initialize(int fontsize, const std::string& fontname)
{
//    return m_Drawer.Load(NULL, NULL, fontsize, fontsize, fontname);
    g_GuiContainerList.SetFontName(fontname);
    m_Drawer = g_GuiContainerList.GetDrawer(fontsize);
    return (m_Drawer != NULL);
}
#else
bool TGuichanFontSJIS::Initialize(SDL_PixelFormat *format, int fontsize, const char* fontname)
{
    return m_Drawer.Initialize(format, fontsize, fontname) != -1;
}
#endif
//---------------------------------------------------------------------------
void TGuichanFontSJIS::drawString(gcn::Graphics *graphics, const std::string &text, int x, int y)
{
    if(text.empty()){
        return;
    }
#if (USE_OPENGL)
    if(!m_Drawer->DrawString(graphics, x, y, text.c_str(), graphics->getColor())){
        OFERRLOG((boost::format("DrawString failed. GetLastError = %s str = \"%s\"") % m_Drawer->GetLastError() % text).str());
    }
#else
    gcn::SDLGraphics *sdlg = (gcn::SDLGraphics*)graphics;
    gcn::ClipRectangle r = graphics->getCurrentClipArea();
    SDL_Rect srcr, dstr;
    int w = getWidth(text);
    int h = getHeight();
    srcr.x = r.x + x;
    srcr.y = r.y + y;
    srcr.w = w;
    srcr.h = h;
    dstr.x = 0;
    dstr.y = 0;
    dstr.w = w;
    dstr.h = h;
    SDL_BlitSurface(sdlg->getTarget(), &srcr, m_Surface, &dstr);
    gcn::Color c = graphics->getColor();
    if(!c.r && !c.g && !c.b){
        c.r = c.g = c.b = 1;
    }
    Uint32 rgba = ((c.r<<24) | (c.g<<16) | (c.b<<8) | c.a);
    m_Drawer.DrawString(m_Surface, 0, 0, text.c_str(), rgba);
    graphics->drawImage(m_Image, 0, 0, x, y, m_Drawer.GetStringWidth(text.c_str()), m_Drawer.GetCharHeight());
#endif
}
//---------------------------------------------------------------------------
int TGuichanFontSJIS::getHeight() const
{
    return m_Drawer->GetCharHeight();
}
//---------------------------------------------------------------------------
int TGuichanFontSJIS::getWidth(const std::string& text) const
{
    return m_Drawer->GetStringWidth(text.c_str());
}
//---------------------------------------------------------------------------
