//---------------------------------------------------------------------------
#include "hintpane.h"
#include "gui.h"
#include "drawutil.h"
#include "logger.h"
//---------------------------------------------------------------------------
TGcnHintPane::TGcnHintPane() :
    m_Alpha(0),
    m_ShowStep(0)
{
    setBaseColor(gcn::Color(255,255,225));
    m_Drawer = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_SMALL);
}
//---------------------------------------------------------------------------
TGcnHintPane::~TGcnHintPane()
{
}
//---------------------------------------------------------------------------
void TGcnHintPane::SetCaption(const std::string& caption)
{
    m_Caption = caption;

    int w, h;
    GetNeedSize(caption, w, h);
    setSize(w, h);
//OFDBGLOG((boost::format("%d,%d") % w % h).str());
}
//---------------------------------------------------------------------------
void TGcnHintPane::draw(gcn::Graphics* g)
{
    gcn::Rectangle r = getDimension();
    r.x = r.y = 0;
    gcn::Rectangle r2 = r;
    r2.x = r2.y = 1;
    r2.width -= 2;
    r2.height -= 2;
    g->setColor(getForegroundColor());
    //g->drawRectangle(r);
    g_DrawUtil.DrawRectangle(g, r);
    g->setColor(getBackgroundColor());
    g->fillRectangle(r2);
    m_Drawer->DrawStringOnRect(g, PADDING, PADDING, r, LINEPADDING, m_Caption.c_str(), getForegroundColor());
//OFDBGLOG((boost::format("%d,%d,%d,%d") % r.x % r.y % r.width % r.height).str());
}
//---------------------------------------------------------------------------
void TGcnHintPane::logic()
{
    if(!m_ShowStep){
        return;
    }
    m_Alpha += m_ShowStep;
    if(m_Alpha <= 0){
        setVisible(false);
    }
    else{
        if(m_Alpha >= 255){
            m_Alpha = 255;
            m_ShowStep = 0;
        }
        gcn::Color c(getBaseColor());
        c.a = m_Alpha;
        setBackgroundColor(c);
        setForegroundColor(gcn::Color(0, 0, 0, m_Alpha));
    }
}
//---------------------------------------------------------------------------
void TGcnHintPane::GoShown()
{
    setVisible(true);
    m_ShowStep = 64;
}
//---------------------------------------------------------------------------
void TGcnHintPane::GoHidden()
{
    m_ShowStep = -64;
}
//---------------------------------------------------------------------------
void TGcnHintPane::GetNeedSize(const std::string& caption, int& w, int& h) const
{
    int fontsize = TGuiContainerList::FONT_SIZE_SMALL;
    w = PADDING + 1;
    h = PADDING + fontsize + 1;
    int maxw = PADDING + 1;
    const char *p = &(*caption.begin());
    while(*p != '\0'){
        if(*p == '\n'){
            h += fontsize + LINEPADDING;
            if(w < maxw){
                w = maxw;
            }
            maxw = PADDING + 1;
        }
        else{
            maxw += fontsize/2;
        }
        ++p;
    }
    if(w < maxw){
        w = maxw;
    }
    w += PADDING + 1;
    h += PADDING + 1;
}
//---------------------------------------------------------------------------
