#include "iniload.h"
#include <cstdlib>
#include <cstring>
#include "boostformat.h"
#include <boost/filesystem/operations.hpp>
#include "logger.h"

namespace
{
    // chr SJIS2byte1byteڂ̒l
    bool is_sjis_2byte_1st(unsigned char chr)
    {
        return (chr >= 0x81 && chr <= 0x9F) || (chr >= 0xE0 && chr <= 0xFC);
    }

    // chr SJIS2byte2byteڂ̒l
    bool is_sjis_2byte_2nd(unsigned char chr)
    {
        return (chr >= 0x40 && chr <= 0x7E) || (chr >= 0x80 && chr <= 0xFC);
    }

    size_t FindChar(const std::string& str, unsigned char chr)
    {
        size_t len = str.length();
        for(size_t i=0; i<len; i++){
            if(str.at(i) == chr){
                return i;
            }
            if(is_sjis_2byte_1st(str.at(i))){
                i++;
            }
            if(!is_sjis_2byte_2nd(str.at(i))){
                return -1;
            }
        }
        return -1;
    }
};
//---------------------------------------------------------------------------
TIniLoader::TIniLoader(){
}
//---------------------------------------------------------------------------
TIniLoader::~TIniLoader(){
}
//---------------------------------------------------------------------------
bool TIniLoader::LoadFromFile(const std::string& filename)
{
    m_ValueMap.clear();
    std::FILE *fp = std::fopen(filename.c_str(), "r");
    if(fp == NULL){
        return false;
    }

    char buf[1024];
    bool isend = false;
    std::string line;
    std::string catkey;
    for(;;){
        if(std::fgets(buf, sizeof buf, fp) == NULL){
            if(line.empty()){
                break;
            }
            else{
                buf[0] = '\0';
                isend = true;
            }
        }
#if defined(WIN32)
        char *p = strchr(buf, '\n');
#else
        char *p = strchr(buf, '\r');
        if(p == NULL){
            p = strchr(buf, '\n');
        }
#endif
        if(p != NULL){
            *p = '\0';
            line += buf;
        }
        else{
            line += buf;
            if(!isend){
                continue;
            }
        }

        // sƃRg͖
        if(line.empty() || line.at(0) == '#'){
            line.clear();
            continue;
        }

        // JeSL[XV
        size_t len = line.length();
        if(line.at(0) == '[' && line.at(len-1) == ']'){
            catkey = line.substr(1, len - 2);
            line.clear();
            continue;
        }

        size_t sz = FindChar(line, '=');
        if(sz == -1){
            line.clear();
            continue;
        }

        std::string key = line.substr(0, sz);
        m_ValueMap[catkey][key] = line.c_str() + sz + 1;
        if(isend){
            break;
        }
        line.clear();
    }
    std::fclose(fp);
    return true;
}
//---------------------------------------------------------------------------
bool TIniLoader::GetString(const std::string& catkey, const std::string& key, std::string& val) const
{
    std::map<std::string, std::map<std::string, std::string> >::const_iterator catitr = m_ValueMap.find(catkey);
    if(catitr == m_ValueMap.end()){
        return false;
    }
    std::map<std::string, std::string>::const_iterator itr = catitr->second.find(key);
    if(itr == catitr->second.end()){
        return false;
    }
    val = itr->second;
    return true;
}
//---------------------------------------------------------------------------
bool TIniLoader::GetInt(const std::string& catkey, const std::string& key, int& val) const
{
    std::string strval;
    if(!GetString(catkey, key, strval)){
        return false;
    }
    return sscanf(strval.c_str(), "%d", &val) == 1;
}
//---------------------------------------------------------------------------
bool TIniLoader::GetIntHex(const std::string& catkey, const std::string& key, int& val) const
{
    std::string strval;
    if(!GetString(catkey, key, strval)){
        return false;
    }
    return sscanf(strval.c_str(), "%x", &val) == 1;
}
//---------------------------------------------------------------------------
std::string TIniLoader::GetString(const std::string& catkey, const std::string& key) const
{
    std::string strval;
    GetString(catkey, key, strval);
    return strval;
}
//---------------------------------------------------------------------------
int TIniLoader::GetInt(const std::string& catkey, const std::string& key) const
{
    std::string strval;
    if(!GetString(catkey, key, strval)){
        return 0;
    }
    int val = 0;
    sscanf(strval.c_str(), "%d", &val);
    return val;
}
//---------------------------------------------------------------------------
int TIniLoader::GetIntHex(const std::string& catkey, const std::string& key) const
{
    std::string strval;
    if(!GetString(catkey, key, strval)){
        return 0;
    }
    int val = 0;
    sscanf(strval.c_str(), "%x", &val);
    return val;
}
//---------------------------------------------------------------------------
bool TIniLoader::SaveToFile(const std::string& filename) const
{
    std::FILE *fp = std::fopen(filename.c_str(), "r");
    if(fp == NULL){
        return false;
    }

    const char *tmpfile = "tmp.txt";
    std::FILE *fp_out = std::fopen(tmpfile, "w");
    if(fp_out == NULL){
        std::fclose(fp);
        return false;
    }

    std::map<std::string, std::map<std::string, std::string> >::const_iterator catitr = m_ValueMap.end();
    std::map<std::string, bool> writtenFlagCat; // oJeS̃tOB
    std::map<std::string, bool> writtenFlag;    // JeSŏoGg̃tOB
    char buf[1024];
    bool isend = false;
    std::string linestr;
    std::string catkey;
    bool knownCategory = false;
    for(;;){
        if(std::fgets(buf, sizeof buf, fp) == NULL){
            if(linestr.empty()){
                break;
            }
            else{
                buf[0] = '\0';
                isend = true;
            }
        }
#if defined(WIN32)
        char *p = strchr(buf, '\n');
#else
        char *p = strchr(buf, '\r');
#endif
        if(p != NULL){
            *p = '\0';
            linestr += buf;
        }
        else{
            linestr += buf;
            if(!isend){
                continue;
            }
        }

        // sƃRg͂̂܂܏o
        if(linestr.empty() || linestr.at(0) == '#'){
            std::fprintf(fp_out, "%s\n", linestr.c_str());
            linestr.clear();
            continue;
        }

        // JeSL[XV
        size_t len = linestr.length();
        if(linestr.at(0) == '[' && linestr.at(len-1) == ']'){
            if(knownCategory){
                // f[^̂JeSĂꍇÃJeŜ܂oĂȂGgoB
                for(std::map<std::string, std::string>::const_iterator itr = catitr->second.begin(); itr != catitr->second.end(); ++itr){
                    if(writtenFlag.find(itr->first) == writtenFlag.end()){
                        std::fprintf(fp_out, "%s=%s\n", itr->first.c_str(), itr->second.c_str());
                    }
                }
                writtenFlag.clear();
            }

            catkey = linestr.substr(1, len - 2);
            writtenFlagCat[catkey] = true;

            catitr = m_ValueMap.find(catkey);
            if(catitr != m_ValueMap.end()){
                // f[^̂JeSB
                knownCategory = true;
            }
            else{
                // f[^̖JeSB
                knownCategory = false;
            }
            std::fprintf(fp_out, "%s\n", linestr.c_str());
            linestr.clear();
            continue;
        }

        size_t sz = FindChar(linestr, '=');
        if(sz == -1){
            std::fprintf(fp_out, "%s\n", linestr.c_str());
            linestr.clear();
            continue;
        }

        bool written = false;
        std::string key = linestr.substr(0, sz);
        if(knownCategory){
            std::map<std::string, std::string>::const_iterator itr = catitr->second.find(key);
            if(itr != catitr->second.end()){
                // f[^̂GgȂt@C̒l̑Ɏ̎Ăf[^oB
                std::fprintf(fp_out, "%s=%s\n", key.c_str(), itr->second.c_str());
                writtenFlag[key] = true;
                written = true;
            }
        }
        // f[^ꍇ̂܂܏oB
        if(!written){
            std::fprintf(fp_out, "%s\n", linestr.c_str());
        }

        if(isend){
            break;
        }
        linestr.clear();
    }
    // ȏŁAIWiiniǂݍ񂾕̏oIB

    if(knownCategory){
        // f[^̂JeSĂꍇÃJeŜ܂oĂȂGgoB
        for(std::map<std::string, std::string>::const_iterator itr = catitr->second.begin(); itr != catitr->second.end(); ++itr){
            if(writtenFlag.find(itr->first) == writtenFlag.end()){
                std::fprintf(fp_out, "%s=%s\n", itr->first.c_str(), itr->second.c_str());
            }
        }
    }

    // ܂oĂȂJeSoB
    for(std::map<std::string, std::map<std::string, std::string> >::const_iterator catitr2 = m_ValueMap.begin(); catitr2 != m_ValueMap.end(); ++catitr2){
        if(writtenFlagCat.find(catitr2->first) == writtenFlagCat.end()){
            std::fprintf(fp_out, "[%s]\n", catitr2->first.c_str());
            for(std::map<std::string, std::string>::const_iterator itr = catitr2->second.begin(); itr != catitr2->second.end(); ++itr){
                std::fprintf(fp_out, "%s=%s\n", itr->first.c_str(), itr->second.c_str());
            }
        }
    }

    std::fclose(fp);
    std::fclose(fp_out);
    // iniւB
    const char *bakfile = "openfaren.bak";
    if(boost::filesystem::exists(bakfile)){
        if(remove(bakfile)){
            OFERRLOG((boost::format("%s ̍폜Ɏs܂B̌ۂ҂ɕ񍐂ĉB") % bakfile).str());
        }
    }
    if(!rename(filename.c_str(), bakfile)){
        if(rename(tmpfile, filename.c_str())){
            OFERRLOG((boost::format("%s => %S renameɎs܂B̌ۂ҂ɕ񍐂ĉB") % tmpfile % filename).str());
        }
    }
    else{
        OFERRLOG((boost::format("%s => %S renameɎs܂B̌ۂ҂ɕ񍐂ĉB") % filename % bakfile).str());
    }
    return true;
}
//---------------------------------------------------------------------------
void TIniLoader::SetString(const std::string& catkey, const std::string& key, std::string& val)
{
    std::map<std::string, std::map<std::string, std::string> >::iterator catitr = m_ValueMap.find(catkey);
    if(catitr == m_ValueMap.end()){
        std::map<std::string, std::string> cat;
        catitr = m_ValueMap.insert(std::pair<std::string, std::map<std::string, std::string> >(catkey, cat)).first;
    }
    std::map<std::string, std::string>::iterator itr = catitr->second.find(key);
    if(itr == catitr->second.end()){
        catitr->second.insert(std::pair<std::string, std::string>(key, val));
    }
    else{
        itr->second = val;
    }
}
//---------------------------------------------------------------------------
void TIniLoader::SetInt(const std::string& catkey, const std::string& key, int val)
{
    std::string valstr = (boost::format("%d") % val).str();
    return SetString(catkey, key, valstr);
}
