//---------------------------------------------------------------------------
#include "logger.h"
#include "boostformat.h"
#include <boost/date_time/posix_time/posix_time.hpp>

TOpenFarenLogger g_Logger;

using namespace boost::posix_time;

//---------------------------------------------------------------------------
TOpenFarenLogger::TOpenFarenLogger() :
    m_LogLevel(LOGLEVEL_WARNING),
    m_EveryFlush(false)
{
    m_fp = std::fopen("log.txt", "w");
}
//---------------------------------------------------------------------------
TOpenFarenLogger::~TOpenFarenLogger()
{
    if(m_fp != NULL){
        std::fclose(m_fp);
    }
}
//---------------------------------------------------------------------------
void TOpenFarenLogger::SetLogLevel(int level)
{
    m_LogLevel = level;
}
//---------------------------------------------------------------------------
void TOpenFarenLogger::ErrorLog(const std::string& str)
{
    if(m_fp != NULL){
        //std::fprintf(m_fp, "%s Error %s\n", to_simple_string(microsec_clock::local_time()).c_str(), str.c_str());
        std::fprintf(m_fp, "%s Error %s\n", GetNowDateStr().c_str(), str.c_str());
        if(m_EveryFlush){
            std::fflush(m_fp);
        }
    }
}
//---------------------------------------------------------------------------
void TOpenFarenLogger::WarningLog(const std::string& str)
{
    if(m_fp != NULL && m_LogLevel >= LOGLEVEL_WARNING){
        //std::fprintf(m_fp, "%s Warning %s\n", to_simple_string(microsec_clock::local_time()).c_str(), str.c_str());
        std::fprintf(m_fp, "%s Warning %s\n", GetNowDateStr().c_str(), str.c_str());
        if(m_EveryFlush){
            std::fflush(m_fp);
        }
    }
}
//---------------------------------------------------------------------------
void TOpenFarenLogger::DebugLog(const std::string& str)
{
    if(m_fp != NULL && m_LogLevel >= LOGLEVEL_DEBUG){
//        std::fprintf(m_fp, "%s Debug %s\n", to_simple_string(microsec_clock::local_time()).c_str(), str.c_str());
        std::fprintf(m_fp, "%s Debug %s\n", GetNowDateStr().c_str(), str.c_str());
        if(m_EveryFlush){
            std::fflush(m_fp);
        }
    }
}
//---------------------------------------------------------------------------
std::string GetNowDateStr()
{
    ptime now = microsec_clock::local_time();
    return (boost::format("%04d/%02d/%02d %02d:%02d:%02d.%06d") %
            static_cast<unsigned short>(now.date().year()) %
            static_cast<unsigned short>(now.date().month()) %
            static_cast<unsigned short>(now.date().day()) %
            now.time_of_day().hours() % now.time_of_day().minutes() %
            now.time_of_day().seconds() % now.time_of_day().fractional_seconds()).str();
}
//---------------------------------------------------------------------------
