#include "magicwindow.h"
#include "gamemaster.h"
#include "ofimage.h"
#include "logger.h"
#include "ofini.h"
#include "gcnskinmanager.h"
#include "gui.h"
#include "drawutil.h"
#include "boostformat.h"
#include "boostlexicalcast.h"

const int MAGICLEVEL_MAX = 4;
const int LINEHEIGHT = 24;
const int BUTTONXWIDTH = 20;
const int LABELWIDTH = 128;
const int BUTTONWIDTH = 20;
const int BUTTONHEIGHT = 20;
const int TITLEWIDTH = 24;
//---------------------------------------------------------------------------
// gp񐔐͍lĂȂBɖ@gقMPΎg閂@͂ƉB
// TODO:蔲߂B@{^̃oOŏCׂB
bool HasUsableMagic(const TCharacterInstance& chara)
{
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
    int groups = data->GetMagicGroupCount();
    for(int groupindex = 0; groupindex < groups; ++groupindex){
        const TMagicGroup& group = data->GetMagicGroup(groupindex);
        for(unsigned u=0; u<group.m_magiclist.size(); ++u){
            const TMagicData& magicdata = data->GetMagicData(group.m_magiclist[u]);
            for(unsigned rank=0; rank<magicdata.m_magics.size(); ++rank){
                const TMagic& magic = magicdata.m_magics[rank];
                const TSpecial& spe = data->GetSpecial(magic.m_special);
                if(magic.m_needrank <= charadata.m_magicrank[groupindex] && spe.m_consumemp <= chara.m_mp){
                    return true;
                }
            }
        }
    }
    return false;
}
//---------------------------------------------------------------------------
// w肵@w肵L̎g閂@ǂ肷B
bool IsUsableMagic(const TCharacterInstance& chara, int special)
{
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
    int groups = data->GetMagicGroupCount();
    for(int groupindex = 0; groupindex < groups; ++groupindex){
        const TMagicGroup& group = data->GetMagicGroup(groupindex);
        for(unsigned u=0; u<group.m_magiclist.size(); ++u){
            const TMagicData& magicdata = data->GetMagicData(group.m_magiclist[u]);
            for(unsigned rank=0; rank<magicdata.m_magics.size(); ++rank){
                const TMagic& magic = magicdata.m_magics[rank];
                if(special == magic.m_special){
                    const TSpecial& spe = data->GetSpecial(special);
                    if(magic.m_needrank <= charadata.m_magicrank[groupindex] && spe.m_consumemp <= chara.m_mp){
                        return true;
                    }
                    else{
                        return false;
                    }
                }
            }
        }
    }
    return false; // ZnꂽꍇɗB
}
//---------------------------------------------------------------------------
TMagicGroupPane::TMagicGroupPane() :
    m_ActionListener(NULL),
    m_ActiveMagicIndex(-1)
{
    m_CharDrawer = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_SMALL);
    m_CharDrawerBig = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_NORMAL);
    addMouseListener(this);
    m_ActiveButtonRect.x = -1;
}
//---------------------------------------------------------------------------
void TMagicGroupPane::SetMagicGroup(int charatype, int mp, int groupindex, const std::vector<int> *usecount)
{
    Clear();
    m_CharaType = charatype;
    m_MP = mp;
    m_MagicGroupIndex = groupindex;
    m_MagicUseCount = usecount;
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(charatype);
    const TMagicGroup& group = data->GetMagicGroup(groupindex);
    m_SpecialList.assign(group.m_magiclist.size()*MAGICLEVEL_MAX, -1);
    m_ButtonEnabledList.assign(group.m_magiclist.size()*MAGICLEVEL_MAX, false);
    for(unsigned u=0; u<group.m_magiclist.size(); ++u){
        int mdindex = group.m_magiclist[u];
        const TMagicData& magicdata = data->GetMagicData(mdindex);
        // gp񐔐
        bool limitng = (magicdata.m_limit && (*m_MagicUseCount)[mdindex] >= magicdata.m_limit);
        bool bExists = false;
        for(unsigned rank=0; rank<magicdata.m_magics.size(); ++rank){
            const TMagic& magic = magicdata.m_magics[rank];
            const TSpecial& spe = data->GetSpecial(magic.m_special);
            // gp\Ȃ{^ǉB
//g_Logger.DebugLog((boost::format("%s need=%d rank=%d needmp=%d mp=%d") % spe.m_name % magic.m_needrank % charadata.m_magicrank[groupindex] % spe.m_consumemp % chara.m_mp).str());
            if(magic.m_needrank <= charadata.m_magicrank[groupindex]){
                m_ButtonEnabledList[u*MAGICLEVEL_MAX + rank] = (spe.m_consumemp <= mp && !limitng);
                m_SpecialList[u*MAGICLEVEL_MAX + rank] = magic.m_special;
                bExists = true;
            }
        }
        if(bExists){
            m_MagicGroupIndexList.push_back(u);
            m_MagicIndexList.push_back(group.m_magiclist[u]);
        }
    }
}
//---------------------------------------------------------------------------
void TMagicGroupPane::Clear()
{
    m_SpecialList.clear();
    m_MagicGroupIndexList.clear();
    m_MagicIndexList.clear();
    m_ActiveMagicIndex = -1;
}
//---------------------------------------------------------------------------
void TMagicGroupPane::SetActionListener(gcn::ActionListener* listener)
{
    m_ActionListener = listener;
}
//---------------------------------------------------------------------------
int TMagicGroupPane::GetSpecialAt(int x, int y, int& line, int& rank, int& magicindex, bool& usable)
{
    int mx = x - LABELWIDTH;
    if(mx < 0){
        return -1;
    }
    int my = y - TITLEWIDTH;
    if(my < 0 || my%LINEHEIGHT > BUTTONHEIGHT){
        return -1;
    }
    line = (my / LINEHEIGHT);
    rank = mx / BUTTONXWIDTH;
    if(rank >= MAGICLEVEL_MAX){
        return -1;
    }
    int magicgroupindex;
    if(line < m_MagicIndexList.size()){
        magicindex = m_MagicIndexList[line];
        magicgroupindex = m_MagicGroupIndexList[line];
    }
    else{
        return -1;
    }
    int index = magicgroupindex * MAGICLEVEL_MAX + rank;
    usable = m_ButtonEnabledList[index];
    return m_SpecialList[index];
}
//---------------------------------------------------------------------------
void TMagicGroupPane::mousePressed(gcn::MouseEvent &mouseEvent)
{
    if(m_ActionListener != NULL){
        if(mouseEvent.getButton() == gcn::MouseEvent::LEFT){
            int row, rank;
            int magicindex;
            bool usable;
            int speindex = GetSpecialAt(mouseEvent.getX(), mouseEvent.getY(), row, rank, magicindex, usable);
            if(usable){
                m_ActionListener->action(gcn::ActionEvent(this, (boost::format("%d") % speindex).str()));
            }
            //int speindex = GetSpecialAt(mouseEvent.getX(), mouseEvent.getY(), magicindex, usable);
            //if(usable){
            //    m_ActionListener->action(gcn::ActionEvent(this, (boost::format("%d") % m_SpecialList[speindex]).str()));
            //}
        }
        else{
            m_ActionListener->action(gcn::ActionEvent(this, "-1"));
        }
    }
}
//---------------------------------------------------------------------------
void TMagicGroupPane::mouseMoved(gcn::MouseEvent &mouseEvent)
{
    int line, rank;
    int magicindex;
    bool usable;
    int speindex = GetSpecialAt(mouseEvent.getX(), mouseEvent.getY(), line, rank, magicindex, usable);
    if(speindex != -1){
        m_ActiveButtonRect.setAll(LABELWIDTH + BUTTONXWIDTH*rank, TITLEWIDTH + LINEHEIGHT*line, BUTTONWIDTH, BUTTONHEIGHT);
        m_ActiveMagicIndex = magicindex;
        m_ActiveSpecialIndex = speindex;
        m_ActiveUsable = usable;
    }
    else{
        m_ActiveMagicIndex = -1;
    }
}
//---------------------------------------------------------------------------
void TMagicGroupPane::mouseExited(gcn::MouseEvent &mouseEvent)
{
    m_ActiveButtonRect.x = -1;
    m_ActiveMagicIndex = -1;
}
//---------------------------------------------------------------------------
void TMagicGroupPane::draw(gcn::Graphics *g)
{
    g->setColor(g_GameMaster.GetSkinManager()->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_MAGICSELECTPANE));
    gcn::Rectangle panerect = getDimension();
    panerect.x = 0;
    panerect.y = 0;
    g->fillRectangle(panerect);

    const TOpenFarenData* data = g_GameMaster.GetData();
    const TMagicGroup& group = data->GetMagicGroup(m_MagicGroupIndex);
    const TBattleCharacter& charadata = data->GetCharacter(m_CharaType);

    gcn::Color charcolor(255, 255, 255);
    gcn::Color disablecolor(128, 128, 128);
    std::string title = (boost::format("%s@") % group.m_name).str();
    int myWidth = getWidth();
    int titlew = title.size() * 8;
    int titlex = (myWidth - titlew) / 2;
    bool enabled = charadata.m_magicrank[m_MagicGroupIndex];
    if(enabled){
        m_CharDrawerBig->DrawStringARGB(g, titlex + 2, 4, title.c_str(), group.m_color);
        g->setColor(IntToGcnColor(group.m_color));
    }
    else{
        g->setColor(disablecolor);
    }
    // gB
    if(titlex > 12){
        gcn::Rectangle decr(8, 10, titlex - 12, 4);
        g->fillRectangle(decr);
    }
    if(titlex + titlew + 12 < myWidth){
        gcn::Rectangle decr(titlex + titlew + 4, 10, myWidth - titlex - titlew - 12, 4);
        g->fillRectangle(decr);
    }
    // ^CgB
    if(enabled){
        g->setColor(charcolor);
    }
    else{
        g->setColor(disablecolor);
    }
    g->drawText(title, titlex, 4);
    g->drawText(title, titlex + 1, 4);

    if(!charadata.m_magicrank[m_MagicGroupIndex]){
        return;
    }
    for(unsigned u=0; u<m_MagicIndexList.size(); ++u){
        const TMagicData& magicdata = data->GetMagicData(m_MagicIndexList[u]);
        //g->drawText(magicdata.m_name, 2, LINEHEIGHT*u + 2);
        m_CharDrawer->DrawString(g, 2, TITLEWIDTH + LINEHEIGHT*u + 4, magicdata.m_name.c_str(), charcolor);
    }
    if(m_ActiveMagicIndex != -1){
        // KC_X\B
        const TMagicData& magicdata = data->GetMagicData(m_ActiveMagicIndex);
        int x = (myWidth - magicdata.m_guidance.size() * 8) / 2;
        g->drawText(magicdata.m_guidance.c_str(), x, getHeight() - 20);
        // lphB
        //const TSpecial& spe = data->GetSpecial(m_ActiveSpecialIndex);
        //if(spe.m_consumemp <= m_MP){
        if(m_ActiveUsable){
            g->setColor(gcn::Color(255, 192, 0));
        }
        else{
            g->setColor(disablecolor);
        }
        g->fillRectangle(m_ActiveButtonRect);
    }
    int line = 0;
    for(unsigned u=0; u<group.m_magiclist.size(); ++u){
        const TMagicData& magicdata = data->GetMagicData(group.m_magiclist[u]);
        bool bExists = false;
        for(unsigned rank=0; rank<magicdata.m_magics.size(); ++rank){
            const TMagic& magic = magicdata.m_magics[rank];
            if(magic.m_needrank <= charadata.m_magicrank[m_MagicGroupIndex]){
                if(m_ButtonEnabledList[u * MAGICLEVEL_MAX + rank]){
                    g->setColor(charcolor);
                }
                else{
                    g->setColor(disablecolor);
                }
                gcn::Rectangle r(LABELWIDTH + BUTTONXWIDTH*rank, TITLEWIDTH + LINEHEIGHT*line, BUTTONWIDTH, BUTTONHEIGHT);
                //g->drawRectangle(r);
                g_DrawUtil.DrawRectangle(g, r);
                g->drawText((boost::format("%d") % rank).str(), r.x + 7, r.y + 2);
                bExists = true;
            }
        }
        if(bExists){
            ++line;
        }
    }
}
//---------------------------------------------------------------------------
TMagicWindow::TMagicWindow()
{
    setOpaque(false);
}
//---------------------------------------------------------------------------
void TMagicWindow::Show(const TCharacterInstance& chara, const std::vector<int> *usecount)
{
    const int PANEWIDTH = 240;
    const int PANEHEIGHT = 264;
    const int PANEXCOUNT = 3;

    Clear();
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
    m_GroupList.clear();
    for(unsigned u=0; u<charadata.m_magicrank.size(); ++u){
        //if(charadata.m_magicrank[u]){
        //    m_GroupList.push_back(u);
        //}
        m_GroupList.push_back(u);
    }

    //m_PaneList.resize(m_GroupList.size());
    for(unsigned u=0; u<m_GroupList.size(); ++u){
        //m_PaneList[u].setPosition((u%PANEXCOUNT) * PANEWIDTH, (u/PANEXCOUNT) * PANEHEIGHT);
        //m_PaneList[u].setSize(PANEWIDTH, PANEHEIGHT);
        //m_PaneList[u].SetMagicGroup(chara, m_GroupList[u]);
        //add(&m_PaneList[u]);
        boost::shared_ptr<TMagicGroupPane> pw(new TMagicGroupPane());
        pw->setPosition((u%PANEXCOUNT) * PANEWIDTH, (u/PANEXCOUNT) * PANEHEIGHT);
        pw->setSize(PANEWIDTH, PANEHEIGHT);
        pw->SetMagicGroup(chara.m_type, chara.m_mp, m_GroupList[u], usecount);
        pw->SetActionListener(m_ActionListener);
        m_PaneList.push_back(pw);
        add(pw.get());
    }
    resizeToContent();
    setVisible(true);
    g_GuiContainerList.MoveWidgetToCenter(this);
    g_GuiContainerList.GetGui()->getTop()->moveToTop(this);
}
//---------------------------------------------------------------------------
void TMagicWindow::Clear()
{
    m_GroupList.clear();
    // yC폜B
    clear();
    m_PaneList.clear();
}
//---------------------------------------------------------------------------
void TMagicWindow::SetActionListener(gcn::ActionListener* listener)
{
    m_ActionListener = listener;
}
//---------------------------------------------------------------------------
bool TMagicWindow::IsMyAction(const gcn::Widget* source) const
{
    for(unsigned u=0; u<m_PaneList.size(); ++u){
        if(m_PaneList[u].get() == source){
            return true;
        }
    }
    return false;
}
//---------------------------------------------------------------------------
