//---------------------------------------------------------------------------
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/shared_ptr.hpp>
#include "masterselect.h"
#include "ofdata.h"
#include "ofimage.h"
#include "ofini.h"
#include "msgdlg.h"
#include "gui.h"
#include "gamemaster.h"
#include "gcnskinmanager.h"

#include <boost/format.hpp>
#include "logger.h"

#define MASTER_WIDTH    128
#define MASTER_HEIGHT   128
//---------------------------------------------------------------------------
TGcnMasterPane::TGcnMasterPane() :
    m_ContainData(false),
    m_Selected(0),
    m_XCount(0),
    m_YCount(2)
{
    m_Drawer = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_NORMAL);
    addMouseListener(this);
}
//---------------------------------------------------------------------------
void TGcnMasterPane::draw(gcn::Graphics *g)
{
    static const gcn::Color shade(0, 0, 0);
    static const char *player = "Player";

    if(!m_ContainData){
        return;
    }

    const TOpenFarenData *data = g_GameMaster.GetData();

    //TGcnCharDrawer *drawer = g_GuiContainerList.GetDrawer(g_Ini.GetFontHeight());
    TGcnCharDrawer *drawer = m_Drawer;
    g->setColor(getBackgroundColor());
    gcn::Rectangle r(0, 0, getWidth(), getHeight());
    g->fillRectangle(r);
    int count = m_MasterIndexList.size();
    for(int i=0; i<count; ++i){
        //int x = i%m_XCount;
        //int y = i/m_XCount;
        int x = i/m_YCount;
        int y = i%m_YCount;
        int index = m_MasterIndexList[i];
        if(index != -1){
            int chrindex = data->GetMaster(index).m_facecharaindex;
            if(chrindex != -1){
                const TBattleCharacter& chara = data->GetCharacter(chrindex);
                int facex = x*MASTER_WIDTH;
                int facey = y*MASTER_HEIGHT;
                g_GameMaster.GetImage()->DrawCharaFaceOrChip(g, facex+16, facey, chara.m_faceindex, chara.m_chipindex);
                if(drawer){
                    int w = drawer->GetStringWidth(chara.m_name.c_str());
                    int margin = (MASTER_WIDTH - w) / 2;
                    drawer->DrawString(g, facex+margin, facey+100, chara.m_name.c_str(), getForegroundColor());
                    if(m_Selected[i]){
                        w = drawer->GetStringWidth("Player");
                        margin = (MASTER_WIDTH - w) / 2;
                        drawer->DrawString(g, facex+margin+1, facey+72, player, shade);
                        drawer->DrawString(g, facex+margin, facey+72, player, getForegroundColor());
                    }
                }
            }
        }
    }
}
//---------------------------------------------------------------------------
void TGcnMasterPane::StartOperation()
{
    const TOpenFarenData* data = g_GameMaster.GetData();
    // vC[gp\ȃ}X^[𐔂B
    int allcount = data->GetChapter().GetMasterCount();
    int count = 0;
    for(int i=0; i<allcount; ++i){
        if(!data->GetChapter().GetMaster(i).m_notplayer){
            ++count;
        }
    }
    // ̃[vŎgp\}X^[𐔂ƓɂKviς邱Ƃ΁jB
    m_MasterIndexList.resize(count);
    int index = 0;
    for(int i=0; i<allcount; ++i){
        const TFarenChapterMaster& chapmaster = data->GetChapter().GetMaster(i);
        if(!chapmaster.m_notplayer){
            m_MasterIndexList[index] = g_GameMaster.GetData()->GetMasterIndex(chapmaster.m_id);
            ++index;
        }
    }
    m_Selected.resize(count);
    std::fill(m_Selected.begin(), m_Selected.end(), 0);
    m_XCount = (count + m_YCount - 1) / m_YCount;
    int w = MASTER_WIDTH*m_XCount;
    int h = MASTER_HEIGHT*m_YCount;
    m_Browsed = -1;

    setWidth(w);
    setHeight(h);
    m_ContainData = true;
}
//---------------------------------------------------------------------------
void TGcnMasterPane::EndOperation()
{
    m_ContainData = false;
}
//---------------------------------------------------------------------------
void TGcnMasterPane::SetSelected(int x, int y)
{
    if(x < 0 || y < 0 || x >= MASTER_WIDTH*m_XCount || y >= MASTER_HEIGHT*m_YCount){
        return;
    }
    int sx = x / MASTER_WIDTH;
    int sy = y / MASTER_HEIGHT;
    //int i = sx + sy*m_XCount;
    int i = sx*m_YCount + sy;
    if(i < m_MasterIndexList.size()){
        m_Selected[i] ^= 1;
    }
}
//---------------------------------------------------------------------------
void TGcnMasterPane::SetBrowsed(int x, int y)
{
    if(x < 0 || y < 0 || x >= MASTER_WIDTH*m_XCount || y >= MASTER_HEIGHT*m_YCount){
        return;
    }
    int sx = x / MASTER_WIDTH;
    int sy = y / MASTER_HEIGHT;
    //int i = sx + sy*m_XCount;
    int i = sx*m_YCount + sy;
    if(i < m_MasterIndexList.size()){
        m_Browsed = i;
    }
}
//---------------------------------------------------------------------------
void TGcnMasterPane::GetSelectedMasterIndex(std::vector<int>& indice) const
{
    indice.clear();
    int i = 0;
    for(std::vector<int>::const_iterator itr = m_Selected.begin(); itr != m_Selected.end(); itr++){
        if(*itr){
            indice.push_back(m_MasterIndexList[i]);
        }
        i++;
    }
}
//---------------------------------------------------------------------------
void TGcnMasterPane::AllSelect()
{
    m_Selected.assign(m_Selected.size(), 1);
}
//---------------------------------------------------------------------------
void TGcnMasterPane::AllClear()
{
    m_Selected.assign(m_Selected.size(), 0);
}
//---------------------------------------------------------------------------
void TGcnMasterPane::mousePressed(gcn::MouseEvent &mouseEvent)
{
    if(mouseEvent.getButton() == gcn::MouseEvent::LEFT){
        SetSelected(mouseEvent.getX(), mouseEvent.getY());
        mouseEvent.consume();
    }
}
//---------------------------------------------------------------------------
void TGcnMasterPane::mouseMoved(gcn::MouseEvent &mouseEvent)
{
    SetBrowsed(mouseEvent.getX(), mouseEvent.getY());
    mouseEvent.consume();
}
//---------------------------------------------------------------------------
TMasterSelectPane::TMasterSelectPane()
{
    const int WIDTH = 624;

    setSize(WIDTH, 532);
    const TGcnSkinManager *skinmng = g_GameMaster.GetSkinManager();
    gcn::Color dark = skinmng->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE);

    m_MasterPane.setPosition(0, 0);
    m_MasterPane.setForegroundColor(g_Ini.GetFontColor());
    m_MasterPane.setBackgroundColor(dark);

    m_MasterScrollArea.setPosition(4, 4);
    m_MasterScrollArea.setSize(WIDTH - 8, 288);
    m_MasterScrollArea.setContent(&m_MasterPane);
    skinmng->ApplyScrollAreaSkin(&m_MasterScrollArea);
    add(&m_MasterScrollArea);

    m_TextHandle.setPosition(4, 296);
    m_TextHandle.setSize(WIDTH - 8, g_Ini.GetFontHeight());
    m_TextHandle.SetFontSize(g_Ini.GetFontHeight());
    m_TextHandle.setForegroundColor(g_Ini.GetFontColor());
    m_TextHandle.setBackgroundColor(dark);
    add(&m_TextHandle);

    m_TextDiffculty.setPosition(4, 316);
    m_TextDiffculty.setSize(WIDTH - 8, g_Ini.GetFontHeight());
    m_TextDiffculty.SetFontSize(g_Ini.GetFontHeight());
    m_TextDiffculty.setForegroundColor(gcn::Color(255,255,0));
    m_TextDiffculty.setBackgroundColor(dark);
    add(&m_TextDiffculty);

    m_TextExplanation.setPosition(4, 336);
    m_TextExplanation.setSize(WIDTH - 8, 160);
    m_TextExplanation.SetFontSize(g_Ini.GetFontHeight());
    m_TextExplanation.setForegroundColor(g_Ini.GetFontColor());
    m_TextExplanation.setBackgroundColor(dark);
    add(&m_TextExplanation);

    m_ButtonAllSelect.setCaption("SđI");
    m_ButtonAllSelect.setSize(80, 24);
    m_ButtonAllSelect.setPosition(4, 502);
    g_GameMaster.GetSkinManager()->ApplyButtonEnabled(&m_ButtonAllSelect);
    m_ButtonAllSelect.addActionListener(this);
    add(&m_ButtonAllSelect);

    m_ButtonAllClear.setCaption("Sĉ");
    m_ButtonAllClear.setSize(80, 24);
    m_ButtonAllClear.setPosition(108, 502);
    g_GameMaster.GetSkinManager()->ApplyButtonEnabled(&m_ButtonAllClear);
    m_ButtonAllClear.addActionListener(this);
    add(&m_ButtonAllClear);

    m_ButtonOk.setCaption("Jn");
    m_ButtonOk.setSize(56, 24);
    m_ButtonOk.setPosition(WIDTH - 156, 502);
    g_GameMaster.GetSkinManager()->ApplyButtonDisabled(&m_ButtonOk);
    m_ButtonOk.addActionListener(this);
    add(&m_ButtonOk);

    m_ButtonCancel.setCaption("");
    m_ButtonCancel.setSize(56, 24);
    m_ButtonCancel.setPosition(WIDTH - 60, 502);
    g_GameMaster.GetSkinManager()->ApplyButtonEnabled(&m_ButtonCancel);
    m_ButtonCancel.addActionListener(this);
    add(&m_ButtonCancel);

    g_GameMaster.GetSkinManager()->ApplyGrayPaneSkin(this);
}
//---------------------------------------------------------------------------
void TMasterSelectPane::action(const gcn::ActionEvent& actionEvent)
{
    gcn::Widget *src = actionEvent.getSource();
    if(src == &m_ButtonOk){
        std::vector<int> v;
        m_MasterPane.GetSelectedMasterIndex(v);
        g_GameMaster.StartNewGame(v);
        //setVisible(false);
    }
    else if(src == &m_ButtonCancel){
        g_GameMaster.SetMasterSelectVisible(false);
    }
    else if(src == &m_ButtonAllSelect){
        m_MasterPane.AllSelect();
        g_GameMaster.GetSkinManager()->ApplyButtonEnabled(&m_ButtonOk);
    }
    else if(src == &m_ButtonAllClear){
        m_MasterPane.AllClear();
        g_GameMaster.GetSkinManager()->ApplyButtonDisabled(&m_ButtonOk);
    }
}
//---------------------------------------------------------------------------
void TMasterSelectPane::logic()
{
    gcn::Container::logic();
    if(!m_MasterPane.IsContainData()){
        return;
    }
    int browsed = m_MasterPane.GetBrowsed();
    if(browsed != -1){
        TFarenChapterMaster master = g_GameMaster.GetData()->GetChapter().GetMaster(browsed);
        m_TextHandle.SetText(master.m_handle);
        m_TextDiffculty.SetText(master.m_diffculty);
        m_TextExplanation.SetText(master.m_explanation);
    }
    else{
        m_TextExplanation.SetText("");
    }
    std::vector<int> v;
    m_MasterPane.GetSelectedMasterIndex(v);
    if(!v.empty()){
        g_GameMaster.GetSkinManager()->ApplyButtonEnabled(&m_ButtonOk);
    }
    else{
        g_GameMaster.GetSkinManager()->ApplyButtonDisabled(&m_ButtonOk);
    }
}
//---------------------------------------------------------------------------
void TMasterSelectPane::StartOperation()
{
    m_MasterPane.StartOperation();
}
//---------------------------------------------------------------------------
void TMasterSelectPane::EndOperation()
{
    m_MasterPane.EndOperation();
}
//---------------------------------------------------------------------------
TMasterSelectWindow::TMasterSelectWindow()
{
    setOpaque(false);
    m_Pane.setPosition(0, 0);
    add(&m_Pane);
    resizeToContent();
    g_GameMaster.GetSkinManager()->ApplyWindowSkin(this);
}
//---------------------------------------------------------------------------
