//---------------------------------------------------------------------------
#include "msgdlg.h"
#include "gui.h"
#include "ofini.h"
#include "logger.h"
#include "gcnskinmanager.h"
#include "gamemaster.h"
//---------------------------------------------------------------------------
TGcnMsgDlg::TGcnMsgDlg(const std::string& caption, const std::string& msg, int button, int timeout)
    : TGcnWindow(caption),
    m_msglabel(msg),
    m_Pressed(0),
    m_ButtonFlag(0),
    m_Tick(0),
    m_Timeout(timeout)
{
    static const char *caps[BUTTON_MAX] = {"@OK@", "@͂@", "@@", "LZ"};

    for(int i=0; i<BUTTON_MAX; i++){
        m_button[i].setCaption(caps[i]);
        m_button[i].adjustSize();
    }

    add(&m_msglabel);
    for(int i=0; i<BUTTON_MAX; i++){
        m_button[i].addActionListener(this);
        add(&m_button[i]);
    }
    g_GameMaster.GetSkinManager()->ApplyWindowSkin(this);

    m_ButtonFlag = button;
    UpdateContent();
}
//---------------------------------------------------------------------------
TGcnMsgDlg::~TGcnMsgDlg()
{
}
//---------------------------------------------------------------------------
void TGcnMsgDlg::UpdateContent()
{
    int labelxmargin = 8;
    m_msglabel.setPosition(labelxmargin, 16);
    m_msglabel.AdjustSize();
    m_msglabel.setBackgroundColor(g_Ini.GetMsgDlgColor());
    //m_msglabel.setBackgroundColor(gcn::Color(255,255,255));

    int margin = 16;
    int width = margin;
    int bw = 0;
    int bcount = 0;
    for(int i=0; i<BUTTON_MAX; i++){
        if(m_ButtonFlag & (1<<i)){
            width += m_button[i].getWidth() + margin;
            bw += m_button[i].getWidth();
            bcount++;
        }
    }
    if(width < m_msglabel.getWidth() + labelxmargin*2){
        width = m_msglabel.getWidth() + labelxmargin*2;
    }
    margin = (width - bw) / (1 + bcount);
    width = margin;
    int y = m_msglabel.getY() + m_msglabel.getHeight() + 32;
    int bheight = 0;
    for(int i=0; i<BUTTON_MAX; i++){
        if(m_ButtonFlag & (1<<i)){
            m_button[i].setVisible(true);
            m_button[i].setPosition(width, y);
            width += m_button[i].getWidth() + margin;
            bheight = m_button[i].getHeight();
        }
        else{
            m_button[i].setVisible(false);
        }
    }
    setWidth(width + labelxmargin*2);
    setHeight(y + bheight + 32);
    //setBackgroundColor(g_Ini.GetMsgDlgColor());
    setBaseColor(g_Ini.GetMsgDlgColor());
    m_Pressed = 0;
}
//---------------------------------------------------------------------------
void TGcnMsgDlg::Show()
{
    //gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
    gcn::Widget* top = g_GuiContainerList.GetGui()->getTop();
    int x = (top->getWidth() - getWidth()) / 2;
    int y = (top->getHeight() - getHeight()) / 2;
    setPosition(x, y);
/*
    SDL_MouseButtonEvent dmyevent;
    dmyevent.type = SDL_MOUSEBUTTONDOWN;
    dmyevent.button = SDL_BUTTON_LEFT;
    dmyevent.state = SDL_PRESSED;
    dmyevent.x = 0;
    dmyevent.y = 0;
    SDL_PushEvent((SDL_Event*)&dmyevent);
    dmyevent.type = SDL_MOUSEBUTTONUP;
    dmyevent.state = SDL_RELEASED;
    SDL_PushEvent((SDL_Event*)&dmyevent);
*/
    setVisible(true);
    requestModalFocus();
    top->moveToTop(this);
    // OK or YESIB
    if(m_button[0].isVisible()){
        m_button[0].requestFocus();
    }
    else if(m_button[1].isVisible()){
        m_button[1].requestFocus();
    }
    m_Pressed = 0;
/*
    while(g_GuiContainerList.GcnProcessMessages()){
        if(m_Pressed){
            break;
        }
    }
    return m_Pressed;
*/
}
//---------------------------------------------------------------------------
void TGcnMsgDlg::SetLabel(const std::string& msg)
{
    m_msglabel.SetText(msg);
    //m_msglabel.setCaption(msg);
    UpdateContent();
}
//---------------------------------------------------------------------------
void TGcnMsgDlg::SetButton(int button)
{
    m_ButtonFlag = button;
    UpdateContent();
}
//---------------------------------------------------------------------------
void TGcnMsgDlg::SetTimeout(int timeout)
{
    m_Timeout = timeout;
    m_Tick = 0;
}
//---------------------------------------------------------------------------
void TGcnMsgDlg::action(const gcn::ActionEvent& actionEvent)
{
    gcn::Widget* src = actionEvent.getSource();
    for(int i=0; i<BUTTON_MAX; i++){
        if(src == &m_button[i]){
            m_Pressed = (1<<i);
            releaseModalFocus();
            setVisible(false);
            break;
        }
    }
}
//---------------------------------------------------------------------------
void TGcnMsgDlg::logic()
{
    if(isVisible() && m_Timeout){
        if(m_Tick >= m_Timeout){
            releaseModalFocus();
            setVisible(false);
            return;
        }
        ++m_Tick;
    }
}
//---------------------------------------------------------------------------
void TGcnMsgDlg::Show(const std::string& caption, const std::string& msg, int button, int timeout)
{
/*
    boost::shared_ptr<TGcnMsgDlg> dlg(new TGcnMsgDlg(caption, msg, button));
    gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
    //dlg->setAlignment(gcn::Graphics::CENTER);
    top->add(dlg.get());
    int i = dlg->Show();
    top->remove(dlg.get());
    return i;
*/
    if(m_StaticMsgDlg == NULL){
        m_StaticMsgDlg = new TGcnMsgDlg(caption, msg, button, timeout);
        g_GuiContainerList.Add(boost::shared_ptr<Widget>(m_StaticMsgDlg));
        gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
        top->add(m_StaticMsgDlg);
    }
    else{
        m_StaticMsgDlg->setCaption(caption);
        m_StaticMsgDlg->SetLabel(msg);
        m_StaticMsgDlg->SetButton(button);
        m_StaticMsgDlg->SetTimeout(timeout);
    }
    m_StaticMsgDlg->Show();
}
//---------------------------------------------------------------------------
bool TGcnMsgDlg::IsVisible()
{
    return m_StaticMsgDlg->isVisible();
}
//---------------------------------------------------------------------------
TGcnMsgDlg* TGcnMsgDlg::m_StaticMsgDlg;
