//---------------------------------------------------------------------------
#include <algorithm>
#include <exception>
#include <fstream>
#include <iterator>
#include <boost/filesystem/operations.hpp>
//#include <boost/lexical_cast.hpp>
#include <boost/algorithm/string.hpp>
#include "boostlexicalcast.h"
#include "ofdata.h"
#include "ofmixer.h"
#include "ofxml.h"
#include "oftypes.h"
#include "msgdlg.h"
#include "boostformat.h"
#include "logger.h"
#if !defined(WIN32)
#include "ofini.h"
#endif

namespace
{
    // UɂǔjSpecialIDB
    const char* SYSTEM_SPECIAL_ATTACKWALL = "AttackWall";
}

//#include <windows.h>
//---------------------------------------------------------------------------
namespace{
const char *RangeTypeName[RANGE_MAX] = {"None", "Normal", "Me", "Stream", "AllMySide", "AllEnemy", "AllWall",
    "LastDeadMySide", "LastDeadEnemy", "AllDeadMySide", "AllDeadEnemy",
    "LowerThan"};
const char *AttributeTypeName[ATTRTYPE_MAX] = {"None", "Attack", "Boost", "CrashWall", "Summon", "Revive"};
const char *PowerTypeName[POWER_MAX] = {"None", "HP", "MP", "PhysicalAttack", "PhysicalDefense", "Skill", "Speed", "MagicAttack", "MagicDefense"};
const char *AnimationTypeName[ANIMATION_MAX] = {"None", "Chip", "Center", "Stream", "Shot", "ScreenCenter"};
const char *FormulaName[FORMULA_MAX] = {"None", "FTAttack", "FTSpecial", "FTMagic", "Anytime", "FTAttackHit", "FTSpecialHit", "FTMagicHit"};
const char *HitJudgeTypeName[HITJUDGE_MAX] = {"None", "SingleJudge", "DoubleJudge", "SeparateJudge"};
const char *SystemEffectName[SYSTEM_EFFECTMAX] =
        {"None", "NoEffect", "Poison", "Stone", "Paralyze", "Sleep", "Dream", "Death", "Absorb",
         "HPRecover", "MPRecover", "CrashWall", "PlusPhysicalAttack", "PlusPhysicalDefense", "PlusSkill", "PlusSpeed", "PlusMagicAttack", "PlusMagicDefense",
         "PlusHPRecover", "PlusMPRecover", "PlusAttackCount",
         "CurePoison", "CureStone", "CurePralyze", "CureSleep", "CureDream", "CureAll",
         "ChangeResistance", "ChangeMoveType", "Summon", "Revive"};
const char *ReviveTypeName[REVIVE_MAX] = {"None", "Same", "Employable", "FromList"};
const char *EventTypeName[SPECIALEVENT_MAX] = {"None", "MeDead", "MySideDead", "EnemyDead"};
const char *ResistanceName[RES_MAX] = {"none", "weak", "normal", "strong", "absorb"};
}
//---------------------------------------------------------------------------
// 񂩂񋓌^̒lɂBɎsꍇ0ԂB
template <typename ENUM_TYPE>
ENUM_TYPE GetIndex(const char **names, int namecount, const std::string& name)
{
    const char **itr = std::find(names, names + namecount, name);
    if(itr != names + namecount){
        return static_cast<ENUM_TYPE>(std::distance(names, itr));
    }
    else{
        return static_cast<ENUM_TYPE>(0);
    }
}
//---------------------------------------------------------------------------
TResistance GetResistance(const std::string& name)
{
    return GetIndex<TResistance>(ResistanceName, RES_MAX, name);
}
//---------------------------------------------------------------------------
unsigned GetColorABGR(const std::string& str)
{
    unsigned tmp = 0xff000000;
    if(sscanf(str.c_str(), "%x", &tmp) == 1){
        // RBւB
        tmp = (tmp & 0xff00ff00) | ((tmp & 0xff0000)>>16) | ((tmp & 0xff)<<16);
    }
    return tmp;
}
//---------------------------------------------------------------------------
bool TOpenFarenData::LoadNamedType(const std::string& filename, TNamedTypeList& l)
{
    l.clear();

    if(!boost::filesystem::exists(filename)){
        g_Logger.ErrorLog(filename + " file not found");
        return false;
    }

    TiXmlDocument doc(filename.c_str());
    if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog(filename + " XML parse error");
        return false;
    }
    const TiXmlElement *top = doc.FirstChildElement();
    if(top == NULL){
        g_Logger.ErrorLog(filename + " ɒgH");
        return false;
    }
    for(const TiXmlElement *child = top->FirstChildElement(); child != NULL; child = child->NextSiblingElement()){
        TNamedType nt;
        nt.m_id = GetElementAttribute(*child, "id");
        if(nt.m_id.empty()){
            return false;
        }
        const TiXmlElement *e = child->FirstChildElement("Name");
        if(e == NULL){
            return false;
        }
        nt.m_name = GetElementText(*e);
        l.push_back(nt);
    }
    std::sort(l.begin(), l.end(), TIDHolderLess<TNamedType>());
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenData::LoadRace(const std::string& filename)
{
    m_RaceList.clear();

    if(!boost::filesystem::exists(filename)){
        g_Logger.ErrorLog(filename + " file not found");
        return false;
    }

    TiXmlDocument doc(filename.c_str());
    if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog(filename + " XML parse error");
        return false;
    }
    const TiXmlElement *top = doc.FirstChildElement();
    if(top == NULL){
        g_Logger.ErrorLog(filename + " ɒgH");
        return false;
    }
    for(const TiXmlElement *child = top->FirstChildElement(); child != NULL; child = child->NextSiblingElement()){
        TRace race;
        race.m_id = GetElementAttribute(*child, "id");
        if(race.m_id.empty()){
            g_Logger.ErrorLog(filename + " id not found");
            return false;
        }
        const TiXmlElement *e = child->FirstChildElement("Name");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " Name not found");
            return false;
        }
        race.m_name = GetElementText(*e);
        m_RaceList.push_back(race);
    }
    std::sort(m_RaceList.begin(), m_RaceList.end(), TIDHolderLess<TRace>());

    // 񃋁[vB
    //int index = 0;
    for(const TiXmlElement *child = top->FirstChildElement(); child != NULL; child = child->NextSiblingElement()){
        std::string myid = GetElementAttribute(*child, "id");
        int index = GetRaceIndex(myid); // K͂B
        m_RaceList[index].m_distance.resize(m_RaceList.size());
        int othercount = 0;
        const TiXmlElement *e_aff = child->FirstChildElement("Affinity");
        if(e_aff != NULL){
            for(const TiXmlElement *e_dist = e_aff->FirstChildElement(); e_dist != NULL; e_dist = e_dist->NextSiblingElement()){
                std::string id = GetElementAttribute(*e_dist, "id");
                int other = GetRaceIndex(id);
                if(other == -1){
                    OFWRNLOG((boost::format("푰 %s Affinityɖid=%s܂B") % m_RaceList[index].m_id % id).str());
                    continue;
                }
                e_dist->QueryIntAttribute("distance", &m_RaceList[index].m_distance[other]);
                ++othercount;
            }
            if(othercount < m_RaceList.size() - 1){
                OFWRNLOG((boost::format("푰 %s Affinity̑푰%dłiS푰%d1ljB`Ȃ푰̗FDx͍ōƂȂ܂B") % m_RaceList[index].m_id % othercount % (m_RaceList.size() - 1)).str());
            }
        }
        ++index;
    }
    return true;
}
//---------------------------------------------------------------------------
// LoadMoveAndSkill()ɈˑB
bool TOpenFarenData::LoadAttribute(const std::string& filename)
{
    m_AttributeList.clear();

    if(!boost::filesystem::exists(filename)){
        g_Logger.ErrorLog(filename + " file not found");
        return false;
    }

    TiXmlDocument doc(filename.c_str());
    if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog(filename + " XML parse error");
        return false;
    }
    const TiXmlElement *top = doc.FirstChildElement();
    if(top == NULL){
        g_Logger.ErrorLog(filename + " ɒgH");
        return false;
    }
    for(const TiXmlElement *child = top->FirstChildElement(); child != NULL; child = child->NextSiblingElement()){
        TAttribute attr;
        attr.m_id = GetElementAttribute(*child, "id");
        if(attr.m_id.empty()){
            g_Logger.ErrorLog(filename + " id not found");
            return false;
        }

        std::string attrtype = GetElementAttribute(*child, "type");
        if(attrtype.empty()){
            g_Logger.ErrorLog(filename + " type not found");
            return false;
        }
        // ނ̃CfbNXID狁߂B
        attr.m_type = GetIndex<TAttributeType>(AttributeTypeName, ATTRTYPE_MAX, attrtype);
        // ŏ̗vf̂ݑ^CvȂeB
        if(!attr.m_type && !m_AttributeList.empty()){
            g_Logger.ErrorLog(filename + " ^Cv " + attrtype + " ̔FɎs܂B");
            return false;
        }

        const TiXmlElement *e = child->FirstChildElement("Name");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " Name not found");
            return false;
        }
        attr.m_name = GetElementText(*e);
        e = child->FirstChildElement("ShortName");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " ShortName not found");
            return false;
        }
        attr.m_shortname = GetElementText(*e);

        e = child->FirstChildElement("SystemEffect");
        if(e != NULL){
            // VXe`̌ʂ̃CfbNXID狁߂B
            std::string id = GetElementAttribute(*e, "id");
            attr.m_systemeffect = GetIndex<TSystemEffect>(SystemEffectName, SYSTEM_EFFECTMAX, id);
            if(!attr.m_systemeffect){
                g_Logger.ErrorLog(filename + " SystemEffect " + id + " ̔FɎs܂B");
                return false;
            }
        }
        else{
            // m[hꍇ͖ʂɐݒ肷B
            attr.m_systemeffect = SYSTEM_NOEFFECT;
        }
        m_AttributeList.push_back(attr);
    }
    // ̓\[gȂB
    //std::sort(m_AttributeList.begin(), m_AttributeList.end(), TIDHolderLess<TAttribute>());
    return true;
}
//---------------------------------------------------------------------------
// n`Ə㕨̖O[hB̓CfbNXŵ߁AOƐKvƂȂB
bool TOpenFarenData::LoadMapChipType(const std::string& filename)
{
    m_MapGroundList.clear();
    m_MapObjectList.clear();

    if(!boost::filesystem::exists(filename)){
        g_Logger.ErrorLog(filename + " file not found");
        return false;
    }

    TiXmlDocument doc(filename.c_str());
    if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog(filename + " XML parse error");
        return false;
    }
    const TiXmlElement *top = doc.FirstChildElement("GroundList");
    if(top == NULL){
        g_Logger.ErrorLog(filename + " GroundLists");
        return false;
    }
    // n`
    const TiXmlElement *e_gnd = top->FirstChildElement("MapGroundList");
    if(e_gnd == NULL){
        g_Logger.ErrorLog(filename + " MapGroundLists");
        return false;
    }
    for(const TiXmlElement *child = e_gnd->FirstChildElement("MapGround"); child != NULL; child = child->NextSiblingElement("MapGround")){
        int gndindex;
        if(child->QueryIntAttribute("index", &gndindex) != TIXML_SUCCESS){
            g_Logger.ErrorLog(filename + " indexǍs");
            return false;
        }
        if(gndindex != m_MapGroundList.size()){
            g_Logger.WarningLog(filename + " index̒lƂł邩A܂͕słBindex̓`FbN݂̂Ăۂɂ͗vf̕яŃCfbNX蓖Ă܂B");
        }
        const TiXmlElement *e = child->FirstChildElement("Name");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " Names");
            return false;
        }
        m_MapGroundList.push_back(GetElementText(*e));
    }
    // 㕨
    const TiXmlElement *e_obj = top->FirstChildElement("MapObjectList");
    if(e_obj == NULL){
        g_Logger.ErrorLog(filename + " MapObjectLists");
        return false;
    }
    for(const TiXmlElement *child = e_obj->FirstChildElement("MapObject"); child != NULL; child = child->NextSiblingElement("MapObject")){
        int gndindex;
        if(child->QueryIntAttribute("index", &gndindex) != TIXML_SUCCESS){
            g_Logger.ErrorLog(filename + " indexǍs");
            return false;
        }
        if(gndindex != m_MapObjectList.size()){
            g_Logger.WarningLog(filename + " index̒lƂł邩A܂͕słBindex̓`FbN݂̂Ăۂɂ͗vf̕яŃCfbNX蓖Ă܂B");
        }
        const TiXmlElement *e = child->FirstChildElement("Name");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " Names");
            return false;
        }
        m_MapObjectList.push_back(GetElementText(*e));
    }
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenData::LoadRevive(const std::string& filename)
{
    m_ReviveList.clear();

    if(!boost::filesystem::exists(filename)){
        g_Logger.ErrorLog(filename + " file not found");
        return false;
    }

    TiXmlDocument doc(filename.c_str());
    if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog(filename + " XML parse error");
        return false;
    }
    const TiXmlElement *top = doc.FirstChildElement("ReviveList");
    if(top == NULL){
        g_Logger.ErrorLog(filename + " ReviveLists");
        return false;
    }
    for(const TiXmlElement *child = top->FirstChildElement("Revive"); child != NULL; child = child->NextSiblingElement("Revive")){
        TRevive rev;
        rev.m_id = GetElementAttribute(*child, "id");
        const TiXmlElement *e = child->FirstChildElement("SelectMethod");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " SelectMethods");
            return false;
        }
        rev.m_selectmethod = GetIndex<TReviveType>(ReviveTypeName, REVIVE_MAX, GetElementAttribute(*e, "value"));
        if(!rev.m_selectmethod){
            g_Logger.ErrorLog(filename + " ReviveTypeFs");
            return false;
        }
        if(rev.m_selectmethod == REVIVE_FROMLIST){
            const TiXmlElement *e_list = e->FirstChildElement("CharacterList");
            if(e_list == NULL){
                g_Logger.ErrorLog(filename + " ReviveTypeFromListw肳܂CharacterList܂łB");
                return false;
            }
            for(const TiXmlElement *e = e_list->FirstChildElement("Character"); e != NULL; e = e->NextSiblingElement("Character")){
                rev.m_charaidlist.push_back(GetElementAttribute(*e, "id"));
            }
            if(rev.m_charaidlist.empty()){
                g_Logger.ErrorLog(filename + " ReviveTypeFromListw肳܂CharacterListCharacter܂łB");
                return false;
            }
        }
        m_ReviveList.push_back(rev);
    }
    std::sort(m_ReviveList.begin(), m_ReviveList.end(), TIDHolderLess<TRevive>());
    return true;
}
//---------------------------------------------------------------------------
// LoadMapChipType()ɈˑB
bool TOpenFarenData::LoadMoveAndSkill(const std::string& filename)
{
    m_MoveTypeList.clear();

    if(!boost::filesystem::exists(filename)){
        g_Logger.ErrorLog(filename + " file not found");
        return false;
    }

    TiXmlDocument doc(filename.c_str());
    if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog(filename + " XML parse error");
        return false;
    }
    const TiXmlElement *top = doc.FirstChildElement("MoveTypeList");
    if(top == NULL){
        g_Logger.ErrorLog(filename + " MoveTypeLists");
        return false;
    }
    for(const TiXmlElement *child = top->FirstChildElement("MoveType"); child != NULL; child = child->NextSiblingElement("MoveType")){
        TMoveType mt;
        mt.m_id = GetElementAttribute(*child, "id");
        const TiXmlElement *e = child->FirstChildElement("Name");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " Names");
            return false;
        }
        mt.m_name = GetElementText(*e);
        mt.m_cost.resize(GetMapGroundCount() + GetMapObjectCount());
        mt.m_skill.resize(GetMapGroundCount() + GetMapObjectCount());
        int readcount = 0;
        // n`̈ړRXgƃXLB
        for(const TiXmlElement *e = child->FirstChildElement("MapGround"); e != NULL; e = e->NextSiblingElement("MapGround")){
            int gndindex;
            if(e->QueryIntAttribute("index", &gndindex) != TIXML_SUCCESS){
                g_Logger.ErrorLog(filename + " indexǍs");
                return false;
            }
            if(gndindex >= GetMapGroundCount()){
                g_Logger.ErrorLog((boost::format("%s ړ^Cv\"%s\"index %dn`f[^͈̔(%d)𒴂Ă܂B") % filename % mt.m_name % gndindex % GetMapGroundCount()).str());
                return false;
            }
            if(e->QueryIntAttribute("MoveCost", &mt.m_cost[gndindex]) != TIXML_SUCCESS){
                g_Logger.ErrorLog(filename + " MoveCostǍs");
                return false;
            }
            if(e->QueryIntAttribute("Skill", &mt.m_skill[gndindex]) != TIXML_SUCCESS){
                g_Logger.ErrorLog(filename + " SkillǍs");
                return false;
            }
//g_Logger.DebugLog((boost::format("gnd=%d cost=%d skill=%d") % gndindex % mt.m_cost[gndindex] % mt.m_skill[gndindex]).str());
            readcount++;
        }

        // 㕨̈ړRXgƃXLBPɒn`̌ɒǉB
        for(const TiXmlElement *e = child->FirstChildElement("MapObject"); e != NULL; e = e->NextSiblingElement("MapObject")){
            int gndindex;
            if(e->QueryIntAttribute("index", &gndindex) != TIXML_SUCCESS){
                g_Logger.ErrorLog(filename + " indexǍs");
                return false;
            }
            if(gndindex >= GetMapObjectCount()){
                g_Logger.ErrorLog((boost::format("%s ړ^Cv\"%s\"index %d㕨f[^͈̔(%d)𒴂Ă܂B") % filename % mt.m_name % gndindex % GetMapObjectCount()).str());
                return false;
            }
            gndindex += GetMapGroundCount();
            if(e->QueryIntAttribute("MoveCost", &mt.m_cost[gndindex]) != TIXML_SUCCESS){
                g_Logger.ErrorLog(filename + " MoveCostǍs");
                return false;
            }
            if(e->QueryIntAttribute("Skill", &mt.m_skill[gndindex]) != TIXML_SUCCESS){
                g_Logger.ErrorLog(filename + " SkillǍs");
                return false;
            }
//g_Logger.DebugLog((boost::format("gnd=%d cost=%d skill=%d") % gndindex % mt.m_cost[gndindex] % mt.m_skill[gndindex]).str());
            readcount++;
        }
        if(readcount < GetMapGroundCount() + GetMapObjectCount()){
            g_Logger.WarningLog(filename + " " + mt.m_name + " MapObject̐܂Bǂݍ߂Ȃ̃f[^0Ƃ݂Ȃ܂B");
        }
        if(std::find(mt.m_cost.begin(), mt.m_cost.end(), 0) != mt.m_cost.end()){
            g_Logger.ErrorLog((boost::format("%s ړ^Cv\"%s\"̈ړRXg0̗vf܂܂Ă܂B") % filename % mt.m_name).str());
            return false;
        }
        m_MoveTypeList.push_back(mt);
    }
    std::sort(m_MoveTypeList.begin(), m_MoveTypeList.end(), TIDHolderLess<TMoveType>());
    return true;
}
//---------------------------------------------------------------------------
struct TStringToInt
{
    int operator()(const std::string& s){
        return boost::lexical_cast<int>(s);
    }
};

bool TOpenFarenData::LoadIntArrayTxt(const std::string& filename, std::vector<int>& array)
{
    // P[w肵ȂƓ{pX̃t@C̓ǂݍ݂ɎsB
    // ifstream::imbue()ŃP[w肵ĂŝŎdȂglobalwB
    bool localeok = false;
    try{
        std::locale::global(std::locale("japanese"));
        localeok = true;
    }
    catch(std::exception&){
    }
    // eLXgt@CSǂށB
    std::ifstream file(filename.c_str());
    if(!file.good()){
        g_Logger.ErrorLog(filename + " file not found");
        return false;
    }
    std::istreambuf_iterator<char> from(file.rdbuf()), to;
    std::string buf(from, to);
    // 󔒂ŕB
    std::vector<std::string> v;
    boost::algorithm::split(v, buf, boost::is_space());
    // 󕶎łȂvfz̑OɎĂċ󕶎vf폜B
    v.erase(std::remove(v.begin(), v.end(), ""), v.end());
    // vf𐔒lɕϊĔzɓB
    try{
        std::transform(v.begin(), v.end(), std::back_inserter(array), TStringToInt());
    }
    catch(boost::bad_lexical_cast&){
        g_Logger.ErrorLog(filename + " Ǎsi񁨐lϊG[j");
        if(localeok){
            std::locale::global(std::locale::classic());
        }
        return false;
    }
    if(localeok){
        std::locale::global(std::locale::classic());
    }
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenData::LoadMapObjTxt(const std::string& filename)
{
    m_MapObjList.clear();
    return LoadIntArrayTxt(filename, m_MapObjList);
}
//---------------------------------------------------------------------------
bool TOpenFarenData::LoadMapChipTxt(const std::string& filename)
{
    m_MapChipList.clear();
    return LoadIntArrayTxt(filename, m_MapChipList);
}
//---------------------------------------------------------------------------
bool TOpenFarenData::LoadMagicTable(const std::string& filename)
{
    m_MagicDataList.clear();

    TiXmlDocument doc(filename.c_str());
    if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog(filename + " XML parse error");
        return false;
    }
    const TiXmlElement *top = doc.FirstChildElement("MagicList");
    if(top == NULL){
        g_Logger.ErrorLog(filename + " MagicLists");
        return false;
    }
    for(const TiXmlElement *child = top->FirstChildElement("Magic"); child != NULL; child = child->NextSiblingElement("Magic")){
        TMagicData md;
        md.m_id = GetElementAttribute(*child, "id");
        md.m_groupid = GetElementAttribute(*child, "groupid");
        // O
        const TiXmlElement *e = child->FirstChildElement("Name");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " MagicNames");
            return false;
        }
        md.m_name = GetElementText(*e);
        // 
        e = child->FirstChildElement("Guidance");
        if(e != NULL){
            md.m_guidance = GetElementText(*e);
        }
        // gp񐔐
        child->QueryIntAttribute("limit", &md.m_limit);
        // specialid
        for(const TiXmlElement *entry = child->FirstChildElement("Entry"); entry != NULL; entry = entry->NextSiblingElement("Entry")){
            TMagic magic;
            magic.m_specialid = GetElementAttribute(*entry, "specialid");
            //md.m_specialid.push_back(GetElementAttribute(*entry, "specialid"));
            // Kv@x
            //int needrank;
            //if(entry->QueryIntAttribute("rank", &needrank) != TIXML_SUCCESS){
            if(entry->QueryIntAttribute("rank", &magic.m_needrank) != TIXML_SUCCESS){
                g_Logger.ErrorLog("Magicrank");
                return false;
            }
            //md.m_needrank.push_back(needrank);
            md.m_magics.push_back(magic);
        }
        m_MagicDataList.push_back(md);
    }
    //std::sort(m_MagicList.begin(), m_MagicList.end(), TIDHolderLess<TMagicData>());
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenData::LoadMagicGroup(const std::string& filename)
{
    m_MagicGroupList.clear();

    TiXmlDocument doc(filename.c_str());
    if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog(filename + " XML parse error");
        return false;
    }
    const TiXmlElement *top = doc.FirstChildElement("MagicGroupList");
    if(top == NULL){
        g_Logger.ErrorLog(filename + " MagicGroupLists");
        return false;
    }
    for(const TiXmlElement *child = top->FirstChildElement("MagicGroup"); child != NULL; child = child->NextSiblingElement("MagicGroup")){
        TMagicGroup mg;
        mg.m_id = GetElementAttribute(*child, "id");
        // O
        const TiXmlElement *e = child->FirstChildElement("Name");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " MagicGroupNames");
            return false;
        }
        mg.m_name = GetElementText(*e);
        // @O[ṽC[WF
        e = child->FirstChildElement("Color");
        if(e != NULL){
            std::string grpcolor = GetElementText(*e);
            mg.m_color = GetColorABGR(grpcolor);
        }
        // O[vɏ閂@̃CfbNXLB
        for(unsigned u=0; u<m_MagicDataList.size(); ++u){
            if(m_MagicDataList[u].m_groupid == mg.m_id){
                mg.m_magiclist.push_back(u);
//g_Logger.DebugLog((boost::format("group = %s magic = %s") % mg.m_name % m_MagicList[u].m_name).str());
            }
        }
        m_MagicGroupList.push_back(mg);
    }
    return true;
}
//---------------------------------------------------------------------------
// LoadRevive()ɈˑB
bool TOpenFarenData::LoadSpecialTable(const std::string& filename, int dirno)
{
    if(!boost::filesystem::exists(filename)){
        return true;
    }
    TiXmlDocument doc(filename.c_str());
    if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog(filename + " XML parse error");
        return false;
    }
    const TiXmlElement *top = doc.FirstChildElement("SpecialList");
    if(top == NULL){
        g_Logger.ErrorLog(filename + " SpecialLists");
        return false;
    }
    for(const TiXmlElement *child = top->FirstChildElement("Special"); child != NULL; child = child->NextSiblingElement("Special")){
        TSpecial spe;
        spe.m_id = GetElementAttribute(*child, "id");
        // ˒̎
        std::string rangetype = GetElementAttribute(*child, "rangetype");
        spe.m_rangetype = GetIndex<TRangeType>(RangeTypeName, RANGE_MAX, rangetype);
        if(!spe.m_rangetype){
            g_Logger.ErrorLog(filename + " Unknown Range Type " + rangetype);
            return false;
        }
        if(spe.m_rangetype == RANGE_LOWERTHAN){
            if(child->QueryIntAttribute("rangeheight", &spe.m_rangeheight) != TIXML_SUCCESS){
                g_Logger.ErrorLog(filename + " rangetypeLowerThanw肳ꂽrangeheights");
                return false;
            }
        }
        // O
        const TiXmlElement *e = child->FirstChildElement("Name");
        if(e == NULL){
            return false;
        }
        spe.m_name = GetElementText(*e);
        if(!FindIntAttribute(child, "ConsumeMP", "value", &spe.m_consumemp, "id =" + spe.m_id + " ConsumeMP̌Ɏs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "MaxRange", "value", &spe.m_maxrange, "id =" + spe.m_id + " MaxRangěɎs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "MaxRadius", "value", &spe.m_maxradius, "id =" + spe.m_id + " MaxRadiušɎs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "Power", "value", &spe.m_power, "id =" + spe.m_id + " PoweřɎs܂B")){
            return false;
        }
        // UɎgp[^ID狁߂B
        e = child->FirstChildElement("PowerType");
        if(e == NULL){
            g_Logger.ErrorLog((boost::format("%s %s PowerType not found") % filename % spe.m_id).str());
            return false;
        }
        std::string atktype = GetElementText(*e);
        spe.m_powertype = GetIndex<TPowerType>(PowerTypeName, POWER_MAX, atktype);
//        if(!spe.m_powertype){
//            g_Logger.ErrorLog(filename + " U^Cv " + atktype + " ̔FɎs܂B");
//            return false;
//        }
        // hɎgp[^ID狁߂B
        e = child->FirstChildElement("GuardType");
        if(e == NULL){
            g_Logger.ErrorLog((boost::format("%s %s GuardType not found") % filename % spe.m_id).str());
            return false;
        }
        std::string grdtype = GetElementText(*e);
        spe.m_guardtype = GetIndex<TPowerType>(PowerTypeName, POWER_MAX, grdtype);
//        if(!spe.m_guardtype){
//            g_Logger.ErrorLog(filename + " U^Cv " + grdtype + " ̔FɎs܂B");
//            return false;
//        }
        // Ɏgp[^ID狁߂B
        e = child->FirstChildElement("HitType");
        if(e == NULL){
            g_Logger.ErrorLog((boost::format("%s %s HitType not found") % filename % spe.m_id).str());
            return false;
        }
        std::string hittype = GetElementText(*e);
        spe.m_hittype = GetIndex<TPowerType>(PowerTypeName, POWER_MAX, hittype);
//        if(!spe.m_hittype){
//            g_Logger.ErrorLog(filename + " U^Cv " + hittype + " ̔FɎs܂B");
//            return false;
//        }
        // Ɏgp[^ID狁߂B
        e = child->FirstChildElement("AvoidType");
        if(e == NULL){
            g_Logger.ErrorLog((boost::format("%s %s AvoidType not found") % filename % spe.m_id).str());
            return false;
        }
        std::string avoidtype = GetElementText(*e);
        spe.m_avoidtype = GetIndex<TPowerType>(PowerTypeName, POWER_MAX, avoidtype);
//        if(!spe.m_avoidtype){
//            g_Logger.ErrorLog(filename + " U^Cv " + avoidtype + " ̔FɎs܂B");
//            return false;
//        }
        // CfbNX
        e = child->FirstChildElement("Attribute");
        if(e == NULL){
            g_Logger.ErrorLog((boost::format("%s %s Attribute not found") % filename % spe.m_id).str());
            return false;
        }
        spe.m_attribute = GetAttributeIndex(GetElementText(*e));
        if(spe.m_attribute == -1){
            g_Logger.ErrorLog((boost::format("%s %s Attribute ̌Ɏs܂B") % filename % spe.m_id).str());
            return false;
        }
        // XNvg
        for(e = child->FirstChildElement("Script"); e != NULL; e = e->NextSiblingElement("Script")){
            std::string timing = GetElementAttribute(*e, "timing");
            if(timing == "before"){
                spe.m_script_before = GetElementAttribute(*e, "function");
                if(spe.m_script_before.empty()){
                    g_Logger.ErrorLog((boost::format("%s %s Scriptfunctionw肳Ă܂B") % filename % spe.m_id).str());
                    return false;
                }
            }
            else if(timing == "after"){
                spe.m_script_after = GetElementAttribute(*e, "function");
                if(spe.m_script_after.empty()){
                    g_Logger.ErrorLog((boost::format("%s %s Scriptfunctionw肳Ă܂B") % filename % spe.m_id).str());
                    return false;
                }
            }
            else{
                g_Logger.ErrorLog((boost::format("%s %s Scripttiming̉߂Ɏs܂B") % filename % spe.m_id).str());
                return false;
            }
        }
        // Oړ^Cv
        for(e = child->FirstChildElement("ExcludeMoveType"); e != NULL; e = e->NextSiblingElement("ExcludeMoveType")){
            int mt = GetMoveTypeIndex(GetElementAttribute(*e, "id"));
            if(mt == -1){
                g_Logger.ErrorLog((boost::format("%s %s ExcludeMoveTypeiďɎs܂B") % filename % spe.m_id).str());
                return false;
            }
            spe.m_excludemt.push_back(mt);
        }
        // L
        e = child->FirstChildElement("SummonCharacter");
        if(e != NULL){
            spe.m_summoncharaid = GetElementAttribute(*e, "id");
        }
        // ϐ
        const TAttribute& attr = GetAttribute(spe.m_attribute);
        e = child->FirstChildElement("Resistance");
        if(e != NULL){
            if(attr.m_systemeffect != SYSTEM_CHANGE_RESISTANCE){
                g_Logger.WarningLog(filename + " " + spe.m_id + " ɂResistancew肳܂ASystemEffect=ChangeResistanceȊOł͒Pɖ܂B");
            }
            spe.m_resistanceattr = GetAttributeIndex(GetElementAttribute(*e, "attribute"));
            if(spe.m_resistanceattr == -1){
                g_Logger.ErrorLog(filename + " " + spe.m_id + " ɂ ChangeResistance  attribute ̌Ɏs܂B");
                return false;
            }
            spe.m_resistancetype = GetResistance(GetElementAttribute(*e, "resistance"));
            if(!spe.m_resistancetype){
                g_Logger.ErrorLog(filename + " " + spe.m_id + " ɂ ChangeResistance  resistance ̔FɎs܂B");
                return false;
            }
        }
        else{
            if(attr.m_systemeffect == SYSTEM_CHANGE_RESISTANCE){
                g_Logger.ErrorLog(filename + " " + spe.m_id + " ɂđSystemEffect=ChangeResistancew肳܂AResistance܂łB");
                return false;
            }
        }
        // g^[Qbg
        e = child->FirstChildElement("Target");
        if(e != NULL){
            std::string targetid = GetElementAttribute(*e, "id");
            std::map<std::string, int>::const_iterator itr = m_TargetTable.find(targetid);
            if(itr == m_TargetTable.end()){
                spe.m_target = m_TargetTable.size();
                m_TargetTable[targetid] = spe.m_target;
            }
            else{
                spe.m_target = itr->second;
            }
        }
        // ړ^Cv
        e = child->FirstChildElement("MoveType");
        if(e != NULL){
            if(attr.m_systemeffect != SYSTEM_CHANGE_MOVETYPE){
                g_Logger.WarningLog(filename + " " + spe.m_id + " ɂMoveTypew肳܂ASystemEffect=ChangeMoveTypeȊOł͒Pɖ܂B");
            }
            std::string mtid = GetElementAttribute(*e, "id");
            spe.m_movetype = GetMoveTypeIndex(mtid);
            if(spe.m_movetype == -1){
                g_Logger.ErrorLog((boost::format("%s ChangeMoveType  MoveType=%s ̔FɎs܂B") % filename % mtid).str());
                return false;
            }
        }
        else{
            if(attr.m_systemeffect == SYSTEM_CHANGE_MOVETYPE){
                g_Logger.ErrorLog(filename + " " + spe.m_id + " ɂđSystemEffect=ChangeMoveTypew肳܂AMoveType܂łB");
                return false;
            }
        }
        // ^Cv
        e = child->FirstChildElement("Revive");
        if(e != NULL){
            if(attr.m_systemeffect != SYSTEM_REVIVE){
                g_Logger.WarningLog(filename + " " + spe.m_id + " ɂRevivew肳܂ASystemEffect=ReviveȊOł͒Pɖ܂B");
            }
            spe.m_revive = GetReviveIndex(GetElementAttribute(*e, "id"));
            if(spe.m_revive == -1){
                g_Logger.ErrorLog(filename + " Revive  id ̔FɎs܂B");
                return false;
            }
        }
        else{
            if(attr.m_systemeffect == SYSTEM_REVIVE){
                g_Logger.ErrorLog(filename + " " + spe.m_id + " ɂđSystemEffect=Revivew肳܂ARevive܂łB");
                return false;
            }
        }
        // ʌvZ
        e = child->FirstChildElement("PowerFormula");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " PowerFormula not found");
            return false;
        }
        std::string pformula = GetElementText(*e);
        spe.m_powerformula = GetIndex<TFormula>(FormulaName, FORMULA_MAX, pformula);
        if(!spe.m_powerformula){
            g_Logger.ErrorLog(filename + " vZ " + pformula + " ̔FɎs܂B");
            return false;
        }
        if(spe.m_powerformula >= FORMULA_ANYTIME){
            g_Logger.ErrorLog(filename + " ʒlvZ " + pformula + " w肳܂A̎w̓G[ƂĂ܂B");
            return false;
        }
        // vZ
        e = child->FirstChildElement("HitFormula");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " HitFormula not found");
            return false;
        }
        std::string hformula1 = GetElementText(*e);
        spe.m_hitformula1 = GetIndex<TFormula>(FormulaName, FORMULA_MAX, hformula1);
        if(!spe.m_hitformula1){
            g_Logger.ErrorLog(filename + " vZ " + hformula1 + " ̔FɎs܂B");
            return false;
        }
        // ǉʖvZ
        e = child->FirstChildElement("AdditionalHitFormula");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " AdditionalHitFormula not found");
            return false;
        }
        std::string hformula2 = GetElementText(*e);
        spe.m_hitformula2 = GetIndex<TFormula>(FormulaName, FORMULA_MAX, hformula2);
        if(!spe.m_hitformula2){
            g_Logger.ErrorLog(filename + " vZ " + hformula2 + " ̔FɎs܂B");
            return false;
        }
        // 
        e = child->FirstChildElement("HitJudgeType");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " HitJudgeType not found");
            return false;
        }
        std::string judge = GetElementText(*e);
        spe.m_hitjudgetype = GetIndex<THitJudgeType>(HitJudgeTypeName, HITJUDGE_MAX, judge);
        if(!spe.m_hitjudgetype){
            g_Logger.ErrorLog(filename + "  " + judge + " ̔FɎs܂B");
            return false;
        }
        // 
        e = child->FirstChildElement("Attribute");
        if(e == NULL){
            g_Logger.ErrorLog(filename + ":" + spe.m_id + " Attribute܂łB");
            return false;
        }
        std::string attrid = GetElementText(*e);
        spe.m_attribute = GetAttributeIndex(attrid);
        if(spe.m_attribute == -1){
            g_Logger.ErrorLog(filename + ":" + spe.m_id + " " + attrid + " ̔FɎs܂B");
            return false;
        }
        // ǉ
        e = child->FirstChildElement("AdditionalString");
        if(e != NULL){
            spe.m_addstr = GetElementAttribute(*e, "value");
        }
//g_Logger.DebugLog((boost::format("spe=%s attr=%d") % spe.m_id % spe.m_attribute).str());
        // 摜t@C
        e = child->FirstChildElement("Image");
        if(e != NULL){
#if defined(WIN32)
            spe.m_imagefile = GetElementText(*e);
#else
            std::string tmp = GetElementTextEncoding(*e, ENCODING_UTF8);
            g_Ini.EncodeFileNameFromUTF8(tmp, spe.m_imagefile);
#endif
//#ifdef WIN32
//            spe.m_imagefile = GetElementTextEncoding(*e);
//#else
//            spe.m_imagefile = GetElementTextEncoding(*e, ENCODING_UTF8);
//#endif
            // Aj[V̎
            std::string animtype = GetElementAttribute(*e, "type");
            spe.m_animtype = GetIndex<TAnimationType>(AnimationTypeName, ANIMATION_MAX, animtype);
            if(!spe.m_animtype){
                g_Logger.ErrorLog(filename + " Aj[V^Cv " + animtype + " ̔FɎs܂B");
                return false;
            }
            if(e->QueryIntAttribute("width", &spe.m_animwidth) != TIXML_SUCCESS){
                g_Logger.ErrorLog(filename + " Image width not found " + animtype);
                return false;
            }
            if(e->QueryIntAttribute("height", &spe.m_animheight) != TIXML_SUCCESS){
                g_Logger.ErrorLog(filename + " Image height not found " + animtype);
                return false;
            }
            if(e->QueryIntAttribute("animation", &spe.m_animcount) != TIXML_SUCCESS){
                g_Logger.ErrorLog(filename + " Image animation not found " + animtype);
                return false;
            }
        }
        // ʉ
        e = child->FirstChildElement("Sound");
        if(e != NULL){
#if defined(WIN32)
            spe.m_soundfile = GetElementText(*e);
#else
            std::string tmp = GetElementTextEncoding(*e, ENCODING_UTF8);
            g_Ini.EncodeFileNameFromUTF8(tmp, spe.m_soundfile);
#endif
        }
        spe.m_dirno = dirno;
        m_SpecialList.push_back(spe);
    }
    std::sort(m_SpecialList.begin(), m_SpecialList.end(), TIDHolderLess<TSpecial>());

    m_AttackWallSpecial = GetSpecialIndex(SYSTEM_SPECIAL_ATTACKWALL);
    if(m_AttackWallSpecial == -1){
        g_Logger.ErrorLog((boost::format("%s %s not found.") % filename % SYSTEM_SPECIAL_ATTACKWALL).str());
        return false;
    }
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenData::LoadMasterData(const std::string& filename)
{
    m_CharaDirList.clear();
    m_MasterList.clear();

    TiXmlDocument doc(filename.c_str());
    if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog(filename + " XML parse error");
        return false;
    }
    const TiXmlElement *top = doc.FirstChildElement("MasterList");
    if(top == NULL){
        g_Logger.ErrorLog(filename + " MasterLists");
        return false;
    }
    // Lf[^`fBNgXg
    const TiXmlElement *e_chara = top->FirstChildElement("Character");
    for(const TiXmlElement *e_charadir = e_chara->FirstChildElement("Directory"); e_charadir != NULL; e_charadir = e_charadir->NextSiblingElement("Directory")){
        m_CharaDirList.push_back(GetElementText(*e_charadir));
    }

    m_CommonMusicList.resize(TOpenFarenMixer::MUSIC_MAX);
    const char *musicname[TOpenFarenMixer::MUSIC_MAX] = {"EventMusic", "DecisiveMusic"};
    const TiXmlElement *e_commonmusic = top->FirstChildElement("CommonMusic");
    if(e_commonmusic != NULL){
        for(int i=0; i<TOpenFarenMixer::MUSIC_MAX; ++i){
            const TiXmlElement *e = e_commonmusic->FirstChildElement(musicname[i]);
            if(e != NULL){
                m_CommonMusicList[i] = GetElementText(*e);
            }
        }
    }

    for(const TiXmlElement *child = top->FirstChildElement("Master"); child != NULL; child = child->NextSiblingElement("Master")){
        TMaster master;
        master.m_id = GetElementAttribute(*child, "id");
        const TiXmlElement *e = child->FirstChildElement("Name");
        if(e == NULL){
            return false;
        }
        master.m_name = GetElementText(*e);
        e = child->FirstChildElement("PowerMapColor");
        if(e != NULL){
            std::string pcolor = GetElementText(*e);
            master.m_powermapcolor = GetColorABGR(pcolor);
        }
        e = child->FirstChildElement("Music");
        if(e != NULL){
            master.m_musicfile = GetElementText(*e);
        }
        e = child->FirstChildElement("BattleMusic");
        if(e != NULL){
            master.m_battlemusicfile = GetElementText(*e);
        }
        e = child->FirstChildElement("MasterBattleMusic");
        if(e != NULL){
            master.m_masterbattlemusicfile = GetElementText(*e);
        }
        e = child->FirstChildElement("FlagFile");
        if(e != NULL){
            master.m_flagfile = GetElementText(*e);
        }
        m_MasterList.push_back(master);
    }
    std::sort(m_MasterList.begin(), m_MasterList.end(), TIDHolderLess<TMaster>());
    return true;
}
//---------------------------------------------------------------------------
// LoadSpecialData()ɈˑB
bool TOpenFarenData::LoadCharacterData(const std::string& filename, int dirno)
{
    TiXmlDocument doc(filename.c_str());
    if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog(filename + " XML parse error");
        return false;
    }
    const TiXmlElement *e_top = doc.FirstChildElement("CharacterList");
    for(const TiXmlElement *child = e_top->FirstChildElement("Character"); child != NULL; child = child->NextSiblingElement("Character")){
        TBattleCharacter chara;
        chara.m_id = GetElementAttribute(*child, "id");

        // O
        const TiXmlElement *e = child->FirstChildElement("Name");
        if(e == NULL){
            g_Logger.ErrorLog("id =" + chara.m_id + " Names");
            return false;
        }
        chara.m_name = GetElementText(*e);

        // @
        chara.m_magicrank.resize(GetMagicGroupCount());
        int cnt = 0;
        for(e = child->FirstChildElement("Magic"); e != NULL; e = e->NextSiblingElement("Magic")){
            std::string id = GetElementAttribute(*e, "id");
            int i = GetMagicGroupIndex(id);
            if(i == -1){
                g_Logger.ErrorLog("mmagicgroupid " + id);
                return false;
            }
            if(e->QueryIntAttribute("rank", &chara.m_magicrank[i]) != TIXML_SUCCESS){
                g_Logger.ErrorLog("Magicrank");
                return false;
            }
            ++cnt;
        }
        if(cnt < GetMagicGroupCount()){
            g_Logger.WarningLog("xFid = " + chara.m_id + " Magicvf܂B0Ƃ݂Ȃ܂B");
        }

        // 
        chara.m_resistance.resize(GetAttributeCount());
        cnt = 0;
        for(e = child->FirstChildElement("Resistance"); e != NULL; e = e->NextSiblingElement("Resistance")){
            std::string id = GetElementAttribute(*e, "id");
            int i = GetAttributeIndex(id);
            if(i == -1){
                g_Logger.ErrorLog((boost::format("%s %s %s m̑ID") % filename % chara.m_id % id).str());
                return false;
            }
            std::string v = GetElementAttribute(*e, "value");
            chara.m_resistance[i] = GetResistance(v);
            if(!chara.m_resistance[i]){
                g_Logger.ErrorLog((boost::format("%s %s %s m̑ϐID") % filename % chara.m_id % v).str());
                return false;
            }
            cnt++;
        }
        // 0ԖڂgĂȂ񕜑̂Ń`FbNĂdȂB
        //if(cnt < GetMagicGroupCount()){
        //    g_Logger.ErrorLog("xFid = " + chara.m_id + " Registancevf܂B͕ʂƂ݂Ȃ܂B");
        //}
//MessageBox(NULL, "4", "", MB_OK);

        // l
        e = child->FirstChildElement("Hero");
        if(e != NULL){
            if(!GetElementAttribute(*e, "flag").compare("true")){
                chara.m_hero = true;
                ++m_HeroCount;
            }
        }

        // SbZ[WtO
        e = child->FirstChildElement("DeadMessage");
        if(e != NULL){
            if(!GetElementAttribute(*e, "flag").compare("true")){
                chara.m_deadmessage = true;
            }
        }

        // QȂ
        e = child->FirstChildElement("NoWander");
        if(e != NULL){
            if(!GetElementAttribute(*e, "flag").compare("true")){
                chara.m_nowander = true;
            }
        }

        e = child->FirstChildElement("Race");
        if(e != NULL){
            chara.m_race = GetRaceIndex(GetElementAttribute(*e, "value"));
        }
        if(e == NULL || chara.m_race == -1){
            g_Logger.ErrorLog("id =" + chara.m_id + " RacěɎs܂B");
            return false;
        }

        if(!FindIntAttribute(child, "EmployCost", "value", &chara.m_employcost, "id =" + chara.m_id + " EmployCosťɎs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "TurnCost", "value", &chara.m_turncost, "id =" + chara.m_id + " TurnCosťɎs܂B")){
            return false;
        }
        e = child->FirstChildElement("MoveType");
        if(e != NULL){
            chara.m_movetype = GetMoveTypeIndex(GetElementAttribute(*e, "value"));
        }
        if(e == NULL || chara.m_movetype == -1){
            g_Logger.ErrorLog("id =" + chara.m_id + " MoveTypěɎs܂B");
            return false;
        }
        if(!FindIntAttribute(child, "Movable", "value", &chara.m_movable, "id =" + chara.m_id + " MovablěɎs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "HP", "value", &chara.m_hpmax, "id =" + chara.m_id + " HP̌Ɏs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "MP", "value", &chara.m_mpmax, "id =" + chara.m_id + " MP̌Ɏs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "PhysicalAttack", "value", &chara.m_pa, "id =" + chara.m_id + " PhysicalAttacǩɎs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "PhysicalDefense", "value", &chara.m_pd, "id =" + chara.m_id + " PhysicalDefensěɎs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "Skill", "value", &chara.m_skill, "id =" + chara.m_id + " SkilľɎs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "Speed", "value", &chara.m_speed, "id =" + chara.m_id + " SpeeďɎs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "MagicAttack", "value", &chara.m_ma, "id =" + chara.m_id + " MagicAttacǩɎs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "MagicDefense", "value", &chara.m_md, "id =" + chara.m_id + " MagicDefensěɎs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "Morale", "value", &chara.m_morale, "id =" + chara.m_id + " MoralěɎs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "Aggressive", "value", &chara.m_aggressive, "id =" + chara.m_id + " AggressivěɎs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "HPRecover", "value", &chara.m_hprecover, "id =" + chara.m_id + " HPRecoveřɎs܂B")){
            return false;
        }
        if(!FindIntAttribute(child, "MPRecover", "value", &chara.m_mprecover, "id =" + chara.m_id + " MPRecoveřɎs܂B")){
            return false;
        }

        // KEZ
        e = child->FirstChildElement("Special");
        if(e != NULL){
            std::string speid = GetElementAttribute(*e, "id");
            chara.m_specialindex = GetSpecialIndex(speid);
            if(chara.m_specialindex == -1){
                g_Logger.ErrorLog((boost::format("L %s Special id = %s s") % chara.m_id % speid).str());
                return false;
            }
            const TSpecial& spe = GetSpecial(chara.m_specialindex);
            if(spe.m_imagefile.empty()){
                g_Logger.ErrorLog((boost::format("L %s ̓ZɃAj[V`̖Special %s w肳܂BAj[V`̖Special͍Uɂ̂ݎw\łB") % chara.m_id % speid).str());
                return false;
            }
            e->QueryIntAttribute("count", &chara.m_specialcount);
        }

        // KEZ
        e = child->FirstChildElement("AutoSpecial");
        if(e != NULL){
            chara.m_autospecialindex = GetSpecialIndex(GetElementAttribute(*e, "id"));
            if(chara.m_autospecialindex == -1){
                g_Logger.ErrorLog("id = " + chara.m_id + " AutoSpecials");
                return false;
            }
            const TSpecial& autospe = GetSpecial(chara.m_autospecialindex);
            if(autospe.m_rangetype == RANGE_NORMAL || autospe.m_rangetype == RANGE_STREAM){
                g_Logger.ErrorLog((boost::format("L %s ̎KEZɕKEZ %s w肳܂A˒Normal܂Stream̕KEZKEZɎw肷邱Ƃ͂ł܂B") % chara.m_id % autospe.m_id).str());
                return false;
            }
            chara.m_autospecialtype = GetIndex<TSpecialEventType>(EventTypeName, SPECIALEVENT_MAX, GetElementAttribute(*e, "event"));
            if(!chara.m_autospecialtype){
                g_Logger.ErrorLog((boost::format("L %s AutoSpecialeventFł܂łB") % chara.m_id).str());
                return false;
            }
        }

        // XNvg
        e = child->FirstChildElement("AutoScript");
        if(e != NULL){
            chara.m_autoscript = GetElementAttribute(*e, "function");
            chara.m_autoscripttype = GetIndex<TSpecialEventType>(EventTypeName, SPECIALEVENT_MAX, GetElementAttribute(*e, "event"));
            if(!chara.m_autoscripttype){
                g_Logger.ErrorLog((boost::format("L %s AutoScripteventFł܂łB") % chara.m_id).str());
                return false;
            }
        }

        e = child->FirstChildElement("ClassChange");
        if(e != NULL){
            chara.m_classchange_id = GetElementAttribute(*e, "next");
        }

        // ʂɖĂ̂ŃG[͖B̓G[ɂ邩B
        FindIntAttribute(child, "Power", "value", &chara.m_power, "id =" + chara.m_id + " PoweřɎs܂B");
        if(!chara.m_power){
            chara.m_power = 1;
        }

        if(!FindIntAttribute(child, "Experience", "value", &chara.m_experience, "id =" + chara.m_id + " ExperiencěɎs܂B")){
            return false;
        }
        if(!chara.m_experience){
            g_Logger.WarningLog("id = " + chara.m_id + " ol0̃LN^[̓T|[gĂȂߌol1ɏC܂B");
            chara.m_experience = 1;
        }

        // o
        e = child->FirstChildElement("Appearance");
        if(e != NULL){
            e->QueryIntAttribute("TurnMin", &chara.m_turnmin);
            e->QueryIntAttribute("TurnMax", &chara.m_turnmax);
        }

        // U
        for(const TiXmlElement *e_atk = child->FirstChildElement("Attack"); e_atk != NULL; e_atk = e_atk->NextSiblingElement("Attack")){
            TAttack atk;
            e_atk->QueryIntAttribute("Power", &atk.m_power);
            //atk.m_attribute = GetAttributeIndex(GetElementAttribute(*e_atk, "Attribute"));
            atk.m_special = GetSpecialIndex(GetElementAttribute(*e_atk, "specialid"));
            chara.m_attacklist.push_back(atk);
        }

        // ٗp\L
        for(const TiXmlElement *e_emp = child->FirstChildElement("Employable"); e_emp != NULL; e_emp = e_emp->NextSiblingElement("Employable")){
            chara.m_employable_id.push_back(GetElementText(*e_emp));
        }

        // r[XgeC}[
        e = child->FirstChildElement("BeastTamer");
        if(e != NULL){
            for(const TiXmlElement *e_com = e->FirstChildElement("Command"); e_com != NULL; e_com = e_com->NextSiblingElement("Command")){
                int comrace = GetRaceIndex(GetElementText(*e_com));
                if(comrace != -1){
                    chara.m_commandrace.push_back(comrace);
                }
                else{
                    g_Logger.WarningLog("id = " + chara.m_id + " BeastTamerCommandŎw肳ꂽ푰IDs");
                }
            }
        }

        // s
        e = child->FirstChildElement("Target");
        if(e != NULL){
            std::string targetid = GetElementAttribute(*e, "id");
            std::map<std::string, int>::const_iterator itr = m_TargetTable.find(targetid);
            if(itr != m_TargetTable.end()){
                chara.m_target = itr->second;
//g_Logger.DebugLog((boost::format("Chara %s target=%d") % chara.m_id % chara.m_target).str());
            }
            else{
                g_Logger.WarningLog((boost::format("id = %s TargetŎw肳ꂽid = %s TargetƂSpecial݂͑܂BnLăNCGn̖@^ZꍇǍx͖QłB") % chara.m_id % targetid).str());
            }
        }

        // `bvt@C
        e = child->FirstChildElement("BattleChip");
        if(e == NULL){
            g_Logger.ErrorLog("id = " + chara.m_id + " BattleChips");
            return false;
        }
        chara.m_chipfile = GetElementText(*e);

        // 摜
        e = child->FirstChildElement("Face");
        if(e != NULL){
            chara.m_facefile = GetElementText(*e);
        }
        //else{
        //    g_Logger.ErrorLog("id = " + chara.m_id + " Faces");
        //}

        chara.m_dirno = dirno;
        m_CharaList.push_back(chara);
    }
//            g_Logger.ErrorLog("id = " + m_CharaList[1].m_id + " Face="+m_CharaList[1].m_facefile);
//            g_Logger.ErrorLog("id = " + m_CharaList[1].m_id + " Face="+m_CharaList[1].m_facefile);
//            g_Logger.ErrorLog((boost::format("cnt = %d") % m_CharaList.size()).str());
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenData::LoadPowerMap(const std::string& filename)
{
    m_PowerMapPointList.clear();

    if(!boost::filesystem::exists(filename)){
        g_Logger.ErrorLog(filename + " file not found");
        return false;
    }

    TiXmlDocument doc(filename.c_str());
    if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog(filename + " XML parse error");
        return false;
    }
    const TiXmlElement *top = doc.FirstChildElement();
    if(top == NULL){
        g_Logger.ErrorLog(filename + " ɒgH");
        return false;
    }

    const TiXmlElement *e_image = top->FirstChildElement("Image");
    if(e_image == NULL){
        g_Logger.ErrorLog(filename + " Image not found");
        return false;
    }
    const TiXmlElement *e_file = e_image->FirstChildElement("File");
    if(e_file == NULL){
        g_Logger.ErrorLog(filename + " File not found");
        return false;
    }
    m_PowerMapFile = GetElementText(*e_file);

    const TiXmlElement *e_list = top->FirstChildElement("AreaList");
    for(const TiXmlElement *child = e_list->FirstChildElement("Area"); child != NULL; child = child->NextSiblingElement("Area")){
        TPowerMapPoint pp;
        pp.m_id = GetElementAttribute(*child, "id");
        if(pp.m_id.empty()){
            g_Logger.ErrorLog(filename + " id not found");
            return false;
        }
        for(const TiXmlElement *e_point = child->FirstChildElement("PaintPoint"); e_point != NULL; e_point = child->NextSiblingElement("PaintPoint")){
            point_t p;
            e_point->QueryIntAttribute("x", &p.x);
            e_point->QueryIntAttribute("y", &p.y);
            if(p.x || p.y){
                pp.m_points.push_back(p);
            }
        }
        m_PowerMapPointList.push_back(pp);
    }
    std::sort(m_PowerMapPointList.begin(), m_PowerMapPointList.end(), TIDHolderLess<TPowerMapPoint>());
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenData::LoadFieldMap(const std::string& filename)
{
    TiXmlDocument doc(filename.c_str());
    if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog(filename + " XML parse error");
        return false;
    }
    const TiXmlElement *e_top = doc.FirstChildElement("FieldMap");
    if(e_top == NULL){
        g_Logger.ErrorLog(filename + " FieldMaps");
        return false;
    }
    if(e_top->QueryIntAttribute("Width", &m_FieldMapSize.first) != TIXML_SUCCESS){
        g_Logger.ErrorLog(filename + " FieldMap Widths");
        return false;
    }
    if(e_top->QueryIntAttribute("Height", &m_FieldMapSize.second) != TIXML_SUCCESS){
        g_Logger.ErrorLog(filename + " FieldMap Heights");
        return false;
    }
    // Lf[^`fBNgXg
    const TiXmlElement *e_img = e_top->FirstChildElement("Image");
    if(e_img != NULL){
        const TiXmlElement *e_file = e_img->FirstChildElement("File");
        if(e_file != NULL){
            m_FieldMapFile = GetElementText(*e_file);
        }
    }
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenData::ResolveMagic()
{
    for(TMagicDataList::iterator itr = m_MagicDataList.begin(); itr != m_MagicDataList.end(); ++itr){
        //itr->m_special.resize(itr->m_specialid.size());
        //for(unsigned u = 0; u < itr->m_specialid.size(); ++u){
        for(unsigned u = 0; u < itr->m_magics.size(); ++u){
            //itr->m_special[u] = GetSpecialIndex(itr->m_specialid[u]);
            //if(itr->m_special[u] == -1){
            //    g_Logger.ErrorLog("id = " + itr->m_specialid[u] + " ̌Ɏs܂B");
            //    return false;
            //}
//g_Logger.DebugLog((boost::format("magic = %s") % itr->m_magics[u].m_specialid).str());
            itr->m_magics[u].m_special = GetSpecialIndex(itr->m_magics[u].m_specialid);
            if(itr->m_magics[u].m_special == -1){
                g_Logger.ErrorLog("id = " + itr->m_magics[u].m_specialid + " ̌Ɏs܂B");
                return false;
            }
            const TSpecial& spe = GetSpecial(itr->m_magics[u].m_special);
            if(spe.m_imagefile.empty()){
                g_Logger.ErrorLog((boost::format("@ %s ɃAj[V`̖Special %s w肳܂BAj[V`̖Special͍Uɂ̂ݎw\łB") % itr->m_id % itr->m_magics[u].m_specialid).str());
                return false;
            }
        }
    }
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenData::ResolveCharacterID()
{
    for(TSpecialList::iterator itr = m_SpecialList.begin(); itr != m_SpecialList.end(); ++itr){
        if(!itr->m_summoncharaid.empty()){
            itr->m_summonchara = GetCharacterIndex(itr->m_summoncharaid);
            if(itr->m_summonchara == -1){
                g_Logger.ErrorLog("Special id = " + itr->m_id + "  SummonCharacter " + itr->m_summoncharaid + " ̌Ɏs܂B");
                return false;
            }
        }
    }

    for(TReviveList::iterator itr = m_ReviveList.begin(); itr != m_ReviveList.end(); ++itr){
        itr->m_charalist.resize(itr->m_charaidlist.size());
        for(unsigned u=0; u<itr->m_charaidlist.size(); ++u){
            itr->m_charalist[u] = GetCharacterIndex(itr->m_charaidlist[u]);
            if(itr->m_charalist[u] == -1){
                g_Logger.ErrorLog("Revive id = " + itr->m_id + "  Character " + itr->m_charaidlist[u] + " ̌Ɏs܂B");
                return false;
            }
        }
    }
    return true;
}
//---------------------------------------------------------------------------
/*
bool TOpenFarenData::FindIntAttribute(const TiXmlElement *e_src, const std::string& ename, const std::string& attrname, int *dst, const std::string& errlog)
{
    const TiXmlElement *e = e_src->FirstChildElement(ename.c_str());
    if(e != NULL){
        if(e->QueryIntAttribute(attrname.c_str(), dst) != TIXML_SUCCESS){
            g_Logger.ErrorLog(errlog);
            return false;
        }
    }
    else{
        g_Logger.ErrorLog(errlog);
        return false;
    }
    return true;
}
*/
//---------------------------------------------------------------------------
int TOpenFarenData::GetNamedTypeIndex(const std::string& id, const TNamedTypeList& l)
{
    return FindFromSorted<TNamedType>(l, id);
}
//---------------------------------------------------------------------------
TOpenFarenData::TOpenFarenData() :
    m_HeroCount(0)
{
}
//---------------------------------------------------------------------------
bool TOpenFarenData::Load(const std::string& dir, int chapindex)
{
    bool ret = true;

    using std::exception;

    m_MoneyUnit = "Ley";
    try{
        if(!LoadMagicTable(dir + "/Data/MagicTable.xml")){
            TGcnMsgDlg::Show("Error", "@̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("MagicTable.xml OK");

        if(!LoadMagicGroup(dir + "/Data/MagicGroup.xml")){
            TGcnMsgDlg::Show("Error", "@O[v̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("MagicGroup.xml OK");

        //if(!LoadNamedType(dir + "/Data/Race.xml", m_RaceList)){
        if(!LoadRace(dir + "/Data/Race.xml")){
            TGcnMsgDlg::Show("Error", "푰̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("Race.xml OK");

        if(!LoadMapChipType(dir + "/Data/MapChipType.xml")){
            TGcnMsgDlg::Show("Error", "n`̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("MapChipType.xml OK");

        if(!LoadMoveAndSkill(dir + "/Data/MoveAndSkill.xml")){
            TGcnMsgDlg::Show("Error", "ړ^Cv̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("MoveAndSkill.xml OK");

        if(!LoadAttribute(dir + "/Data/Attribute.xml")){
            TGcnMsgDlg::Show("Error", "̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("Attribute.xml OK");

        if(!LoadRevive(dir + "/Data/Revive.xml")){
            TGcnMsgDlg::Show("Error", "̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("Revive.xml OK");

        if(!LoadMapObjTxt(dir + "/Data/MapObj.txt")){
            TGcnMsgDlg::Show("Error", "㕨摜`bv̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("MapObj.txt OK");

        if(!LoadMapChipTxt(dir + "/Data/MapChip.txt")){
            TGcnMsgDlg::Show("Error", "n`摜`bv̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("MapChip.txt OK");
        if(!LoadMasterData(dir + "/Data/MasterData.xml")){
            TGcnMsgDlg::Show("Error", "}X^[̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("MasterData.xml OK");
        if(!LoadPowerMap(dir + "/Data/PowerMap.xml")){
            TGcnMsgDlg::Show("Error", "͒n}̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("PowerMap.xml OK");
        if(!LoadFieldMap(dir + "/Data/FieldMap.xml")){
            TGcnMsgDlg::Show("Error", "tB[h}bv̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("FieldMap.xml OK");

        m_SpecialList.clear();
        m_CharaList.clear();
        for(unsigned ui = 0; ui < m_CharaDirList.size(); ui++){
            if(!LoadSpecialTable(dir + "/" + m_CharaDirList[ui] + "/SpecialTable.xml", ui)){
                TGcnMsgDlg::Show("Error", "Z̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
                throw exception();
            }
        }
        g_Logger.DebugLog("SpecialTable.xml OK");

        for(unsigned ui = 0; ui < m_CharaDirList.size(); ui++){
            if(!LoadCharacterData(dir + "/" + m_CharaDirList[ui] + "/CharacterData.xml", ui)){
                TGcnMsgDlg::Show("Error", "L̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
                throw exception();
            }
        }
        std::sort(m_CharaList.begin(), m_CharaList.end(), TIDHolderLess<TBattleCharacter>());
        g_Logger.DebugLog("CharacterData.xml OK");

        // @ID
        if(!ResolveMagic()){
            TGcnMsgDlg::Show("Error", "@IDɎs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("ResolveMagic OK");

        // ID
        if(!ResolveCharacterID()){
            TGcnMsgDlg::Show("Error", "LIDɎs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("ResolveCharacterID OK");

        // ٗp\LIDB
        for(TBattleCharacterList::iterator itr = m_CharaList.begin(); itr != m_CharaList.end(); itr++){
            TBattleCharacter& chara = *itr;
            chara.m_employable.resize(chara.m_employable_id.size());
            for(unsigned u=0; u<chara.m_employable_id.size(); u++){
                int i = GetCharacterIndex(chara.m_employable_id[u]);
                if(i != -1){
                    chara.m_employable[u] = i;
                }
                else{
                    g_Logger.ErrorLog("id =" + chara.m_id + " Employable " + chara.m_employable_id[u] + " s");
                    TGcnMsgDlg::Show("Error", "L̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
                    throw exception();
                }
            }
            if(!chara.m_classchange_id.empty()){
                chara.m_classchange = GetCharacterIndex(chara.m_classchange_id);
            }
        }
        g_Logger.DebugLog("ResolveEmployable OK");

        for(int i=0; i<GetMasterCount(); i++){
            TMaster& master = GetMaster(i);
            int chrindex = GetCharacterIndex(master.m_id);
            if(chrindex != -1){
                master.m_facecharaindex = chrindex;
                m_CharaList[chrindex].m_master = true;
            }
            else{
                TGcnMsgDlg::Show("Error", "}X^[ID " + master.m_id + " ̉Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
                throw exception();
            }
        }
        g_Logger.DebugLog("ResolveMasterID OK");

        TFarenScenario sce;
        if(!sce.LoadFromDirectory(dir.c_str())){
            TGcnMsgDlg::Show("Error", "ViI̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("Scenario.xml OK");
        m_ScenarioTitle = sce.GetTitle();

        std::string scefile = (boost::format("%s/Data/Scenario%d.xml") % dir.c_str() % (chapindex+1)).str();
        if(!m_Chapter.LoadFromFile(scefile)){
            TGcnMsgDlg::Show("Error", "ViIJn̓ǂݍ݂Ɏs܂B", TGcnMsgDlg::BUTTON_OK);
            throw exception();
        }
        g_Logger.DebugLog("ScenarioX.xml OK");
    }
    catch(exception&){
        ret = false;
    }

    return ret;
}
//---------------------------------------------------------------------------
int TOpenFarenData::GetRaceIndex(const std::string& id) const
{
    //return FindFromSorted<TNamedType>(m_RaceList, id);
    return FindFromSorted<TRace>(m_RaceList, id);
}
//---------------------------------------------------------------------------
bool operator==(const TAttribute& lhs, const TAttribute& rhs){
    return lhs.m_id == rhs.m_id;
}

int TOpenFarenData::GetAttributeIndex(const std::string& id) const
{
    // ̓\[gȂB
    //return FindFromSorted<TAttribute>(m_AttributeList, id);
    return FindFromNotSorted<TAttribute>(m_AttributeList, id);
}
//---------------------------------------------------------------------------
int TOpenFarenData::GetMoveTypeIndex(const std::string& id) const
{
    return FindFromSorted<TMoveType>(m_MoveTypeList, id);
}
//---------------------------------------------------------------------------
int TOpenFarenData::GetMasterIndex(const std::string& id) const
{
    return FindFromSorted<TMaster>(m_MasterList, id);
}
//---------------------------------------------------------------------------
void TOpenFarenData::SetMasterMusic(int index, int music, int battlemusic, int masterbattlemusic)
{
    m_MasterList[index].m_music = music;
    m_MasterList[index].m_battlemusic = battlemusic;
    m_MasterList[index].m_masterbattlemusic = masterbattlemusic;
}
//---------------------------------------------------------------------------
void TOpenFarenData::SetCharacterImage(int index, int faceindex, int chipindex)
{
    m_CharaList[index].m_faceindex = faceindex;
    m_CharaList[index].m_chipindex = chipindex;
}
//---------------------------------------------------------------------------
int TOpenFarenData::GetPowerMapPointIndex(const std::string& id) const
{
    return FindFromSorted<TPowerMapPoint>(m_PowerMapPointList, id);
}
//---------------------------------------------------------------------------
int TOpenFarenData::GetReviveIndex(const std::string& id) const
{
    return FindFromSorted<TRevive>(m_ReviveList, id);
}
//---------------------------------------------------------------------------
int TOpenFarenData::GetMagicGroupIndex(const std::string& id) const
{
    return FindFromSorted<TMagicGroup>(m_MagicGroupList, id);
}
//---------------------------------------------------------------------------
int TOpenFarenData::GetSpecialIndex(const std::string& id) const
{
    return FindFromSorted<TSpecial>(m_SpecialList, id);
}
//---------------------------------------------------------------------------
int TOpenFarenData::GetCharacterIndex(const std::string& id) const
{
    return FindFromSorted<TBattleCharacter>(m_CharaList, id);
}
//---------------------------------------------------------------------------
const std::string& TOpenFarenData::GetSystemEffectName(TSystemEffect se) const
{
    // ꉞS񋓂Ă邪Aۂɂ*tĂ镶񂵂gȂB
    static const std::string names[SYSTEM_EFFECTMAX] = {
        "Ȃ",
        "ʏ",
        "",   // *
        "Ή", // *
        "", // *
        "", // *
        "z", // *
        "",
        "z",
        "HP",
        "MP",
        "ǔj",
        "U͑",
        "h䑝",
        "͑",
        "Zʑ",
        "x",
        "R",
        "HP񕜑",
        "MP񕜑",
        "U񐔑",
        "Ŏ",
        "Ή",
        "჎",
        "",
        "z",
        "",
        "R",
        "ړ^CvύX",
        "",
        ""
        };
//g_Logger.DebugLog((boost::format("GetSystemEffectName = %d") % se).str());
    return names[se];
}
//---------------------------------------------------------------------------
/*void TOpenFarenData::ErrorLog(const std::string& txt) const
{
    fprintf(m_fp, "%s\n", txt.c_str());
}*/
