//---------------------------------------------------------------------------
#include <exception>
#include <algorithm>
#include "ofgame.h"
#include "ofxml.h"
#include "bmap.h"
#include "logger.h"
#include "boostformat.h"
#include "boostlexicalcast.h"
#include "mtrand.h"
#include <boost/filesystem/operations.hpp>
#include "ofini.h"   // {vȂ̂ł̂폜H
// F[hg_GameMaster̃f[^ݒȂ̂ł̃\[Xg_GameMastergĂ͂ȂB
// #include "gamemaster.h"ȂƁB

//#include <windows.h>
namespace
{
    const char* XMLELM_SAVETOP = "Save";
    const char* XMLELM_AREATOP = "AreaList";
    const char* XMLELM_BMAPTOP = "BattleMap";
    const char* XMLELM_MASTERTOP = "MasterList";
//    const char* XMLELM_CHAPTER = "Chapter";
    const char* XMLELM_GAMELEVEL = "GameLevel";
    const char* XMLELM_TURN = "Turn";
    const char* XMLELM_PROGRESSEDTURN = "ProgressedTurn";
    const char* XMLELM_ALLIANCETOP = "AllianceList";
    const char* XMLELM_ALLIANCE = "Alliance";
    const char* XMLELM_ORDERTOP = "MasterOrder";
    const char* XMLELM_CHARATOP = "CharacterList";
    const char* ALLIANCE_INFINITE = "infinite";
    const char* XMLVAL_AREAOPE[AREAOPE_MAX] = {"Training", "SearchHero", "BuildCity", "BuildWall", "BuildRoad"};

    // ͎wł̍~\[gpB
    struct TCharacterLess
    {
        const TOpenFarenData* m_Data;
        int m_MasterRace;
        TCharacterLess(const TOpenFarenData* data, int race) : m_Data(data), m_MasterRace(race) {}

        bool operator()(const TCharacterInstance& lhs, const TCharacterInstance& rhs)
        {
            const TBattleCharacter& lcharadata = m_Data->GetCharacter(lhs.m_type);
            const TBattleCharacter& rcharadata = m_Data->GetCharacter(rhs.m_type);
            if(lcharadata.m_master){
                return true;
            }
            if(rcharadata.m_master){
                return false;
            }
            if(m_MasterRace != -1){
                // ǂ炩̎푰}X^[̎푰Ɠ炻ő召B
                if(lcharadata.m_race == m_MasterRace){
                    if(rcharadata.m_race != m_MasterRace){
                        return true;
                    }
                }
                else if(rcharadata.m_race == m_MasterRace){
                    return false;
                }
            }
            if(lcharadata.m_hero){
                return true;
            }
            if(rcharadata.m_hero){
                return false;
            }
            return lhs.m_power + lhs.m_experience > rhs.m_power + rhs.m_experience;
        }
    };

    // jł̍~\[gpB
    struct TCharacterWinGreater
    {
        bool operator()(const TCharacterInstance& lhs, const TCharacterInstance& rhs)
        {
            return lhs.m_wincount > rhs.m_wincount;
        }
    };
}
//---------------------------------------------------------------------------
const char *TOpenFarenGame::AreaOperationName[AREAOPE_MAX] = {"P", "lޑ{", "X", "ǌ", "H"};
//---------------------------------------------------------------------------
TOpenFarenGame::TOpenFarenGame() :
    m_CurrentMaster(-1)
{
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::LoadNew(const TOpenFarenData& data, const std::string& dir, int chapindex, int gamelevel)
{
    m_AreaInstanceList.clear();
    m_MasterInstanceList.clear();

    m_Data = &data;

    std::string areafile = dir + "/Data/AreaData.xml";
    if(!boost::filesystem::exists(areafile)){
        g_Logger.ErrorLog((boost::format("%s File not found") % areafile).str());
        return false;
    }
    TiXmlDocument doc_area(areafile.c_str());
    if(!doc_area.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog((boost::format("%s XML parse error") % areafile).str());
        return false;
    }
    const TiXmlElement *e_area = doc_area.FirstChildElement(XMLELM_AREATOP);
    if(e_area == NULL){
        g_Logger.ErrorLog((boost::format("%s %ss") % areafile % XMLELM_AREATOP).str());
        return false;
    }
    if(!LoadAreaInstance(*e_area)){
        g_Logger.ErrorLog((boost::format("%s GAf[^Ǎs") % areafile).str());
        return false;
    }

    if(!LoadParmanentData(dir)){
        return false;
    }

    m_GameLevel = gamelevel;

    g_Logger.DebugLog("Loading chapter file...");
    std::string chapfile = (boost::format("%s/Data/Scenario%d.xml") % dir.c_str() % (chapindex+1)).str();
    TFarenChapter chapter;
    if(!chapter.LoadFromFile(chapfile)){
        OFERRLOG((boost::format("failed to load %s") % chapfile).str());
        return false;
    }
    g_Logger.DebugLog("Constructing master instance...");
    for(int i=0; i<chapter.GetMasterCount(); i++){
        const TFarenChapterMaster& master = chapter.GetMaster(i);
        TMasterInstance mi;
        mi.m_type = data.GetMasterIndex(master.m_id);
        if(mi.m_type == -1){
            g_Logger.ErrorLog(chapfile + " }X^[ID " + master.m_id + " ̌Ɏs");
            return false;
        }
        mi.m_money = 100;
        m_MasterInstanceList.push_back(mi);
    }

    g_Logger.DebugLog("Constructing area instance...");
    // }X^[ۗL̃GA肷B
    for(int i=0; i<chapter.GetAreaCount(); i++){
        const TFarenChapterArea& chaparea = chapter.GetArea(i);
        int ai = GetAreaInstanceIndex(chaparea.m_id);
        if(ai != -1){
            int mastertype = data.GetMasterIndex(chaparea.m_master);
            TAreaInstance& area = m_AreaInstanceList[ai];
            area.m_master = GetMasterInstanceIndex(mastertype);
            area.m_ignore = area.m_master == -1 && chaparea.m_ignore;
        }
        else{
            g_Logger.ErrorLog((boost::format("}X^[ۗLGAݒ菈F Area %s ̌Ɏs") % chaparea.m_id).str());
            return false;
        }
    }

    // zuL
    g_Logger.DebugLog("Placing character on area...");
    // zuꂽLޖɐBlނ̔zuςݔpB
    int chrtypes = data.GetCharacterCount();
    std::vector<int> placedCharaCount(chrtypes);
    for(int i=0; i<chapter.GetCharacterCount(); i++){
        const TFarenChapterCharacter& chara = chapter.GetCharacter(i);
        int ai = GetAreaInstanceIndex(chara.m_area);
        if(ai != -1){
            g_Logger.DebugLog((boost::format("charazu Area %s %s") % chara.m_area % chara.m_id).str());
            int ctype = data.GetCharacterIndex(chara.m_id);
            if(ctype != -1){
                m_AreaInstanceList[ai].m_charalist.push_back(CreateCharaInstance(ctype, chara.m_rank));
                ++placedCharaCount[ctype];
            }
            else{
                g_Logger.WarningLog((boost::format("Chara %s ̌Ɏs") % chara.m_id).str());
            }
        }
        else{
            g_Logger.WarningLog((boost::format("Area %s ̌Ɏs") % chara.m_area).str());
        }
        g_Logger.DebugLog((boost::format("charazu Area %s %s OK") % chara.m_area % chara.m_id).str());
    }

    // 㕨KɔzuB
    for(int i=0; i<m_AreaInstanceList.size(); ++i){
        int baseArea = static_cast<int>(IsBaseArea(i));
        TAreaInstance& area = m_AreaInstanceList[i];
        if(area.m_master == -1){
            area.m_city = area.m_citymax/10;
            area.m_road = area.m_roadmax/10;
            area.m_wall = (area.m_wallmax * area.m_wallini) / 100;
        }
        else{
            if(baseArea){
                area.m_city = area.m_citymax/3;
                area.m_road = area.m_roadmax/3;
                area.m_wall = area.m_wallmax;
            }
            else{
                area.m_city = area.m_citymax/6;
                area.m_road = area.m_roadmax/6;
                area.m_wall = (area.m_wallmax * (area.m_wallini + 40)) / 100;
                if(area.m_wall > area.m_wallmax){
                    area.m_wall = area.m_wallmax;
                }
            }
        }
        area.m_bmap->SetCity(area.m_city);
        area.m_bmap->SetWall(area.m_wall);
        area.m_bmap->SetRoad(area.m_road);
    }

    int alcount = chapter.GetAllianceCount();
    for(int i=0; i<alcount; ++i){
        const TFarenChapterAlliance& cal = chapter.GetAlliance(i);

        int mastertype1 = data.GetMasterIndex(cal.m_id1);
        if(mastertype1 == -1){
            g_Logger.ErrorLog((boost::format("AllianceŎw肳ꂽid1 %s ̌Ɏs") % cal.m_id1).str());
            return false;
        }
        int master1 = GetMasterInstanceIndex(mastertype1);
        if(master1 == -1){
            g_Logger.WarningLog((boost::format("AllianceŎw肳ꂽid1 %s ̃}X^[͂̏͂ɑ݂܂B̓ݒ𖳎܂B") % cal.m_id1).str());
            continue;
        }

        int mastertype2 = data.GetMasterIndex(cal.m_id2);
        if(mastertype2 == -1){
            g_Logger.ErrorLog((boost::format("AllianceŎw肳ꂽid2 %s ̌Ɏs") % cal.m_id2).str());
            return false;
        }
        int master2 = GetMasterInstanceIndex(mastertype2);
        if(master2 == -1){
            g_Logger.WarningLog((boost::format("AllianceŎw肳ꂽid2 %s ̃}X^[͂̏͂ɑ݂܂B̓ݒ𖳎܂B") % cal.m_id2).str());
            continue;
        }
        SetAlliance(master1, master2, cal.m_turn);
    }
    m_ChapterNumber = chapindex;
    m_ProgressedTurn = -1;
    m_Turn = chapter.GetTurn() - 1;

    g_Logger.DebugLog("lނ̕Q󋵂\z܂...");
    for(int i=0; i<chrtypes; ++i){
        // zuς݂͂Ƃ΂B
        if(placedCharaCount[i]){
            continue;
        }
        const TBattleCharacter& charadata = data.GetCharacter(i);
        //// lށEQEo^[B
        //if(!charadata.m_hero || charadata.m_nowander ||
        //   m_Turn < charadata.m_turnmin ||
        //   (charadata.m_turnmax != -1 && charadata.m_turnmax < m_Turn)){
        //    continue;
        //}
        if(charadata.m_hero && !charadata.m_nowander){
            m_WanderingCharaList.push_back(CreateCharaInstance(i, 0));
        }
    }

    g_Logger.DebugLog("All data and instance OK, starting game...");
    m_IsEnding = false;
    UpdateMasterPower();
    //OrderMaster();
    //ActivateNextMaster();
    m_MasterOrderIndex = m_MasterInstanceList.size();
    g_Logger.DebugLog("New Game start.");
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::LoadSaved(const TOpenFarenData& data, const std::string& dir, int chapindex, const TiXmlElement& e)
{
    m_AreaInstanceList.clear();
    m_MasterInstanceList.clear();

    m_Data = &data;

//    const TiXmlElement *e_chap = e.FirstChildElement(XMLELM_CHAPTER);
//    if(e_chap == NULL){
//        g_Logger.ErrorLog((boost::format("%s s") % XMLELM_CHAPTER).str());
//        return false;
//    }
//    if(e_chap->QueryIntAttribute("index", &m_ChapterNumber) != TIXML_SUCCESS){
//        g_Logger.ErrorLog((boost::format("%s index not found.") % XMLELM_CHAPTER).str());
//        return false;
//    }
//    if(!FindIntAttribute(&e, XMLELM_CHAPTER, "index", &m_ChapterNumber, (boost::format("%s index not found.") % XMLELM_CHAPTER).str())){
//        return false;
//    }
    if(!FindIntAttribute(&e, XMLELM_GAMELEVEL, "value", &m_GameLevel, (boost::format("%s value not found.") % XMLELM_GAMELEVEL).str())){
        return false;
    }
    if(!FindIntAttribute(&e, XMLELM_TURN, "value", &m_Turn, (boost::format("%s value not found.") % XMLELM_TURN).str())){
        return false;
    }
    if(!FindIntAttribute(&e, XMLELM_PROGRESSEDTURN, "value", &m_ProgressedTurn, (boost::format("%s value not found.") % XMLELM_PROGRESSEDTURN).str())){
        return false;
    }

    // }X^[
    const TiXmlElement *e_mtop = e.FirstChildElement(XMLELM_MASTERTOP);
    if(e_mtop == NULL){
        g_Logger.ErrorLog((boost::format("%s s") % XMLELM_MASTERTOP).str());
        return false;
    }
    for(const TiXmlElement *child = e_mtop->FirstChildElement("Master"); child != NULL; child = child->NextSiblingElement("Master")){
        TMasterInstance masterins;
        std::string id = GetElementAttribute(*child, "id");
        masterins.m_type = m_Data->GetMasterIndex(id);
        if(masterins.m_type == -1){
            g_Logger.ErrorLog((boost::format("%s Master id = %s sB") % XMLELM_MASTERTOP % id).str());
            return false;
        }

        int i;
        if(child->QueryIntAttribute("exists", &i) != TIXML_SUCCESS){
            g_Logger.ErrorLog((boost::format("%s Master id = %s existsosB") % XMLELM_MASTERTOP % id).str());
            return false;
        }
        masterins.m_exists = i;

        if(child->QueryIntAttribute("player", &i) != TIXML_SUCCESS){
            g_Logger.ErrorLog((boost::format("%s Master id = %s playerosB") % XMLELM_MASTERTOP % id).str());
            return false;
        }
        masterins.m_player = i;

        if(child->QueryIntAttribute("money", &masterins.m_money) != TIXML_SUCCESS){
            g_Logger.ErrorLog((boost::format("%s Master id = %s moneyosB") % XMLELM_MASTERTOP % id).str());
            return false;
        }
        m_MasterInstanceList.push_back(masterins);
    }
    //std::sort(m_MasterInstanceList.begin(), m_MasterInstanceList.end(), TMasterInstanceLess());

    // 
    const TiXmlElement *e_atop = e.FirstChildElement(XMLELM_ALLIANCETOP);
    if(e_atop == NULL){
        g_Logger.ErrorLog((boost::format("%s s") % XMLELM_ALLIANCETOP).str());
        return false;
    }
    for(const TiXmlElement *child = e_atop->FirstChildElement(XMLELM_ALLIANCE); child != NULL; child = child->NextSiblingElement(XMLELM_ALLIANCE)){
        TAlliance al;

        std::string master1 = GetElementAttribute(*child, "id1");
        al.master1 = GetMasterInstanceIndex(master1);
        if(al.master1 == -1){
            g_Logger.ErrorLog((boost::format("%s Master id = %s ܂łB") % XMLELM_ALLIANCE % master1).str());
            return false;
        }

        std::string master2 = GetElementAttribute(*child, "id2");
        al.master2 = GetMasterInstanceIndex(master2);
        if(al.master2 == -1){
            g_Logger.ErrorLog((boost::format("%s Master id = %s ܂łB") % XMLELM_ALLIANCE % master2).str());
            return false;
        }

        std::string turnstr = GetElementAttribute(*child, "turn");
        if(!turnstr.compare(ALLIANCE_INFINITE)){
            al.turn = -1;
        }
        else{
            try{
                al.turn = boost::lexical_cast<int>(turnstr);
            }
            catch(boost::bad_lexical_cast&){
                g_Logger.ErrorLog((boost::format("%s turnߎs") % XMLELM_ALLIANCE).str());
                return false;
            }
        }
        m_AllianceList.push_back(al);
    }

    // }X^[
    const TiXmlElement *e_order = e.FirstChildElement(XMLELM_ORDERTOP);
    if(e_order == NULL){
        g_Logger.ErrorLog((boost::format("%s s") % XMLELM_ORDERTOP).str());
        return false;
    }
    for(const TiXmlElement *child = e_order->FirstChildElement("Master"); child != NULL; child = child->NextSiblingElement("Master")){
        std::string id = GetElementAttribute(*child, "id");
        int masterindex = GetMasterInstanceIndex(id);
        if(masterindex == -1){
            g_Logger.ErrorLog((boost::format("%s Master id = %s ܂łB") % XMLELM_ORDERTOP % id).str());
            return false;
        }
        m_MasterOrder.push_back(std::pair<int, int>(masterindex, 0));
    }
    if(e_order->QueryIntAttribute("index", &m_MasterOrderIndex) != TIXML_SUCCESS){
        g_Logger.ErrorLog((boost::format("%s indexosB") % XMLELM_MASTERTOP).str());
        return false;
    }
    if(m_MasterOrderIndex >= m_MasterOrder.size()){
        g_Logger.ErrorLog((boost::format("%s indexlMastervf𒴂Ă܂B") % XMLELM_ORDERTOP).str());
        return false;
    }

    // GA
    const TiXmlElement *e_area = e.FirstChildElement(XMLELM_AREATOP);
    if(e_area == NULL){
        g_Logger.ErrorLog((boost::format("%s s") % XMLELM_AREATOP).str());
        return false;
    }
    if(!LoadAreaInstance(*e_area)){
        g_Logger.ErrorLog((boost::format("GAf[^Ǎs")).str());
        return false;
    }

    if(!LoadParmanentData(dir)){
        return false;
    }

    // GA}bv̒񉻓ǂݍ
    const TiXmlElement *e_maptop = e.FirstChildElement(XMLELM_BMAPTOP);
    if(e_maptop == NULL){
        g_Logger.ErrorLog((boost::format("%s s") % XMLELM_BMAPTOP).str());
        return false;
    }
    for(TAreaInstanceList::iterator itr = m_AreaInstanceList.begin(); itr != m_AreaInstanceList.end(); itr++){
        const TiXmlElement *e_map = e_maptop->FirstChildElement(itr->m_bmapfile.c_str());
        if(e_map == NULL){
            g_Logger.ErrorLog((boost::format("%s s") % itr->m_bmapfile).str());
            return false;
        }
        if(!itr->m_bmap->LoadFromElementSerialize(e_map)){
            g_Logger.ErrorLog((boost::format("%s ǂݍݎs") % itr->m_bmapfile).str());
            return false;
        }
    }

    // L
    const TiXmlElement *e_charalist = e.FirstChildElement(XMLELM_CHARATOP);
    if(e_charalist == NULL){
        OFERRLOG((boost::format("%s s") % XMLELM_CHARATOP).str());
        return false;
    }
    for(const TiXmlElement *child = e_charalist->FirstChildElement("Character"); child != NULL; child = child->NextSiblingElement("Character")){
        std::string id = GetElementAttribute(*child, "id");
        int ctype = m_Data->GetCharacterIndex(id);
        if(ctype == -1){
            OFERRLOG((boost::format("%s Character id = %s sB") % XMLELM_CHARATOP % id).str());
            return false;
        }

        int rank = 0;
        child->QueryIntAttribute("rank", &rank);  // G[͖B
        TCharacterInstance chara = CreateCharaInstance(ctype, rank);
        child->QueryIntAttribute("experience", &chara.m_experience);  // G[͖B

        const TBattleCharacter& charadata = m_Data->GetCharacter(ctype);
        if(charadata.m_hero || charadata.m_master){
            child->QueryIntAttribute("wincount", &chara.m_wincount);
        }

        std::string areaid = GetElementAttribute(*child, "area");
        if(!areaid.empty()){
            // GAɋLB
            int ai = GetAreaInstanceIndex(areaid);
            if(ai == -1){
                OFERRLOG((boost::format("%s area = %s sB") % XMLELM_CHARATOP % areaid).str());
                return false;
            }
            int actioned = 0;
            child->QueryIntAttribute("actioned", &actioned);  // G[͖B
            chara.m_actioned = actioned;

            m_AreaInstanceList[ai].m_charalist.push_back(chara);
        }
        else{
            bool wander = !GetElementAttribute(*child, "wander").compare("true");
            if(wander){
                // Q̃LB
                m_WanderingCharaList.push_back(chara);
            }
            else{
                bool dead = !GetElementAttribute(*child, "dead").compare("true");
                if(dead){
                    // SLB
                    m_DeadCharaList.push_back(chara);
                }
                else{
                    OFWRNLOG((boost::format("%s Character id = %s ̓GA݂łQłSłȂH") % XMLELM_CHARATOP % id).str());
                }
            }
        }
    }

    m_ChapterNumber = chapindex;
    m_IsEnding = false;
    UpdateMasterIncome();
    UpdateMasterPower();
    m_CurrentMaster = m_MasterOrder[m_MasterOrderIndex].first;
    g_Logger.DebugLog("Saved Game start.");
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::Save(TiXmlElement& e) const
{
//    TiXmlElement e_chap(XMLELM_CHAPTER);
//    e_chap.SetAttribute("index", m_ChapterNumber);
//    e.InsertEndChild(e_chap);

    TiXmlElement e_level(XMLELM_GAMELEVEL);
    e_level.SetAttribute("value", m_GameLevel);
    e.InsertEndChild(e_level);

    TiXmlElement e_turn(XMLELM_TURN);
    e_turn.SetAttribute("value", m_Turn);
    e.InsertEndChild(e_turn);

    TiXmlElement e_pturn(XMLELM_PROGRESSEDTURN);
    e_pturn.SetAttribute("value", m_ProgressedTurn);
    e.InsertEndChild(e_pturn);

    // }X^[
    TiXmlElement e_mtop(XMLELM_MASTERTOP);
    for(TMasterInstanceList::const_iterator itr = m_MasterInstanceList.begin(); itr != m_MasterInstanceList.end(); ++itr){
        const TMaster& master = m_Data->GetMaster(itr->m_type);
        TiXmlElement e_master("Master");
        SetTextAttribute(e_master, "id", master.m_id.c_str());
        e_master.SetAttribute("exists", itr->m_exists);
        e_master.SetAttribute("player", itr->m_player);
        e_master.SetAttribute("money", itr->m_money);
        e_mtop.InsertEndChild(e_master);
    }
    e.InsertEndChild(e_mtop);

    // 
    TiXmlElement e_atop(XMLELM_ALLIANCETOP);
    for(TAllianceList::const_iterator itr = m_AllianceList.begin(); itr != m_AllianceList.end(); ++itr){
        TiXmlElement e_alliance(XMLELM_ALLIANCE);
        SetTextAttribute(e_alliance, "id1", m_Data->GetMaster(m_MasterInstanceList[itr->master1].m_type).m_id.c_str());
        SetTextAttribute(e_alliance, "id2", m_Data->GetMaster(m_MasterInstanceList[itr->master2].m_type).m_id.c_str());
        if(itr->turn == -1){
            e_alliance.SetAttribute("turn", ALLIANCE_INFINITE);
        }
        else{
            e_alliance.SetAttribute("turn", itr->turn);
        }
        e_atop.InsertEndChild(e_alliance);
    }
    e.InsertEndChild(e_atop);

    // }X^[
    TiXmlElement e_order(XMLELM_ORDERTOP);
    e_order.SetAttribute("index", m_MasterOrderIndex);
    for(unsigned u=0; u<m_MasterOrder.size(); ++u){
        TiXmlElement e_orderchild("Master");
        SetTextAttribute(e_orderchild, "id", m_Data->GetMaster(m_MasterInstanceList[m_MasterOrder[u].first].m_type).m_id.c_str());
        e_order.InsertEndChild(e_orderchild);
    }
    e.InsertEndChild(e_order);

    // GA
    TiXmlElement e_area(XMLELM_AREATOP);
    SaveAreaInstance(e_area);
    e.InsertEndChild(e_area);

    TiXmlElement e_bmap(XMLELM_BMAPTOP);
    for(TAreaInstanceList::const_iterator itr = m_AreaInstanceList.begin(); itr != m_AreaInstanceList.end(); itr++){
        TiXmlElement e_map(itr->m_bmapfile.c_str());
        itr->m_bmap->SaveToElementSerialize(&e_map);
        e_bmap.InsertEndChild(e_map);
    }
    e.InsertEndChild(e_bmap);

    // L
    TiXmlElement e_charalist(XMLELM_CHARATOP);
    unsigned sz = m_AreaInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        const TAreaInstance area = m_AreaInstanceList[u];
        unsigned characount = area.m_charalist.size();
        for(unsigned chr=0; chr<characount; ++chr){
            const TCharacterInstance& chara = area.m_charalist[chr];
            const TBattleCharacter& charadata = m_Data->GetCharacter(chara.m_type);
            TiXmlElement e_chara("Character");
            SetTextAttribute(e_chara, "id", charadata.m_id.c_str());
            SetTextAttribute(e_chara, "area", area.m_id.c_str());
            e_chara.SetAttribute("rank", chara.m_rank);
            e_chara.SetAttribute("experience", chara.m_experience);
            if(charadata.m_hero || charadata.m_master){
                e_chara.SetAttribute("wincount", chara.m_wincount);
            }
            e_chara.SetAttribute("actioned", chara.m_actioned);
            e_charalist.InsertEndChild(e_chara);
        }
    }
    // QL
    unsigned wsz = m_WanderingCharaList.size();
    for(unsigned u=0; u<wsz; ++u){
        const TCharacterInstance& chara = m_WanderingCharaList[u];
        const TBattleCharacter& charadata = m_Data->GetCharacter(chara.m_type);
        TiXmlElement e_chara("Character");
        SetTextAttribute(e_chara, "id", charadata.m_id.c_str());
        e_chara.SetAttribute("rank", chara.m_rank);
        e_chara.SetAttribute("experience", chara.m_experience);
        if(charadata.m_hero || charadata.m_master){
            e_chara.SetAttribute("wincount", chara.m_wincount);
        }
        e_chara.SetAttribute("wander", "true");
        e_charalist.InsertEndChild(e_chara);
    }
    // SL
    unsigned dsz = m_DeadCharaList.size();
    for(unsigned u=0; u<dsz; ++u){
        const TCharacterInstance& chara = m_DeadCharaList[u];
        const TBattleCharacter& charadata = m_Data->GetCharacter(chara.m_type);
        TiXmlElement e_chara("Character");
        SetTextAttribute(e_chara, "id", charadata.m_id.c_str());
        e_chara.SetAttribute("rank", chara.m_rank);
        e_chara.SetAttribute("experience", chara.m_experience);
        e_chara.SetAttribute("wincount", chara.m_wincount);
        e_chara.SetAttribute("dead", "true");
        e_charalist.InsertEndChild(e_chara);
    }
    e.InsertEndChild(e_charalist);

    return true;
}
//---------------------------------------------------------------------------
void TOpenFarenGame::GetSaveTitleString(std::string& str) const
{
    const TFarenChapter& chap = m_Data->GetChapter();
    const TMaster& master = m_Data->GetMaster(m_MasterInstanceList[m_CurrentMaster].m_type);
    str = (boost::format("%s %s %s %d^[") % m_Data->GetScenarioTitle() % chap.GetName() % master.m_name % m_Turn).str();
}
//---------------------------------------------------------------------------
int TOpenFarenGame::GetAreaInstanceIndex(const std::string& id) const
{
    return FindFromSorted<TAreaInstance>(m_AreaInstanceList, id);
}
//---------------------------------------------------------------------------
int TOpenFarenGame::GetMasterInstanceIndex(int mastertype) const
{
    /*if(1){
        unsigned sz = m_MasterInstanceList.size();
        for(unsigned u=0; u<sz; ++u){
            OFDBGLOG((boost::format("%u:%d:%s") % u % m_MasterInstanceList[u].m_type % (m_Data->GetMaster(m_MasterInstanceList[u].m_type).m_name)).str());
        }
    }*/

    /*TMasterInstance key;
    key.m_type = mastertype;
    TMasterInstanceList::const_iterator itr = std::lower_bound(m_MasterInstanceList.begin(), m_MasterInstanceList.end(), key, TMasterInstanceLess());
    if(itr != m_MasterInstanceList.end() && itr->m_type == mastertype){
        return std::distance(m_MasterInstanceList.begin(), itr);
    }*/
    int sz = m_MasterInstanceList.size();
    for(int i=0; i<sz; ++i){
        if(m_MasterInstanceList[i].m_type == mastertype){
            return i;
        }
    }
    return -1;
}
//---------------------------------------------------------------------------
int TOpenFarenGame::GetMasterInstanceIndex(const std::string& id) const
{
    int mastertype = m_Data->GetMasterIndex(id);
    if(mastertype == -1){
        return -1;
    }
    return GetMasterInstanceIndex(mastertype);
}
//---------------------------------------------------------------------------
int TOpenFarenGame::GetComEmployCost(int charatype) const
{
    static const int RateData[] = {60,100,180,270,400};
    if(m_GameLevel<1 || m_GameLevel>GAMELEVEL_MAX){ g_Logger.ErrorLog("TOpenFarenGame::GetCOMEmployCost gorua"); return 1; }
    return m_Data->GetCharacter(charatype).m_employcost * RateData[m_GameLevel - 1] / 100;      // TODO:ٗpRXg
}
//---------------------------------------------------------------------------
// GAƌٗpSLw肵Aٗp\LXg𓾂BRXg͍lȂB
void TOpenFarenGame::GetEmployable(int areaindex, int chrindex, std::vector<int>& employable) const
{
    const TAreaInstance& area = m_AreaInstanceList[areaindex];
    const TBattleCharacter& charadata = m_Data->GetCharacter(area.m_charalist[chrindex].m_type);
    employable = charadata.m_employable;
    employable.insert(employable.end(), area.m_employable.begin(), area.m_employable.end());
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::EmployCharacter(int areaindex, int chrindex, int employtype)
{
    if(areaindex < 0 || areaindex >= m_AreaInstanceList.size()){
        g_Logger.ErrorLog((boost::format("TOpenFarenGame::EmployCharacter(%d, %d, %d) invalid parameter areaindex.") % areaindex % chrindex % employtype).str());
        return false;
    }
    TAreaInstance& area = m_AreaInstanceList[areaindex];
    if(chrindex < 0 || chrindex >= area.m_charalist.size() || area.m_charalist[chrindex].m_actioned){
        g_Logger.ErrorLog((boost::format("TOpenFarenGame::EmployCharacter(%d, %d, %d) invalid parameter chrindex.") % areaindex % chrindex % employtype).str());
        return false;
    }
    if(area.m_charalist.size() >= AREACHARA_MAX){
        g_Logger.ErrorLog((boost::format("TOpenFarenGame::EmployCharacter(%d, %d, %d) invalid operation. character overflow.") % areaindex % chrindex % employtype).str());
        return false;
    }
    if(area.m_master == -1){
        g_Logger.ErrorLog((boost::format("TOpenFarenGame::EmployCharacter(%d, %d, %d) invalid operation. no master area.") % areaindex % chrindex % employtype).str());
        return false;
    }
    if(employtype < 0 || employtype >= m_Data->GetCharacterCount()){
        g_Logger.ErrorLog((boost::format("TOpenFarenGame::EmployCharacter(%d, %d, %d) invalid parameter employtype.") % areaindex % chrindex % employtype).str());
        return false;
    }

    // COM}X^[͌ٗpL̃NRXgႢB
    const TMasterInstance& masterins = m_MasterInstanceList[area.m_master];
    const TBattleCharacter& charadata_e = m_Data->GetCharacter(employtype);
    int rank;
    int cost;
    if(masterins.m_player){
        rank = 0;
        cost = charadata_e.m_employcost;
    }
    else{
        rank = m_GameLevel / 2;
        cost = GetComEmployCost(employtype);
    }

    if(m_MasterInstanceList[area.m_master].m_money < cost){
        g_Logger.ErrorLog((boost::format("TOpenFarenGame::EmployCharacter(%d, %d, %d) invalid operation. master money < character cost.") % areaindex % chrindex % employtype).str());
        return false;
    }

    // 푰ႤAGAٗp\LłȂΌٗpSsς݂ɂB
    const TBattleCharacter& charadata = m_Data->GetCharacter(area.m_charalist[chrindex].m_type);
    if(charadata.m_race != charadata_e.m_race ||
       std::find(charadata.m_employable.begin(), charadata.m_employable.end(), employtype) != charadata.m_employable.end()){
        area.m_charalist[chrindex].m_actioned = true;
    }
    // ٗpLǉB
    TCharacterInstance c = CreateCharaInstance(employtype, rank);
    c.m_actioned = true;
    area.m_charalist.push_back(c);
    // \[gB
    SortAreaCharacter(areaindex);
    // ٗpRXgB
    m_MasterInstanceList[area.m_master].m_money -= cost;
    return true;
}
//---------------------------------------------------------------------------
// 
bool TOpenFarenGame::DismissCharacter(int areaindex, int chrindex)
{
    if(areaindex < 0 || areaindex >= m_AreaInstanceList.size()){
        g_Logger.ErrorLog((boost::format("TOpenFarenGame::DismissCharacter(%d, %d) invalid parameter areaindex.") % areaindex % chrindex).str());
        return false;
    }
    TAreaInstance& area = m_AreaInstanceList[areaindex];
    if(chrindex < 0 || chrindex >= area.m_charalist.size() || area.m_charalist[chrindex].m_actioned){
        g_Logger.ErrorLog((boost::format("TOpenFarenGame::DismissCharacter(%d, %d, %d) invalid parameter chrindex.") % areaindex % chrindex).str());
        return false;
    }
    if(area.m_master == -1){
        g_Logger.ErrorLog((boost::format("TOpenFarenGame::DismissCharacter(%d, %d, %d) invalid operation. no master area.") % areaindex % chrindex).str());
        return false;
    }
    area.m_charalist.erase(area.m_charalist.begin() + chrindex);
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::MoveCharacter(int area_from, int area_to, const std::vector<int>& indice)
{
    std::vector<int> dmy;
    return MoveCharacter(area_from, area_to, indice, dmy);
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::MoveCharacter(int area1, int area2, const std::vector<int>& indice1to2, const std::vector<int>& indice2to1)
{
    TAreaInstance& area1i = m_AreaInstanceList[area1];
    TAreaInstance& area2i = m_AreaInstanceList[area2];
    if(indice1to2.size() > area1i.m_charalist.size() || indice1to2.size() + area2i.m_charalist.size() - indice2to1.size() > AREACHARA_MAX ||
       indice2to1.size() > area2i.m_charalist.size() || indice2to1.size() + area1i.m_charalist.size() - indice1to2.size() > AREACHARA_MAX ){
        g_Logger.ErrorLog((boost::format("TOpenFarenGame::MoveCharacter invalid parameter(too many indice). Area1=%d(characters=%d) Area2=%d(characters=%d) indice size=%d,%d") % area1 % area1i.m_charalist.size() % area2 % area2i.m_charalist.size() % indice1to2.size() % indice2to1.size()).str());
        return false;
    }
    // dvf`FbNB
    std::vector<int> uniquecheck1 = indice1to2;
    std::vector<int> uniquecheck2 = indice2to1;
    std::sort(uniquecheck1.begin(), uniquecheck1.end());
    std::sort(uniquecheck2.begin(), uniquecheck2.end());
    if(std::unique(uniquecheck1.begin(), uniquecheck1.end()) != uniquecheck1.end() ||
       std::unique(uniquecheck2.begin(), uniquecheck2.end()) != uniquecheck2.end()){
        g_Logger.ErrorLog((boost::format("TOpenFarenGame::MoveCharacter invalid parameter(indice contains duplication). Area1=%d Area2=%d") % area1 % area2).str());
        return false;
    }

    for(int i = uniquecheck1.size() - 1; i >= 0; --i){
        int index = uniquecheck1[i];
        if(index < 0 || index >= area1i.m_charalist.size()){
            OFERRLOG((boost::format("invalid parameter(index out of range). Area1=%d Area2=%d") % area1 % area2).str());
            return false;
        }
        area1i.m_charalist[index].m_actioned = true;
        area2i.m_charalist.push_back(area1i.m_charalist[index]);
        area1i.m_charalist.erase(area1i.m_charalist.begin() + index);
    }
    for(int i = uniquecheck2.size() - 1; i >= 0; --i){
        int index = uniquecheck2[i];
        if(index < 0 || index >= area2i.m_charalist.size()){
            OFERRLOG((boost::format("invalid parameter(index out of range). Area1=%d Area2=%d") % area1 % area2).str());
            return false;
        }
        area2i.m_charalist[index].m_actioned = true;
        area1i.m_charalist.push_back(area2i.m_charalist[index]);
        area2i.m_charalist.erase(area2i.m_charalist.begin() + index);
    }

    SortAreaCharacter(area1);
    SortAreaCharacter(area2);
    return true;
}
//---------------------------------------------------------------------------
// GA𒆗nтɂB
void TOpenFarenGame::SetAreaNoMaster(int masterindex)
{
    int sz = m_AreaInstanceList.size();
    for(int i=0; i<sz; ++i){
        if(m_AreaInstanceList[i].m_master == masterindex){
            // lރLāAQ\lނȂQB
            unsigned sz = m_AreaInstanceList[i].m_charalist.size();
            for(unsigned u=0; u<sz; ++u){
                const TCharacterInstance& chara = m_AreaInstanceList[i].m_charalist[u];
                const TBattleCharacter& charadata = m_Data->GetCharacter(chara.m_type);
                if(charadata.m_hero && !charadata.m_nowander){
                    AddWanderChara(chara);
                }
            }
            m_AreaInstanceList[i].m_master = -1;
            m_AreaInstanceList[i].m_charalist.clear();
            PlaceAreaCharaRandom(i);
        }
    }
}
//---------------------------------------------------------------------------
// vC[tO𗧂ĂBmastertypeTMaster̃CfbNXB
// ̊֐̃C^[tF[X͒PɃ}X^[IʂƂ̌ˍŌ܂́B
void TOpenFarenGame::SetMasterPlayer(int mastertype, bool isPlayer)
{
    // }X^[͕Ȃ̂ƂB
    for(unsigned u=0; u<m_MasterInstanceList.size(); u++){
        if(m_MasterInstanceList[u].m_type == mastertype){
            m_MasterInstanceList[u].m_player = isPlayer;
            g_Logger.DebugLog((boost::format("SetMasterPlayer=%u") % u).str());
            break;
        }
    }
}
//---------------------------------------------------------------------------
TCharacterInstance TOpenFarenGame::CreateCharaInstance(int ctype, int rank) const
{
    const TBattleCharacter& charadata = m_Data->GetCharacter(ctype);
    TCharacterInstance c;
    c.m_dummy = 0;
    c.m_rank = rank;
    c.m_type = ctype;
    UpdateCharaRank(c);
    c.m_hp = c.m_hpmax_rank;
    c.m_mp = c.m_mpmax_rank;
    c.m_resistance = charadata.m_resistance;
    c.m_movetype = charadata.m_movetype;
    c.m_power = EvalCharaPower(c);
    return c;
}
//---------------------------------------------------------------------------
void TOpenFarenGame::UpdateCharaRank(TCharacterInstance& c) const
{
    // b\͒l100̏ꍇAENF100ADNF108ACNF116ABNF125AANF133ASN:166B
    static const int rankpower[RANK_MAX+1] = {100, 108, 116, 125, 133, 166};
    const TBattleCharacter& charadata = m_Data->GetCharacter(c.m_type);
    c.m_hpmax_rank = charadata.m_hpmax;
    c.m_mpmax_rank = charadata.m_mpmax;
    c.m_pa_rank = charadata.m_pa * rankpower[c.m_rank] / 100;
    c.m_pd_rank = charadata.m_pd * rankpower[c.m_rank] / 100;
    c.m_skill_rank = charadata.m_skill * rankpower[c.m_rank] / 100;
    c.m_speed_rank = charadata.m_speed * rankpower[c.m_rank] / 100;
    c.m_ma_rank = charadata.m_ma * rankpower[c.m_rank] / 100;
    c.m_md_rank = charadata.m_md * rankpower[c.m_rank] / 100;
    // ͎wXVB
    c.m_power = EvalCharaPower(c);
}
//---------------------------------------------------------------------------
// ol烉NAbvsBKvȂNX`FWB
bool TOpenFarenGame::CheckRankup(TCharacterInstance& c) const
{
    bool classchange = false;
    if(c.m_experience > TOpenFarenGame::EXPERIENCE_MAX){
        if(c.m_rank < TOpenFarenGame::RANK_MAX){
            c.m_experience = 0;
            ++c.m_rank;
            if(c.m_rank >= TOpenFarenGame::RANK_MAX){
                const TBattleCharacter& charadata = m_Data->GetCharacter(c.m_type);
                if(charadata.m_classchange != -1){
                    classchange = true;
                    const TBattleCharacter& charadata2 = m_Data->GetCharacter(charadata.m_classchange);
                    int hpup = charadata2.m_hpmax - c.m_hpmax_rank;
                    int mpup = charadata2.m_mpmax - c.m_mpmax_rank;
                    c.m_type = charadata.m_classchange;
                    c.m_rank = 0;
                    c.m_hp += hpup;
                    c.m_mp += mpup;
                    if(c.m_hp <= 0){
                        c.m_hp = 1;
                    }
                    if(c.m_mp < 0){
                        c.m_mp = 0;
                    }
                }
            }
            UpdateCharaRank(c);
        }
        else{
            c.m_experience = TOpenFarenGame::EXPERIENCE_MAX;
        }
    }
    return classchange;
}
//---------------------------------------------------------------------------
int TOpenFarenGame::EvalCharaPower(const TCharacterInstance& c) const
{
    const TBattleCharacter& charadata = m_Data->GetCharacter(c.m_type);
    return charadata.m_power * 1000 + c.m_rank * 101 + c.m_power;
}
//---------------------------------------------------------------------------
int TOpenFarenGame::GetAreaIncome(int areaindex) const
{
    const TAreaInstance& area = m_AreaInstanceList[areaindex];
    return (area.m_baseincome + (area.m_city / 4)) * (1 + (area.m_road == area.m_roadmax));
}
//---------------------------------------------------------------------------
int TOpenFarenGame::GetAreaPower(int ai) const
{
    const TAreaInstance& area = m_AreaInstanceList[ai];
    unsigned sz = area.m_charalist.size();
    int power = 0;
    for(unsigned u=0; u<sz; ++u){
        power += area.m_charalist[u].m_power;
    }
    return power;
}
//---------------------------------------------------------------------------
int TOpenFarenGame::GetMasterBaseArea(int masterindex) const
{
    // }X^[̊LGAB
    int facechara = m_Data->GetMaster(m_MasterInstanceList[masterindex].m_type).m_facecharaindex;
    int areaindex, charaindex;
    if(FindCharaExistsArea(masterindex, facechara, areaindex, charaindex)){
        return areaindex;
    }
    else{
        return -1;
    }
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::FindCharaExistsArea(int masterindex, int charatype, int& areaindex, int& charaindex) const
{
    unsigned sz = m_AreaInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        if(m_AreaInstanceList[u].m_master == masterindex){
            unsigned chrs = m_AreaInstanceList[u].m_charalist.size();
            for(unsigned chr = 0; chr < chrs; ++chr){
                if(charatype == m_AreaInstanceList[u].m_charalist[chr].m_type){
                    areaindex = u;
                    charaindex = chr;
                    return true;
                }
            }
        }
    }
    return false;
}
//---------------------------------------------------------------------------
// }X^[̗̒n擾B
int TOpenFarenGame::GetMasterAreaCount(int masterindex) const
{
    int count = 0;
    unsigned sz = m_AreaInstanceList.size();
    for(int ai=0; ai<sz; ++ai){
        if(m_AreaInstanceList[ai].m_master == masterindex){
            ++count;
        }
    }
    return count;
}
//---------------------------------------------------------------------------
// }X^[̐lސ擾B
int TOpenFarenGame::GetMasterHeroCount(int masterindex) const
{
    int count = 0;
    unsigned sz = m_AreaInstanceList.size();
    for(int ai=0; ai<sz; ++ai){
        if(m_AreaInstanceList[ai].m_master == masterindex){
            unsigned chrs = m_AreaInstanceList[ai].m_charalist.size();
            for(unsigned chr = 0; chr < chrs; ++chr){
                const TBattleCharacter& charadata = m_Data->GetCharacter(m_AreaInstanceList[ai].m_charalist[chr].m_type);
                if(charadata.m_hero){
                    ++count;
                }
            }
        }
    }
    return count;
}
//---------------------------------------------------------------------------
// }X^[͎̐w擾B
int TOpenFarenGame::GetMasterPower(int masterindex) const
{
    int power = 0;
    unsigned sz = m_AreaInstanceList.size();
    for(int ai=0; ai<sz; ++ai){
        if(m_AreaInstanceList[ai].m_master == masterindex){
            power += GetAreaPower(ai);
        }
    }
    return power;
}
//---------------------------------------------------------------------------
// VKLǉłGABforce=trueȂ疳iŌ̃L㏑jB
int TOpenFarenGame::FindCharaInsertableArea(int masterindex, bool force) const
{
    int minpower = INT_MAX; // ԎアL͎̐wB
    int minarea = -1;       // ԎアLGAB
    unsigned sz = m_AreaInstanceList.size();
    for(int ai=0; ai<sz; ++ai){
        if(m_AreaInstanceList[ai].m_master == masterindex){
            if(m_AreaInstanceList[ai].m_charalist.size() < AREACHARA_MAX){
                return ai;
            }
            else if(force){
                if(minpower > m_AreaInstanceList[ai].m_charalist[AREACHARA_MAX-1].m_power){
                    minpower = m_AreaInstanceList[ai].m_charalist[AREACHARA_MAX-1].m_power;
                    minarea = ai;
                }
            }
        }
    }
    return minarea;
}
//---------------------------------------------------------------------------
void TOpenFarenGame::InsertChara(int areaindex, int charatype)
{
    const TBattleCharacter& charadata = m_Data->GetCharacter(charatype);
    TAreaInstance& area = m_AreaInstanceList[areaindex];

    // lނŁAQĂ̂ȂQXgoăGAɔzuB
    if(charadata.m_hero){
        int index = FindCharaInstance(m_WanderingCharaList, charatype);
        if(index != -1){
            m_AreaInstanceList[areaindex].m_charalist.push_back(*(m_WanderingCharaList.begin() + index));
            SortAreaCharacter(areaindex);
            m_WanderingCharaList.erase(m_WanderingCharaList.begin() + index);
            return;
        }
    }

    // lނłȂQĂȂȂPɐVBlނ̓dzu͂ł̓`FbNĂȂB
    int rank = 0;
    if(area.m_master != -1){
        // COM}X^[͏oL̃NB
        const TMasterInstance& masterins = m_MasterInstanceList[area.m_master];
        if(masterins.m_player){
            rank = 0;
        }
        else{
            rank = m_GameLevel / 2;
        }
    }
    // LǉB
    TCharacterInstance c = CreateCharaInstance(charatype, rank);
    area.m_charalist.push_back(c);
    // \[gB
    SortAreaCharacter(areaindex);
}
//---------------------------------------------------------------------------
void TOpenFarenGame::RemoveChara(int areaindex, int charaindex)
{
    TAreaInstance& area = m_AreaInstanceList[areaindex];
    area.m_charalist.erase(area.m_charalist.begin() + charaindex);
}
//---------------------------------------------------------------------------
// SGAă}X^[͎̐wXVB
void TOpenFarenGame::UpdateMasterPower()
{
    unsigned sz = m_MasterInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        m_MasterInstanceList[u].m_power = 0;
    }
    sz = m_AreaInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        if(m_AreaInstanceList[u].m_master != -1){
            m_MasterInstanceList[m_AreaInstanceList[u].m_master].m_power += GetAreaPower(u);
        }
    }
}
//---------------------------------------------------------------------------
// }X^[̎xԂXVB
void TOpenFarenGame::UpdateMasterIncome()
{
    unsigned sz = m_MasterInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        m_MasterInstanceList[u].m_income = 0;
        m_MasterInstanceList[u].m_outgo = 0;
    }
    // SGA̎WvB
    // ܂ASGA̐lނ̈ێWvB
    sz = m_AreaInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        if(m_AreaInstanceList[u].m_master != -1){
            m_MasterInstanceList[m_AreaInstanceList[u].m_master].m_income += GetAreaIncome(u);
            unsigned chrs = m_AreaInstanceList[u].m_charalist.size();
            for(unsigned chr = 0; chr < chrs; chr++){
                const TBattleCharacter& charadata = m_Data->GetCharacter(m_AreaInstanceList[u].m_charalist[chr].m_type);
                if(charadata.m_hero){
                    m_MasterInstanceList[m_AreaInstanceList[u].m_master].m_outgo += charadata.m_turncost;
                }
            }
        }
    }
}
//---------------------------------------------------------------------------
// }X^[̑ݏԂXVB
void TOpenFarenGame::UpdateMasterExists()
{
    bool updateAlliance = false;
    unsigned sz = m_MasterInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        if(GetMasterBaseArea(u) == -1){
            SetAreaNoMaster(u);
            m_MasterInstanceList[u].m_exists = false;
            for(TAllianceList::iterator itr = m_AllianceList.begin(); itr != m_AllianceList.end(); ++itr){
                if(itr->master1 == u || itr->master2 == u){
                    itr->turn = 0;
                    updateAlliance = true;
                }
            }
        }
    }
    if(updateAlliance){
        RemoveExpiredAlliance();
    }
}
//---------------------------------------------------------------------------
// JudgeOvercome()^ɂȂɌĂяoB
void TOpenFarenGame::StartEnding()
{
    m_IsEnding = true;
    // ݂̃}X^[cĂꍇÃ}X^[̃GfBOƌȂB
    if(m_MasterInstanceList[m_CurrentMaster].m_exists){
        return;
    }
    // ݂̃}X^[łłꍇA݃}X^[̎Ƃđɑ݂Ă}X^[̃GfBOƌȂB
    unsigned sz = m_MasterInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        if(m_MasterInstanceList[u].m_exists){
            m_CurrentMaster = u;
            break;
        }
    }
}
//---------------------------------------------------------------------------
int TOpenFarenGame::FindAlliance(int master1, int master2) const
{
    for(TAllianceList::const_iterator itr = m_AllianceList.begin(); itr != m_AllianceList.end(); ++itr){
        if((itr->master1 == master1 && itr->master2 == master2) ||
           (itr->master1 == master2 && itr->master2 == master1)){
            return std::distance(m_AllianceList.begin(), itr);
        }
    }
    return -1;
}
//---------------------------------------------------------------------------
const TAlliance& TOpenFarenGame::GetAlliance(int index) const
{
    return m_AllianceList[index];
}
//---------------------------------------------------------------------------
void TOpenFarenGame::SetAlliance(int master1, int master2, int turn)
{
    int index = FindAlliance(master1, master2);
    if(index != -1){
        m_AllianceList[index].turn = turn;
        return;
    }
    TAlliance al;
    al.master1 = master1;
    al.master2 = master2;
    al.turn = turn;
    m_AllianceList.push_back(al);
}
//---------------------------------------------------------------------------
// P폜B
void TOpenFarenGame::RemoveAlliance(int index)
{
    m_AllianceList.erase(m_AllianceList.begin() + index);
}
//---------------------------------------------------------------------------
// t@[gD[K1.3xTEventForm::LeagueLey()ƓB
int TOpenFarenGame::EvalAllianceCost(int masterindex1, int masterindex2, int turn) const
{
    if(turn == -1){
        turn = DEFAULT_ALLIANCETURN;
        int alindex = FindAlliance(masterindex1, masterindex2);
        if(alindex != -1){
            turn -= m_AllianceList[alindex].turn;
        }
    }
    int cost = m_MasterInstanceList[masterindex2].m_income / 10;
    if(cost < 1){
        cost = 1;
    }
    return cost * turn;
}
//---------------------------------------------------------------------------
// Q[[ɑB\񂾑̎炷B
void TOpenFarenGame::ConcludeAlliance(int master1, int master2, int turn, int cost)
{
    m_MasterInstanceList[master1].m_money -= cost;
    m_MasterInstanceList[master2].m_money += cost;
    int index = FindAlliance(master1, master2);
    if(index != -1){
        m_AllianceList[index].turn += turn;
        return;
    }
    TAlliance al;
    al.master1 = master1;
    al.master2 = master2;
    al.turn = turn;
    m_AllianceList.push_back(al);
}
//---------------------------------------------------------------------------
/*bool TOpenFarenGame::ProposeAlliance(int masterindex, int turn)
{
    if 
    m_ProposedAllianceList.push_back(boost::tuple<int, int, int>(GetCurrentMasterIndex(), masterindex, turn));
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::GetProposedAlliance(int& masterindex, int& turn) const
{
    int curmaster = GetCurrentMasterIndex();
    for(TProposedAllianceList::const_iterator itr = m_ProposedAllianceList.begin(); itr != m_ProposedAllianceList.end(); ++itr){
        if(itr->get<0>() == curmaster){
            masterindex = itr->get<1>();
            turn = itr->get<2>();
            return true;
        }
    }
    return false;
}
//---------------------------------------------------------------------------
void TOpenFarenGame::PopProposedAlliance()
{
    int curmaster = GetCurrentMasterIndex();
    for(TProposedAllianceList::iterator itr = m_ProposedAllianceList.begin(); itr != m_ProposedAllianceList.end(); ++itr){
        if(itr->get<0>() == curmaster){
            m_ProposedAllianceList.erase(itr);
            break;
        }
    }
}*/
//---------------------------------------------------------------------------
// ^[JnɃ}X^[̍s߂B
void TOpenFarenGame::OrderMaster()
{
    unsigned sz = m_MasterInstanceList.size();
    m_MasterOrder.resize(sz);
    for(unsigned u=0; u<sz; ++u){
        m_MasterOrder[u].first = u;
        m_MasterOrder[u].second = m_MasterInstanceList[u].m_power;
    }
    std::sort(m_MasterOrder.begin(), m_MasterOrder.end(), TMasterPowerLess());
    m_MasterOrderIndex = -1;
}
//---------------------------------------------------------------------------
// ̃}X^[̔ԂɂB^[̏IȂ^[XVB^[XVꍇA߂l͐^B
bool TOpenFarenGame::ActivateNextMaster()
{
/*
    int minwait = INT_MAX;
    int nextmaster = -1;
    unsigned sz = m_MasterInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        if(minwait > m_MasterInstanceList[u].m_wait){
            minwait = m_MasterInstanceList[u].m_wait;
            nextmaster = u;
        }
    }
    for(unsigned u=0; u<sz; ++u){
        m_MasterInstanceList[u].m_wait -= minwait;
    }
    m_CurrentMaster = nextmaster;
*/

OFDBGLOG("A");
    // xXVB
    UpdateMasterIncome();

    // ̐ʂ𔻒B
    if(m_CurrentMaster != -1){
        ApplyAdministration();
    }

    // sς݃tONAB
    unsigned areacount = m_AreaInstanceList.size();
    for(unsigned u=0; u<areacount; ++u){
        TAreaInstance& area = m_AreaInstanceList[u];
        for(unsigned chr=0; chr<area.m_charalist.size(); ++chr){
            area.m_charalist[chr].m_actioned = false;
        }
    }

    ++m_MasterOrderIndex;
    int nextmaster = -1;
    while(m_MasterOrderIndex < m_MasterOrder.size()){
        int i = m_MasterOrder[m_MasterOrderIndex].first;
        if(m_MasterInstanceList[i].m_exists){
            nextmaster = i;
            break;
        }
        ++m_MasterOrderIndex;
    }
    if(nextmaster != -1){
        //g_Logger.DebugLog((boost::format("CurrentMaster=%d") % nextmaster).str());
        m_CurrentMaster = nextmaster;
        // lޑ{ʔB
        CheckSearchHero();
OFDBGLOG("B");
        return false;
    }

    // Ƃ肠Ń}X^[̎ȂǁB
    unsigned sz = m_MasterInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        m_MasterInstanceList[u].m_money += m_MasterInstanceList[u].m_income;
        m_MasterInstanceList[u].m_money -= m_MasterInstanceList[u].m_outgo;
    }

    RemoveExpiredAlliance();
    // ̎c^[炷B
    // ̌Ăяo܂łRemoveExpiredAlliance()Ă΂turn=0̗vf͍폜̂ƂB2008/11/09 Ăяoʒu܂łȂ̂łŌĂԁB
    for(TAllianceList::iterator itr = m_AllianceList.begin(); itr != m_AllianceList.end(); ++itr){
        if(itr->turn != -1){
            --itr->turn;
        }
    }

    // fobOplރ`FbNB
    if(g_Ini.GetLogLevel() > 2){
        std::set<int> herolist;
        for(TAreaInstanceList::const_iterator itr = m_AreaInstanceList.begin(); itr != m_AreaInstanceList.end(); ++itr){
            // GA݃L`FbNB
            const TAreaInstance& area = *itr;
            unsigned sz = area.m_charalist.size();
            for(unsigned u=0; u<sz; ++u){
                int ctype = area.m_charalist[u].m_type;
                const TBattleCharacter& charadata = m_Data->GetCharacter(ctype);
                if(charadata.m_hero){
                    if(!herolist.insert(ctype).second){
                        g_Logger.DebugLog((boost::format("`FbNFlށiGA݁j[%s]H") % charadata.m_name).str());
                    }
                }
            }
        }
        unsigned sz = m_WanderingCharaList.size();
        for(unsigned u=0; u<sz; ++u){
            int ctype = m_WanderingCharaList[u].m_type;
            const TBattleCharacter& charadata = m_Data->GetCharacter(ctype);
            if(charadata.m_hero){
                if(!herolist.insert(ctype).second){
                    g_Logger.DebugLog((boost::format("`FbNFlށiQj[%s]H") % charadata.m_name).str());
                }
            }
        }
        sz = m_DeadCharaList.size();
        for(unsigned u=0; u<sz; ++u){
            int ctype = m_DeadCharaList[u].m_type;
            const TBattleCharacter& charadata = m_Data->GetCharacter(ctype);
            if(charadata.m_hero){
                if(!herolist.insert(ctype).second){
                    g_Logger.DebugLog((boost::format("`FbNFlށiSj[%s]H") % charadata.m_name).str());
                }
            }
        }
        sz = m_Data->GetCharacterCount();
        for(unsigned u=0; u<sz; ++u){
            const TBattleCharacter& charadata = m_Data->GetCharacter(u);
            if(charadata.m_hero && !charadata.m_nowander){
                if(herolist.find(u) == herolist.end()){
                    if(m_Turn > charadata.m_turnmin && (charadata.m_turnmax == -1 || m_Turn < charadata.m_turnmax)){
                        g_Logger.DebugLog((boost::format("`FbNFl[%s]ssiGA݂łQłSłȂj") % charadata.m_name).str());
                    }
                }
            }
        }
    }

    ++m_Turn;
    ++m_ProgressedTurn;
    OrderMaster();
    ++m_MasterOrderIndex;
    // [vB
    while(m_MasterOrderIndex < m_MasterOrder.size()){
        int i = m_MasterOrder[m_MasterOrderIndex].first;
        if(m_MasterInstanceList[i].m_exists){
            nextmaster = i;
            break;
        }
        ++m_MasterOrderIndex;
    }
    if(nextmaster == -1){
        g_Logger.ErrorLog("̃}X^[̌Ɏsi}X^[SŁHj");
        return true;
    }
    //g_Logger.DebugLog((boost::format("CurrentMaster=%d") % nextmaster).str());
    m_CurrentMaster = nextmaster;
    // ^[isɂQlޑB
    UpdateWanderingHero();
    // lޑ{ʔB
    CheckSearchHero();
OFDBGLOG("C");
    return true;
}
//---------------------------------------------------------------------------
// w肵GÃLN^[\[gB
void TOpenFarenGame::SortAreaCharacter(const std::set<int>& arealist)
{
    for(std::set<int>::const_iterator itr = arealist.begin(); itr != arealist.end(); ++itr){
        SortAreaCharacter(*itr);
    }
}
//---------------------------------------------------------------------------
// w肵GÃLN^[\[gB
void TOpenFarenGame::SortAreaCharacter(int areaindex)
{
    TAreaInstance& area = m_AreaInstanceList[areaindex];
    int race = -1;
    if(area.m_master != -1){
        const TMaster& master = m_Data->GetMaster(m_MasterInstanceList[area.m_master].m_type);
        race = m_Data->GetCharacter(master.m_facecharaindex).m_race;
    }
    std::stable_sort(area.m_charalist.begin(), area.m_charalist.end(), TCharacterLess(m_Data, race));
}
//---------------------------------------------------------------------------
// 퓬AolȂǂGÃLCX^XɃRs[B
void TOpenFarenGame::ApplyCharaAfterBattle(TCharacterInstance& chara_area, const TCharacterInstance& chara_bt) const
{
    //chara_area.m_type = chara_bt.m_type;
    //chara_area.m_rank = chara_bt.m_rank;
    if(chara_area.m_type != chara_bt.m_type){
        chara_area = CreateCharaInstance(chara_bt.m_type, chara_bt.m_rank);
    }
    chara_area.m_experience = chara_bt.m_experience;
    chara_area.m_wincount = chara_bt.m_wincount;
    //const std::string& name = m_Data->GetCharacter(chara_area.m_type).m_name;
}
//---------------------------------------------------------------------------
// GACfbNXƃLCfbNX̃yÃXgŃL폜B
// LCfbNX͏ŕł̂ƂB
// savrestlist͎Lꎞޔ̈BskꍇɎclރLɓBlޕQpB
// A}X^[łȂꍇ͏ʂłsavrestlistɎgȂiPɖʂɂȂjB
void TOpenFarenGame::MoveCharaAfterBattleDepWin(const std::vector<std::pair<int, int> >& indexlist,
    const TCharacterInstanceList& depcharalist, const TCharacterInstanceList& savcharalist,
    TCharacterInstanceList& savrestlist,
    int areaindex, int depmaster, bool depwin)
{
    std::set<int> updatearea;   // LXgXVGÃXgB

    TAreaInstance& area_target = m_AreaInstanceList[areaindex];
    for(int i = 0; i < area_target.m_charalist.size(); ++i){
        TCharacterInstance& chara_area = area_target.m_charalist[i];    // GÃLCX^XB
        const TCharacterInstance& chara_bt = savcharalist[i];           // 퓬ɎgLCX^XB
        if(!chara_bt.m_hp){
            const TBattleCharacter& charadata = m_Data->GetCharacter(chara_bt.m_type);
            if(charadata.m_hero){
                // SlރXgɒǉB
                m_DeadCharaList.push_back(chara_bt);
            }
            continue;
        }
        // 퓬̌olȂǂRs[B
        ApplyCharaAfterBattle(chara_area, chara_bt);
        chara_area.m_actioned = true;
        // LSĂ炸SĂΗאڃGAɈړB
        if(chara_bt.m_runaway){
            int narea = MoveCharaToNeighbor(area_target, area_target.m_master, chara_area);
            if(narea != -1){
                // GA\[gΏۂɒǉB
                updatearea.insert(narea);
            }
            else{
                OFERRLOG("vÕoOFLڂאڃGA");
            }
        }
    }

    if(!depwin){
        // ̏ꍇASƓSL폜B
        for(int i = area_target.m_charalist.size() - 1; i >= 0; --i){
            if(!savcharalist[i].m_hp || savcharalist[i].m_runaway){
                area_target.m_charalist.erase(area_target.m_charalist.begin() + i);
            }
        }
    }

    // ꂽLȊOŁA퓬LiŉLjGAɒǉB
    // isk̏ꍇ͓LȊȌ͌ǖʂɂȂ邯ǁj
    // 
    for(int i = area_target.m_charalist.size(); i < savcharalist.size(); ++i){
        const TCharacterInstance& chara_bt = savcharalist[i];           // 퓬ɎgLCX^XB
        // Sє폢L͂Ƃ΂B
        if(!chara_bt.m_hp || chara_bt.m_elementstatus == TCharacterInstance::ELEMENT_SUMMONED){
            continue;
        }
        // ɂȂGAL̂ŐV쐬B
        TCharacterInstance chara_area = CreateCharaInstance(chara_bt.m_type, chara_bt.m_rank);
        ApplyCharaAfterBattle(chara_area, chara_bt);
        chara_area.m_actioned = true;
        if(chara_bt.m_runaway){
            int narea = MoveCharaToNeighbor(area_target, area_target.m_master, chara_area);
            if(narea != -1){
                // GA\[gΏۂɒǉB
                updatearea.insert(narea);
            }
            else{
                const TBattleCharacter& charadata = m_Data->GetCharacter(chara_area.m_type);
                OFWRNLOG((boost::format("L %s ڂאڃGÂŔj܂i앜ŔLӂꂽHj") % charadata.m_name).str());
            }
        }
        else{
            if(area_target.m_charalist.size() < AREACHARA_MAX){
                area_target.m_charalist.push_back(chara_area);
            }
            else{
                const TBattleCharacter& charadata = m_Data->GetCharacter(chara_area.m_type);
                OFWRNLOG((boost::format("L %s GAɒuȂ̂Ŕj܂i앜ŔLӂꂽHj") % charadata.m_name).str());
            }
        }
    }

    // ŎGȀLXVB
    if(depwin){
        // cĂLŐlނQLޔB
        for(int i=0; i<area_target.m_charalist.size(); ++i){
            if(area_target.m_charalist[i].m_hp){
                const TBattleCharacter& charadata = m_Data->GetCharacter(area_target.m_charalist[i].m_type);
                if(charadata.m_hero && !charadata.m_nowander){
                    savrestlist.push_back(area_target.m_charalist[i]);
                }
            }
        }
        area_target.m_charalist.clear();    // ̃LB
        area_target.m_master = depmaster;   // }X^[XVB
        area_target.m_operationeffort = 0;  // ʂB
    }

    // ŨLɐ퓬ʓKp
    for(int i = indexlist.size() - 1; i >= 0; --i){
        TAreaInstance& area_from = m_AreaInstanceList[indexlist[i].first];
        TCharacterInstance& chara_area = area_from.m_charalist[indexlist[i].second]; // GÃLCX^XB
        const TCharacterInstance& chara_bt = depcharalist[i];   // 퓬ɎgLCX^XB
        if(!chara_bt.m_hp){
            const TBattleCharacter& charadata = m_Data->GetCharacter(chara_bt.m_type);
            if(charadata.m_hero){
                // SlރXgɒǉB
                m_DeadCharaList.push_back(chara_bt);
            }
            continue;
        }
        // 퓬̌olȂǂRs[B
        ApplyCharaAfterBattle(chara_area, chara_bt);
        chara_area.m_actioned = true;
        // ȔꍇAŨLړ
        if(depwin){
            // SĂ炸ĂȂΏow̃GAɒǉB
            if(!chara_bt.m_runaway){
                area_target.m_charalist.push_back(chara_area);
                // owGA\[gΏۂɒǉB
                updatearea.insert(areaindex);
            }
        }
    }

    // ŨL폜
    for(int i = indexlist.size() - 1; i >= 0; --i){
        TAreaInstance& area_from = m_AreaInstanceList[indexlist[i].first];
        TCharacterInstance& chara_area = area_from.m_charalist[indexlist[i].second]; // GÃLCX^XB
        const TCharacterInstance& chara_bt = depcharalist[i];   // 퓬ɎgLCX^XB
        // SĂ邩AUĂȂΏow̃GA폜B
        if(!chara_bt.m_hp || (depwin && !chara_bt.m_runaway)){
            area_from.m_charalist.erase(area_from.m_charalist.begin() + indexlist[i].second);
        }
        else{
            chara_area.m_actioned = true;
        }
    }

    // ꂽLȊOŁA퓬LiŉLjGAɒǉB
    // U
    for(int i = indexlist.size(); i < depcharalist.size(); ++i){
        const TCharacterInstance& chara_bt = depcharalist[i];           // 퓬ɎgLCX^XB
        // Sє폢L͂Ƃ΂B
        if(!chara_bt.m_hp || chara_bt.m_elementstatus == TCharacterInstance::ELEMENT_SUMMONED){
            continue;
        }
        // ɂȂGAL̂ŐV쐬B
        TCharacterInstance chara_area = CreateCharaInstance(chara_bt.m_type, chara_bt.m_rank);
        ApplyCharaAfterBattle(chara_area, chara_bt);
        chara_area.m_actioned = true;
        // ow̔sk܂͓ĂꍇAאڃGAցBȊOȂGAցB
        if(!depwin || chara_bt.m_runaway){
            int narea = MoveCharaToNeighbor(area_target, depmaster, chara_area);
            if(narea != -1){
                // ދpGA\[gΏۂɒǉB
                updatearea.insert(narea);
            }
            else{
                const TBattleCharacter& charadata = m_Data->GetCharacter(chara_area.m_type);
                OFWRNLOG((boost::format("UދpL %s ڂאڃGÂŔj܂i앜ŔLӂꂽHj") % charadata.m_name).str());
            }
        }
        else{
            if(area_target.m_charalist.size() < AREACHARA_MAX){
                area_target.m_charalist.push_back(chara_area);
            }
            else{
                const TBattleCharacter& charadata = m_Data->GetCharacter(chara_area.m_type);
                OFWRNLOG((boost::format("UL %s GAɒuȂ̂Ŕj܂i앜ŔLӂꂽHj") % charadata.m_name).str());
            }
        }
    }

    SortAreaCharacter(updatearea);
}
//---------------------------------------------------------------------------
// ̃LȂĖ퓬ŏowړ̃LړB
// LCfbNX͏ŕł̂ƂB
void TOpenFarenGame::MoveCharaAfterNoBattle(const std::vector<std::pair<int, int> >& indexlist, int areaindex, int depmaster)
{
    TAreaInstance& area_target = m_AreaInstanceList[areaindex];
    area_target.m_master = depmaster;
    for(int i = indexlist.size() - 1; i >= 0; --i){
        TAreaInstance& area_from = m_AreaInstanceList[indexlist[i].first];
        TCharacterInstance chara_area = area_from.m_charalist[indexlist[i].second];
        chara_area.m_actioned = true;
        area_target.m_charalist.push_back(chara_area);
        area_from.m_charalist.erase(area_from.m_charalist.begin() + indexlist[i].second);
    }

    SortAreaCharacter(areaindex);
}
//---------------------------------------------------------------------------
/*
// 퓬Ƀ}X^[SꍇɌĂяoBQΏۂ̐lނoĕQB
// masterindex͎S}X^[B
// herolist͐퓬ɎQĐclރXgBɓĂlނ́AGAォCX^XĂ͂B
void TOpenFarenGame::WanderAfterWar(int masterindex, const TCharacterInstanceList& herolist)
{
    unsigned sz = m_AreaInstanceList.size();
    for(int ai=0; ai<sz; ++ai){
        TAreaInstance& area = m_AreaInstanceList[ai];
        if(area.m_master == masterindex){
            for(int i=0; i<area.m_charalist.size(); ++i){
                const TCharacterInstance& chara = area.m_charalist[i];
                if(m_Data->GetCharacter(chara.m_type).m_hero){
                    if(std::find(deadidlist.begin(), deadidlist.end(), chara.m_type) != deadidlist.end()){
                        m_WanderingCharaList.push_back(chara);
                    }
                }
            }
        }
    }
}
*/
//---------------------------------------------------------------------------
// QlޒǉB
void TOpenFarenGame::AddWanderChara(const TCharacterInstance& hero)
{
    //if(std::find(m_DeadCharaList.begin(), m_DeadCharaList.end(), hero.m_type) != m_DeadCharaList.end()){
    if(FindCharaInstance(m_DeadCharaList, hero.m_type) != -1){
        const TBattleCharacter& charadata = m_Data->GetCharacter(hero.m_type);
        OFERRLOG((boost::format("l %s Q悤Ƃ܂A̐lނ͊ɎSXgɓĂ܂ivO܂EventScript̃oOjB") % charadata.m_name).str());
    }
    if(FindCharaInstance(m_WanderingCharaList, hero.m_type) != -1){
        const TBattleCharacter& charadata = m_Data->GetCharacter(hero.m_type);
        OFERRLOG((boost::format("l %s Q悤Ƃ܂A̐lނ͊ɕQĂ܂ivO܂EventScript̃oOjB") % charadata.m_name).str());
    }
    m_WanderingCharaList.push_back(hero);
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::IsDepaturableArea(int areaindex) const
{
    const TAreaInstance& area = m_AreaInstanceList[areaindex];
    if(area.m_master != -1){
        if(area.m_master == m_CurrentMaster){
            return false;
        }
        if(FindAlliance(m_CurrentMaster, area.m_master) != -1){
            return false;
        }
    }
    for(int n = 0; n < area.m_neighbor.size(); ++n){
        const TAreaInstance& area_n = m_AreaInstanceList[area.m_neighbor[n]];
        if(area_n.m_master == m_CurrentMaster){
            return true;
        }
    }
    return false;
}
//---------------------------------------------------------------------------
/*int TOpenFarenGame::GetCurrentMaster() const
{
    return m_CurrentMaster;
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::GetCharaExistsArea(int masterindex, int charatype, int& areaindex, int& charaindex) const
{
    return GetCharaExistsArea(masterindex, charatype, areaindex, charaindex);
}*/
//---------------------------------------------------------------------------
int TOpenFarenGame::GetRunnableArea(int areaindex, std::vector<int>& areaindexlist, std::vector<int>& runable) const
{
    const TAreaInstance& area = m_AreaInstanceList[areaindex];
    int count = 0;
    for(int n = 0; n < area.m_neighbor.size(); ++n){
        const TAreaInstance& area_n = m_AreaInstanceList[area.m_neighbor[n]];
        if(area_n.m_master == area.m_master){
            areaindexlist.push_back(area.m_neighbor[n]);
            int space = AREACHARA_MAX - area_n.m_charalist.size();
            runable.push_back(space);
            count += space;
        }
    }
    return count;
}
//---------------------------------------------------------------------------
void TOpenFarenGame::ChangeMasterType(int masterindex, int mastertype, int usecharatype)
{
    //OFDBGLOG((boost::format("}X^[ύX %s -> %s") % (m_Data->GetMaster(m_MasterInstanceList[masterindex].m_type).m_name) % (m_Data->GetMaster(mastertype).m_name)).str());
    m_MasterInstanceList[masterindex].m_type = mastertype;
    const TMaster& masterdata = m_Data->GetMaster(mastertype);
    int areaindex, charaindex;
    if(FindCharaExistsArea(masterindex, usecharatype, areaindex, charaindex)){
        // lރL΂̃LV}X^[LɕύXB
        const TCharacterInstance& chara = m_AreaInstanceList[areaindex].m_charalist[charaindex];
        m_AreaInstanceList[areaindex].m_charalist[charaindex] = CreateCharaInstance(masterdata.m_facecharaindex, chara.m_rank);
        m_AreaInstanceList[areaindex].m_charalist[charaindex].m_experience = chara.m_experience;
    }
    else{
        // lރLȂΐV}X^[L󂫂̂GAɔzuB
        int insarea = FindCharaInsertableArea(masterindex, true);
        if(insarea == -1){
            OFERRLOG((boost::format("V}X^[zułGA܂B")).str());
            return;
        }
        TCharacterInstance chara = CreateCharaInstance(masterdata.m_facecharaindex, 0);
        if(m_AreaInstanceList[insarea].m_charalist.size() < AREACHARA_MAX){
            m_AreaInstanceList[insarea].m_charalist.push_back(chara);
        }
        else{
            m_AreaInstanceList[insarea].m_charalist[AREACHARA_MAX-1] = chara;
        }
    }
}
//---------------------------------------------------------------------------
void TOpenFarenGame::JudgeLostMaster(std::vector<int>& lostmaster) const
{
    unsigned sz = m_MasterInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        if(m_MasterInstanceList[u].m_exists){
            if(GetMasterBaseArea(u) == -1){
                lostmaster.push_back(u);
            }
        }
    }
}
//---------------------------------------------------------------------------
// vC[}X^[SłB
bool TOpenFarenGame::JudgePlayerGameOver() const
{
    unsigned sz = m_MasterInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        if(m_MasterInstanceList[u].m_exists && m_MasterInstanceList[u].m_player){
            return false;
        }
    }
    return true;
}
//---------------------------------------------------------------------------
// Q[IB
bool TOpenFarenGame::JudgeOvercome() const
{
    unsigned sz = m_MasterInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        if(m_MasterInstanceList[u].m_exists && ExistsConstantEnemyMaster(u)){
            return false;
        }
    }
    return true;
}
//---------------------------------------------------------------------------
void TOpenFarenGame::PlaceAllAreaCharaRandom()
{
    int sz = m_AreaInstanceList.size();
    for(int i=0; i<sz; ++i){
        PlaceAreaCharaRandom(i);
    }
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::PopFoundWanderingChara(int& areaindex, int& charatype)
{
    if(m_FoundWanderingCharaList.empty()){
        return false;
    }
    areaindex = m_FoundWanderingCharaList.back().first;
    charatype = m_FoundWanderingCharaList.back().second;
    m_FoundWanderingCharaList.pop_back();
    return true;
}
//---------------------------------------------------------------------------
// lނopł邩BƗŔ肵opʂԂB
bool TOpenFarenGame::JudgeEmployWanderingChara(int masterindex, int charatype) const
{
    int facechara = m_Data->GetMaster(m_MasterInstanceList[masterindex].m_type).m_facecharaindex;
    const TBattleCharacter& charadata1 = m_Data->GetCharacter(facechara);
    const TBattleCharacter& charadata2 = m_Data->GetCharacter(charatype);
    if(charadata1.m_race == charadata2.m_race){
        return true;
    }
    const TRace& race1 = m_Data->GetRace(charadata1.m_race);
    if(race1.m_distance[charadata2.m_race] >= 3){
        // ̋3ȏ͐ΌٗpsB
        return false;
    }
    return !mtrand_n(race1.m_distance[charadata2.m_race] + 1);
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::EmployWanderingChara(int areaindex, int charatype)
{
    if(m_WanderingCharaList.empty()){
        OFERRLOG("QlނȂ̂ɐlޓop悤ƂH");
        return false;
    }
    TCharacterInstanceList::iterator itr;
    for(itr = m_WanderingCharaList.begin(); itr != m_WanderingCharaList.end(); ++itr){
        if(itr->m_type == charatype){
            break;
        }
    }
    if(itr == m_WanderingCharaList.end()){
        OFERRLOG("QĂȂlނop悤ƂH");
        return false;
    }
    int masindex = m_AreaInstanceList[areaindex].m_master;
    if(masindex == -1){
        OFERRLOG("̒nŐlނop悤ƂH");
        return false;
    }
    const TBattleCharacter& charadata = m_Data->GetCharacter(charatype);
    if(m_MasterInstanceList[masindex].m_money < charadata.m_employcost){
        OFERRLOG("Ȃ̂ɐlނop悤ƂH");
        return false;
    }
    m_MasterInstanceList[masindex].m_money -= charadata.m_employcost;
    m_AreaInstanceList[areaindex].m_charalist.push_back(*itr);
    SortAreaCharacter(areaindex);
    m_WanderingCharaList.erase(itr);
    return true;
}
//---------------------------------------------------------------------------
TCharacterInstance TOpenFarenGame::GetWanderingChara(int charatype) const
{
    TCharacterInstanceList::const_iterator itr;
    for(itr = m_WanderingCharaList.begin(); itr != m_WanderingCharaList.end(); ++itr){
        if(itr->m_type == charatype){
            break;
        }
    }
    if(itr == m_WanderingCharaList.end()){
        OFERRLOG("QĂȂlނ悤ƂH");
        return CreateCharaInstance(0, 0);   // ꂵȂŃS~ԂĂB
    }
    return *itr;
}
//---------------------------------------------------------------------------
void TOpenFarenGame::UpdateAreaAndBMapAfterBattle(int areaindex)
{
    TAreaInstance& area = m_AreaInstanceList[areaindex];
    area.m_city = area.m_bmap->CountObj(MAPOBJ_CITY);
    area.m_wall = area.m_bmap->CountObj(MAPOBJ_WALL);
    area.m_road = area.m_bmap->CountObj(MAPOBJ_ROAD);
    area.m_bmap->RestoreAfterBattle();
}
//---------------------------------------------------------------------------
void TOpenFarenGame::PlusLey(int master, int ley)
{
    m_MasterInstanceList[master].m_money += ley;
}
//---------------------------------------------------------------------------
int TOpenFarenGame::GetAreaCity(int areaindex) const
{
    return m_AreaInstanceList[areaindex].m_city;
}
//---------------------------------------------------------------------------
void TOpenFarenGame::SetAreaCity(int areaindex, int city)
{
    TAreaInstance& area = m_AreaInstanceList[areaindex];
    if(city > area.m_citymax){
        city = area.m_citymax;
    }
    else if(city < 0){
        city = 0;
    }
    area.m_city = city;
    area.m_bmap->SetCity(area.m_city);
}
//---------------------------------------------------------------------------
int TOpenFarenGame::GetAreaWall(int areaindex) const
{
    return m_AreaInstanceList[areaindex].m_wall;
}
//---------------------------------------------------------------------------
void TOpenFarenGame::SetAreaWall(int areaindex, int wall)
{
    TAreaInstance& area = m_AreaInstanceList[areaindex];
    if(wall > area.m_wallmax){
        wall = area.m_wallmax;
    }
    else if(wall < 0){
        wall = 0;
    }
    area.m_wall = wall;
    area.m_bmap->SetWall(area.m_wall);
}
//---------------------------------------------------------------------------
int TOpenFarenGame::GetAreaRoad(int areaindex) const
{
    return m_AreaInstanceList[areaindex].m_road;
}
//---------------------------------------------------------------------------
void TOpenFarenGame::SetAreaRoad(int areaindex, int road)
{
    TAreaInstance& area = m_AreaInstanceList[areaindex];
    if(road > area.m_roadmax){
        road = area.m_roadmax;
    }
    else if(road < 0){
        road = 0;
    }
    area.m_road = road;
    area.m_bmap->SetRoad(area.m_road);
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::GetChampion(int num, std::string& champid) const
{
    TCharacterInstanceList lst;
    unsigned sz = m_AreaInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        const TAreaInstance& areains = m_AreaInstanceList[u];
        int chrcount = areains.m_charalist.size();
        for(int i=0; i<chrcount; ++i){
            int charatype = areains.m_charalist[i].m_type;
            if(m_Data->GetCharacter(charatype).m_hero){
                lst.push_back(areains.m_charalist[i]);
            }
        }
    }
    std::sort(lst.begin(), lst.end(), TCharacterWinGreater());
    if(lst.size() > num && lst[num].m_wincount > 0){
        champid = m_Data->GetCharacter(lst[num].m_type).m_id;
        return true;
    }
    else{
        return false;
    }
}
//---------------------------------------------------------------------------
void TOpenFarenGame::SaveAreaInstance(TiXmlElement& e) const
{
    unsigned sz = m_AreaInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        const TAreaInstance& area = m_AreaInstanceList[u];
        TiXmlElement e_area("Area");
        SetTextAttribute(e_area, "id", area.m_id.c_str());

        if(area.m_master != -1){
            SetTextAttribute(e_area, "master", m_Data->GetMaster(m_MasterInstanceList[area.m_master].m_type).m_id.c_str());
        }
        else if(area.m_ignore){
            SetTextAttribute(e_area, "ignore", "true");
        }

        TiXmlElement e_name("Name");
        InsertTextChild(e_name, area.m_name.c_str());
        e_area.InsertEndChild(e_name);

        TiXmlElement e_areachara("AreaCharacter");
        TiXmlElement e_inhabit("Inhabit");
        for(std::vector<int>::const_iterator itr = area.m_inhabit.begin(); itr != area.m_inhabit.end(); ++itr){
            TiXmlElement e_chara("Character");
            SetTextAttribute(e_chara, "id", m_Data->GetCharacter(*itr).m_id.c_str());
            e_inhabit.InsertEndChild(e_chara);
        }
        e_areachara.InsertEndChild(e_inhabit);

        TiXmlElement e_employ("Employable");
        for(std::vector<int>::const_iterator itr = area.m_employable.begin(); itr != area.m_employable.end(); ++itr){
            TiXmlElement e_chara("Character");
            SetTextAttribute(e_chara, "id", m_Data->GetCharacter(*itr).m_id.c_str());
            e_employ.InsertEndChild(e_chara);
        }
        e_areachara.InsertEndChild(e_employ);
        e_area.InsertEndChild(e_areachara);

        TiXmlElement e_baseincome("BaseIncome");
        InsertTextChild(e_baseincome, (boost::format("%d") % area.m_baseincome).str().c_str());
        e_area.InsertEndChild(e_baseincome);

        TiXmlElement e_city("City");
        e_city.SetAttribute("max", area.m_citymax);
        e_city.SetAttribute("value", area.m_city);
        e_area.InsertEndChild(e_city);

        TiXmlElement e_wall("Wall");
        e_wall.SetAttribute("max", area.m_wallmax);
        e_wall.SetAttribute("ini", area.m_wallini);
        e_wall.SetAttribute("value", area.m_wall);
        e_area.InsertEndChild(e_wall);

        TiXmlElement e_road("Road");
        e_road.SetAttribute("max", area.m_roadmax);
        e_road.SetAttribute("value", area.m_road);
        e_area.InsertEndChild(e_road);

        TiXmlElement e_inhcost("InhabitCost");
        e_inhcost.SetAttribute("value", area.m_inhabitcost);
        e_area.InsertEndChild(e_inhcost);

        if(area.m_master != -1){
            TiXmlElement e_ope("Operation");
            e_ope.SetAttribute("value", XMLVAL_AREAOPE[area.m_operation]);
            e_area.InsertEndChild(e_ope);

            TiXmlElement e_opeef("OperationEffort");
            e_opeef.SetAttribute("value", area.m_operationeffort);
            e_area.InsertEndChild(e_opeef);
        }

        TiXmlElement e_loc("Location");
        e_loc.SetAttribute("x", area.m_location.x);
        e_loc.SetAttribute("y", area.m_location.y);
        e_area.InsertEndChild(e_loc);

        for(unsigned n=0; n<area.m_neighborid.size(); ++n){
            TiXmlElement e_neighbor("NeighborArea");
            e_neighbor.SetAttribute("index", n);
            InsertTextChild(e_neighbor, area.m_neighborid[n].c_str());
            e_area.InsertEndChild(e_neighbor);
        }

        TiXmlElement e_music("Music");
        InsertTextChild(e_music, area.m_musicfile.c_str());
        e_area.InsertEndChild(e_music);

        TiXmlElement e_bmap("BattleMapFile");
        InsertTextChild(e_bmap, area.m_bmapfile.c_str());
        e_area.InsertEndChild(e_bmap);

        e.InsertEndChild(e_area);
    }
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::LoadAreaInstance(const TiXmlElement& top)
{
    m_AreaInstanceList.clear();

    for(const TiXmlElement *child = top.FirstChildElement("Area"); child != NULL; child = child->NextSiblingElement("Area")){
        TAreaInstance area;
        area.m_id = GetElementAttribute(*child, "id");
        std::string masterid = GetElementAttribute(*child, "master");
        if(!masterid.empty()){
            area.m_master = GetMasterInstanceIndex(masterid);
        }
        else{
            std::string ignore = GetElementAttribute(*child, "ignore");
            area.m_ignore = !ignore.compare("true");
        }

        g_Logger.DebugLog((boost::format("Area %s loading...") % area.m_id).str());
        const TiXmlElement *e = child->FirstChildElement("Name");
        if(e == NULL){
            g_Logger.ErrorLog("TOpenFarenGame::LoadAreaInstance() Names");
            return false;
        }
        area.m_name = GetElementText(*e);

        e = child->FirstChildElement("AreaCharacter");
        if(e != NULL){
            const TiXmlElement *e_inhabit = e->FirstChildElement("Inhabit");
            if(e_inhabit != NULL){
                for(const TiXmlElement *e_chara = e_inhabit->FirstChildElement("Character"); e_chara != NULL; e_chara = e_chara->NextSiblingElement("Character")){
                    std::string id = GetElementAttribute(*e_chara, "id");
                    int i = m_Data->GetCharacterIndex(id);
                    if(i != -1){
                        area.m_inhabit.push_back(i);
                    }
                    else{
                        g_Logger.ErrorLog("TOpenFarenGame::LoadAreaInstance() InhabitLIĎs id=" + id);
                        return false;
                    }
                }
            }
            const TiXmlElement *e_employ = e->FirstChildElement("Employable");
            if(e_employ != NULL){
                for(const TiXmlElement *e_chara = e_employ->FirstChildElement("Character"); e_chara != NULL; e_chara = e_chara->NextSiblingElement("Character")){
                    std::string id = GetElementAttribute(*e_chara, "id");
                    int i = m_Data->GetCharacterIndex(id);
                    if(i != -1){
                        area.m_employable.push_back(i);
                    }
                    else{
                        g_Logger.ErrorLog("TOpenFarenGame::LoadAreaInstance() EmployableLIĎs id=" + id);
                        return false;
                    }
                }
            }
        }
        e = child->FirstChildElement("BaseIncome");
        if(e != NULL){
            area.m_baseincome = GetElementInt(*e);
        }
        // City,Wall,Roadvfvalue͐VKQ[͖̎ŃQ[Jnɏ㏑̂ł悵ƂB
        e = child->FirstChildElement("City");
        if(e != NULL){
            e->QueryIntAttribute("max", &area.m_citymax);
            e->QueryIntAttribute("value", &area.m_city);
        }
        e = child->FirstChildElement("Wall");
        if(e != NULL){
            e->QueryIntAttribute("max", &area.m_wallmax);
            e->QueryIntAttribute("ini", &area.m_wallini);
            e->QueryIntAttribute("value", &area.m_wall);
        }
        e = child->FirstChildElement("Road");
        if(e != NULL){
            e->QueryIntAttribute("max", &area.m_roadmax);
            e->QueryIntAttribute("value", &area.m_road);
        }
        e = child->FirstChildElement("InhabitCost");
        if(e != NULL){
            e->QueryIntAttribute("value", &area.m_inhabitcost);
        }
        if(area.m_master != -1){
            // ̒nłȂΑҋ@̍sB
            e = child->FirstChildElement("Operation");
            if(e != NULL){
                std::string opestr = GetElementAttribute(*e, "value");
                const char **p = std::find(XMLVAL_AREAOPE, XMLVAL_AREAOPE + AREAOPE_MAX, opestr);
                if(p != XMLVAL_AREAOPE + AREAOPE_MAX){
                    area.m_operation = static_cast<TAreaOperation>(p - XMLVAL_AREAOPE);
                }
            }
            // ʁB
            e = child->FirstChildElement("OperationEffort");
            if(e != NULL){
                e->QueryIntAttribute("value", &area.m_operationeffort);
            }
        }
        e = child->FirstChildElement("Location");
        if(e == NULL){
            g_Logger.ErrorLog("TOpenFarenGame::LoadAreaInstance() Locations");
            return false;
        }
        e->QueryIntAttribute("x", &area.m_location.x);
        e->QueryIntAttribute("y", &area.m_location.y);
        for(const TiXmlElement *e_n = child->FirstChildElement("NeighborArea"); e_n != NULL; e_n = e_n->NextSiblingElement("NeighborArea")){
            area.m_neighborid.push_back(GetElementText(*e_n));
        }
        e = child->FirstChildElement("Music");
        if(e != NULL){
            area.m_musicfile = GetElementText(*e);
        }
        e = child->FirstChildElement("BattleMapFile");
        if(e != NULL){
            area.m_bmapfile = GetElementText(*e);
        }
        m_AreaInstanceList.push_back(area);
    }
    std::sort(m_AreaInstanceList.begin(), m_AreaInstanceList.end(), TIDHolderLess<TAreaInstance>());
    for(unsigned u=0; u<m_AreaInstanceList.size(); ++u){
        g_Logger.DebugLog((boost::format("%u:%s") % u % m_AreaInstanceList[u].m_name.c_str()).str());
    }

    g_Logger.DebugLog("Resoliving neighbor area id...");
    // אڃGÃCfbNXB
    for(TAreaInstanceList::iterator itr = m_AreaInstanceList.begin(); itr != m_AreaInstanceList.end(); itr++){
        itr->m_neighbor.resize(itr->m_neighborid.size());
        for(unsigned u = 0; u < itr->m_neighborid.size(); u++){
            itr->m_neighbor[u] = GetAreaInstanceIndex(itr->m_neighborid[u]);
        }
        // łɐ͒n}Rs[B
        int pmindex = m_Data->GetPowerMapPointIndex(itr->m_id);
        if(pmindex != -1){
            itr->m_powermappoint = m_Data->GetPowerMapPoint(pmindex).m_points;
        }
        else{
            OFWRNLOG((boost::format("GA %s ̐͒n}񂪌܂łB̃GA͐͒n}ɕ\܂B") % (itr->m_id)).str());
        }
    }
    g_Logger.DebugLog("All area load OK.");
    return true;
}
//---------------------------------------------------------------------------
// VKQ[ƃZ[uf[^ŋʂ̃f[^ǂݍށB
bool TOpenFarenGame::LoadParmanentData(const std::string& dir)
{
    // GA}bv̓ǂݍ
    g_Logger.DebugLog("Loading areamap file...");
    for(TAreaInstanceList::iterator itr = m_AreaInstanceList.begin(); itr != m_AreaInstanceList.end(); itr++){
        std::string mapfile = (boost::format("%s/Data/%s") % dir.c_str() % itr->m_bmapfile.c_str()).str();
        if(!boost::filesystem::exists(mapfile)){
            g_Logger.ErrorLog(mapfile + " File not found");
            return false;
        }
        boost::shared_ptr<TBattleMap> bmap(new TBattleMap());
        TiXmlDocument doc(mapfile.c_str());
        if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
            g_Logger.ErrorLog((boost::format("%s : XML Parse Error.") % mapfile).str());
            return false;
        }
        const TiXmlElement *e_map = doc.FirstChildElement(XMLELM_BMAPTOP);
        if(e_map == NULL){
            g_Logger.ErrorLog((boost::format("%s %ss") % mapfile % XMLELM_BMAPTOP).str());
            return false;
        }
        if(!bmap->LoadFromElement(e_map, mapfile.c_str(), m_Data)){
            g_Logger.ErrorLog(mapfile + " GA}bvǍs");
            return false;
        }

        // אڃGÃCfbNX߂Ċi[B
        for(int i = 0; i < bmap->GetNeighborCount(); ++i){
            int ni = GetAreaInstanceIndex(bmap->GetNeighborID(i));
            if(ni == -1){
                g_Logger.ErrorLog(mapfile + " אڃGA " + bmap->GetNeighborID(i) + " ̌Ɏs");
                return false;
            }
            bmap->SetNeighborIndex(i, ni);
        }
        itr->m_bmap = bmap;
    }
    return true;
}
//---------------------------------------------------------------------------
// ̏ http://kobe.cool.ne.jp/nyx9/Faren/data/index.html QlɂĂB
void TOpenFarenGame::PlaceAreaCharaRandom(int areaindex)
{
    TAreaInstance& area = m_AreaInstanceList[areaindex];
    const int inhcost[GAMELEVEL_MAX] = {3, 5, 7, 10, 14};   // x12̏ꍇ͎̂ŕsB
    int empcost;
    int rank = 0;
    //bool playerarea;
    const std::vector<int> *idlist;
    if(area.m_master == -1){
        // nт̏LB
        //charamax = 8 + m_GameLevel*2;
        empcost = (area.m_inhabitcost * inhcost[m_GameLevel - 1] * (35 + mtrand_n(90))) / 100;
        rank = m_GameLevel / 2;
        idlist = &area.m_inhabit;
        //playerarea = false;
    }
    else{
        // ̃GÃ}X^[ٗp\ȃLXgB
        const TMasterInstance& masterins = m_MasterInstanceList[area.m_master];
        idlist = &m_Data->GetCharacter(m_Data->GetMaster(masterins.m_type).m_facecharaindex).m_employable;
        if(IsBaseArea(areaindex)){
            empcost = 500;  // {n͌ٗpp500B
        }
        else{
            empcost = 100;  // }X^[GAɗאڂĂȂꍇ͌ٗpp100B
            for(int i=0; i<area.m_neighbor.size(); ++i){
                if(m_AreaInstanceList[area.m_neighbor[i]].m_master != area.m_master){
                    empcost = 350;  // }X^[GAɗאڂĂꍇ͌ٗpp350B
                    break;
                }
            }
        }
        //playerarea = masterins.m_player;
    }
    // zuIΓRȂB
    if(idlist->empty()){
        return;
    }
    int cnt = AREACHARA_MAX - area.m_charalist.size();
    int idsize = idlist->size();
    for(int i=0; i<cnt; ++i){
        //int ri = mtrand_n(idlist->size());
        //int ctype = (*idlist)[ri];
        //area.m_charalist.push_back(CreateCharaInstance(ctype, rank));
        int maxcost = 0;
        int maxindex = -1;
        for(int idindex = 0; idindex < idsize; ++idindex){
            int ctype = (*idlist)[idindex];
            const TBattleCharacter& charadata = m_Data->GetCharacter(ctype);
            int cost = charadata.m_employcost;
            /*int cost;
            if(playerarea){
                const TBattleCharacter& charadata = m_Data->GetCharacter(ctype);
                cost = charadata.m_employcost;
            }
            else{
                cost = GetComEmployCost(ctype);
            }*/
            if(maxindex != -1 && !mtrand_n(3)){
                continue;
            }
            if(cost <= empcost && maxcost < cost){
                maxcost = cost;
                maxindex = idindex;
            }
        }
        if(maxindex != -1){
            area.m_charalist.push_back(CreateCharaInstance((*idlist)[maxindex], rank));
            empcost -= maxcost;
        }
        else{
            break;
        }
    }
    SortAreaCharacter(areaindex);
}
//---------------------------------------------------------------------------
// ؂ꂽ폜B
void TOpenFarenGame::RemoveExpiredAlliance()
{
    for(TAllianceList::reverse_iterator ri = m_AllianceList.rbegin(); ri != m_AllianceList.rend(); ++ri){
        if(!ri->turn){
            m_AllianceList.erase((ri + 1).base());
        }
    }
}
//---------------------------------------------------------------------------
// ivłȂ}X^[邩ǂB
bool TOpenFarenGame::ExistsConstantEnemyMaster(int master) const
{
    unsigned sz = m_MasterInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        if(master == u){
            continue;
        }
        int index = FindAlliance(master, u);
        if(index != -1 && m_AllianceList[index].turn == -1){
            continue;
        }
        const TMasterInstance& masterins = m_MasterInstanceList[u];
        g_Logger.DebugLog((boost::format("%s %d %d") % FUNCNAME % u % masterins.m_exists).str());
        if(masterins.m_exists){
            return true;
        }
    }
    return false;
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::IsBaseArea(int areaindex) const
{
    const TAreaInstance& area = m_AreaInstanceList[areaindex];
    unsigned sz = area.m_charalist.size();
    for(unsigned u=0; u<sz; ++u){
        const TBattleCharacter& charadata = m_Data->GetCharacter(area.m_charalist[u].m_type);
        if(charadata.m_master){
            return true;
        }
    }
    return false;
}
//---------------------------------------------------------------------------
// ߖTGAɃLǉB߂l͒ǉGÃCfbNXBs-1B
int TOpenFarenGame::MoveCharaToNeighbor(TAreaInstance& area, int masterindex, const TCharacterInstance& chara)
{
    bool ok = false;
    for(int n = 0; n < area.m_neighbor.size(); ++n){
        TAreaInstance& area_n = m_AreaInstanceList[area.m_neighbor[n]];
        if(area_n.m_master == masterindex &&
           area_n.m_charalist.size() < AREACHARA_MAX){
            // אڃGAw肵}X^[ALtłȂΓB
            area_n.m_charalist.push_back(chara);
            return area.m_neighbor[n];
        }
    }
    // GÃL͂ŎsƂB
    if(masterindex == -1){
        return -1;
    }
    // אڃGAꍇ
    const TMasterInstance& master = GetMasterInstance(masterindex);
    const TMaster& masterdata = m_Data->GetMaster(master.m_type);
    if(chara.m_type == masterdata.m_facecharaindex){
        // L}X^[̏ꍇ͖㏑œB
        for(int n = 0; n < area.m_neighbor.size(); ++n){
            TAreaInstance& area_n = m_AreaInstanceList[area.m_neighbor[n]];
            if(area_n.m_master == masterindex){
                area_n.m_charalist[AREACHARA_MAX - 1] = chara;
                return area.m_neighbor[n];
            }
        }
    }
    return false;
}
//---------------------------------------------------------------------------
// ʔBlޑ{͒Ŝ݋L^Ď͕ʂɍsB
void TOpenFarenGame::ApplyAdministration()
{
    unsigned areacount = m_AreaInstanceList.size();
    for(unsigned u=0; u<areacount; ++u){
        TAreaInstance& area = m_AreaInstanceList[u];
        if(area.m_master != m_CurrentMaster){
            continue;
        }
        if(area.m_operation == AREAOPE_TRAINING){
            // PB
            for(unsigned chr=0; chr<area.m_charalist.size(); ++chr){
                TCharacterInstance& chara = area.m_charalist[chr];
                if(!chara.m_actioned){
                    // PɂExp̑ F8 - N + random(12 - Lv * 2)
                    chara.m_experience += 8 - chara.m_rank + mtrand_n(12 - m_GameLevel * 2);
                    CheckRankup(chara);
                }
            }
            SortAreaCharacter(u);
            continue;
        }
        int actcnt = 0;
        for(unsigned chr=0; chr<area.m_charalist.size(); ++chr){
            if(!area.m_charalist[chr].m_actioned){
                ++actcnt;
            }
        }
        if(area.m_operation == AREAOPE_SEARCHHERO){
            // lޑ{
            area.m_operationeffort = actcnt;
        }
        else if(area.m_operation == AREAOPE_BUILDCITY){
            // XJ
            area.m_city += actcnt / 4;
            if(actcnt%4 && mtrand_n(4) < actcnt%4){
                ++area.m_city;
            }
            if(area.m_city > area.m_citymax){
                area.m_city = area.m_citymax;
            }
            area.m_bmap->SetCity(area.m_city);
        }
        else if(area.m_operation == AREAOPE_BUILDWALL){
            // ǌ
            area.m_wall += actcnt;
            if(area.m_wall > area.m_wallmax){
                area.m_wall = area.m_wallmax;
            }
            area.m_bmap->SetWall(area.m_wall);
        }
        else if(area.m_operation == AREAOPE_BUILDROAD){
            // H
            // {͈l0.4̑ɂȂ炵ʓ|Ȃ̂0.5ŁB
            area.m_road += actcnt / 2;
            if(actcnt%2 && mtrand_n(2)){
                ++area.m_road;
            }
            if(area.m_road > area.m_roadmax){
                area.m_road = area.m_roadmax;
            }
            area.m_bmap->SetRoad(area.m_road);
        }
    }
}
//---------------------------------------------------------------------------
bool TOpenFarenGame::CharaAppeared(int charatype) const
{
    // ݃GA
    unsigned sz = m_AreaInstanceList.size();
    for(unsigned u=0; u<sz; ++u){
        unsigned chrs = m_AreaInstanceList[u].m_charalist.size();
        for(unsigned chr = 0; chr < chrs; ++chr){
            if(charatype == m_AreaInstanceList[u].m_charalist[chr].m_type){
                return true;
            }
        }
    }
    // Q
    sz = m_WanderingCharaList.size();
    for(unsigned u=0; u<sz; ++u){
        if(charatype == m_WanderingCharaList[u].m_type){
            return true;
        }
    }
    // S
    sz = m_DeadCharaList.size();
    for(unsigned u=0; u<sz; ++u){
        if(charatype == m_DeadCharaList[u].m_type){
            return true;
        }
    }
    return false;
}
//---------------------------------------------------------------------------
// ^[isɂČQlނǉB
void TOpenFarenGame::UpdateWanderingHero()
{
    int chrtypes = m_Data->GetCharacterCount();
    for(int i=0; i<chrtypes; ++i){
        const TBattleCharacter& charadata = m_Data->GetCharacter(i);
        // lށEQEo^[B
        if(!charadata.m_hero || charadata.m_nowander ||
           m_Turn < charadata.m_turnmin ||
           (charadata.m_turnmax != -1 && charadata.m_turnmax < m_Turn)){
            continue;
        }
        // ɓoς݂ȂƂ΂B
        if(CharaAppeared(i)){
            continue;
        }
        m_WanderingCharaList.push_back(CreateCharaInstance(i, 0));
    }
}
//---------------------------------------------------------------------------
// ԂĂ}X^[̐lޑ{̐ʂ𔻒肷B
void TOpenFarenGame::CheckSearchHero()
{
    m_FoundWanderingCharaList.clear();
    if(m_WanderingCharaList.empty()){
        // NQĂȂ牽ȂB
        return;
    }
    // QĂlސ / Slނ̐ ̊mŐlނ炵̂ōČB
    int allhero = m_Data->GetHeroCount();
    if(!allhero){
        // lނSȂ0ZɂȂ̂ňꉞ`FbNB
        return;
    }
    int wander = m_WanderingCharaList.size();
    std::set<int> found;
    unsigned areacount = m_AreaInstanceList.size();
    for(unsigned u=0; u<areacount; ++u){
        TAreaInstance& area = m_AreaInstanceList[u];
        // ݂̃}X^[̗̓yŐlޑ{ő{SLꍇΏہB
        if(area.m_master != m_CurrentMaster ||
           area.m_operation != AREAOPE_SEARCHHERO ||
           !area.m_operationeffort){
            continue;
        }
        for(int i=0; i<area.m_operationeffort; ++i){
            if(area.m_charalist.size() >= AREACHARA_MAX){
                break;
            }
            // 1/12̊mQĂlސ / Slނ̐ ̊mŔB
            if(mtrand_n(12) || mtrand_n(allhero) > wander){
                continue;
            }
            //int ctype = m_WanderingCharaList[mtrand_n(wander)];
            // ꂱŗƕɂ̂ŃK[hĂ݂B
            int index = mtrand_n(wander);
            if(index >= wander){
                OFERRLOG("mtrand_n̎gԈႦĂBvOB");
                continue;
            }
            int ctype = m_WanderingCharaList[index].m_type;
            // ς݂͂Ƃ΂B
            if(found.find(ctype) != found.end()){
                continue;
            }
            m_FoundWanderingCharaList.push_back(std::pair<int, int>(u, ctype));
            found.insert(ctype);
        }
    }
}
//---------------------------------------------------------------------------
int TOpenFarenGame::FindCharaInstance(const TCharacterInstanceList& clist, int ctype) const
{
    unsigned sz = clist.size();
    for(unsigned u=0; u<sz; ++u){
        if(clist[u].m_type == ctype){
            return u;
        }
    }
    return -1;
}
//---------------------------------------------------------------------------
