//---------------------------------------------------------------------------
#include <algorithm>
#include <boost/foreach.hpp>
#include <boost/scoped_array.hpp>
#include "ofimage.h"
#include "gui.h"
//#include <windows.h>
#include "boostformat.h"
#include "logger.h"
#include <cstring>

#define FLAG_WIDTH 32
//---------------------------------------------------------------------------
TOpenFarenImage::TOpenFarenImage() :
    m_MapChipImage(NULL),
    m_SDLPowerMap(NULL),
    m_PowerMapImage(NULL)
{
}
//---------------------------------------------------------------------------
TOpenFarenImage::~TOpenFarenImage()
{
    BOOST_FOREACH(gcn::Image* img, m_MyImageList){
        delete img;
    }
    if(m_SDLPowerMap != NULL){
        SDL_FreeSurface(m_SDLPowerMap);
    }
    if(m_PowerMapImage != NULL){
        delete m_PowerMapImage;
    }
}
//---------------------------------------------------------------------------
int TOpenFarenImage::LoadFlag(const std::string& dir, const std::string& filename)
{
    gcn::Image *img = LoadMyImage(dir + "/" + filename);
    if(img == NULL){
        return -1;
    }
    int index = m_FlagImage.size();
    m_FlagImage.push_back(img);
    return index;
}
//---------------------------------------------------------------------------
int TOpenFarenImage::LoadFace(const std::string& dir, const std::string& filename)
{
    gcn::Image *img = LoadMyImage(dir + "/" + filename);
    if(img == NULL){
        return -1;
    }
    int index = m_FaceImage.size();
    m_FaceImage.push_back(img);
    return index;
}
//---------------------------------------------------------------------------
int TOpenFarenImage::LoadCharaChip(const std::string& dir, const std::string& filename)
{
    gcn::Image *img = LoadMyImage(dir + "/" + filename);
    if(img == NULL){
        return -1;
    }
    int index = m_CharaChipImage.size();
    m_CharaChipImage.push_back(img);
    return index;
}
//---------------------------------------------------------------------------
int TOpenFarenImage::LoadEffect(const std::string& filename)
{
    gcn::Image *img = LoadMyImage(filename);
    if(img == NULL){
        return -1;
    }
    int index = m_EffectImage.size();
    m_EffectImage.push_back(img);
//g_Logger.DebugLog((boost::format("%d %s %d %d") % index% filename % img->getWidth() % img->getHeight()).str());
    return index;
}
//---------------------------------------------------------------------------
bool TOpenFarenImage::LoadMapChip(const std::string& filename)
{
    m_MapChipImage = LoadMyImage(filename);
    if(m_MapChipImage == NULL){
        return false;
    }
    m_MapChipXCount = m_MapChipImage->getWidth() / MAPCHIP_WIDTH;
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenImage::LoadMapObj(const std::string& filename)
{
    m_MapObjImage = LoadMyImage(filename);
    if(m_MapObjImage == NULL){
        return false;
    }
    m_MapObjXCount = m_MapObjImage->getWidth() / MAPCHIP_WIDTH;
    return true;
}
//---------------------------------------------------------------------------
int TOpenFarenImage::LoadEvent(const std::string& filename)
{
    // ɓǂݍݍς݂Ȃ炻̃CfbNXԂB
    std::map<std::string, int>::const_iterator itr = m_EventIndex.find(filename);
    if(itr != m_EventIndex.end()){
        return itr->second;
    }

    gcn::Image *img = LoadMyImage(filename);
    if(img == NULL){
        return -1;
    }
    int index = m_EventImage.size();
    m_EventIndex[filename] = index;
    m_EventImage.push_back(img);
    return index;
}
//---------------------------------------------------------------------------
/*// 摜[hĕԂAɋLBL摜TOpenFarenImage()̔jɔjB̊֐̍폜H
gcn::Image* TOpenFarenImage::LoadImageAndMemory(const std::string& filename)
{
    return LoadMyImage(filename);
}*/
//---------------------------------------------------------------------------
int TOpenFarenImage::GenerateAreaName(const std::string& name)
{
    TGcnCharDrawer* drawer = g_GuiContainerList.GetDrawer(16);
    gcn::Image *img = drawer->GenerateRichFontImage(name.c_str(), 2, 1, 0xffffffff, 1, 1, 0xff000000);
    if(img == NULL){
        return -1;
    }
    m_MyImageList.push_back(img);
    int index = m_AreaNameImage.size();
    m_AreaNameImage.push_back(img);
    return index;
}
//---------------------------------------------------------------------------
void TOpenFarenImage::ClearAreaName()
{
    BOOST_FOREACH(gcn::Image* img, m_AreaNameImage){
        RemoveMyImage(img);
    }
    m_AreaNameImage.clear();
}
//---------------------------------------------------------------------------
bool TOpenFarenImage::LoadFieldMapImage(const std::string& filename)
{
    if(!m_FieldMapImage.empty()){
        for(unsigned u=0; u<m_FieldMapImage.size(); ++u){
            RemoveMyImage(m_FieldMapImage[u]);
        }
        m_FieldMapImage.clear();
    }
    SDL_Surface *sdlsurface = IMG_Load(filename.c_str());
    if(sdlsurface == NULL){
        return false;
    }

    // őeNX`TCY擾B
    int max_texture_size;
    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &max_texture_size);
    // 悭mȂ̂܂܍őTCYł͂Ȃۂ͔̏炢炵B
    max_texture_size /= 2;

    // OpenGLT[tFCX1̃sNZzB
    int glsw = 16;
    while(glsw < sdlsurface->w){
        if(glsw == max_texture_size){
            break;
        }
        glsw *= 2;
    }
    int glsh = 16;
    while(glsh < sdlsurface->h){
        if(glsh == max_texture_size){
            break;
        }
        glsh *= 2;
    }

    int xcount = (sdlsurface->w + glsw - 1) / glsw;
    int ycount = (sdlsurface->h + glsh - 1) / glsh;
    if(!xcount){
        xcount = 1;
    }
    if(!ycount){
        ycount = 1;
    }
    m_FieldMapImage.reserve(xcount * ycount);

    boost::scoped_array<unsigned> pixels(new unsigned[glsw * glsh]);
    for(int sy=0; sy<ycount; ++sy){
        int srcy = glsh * sy;
        int blth = sdlsurface->h - srcy;
        if(blth > glsh){
            blth = glsh;
        }
        for(int sx=0; sx<xcount; ++sx){
            int srcx = glsw * sx;
            int bltw = sdlsurface->w - srcx;
            if(bltw > glsw){
                bltw = glsw;
            }
            if(bltw < glsw || blth < glsh){
                // [ɗ]肪摜̏ꍇ0NAiSNA͖ʂ邪jB
                memset(pixels.get(), 0, glsw * glsh * sizeof(unsigned));
            }
            BltSDLToOpenGL(sdlsurface, pixels.get(), srcx, srcy, bltw, blth, glsw);
            gcn::OpenGLImage *newimg = NULL;
            try{
                newimg = new gcn::OpenGLImage(pixels.get(), glsw, glsh);
            }catch(gcn::Exception& e){
            }
            if(newimg == NULL){
                OFERRLOG("failed to create gcn::OpenGLImage");
                break;
            }
            m_FieldMapImage.push_back(newimg);
        }
    }
    SDL_FreeSurface(sdlsurface);

    if(m_FieldMapImage.size() < xcount * ycount){
        // s琶Ė߂B
        for(unsigned u=0; u<m_FieldMapImage.size(); ++u){
            delete m_FieldMapImage[u];
        }
        m_FieldMapImage.clear();
        return false;
    }
    // 摜Xgɓo^B
    for(unsigned u=0; u<m_FieldMapImage.size(); ++u){
        m_MyImageList.push_back(m_FieldMapImage[u]);
    }
    m_FieldMapImageXCount = xcount;
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenImage::LoadPowerMap(const std::string& filename)
{
    if(m_SDLPowerMap != NULL){
        SDL_FreeSurface(m_SDLPowerMap);
    }
    m_SDLPowerMap = IMG_Load(filename.c_str());
    if(m_SDLPowerMap == NULL){
        OFERRLOG((boost::format("failed to load %s") % filename).str());
        return false;
    }
    int pixelbytes = m_SDLPowerMap->format->BytesPerPixel;
    if(pixelbytes != 4 && pixelbytes != 3 && pixelbytes != 1){
        //OFERRLOG((boost::format("%s %dbpp̉摜łA32bpp24bppłȂƈ܂B") % filename % (pixelbytes*8)).str());
        OFERRLOG((boost::format("unknown pixel format (BytesPerPixel = %d)") % pixelbytes).str());
        return false;
    }
    return true;
}
//---------------------------------------------------------------------------
void TOpenFarenImage::InitPowerMap()
{
    if(m_PowerMapImage != NULL){
        delete m_PowerMapImage;
        m_PowerMapImage = NULL;
    }

    SDL_Surface* surface = m_SDLPowerMap;

    int pixelbytes = surface->format->BytesPerPixel;
    int w = surface->w;
    int h = surface->h;
    boost::shared_array<unsigned> pixels(new unsigned[w*h]);
    std::memset(pixels.get(), 0, sizeof(unsigned)*w*h);
    if(pixelbytes == 4 || pixelbytes == 3){
        // ꂼ̐F̈ʒuB
        int ri = surface->format->Rshift >> 3;
        int gi = surface->format->Gshift >> 3;
        int bi = surface->format->Bshift >> 3;
        int i = 0;
        for(int y=0; y<h; ++y){
            unsigned char *psrc = (unsigned char*)surface->pixels + surface->pitch * y;
            for(int x=0; x<w; ++x){
                // ƌ̂@邾낤ǁA܂ccB蔲B
                unsigned r = psrc[ri];
                unsigned g = psrc[gi];
                unsigned b = psrc[bi];
                pixels[i++] = 0xff000000 | (b<<16) | (g<<8) | r;
                psrc += pixelbytes;
            }
        }
    }
    else if(pixelbytes == 1){
        const SDL_Palette *palette = surface->format->palette;
        int i = 0;
        for(int y=0; y<h; ++y){
            unsigned char *psrc = (unsigned char*)surface->pixels + surface->pitch * y;
            for(int x=0; x<w; ++x){
                if(*psrc < palette->ncolors){
                    const SDL_Color& c = palette->colors[*psrc];
                    pixels[i] = 0xff000000 | (c.b<<16) | (c.g<<8) | c.r;
                }
                ++i;
                ++psrc;
            }
        }
    }
    else{
        OFERRLOG((boost::format("unknown pixel format (BytesPerPixel = %d)") % pixelbytes).str());
        return;
    }
    // 1sNZ狫EFočɂB
    m_PowerMapEdgeColor = pixels[0];
    pixels[0] = 0xff000000;
    m_PowerMapPixels = pixels;
}
//---------------------------------------------------------------------------
void TOpenFarenImage::SetPowerMapPoints(unsigned color, const std::vector<point_t>& plist)
{
    for(std::vector<point_t>::const_iterator itr = plist.begin(); itr != plist.end(); ++itr){
        if(itr->x >= 0 && itr->x < m_SDLPowerMap->w && itr->y >= 0 && itr->y < m_SDLPowerMap->h){
            TracePaintData(m_PowerMapPixels.get(), m_SDLPowerMap->w, m_SDLPowerMap->h, itr->x, itr->y, color, m_PowerMapEdgeColor);
        }
    }
}
//---------------------------------------------------------------------------
gcn::Image* TOpenFarenImage::GetPowerMap()
{
    try{
        gcn::OpenGLImage *newimg = new gcn::OpenGLImage(m_PowerMapPixels.get(), m_SDLPowerMap->w, m_SDLPowerMap->h);
        m_PowerMapImage = newimg;
    }catch(gcn::Exception& e){
        OFERRLOG((boost::format("failed to create gcn::OpenGLImage exception=%s") % e.getMessage()).str());
    }
    return m_PowerMapImage;
}
//---------------------------------------------------------------------------
bool TOpenFarenImage::LoadButtonImage()
{
    static const char *filenames[BUTTONIMAGE_MAX] = {
        "command_wait.png",
        "command_move.png",
        "command_attack.png",
        "command_magic.png",
        "command_special.png",
        "command_charge.png",
        "command_redo.png",
        "command_redo2.png",
        "command_runaway.png"};
    for(int i = BUTTONIMAGE_WAIT; i < BUTTONIMAGE_MAX; ++i){
        std::string imgpath = m_BaseDir + "/System/" + filenames[i];
        //m_ButtonImage[i] = LoadMyImage(m_BaseDir + "/System/" + filenames[i]);
        m_ButtonImage[i] = LoadMyImage(imgpath);
        if(m_ButtonImage[i] == NULL){
            std::string imgpath = m_SystemImageDir + "/" + filenames[i];
            m_ButtonImage[i] = LoadMyImage(imgpath);
            if(m_ButtonImage[i] == NULL){
                return false;
            }
        }
    }
    return true;
}
//---------------------------------------------------------------------------
gcn::Image* TOpenFarenImage::GetButtonImage(TButtonImageIndex i)
{
    return m_ButtonImage[i];
}
//---------------------------------------------------------------------------
bool TOpenFarenImage::LoadChipImage()
{
    static const char *filenames[CHIPIMAGE_MAX] = {
        "chip_defense.png",
        "chip_departure.png"};
    for(int i = CHIPIMAGE_DEFENSE; i < CHIPIMAGE_MAX; ++i){
        m_ChipImage[i] = LoadMyImage(m_BaseDir + "/System/" + filenames[i]);
        if(m_ChipImage[i] == NULL){
            return false;
        }
    }
    return true;
}
//---------------------------------------------------------------------------
// tick͔Cӂ0ȏ̐BĂяoŒIɃCNgB
void TOpenFarenImage::DrawFlag(gcn::Graphics* g, int x, int y, int index, int tick) const
{
    int w = m_FlagImage[index]->getWidth();
    int srcx = (tick % (w / FLAG_WIDTH))*FLAG_WIDTH;
    g->drawImage(m_FlagImage[index], srcx, 0, x, y, FLAG_WIDTH, m_FlagImage[index]->getHeight());
}
//---------------------------------------------------------------------------
void TOpenFarenImage::DrawFace(gcn::Graphics* g, int x, int y, int index) const
{
    g->drawImage(m_FaceImage[index], x, y);
}
//---------------------------------------------------------------------------
void TOpenFarenImage::DrawFaceStretch(gcn::Graphics* g, int x, int y, int index, int xzoom, int yzoom) const
{
    DrawImageStretchSimple(g, static_cast<gcn::OpenGLImage*>(m_FaceImage[index]), x, y, xzoom, yzoom);
}
//---------------------------------------------------------------------------
void TOpenFarenImage::DrawCharaChip(gcn::Graphics* g, int x, int y, int index) const
{
    g->drawImage(m_CharaChipImage[index], x, y);
}
//---------------------------------------------------------------------------
void TOpenFarenImage::DrawCharaChipStretch(gcn::Graphics* g, int x, int y, int index, int xzoom, int yzoom) const
{
    DrawImageStretchSimple(g, static_cast<gcn::OpenGLImage*>(m_CharaChipImage[index]), x, y, xzoom, yzoom);
}
//---------------------------------------------------------------------------
void TOpenFarenImage::DrawEffect(gcn::Graphics* g, int x, int y, int index, int width, int height, int anim, int direction) const
{
    if(index >= m_EffectImage.size()){
        g_Logger.ErrorLog("effect image out of range");
        return;
    }
    gcn::Image* img = m_EffectImage[index];
    int chipcount = img->getWidth() / width;
    if(!chipcount){
        g_Logger.ErrorLog("effect image invalid size for animation");
        return;
    }
    int srcx = (anim % chipcount) * width;
    int srcy = direction * height;
//g_Logger.DebugLog((boost::format("%d %d -> %d %d , %d %d") % srcx% srcy% x% y% width% height).str());
    g->drawImage(img, srcx, srcy, x, y, width, height);
}
//---------------------------------------------------------------------------
void TOpenFarenImage::DrawMapChip(gcn::Graphics* g, int x, int y, int index) const
{
    int srcx = (index%m_MapChipXCount) * MAPCHIP_WIDTH;
    int srcy = (index/m_MapChipXCount) * MAPCHIP_HEIGHT;
    if(m_MapChipImage == NULL){
        g_Logger.WarningLog("map chip image is null");
        return;
    }
    if(srcx + MAPCHIP_WIDTH > m_MapChipImage->getWidth() ||
       srcy + MAPCHIP_HEIGHT > m_MapChipImage->getHeight()){
        g_Logger.WarningLog((boost::format("map chip index out of range %d") % index).str());
        return;
    }
    g->drawImage(m_MapChipImage, srcx, srcy, x, y, MAPCHIP_WIDTH, MAPCHIP_HEIGHT);
}
//---------------------------------------------------------------------------
void TOpenFarenImage::DrawMapObj(gcn::Graphics* g, int x, int y, int index) const
{
    int srcx = (index%m_MapObjXCount) * MAPCHIP_WIDTH;
    int srcy = (index/m_MapObjXCount) * MAPCHIP_HEIGHT;
    if(m_MapObjImage == NULL){
        g_Logger.WarningLog("map obj image is null");
        return;
    }
    if(srcx + MAPCHIP_WIDTH > m_MapObjImage->getWidth() ||
       srcy + MAPCHIP_HEIGHT > m_MapObjImage->getHeight()){
        g_Logger.WarningLog((boost::format("map obj index out of range %d") % index).str());
        return;
    }
    g->drawImage(m_MapObjImage, srcx, srcy, x, y, MAPCHIP_WIDTH, MAPCHIP_HEIGHT);
}
//---------------------------------------------------------------------------
void TOpenFarenImage::DrawChip(gcn::Graphics* g, int x, int y, TChipImageIndex index) const
{
    g->drawImage(m_ChipImage[index], x, y);
}
//---------------------------------------------------------------------------
void TOpenFarenImage::DrawEvent(gcn::Graphics* g, int x, int y, int index) const
{
    g->drawImage(m_EventImage[index], x, y);
}
//---------------------------------------------------------------------------
void TOpenFarenImage::DrawEventStretch(gcn::Graphics* g, int x, int y, int index, int xzoom, int yzoom) const
{
    DrawImageStretchSimple(g, static_cast<gcn::OpenGLImage*>(m_EventImage[index]), x, y, xzoom, yzoom);
}
//---------------------------------------------------------------------------
void TOpenFarenImage::DrawAreaName(gcn::Graphics* g, int x, int y, int index) const
{
    g->drawImage(m_AreaNameImage[index], x, y);
}
//---------------------------------------------------------------------------
void TOpenFarenImage::DrawCharaFaceOrChip(gcn::Graphics* g, int x, int y, int faceindex, int chipindex) const
{
    if(faceindex != -1){
        g->drawImage(m_FaceImage[faceindex], x, y);
    }
    else{
        //int plusx = (FACE_WIDTH - MAPCHIP_WIDTH) / 2;
        //int plusy = (FACE_HEIGHT - MAPCHIP_HEIGHT) / 2;
        //g->drawImage(m_CharaChipImage[chipindex], x + plusx, y + plusy);
        DrawImageStretch(g, static_cast<gcn::OpenGLImage*>(m_CharaChipImage[chipindex]), 0, 0, MAPCHIP_WIDTH, MAPCHIP_HEIGHT, x, y, FACE_WIDTH, FACE_HEIGHT);
    }
}
//---------------------------------------------------------------------------
void TOpenFarenImage::GetEffectSize(int index, int& width, int& height) const
{
    width = m_EffectImage[index]->getWidth();
    height = m_EffectImage[index]->getHeight();
}
//---------------------------------------------------------------------------
void TOpenFarenImage::GetEventSize(int index, int& width, int& height) const
{
    width = m_EventImage[index]->getWidth();
    height = m_EventImage[index]->getHeight();
}
//---------------------------------------------------------------------------
gcn::Image* TOpenFarenImage::LoadMyImage(const std::string& filename)
{
    gcn::Image* img;
    try{
        img = gcn::Image::load(filename);
    }
    catch(gcn::Exception){
        OFERRLOG((boost::format("Failed to load image %s") % filename.c_str()).str());
        return NULL;
    }
    if(img == NULL){
        return NULL;
    }
    m_MyImageList.push_back(img);
    return img;
}
//---------------------------------------------------------------------------
void TOpenFarenImage::RemoveMyImage(gcn::Image *img)
{
    std::vector<gcn::Image*>::iterator itr = std::find(m_MyImageList.begin(), m_MyImageList.end(), img);
    if(itr != m_MyImageList.end()){
        delete *itr;
        m_MyImageList.erase(itr);
    }
    else{
        OFERRLOG("݂Ȃ摜폜悤Ƃ܂B");
    }
}
//---------------------------------------------------------------------------
void TOpenFarenImage::DrawImageStretch(gcn::Graphics* g, const gcn::OpenGLImage *srcImage, int srcX, int srcY, int srcW, int srcH, int dstX, int dstY, int dstW, int dstH) const
{
    const gcn::ClipRectangle& top = g->getCurrentClipArea();
    dstX += top.xOffset;
    dstY += top.yOffset;

    // Find OpenGL texture coordinates
    float texX1 = srcX / (float)srcImage->getTextureWidth();
    float texY1 = srcY / (float)srcImage->getTextureHeight();
    float texX2 = (srcX+srcW) / (float)srcImage->getTextureWidth();
    float texY2 = (srcY+srcH) / (float)srcImage->getTextureHeight();

    glBindTexture(GL_TEXTURE_2D, srcImage->getTextureHandle());

    glEnable(GL_TEXTURE_2D);

    //glEnable(GL_BLEND);

    // Draw a textured quad -- the image
    glBegin(GL_QUADS);
    glTexCoord2f(texX1, texY1);
    glVertex3i(dstX, dstY, 0);

    glTexCoord2f(texX1, texY2);
    glVertex3i(dstX, dstY + dstH, 0);

    glTexCoord2f(texX2, texY2);
    glVertex3i(dstX + dstW, dstY + dstH, 0);

    glTexCoord2f(texX2, texY1);
    glVertex3i(dstX + dstW, dstY, 0);
    glEnd();
    glDisable(GL_TEXTURE_2D);

    glDisable(GL_BLEND);
}
//---------------------------------------------------------------------------
void TOpenFarenImage::DrawImageStretchSimple(gcn::Graphics* g, const gcn::OpenGLImage *img, int dstx, int dsty, int xzoom, int yzoom) const
{
    int w = img->getWidth();
    int h = img->getHeight();
    DrawImageStretch(g, img, 0, 0, w, h, dstx, dsty, ((w*xzoom)>>8), ((h*yzoom)>>8));
}
//---------------------------------------------------------------------------
// HHȓhׂ@B̂ifBA̐͒n}ɓhco邵B
void TOpenFarenImage::TracePaintData(unsigned *pixels, int w, int h, int startx, int starty, unsigned paintcolor, unsigned edgecolor)
{
    std::vector<point_t> seeds;
    seeds.push_back(point_t(startx, starty));
    while(!seeds.empty()){
        int sx = seeds.back().x;
        int sy = seeds.back().y;
        seeds.pop_back();

        unsigned *p = pixels + w * sy + sx;
        int x2;
        for(x2=sx; x2<w; ++x2){
            if(*p == edgecolor || *p == paintcolor){
                break;
            }
            *p = paintcolor;
            ++p;
        }
        --x2;
        //if(x2 > w - 1){
        //    x2 = w - 1;
        //}

        p = pixels + w * sy + sx - 1;
        int x1;
        for(x1=sx-1; x1>=0; --x1){
            if(*p == edgecolor || *p == paintcolor){
                break;
            }
            *p = paintcolor;
            --p;
        }
        ++x1;
        //if(x1 < 0){
        //    x1 = 0;
        //}

        if(sy > 0){
            p = pixels + w * (sy - 1) + x1;
            for(int x=x1; x<=x2;){
                if(*p != edgecolor && *p != paintcolor){
                    seeds.push_back(point_t(x, sy - 1));
                    while(x <= x2 && *p != edgecolor && *p != paintcolor){
                        ++x;
                        ++p;
                    }
                }
                else{
                    ++x;
                    ++p;
                }
            }
        }
        if(sy < h - 1){
            p = pixels + w * (sy + 1) + x1;
            for(int x=x1; x<=x2;){
                if(*p != edgecolor && *p != paintcolor){
                    seeds.push_back(point_t(x, sy + 1));
                    while(x <= x2 && *p != edgecolor && *p != paintcolor){
                        ++x;
                        ++p;
                    }
                }
                else{
                    ++x;
                    ++p;
                }
            }
        }
    }
}
//---------------------------------------------------------------------------
// ABGR`̐gcn::ColorɕϊB
gcn::Color IntToGcnColor(unsigned u)
{
    return gcn::Color((u & 0xff), ((u & 0xff00)>>8), ((u & 0xff0000)>>16), ((u & 0xff000000)>>24));
}
//---------------------------------------------------------------------------
// SDL_SurfaceOpenGLT[tFCXpsNZzɃRs[B
// dstarray̗vf͏ȂƂ dstw * blth ̂ƂB܂Abltw <= dstw ƂB
void TOpenFarenImage::BltSDLToOpenGL(SDL_Surface* surface_src, unsigned *dstarray, int srcx, int srcy, int bltw, int blth, int dstw)
{
    int pixelbytes = surface_src->format->BytesPerPixel;
    if(pixelbytes == 4 || pixelbytes == 3){
        // ꂼ̐F̈ʒuB
        int ri = surface_src->format->Rshift >> 3;
        int gi = surface_src->format->Gshift >> 3;
        int bi = surface_src->format->Bshift >> 3;
        for(int y=0; y<blth; ++y){
            unsigned char *psrc = (unsigned char*)surface_src->pixels + surface_src->pitch * (srcy + y) + srcx * pixelbytes;
            unsigned *pdst = dstarray + y * dstw;
            for(int x=0; x<bltw; ++x){
                // ƌ̂@邾낤ǁA܂ccB蔲B
                unsigned r = psrc[ri];
                unsigned g = psrc[gi];
                unsigned b = psrc[bi];
                *pdst = 0xff000000 | (b<<16) | (g<<8) | r;
                psrc += pixelbytes;
                ++pdst;
            }
        }
    }
    else if(pixelbytes == 1){
        const SDL_Palette *palette = surface_src->format->palette;
        int i = 0;
        for(int y=0; y<blth; ++y){
            unsigned char *psrc = (unsigned char*)surface_src->pixels + surface_src->pitch * (srcy + y) + srcx * pixelbytes;
            unsigned *pdst = dstarray + y * dstw;
            for(int x=0; x<bltw; ++x){
                if(*psrc < palette->ncolors){
                    const SDL_Color& c = palette->colors[*psrc];
                    *pdst = 0xff000000 | (c.b<<16) | (c.g<<8) | c.r;
                }
                ++psrc;
                ++pdst;
            }
        }
    }
    else{
        OFERRLOG((boost::format("unknown pixel format (BytesPerPixel = %d)") % pixelbytes).str());
        return;
    }
}
//---------------------------------------------------------------------------
