//---------------------------------------------------------------------------
#ifndef ofimageH
#define ofimageH

#include <vector>
#include <map>
#include <guichan.hpp>
#include <guichan/opengl/openglimage.hpp>
#include <SDL.h>     // for SDL_Surface
#include <boost/shared_array.hpp>
#include "oftypes.h" // for point_t
//---------------------------------------------------------------------------
class TOpenFarenImage
{
public:
    enum TButtonImageIndex{
        BUTTONIMAGE_WAIT,
        BUTTONIMAGE_MOVE,
        BUTTONIMAGE_ATTACK,
        BUTTONIMAGE_MAGIC,
        BUTTONIMAGE_SPECIAL,
        BUTTONIMAGE_CHARGE,
        BUTTONIMAGE_REDO,
        BUTTONIMAGE_REDO2,
        BUTTONIMAGE_RUNAWAY,
        BUTTONIMAGE_MAX
    };
    enum TChipImageIndex{
        CHIPIMAGE_DEFENSE,
        CHIPIMAGE_DEPARTURE,
        CHIPIMAGE_MAX
    };
    static const int MAPCHIP_WIDTH = 32;
    static const int MAPCHIP_HEIGHT = 32;
    static const int FACE_WIDTH = 96;
    static const int FACE_HEIGHT = 96;

    TOpenFarenImage();
    ~TOpenFarenImage();
    void SetDirectory(const std::string& dir) { m_BaseDir = dir; };
    void SetSystemImageDirectory(const std::string& dir) { m_SystemImageDir = dir; };
    // LN^[fBNgɂ摜͂̃NXŃpXȂ̂ŕϑIȃt@CwɂȂB
    int LoadFlag(const std::string& dir, const std::string& filename);
    int LoadFace(const std::string& dir, const std::string& filename);
    int LoadCharaChip(const std::string& dir, const std::string& filename);
    int LoadEffect(const std::string& filename);
    bool LoadMapChip(const std::string& filename);
    bool LoadMapObj(const std::string& filename);
    int LoadEvent(const std::string& filename);
    //gcn::Image* LoadImageAndMemory(const std::string& filename);
    int GenerateAreaName(const std::string& name);
    void ClearAreaName();

    bool LoadFieldMapImage(const std::string& filename);
    const std::vector<gcn::Image*>& GetFieldMapImage() const { return m_FieldMapImage; }
    int GetFieldMapImageXCount() const { return m_FieldMapImageXCount; }
    bool LoadPowerMap(const std::string& filename);
    void InitPowerMap();
    void SetPowerMapPoints(unsigned pixel, const std::vector<point_t>& plist);
    gcn::Image* GetPowerMap();
    // R}h{^̉摜
    bool LoadButtonImage();
    gcn::Image* GetButtonImage(TButtonImageIndex i);
    bool LoadChipImage();

    void DrawFace(gcn::Graphics* g, int x, int y, int index) const;
    void DrawFaceStretch(gcn::Graphics* g, int x, int y, int index, int xzoom, int yzoom) const;
    void DrawFlag(gcn::Graphics* g, int x, int y, int index, int tick) const;
    void DrawCharaChip(gcn::Graphics* g, int x, int y, int index) const;
    void DrawCharaChipStretch(gcn::Graphics* g, int x, int y, int index,  int xzoom, int yzoom) const;
    void DrawEffect(gcn::Graphics* g, int x, int y, int index, int width, int height, int anim, int direction) const;
    void DrawMapChip(gcn::Graphics* g, int x, int y, int index) const;
    void DrawMapObj(gcn::Graphics* g, int x, int y, int index) const;
    void DrawChip(gcn::Graphics* g, int x, int y, TChipImageIndex index) const;
    void DrawEvent(gcn::Graphics* g, int x, int y, int index) const;
    void DrawEventStretch(gcn::Graphics* g, int x, int y, int index, int xzoom, int yzoom) const;
    void DrawAreaName(gcn::Graphics* g, int x, int y, int index) const;
    void DrawCharaFaceOrChip(gcn::Graphics* g, int x, int y, int faceindex, int chipindex) const;

    void GetEffectSize(int index, int& width, int& height) const;
    void GetEventSize(int index, int& width, int& height) const;
private:
    std::vector<gcn::Image*> m_FlagImage;
    std::vector<gcn::Image*> m_FaceImage;
    std::vector<gcn::Image*> m_CharaChipImage;
    std::vector<gcn::Image*> m_EffectImage;
    std::vector<gcn::Image*> m_AreaNameImage;
    std::vector<gcn::Image*> m_EventImage;
    std::map<std::string, int> m_EventIndex;
    gcn::Image* m_MapChipImage;
    gcn::Image* m_MapObjImage;
    int m_MapChipXCount;
    int m_MapObjXCount;
    std::vector<gcn::Image*> m_FieldMapImage;
    int m_FieldMapImageXCount;                  // tB[h}bv摜ɕׂ鐔B
    gcn::Image* m_ButtonImage[BUTTONIMAGE_MAX];
    gcn::Image* m_ChipImage[CHIPIMAGE_MAX];

    SDL_Surface* m_SDLPowerMap;
    gcn::Image* m_PowerMapImage;
    boost::shared_array<unsigned> m_PowerMapPixels; // ABGR`̃sNZړĂzB
    unsigned m_PowerMapEdgeColor;                   // ABGR`̋EFB

    std::string m_BaseDir;  // ViItH_̃tpXBႦDefaultȂScenario/Default̃tpXB
    std::string m_SystemImageDir;  // ViItH_Ɋ܂܂ĂȂ摜TtpXB̂Ƃ"st@Ĉꏊ/System"̃NX͋̓IȒlmȂŁB

    // ĂS摜̃|C^Bjdelete邽߂ɂB
    std::vector<gcn::Image*> m_MyImageList;

    gcn::Image* LoadMyImage(const std::string& filename);

    void RemoveMyImage(gcn::Image *img);
    void DrawImageStretch(gcn::Graphics* g, const gcn::OpenGLImage *srcImage, int srcX, int srcY, int srcW, int srcH, int dstX, int dstY, int dstW, int dstH) const;
    void DrawImageStretchSimple(gcn::Graphics* g, const gcn::OpenGLImage *img, int dstx, int dsty, int xzoom, int yzoom) const;
    void TracePaintData(unsigned *pixels, int w, int h, int startx, int starty, unsigned paintcolor, unsigned edgecolor);
    void BltSDLToOpenGL(SDL_Surface* surface_src, unsigned *dstarray, int srcx, int srcy, int bltw, int blth, int dstw);
};
//---------------------------------------------------------------------------
// ̃\[XɒuH
gcn::Color IntToGcnColor(unsigned u);
//---------------------------------------------------------------------------
#endif
