#include "ofini.h"
#include "logger.h"
#include <babel.h>

TOpenFarenIni g_Ini;
//---------------------------------------------------------------------------
namespace
{
    gcn::Color GetColor(const TIniLoader& loader, const std::string& catkey, const std::string& key, int defc)
    {
        int i = defc;
        loader.GetIntHex(catkey, key, i);
        return gcn::Color((i>>16) & 0xff, (i>>8) & 0xff, i & 0xff, (i>>24) & 0xff);
    }

    int ColorToInt(const gcn::Color c){
        return c.a<<24 | c.r<<16 | c.g<<8 | c.b;
    }

    struct TBoolValItem{
        const char *category;
        const char *name;
        //bool default_value;
    };

    struct TIntValItem{
        const char *category;
        const char *name;
        int default_value;
    };

    struct TColorValItem{
        const char *category;
        const char *name;
        int default_value;
    };

    struct TStringValItem{
        const char *category;
        const char *name;
        const char *default_value;
    };

    struct TShortcutItem{
        const char *category;
        const char *name;
    };

    const TBoolValItem BoolItem[TOpenFarenIni::BOOLVAL_MAX] = {
        {"Window", "DisableSwapInterval"},
        {"Mixer", "MusicEnabled"},
        {"Mixer", "SoundEnabled"},
        {"Watch", "Use"},
        {"Watch", "OnlyLarge"},
        {"Watch", "OnlyMaster"},
        {"Watch", "NoNeutral"},
        {"Watch", "MaxSpeed"}
    };

    const TIntValItem IntItem[TOpenFarenIni::INTVAL_MAX] = {
        {"Window", "FPS", 60},
        {"Window", "Width", 800},
        {"Window", "Height", 600},
        {"GUI", "ScrollbarWidth", 16},
        {"Battle", "AnimationSpeed", 3},
        {"Battle", "TurnMax", 10},
        {"Screen", "DrawRectangleSlideLeft", 0},
        {"Screen", "DrawRectangleSlideRight", 0},
        {"Screen", "DrawRectangleSlideTop", 0},
        {"Screen", "DrawRectangleSlideBottom", 0}
    };

    const TColorValItem ColorItem[TOpenFarenIni::COLORVAL_MAX] = {
        {"GUI", "TopColor", 0xff606080}
    };

    const TStringValItem StringItem[TOpenFarenIni::STRINGVAL_MAX] = {
        {"File", "FileNameEncoding", "utf-8"}};

    const TShortcutItem ShortcutItem[TOpenFarenIni::SHORTCUTVAL_MAX] = {
        {"Shortcut", "Wait"},
        {"Shortcut", "Move"},
        {"Shortcut", "Attack"},
        {"Shortcut", "Charge"},
        {"Shortcut", "Special"},
        {"Shortcut", "Magic"},
        {"Shortcut", "RunAway"}};

    const TColorValItem SkinColorItem[TOpenFarenSkin::SKINCOLORVAL_MAX] = {
        {"GUI", "WindowColor", 0xff4040ff},
        {"GUI", "ButtonEnabledColor", 0xff808080},
        {"GUI", "ButtonDisabledColor", 0xff404040},
        {"GUI", "DarkPaneColor", 0xff000020},
        {"GUI", "GrayPaneColor", 0xffd0d0d0},
        {"GUI", "EventPaneColor", 0xff0020c0},
        {"GUI", "ListBoxBackgroundColor", 0xff000000},
        {"GUI", "ListBoxSelectionColor", 0xff000080},
        {"GUI", "SelectedFrameColor", 0x80ff0000},
        {"GUI", "RadioButtonBGColor", 0xff000000},
        {"GUI", "MagicSelectPaneColor", 0x80000080},
        {"GUI", "BattleCharaListPaneColor", 0x80000080}
    };
}
//---------------------------------------------------------------------------
bool TOpenFarenIni::LoadFromFile(const std::string& filename){
    TIniLoader& loader = m_Loader;
    // ȉ̃ftHglKp邽ߓǂݍ݃G[͖B
    if(!loader.LoadFromFile(filename)){
        g_Logger.WarningLog(filename + " ̓ǍɎs܂ftHglpđs܂B");
    }

    if(!loader.GetString("GUI", "FontName", m_FontName)){
        m_FontName.clear();
    }
    m_LogLevel = 3;
    loader.GetInt("Log", "LogLevel", m_LogLevel);
    m_LogEveryFlush = true;
    int i;
    if(loader.GetInt("Log", "LogEveryFlush", i)){
        m_LogEveryFlush = (i != 0);
    }
    m_FontHeight = 16;
    m_FontColor = GetColor(loader, "GUI", "FontColor", 0xffffffff);
    m_FontDisabledColor = GetColor(loader, "GUI", "FontDisabledColor", 0xff808080);
    //m_PaneColor = GetColor(loader, "GUI", "PaneColor", 0xff000000);
    m_MsgDlgColor = GetColor(loader, "GUI", "MsgDlgColor", 0xffa0a0a0);
    m_WorldMapEdgeColor = GetColor(loader, "GUI", "WorldMapEdgeColor", 0xc0ff0000);
    m_BattleMapShadeColor = GetColor(loader, "GUI", "BattleMapShadeColor", 0x80000000);
    m_DoubleClickFrame = 30;
    loader.GetInt("GUI", "DoubleClickFrame", m_DoubleClickFrame);
    //m_DepartureAnimationFrame = 30;
    //loader.GetInt("GUI", "DepartureAnimationFrame", m_DepartureAnimationFrame);
    m_MessageTimeoutFrame = 60;
    loader.GetInt("GUI", "MessageTimeoutFrame", m_MessageTimeoutFrame);

    m_DefaultSkin.LoadFromIni(loader);

    m_BoolValues.resize(BOOLVAL_MAX);
    for(int i=0; i<BOOLVAL_MAX; ++i){
        m_BoolValues[i] = loader.GetInt(BoolItem[i].category, BoolItem[i].name);
    }

    m_IntValues.resize(INTVAL_MAX);
    for(int i=0; i<INTVAL_MAX; ++i){
        m_IntValues[i] = IntItem[i].default_value;
        loader.GetInt(IntItem[i].category, IntItem[i].name, m_IntValues[i]);
    }

    m_ColorValues.resize(COLORVAL_MAX);
    for(int i=0; i<COLORVAL_MAX; ++i){
        m_ColorValues[i] = GetColor(loader, ColorItem[i].category, ColorItem[i].name, ColorItem[i].default_value);
    }

    m_StringValues.resize(STRINGVAL_MAX);
    for(int i=0; i<STRINGVAL_MAX; ++i){
        m_StringValues[i] = StringItem[i].default_value;
        loader.GetString(StringItem[i].category, StringItem[i].name, m_StringValues[i]);
    }

    m_ShortcutValues.resize(SHORTCUTVAL_MAX);
    for(int i=0; i<SHORTCUTVAL_MAX; ++i){
        m_ShortcutValues[i] = SDLK_UNKNOWN;
        int v;
        if(loader.GetInt(ShortcutItem[i].category, ShortcutItem[i].name, v)){
             m_ShortcutValues[i] = static_cast<SDLKey>(v);
        }
    }

    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenIni::SaveToFile(const std::string& filename) const
{
    return m_Loader.SaveToFile(filename);
}
//---------------------------------------------------------------------------
void TOpenFarenIni::SetRealWindowSize(int w, int h)
{
    m_RealWindowWidth = w;
    m_RealWindowHeight = h;
}
//---------------------------------------------------------------------------
void TOpenFarenIni::GetRealWindowSize(int& w, int& h) const
{
    w = m_RealWindowWidth;
    h = m_RealWindowHeight;
}
//---------------------------------------------------------------------------
/*
void TOpenFarenIni::ApplyButtonEnabled(gcn::Button& b)
{
    b.setForegroundColor(GetFontColor());
    b.setBaseColor(GetColorValue(GUICOLOR_BUTTONENABLED));
    b.setEnabled(true);
}
//---------------------------------------------------------------------------
void TOpenFarenIni::ApplyButtonDisabled(gcn::Button& b)
{
    b.setForegroundColor(GetFontDisabledColor());
    b.setBaseColor(GetColorValue(GUICOLOR_BUTTONDISABLED));
    b.setEnabled(false);
}
//---------------------------------------------------------------------------
void TOpenFarenIni::ApplyCheckBoxEnabled(gcn::CheckBox& cb)
{
    cb.setForegroundColor(GetFontColor());
    cb.setBackgroundColor(gcn::Color(0, 0, 0));
    cb.setEnabled(true);
}
//---------------------------------------------------------------------------
void TOpenFarenIni::ApplyCheckBoxDisabled(gcn::CheckBox& cb)
{
    cb.setForegroundColor(gcn::Color(192, 192, 192)); // GetFontDisabledColor()ƂقƂǌȂȂB
    cb.setBackgroundColor(gcn::Color(128, 128, 128));
    cb.setEnabled(false);
}
//---------------------------------------------------------------------------
void TOpenFarenIni::ApplyListBoxColor(gcn::ListBox& lb)
{
    lb.setForegroundColor(GetFontColor());
    lb.setBackgroundColor(GetColorValue(GUICOLOR_LISTBOXBGCOLOR));
    lb.setSelectionColor(GetColorValue(GUICOLOR_LISTBOXSELCOLOR));
}
//---------------------------------------------------------------------------
void TOpenFarenIni::ApplyScrollAreaProperty(gcn::ScrollArea& sa)
{
    sa.setBackgroundColor(g_GameMaster.GetSkinManager()->GetSkin()->GetColorValue(SKINCOLOR_DARKPANE));
    sa.setScrollbarWidth(GetIntValue(SCROLLBAR_WIDTH));
}
*/
//---------------------------------------------------------------------------
void TOpenFarenIni::EncodeFileNameFromUTF8(const std::string& src, std::string& dst)
{
#if defined(WIN32)
    dst = babel::utf8_to_sjis(src.c_str());
#else
    std::string enc = GetStringValue(FILENAME_ENCODING);
    if(!enc.compare("utf-8")){
        dst = src;
    }
    else if(!enc.compare("euc-jp")){
        dst = babel::utf8_to_euc(src.c_str());
    }
    else{
        OFERRLOG((boost::format("unknown encoding(value of openfaren.ini [File] FileNameEncoding):%s") % enc).str());
    }
#endif
}
//---------------------------------------------------------------------------
void TOpenFarenIni::EncodeFileNameFromSJIS(const std::string& src, std::string& dst)
{
#if defined(WIN32)
    dst = src;
#else
    std::string enc = GetStringValue(FILENAME_ENCODING);
    if(!enc.compare("utf-8")){
        dst = babel::sjis_to_utf8(src.c_str());
    }
    else if(!enc.compare("euc-jp")){
        dst = babel::sjis_to_euc(src.c_str());
    }
    else{
        OFERRLOG((boost::format("unknown encoding(value of openfaren.ini [File] FileNameEncoding):%s") % enc).str());
    }
#endif
}
//---------------------------------------------------------------------------
void TOpenFarenIni::SetBoolValue(TBoolValue valtype, bool val)
{
    m_BoolValues[valtype] = val;
    m_Loader.SetInt(BoolItem[valtype].category, BoolItem[valtype].name, val);
}
//---------------------------------------------------------------------------
void TOpenFarenIni::SetIntValue(TIntValue valtype, int val)
{
    m_IntValues[valtype] = val;
    m_Loader.SetInt(IntItem[valtype].category, IntItem[valtype].name, val);
}
//---------------------------------------------------------------------------
void TOpenFarenIni::SetShortcutValue(TShortcutValue valtype, SDLKey val)
{
    m_ShortcutValues[valtype] = val;
    m_Loader.SetInt(ShortcutItem[valtype].category, ShortcutItem[valtype].name, static_cast<int>(val));
}
//---------------------------------------------------------------------------
bool TOpenFarenIni::FindShortcutValue(SDLKey val, TShortcutValue& valtype)
{
    for(int i=SHORTCUT_WAIT; i<SHORTCUTVAL_MAX; ++i){
        if(val == m_ShortcutValues[i]){
            valtype = static_cast<TShortcutValue>(i);
            return true;
        }
    }
    return false;
}
//---------------------------------------------------------------------------
void TOpenFarenSkin::LoadFromIni(const TIniLoader& loader)
{
    for(int i=0; i<SKINCOLORVAL_MAX; ++i){
        m_ColorValues[i] = GetColor(loader, SkinColorItem[i].category, SkinColorItem[i].name, SkinColorItem[i].default_value);
    }
}
