#ifndef ofiniH
#define ofiniH

#include <string>
#include <vector>
#include <guichan.hpp>
#include "SDL.h" // SDLKey
#include "iniload.h"
//---------------------------------------------------------------------------
class TOpenFarenSkin
{
public:
    enum TSkinColorValue{
        SKINCOLOR_WINDOW,
        SKINCOLOR_BUTTONENABLED,
        SKINCOLOR_BUTTONDISABLED,
        SKINCOLOR_DARKPANE,
        SKINCOLOR_GRAYPANE,
        //SKINCOLOR_SCROLLBAR,
        SKINCOLOR_EVENTPANE,
        SKINCOLOR_LISTBOXBGCOLOR,
        SKINCOLOR_LISTBOXSELCOLOR,
        SKINCOLOR_SELFRAMECOLOR,
        SKINCOLOR_RADIOBUTTONBG,
        SKINCOLOR_MAGICSELECTPANE,
        SKINCOLOR_BATTLECHARALISTPANE,
        SKINCOLORVAL_MAX
    };

    void LoadFromIni(const TIniLoader& loader);
    gcn::Color GetColorValue(TSkinColorValue valtype) const { return m_ColorValues[valtype]; }
private:
    gcn::Color m_ColorValues[SKINCOLORVAL_MAX];
};
//---------------------------------------------------------------------------
class TOpenFarenIni
{
public:
    enum TBoolValue{
        WINDOW_DISABLESWAPINTERVAL,
        MUSIC_ENABLED,
        SOUND_ENABLED,
        WATCH_USE,
        WATCH_ONLYLARGE,
        WATCH_ONLYMASTER,
        WATCH_NONEUTRAL,
        WATCH_MAXSPEED,
        BOOLVAL_MAX
    };
    enum TIntValue{
        WINDOW_FPS,
        WINDOW_WIDTH,
        WINDOW_HEIGHT,
        SCROLLBAR_WIDTH,
        BATTLE_ANIMSPEED,
        BATTLE_TURNMAX,
        DRAWRECT_SLIDEL,
        DRAWRECT_SLIDER,
        DRAWRECT_SLIDET,
        DRAWRECT_SLIDEB,
        INTVAL_MAX
    };
    enum TColorValue{
        GUICOLOR_TOP,
        COLORVAL_MAX
    };
    enum TStringValue{
        FILENAME_ENCODING,
        STRINGVAL_MAX
    };
    enum TShortcutValue
    {
        SHORTCUT_WAIT,
        SHORTCUT_MOVE,
        SHORTCUT_ATTACK,
        SHORTCUT_CHARGE,
        SHORTCUT_SPECIAL,
        SHORTCUT_MAGIC,
        SHORTCUT_RUNAWAY,
        SHORTCUTVAL_MAX
    };

    bool LoadFromFile(const std::string& filename);
    bool SaveToFile(const std::string& filename) const;
    void SetRealWindowSize(int w, int h);
    void GetRealWindowSize(int& w, int& h) const;
//    void ApplyButtonEnabled(gcn::Button& b);
//    void ApplyButtonDisabled(gcn::Button& b);
//    void ApplyCheckBoxEnabled(gcn::CheckBox& cb);
//    void ApplyCheckBoxDisabled(gcn::CheckBox& cb);
//    void ApplyListBoxColor(gcn::ListBox& lb);
//    void ApplyScrollAreaProperty(gcn::ScrollArea& sa);
    int GetLogLevel() const { return m_LogLevel; }
    bool GetLogEveryFlush() const { return m_LogEveryFlush; }
    const std::string& GetFontName() const { return m_FontName; }
    int GetFontHeight() const { return m_FontHeight; }
    gcn::Color GetFontColor() const { return m_FontColor; }
    gcn::Color GetFontDisabledColor() const { return m_FontDisabledColor; }
    gcn::Color GetMsgDlgColor() const { return m_MsgDlgColor; }
    gcn::Color GetWorldMapEdgeColor() const { return m_WorldMapEdgeColor; }
    gcn::Color GetBattleMapShadeColor() const { return m_BattleMapShadeColor; }
    //gcn::Color GetSelectedFrameColor() const { return m_SelectedFrameColor; }
    int GetDoubleClickFrame() const { return m_DoubleClickFrame; }
    //int GetScrollbarWidth() const { return m_ScrollbarWidth; }
    //int GetDepartureAnimationFrame() const { return m_DepartureAnimationFrame; }
    int GetMessageTimeoutFrame() const { return m_MessageTimeoutFrame; }
    const TOpenFarenSkin* GetDefaultSkin() const { return &m_DefaultSkin; }
    int GetBoolValue(TBoolValue valtype) const { return m_BoolValues[valtype]; }
    int GetIntValue(TIntValue valtype) const { return m_IntValues[valtype]; }
    SDLKey GetShortcutValue(TShortcutValue valtype) const { return m_ShortcutValues[valtype]; }
    gcn::Color GetColorValue(TColorValue valtype) const { return m_ColorValues[valtype]; }
    const std::string& GetStringValue(TStringValue valtype) const { return m_StringValues[valtype]; }
    void EncodeFileNameFromUTF8(const std::string& src, std::string& dst);
    void EncodeFileNameFromSJIS(const std::string& src, std::string& dst);
    void SetBoolValue(TBoolValue valtype, bool val);
    void SetIntValue(TIntValue valtype, int val);
    void SetShortcutValue(TShortcutValue valtype, SDLKey val);
    bool FindShortcutValue(SDLKey val, TShortcutValue& valtype);
private:
    TIniLoader m_Loader;
    TOpenFarenSkin m_DefaultSkin;
    int m_LogLevel;
    bool m_LogEveryFlush;
    std::string m_FontName;
    int m_FontHeight;
    gcn::Color m_FontColor;
    gcn::Color m_FontDisabledColor;
    //gcn::Color m_PaneColor;
    gcn::Color m_ListBoxBackgroundColor;
    gcn::Color m_ListBoxSelectionColor;
    gcn::Color m_MsgDlgColor;
    gcn::Color m_WorldMapEdgeColor;
    gcn::Color m_BattleMapShadeColor;
    int m_DoubleClickFrame;
    //int m_DepartureAnimationFrame;
    int m_MessageTimeoutFrame;
    int m_RealWindowWidth;
    int m_RealWindowHeight;
    std::deque<bool> m_BoolValues;
    std::vector<int> m_IntValues;
    std::vector<gcn::Color> m_ColorValues;
    std::vector<std::string> m_StringValues;
    std::vector<SDLKey> m_ShortcutValues;
};
//---------------------------------------------------------------------------
extern TOpenFarenIni g_Ini;
//---------------------------------------------------------------------------
#endif
