//---------------------------------------------------------------------------
#include "oflua.h"
#include "luautil.h"
#include "luaxml.h"
#include "logger.h"
#include "boostformat.h"
#include "ofgame.h"
#include "worldmap.h"
#include "neighbor.h"
#include "msgdlg.h"
#include "ofini.h"
#include "gamemaster.h"

namespace
{
    TScriptAPI *ScriptAPI;
    TBattleInterface *BattleIf;
    TWorldMapLogic *WorldMapLogic;
    int BattleMasterIndex;

    std::vector<int> EndingDisabledMaster;
}

#if defined(WIN32)
#define FUNCNAME __FUNCTION__
#else
#define FUNCNAME __func__
#endif

namespace
{
//---------------------------------------------------------------------------
void error_log_withinfo(lua_State *L, const char *str)
{
    const char *src;
    int curline;

    if(luautil_currentsrc(L, &src, &curline) != -1){
        g_Logger.ErrorLog((boost::format("%s Line %d : %s") % src % curline % str).str());
    }
    else{
        g_Logger.ErrorLog((boost::format("? Line ? : %s") % str).str());
    }
}
//---------------------------------------------------------------------------
int debug_log(lua_State *L)
{
    char str[1024];

    luautil_args2str(L, str, sizeof str);
    lua_settop(L, 0);
    g_Logger.DebugLog((boost::format("Lua Log: %s") % str).str());
    return 0;
}
//---------------------------------------------------------------------------
int api_battle_get_turn( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but not battle.") % FUNCNAME).str());
        return 0;
    }
    // X^bNNA
    lua_settop(L, 0);
    lua_pushnumber(L, BattleIf->GetTurn());
    return 1;
}
//---------------------------------------------------------------------------
int api_battle_get_map_size( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but not battle.") % FUNCNAME).str());
        return 0;
    }
    int w, h;
    BattleIf->GetMapSize(w, h);
    lua_pushnumber(L, w);
    lua_pushnumber(L, h);
    return 2;
}
//---------------------------------------------------------------------------
int api_battle_get_map_height( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 2 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int x = lua_tonumber(L, 1);
    int y = lua_tonumber(L, 2);
    // X^bNNA
    lua_settop(L, 0);
    lua_pushnumber(L, BattleIf->GetMapHeight(x, y));
    return 1;
}
//---------------------------------------------------------------------------
int api_battle_get_movetype_totalcost( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 1 || !lua_isnumber(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int movetype = lua_tonumber(L, 1);
    // X^bNNA
    lua_settop(L, 0);
    lua_pushnumber(L, BattleIf->GetMoveTypeCost(movetype));
    return 1;
}
//---------------------------------------------------------------------------
int api_battle_get_movetype_totalskill( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 1 || !lua_isnumber(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int movetype = lua_tonumber(L, 1);
    // X^bNNA
    lua_settop(L, 0);
    lua_pushnumber(L, BattleIf->GetMoveTypeSkill(movetype));
    return 1;
}
//---------------------------------------------------------------------------
// ړ\ȍW̃Xg𓾂B
int api_battle_get_movable( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but not battle.") % FUNCNAME).str());
        return 0;
    }

    int w, h;
    BattleIf->GetMapSize(w, h);
    TValueLayer<int> layer(0, 0);
    BattleIf->GetMovableLayer(layer);
    int count = 0;
    // Ve[uǉ
    lua_newtable(L);
//    for(int y=0; y<h; ++y){
//        for(int x=0; x<w; ++x){
//            if(layer.GetValue(x, y)){
//                lua_pushnumber(L, count++);
//                // Ve[uǉ
//                lua_newtable(L);
//                lua_pushstring(L, "x");
//                lua_pushnumber(L, x);
//                lua_rawset(L, -3);
//                lua_pushstring(L, "y");
//                lua_pushnumber(L, y);
//                lua_rawset(L, -3);
//                lua_settable(L, 1);
//            }
//        }
//    }

    int mi, ci;
    BattleIf->GetActiveChara(mi, ci);
    TCharacterInstance chara;
    BattleIf->GetCharaStatus(mi, ci, chara);
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
    int movable = charadata.m_movable;
    if(movable > TNeighbor::NEIGHBOR_MAX){
        movable = TNeighbor::NEIGHBOR_MAX;
    }
    int from = TNeighbor::GetNeighborIndex(0);
    int to = TNeighbor::GetNeighborIndex(movable + 1);
    int mode = chara.m_location.y%2;
    for(int i=from; i<to; ++i){
        point_t p = TNeighbor::GetNeighbor(mode, i);
        int x = chara.m_location.x + p.x;
        int y = chara.m_location.y + p.y;
        if(x >= 0 && x < w && y >= 0 && y < h && layer.GetValue(x, y)){
            lua_pushnumber(L, count++);
            // Ve[uǉ
            lua_newtable(L);
            lua_pushstring(L, "x");
            lua_pushnumber(L, x);
            lua_rawset(L, -3);
            lua_pushstring(L, "y");
            lua_pushnumber(L, y);
            lua_rawset(L, -3);
            lua_settable(L, 1);
        }
    }
    return 1;
}
//---------------------------------------------------------------------------
int api_battle_get_movable_layer( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but not battle.") % FUNCNAME).str());
        return 0;
    }

    int mi, ci;
    BattleIf->GetActiveChara(mi, ci);

    int w, h;
    BattleIf->GetMapSize(w, h);
    TValueLayer<int> layer(0, 0);
    BattleIf->GetMovableLayer(layer);
    int count = 0;
    // Ve[uǉ
    lua_createtable(L, w * h, 0);
    for(int y=0; y<h; ++y){
        for(int x=0; x<w; ++x){
            lua_pushnumber(L, layer.GetValue(x, y));
            lua_rawseti(L, 1, count++);
        }
    }
    return 1;
}
//---------------------------------------------------------------------------
int api_battle_get_move_layer_from( lua_State *L )
{
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but not battle.") % FUNCNAME).str());
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 6 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2) || !lua_isnumber(L, 3) 
              || !lua_isnumber(L, 4) || !lua_isnumber(L, 5) || !lua_isboolean(L, 6)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int mi = lua_tonumber(L, 1);
    int ci = lua_tonumber(L, 2);
    int x = lua_tonumber(L, 3);
    int y = lua_tonumber(L, 4);
    int dist = lua_tonumber(L, 5);
    bool loose = lua_toboolean(L, 6);
    // X^bNNA
    lua_settop(L, 0);

    TCharacterInstance chara;
    BattleIf->GetCharaStatus(mi, ci, chara);
    int w, h;
    BattleIf->GetMapSize(w, h);
    TValueLayer<int> layer(w, h);
    TValueLayer<point_t> movefromlayer(w, h);
    BattleIf->GetMovableLayerFrom(mi, chara, x, y, dist, layer, movefromlayer, loose);
    int count = 0;
    // Ve[uǉ
    lua_createtable(L, w * h, 0);
    for(int y=0; y<h; ++y){
        for(int x=0; x<w; ++x){
            // Ve[uǉ
            lua_newtable(L);
            // ړl
            lua_pushstring(L, "value");
            lua_pushnumber(L, layer.GetValue(x, y));
            lua_rawset(L, -3);
            // ړW
            lua_pushstring(L, "fromx");
            lua_pushnumber(L, movefromlayer.GetValue(x, y).x);
            lua_rawset(L, -3);
            lua_pushstring(L, "fromy");
            lua_pushnumber(L, movefromlayer.GetValue(x, y).y);
            lua_rawset(L, -3);
            lua_rawseti(L, 1, count++);
        }
    }
    return 1;
}
//---------------------------------------------------------------------------
int api_battle_get_neighbor( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 3 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2) ||
                 !lua_isnumber(L, 3)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int radius = lua_tonumber(L, 3);
    if(radius > TNeighbor::NEIGHBOR_MAX){
        g_Logger.ErrorLog("api_get_neighbor too large radius.");
        lua_settop(L, 0);
        return 0;
    }
    int x = lua_tonumber(L, 1);
    int y = lua_tonumber(L, 2);

    // X^bNNA
    lua_settop(L, 0);

    int w, h;
    BattleIf->GetMapSize(w, h);
    int index = 0;
    int from = TNeighbor::GetNeighborIndex(0);
    int to = TNeighbor::GetNeighborIndex(radius + 1);
    int mode = y%2;
    // Ve[uǉ
    lua_createtable(L, to - from, 0);
    for(int i=from; i<to; ++i){
        point_t p = TNeighbor::GetNeighbor(mode, i);
        int nx = x + p.x;
        int ny = y + p.y;
        if(nx >= 0 && nx < w && ny >= 0 && ny < h){
            // Ve[uǉ
            lua_newtable(L);
            lua_pushstring(L, "x");
            lua_pushnumber(L, x + p.x);
            lua_rawset(L, -3);
            lua_pushstring(L, "y");
            lua_pushnumber(L, y + p.y);
            lua_rawset(L, -3);
            lua_rawseti(L, 1, ++index);
        }
    }
    return 1;
}
//---------------------------------------------------------------------------
int api_battle_get_neighbor_sorted( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 3 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2) ||
                 !lua_isnumber(L, 3)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, "api_get_neighbor_sorted");
        return 0;
    }

    int radius = lua_tonumber(L, 3);
    if(radius > TNeighbor::NEIGHBOR_MAX){
        g_Logger.ErrorLog("api_get_neighbor_sorted too large radius.");
        lua_settop(L, 0);
        return 0;
    }
    int x = lua_tonumber(L, 1);
    int y = lua_tonumber(L, 2);

    // X^bNNA
    lua_settop(L, 0);

    int w, h;
    BattleIf->GetMapSize(w, h);
    std::vector<point_t> plist;
    TNeighbor::GetRadius(x, y, w, h, radius, plist);
    std::sort(plist.begin(), plist.end(), TPointLess());
    int i = 0;
    // Ve[uǉ
    lua_createtable(L, plist.size(), 0);
    for(std::vector<point_t>::const_iterator itr = plist.begin(); itr != plist.end(); ++itr){
        // Ve[uǉ
        lua_newtable(L);
        lua_pushstring(L, "x");
        lua_pushnumber(L, itr->x);
        lua_rawset(L, -3);
        lua_pushstring(L, "y");
        lua_pushnumber(L, itr->y);
        lua_rawset(L, -3);
        lua_rawseti(L, 1, ++i);
    }
    return 1;
}
//---------------------------------------------------------------------------
// ߂lF}X^[ǂ
int api_battle_get_is_defense_side( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 1 || !lua_isnumber(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int mi = lua_tonumber(L, 1);
    // X^bNNA
    lua_settop(L, 0);
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but not battle.") % FUNCNAME).str());
        return 0;
    }
    // }X^[CfbNX0ȂB
    lua_pushboolean(L, mi == 0);
    return 1;
}
//---------------------------------------------------------------------------
// ߂lF퓬Q}X^[CfbNX̃Xg
int api_battle_get_master_list( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but not battle.") % FUNCNAME).str());
        return 0;
    }
    lua_newtable(L);
    lua_pushnumber(L, BattleMasterIndex);
    lua_rawseti(L, 1, 1);
    lua_pushnumber(L, !BattleMasterIndex);
    lua_rawseti(L, 1, 2);
    return 1;
}
//---------------------------------------------------------------------------
// ߂lF퓬}X^[CfbNX
int api_battle_get_my_master( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but not battle.") % FUNCNAME).str());
        return 0;
    }
    lua_pushnumber(L, BattleMasterIndex);
    return 1;
}
//---------------------------------------------------------------------------
// ߂lFG퓬}X^[CfbNX̃Xg
int api_battle_get_enemy_master( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but not battle.") % FUNCNAME).str());
        return 0;
    }
    // Ve[uǉ
    lua_newtable(L);
    lua_pushnumber(L, !BattleMasterIndex);
    lua_rawseti(L, 1, 1);
    return 1;
}
//---------------------------------------------------------------------------
// ߂lFL
int api_battle_get_chara_count( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 1 || !lua_isnumber(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int mi = lua_tonumber(L, 1);
    // X^bNNA
    lua_settop(L, 0);
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but not battle.") % FUNCNAME).str());
        return 0;
    }

    // L擾
    lua_pushnumber(L, BattleIf->GetCharaCount(mi));
    return 1;
}
//---------------------------------------------------------------------------
// ߂lF퓬}X^[CfbNXALCfbNX
int api_battle_get_active_chara( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog("api_battle_get_active_chara was called, but not battle.");
        return 0;
    }

    // ANeBuL擾
    int mi, ci;
    BattleIf->GetActiveChara(mi, ci);
    lua_pushnumber(L, mi);
    lua_pushnumber(L, ci);
    return 2;
}
//---------------------------------------------------------------------------
// ߂lF퓬}X^[CfbNXALCfbNX
int api_battle_get_chara_on( lua_State *L )
{
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog("api_battle_get_chara_on was called, but not battle.");
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 2 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, "api_battle_get_chara_on");
        return 0;
    }

    int x = lua_tonumber(L, 1);
    int y = lua_tonumber(L, 2);
    // X^bNNA
    lua_settop(L, 0);
    // L擾
    int mi, ci;
    if(BattleIf->GetCharaOn(x, y, mi, ci)){
        lua_pushnumber(L, mi);
        lua_pushnumber(L, ci);
        return 2;
    }
    else{
        return 0;
    }
}
//---------------------------------------------------------------------------
int api_battle_get_chara_status( lua_State *L )
{
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog("api_battle_get_chara_status was called, but not battle.");
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 2 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, "api_battle_get_chara_status");
        return 0;
    }

    int mi = lua_tonumber(L, 1);
    int ci = lua_tonumber(L, 2);
    // X^bNNA
    lua_settop(L, 0);
    // LԎ擾
    TCharacterInstance chara;
    BattleIf->GetCharaStatus(mi, ci, chara);
    // Ve[uǉ
    lua_newtable(L);
    // W
    lua_pushstring(L, "x");
    lua_pushnumber(L, chara.m_location.x);
    lua_rawset(L, -3);
    lua_pushstring(L, "y");
    lua_pushnumber(L, chara.m_location.y);
    lua_rawset(L, -3);
    // HPȂ
    lua_pushstring(L, "hp");
    lua_pushnumber(L, chara.m_hp);
    lua_rawset(L, -3);
    lua_pushstring(L, "hpmax");
    lua_pushnumber(L, chara.m_hpmax_rank);
    lua_rawset(L, -3);
    lua_pushstring(L, "mp");
    lua_pushnumber(L, chara.m_mp);
    lua_rawset(L, -3);
    lua_pushstring(L, "mpmax");
    lua_pushnumber(L, chara.m_mpmax_rank);
    lua_rawset(L, -3);
    lua_pushstring(L, "pa");
    lua_pushnumber(L, chara.m_pa_rank);
    lua_rawset(L, -3);
    lua_pushstring(L, "pd");
    lua_pushnumber(L, chara.m_pd_rank);
    lua_rawset(L, -3);
    lua_pushstring(L, "skill");
    lua_pushnumber(L, chara.m_skill_rank);
    lua_rawset(L, -3);
    lua_pushstring(L, "speed");
    lua_pushnumber(L, chara.m_speed_rank);
    lua_rawset(L, -3);
    lua_pushstring(L, "ma");
    lua_pushnumber(L, chara.m_ma_rank);
    lua_rawset(L, -3);
    lua_pushstring(L, "md");
    lua_pushnumber(L, chara.m_md_rank);
    lua_rawset(L, -3);
    lua_pushstring(L, "brave");
    lua_pushnumber(L, chara.m_brave);
    lua_rawset(L, -3);
    lua_pushstring(L, "power");
    lua_pushnumber(L, chara.m_power);
    lua_rawset(L, -3);
    lua_pushstring(L, "element");
    lua_pushnumber(L, chara.m_elementstatus);
    lua_rawset(L, -3);
    lua_pushstring(L, "movetype");
    lua_pushnumber(L, chara.m_movetype);
    lua_rawset(L, -3);
    // 
    lua_pushstring(L, "pa_plus");
    lua_pushnumber(L, chara.m_pa_plus);
    lua_rawset(L, -3);
    lua_pushstring(L, "pd_plus");
    lua_pushnumber(L, chara.m_pd_plus);
    lua_rawset(L, -3);
    lua_pushstring(L, "skill_plus");
    lua_pushnumber(L, chara.m_skill_plus);
    lua_rawset(L, -3);
    lua_pushstring(L, "speed_plus");
    lua_pushnumber(L, chara.m_speed_plus);
    lua_rawset(L, -3);
    lua_pushstring(L, "ma_plus");
    lua_pushnumber(L, chara.m_ma_plus);
    lua_rawset(L, -3);
    lua_pushstring(L, "md_plus");
    lua_pushnumber(L, chara.m_md_plus);
    lua_rawset(L, -3);
    lua_pushstring(L, "hprecover_plus");
    lua_pushnumber(L, chara.m_hprecover_plus);
    lua_rawset(L, -3);
    lua_pushstring(L, "mprecover_plus");
    lua_pushnumber(L, chara.m_mprecover_plus);
    lua_rawset(L, -3);
    lua_pushstring(L, "attackcount_plus");
    lua_pushnumber(L, chara.m_attackcount_plus);
    lua_rawset(L, -3);
    lua_pushstring(L, "movable_plus");
    lua_pushnumber(L, chara.m_movable_plus);
    lua_rawset(L, -3);
    // Xe[^Xُizj
    lua_pushstring(L, "status");
    lua_newtable(L);
    for(unsigned u=0; u<chara.m_seslist.size(); ++u){
        lua_pushnumber(L, u);
        //lua_pushnumber(L, static_cast<int>(chara.m_seslist[u]));
        lua_pushnumber(L, chara.m_seslist[u].m_type);
        lua_rawset(L, -3);
    }
    lua_rawset(L, -3);
    // ϐizj
    lua_pushstring(L, "resistance");
    lua_newtable(L);
    for(unsigned u=0; u<chara.m_resistance.size(); ++u){
        lua_pushnumber(L, u);
        lua_pushnumber(L, chara.m_resistance[u]);
        lua_rawset(L, -3);
    }
    lua_rawset(L, -3);
    // ŗL
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
    lua_pushstring(L, "attackcount");
    lua_pushnumber(L, charadata.m_attacklist.size());
    lua_rawset(L, -3);
    lua_pushstring(L, "morale");
    lua_pushnumber(L, charadata.m_morale);
    lua_rawset(L, -3);
    lua_pushstring(L, "aggressive");
    lua_pushnumber(L, charadata.m_aggressive);
    lua_rawset(L, -3);
    lua_pushstring(L, "master");
    lua_pushboolean(L, charadata.m_master);
    lua_rawset(L, -3);
    return 1;
}
//---------------------------------------------------------------------------
int api_battle_get_action_list( lua_State *L )
{
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog("api_get_action_list was called, but not battle.");
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 2 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, "api_get_action_list");
        return 0;
    }

    int movex = lua_tonumber(L, 1);
    int movey = lua_tonumber(L, 2);
    // X^bNNA
    lua_settop(L, 0);

    // \ȍs擾
    TActionList actlist;
    std::vector<point_t> targetlist;
    BattleIf->GetActionList(movex, movey, actlist, targetlist);
    // Ve[uǉ
    lua_newtable(L);
    for(unsigned u=0; u<actlist.size(); ++u){
        lua_pushnumber(L, u+1);
        // Ve[uǉ
        lua_newtable(L);
        // 
        lua_pushstring(L, "actiontype");
        lua_pushnumber(L, actlist[u].acttype);
        lua_rawset(L, -3);
        // W
        lua_pushstring(L, "targetx");
        lua_pushnumber(L, actlist[u].location.x);
        lua_rawset(L, -3);
        lua_pushstring(L, "targety");
        lua_pushnumber(L, actlist[u].location.y);
        lua_rawset(L, -3);
        // KEZ^@CfbNX
        lua_pushstring(L, "special");
        lua_pushnumber(L, actlist[u].special);
/*if(actlist[u].acttype == ACTION_SPECIAL){
const TOpenFarenData* data = g_GameMaster.GetData();
const TSpecial& spe = data->GetSpecial(actlist[u].special);
g_Logger.DebugLog((boost::format("OOO %s") % spe.m_id).str());
}*/
        lua_rawset(L, -3);
        lua_rawset(L, 1);
    }
    return 1;
}
//---------------------------------------------------------------------------
int api_battle_get_last_target( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog("api_battle_get_last_target was called, but not battle.");
        return 0;
    }

    int x, y;
    BattleIf->GetLastTarget(x, y);
    lua_pushnumber(L, x);
    lua_pushnumber(L, y);
    return 2;
}
//---------------------------------------------------------------------------
void put_action_result(lua_State *L, const TEvalResultList& evallist)
{
    const TOpenFarenData* data = g_GameMaster.GetData();
    // Ve[uǉ
    lua_newtable(L);
    for(unsigned u=0; u<evallist.size(); ++u){
        lua_pushnumber(L, u+1);
        // Ve[uǉ
        lua_newtable(L);
        // W
        lua_pushstring(L, "x");
        lua_pushnumber(L, evallist[u].location.x);
        lua_rawset(L, -3);
        lua_pushstring(L, "y");
        lua_pushnumber(L, evallist[u].location.y);
        lua_rawset(L, -3);
        // WI
        lua_pushstring(L, "masterindex");
        lua_pushnumber(L, evallist[u].masterindex);
        lua_rawset(L, -3);
        lua_pushstring(L, "charaindex");
        lua_pushnumber(L, evallist[u].charaindex);
        lua_rawset(L, -3);
        // ʒl
        lua_pushstring(L, "value");
        lua_pushnumber(L, evallist[u].value);
        lua_rawset(L, -3);
        if(evallist[u].systemeffect == SYSTEM_REVIVE){
            lua_pushstring(L, "revivecharaid");
            lua_pushstring(L, data->GetCharacter(evallist[u].revivecharatype).m_id.c_str());
            lua_rawset(L, -3);
        }
        else if(evallist[u].systemeffect == SYSTEM_SUMMON){
            lua_pushstring(L, "summoncharaid");
            lua_pushstring(L, data->GetCharacter(evallist[u].summoncharatype).m_id.c_str());
            lua_rawset(L, -3);
        }
        // 
        lua_pushstring(L, "systemeffect");
        lua_pushnumber(L, evallist[u].systemeffect);
        lua_rawset(L, -3);
        // ϐύX
        lua_pushstring(L, "resattr");
        lua_pushnumber(L, evallist[u].resattr);
        lua_rawset(L, -3);
        lua_pushstring(L, "restype");
        lua_pushnumber(L, evallist[u].restype);
        lua_rawset(L, -3);
        // ړ^CvύX
        lua_pushstring(L, "movetype");
        lua_pushnumber(L, evallist[u].movetype);
        lua_rawset(L, -3);
        lua_rawset(L, 1);
    }
}
//---------------------------------------------------------------------------
int api_battle_eval_action( lua_State *L )
{
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but not battle.") % FUNCNAME).str());
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 6 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2) ||
                 !lua_isnumber(L, 3) || !lua_isnumber(L, 4) ||
                 !lua_isnumber(L, 5) || !lua_isnumber(L, 6)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int movex = lua_tonumber(L, 1);
    int movey = lua_tonumber(L, 2);
    TActionType acttype = static_cast<TActionType>(static_cast<int>(lua_tonumber(L, 3)));
    int special = lua_tonumber(L, 4);
    int targetx = lua_tonumber(L, 5);
    int targety = lua_tonumber(L, 6);
    // X^bNNA
    lua_settop(L, 0);
    TEvalResultList evallist;
    if(!BattleIf->EvalAction(movex, movey, acttype, special, targetx, targety, evallist)){
        return 0;
    }
    put_action_result(L, evallist);
    return 1;
}
//---------------------------------------------------------------------------
int api_battle_set_action( lua_State *L )
{
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but not battle.") % FUNCNAME).str());
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 6 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2) ||
                 !lua_isnumber(L, 3) || !lua_isnumber(L, 4) ||
                 !lua_isnumber(L, 5) || !lua_isnumber(L, 6)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int movex = lua_tonumber(L, 1);
    int movey = lua_tonumber(L, 2);
    int acttype = lua_tonumber(L, 3);
    int special = lua_tonumber(L, 4);
    int targetx = lua_tonumber(L, 5);
    int targety = lua_tonumber(L, 6);
    if(acttype < ACTION_WAIT || acttype > ACTION_RUNAWAY){
        g_Logger.ErrorLog((boost::format("api_set_action error:ȍs^Cv %d w肳܂B") % acttype).str());
    }

    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, BattleIf->SetAction(movex, movey, static_cast<TActionType>(acttype), special, targetx, targety));
    return 1;
}
//---------------------------------------------------------------------------
int api_battle_get_auto_runaway( lua_State *L )
{
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but not battle.") % FUNCNAME).str());
        lua_settop(L, 0);
        return 0;
    }

    // X^bNNA
    lua_settop(L, 0);

    point_t p;
    if(!BattleIf->GetAutoRunAwayPoint(p)){
        return 0;
    }
    lua_pushnumber(L, p.x);
    lua_pushnumber(L, p.y);
    return 2;
}
//---------------------------------------------------------------------------
int api_battle_set_chara_brave( lua_State *L )
{
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but not battle.") % FUNCNAME).str());
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 3 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2) || !lua_isnumber(L, 3)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }
    int mi = lua_tonumber(L, 1);
    int ci = lua_tonumber(L, 2);
    int brave = lua_tonumber(L, 3);
    // LmCݒ
    BattleIf->SetCharaBrave(mi, ci, brave);
    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, 1);
    return 1;
}
//---------------------------------------------------------------------------
int api_battle_set_map_chip( lua_State *L )
{
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog("api_set_map_chip was called, but not battle.");
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 3 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2) ||
                 !lua_isnumber(L, 3)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, "api_set_map_chip");
        return 0;
    }

    int x = lua_tonumber(L, 1);
    int y = lua_tonumber(L, 2);
    int chip = lua_tonumber(L, 3);
    BattleIf->SetMapChip(x, y, chip);

    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, 1);
    return 1;
}
//---------------------------------------------------------------------------
int api_battle_rm_map_obj( lua_State *L )
{
    // 퓬ł邱ƂmFB
    if(!BattleIf->IsBattle()){
        g_Logger.ErrorLog("api_battle_rm_map_obj was called, but not battle.");
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n < 2 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, "api_battle_rm_map_obj");
        return 0;
    }

    int x = lua_tonumber(L, 1);
    int y = lua_tonumber(L, 2);
    if(n > 2){
        if(!lua_isnumber(L, 3)){
            lua_settop(L, 0);
            luautil_arg_error_log(L, "api_battle_rm_map_obj 2");
            return 0;
        }
        TMapObject objtype = static_cast<TMapObject>(static_cast<int>(lua_tonumber(L, 3)));
        BattleIf->RemoveMapObjSpecify(x, y, objtype);
    }
    else{
        BattleIf->RemoveMapObj(x, y);
    }

    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, 1);
    return 1;
}
//---------------------------------------------------------------------------
int api_get_current_master( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    const TOpenFarenData *data = g_GameMaster.GetData();
    const TMasterInstance& masterins = game->GetMasterInstance(game->GetCurrentMasterIndex());
    lua_pushstring(L, data->GetMaster(masterins.m_type).m_id.c_str());
    return 1;
}
//---------------------------------------------------------------------------
/*int api_get_exists_master( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    // Ve[uǉ
    lua_newtable(L);
    int sz = game->GetMasterInstanceCount();
    for(int i=0; i<sz; ++i){
        const TMasterInstance& masterins = game->GetMasterInstance(i);
        if(masterins.m_exists){
            lua_pushnumber(L, i);
            lua_rawset(L, -3);
        }
    }
    return 1;
}*/
//---------------------------------------------------------------------------
int api_change_master_type( lua_State *L )
{
//g_Logger.DebugLog((boost::format("api_change_master_type 1")).str());
    int n = lua_gettop(L);

    if(n != 3 || !lua_isstring(L, 1) || !lua_isstring(L, 2) || !lua_isstring(L, 3)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, "api_change_master_type");
        return 0;
    }

    std::string masterid1 = lua_tostring(L, 1);
    std::string masterid2 = lua_tostring(L, 2);
    std::string charaid = lua_tostring(L, 3);

    const TOpenFarenData *data = g_GameMaster.GetData();
    TOpenFarenGame *game = g_GameMaster.GetGame();

    int mastertype1 = data->GetMasterIndex(masterid1);
    int mastertype2 = data->GetMasterIndex(masterid2);
    int charatype = data->GetCharacterIndex(charaid);

    //int master1 = StrategyIf->GetMasterIndex(masterid1);
    //int master2 = StrategyIf->GetMasterIndex(masterid2);
    //int charatype = StrategyIf->GetCharaType(charaid);
    if(mastertype1 == -1 || mastertype2 == -1 || charatype == -1){
        lua_settop(L, 0);
        g_Logger.ErrorLog((boost::format("api_change_master_type(%s, %s, %s) error:ȃ}X^[ID܂̓LIDw肳܂B") % masterid1 % masterid2 % charaid).str());
        return 0;
    }
    int masterindex1 = game->GetMasterInstanceIndex(mastertype1);
    if(masterindex1 == -1){
        lua_settop(L, 0);
        g_Logger.ErrorLog((boost::format("api_change_master_type(%s, %s, %s) error:ύXÕ}X^[݂܂B") % masterid1 % masterid2 % charaid).str());
        return 0;
    }
    // X^bNNA
    lua_settop(L, 0);
    // }X^[ύX
    game->ChangeMasterType(masterindex1, mastertype2, charatype);
    // 퓬̏ꍇ͐퓬IFɂ}X^[`FWKpB
    if(!BattleIf->IsBattle()){
        int bmaster = BattleIf->FindMasterIndex(masterindex1);
        if(bmaster != -1){
            // pL퓬ɎQĂꍇ̓}X^[LɕύXBiBattleIf̃o֐̒łHj
            int bchara;
            if(BattleIf->GetChara(bmaster, charatype, bchara)){
                const TMaster& master = data->GetMaster(mastertype2);
                BattleIf->ChangeCharaType(bmaster, bchara, master.m_facecharaindex);
            }
        }
    }
    return 0;
}
//---------------------------------------------------------------------------
int api_get_master( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string masterid = lua_tostring(L, 1);
    const TOpenFarenData *data = g_GameMaster.GetData();
    int mastertype = data->GetMasterIndex(masterid);
    if(mastertype == -1){
        lua_settop(L, 0);
        g_Logger.ErrorLog((boost::format("%s(%s) error:}X^[IDĂ܂B") % FUNCNAME % masterid).str());
        return 0;
    }

    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int masterindex = game->GetMasterInstanceIndex(mastertype);
    if(masterindex == -1){
        lua_settop(L, 0);
        g_Logger.ErrorLog((boost::format("%s(%s) error:}X^[݂܂Bmastertype=%d") % FUNCNAME % masterid % mastertype).str());
        return 0;
    }

    const TMasterInstance& masterins = game->GetMasterInstance(masterindex);
    const TMaster& master = data->GetMaster(masterins.m_type);
    // X^bNNA
    lua_settop(L, 0);
    // Ve[uǉ
    lua_newtable(L);
    // 
    lua_pushstring(L, "money");
    lua_pushnumber(L, masterins.m_money);
    lua_rawset(L, -3);
    // vC[tO
    lua_pushstring(L, "player");
    lua_pushboolean(L, masterins.m_player);
    lua_rawset(L, -3);
    // O
    lua_pushstring(L, "name");
    lua_pushstring(L, master.m_name.c_str());
    lua_rawset(L, -3);
    return 1;
}
//---------------------------------------------------------------------------
int api_last_dead_master( lua_State *L )
{
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    std::vector<int> lostmaster;
    game->JudgeLostMaster(lostmaster);
    const TOpenFarenData *data = g_GameMaster.GetData();
    // X^bNNA
    lua_settop(L, 0);
    // Ve[uǉ
    lua_newtable(L);
    for(unsigned u=0; u<lostmaster.size(); ++u){
        const TMasterInstance& masterins = game->GetMasterInstance(lostmaster[u]);
        const TMaster& master = data->GetMaster(masterins.m_type);
        lua_pushnumber(L, u+1);
        lua_pushstring(L, master.m_id.c_str());
        lua_rawset(L, -3);
    }
    return 1;
}
//---------------------------------------------------------------------------
int api_get_chara_type( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, "api_get_chara_type");
        return 0;
    }

    std::string charaid = lua_tostring(L, 5);
    const TOpenFarenData* data = g_GameMaster.GetData();
    int charatype = data->GetCharacterIndex(charaid);
    if(charatype == -1){
        g_Logger.ErrorLog("api_get_chara_type invalid character id.");
        lua_settop(L, 0);
        return 0;
    }

    // X^bNNA
    lua_settop(L, 0);
    lua_pushnumber(L, charatype);
    return 1;
}
//---------------------------------------------------------------------------
/*// H
int api_exists_chara( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 2 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, "api_exists_chara");
        return 0;
    }

    int master = lua_tonumber(L, 1);
    int charatype = lua_tonumber(L, 2);
    // X^bNNA
    lua_settop(L, 0);
    // L擾
    int areaindex, charaindex;
    //lua_pushboolean(L, StrategyIf->FindCharaExistsArea(master, charatype, areaindex, charaindex));
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    lua_pushboolean(L, game->FindCharaExistsArea(master, charatype, areaindex, charaindex));
    return 1;
}*/
//---------------------------------------------------------------------------
int api_exists_hero( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 2 || !lua_isstring(L, 1) || !lua_isstring(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, "api_exists_hero");
        return 0;
    }

    std::string masterid = lua_tostring(L, 1);
    std::string charaid = lua_tostring(L, 2);
    const TOpenFarenData *data = g_GameMaster.GetData();
    int mastertype = data->GetMasterIndex(masterid);
    int charatype = data->GetCharacterIndex(charaid);
    if(mastertype == -1 || charatype == -1){
        g_Logger.ErrorLog((boost::format("api_exists_hero error:ID %s %s w肳܂B") % masterid % charaid).str());
        lua_settop(L, 0);
        return 0;
    }
    // X^bNNA
    lua_settop(L, 0);
    // L擾
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int masterindex = game->GetMasterInstanceIndex(mastertype);
    if(masterindex == -1){
        g_Logger.ErrorLog((boost::format("api_exists_hero error:}X^[ %s ݂܂B") % masterid).str());
        return 0;
    }
g_Logger.DebugLog((boost::format("api_exists_hero 1")).str());
    int areaindex, charaindex;
    if(game->FindCharaExistsArea(masterindex, charatype, areaindex, charaindex)){
g_Logger.DebugLog((boost::format("api_exists_hero 2")).str());
        bool exists = true;
        if(BattleIf->IsBattle()){
g_Logger.DebugLog((boost::format("api_exists_hero 3")).str());
            int bmaster = BattleIf->FindMasterIndex(masterindex);
            if(bmaster != -1){
g_Logger.DebugLog((boost::format("api_exists_hero 4")).str());
                int bcharaindex;
                if(BattleIf->GetChara(bmaster, charatype, bcharaindex)){
g_Logger.DebugLog((boost::format("api_exists_hero 5")).str());
                    // LԎ擾
                    TCharacterInstance chara;
                    BattleIf->GetCharaStatus(bmaster, bcharaindex, chara);
                    exists = chara.m_hp > 0;
                }
            }
        }
        lua_pushboolean(L, exists);
    }
    else{
        lua_pushboolean(L, false);
    }
g_Logger.DebugLog((boost::format("api_exists_hero 6")).str());
    return 1;
}
//---------------------------------------------------------------------------
// yĐ
int api_music_set( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, "api_music_set");
        return 0;
    }

    std::string filename = lua_tostring(L, 1);
    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, ScriptAPI->PlayMusic(filename));
    return 1;
}
//---------------------------------------------------------------------------
int api_gui_get_top_size( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    int w, h;
    WorldMapLogic->GetTopSize(w, h);
    lua_pushnumber(L, w);
    lua_pushnumber(L, h);
    return 2;
}
//---------------------------------------------------------------------------
int api_gui_event_pop( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    TScriptEvent ev;
    if(ScriptAPI->PopEvent(ev)){
        // Ve[uǉ
        lua_newtable(L);
        lua_pushstring(L, "x");
        lua_pushnumber(L, ev.x);
        lua_rawset(L, -3);
        lua_pushstring(L, "y");
        lua_pushnumber(L, ev.y);
        lua_rawset(L, -3);
        lua_pushstring(L, "eventtype");
        lua_pushnumber(L, ev.eventtype);
        lua_rawset(L, -3);
        return 1;
    }
    else{
        return 0;
    }
}
//---------------------------------------------------------------------------
int api_window_new( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    lua_pushnumber(L, ScriptAPI->NewWindow());
    return 1;
}
//---------------------------------------------------------------------------
int api_window_set_size( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 3 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2) || !lua_isnumber(L, 3)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int handle = lua_tonumber(L, 1);
    if(!ScriptAPI->IsValidWindow(handle)){
        g_Logger.ErrorLog("api_window_set_size invalid window handle.");
        lua_settop(L, 0);
        return 0;
    }
    int w = lua_tonumber(L, 2);
    int h = lua_tonumber(L, 3);
    ScriptAPI->SetWindowSize(handle, w, h);
    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, true);
    return 1;
}
//---------------------------------------------------------------------------
int api_window_set_position( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 3 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2) || !lua_isnumber(L, 3)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int handle = lua_tonumber(L, 1);
    if(!ScriptAPI->IsValidWindow(handle)){
        g_Logger.ErrorLog("api_window_set_position invalid window handle.");
        lua_settop(L, 0);
        return 0;
    }
    int x = lua_tonumber(L, 2);
    int y = lua_tonumber(L, 3);
    ScriptAPI->SetWindowPosition(handle, x, y);
    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, true);
    return 1;
}
//---------------------------------------------------------------------------
int api_window_set_visible( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 2 || !lua_isnumber(L, 1) || !lua_isboolean(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int handle = lua_tonumber(L, 1);
    if(!ScriptAPI->IsValidWindow(handle)){
        g_Logger.ErrorLog("api_window_set_visible invalid window handle.");
        lua_settop(L, 0);
        return 0;
    }
    if(lua_toboolean(L, 2)){
        ScriptAPI->ShowWindow(handle);
    }
    else{
        ScriptAPI->HideWindow(handle);
    }
    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, true);
    return 1;
}
//---------------------------------------------------------------------------
int api_window_set_titlebarheight( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 2 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2)){
        lua_settop(L, 0);
        error_log_withinfo(L, (boost::format("%s invalid argument type.") % FUNCNAME).str().c_str());
        return 0;
    }

    int handle = lua_tonumber(L, 1);
    if(!ScriptAPI->IsValidWindow(handle)){
        g_Logger.ErrorLog("api_window_set_titlebarheight invalid window handle.");
        lua_settop(L, 0);
        return 0;
    }
    int height = lua_tonumber(L, 2);
    if(height < 0 || height > 64){
        g_Logger.ErrorLog((boost::format("%s invalid window titlebar height %d. (0-64))") % FUNCNAME % height).str());
        lua_settop(L, 0);
        return 0;
    }
    TEventWindow* winp = ScriptAPI->GetWindow(handle);
    if(winp == NULL){
        g_Logger.ErrorLog((boost::format("%s internal error(invalid window)") % FUNCNAME % height).str());
        lua_settop(L, 0);
        return 0;
    }
    winp->setTitleBarHeight(height);
    winp->resizeToContent();
    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, true);
    return 1;
}
//---------------------------------------------------------------------------
int api_window_set_image( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 6 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2) || !lua_isnumber(L, 3) ||
                 !lua_isnumber(L, 4) || !lua_isnumber(L, 5) || !lua_isnumber(L, 6)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int handle = lua_tonumber(L, 1);
    if(!ScriptAPI->IsValidWindow(handle)){
        g_Logger.ErrorLog("api_window_set_image invalid window handle.");
        lua_settop(L, 0);
        return 0;
    }
    int x = lua_tonumber(L, 2);
    int y = lua_tonumber(L, 3);
    int layer = lua_tonumber(L, 4);
    TImageType imgtype = static_cast<TImageType>(static_cast<int>(lua_tonumber(L, 5)));
    int img;

    if(imgtype == IMAGE_FACE || imgtype == IMAGE_CHARACHIP){
        const TOpenFarenData* data = g_GameMaster.GetData();
        // L֌W摜̃CfbNX擾B
        int charatype = lua_tonumber(L, 6);
        if(charatype < 0 || charatype >= data->GetCharacterCount()){
            g_Logger.ErrorLog("api_window_set_image character type out of range.");
            lua_settop(L, 0);
            return 0;
        }
        const TBattleCharacter& charadata = data->GetCharacter(charatype);
        if(imgtype == IMAGE_CHARACHIP){
            img = charadata.m_chipindex;
        }
        else{
            img = charadata.m_faceindex;
        }
    }
    else{
        img = lua_tonumber(L, 6);
    }

    ScriptAPI->AddImage(handle, x, y, layer, imgtype, img);
    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, true);
    return 1;
}
//---------------------------------------------------------------------------
int api_window_clear_image( lua_State *L )
{
    int n = lua_gettop(L);

    if((n != 1 || !lua_isnumber(L, 1)) &&
       (n != 2 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2))){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int handle = lua_tonumber(L, 1);
    if(!ScriptAPI->IsValidWindow(handle)){
        g_Logger.ErrorLog("api_window_clear_image invalid window handle.");
        lua_settop(L, 0);
        return 0;
    }
    int layer = -1;
    if(n == 2){
        layer = lua_tonumber(L, 2);
    }
    ScriptAPI->ClearImage(handle, layer);
    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, true);
    return 1;
}
//---------------------------------------------------------------------------
int api_window_set_faceorchip( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 5 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2) ||
                 !lua_isnumber(L, 3) || !lua_isnumber(L, 4) ||
                 !lua_isstring(L, 5)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int handle = lua_tonumber(L, 1);
    if(!ScriptAPI->IsValidWindow(handle)){
        g_Logger.ErrorLog((boost::format("%s invalid window handle.") % FUNCNAME).str());
        lua_settop(L, 0);
        return 0;
    }
    int x = lua_tonumber(L, 2);
    int y = lua_tonumber(L, 3);
    int layer = lua_tonumber(L, 4);
    std::string charaid = lua_tostring(L, 5);

    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenData* data = g_GameMaster.GetData();
    int charatype = data->GetCharacterIndex(charaid);
    if(charatype == -1){
        g_Logger.ErrorLog((boost::format("%s invalid character id.") % FUNCNAME).str());
        return 0;
    }
    const TBattleCharacter& charadata = data->GetCharacter(charatype);
//    if(charadata.m_faceindex != -1){
//        ScriptAPI->AddImage(handle, x, y+20, layer, IMAGE_FACE, charadata.m_faceindex);
//    }
//    else{
//        ScriptAPI->AddImage(handle, x, y+20, layer, IMAGE_CHARACHIP, charadata.m_chipindex);
//    }
    ScriptAPI->DrawCharaFaceOrChip(handle, x, y+20, layer, charadata.m_faceindex, charadata.m_chipindex);
    ScriptAPI->AddString(handle, x, y, layer, charadata.m_name);
    lua_pushboolean(L, true);
    return 1;
}
//---------------------------------------------------------------------------
int api_window_set_text( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 5 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2) || !lua_isnumber(L, 3) ||
                 !lua_isnumber(L, 4) || !lua_isstring(L, 5)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int handle = lua_tonumber(L, 1);
    if(!ScriptAPI->IsValidWindow(handle)){
        g_Logger.ErrorLog("api_window_set_text invalid window handle.");
        lua_settop(L, 0);
        return 0;
    }
    int x = lua_tonumber(L, 2);
    int y = lua_tonumber(L, 3);
    int layer = lua_tonumber(L, 4);
    std::string str = lua_tostring(L, 5);
    ScriptAPI->AddString(handle, x, y, layer, str);
    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, true);
    return 1;
}
//---------------------------------------------------------------------------
int api_window_clear_text( lua_State *L )
{
    int n = lua_gettop(L);

    if((n != 1 || !lua_isnumber(L, 1)) &&
       (n != 2 || !lua_isnumber(L, 1) || !lua_isnumber(L, 2))){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int handle = lua_tonumber(L, 1);
    if(!ScriptAPI->IsValidWindow(handle)){
        g_Logger.ErrorLog("api_window_clear_text invalid window handle.");
        lua_settop(L, 0);
        return 0;
    }
    int layer = -1;
    if(n == 2){
        layer = lua_tonumber(L, 2);
    }
    ScriptAPI->ClearString(handle, layer);
    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, true);
    return 1;
}
//---------------------------------------------------------------------------
int api_window_is_visible( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 1 || !lua_isnumber(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int handle = lua_tonumber(L, 1);
    if(!ScriptAPI->IsValidWindow(handle)){
        g_Logger.ErrorLog("api_window_is_visible invalid window handle.");
        lua_settop(L, 0);
        return 0;
    }
    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, ScriptAPI->IsVisibleWindow(handle));
    return 1;
}
//---------------------------------------------------------------------------
int api_window_event_pop( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 1 || !lua_isnumber(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int handle = lua_tonumber(L, 1);
    if(!ScriptAPI->IsValidWindow(handle)){
        g_Logger.ErrorLog("api_window_event_pop invalid window handle.");
        lua_settop(L, 0);
        return 0;
    }

    // X^bNNA
    lua_settop(L, 0);
    TScriptEvent ev;
    if(ScriptAPI->PopEvent(handle, ev)){
        // Ve[uǉ
        lua_newtable(L);
        lua_pushstring(L, "x");
        lua_pushnumber(L, ev.x);
        lua_rawset(L, -3);
        lua_pushstring(L, "y");
        lua_pushnumber(L, ev.y);
        lua_rawset(L, -3);
        lua_pushstring(L, "eventtype");
        lua_pushnumber(L, ev.eventtype);
        lua_rawset(L, -3);
        return 1;
    }
    else{
        return 0;
    }
}
//---------------------------------------------------------------------------
int api_dialog_show( lua_State *L )
{
    int n = lua_gettop(L);

    if(n < 3 || n > 4 || !lua_isstring(L, 1) || !lua_isstring(L, 2) || !lua_isnumber(L, 3)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, "api_dialog_show");
        return 0;
    }

    std::string title = lua_tostring(L, 1);
    std::string msg = lua_tostring(L, 2);
    int btn = lua_tonumber(L, 3);
    bool timeout = false;
    if(n == 4){
        if(!lua_isboolean(L, 4)){
            lua_settop(L, 0);
            luautil_arg_error_log(L, "api_dialog_show");
            return 0;
        }
        timeout = lua_toboolean(L, 4);
    }
    if(timeout){
        TGcnMsgDlg::Show(title, msg, btn, g_Ini.GetMessageTimeoutFrame());
    }
    else{
        TGcnMsgDlg::Show(title, msg, btn);
    }
    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, true);
    return 1;
}
//---------------------------------------------------------------------------
int api_dialog_is_visible( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    lua_pushboolean(L, TGcnMsgDlg::IsVisible());
    return 1;
}
//---------------------------------------------------------------------------
int api_dialog_get_selected( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    lua_pushnumber(L, TGcnMsgDlg::GetPressedButtonStatic());
    return 1;
}
//---------------------------------------------------------------------------
int api_image_load( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string imgfile = lua_tostring(L, 1);
    // X^bNNA
    lua_settop(L, 0);
    int index = ScriptAPI->LoadEventImage(imgfile);
    if(index == -1){
        return 0;
    }
    lua_pushnumber(L, index);
    return 1;
}
//---------------------------------------------------------------------------
int api_get_chapter_number( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    lua_pushnumber(L, game->GetChapterNumber() + 1);
    return 1;
}
//---------------------------------------------------------------------------
int api_get_turn( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    lua_pushnumber(L, game->GetTurn());
    return 1;
}
//---------------------------------------------------------------------------
int api_get_progressed_turn( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    lua_pushnumber(L, game->GetProgressedTurn());
    return 1;
}
//---------------------------------------------------------------------------
int api_find_alliance( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 2 || !lua_isstring(L, 1) || !lua_isstring(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string masterid1 = lua_tostring(L, 1);
    std::string masterid2 = lua_tostring(L, 2);
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenData *data = g_GameMaster.GetData();
    int mastertype1 = data->GetMasterIndex(masterid1);
    if(mastertype1 == -1){
        g_Logger.ErrorLog((boost::format("%s(%s, %s) error:}X^[ID1Ă܂B") % FUNCNAME % masterid1 % masterid2).str());
        return 0;
    }
    int mastertype2 = data->GetMasterIndex(masterid2);
    if(mastertype2 == -1){
        g_Logger.ErrorLog((boost::format("%s(%s, %s) error:}X^[ID2Ă܂B") % FUNCNAME % masterid1 % masterid2).str());
        return 0;
    }

    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int masterindex1 = game->GetMasterInstanceIndex(mastertype1);
    if(masterindex1 == -1){
        return 0;
    }
    int masterindex2 = game->GetMasterInstanceIndex(mastertype2);
    if(masterindex2 == -1){
        return 0;
    }
    int alindex = game->FindAlliance(masterindex1, masterindex2);
    if(alindex == -1){
        return 0;
    }
    const TAlliance& alliance = game->GetAlliance(alindex);
    lua_pushnumber(L, alliance.turn);
    return 1;
}
//---------------------------------------------------------------------------
int api_get_alliance_cost( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 2 || !lua_isstring(L, 1) || !lua_isstring(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string masterid1 = lua_tostring(L, 1);
    std::string masterid2 = lua_tostring(L, 2);
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenData *data = g_GameMaster.GetData();
    int mastertype1 = data->GetMasterIndex(masterid1);
    if(mastertype1 == -1){
        g_Logger.ErrorLog((boost::format("%s(%s, %s) error:}X^[ID1Ă܂B") % FUNCNAME % masterid1 % masterid2).str());
        return 0;
    }
    int mastertype2 = data->GetMasterIndex(masterid2);
    if(mastertype2 == -1){
        g_Logger.ErrorLog((boost::format("%s(%s, %s) error:}X^[ID2Ă܂B") % FUNCNAME % masterid1 % masterid2).str());
        return 0;
    }

    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int masterindex1 = game->GetMasterInstanceIndex(mastertype1);
    if(masterindex1 == -1){
        g_Logger.ErrorLog((boost::format("%s(%s, %s) error:}X^[ID1̃}X^[݂܂B") % FUNCNAME % masterid1 % masterid2).str());
        return 0;
    }
    int masterindex2 = game->GetMasterInstanceIndex(mastertype2);
    if(masterindex2 == -1){
        g_Logger.ErrorLog((boost::format("%s(%s, %s) error:}X^[ID2̃}X^[݂܂B") % FUNCNAME % masterid1 % masterid2).str());
        return 0;
    }
    lua_pushnumber(L, game->EvalAllianceCost(masterindex1, masterindex2, -1));
    return 1;
}
//---------------------------------------------------------------------------
int api_get_allmaster( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);

    // Ve[uǉ
    lua_newtable(L);
    const TOpenFarenData *data = g_GameMaster.GetData();
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int mastercount = game->GetMasterInstanceCount();
    for(int i=0; i<mastercount; ++i){
        lua_pushstring(L, data->GetMaster(game->GetMasterInstance(i).m_type).m_id.c_str());
        lua_rawseti(L, -2, i+1);
    }
    return 1;
}
//---------------------------------------------------------------------------
int api_master_is_exists( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string masterid = lua_tostring(L, 1);
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenData *data = g_GameMaster.GetData();
    int mastertype = data->GetMasterIndex(masterid);
    if(mastertype == -1){
        lua_settop(L, 0);
        g_Logger.ErrorLog((boost::format("%s(%s) error:}X^[IDĂ܂B") % FUNCNAME % masterid).str());
        return 0;
    }

    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int masterindex = game->GetMasterInstanceIndex(mastertype);
    if(masterindex == -1){
        lua_pushboolean(L, 0);
        return 1;
    }
    const TMasterInstance& masterins = game->GetMasterInstance(masterindex);
    lua_pushboolean(L, masterins.m_exists);
    return 1;
}
//---------------------------------------------------------------------------
int api_master_is_player( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string masterid = lua_tostring(L, 1);
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenData *data = g_GameMaster.GetData();
    int mastertype = data->GetMasterIndex(masterid);
    if(mastertype == -1){
        lua_settop(L, 0);
        g_Logger.ErrorLog((boost::format("%s(%s) error:}X^[IDĂ܂B") % FUNCNAME % masterid).str());
        return 0;
    }

    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int masterindex = game->GetMasterInstanceIndex(mastertype);
    if(masterindex == -1){
        lua_pushboolean(L, 0);
        return 1;
    }
    const TMasterInstance& masterins = game->GetMasterInstance(masterindex);
//g_Logger.DebugLog((boost::format("%s(%s) %d %d %d") % FUNCNAME % masterid % mastertype % masterindex % masterins.m_player).str());
    lua_pushboolean(L, masterins.m_player);
    return 1;
}
//---------------------------------------------------------------------------
int api_master_plus_ley( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 2 || !lua_isstring(L, 1) || !lua_isnumber(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string masterid = lua_tostring(L, 1);
    int ley = lua_tonumber(L, 2);
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenData *data = g_GameMaster.GetData();
    int mastertype = data->GetMasterIndex(masterid);
    if(mastertype == -1){
        lua_settop(L, 0);
        g_Logger.ErrorLog((boost::format("%s(%s) error:}X^[IDĂ܂B") % FUNCNAME % masterid).str());
        return 0;
    }

    TOpenFarenGame *game = g_GameMaster.GetGame();
    int masterindex = game->GetMasterInstanceIndex(mastertype);
    if(masterindex == -1){
        return 0;
    }
    game->PlusLey(masterindex, ley);
    lua_pushboolean(L, 1);
    return 1;
}
//---------------------------------------------------------------------------
int api_master_get_ley( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string masterid = lua_tostring(L, 1);
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenData *data = g_GameMaster.GetData();
    int mastertype = data->GetMasterIndex(masterid);
    if(mastertype == -1){
        lua_settop(L, 0);
        g_Logger.ErrorLog((boost::format("%s(%s) error:}X^[IDĂ܂B") % FUNCNAME % masterid).str());
        return 0;
    }

    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int masterindex = game->GetMasterInstanceIndex(mastertype);
    if(masterindex == -1){
        return 0;
    }
    const TMasterInstance& masterins = game->GetMasterInstance(masterindex);
    lua_pushnumber(L, masterins.m_money);
    return 1;
}
//---------------------------------------------------------------------------
int api_get_chara_data( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string charaid = lua_tostring(L, 1);
    const TOpenFarenData *data = g_GameMaster.GetData();
    int charatype = data->GetCharacterIndex(charaid);
    if(charatype == -1){
        g_Logger.ErrorLog((boost::format("%s(%s) error:LIDĂ܂B") % FUNCNAME % charaid).str());
        return 0;
    }
    // X^bNNA
    lua_settop(L, 0);
    // Ve[uǉ
    lua_newtable(L);
    const TBattleCharacter& charadata = data->GetCharacter(charatype);
    lua_pushstring(L, "id");
    lua_pushstring(L, charadata.m_id.c_str());
    lua_rawset(L, -3);
    lua_pushstring(L, "power");
    lua_pushnumber(L, charadata.m_power);
    lua_rawset(L, -3);
    return 1;
}
//---------------------------------------------------------------------------
// FLID
// ߂lFĂ}X^[ID
int api_get_chara_master( lua_State *L )
{
    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string charaid = lua_tostring(L, 1);
    const TOpenFarenData *data = g_GameMaster.GetData();
    int charatype = data->GetCharacterIndex(charaid);
    if(charatype == -1){
        g_Logger.ErrorLog((boost::format("%s(%s) error:LIDĂ܂B") % FUNCNAME % charaid).str());
        return 0;
    }

    int masterindex = -1;
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int areacount = game->GetAreaInstanceCount();
    for(int ai=0; ai<areacount; ++ai){
        const TAreaInstance& areains = game->GetAreaInstance(ai);
        int chrcount = areains.m_charalist.size();
        for(int i=0; i<chrcount; ++i){
            if(areains.m_charalist[i].m_type == charatype){
                masterindex = areains.m_master;
                ai = areacount;
                break;
            }
        }
    }

    // X^bNNA
    lua_settop(L, 0);
    if(masterindex == -1){
        // GAɋ邩AȂꍇnil
        return 0;
    }
    // }X^[IDԂB
    const TMasterInstance& masterins = game->GetMasterInstance(masterindex);
    const TMaster& master = data->GetMaster(masterins.m_type);
    lua_pushstring(L, master.m_id.c_str());
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_is_ending( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(!n){
        // ȂŌĂ΂ꂽꍇAGfBOǂԂB
        // X^bNNA
        lua_settop(L, 0);
        const TOpenFarenGame *game = g_GameMaster.GetGame();
        lua_pushboolean(L, game->IsEnding());
        return 1;
    }
    else if(n == 1 && lua_isstring(L, 1)){
        // }X^[IDw肵ČĂ΂ꂽꍇAvC[}X^[̃GfBOǂԂB
        std::string masterid = lua_tostring(L, 1);
        // X^bNNA
        lua_settop(L, 0);
        const TOpenFarenGame *game = g_GameMaster.GetGame();
        if(!game->IsEnding()){
            lua_pushboolean(L, 0);
            return 1;
        }

        const TOpenFarenData *data = g_GameMaster.GetData();
        int mastertype = data->GetMasterIndex(masterid);
        if(mastertype == -1){
            lua_settop(L, 0);
            g_Logger.ErrorLog((boost::format("%s(%s) error:}X^[IDĂ܂B") % FUNCNAME % masterid).str());
            return 0;
        }

        // }X^[̏͂ɑ݂Ȃꍇ͋UB
        int masterindex = game->GetMasterInstanceIndex(mastertype);
        if(masterindex == -1){
            lua_pushboolean(L, 0);
            return 1;
        }
        // Aei/\̂悤ɃGfBOOFFɂꂽꍇ͋UB
        if(!EndingDisabledMaster.empty()){
            std::vector<int>::const_iterator itr = std::find(EndingDisabledMaster.begin(), EndingDisabledMaster.end(), mastertype);
            if(itr != EndingDisabledMaster.end()){
                lua_pushboolean(L, 0);
                return 1;
            }
        }
        const TMasterInstance& masterins = game->GetMasterInstance(masterindex);
        //lua_pushboolean(L, (masterins.m_player && (masterindex == game->GetCurrentMasterIndex())));
        lua_pushboolean(L, (masterins.m_player && masterins.m_exists));
        return 1;
    }
    else{
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }
}
//---------------------------------------------------------------------------
int api_scenario_set_ending_enabled( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 2 || !lua_isstring(L, 1) || !lua_isboolean(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string masterid = lua_tostring(L, 1);
    bool enabled = lua_toboolean(L, 2);
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenData *data = g_GameMaster.GetData();
    int mastertype = data->GetMasterIndex(masterid);
    if(mastertype == -1){
        lua_settop(L, 0);
        g_Logger.ErrorLog((boost::format("%s(%s) error:}X^[IDĂ܂B") % FUNCNAME % masterid).str());
        return 0;
    }
    if(enabled){
        // ̃[gʂ̂񂯂ǁiȂƂDefaultViIł͒ʂȂjB
        std::vector<int>::iterator itr = std::find(EndingDisabledMaster.begin(), EndingDisabledMaster.end(), mastertype);
        if(itr != EndingDisabledMaster.end()){
            EndingDisabledMaster.erase(itr);
        }
    }
    else{
        EndingDisabledMaster.push_back(mastertype);
    }
    lua_pushboolean(L, true);
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_get_chara_exists_area( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string charaid = lua_tostring(L, 1);
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenData *data = g_GameMaster.GetData();
    int charatype = data->GetCharacterIndex(charaid);
    if(charatype == -1){
        g_Logger.ErrorLog((boost::format("%s(%s) error:LIDĂ܂B") % FUNCNAME % charaid).str());
        lua_settop(L, 0);
        return 0;
    }

    int areaindex = -1;
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int areacount = game->GetAreaInstanceCount();
    for(int ai=0; ai<areacount; ++ai){
        const TAreaInstance& areains = game->GetAreaInstance(ai);
        int chrcount = areains.m_charalist.size();
        for(int i=0; i<chrcount; ++i){
            if(areains.m_charalist[i].m_type == charatype){
                areaindex = ai;
                ai = areacount;
                break;
            }
        }
    }

    if(areaindex != -1){
        const TAreaInstance& areains = game->GetAreaInstance(areaindex);
        lua_pushstring(L, areains.m_id.c_str());
        return 1;
    }
    else{
        return 0;
    }
}
//---------------------------------------------------------------------------
int api_scenario_get_master_exists_area( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        // uNo30.][}ovłnilŌĂ΂邱ƂA͐nȂ̂ŃG[OoȂB
        //luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string masterid = lua_tostring(L, 1);
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenData *data = g_GameMaster.GetData();
    int mastertype = data->GetMasterIndex(masterid);
    if(mastertype == -1){
        g_Logger.ErrorLog((boost::format("%s(%s) error:}X^[IDĂ܂B") % FUNCNAME % masterid).str());
        lua_settop(L, 0);
        return 0;
    }

    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int masterindex = game->GetMasterInstanceIndex(mastertype);
    if(masterindex == -1){
        lua_pushboolean(L, 0);
        return 1;
    }
    // }X^[݃GAȂꍇ̓G[ƂfalseƂBX̃ViȈ̓sB
    int basearea = game->GetMasterBaseArea(masterindex);
    if(basearea == -1){
        lua_pushboolean(L, 0);
        return 1;
    }
    const TAreaInstance& areains = game->GetAreaInstance(basearea);
    lua_pushstring(L, areains.m_id.c_str());
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_focus_area( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string areaid = lua_tostring(L, 1);
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int areaindex = game->GetAreaInstanceIndex(areaid);
    if(areaindex == -1){
        lua_settop(L, 0);
        g_Logger.ErrorLog((boost::format("%s(%s) error:GAIDĂ܂B") % FUNCNAME % areaid).str());
        return 0;
    }
    WorldMapLogic->FocusArea(areaindex);
    lua_pushboolean(L, true);
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_get_allarea( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    // X^bNNA
    lua_settop(L, 0);
    // Ve[uǉ
    lua_newtable(L);
    const TOpenFarenData *data = g_GameMaster.GetData();
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int areacount = game->GetAreaInstanceCount();
    for(int i=0; i<areacount; ++i){
        const TAreaInstance& area = game->GetAreaInstance(i);
        // z̃L[̓GAID
        lua_pushstring(L, area.m_id.c_str());
        // Ve[uǉ
        lua_newtable(L);

        // L}X^[
        if(area.m_master != -1){
            lua_pushstring(L, "master");
            lua_pushstring(L, data->GetMaster(game->GetMasterInstance(area.m_master).m_type).m_id.c_str());
            lua_rawset(L, -3);
        }

        // אڃGAIDXg
        lua_pushstring(L, "neighbor");
        // Ve[uǉ
        lua_newtable(L);
        for(int n=0; n<area.m_neighbor.size(); ++n){
            lua_pushstring(L, game->GetAreaInstance(area.m_neighbor[n]).m_id.c_str());
            lua_rawseti(L, -2, n + 1);
        }
        lua_rawset(L, -3);  // "neighbor"ǉ

        lua_rawset(L, -3);
    }
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_get_area_master( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string areaid = lua_tostring(L, 1);
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int areaindex = game->GetAreaInstanceIndex(areaid);
    if(areaindex == -1){
        lua_settop(L, 0);
        g_Logger.ErrorLog((boost::format("%s(%s) error:GAIDĂ܂B") % FUNCNAME % areaid).str());
        return 0;
    }

    const TAreaInstance& areains = game->GetAreaInstance(areaindex);
    if(areains.m_master == -1){
        return 0;
    }

    const TMasterInstance& masterins = game->GetMasterInstance(areains.m_master);
    const TOpenFarenData *data = g_GameMaster.GetData();
    lua_pushstring(L, data->GetMaster(masterins.m_type).m_id.c_str());
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_get_area_chara_count( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string areaid = lua_tostring(L, 1);
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int areaindex = game->GetAreaInstanceIndex(areaid);
    if(areaindex == -1){
        g_Logger.ErrorLog((boost::format("%s(%s) error:GAIDĂ܂B") % FUNCNAME % areaid).str());
        return 0;
    }
    const TAreaInstance& areains = game->GetAreaInstance(areaindex);
    lua_pushnumber(L, areains.m_charalist.size());
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_get_area_employable( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 2 || !lua_isstring(L, 1) || !lua_isnumber(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string areaid = lua_tostring(L, 1);
    int chrindex = lua_tonumber(L, 2);
    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenData *data = g_GameMaster.GetData();
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int areaindex = game->GetAreaInstanceIndex(areaid);
    if(areaindex == -1){
        g_Logger.ErrorLog((boost::format("%s(%s, %d) error:GAIDĂ܂B") % FUNCNAME % areaid % chrindex).str());
        return 0;
    }
    const TAreaInstance& areains = game->GetAreaInstance(areaindex);
    if(chrindex < 0 || areains.m_charalist.size() <= chrindex){
        g_Logger.ErrorLog((boost::format("%s(%s, %d) error:ٗpSLCfbNX͈͂𒴂Ă܂B") % FUNCNAME % areaid % chrindex).str());
        return 0;
    }

    std::vector<int> employable;
    game->GetEmployable(areaindex, chrindex, employable);
    // Ve[uǉ
    lua_newtable(L);
    for(int i=0; i<employable.size(); ++i){
        // L[:LIDAl:ٗpRXg
        int charatype = employable[i];
        const TBattleCharacter& charadata = data->GetCharacter(charatype);
        lua_pushstring(L, charadata.m_id.c_str());
        lua_pushnumber(L, game->GetComEmployCost(charatype));
        lua_rawset(L, -3);
    }
    return 1;
}
//---------------------------------------------------------------------------
void scenario_get_area_chara( lua_State *L, int areaindex, int chrindex )
{
    // Ve[uǉ
    lua_newtable(L);
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    const TAreaInstance& areains = game->GetAreaInstance(areaindex);
    const TCharacterInstance& chara = areains.m_charalist[chrindex];
    const TOpenFarenData *data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
    lua_pushstring(L, "id");
    lua_pushstring(L, charadata.m_id.c_str());
    lua_rawset(L, -3);
    lua_pushstring(L, "actioned");
    lua_pushboolean(L, chara.m_actioned);
    lua_rawset(L, -3);
    lua_pushstring(L, "power");
    lua_pushnumber(L, charadata.m_experience);
    lua_rawset(L, -3);
}
//---------------------------------------------------------------------------
int api_scenario_get_area_chara( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 2 || !lua_isstring(L, 1) || !lua_isnumber(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string areaid = lua_tostring(L, 1);
    int chrindex = lua_tonumber(L, 2);
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int areaindex = game->GetAreaInstanceIndex(areaid);
    if(areaindex == -1){
        g_Logger.ErrorLog((boost::format("%s(%s, %d) error:GAIDĂ܂B") % FUNCNAME % areaid % chrindex).str());
        return 0;
    }
    const TAreaInstance& areains = game->GetAreaInstance(areaindex);
    if(chrindex < 0 || areains.m_charalist.size() <= chrindex){
        g_Logger.ErrorLog((boost::format("%s(%s, %d) error:LCfbNX͈͂𒴂Ă܂B") % FUNCNAME % areaid % chrindex).str());
        return 0;
    }

    // X^bNNA
    lua_settop(L, 0);
/*
    // Ve[uǉ
    lua_newtable(L);
    const TOpenFarenData *data = g_GameMaster.GetData();
    const TCharacterInstance& chara = areains.m_charalist[chrindex];
    const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
    lua_pushstring(L, "id");
    lua_pushstring(L, charadata.m_id.c_str());
    lua_rawset(L, -3);
    lua_pushstring(L, "actioned");
    lua_pushboolean(L, chara.m_actioned);
    lua_rawset(L, -3);
    lua_pushstring(L, "power");
    lua_pushnumber(L, charadata.m_experience);
    lua_rawset(L, -3);
*/
    scenario_get_area_chara(L, areaindex, chrindex);
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_get_area_allchara( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string areaid = lua_tostring(L, 1);
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int areaindex = game->GetAreaInstanceIndex(areaid);
    if(areaindex == -1){
        g_Logger.ErrorLog((boost::format("%s(%s) error:GAIDĂ܂B") % FUNCNAME % areaid).str());
        return 0;
    }
    const TAreaInstance& areains = game->GetAreaInstance(areaindex);
    // X^bNNA
    lua_settop(L, 0);
    // Ve[uǉ
    lua_newtable(L);
    for(int chrindex = 0; chrindex < areains.m_charalist.size(); ++chrindex){
        scenario_get_area_chara(L, areaindex, chrindex);
        lua_rawseti(L, -2, chrindex + 1);
    }
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_insert_area_chara( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n < 2 || n > 3 || !lua_isstring(L, 1) || !lua_isstring(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }
    bool force = false;
    if(n == 3){
        if(!lua_isboolean(L, 1)){
            lua_settop(L, 0);
            luautil_arg_error_log(L, FUNCNAME);
            return 0;
        }
        force = lua_toboolean(L, 3);
    }

    std::string areaid = lua_tostring(L, 1);
    std::string charaid = lua_tostring(L, 2);
    // X^bNNA
    lua_settop(L, 0);
    TOpenFarenGame *game = g_GameMaster.GetGame();
    int areaindex = game->GetAreaInstanceIndex(areaid);
    if(areaindex == -1){
        g_Logger.ErrorLog((boost::format("%s(%s, %s) error:GAIDĂ܂B") % FUNCNAME % areaid % charaid).str());
        return 0;
    }

    const TOpenFarenData *data = g_GameMaster.GetData();
    int charatype = data->GetCharacterIndex(charaid);
    if(charatype == -1){
        g_Logger.ErrorLog((boost::format("%s(%s, %s) error:LIDĂ܂B") % FUNCNAME % areaid % charaid).str());
        return 0;
    }

    const TAreaInstance& areains = game->GetAreaInstance(areaindex);
    if(force || areains.m_charalist.size() < TOpenFarenGame::AREACHARA_MAX){
        game->InsertChara(areaindex, charatype);
    }
    else{
        g_Logger.ErrorLog((boost::format("%s(%s, %s) error:GÃLɒBĂ邽߃Lǉł܂B") % FUNCNAME % areaid % charaid).str());
        return 0;
    }
    lua_pushboolean(L, 1);
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_move_area_chara( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 3 || !lua_isstring(L, 1) || !lua_isstring(L, 2) || !lua_istable(L, 3)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string areaid1 = lua_tostring(L, 1);
    std::string areaid2 = lua_tostring(L, 2);
    // ړL̃CfbNXXg擾B
    std::vector<int> indice;
    lua_pushnil(L); // ŏ̃L[
    while(lua_next(L, 3)){
        if(!lua_isnumber(L, -1)){
            g_Logger.ErrorLog((boost::format("%s invalid parameter. character index must be number.") % FUNCNAME).str());
            // X^bNNA
            lua_settop(L, 0);
            return 0;
        }
        indice.push_back(lua_tonumber(L, -1));
        lua_pop(L, 1);
    }

    // X^bNNA
    lua_settop(L, 0);
    TOpenFarenGame *game = g_GameMaster.GetGame();
    int area_from = game->GetAreaInstanceIndex(areaid1);
    if(area_from == -1){
        g_Logger.ErrorLog((boost::format("%s(%s, %s) error:GAID1Ă܂B") % FUNCNAME % areaid1 % areaid2).str());
        return 0;
    }
    int area_to = game->GetAreaInstanceIndex(areaid2);
    if(area_to == -1){
        g_Logger.ErrorLog((boost::format("%s(%s, %s) error:GAID2Ă܂B") % FUNCNAME % areaid1 % areaid2).str());
        return 0;
    }
    int curmaster = game->GetCurrentMasterIndex();
    const TAreaInstance& area1 = game->GetAreaInstance(area_from);
    const TAreaInstance& area2 = game->GetAreaInstance(area_to);
    if(curmaster != area1.m_master || curmaster != area2.m_master){
        g_Logger.ErrorLog((boost::format("%s(%s, %s) error:GAłȂGÃLړ悤Ƃ܂B") % FUNCNAME % areaid1 % areaid2).str());
        return 0;
    }

    lua_pushboolean(L, game->MoveCharacter(area_from, area_to, indice));
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_remove_chara( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string charaid = lua_tostring(L, 1);
    const TOpenFarenData *data = g_GameMaster.GetData();
    int charatype = data->GetCharacterIndex(charaid);
    if(charatype == -1){
        lua_settop(L, 0);
        g_Logger.ErrorLog((boost::format("%s(%s) error:ȃLIDw肳܂B") % FUNCNAME % charaid).str());
        return 0;
    }

    // X^bNNA
    lua_settop(L, 0);
    TOpenFarenGame *game = g_GameMaster.GetGame();
    int areacount = game->GetAreaInstanceCount();
    for(int ai=0; ai<areacount; ++ai){
        const TAreaInstance& areains = game->GetAreaInstance(ai);
        int chrcount = areains.m_charalist.size();
        for(int i=0; i<chrcount; ++i){
            if(areains.m_charalist[i].m_type == charatype){
                game->RemoveChara(ai, i);
                break;
            }
        }
    }
    lua_pushboolean(L, 1);
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_plus_area_city( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 2 || !lua_isstring(L, 1) || !lua_isnumber(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string areaid = lua_tostring(L, 1);
    int city = lua_tonumber(L, 2);
    // X^bNNA
    lua_settop(L, 0);
    TOpenFarenGame *game = g_GameMaster.GetGame();
    int areaindex = game->GetAreaInstanceIndex(areaid);
    if(areaindex == -1){
        g_Logger.ErrorLog((boost::format("%s(%s) error:GAIDĂ܂B") % FUNCNAME % areaid).str());
        return 0;
    }
    game->SetAreaCity(areaindex, game->GetAreaCity(areaindex) + city);
    lua_pushboolean(L, 1);
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_plus_area_wall( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 2 || !lua_isstring(L, 1) || !lua_isnumber(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string areaid = lua_tostring(L, 1);
    int wall = lua_tonumber(L, 2);
    // X^bNNA
    lua_settop(L, 0);
    TOpenFarenGame *game = g_GameMaster.GetGame();
    int areaindex = game->GetAreaInstanceIndex(areaid);
    if(areaindex == -1){
        g_Logger.ErrorLog((boost::format("%s(%s) error:GAIDĂ܂B") % FUNCNAME % areaid).str());
        return 0;
    }
    game->SetAreaWall(areaindex, game->GetAreaWall(areaindex) + wall);
    lua_pushboolean(L, 1);
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_plus_area_road( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 2 || !lua_isstring(L, 1) || !lua_isnumber(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string areaid = lua_tostring(L, 1);
    int road = lua_tonumber(L, 2);
    // X^bNNA
    lua_settop(L, 0);
    TOpenFarenGame *game = g_GameMaster.GetGame();
    int areaindex = game->GetAreaInstanceIndex(areaid);
    if(areaindex == -1){
        g_Logger.ErrorLog((boost::format("%s(%s) error:GAIDĂ܂B") % FUNCNAME % areaid).str());
        return 0;
    }
    game->SetAreaRoad(areaindex, game->GetAreaRoad(areaindex) + road);
    lua_pushboolean(L, 1);
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_get_champion( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 1 || !lua_isnumber(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    int num = lua_tonumber(L, 1);

    // X^bNNA
    lua_settop(L, 0);
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    std::string champid;
    if(game->GetChampion(num, champid)){
        lua_pushstring(L, champid.c_str());
        return 1;
    }
    else{
        return 0;
    }
}
//---------------------------------------------------------------------------
int api_scenario_get_wincount( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 1 || !lua_isstring(L, 1)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string charaid = lua_tostring(L, 1);
    const TOpenFarenData *data = g_GameMaster.GetData();
    int charatype = data->GetCharacterIndex(charaid);
    if(charatype == -1){
        g_Logger.ErrorLog((boost::format("%s(%s) error:LIDĂ܂B") % FUNCNAME % charaid).str());
        return 0;
    }

    int wincount = -1;
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    int areacount = game->GetAreaInstanceCount();
    for(int ai=0; ai<areacount; ++ai){
        const TAreaInstance& areains = game->GetAreaInstance(ai);
        int chrcount = areains.m_charalist.size();
        for(int i=0; i<chrcount; ++i){
            if(areains.m_charalist[i].m_type == charatype){
                wincount = areains.m_charalist[i].m_wincount;
                break;
            }
        }
    }

    // X^bNNA
    lua_settop(L, 0);
    if(wincount == -1){
        // Ȃꍇnil
        return 0;
    }
    lua_pushnumber(L, wincount);
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_employ( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 3 || !lua_isstring(L, 1) || !lua_isnumber(L, 2) || !lua_isstring(L, 3)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string areaid = lua_tostring(L, 1);
    int chrindex = lua_tonumber(L, 2);
    std::string charaid = lua_tostring(L, 3);
    // X^bNNA
    lua_settop(L, 0);
    TOpenFarenGame *game = g_GameMaster.GetGame();
    int areaindex = game->GetAreaInstanceIndex(areaid);
    if(areaindex == -1){
        g_Logger.ErrorLog((boost::format("%s(%s, %d, %s) error:GAIDĂ܂B") % FUNCNAME % areaid % chrindex % charaid).str());
        return 0;
    }

    const TOpenFarenData *data = g_GameMaster.GetData();
    int charatype = data->GetCharacterIndex(charaid);
    if(charatype == -1){
        g_Logger.ErrorLog((boost::format("%s(%s, %d, %s) error:LIDĂ܂B") % FUNCNAME % areaid % chrindex % charaid).str());
        return 0;
    }

    const TAreaInstance& areains = game->GetAreaInstance(areaindex);
    if(chrindex < 0 || areains.m_charalist.size() <= chrindex){
        g_Logger.ErrorLog((boost::format("%s(%s, %d, %s) error:ٗpSLCfbNX͈͂𒴂Ă܂B") % FUNCNAME % areaid % chrindex % charaid).str());
        return 0;
    }
    lua_pushboolean(L, game->EmployCharacter(areaindex, chrindex, charatype));
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_dismiss( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 2 || !lua_isstring(L, 1) || !lua_isnumber(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string areaid = lua_tostring(L, 1);
    int chrindex = lua_tonumber(L, 2);
    // X^bNNA
    lua_settop(L, 0);
    TOpenFarenGame *game = g_GameMaster.GetGame();
    int areaindex = game->GetAreaInstanceIndex(areaid);
    if(areaindex == -1){
        g_Logger.ErrorLog((boost::format("%s(%s, %d) error:GAIDĂ܂B") % FUNCNAME % areaid % chrindex).str());
        return 0;
    }
    lua_pushboolean(L, game->DismissCharacter(areaindex, chrindex));
    return 1;
}
//---------------------------------------------------------------------------
int api_scenario_propose_alliance( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 3 || !lua_isstring(L, 1) || !lua_isnumber(L, 2) || !lua_isnumber(L, 3)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string masterid = lua_tostring(L, 1);
    int turn = lua_tonumber(L, 2);
    int cost = lua_tonumber(L, 3);
    // X^bNNA
    lua_settop(L, 0);
    TOpenFarenGame *game = g_GameMaster.GetGame();
    int masterindex = game->GetMasterInstanceIndex(masterid);
    if(masterindex == -1){
        g_Logger.ErrorLog((boost::format("%s(%s) error:}X^[IDĂ邩A݂Ȃ}X^[łB") % FUNCNAME % masterid).str());
        return 0;
    }
    if(!WorldMapLogic->ProposeAlliance(masterindex, turn, cost)){
        return lua_yield(L, 0);
    }
    return 0;
}
//---------------------------------------------------------------------------
int api_scenario_departure( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    int n = lua_gettop(L);

    if(n != 2 || !lua_isstring(L, 1) || !lua_istable(L, 2)){
        lua_settop(L, 0);
        luautil_arg_error_log(L, FUNCNAME);
        return 0;
    }

    std::string areaid_to = lua_tostring(L, 1);
    TOpenFarenGame *game = g_GameMaster.GetGame();
    int area_to = game->GetAreaInstanceIndex(areaid_to);
    if(area_to == -1){
        g_Logger.ErrorLog((boost::format("%s owGAID %s Ă܂B") % FUNCNAME % areaid_to).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }
    // owL̃CfbNXXg擾B
    std::vector<std::pair<int, int> > indice;
    // GAIDL[Ƃe[u̗vf擾B
    lua_pushnil(L); // ŏ̃L[
    while(lua_next(L, 2)){
        if(!lua_isstring(L, -2)){
            g_Logger.ErrorLog((boost::format("%s invalid parameter. 2nd parameter table key must be string.") % FUNCNAME).str());
            // X^bNNA
            lua_settop(L, 0);
            return 0;
        }
//g_Logger.DebugLog((boost::format("%s %s") % FUNCNAME % lua_tostring(L, -2)).str());
        if(!lua_istable(L, -1)){
            g_Logger.ErrorLog((boost::format("%s invalid parameter. 2nd parameter table value must be table.") % FUNCNAME).str());
            // X^bNNA
            lua_settop(L, 0);
            return 0;
        }
        std::string areaid_from = lua_tostring(L, -2);
        int area_from = game->GetAreaInstanceIndex(areaid_from);
        if(area_from == -1){
            g_Logger.ErrorLog((boost::format("%s GAID %s Ă܂B") % FUNCNAME % areaid_from).str());
            return 0;
        }
        const TAreaInstance& areains = game->GetAreaInstance(area_from);
        if(areains.m_master != game->GetCurrentMasterIndex()){
            g_Logger.ErrorLog((boost::format("%s LĂȂGA %s ow悤Ƃ܂B") % FUNCNAME % areaid_from).str());
            // X^bNNA
            lua_settop(L, 0);
            return 0;
        }
        // LCfbNXe[u擾B
        lua_pushnil(L); // ŏ̃L[
        while(lua_next(L, -2)){
            if(!lua_isnumber(L, -1)){
                g_Logger.ErrorLog((boost::format("%s owLCfbNXe[u̒llł͂܂B") % FUNCNAME).str());
                // X^bNNA
                lua_settop(L, 0);
                return 0;
            }
            int chrindex = lua_tonumber(L, -1);
            if(chrindex < 0 || chrindex >= areains.m_charalist.size() || areains.m_charalist[chrindex].m_actioned){
                g_Logger.ErrorLog((boost::format("%s GA %s iL%djɑ΂ĖȏowLCfbNX%dw肳܂B") % FUNCNAME % areaid_from % areains.m_charalist.size() % chrindex).str());
                // X^bNNA
                lua_settop(L, 0);
                return 0;
            }
            indice.push_back(std::pair<int, int>(area_from, chrindex));
            lua_pop(L, 1);  // ulvAuL[v͎̌JԂ̂߂Ɏc
        }
        lua_pop(L, 1);  //  ulvAuL[v͎̌JԂ̂߂Ɏc
    }

    // X^bNNA
    lua_settop(L, 0);

    if(indice.empty()){
        g_Logger.ErrorLog((boost::format("%s owLCfbNXXgłB") % FUNCNAME).str());
        return 0;
    }

    WorldMapLogic->Departure(area_to, indice);
    lua_pushboolean(L, 1);
    return lua_yield(L, 1);
}
//---------------------------------------------------------------------------
int api_scenario_endturn( lua_State *L )
{
    // 퓬łȂƂmFB
    if(BattleIf->IsBattle()){
        g_Logger.ErrorLog((boost::format("%s was called, but now battle.") % FUNCNAME).str());
        // X^bNNA
        lua_settop(L, 0);
        return 0;
    }

    // X^bNNA
    lua_settop(L, 0);
    WorldMapLogic->EndTurn();
    lua_pushboolean(L, 1);
    return 1;
}
//---------------------------------------------------------------------------
int api_get_msec( lua_State *L )
{
    // X^bNNA
    lua_settop(L, 0);
    lua_pushinteger(L, ScriptAPI->GetTick());
    return 1;
}
//---------------------------------------------------------------------------
void regist_lua_literal( lua_State *L )
{
    // s^Cv
    const char *acttype_names[ACTION_MAX] = {"ACTION_WAIT", "ACTION_ATTACK", "ACTION_CHARGE", "ACTION_SPECIAL", "ACTION_RUNAWAY"};
    for(int i=0; i<ACTION_MAX; ++i){
        lua_pushstring(L, acttype_names[i]);
        lua_pushnumber(L, i);
        lua_settable(L, LUA_GLOBALSINDEX);
    }

    // 
    const char *sysef_names[SYSTEM_EFFECTMAX] = {"", "NOEFFECT", "POISON", "STONE", "PARALYZE", "SLEEP", "DREAM", "DEATH", "ABSORB",
        "HPRECOVER", "MPRECOVER", "CRASHWALL", "PLUS_PA", "PLUS_PD", "PLUS_SKILL", "PLUS_SPEED", "PLUS_MA", "PLUS_MD",
        "PLUS_HPRECOVER", "PLUS_MPRECOVER", "PLUS_ATTACKCOUNT",
        "CURE_POISON", "CURE_STONE", "CURE_PARALYZE", "CURE_SLEEP", "CURE_DREAM", "CURE_ALL", "CHANGE_RESISTANCE", "CHANGE_MOVETYPE",
        "SUMMON", "REVIVE"};
    for(int i=1; i<SYSTEM_EFFECTMAX; ++i){
        lua_pushstring(L, sysef_names[i]);
        lua_pushnumber(L, i);
        lua_settable(L, LUA_GLOBALSINDEX);
    }

    // ϐ
    const char *res_names[RES_MAX] = {"", "WEAK", "NORMAL", "STRONG", "ABSORB"};
    for(int i=1; i<RES_MAX; ++i){
        lua_pushstring(L, res_names[i]);
        lua_pushnumber(L, i);
        lua_settable(L, LUA_GLOBALSINDEX);
    }

    // 
    const char *elm_names[TCharacterInstance::ELEMENT_MAX] = {"", "ELEMENT_SUMMON", "ELEMENT_SUMMONED"};
    for(int i=1; i<TCharacterInstance::ELEMENT_MAX; ++i){
        lua_pushstring(L, elm_names[i]);
        lua_pushnumber(L, i);
        lua_settable(L, LUA_GLOBALSINDEX);
    }

    // 㕨
    const char *mapobj_names[MAPOBJ_MAX] = {"", "MAPOBJ_CITY", "MAPOBJ_WALL", "MAPOBJ_ROAD", "MAPOBJ_CASTLE"};
    for(int i=1; i<MAPOBJ_MAX; ++i){
        lua_pushstring(L, mapobj_names[i]);
        lua_pushnumber(L, i);
        lua_settable(L, LUA_GLOBALSINDEX);
    }

    // 摜^Cv
    const char *imgtype_names[IMAGE_FUNCMAX] = {"IMAGE_EVENT", "IMAGE_FACE", "IMAGE_CHARACHIP"};
    for(int i=0; i<IMAGE_FUNCMAX; ++i){
        lua_pushstring(L, imgtype_names[i]);
        lua_pushnumber(L, i);
        lua_settable(L, LUA_GLOBALSINDEX);
    }

    // {^^Cv
    const char *btntype_names[4] = {"BUTTON_OK", "BUTTON_YES", "BUTTON_NO", "BUTTON_CANCEL"};
    const int btntypes[4] = {TGcnMsgDlg::BUTTON_OK, TGcnMsgDlg::BUTTON_YES, TGcnMsgDlg::BUTTON_NO, TGcnMsgDlg::BUTTON_CANCEL};
    for(int i=0; i<4; ++i){
        lua_pushstring(L, btntype_names[i]);
        lua_pushnumber(L, btntypes[i]);
        lua_settable(L, LUA_GLOBALSINDEX);
    }

    // Cxg^Cv
    const char *evttype_names[SCRIPTEVENT_MAX] = {"EVENT_MOUSELPRESSED", "EVENT_MOUSERPRESSED", "EVENT_MOUSEMOVED"};
    for(int i=0; i<SCRIPTEVENT_MAX; ++i){
        lua_pushstring(L, evttype_names[i]);
        lua_pushnumber(L, i);
        lua_settable(L, LUA_GLOBALSINDEX);
    }

    lua_pushstring(L, "AREACHARA_MAX");
    lua_pushnumber(L, TOpenFarenGame::AREACHARA_MAX);
    lua_settable(L, LUA_GLOBALSINDEX);

    lua_pushstring(L, "DEFAULT_ALLIANCETURN");
    lua_pushnumber(L, TOpenFarenGame::DEFAULT_ALLIANCETURN);
    lua_settable(L, LUA_GLOBALSINDEX);
}
//---------------------------------------------------------------------------
void regist_lua_function( lua_State *L )
{
    lua_register(L, "debug_log",                        debug_log);
    lua_register(L, "api_battle_get_turn",              api_battle_get_turn);
    lua_register(L, "api_battle_get_map_size",          api_battle_get_map_size);
    lua_register(L, "api_battle_get_map_height",        api_battle_get_map_height);
    lua_register(L, "api_battle_get_movetype_totalcost",api_battle_get_movetype_totalcost);
    lua_register(L, "api_battle_get_movetype_totalskill",api_battle_get_movetype_totalskill);
    lua_register(L, "api_battle_get_movable",           api_battle_get_movable);
    lua_register(L, "api_battle_get_movable_layer",     api_battle_get_movable_layer);
    lua_register(L, "api_battle_get_move_layer_from",   api_battle_get_move_layer_from);
    lua_register(L, "api_battle_get_neighbor",          api_battle_get_neighbor);
    lua_register(L, "api_battle_get_neighbor_sorted",   api_battle_get_neighbor_sorted);
    lua_register(L, "api_battle_get_is_defense_side",   api_battle_get_is_defense_side);
    lua_register(L, "api_battle_get_master_list",       api_battle_get_master_list);
    lua_register(L, "api_battle_get_my_master",         api_battle_get_my_master);
    lua_register(L, "api_battle_get_enemy_master",      api_battle_get_enemy_master);
    lua_register(L, "api_battle_get_chara_count",       api_battle_get_chara_count);
    lua_register(L, "api_battle_get_active_chara",      api_battle_get_active_chara);
    lua_register(L, "api_battle_get_chara_on",          api_battle_get_chara_on);
    lua_register(L, "api_battle_get_chara_status",      api_battle_get_chara_status);
    lua_register(L, "api_battle_get_action_list",       api_battle_get_action_list);
    lua_register(L, "api_battle_get_last_target",       api_battle_get_last_target);
    lua_register(L, "api_battle_eval_action",           api_battle_eval_action);
    lua_register(L, "api_battle_set_action",            api_battle_set_action);
    lua_register(L, "api_battle_get_auto_runaway",      api_battle_get_auto_runaway);
    lua_register(L, "api_battle_set_chara_brave",       api_battle_set_chara_brave);
    lua_register(L, "api_battle_set_map_chip",          api_battle_set_map_chip);
    lua_register(L, "api_battle_rm_map_obj",            api_battle_rm_map_obj);
    lua_register(L, "api_get_current_master",       api_get_current_master);
    lua_register(L, "api_change_master_type",       api_change_master_type);
    lua_register(L, "api_get_master",               api_get_master);
//    lua_register(L, "api_show_message",             api_show_message);

    lua_register(L, "api_get_chara_type",           api_get_chara_type);
    //lua_register(L, "api_exists_chara",             api_exists_chara);
    lua_register(L, "api_exists_hero",              api_exists_hero);
    lua_register(L, "api_music_set",                api_music_set);
    lua_register(L, "api_gui_get_top_size",         api_gui_get_top_size);
    lua_register(L, "api_gui_event_pop",            api_gui_event_pop);
    lua_register(L, "api_window_new",               api_window_new);
    lua_register(L, "api_window_set_size",          api_window_set_size);
    lua_register(L, "api_window_set_position",      api_window_set_position);
    lua_register(L, "api_window_set_visible",       api_window_set_visible);
    lua_register(L, "api_window_set_titlebarheight", api_window_set_titlebarheight);
    lua_register(L, "api_window_set_image",         api_window_set_image);
    lua_register(L, "api_window_set_faceorchip",    api_window_set_faceorchip);
    lua_register(L, "api_window_clear_image",       api_window_clear_image);
    lua_register(L, "api_window_set_text",          api_window_set_text);
    lua_register(L, "api_window_clear_text",        api_window_clear_text);
    lua_register(L, "api_window_is_visible",        api_window_is_visible);
    lua_register(L, "api_window_event_pop",         api_window_event_pop);
    lua_register(L, "api_dialog_show",              api_dialog_show);
    lua_register(L, "api_dialog_is_visible",        api_dialog_is_visible);
    lua_register(L, "api_dialog_get_selected",      api_dialog_get_selected);
    lua_register(L, "api_image_load",               api_image_load);
    lua_register(L, "api_last_dead_master",         api_last_dead_master);
    lua_register(L, "api_get_chapter_number",       api_get_chapter_number);
    lua_register(L, "api_get_turn",                 api_get_turn);
    lua_register(L, "api_get_progressed_turn",      api_get_progressed_turn);
    lua_register(L, "api_find_alliance",            api_find_alliance);
    lua_register(L, "api_get_alliance_cost",        api_get_alliance_cost);
    lua_register(L, "api_get_allmaster",            api_get_allmaster);
    lua_register(L, "api_master_is_exists",         api_master_is_exists);
    lua_register(L, "api_master_is_player",         api_master_is_player);
    lua_register(L, "api_master_plus_ley",          api_master_plus_ley);
    lua_register(L, "api_master_get_ley",           api_master_get_ley);
    lua_register(L, "api_get_chara_data",           api_get_chara_data);
    lua_register(L, "api_get_chara_master",         api_get_chara_master);
    lua_register(L, "api_scenario_is_ending",       api_scenario_is_ending);
    lua_register(L, "api_scenario_set_ending_enabled", api_scenario_set_ending_enabled);
    lua_register(L, "api_scenario_get_chara_exists_area", api_scenario_get_chara_exists_area);
    lua_register(L, "api_scenario_get_master_exists_area", api_scenario_get_master_exists_area);
    lua_register(L, "api_scenario_focus_area",      api_scenario_focus_area);
    lua_register(L, "api_scenario_get_allarea",     api_scenario_get_allarea);
    lua_register(L, "api_scenario_get_area_master", api_scenario_get_area_master);
    lua_register(L, "api_scenario_get_area_chara_count", api_scenario_get_area_chara_count);
    lua_register(L, "api_scenario_get_area_employable", api_scenario_get_area_employable);
    lua_register(L, "api_scenario_get_area_chara", api_scenario_get_area_chara);
    lua_register(L, "api_scenario_get_area_allchara", api_scenario_get_area_allchara);
    lua_register(L, "api_scenario_insert_area_chara", api_scenario_insert_area_chara);
    lua_register(L, "api_scenario_move_area_chara", api_scenario_move_area_chara);
    lua_register(L, "api_scenario_remove_chara",    api_scenario_remove_chara);
    lua_register(L, "api_scenario_plus_area_city",  api_scenario_plus_area_city);
    lua_register(L, "api_scenario_plus_area_wall",  api_scenario_plus_area_wall);
    lua_register(L, "api_scenario_plus_area_road",  api_scenario_plus_area_road);
    lua_register(L, "api_scenario_get_champion",    api_scenario_get_champion);
    lua_register(L, "api_scenario_get_wincount",    api_scenario_get_wincount);
    lua_register(L, "api_scenario_employ",          api_scenario_employ);
    lua_register(L, "api_scenario_employ",          api_scenario_employ);
    lua_register(L, "api_scenario_dismiss",         api_scenario_dismiss);
    lua_register(L, "api_scenario_propose_alliance", api_scenario_propose_alliance);
    lua_register(L, "api_scenario_departure",       api_scenario_departure);
    lua_register(L, "api_scenario_endturn",         api_scenario_endturn);
    lua_register(L, "api_get_msec",                 api_get_msec);
}
//---------------------------------------------------------------------------
}   // namespace
//---------------------------------------------------------------------------
TOpenFarenLuaManager::TOpenFarenLuaManager() :
    m_L(NULL),
    m_CanResume(false)
{
    luautil_init();
}
//---------------------------------------------------------------------------
TOpenFarenLuaManager::~TOpenFarenLuaManager(void)
{
    if(m_L != NULL){
        luautil_free_lua(m_L);
    }
}
//---------------------------------------------------------------------------
//void TOpenFarenLuaManager::SetIf(TStrategyInterface *si, TBattleInterface *gi)
void TOpenFarenLuaManager::SetIf(TBattleInterface *gi, TScriptAPI *sapi, TWorldMapLogic *worldlogic)
{
    m_BattleIf = gi;
    m_ScriptAPI = sapi;
    m_WorldMapLogic = worldlogic;
}
//---------------------------------------------------------------------------
bool TOpenFarenLuaManager::LoadLua(const char *lua_file)
{
    std::string package_path = (boost::format("?;?.lua;%s/Script/?;%s/Script/?.lua") % g_GameMaster.GetScenarioPath() % g_GameMaster.GetScenarioPath()).str();

    lua_State *L;

    L = luautil_load_lua(lua_file, package_path.c_str());
    if(L == NULL){
        g_Logger.ErrorLog((boost::format("TOpenFarenLuaManager::LoadLua error:%s ̃[hɎs܂B") % lua_file).str());
        return false;
    }

    regist_lua_literal(L);
    regist_lua_function(L);
    if(m_L != NULL){
        luautil_free_lua(m_L);
    }
    m_L = L;
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenLuaManager::CallFunction(const char *funcname)
{
    std::vector<std::string> v;
    return CallFunction(funcname, v);
}
//---------------------------------------------------------------------------
bool TOpenFarenLuaManager::CallFunction(const char *funcname, const std::vector<std::string>& args)
{
OFDBGLOG((boost::format("s %s") % funcname).str());
    BattleIf = m_BattleIf;
    ScriptAPI = m_ScriptAPI;
    WorldMapLogic = m_WorldMapLogic;
    BattleMasterIndex = m_BattleMaster;
    if(m_ScriptAPI != NULL){
        m_ScriptAPI->ClearEvent();
    }
    lua_getglobal(m_L, funcname);
    if(!lua_isfunction(m_L, -1)){
        OFERRLOG((boost::format("֐łȂ(%s)ĂяoƂ܂B") % funcname).str());
        lua_pop(m_L, 1);
        return false;
    }
    // ςށB
    for(unsigned u=0; u<args.size(); ++u){
        lua_pushstring(m_L, args[u].c_str());
    }
    m_LastFunctionValue = false;
    int ret = lua_resume(m_L, args.size());
    if(ret == LUA_ERRRUN || ret == LUA_ERRMEM || ret == LUA_ERRERR){
        OFERRLOG((boost::format("֐ %s resume error %d") % funcname % ret).str());
        luautil_print_lua_error(m_L);
        m_CanResume = false;
        return false;
    }
    m_CanResume = (ret == LUA_YIELD);
    if(!m_CanResume){
        int n = lua_gettop(m_L);
        if(n > 0 && lua_isboolean(m_L, n)){
            m_LastFunctionValue = lua_toboolean(m_L, n);
        }
    }
OFDBGLOG((boost::format("e %s") % funcname).str());
    return true;
}
//---------------------------------------------------------------------------
bool TOpenFarenLuaManager::CanResume(void) const
{
    return m_CanResume;
}
//---------------------------------------------------------------------------
bool TOpenFarenLuaManager::Resume(void)
{
    /*if(!lua_isfunction(m_L, -1)){
        g_Logger.ErrorLog((boost::format("TOpenFarenLuaManager::Resume error:֐łȂ̂resume悤Ƃ܂B")).str());
        return false;
    }*/
OFDBGLOG((boost::format("s")).str());
    BattleIf = m_BattleIf;
    ScriptAPI = m_ScriptAPI;
    WorldMapLogic = m_WorldMapLogic;
    BattleMasterIndex = m_BattleMaster;
    m_LastFunctionValue = false;
    int ret = lua_resume(m_L, 0);
    if(ret == LUA_ERRRUN || ret == LUA_ERRMEM || ret == LUA_ERRERR){
        g_Logger.ErrorLog((boost::format("TOpenFarenLuaManager::Resume error:resume error %d") % ret).str());
        luautil_print_lua_error(m_L);
        m_CanResume = false;
        return false;
    }
    m_CanResume = (ret == LUA_YIELD);
    if(!m_CanResume){
        int n = lua_gettop(m_L);
//g_Logger.DebugLog((boost::format("TOpenFarenLuaManager::Resume %d %s") % n % lua_typename(m_L, lua_type(m_L, n))).str());
        if(n > 0 && lua_isboolean(m_L, n)){
            m_LastFunctionValue = lua_toboolean(m_L, n);
//g_Logger.DebugLog((boost::format("TOpenFarenLuaManager::Resume fv=%d") % m_LastFunctionValue).str());
        }
    }
OFDBGLOG((boost::format("e")).str());
    return true;
}
//---------------------------------------------------------------------------
void TOpenFarenLuaManager::SaveToXML(TiXmlElement& e) const
{
    lua_getglobal(m_L, "Flag");
    if(lua_type(m_L, -1) == LUA_TTABLE){
        TiXmlElement e_flag("Flag");
        if(luavar2xml(m_L, lua_gettop(m_L), &e_flag) == -1){
            g_Logger.ErrorLog("luavar2xml error");
            return;
        }
        e.InsertEndChild(e_flag);
    }
}
//---------------------------------------------------------------------------
bool TOpenFarenLuaManager::LoadFromXML(const TiXmlElement& e)
{
    if(m_L == NULL){
        g_Logger.ErrorLog("TOpenFarenLuaManager::LoadFromXML() error : Lua state null.");
        return false;
    }
    for(const TiXmlElement *child = e.FirstChildElement(); child != NULL; child = child->NextSiblingElement()){
        lua_pushstring(m_L, child->Value());
        lua_newtable(m_L);
        for(const TiXmlElement *e_var = child->FirstChildElement(); e_var != NULL; e_var = e_var->NextSiblingElement()){
//g_Logger.DebugLog((boost::format("TOpenFarenLuaManager::LoadFromXML var=%s") % e_var->Value()).str());
            if(xml2luavar(m_L, lua_gettop(m_L), e_var) == -1){
                g_Logger.ErrorLog("xml2luavar error");
                return false;
            }
        }
        lua_settable(m_L, LUA_GLOBALSINDEX);
    }
    return true;
}
//---------------------------------------------------------------------------
void TOpenFarenLuaManager::SetBattleMasterIndex(int index)
{
    m_BattleMaster = index;
}
//---------------------------------------------------------------------------
// ViIJn̕ϐo^B
void TOpenFarenLuaManager::RegistInitVariable()
{
    lua_pushstring(m_L, "Flag");
    // Ve[uǉ
    lua_createtable(m_L, 499, 0);
    for(int i=1; i<500; ++i){
        lua_pushboolean(m_L, 0);
        lua_rawseti(m_L, -2, i);
    }
    lua_settable(m_L, LUA_GLOBALSINDEX);
}
//---------------------------------------------------------------------------
