#include <iostream>
#include <guichan.hpp>
#include <guichan/sdl.hpp>
#include <SDL.h>
#include <SDL_mixer.h>
#include <babel.h>
#include <boost/shared_ptr.hpp>
#include <boost/filesystem/operations.hpp>

#if defined(USE_OPENGL)
#include <guichan/opengl.hpp>
#include <guichan/opengl/openglsdlimageloader.hpp>
#include <GL/gl.h>
#include <GL/glu.h>
#endif

SDL_Surface* screen;
SDL_Event event;
#if defined(USE_OPENGL)
gcn::OpenGLGraphics* graphics;
gcn::OpenGLSDLImageLoader* imageLoader;
#else
gcn::SDLGraphics* graphics;
gcn::SDLImageLoader* imageLoader;
#endif

//gcn::ImageFont* font;

#include "guichanjp.h"
#include "framewait.h"
#include "gcnfpspane.h"
#include <locale>
#include <vector>
#include <string>
#include <stdio.h>
#include "ofini.h"
#include "logger.h"
#include "gcnchar.h"
#include "gui.h"
#include "ofdata.h"     // H
#include "gamemaster.h"
#include "msgdlg.h"

TGuichanFontSJIS myFont;
TGcnFPSPane *FPSPane = NULL;    // qvȂǁB

#if defined(WIN32)
// OpenGLSwapBuffersWindowsŐ҂Ȃ悤ɂB
void DisableSwapInterval()
{
    //g֐WGL_EXT_swap_controlT|[gĂ邩ׂ
    if(!strstr( (char*)glGetString( GL_EXTENSIONS ), "WGL_EXT_swap_control")){
        //WGL_EXT_swap_controlT|[gĂȂ
        g_Logger.ErrorLog("SwapIntervalDisableIvVw肳܂ÅWGL_EXT_swap_control APIT|[gĂȂ悤łB");
        return;
    }
    else
    {
        //API擾Ďs
        BOOL (WINAPI *wglSwapIntervalEXT)(int) = (BOOL (WINAPI*)(int))wglGetProcAddress("wglSwapIntervalEXT");
        if(wglSwapIntervalEXT){
            wglSwapIntervalEXT(0);
        }
        else{
            g_Logger.ErrorLog("̂WGL_EXT_swap_control API̊֐|C^擾Ɏs܂B");
        }
    }
}
#endif

bool init()
{
    const char *EXE_FARENCNV = "farencnv.exe";
    int width = 800;
    int height = 600;

    //std::locale::global(std::locale("japanese"));

    if(g_Ini.LoadFromFile(TFarenGameMaster::INIFILE)){
        width = g_Ini.GetIntValue(TOpenFarenIni::WINDOW_WIDTH);
        height = g_Ini.GetIntValue(TOpenFarenIni::WINDOW_HEIGHT);
        g_Logger.SetLogLevel(g_Ini.GetLogLevel());
        g_Logger.SetEveryFlush(g_Ini.GetLogEveryFlush());
    }

    if(boost::filesystem::exists(EXE_FARENCNV) && !boost::filesystem::exists("Scenario")){
        g_Logger.ErrorLog("ScenariotH_܂B");
        return false;
    }

#if defined(WIN32)
    // fXNgbv̉𑜓xŐB
    // SDL_GetVideoInfo, SDL_VideoModeOK ł̓fXNgbv𒴂𑜓xłƔ肳Ă܂Windows菈B
    HWND hDesk = GetDesktopWindow();
    HDC hdc = GetDC(hDesk);
    int deskw = GetDeviceCaps(hdc, HORZRES);
    int deskh = GetDeviceCaps(hdc, VERTRES);
    const int marginx = 16, marginy = 64;
    if(width > deskw - marginx || height > deskh - marginy){
        int iniw = width;
        int inih = height;
        width = min(width, deskw - marginx);
        height = min(height, deskh - marginy);
        g_Logger.WarningLog((boost::format("fXNgbv̉𑜓x %dx%d ɑ΂init@CŎw肳ꂽ𑜓x %dx%d傫邽߁A%dx%dŋN܂B")
            % deskw % deskh % iniw % inih % width % height).str());
    }
#endif

    SDL_Init(SDL_INIT_VIDEO);

    SDL_WM_SetCaption("OpenFaren", NULL);
    SDL_Surface *surface_icon = IMG_Load("System/openfaren.png");
    if(surface_icon != NULL){
        if(surface_icon->w == 32 && surface_icon->h == 32){
            SDL_WM_SetIcon(surface_icon, NULL);
        }
        SDL_FreeSurface(surface_icon);
    }

#if defined(USE_OPENGL)
    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 8 );
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 8 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 8 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 24 );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
    screen = SDL_SetVideoMode(width, height, 32, SDL_OPENGL);

    const GLubyte *str;
    str = glGetString(GL_VENDOR);
    g_Logger.DebugLog((boost::format("VENDER = %s") % str).str());
    str = glGetString(GL_RENDERER);
    g_Logger.DebugLog((boost::format("GPU = %s") % str).str());
    str = glGetString(GL_VERSION);
    g_Logger.DebugLog((boost::format("OpenGL Version = %s") % str).str());
    int n;
    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &n);
    g_Logger.DebugLog((boost::format("GL_MAX_TEXTURE_SIZE = %d") % n).str());
#if defined(WIN32)
    if(g_Ini.GetBoolValue(TOpenFarenIni::WINDOW_DISABLESWAPINTERVAL)){
        DisableSwapInterval();
    }
#endif
#else
    screen = SDL_SetVideoMode(width, height, 32, SDL_SWSURFACE);
#endif
    g_Ini.SetRealWindowSize(width, height);
    SDL_EnableUNICODE(1);
//MessageBox(NULL, "d", "", MB_OK);
    SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);

#if defined(USE_OPENGL)
    imageLoader = new gcn::OpenGLSDLImageLoader();
    graphics = new gcn::OpenGLGraphics();
    graphics->setTargetPlane(width, height);
#else
    imageLoader = new gcn::SDLImageLoader();
    graphics = new gcn::SDLGraphics();
    graphics->setTarget(screen);
#endif
    gcn::Image::setImageLoader(imageLoader);
    gcn::SDLInput* input = new gcn::SDLInput();
    g_GuiContainerList.SetInput(input);

    babel::init_babel();

    g_Logger.DebugLog("babel ok.");
    //font = new gcn::ImageFont("fixedfont.bmp", " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789");
    //gcn::Widget::setGlobalFont(font);
    //graphics->setFont(font);

#if defined(USE_OPENGL)
    if(myFont.Initialize(16, g_Ini.GetFontName())){
        gcn::Widget::setGlobalFont(&myFont); 
        graphics->setFont(&myFont);
    }
    else{
        g_Logger.ErrorLog("tHg̏Ɏs܂B");
        return false;
    }
#else
    if(myFont.Initialize(screen->format, 16, "MS SVbN")){
        gcn::Widget::setGlobalFont(&myFont); 
    }
#endif
    g_Logger.DebugLog("font ok.");

    gcn::Gui* gui = new gcn::Gui();
    gui->setGraphics(graphics);
    gui->setInput(input);
    g_GuiContainerList.SetGui(gui);

    g_Logger.DebugLog("gui ok.");

    boost::shared_ptr<gcn::Container> top(new gcn::Container);
    top->setDimension(gcn::Rectangle(0, 0, width, height));
    top->setBaseColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_TOP));
    gui->setTop(top.get());
    g_GuiContainerList.Add(top);

    if(g_Ini.GetBoolValue(TOpenFarenIni::SOUND_ENABLED) || g_Ini.GetBoolValue(TOpenFarenIni::MUSIC_ENABLED)){
        // yƌʉB
        if(Mix_OpenAudio(44100, AUDIO_S16, 2, 4096)){
            g_Logger.ErrorLog((boost::format("Mix_OpenAudio error. %s") % Mix_GetError()).str());
            return false;
        }
        g_GameMaster.SetMixerEnabled();
    }

    int fps = g_Ini.GetIntValue(TOpenFarenIni::WINDOW_FPS);
    boost::shared_ptr<gcn::Window> fpsw(new gcn::Window());
    boost::shared_ptr<TGcnFPSPane> fpspane(new TGcnFPSPane(fps*2));
    fpsw->setOpaque(false);
    fpsw->add(fpspane.get());
    fpsw->resizeToContent();
    top->add(fpsw.get());
    FPSPane = fpspane.get();
    g_GuiContainerList.Add(fpspane);
    g_GuiContainerList.Add(fpsw);

    g_Logger.DebugLog("fps ok.");

#if defined(WIN32)
    if(boost::filesystem::exists(EXE_FARENCNV)){
        g_GameMaster.SetBasePath(".");
    }
    else{
        g_GameMaster.SetBasePath("D:\\temp");
    }
#else
    g_GameMaster.SetBasePath("/home/user1/devsrc");
#endif

    g_GameMaster.Init();
    g_GuiContainerList.SetFPS(fps);
    g_Logger.DebugLog("init() ok.");
    return true;
}

void run()
{
    bool running = true;

    while (running)
    {
        while(SDL_PollEvent(&event))
        {
            if(event.type == SDL_QUIT)
            {
                running = false;
            }
            else if(event.type == SDL_KEYDOWN)
            {
                g_GameMaster.PushKeyQueue(event.key.keysym.sym);
            }

            g_GuiContainerList.GetInput()->pushInput(event);
        }

        g_GuiContainerList.GetGui()->logic();
        g_GuiContainerList.GetGui()->draw();
        g_GuiContainerList.AddTick();

#if defined(USE_OPENGL)
        SDL_GL_SwapBuffers();
#else
        SDL_Flip(screen);
#endif
        Uint32 last_tick;
        elapse_time(g_GuiContainerList.GetFPS(), &last_tick);
        FPSPane->SetTick(last_tick);
    }
}

void halt()
{
    //delete font;

    delete graphics;
    delete imageLoader;
    // guichañNXSDL_Quit̑OɔjȂƗB
    g_GameMaster.Destroy();
    g_GuiContainerList.RemoveAll();
    if(g_Ini.GetBoolValue(TOpenFarenIni::SOUND_ENABLED) || g_Ini.GetBoolValue(TOpenFarenIni::MUSIC_ENABLED)){
        // yƌʉIB
        Mix_CloseAudio();
    }
    SDL_Quit();
}

int main(int argc, char **argv)
{
    try 
    {
        if(init()){
            run();
        }
        halt();
    }
    catch (gcn::Exception e)
    {
        std::cout << e.getMessage() << std::endl;
    }
    catch (std::exception e)
    {
        std::cout << "Std exception: " << e.what() << std::endl;  
    }
    catch (...)
    {
        std::cout << "Unknown exception" << std::endl;
    }

    return 0;
}

