//---------------------------------------------------------------------------
#include "ofmixer.h"
#include "logger.h"
#include "boostformat.h"
#include <boost/foreach.hpp>
//---------------------------------------------------------------------------
TOpenFarenMixer::TOpenFarenMixer(bool enabled) :
    m_Enabled(enabled)
{
}
//---------------------------------------------------------------------------
TOpenFarenMixer::~TOpenFarenMixer()
{
    BOOST_FOREACH(Mix_Chunk* chunk, m_ChunkList){
        Mix_FreeChunk(chunk);
    }
    BOOST_FOREACH(Mix_Music* music, m_MusicList){
        Mix_FreeMusic(music);
    }
}
//---------------------------------------------------------------------------
int TOpenFarenMixer::LoadChunk(const std::string& filepath)
{
    if(!m_Enabled){
        return 0;
    }
    //std::string path = m_BaseDir + "/" + filename;
    Mix_Chunk* chunk = Mix_LoadWAV(filepath.c_str());
    if(chunk == NULL){
        g_Logger.ErrorLog((boost::format("failed to load %s. Mix_LoadWAV error. %s") % filepath % Mix_GetError()).str());
        return -1;
    }
    int ret = m_ChunkList.size();
    m_ChunkList.push_back(chunk);
    return ret;
}
//---------------------------------------------------------------------------
int TOpenFarenMixer::LoadMusic(const std::string& filepath)
{
    if(!m_Enabled){
        return 0;
    }
    std::map<std::string, int>::const_iterator itr = m_LoadedMusic.find(filepath);
    if(itr != m_LoadedMusic.end()){
        return itr->second;
    }
    //std::string path = m_BaseDir + "/" + filename;
    Mix_Music* music = Mix_LoadMUS(filepath.c_str());
    if(music == NULL){
        g_Logger.ErrorLog((boost::format("failed to load %s. Mix_LoadMUS error. %s") % filepath % Mix_GetError()).str());
        return -1;
    }
    int ret = m_MusicList.size();
    m_MusicList.push_back(music);
    m_LoadedMusic.insert(std::pair<std::string, int>(filepath, ret));
    return ret;
}
//---------------------------------------------------------------------------
void TOpenFarenMixer::PlayChunk(int index) const
{
    if(m_Enabled){
        Mix_PlayChannel(-1, m_ChunkList[index], 0);
    }
}
//---------------------------------------------------------------------------
void TOpenFarenMixer::PlayMusic(int index) const
{
    if(m_Enabled){
        Mix_PlayMusic(m_MusicList[index], -1);
    }
}
//---------------------------------------------------------------------------
void TOpenFarenMixer::PlayMusicWithFadeout(int index, int msec) const
{
    if(!m_Enabled){
        return;
    }
    if(Mix_PlayingMusic()){
        FadeOutMusic(msec);
    }
    PlayMusic(index);
}
//---------------------------------------------------------------------------
void TOpenFarenMixer::FadeOutMusic(int msec) const
{
    if(m_Enabled){
        Mix_FadeOutMusic(msec);
    }
}
//---------------------------------------------------------------------------
bool TOpenFarenMixer::LoadPermanentChunk(TPermanentChunk index, const std::string& filepath)
{
    if(!m_Enabled){
        return true;
    }
    return (m_PermanentChunk[index] = LoadChunk(filepath)) != -1;
}
//---------------------------------------------------------------------------
void TOpenFarenMixer::PlayPermanentChunk(TPermanentChunk index) const
{
    if(m_Enabled && m_PermanentChunk[index] != -1){
        PlayChunk(m_PermanentChunk[index]);
    }
}
//---------------------------------------------------------------------------
bool TOpenFarenMixer::LoadPermanentMusic(TPermanentMusic index, const std::string& filepath)
{
    if(!m_Enabled){
        return true;
    }
    return (m_PermanentMusic[index] = LoadMusic(filepath)) != -1;
}
//---------------------------------------------------------------------------
void TOpenFarenMixer::PlayPermanentMusic(TPermanentMusic index) const
{
    if(m_Enabled && m_PermanentMusic[index] != -1){
        PlayMusic(m_PermanentMusic[index]);
    }
}
//---------------------------------------------------------------------------
void TOpenFarenMixer::PlayPermanentMusicWithFadeout(TPermanentMusic index, int msec) const
{
    PlayMusicWithFadeout(m_PermanentMusic[index], msec);
}
//---------------------------------------------------------------------------
bool TOpenFarenMixer::IsPlayingMusic() const
{
    return m_Enabled && (Mix_PlayingMusic() != 0);
}
