//---------------------------------------------------------------------------
#include <babel.h>
#include <stdio.h>
#include "ofxml.h"
#include "logger.h"

//---------------------------------------------------------------------------
std::string GetElementText(const TiXmlElement& e)
{
    const char *p = e.GetText();
    if(p == NULL){
         return "";
    }
    return babel::utf8_to_sjis(p);
}
//---------------------------------------------------------------------------
std::string GetElementTextEncoding(const TiXmlElement& e, TEncoding enc)
{
    const char *p = e.GetText();
    if(p == NULL){
         return "";
    }
#if defined WIN32
    return babel::utf8_to_sjis(p);
#else
    if(enc == ENCODING_SJIS){
        return babel::utf8_to_sjis(p);
    }
    else if(enc == ENCODING_EUCJP){
        return babel::utf8_to_euc(p);
    }
    else{
        return p;
    }
#endif
}
//---------------------------------------------------------------------------
int GetElementInt(const TiXmlElement& e)
{
    const char *p = e.GetText();
    if(p == NULL){
         return 0;
    }
    int i;
    if(sscanf(p, "%d", &i) != 1){
         return 0;
    }
    return i;
}
//---------------------------------------------------------------------------
std::string GetElementAttribute(const TiXmlElement& e, const std::string& attrname)
{
    const char* p = e.Attribute(attrname.c_str());
    if(p == NULL){
         return "";
    }
    return babel::utf8_to_sjis(p);
}
//---------------------------------------------------------------------------
bool FindIntAttribute(const TiXmlElement *e_src, const std::string& ename, const std::string& attrname, int *dst, const std::string& errlog)
{
    const TiXmlElement *e = e_src->FirstChildElement(ename.c_str());
    if(e != NULL){
        if(e->QueryIntAttribute(attrname.c_str(), dst) != TIXML_SUCCESS){
            g_Logger.ErrorLog(errlog);
            return false;
        }
    }
    else{
        g_Logger.ErrorLog(errlog);
        return false;
    }
    return true;
}
//---------------------------------------------------------------------------
void InsertTextChild(TiXmlElement& e, const char* str)
{
    e.InsertEndChild(TiXmlText(babel::sjis_to_utf8(str).c_str()));
}
//---------------------------------------------------------------------------
void SetTextAttribute(TiXmlElement& e, const char* name, const char* str)
{
    e.SetAttribute(name, babel::sjis_to_utf8(str).c_str());
}
