//---------------------------------------------------------------------------
#include <tinyxml.h>
#include <babel.h>
#include <cstdio>
#include <boost/filesystem/operations.hpp>
#include "boostformat.h"
#include "boostlexicalcast.h"
#include "scenario.h"
#include "ofxml.h"
#include "logger.h"

//---------------------------------------------------------------------------
bool TFarenChapter::LoadFromFile(const std::string& filename)
{
    m_MasterList.clear();
    m_CharacterList.clear();

    TiXmlDocument chapdoc(filename.c_str());
    if(!chapdoc.LoadFile(TIXML_ENCODING_UTF8)){
        g_Logger.ErrorLog(filename + " XML Parse Error");
        return false;
    }
    const TiXmlElement *e_top = chapdoc.FirstChildElement("Chapter");
    const TiXmlElement *e_name = e_top->FirstChildElement("Name");
    if(e_name == NULL){
        g_Logger.ErrorLog(filename + " Name܂łB");
        return false;
    }
    m_Name = GetElementText(*e_name);
    const TiXmlElement *e_ex = e_top->FirstChildElement("Explanation");
    if(e_ex == NULL){
        g_Logger.ErrorLog(filename + " Explanation܂łB");
        return false;
    }
    m_Explanation = GetElementText(*e_ex);
    const TiXmlElement *e_turn = e_top->FirstChildElement("Turn");
    if(e_turn == NULL){
        g_Logger.ErrorLog(filename + " Turn܂łB");
        return false;
    }
    m_Turn = GetElementInt(*e_turn);

    // }X^[
    for(const TiXmlElement *child = e_top->FirstChildElement("Master"); child != NULL; child = child->NextSiblingElement("Master")){
        TFarenChapterMaster master;
        master.m_id = GetElementAttribute(*child, "id");
        master.m_area = GetElementAttribute(*child, "area");
        master.m_notplayer = !GetElementAttribute(*child, "notplayer").compare("true");
        const TiXmlElement *e = child->FirstChildElement("Handle");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " Handle܂łB");
            return false;
        }
        master.m_handle = GetElementText(*e);

        e = child->FirstChildElement("Diffculty");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " Diffculty܂łB");
            return false;
        }
        master.m_diffculty = GetElementText(*e);

        e = child->FirstChildElement("Explanation");
        if(e == NULL){
            g_Logger.ErrorLog(filename + " Explanation܂łB");
            return false;
        }
        master.m_explanation = GetElementText(*e);
        m_MasterList.push_back(master);
    }

    // GA͏
    for(const TiXmlElement *child = e_top->FirstChildElement("Area"); child != NULL; child = child->NextSiblingElement("Area")){
        TFarenChapterArea area;
        area.m_id = GetElementAttribute(*child, "id");
        area.m_master = GetElementAttribute(*child, "master");
        if(area.m_master.empty()){
            area.m_ignore = !GetElementAttribute(*child, "ignore").compare("true");
        }
        m_AreaList.push_back(area);
    }

    // 
    for(const TiXmlElement *child = e_top->FirstChildElement("Alliance"); child != NULL; child = child->NextSiblingElement("Alliance")){
        TFarenChapterAlliance alliance;
        alliance.m_id1 = GetElementAttribute(*child, "id1");
        alliance.m_id2 = GetElementAttribute(*child, "id2");
        std::string turnstr = GetElementAttribute(*child, "turn");
        if(!turnstr.compare("infinite")){
            alliance.m_turn = -1;
        }
        else{
            try{
                alliance.m_turn = boost::lexical_cast<int>(turnstr);
            }
            catch(boost::bad_lexical_cast&){
                g_Logger.ErrorLog(filename + " Allianceturn̉߂Ɏs܂B");
                return false;
            }
        }
        m_AllianceList.push_back(alliance);
    }

    // zuL
    for(const TiXmlElement *child = e_top->FirstChildElement("Character"); child != NULL; child = child->NextSiblingElement("Character")){
        TFarenChapterCharacter chara;
        chara.m_id = GetElementAttribute(*child, "id");
        chara.m_area = GetElementAttribute(*child, "area");
        child->QueryIntAttribute("rank", &chara.m_rank);
        m_CharacterList.push_back(chara);
    }

    return true;
}
//---------------------------------------------------------------------------
TFarenScenario::TFarenScenario()
{
}
//---------------------------------------------------------------------------
bool TFarenScenario::LoadFromDirectory(const char* dir)
{
    std::string file;

    file = dir;
    file += "/Scenario.xml";
    TiXmlDocument doc(file.c_str());
    if(!doc.LoadFile(TIXML_ENCODING_UTF8)){
        return false;
    }
    const TiXmlElement *e_top = doc.FirstChildElement("Scenario");
    if(e_top == NULL){
        return false;
    }
    const TiXmlElement *e_title = e_top->FirstChildElement("Title");
    if(e_title == NULL){
        return false;
    }
    //const TiXmlText *text = element_1st->ToText();
    //m_Title = text->Value();
    //m_Title = babel::unicode_to_sjis(babel::bbl_wstring(element_1st->GetText()));
    //m_Title = babel::utf8_to_sjis(element_1st->GetText());
    m_Title = GetElementText(*e_title);

    for(int i=1; ;i++){
        //char chapfile[256];
        //std::sprintf(chapfile, "%s/Data/Scenario%d.xml", dir, i);
        std::string chapfile = (boost::format("%s/Data/Scenario%d.xml") % dir % i).str();
        if(!boost::filesystem::exists(chapfile)){
            break;
        }
        TFarenChapter chap;
        if(!chap.LoadFromFile(chapfile.c_str())){
            break;
        }
        m_ChapterList.push_back(chap);
    }

    return true;
}
//---------------------------------------------------------------------------
