//---------------------------------------------------------------------------
#ifndef scenarioH
#define scenarioH

#include <string>
#include <vector>

//---------------------------------------------------------------------------
struct TFarenChapterMaster
{
    std::string m_id;
    std::string m_area;
    std::string m_handle;
    std::string m_diffculty;
    std::string m_explanation;
    bool m_notplayer;
    TFarenChapterMaster() : m_notplayer(false) {}
};
//---------------------------------------------------------------------------
struct TFarenChapterArea
{
    std::string m_id;
    std::string m_master;
    bool m_ignore;
    TFarenChapterArea() : m_ignore(false) {}
};
//---------------------------------------------------------------------------
struct TFarenChapterCharacter
{
    std::string m_id;
    std::string m_area;
    int m_rank;
    TFarenChapterCharacter() : m_rank(0) {}
};
//---------------------------------------------------------------------------
struct TFarenChapterAlliance
{
    std::string m_id1;
    std::string m_id2;
    int m_turn;
};
//---------------------------------------------------------------------------
class TFarenChapter
{
private:
    std::string m_Name;
    std::string m_Explanation;
    std::vector<TFarenChapterMaster> m_MasterList;
    std::vector<TFarenChapterCharacter> m_CharacterList;
    std::vector<TFarenChapterArea> m_AreaList;
    std::vector<TFarenChapterAlliance> m_AllianceList;
    int m_Turn;
public:
    bool LoadFromFile(const std::string& filename);
    const std::string GetName() const { return m_Name; }
    const std::string GetExplanation() const { return m_Explanation; }
    const int GetTurn() const { return m_Turn; }
    int GetMasterCount() const { return m_MasterList.size(); }
    const TFarenChapterMaster& GetMaster(int index) const { return m_MasterList[index]; }
    int GetCharacterCount() const { return m_CharacterList.size(); }
    const TFarenChapterCharacter& GetCharacter(int index) const { return m_CharacterList[index]; }
    int GetAreaCount() const { return m_AreaList.size(); }
    const TFarenChapterArea& GetArea(int index) const { return m_AreaList[index]; }
    int GetAllianceCount() const { return m_AllianceList.size(); }
    const TFarenChapterAlliance& GetAlliance(int index) const { return m_AllianceList[index]; }
};
//---------------------------------------------------------------------------
class TFarenScenario
{
private:
    std::string m_Title;
    std::vector<TFarenChapter> m_ChapterList;
public:
    TFarenScenario();
    bool LoadFromDirectory(const char* dir);
    const std::string GetTitle() const { return m_Title; }
    int GetChapterCount() const { return m_ChapterList.size(); }
    const TFarenChapter& GetChapter(int i) const { return m_ChapterList[i]; }
};
//---------------------------------------------------------------------------
#endif
