//---------------------------------------------------------------------------
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/shared_ptr.hpp>
#include "scenarioselect.h"
#include "masterselect.h"
#include "msgdlg.h"
#include "gui.h"
#include "ofini.h"
#include "gamemaster.h"
#include "gcnskinmanager.h"
#include "logger.h"
#include "boostformat.h"
//---------------------------------------------------------------------------
TSceSelectListModel::TSceSelectListModel()
    : m_SceList(NULL)
{
}
//---------------------------------------------------------------------------
std::string TSceSelectListModel::getElementAt(int i)
{
    if(m_SceList == NULL){
        return "";
    }
    if(i < 0 || i >= m_SceList->size()){
        return "";
    }
    return (*m_SceList)[i].GetTitle();
}
//---------------------------------------------------------------------------
int TSceSelectListModel::getNumberOfElements()
{
    if(m_SceList == NULL){
        return 0;
    }
    return m_SceList->size();
}
//---------------------------------------------------------------------------
TChapterSelectListModel::TChapterSelectListModel()
    : m_Sce(NULL)
{
}
//---------------------------------------------------------------------------
std::string TChapterSelectListModel::getElementAt(int i)
{
    if(m_Sce == NULL){
        return "";
    }
    return m_Sce->GetChapter(i).GetName();
}
//---------------------------------------------------------------------------
int TChapterSelectListModel::getNumberOfElements()
{
    if(m_Sce == NULL){
        return 0;
    }
    return m_Sce->GetChapterCount();
}
//---------------------------------------------------------------------------
TSceSelectPane::TSceSelectPane()
{
    const int sceareax = 4;
    const int sceareay = 4;
    const int sceareaw = 256;
    const int sceareah = 160;
    const int scbarw = g_Ini.GetIntValue(TOpenFarenIni::SCROLLBAR_WIDTH);
    m_SceSelectListBox.setPosition(0, 0);
    m_SceSelectListBox.setSize(sceareaw - scbarw, sceareah);
    m_SceSelectListBox.setListModel(&m_SceSelectListModel);
    m_SceSelectListBox.addActionListener(this);
    g_GameMaster.GetSkinManager()->ApplyListBoxSkin(&m_SceSelectListBox);

    m_SceSelectScrollArea.setScrollbarWidth(scbarw);
    m_SceSelectScrollArea.setVerticalScrollPolicy(gcn::ScrollArea::SHOW_ALWAYS);
    //m_SceSelectScrollArea.setBackgroundColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_LISTBOXBGCOLOR));
    m_SceSelectScrollArea.setBackgroundColor(g_GameMaster.GetSkinManager()->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_LISTBOXBGCOLOR));
    m_SceSelectScrollArea.setContent(&m_SceSelectListBox);
    m_SceSelectScrollArea.setPosition(sceareax, sceareay);
    m_SceSelectScrollArea.setSize(sceareaw, sceareah);
    add(&m_SceSelectScrollArea);

    m_ChapterSelectListBox.setPosition(sceareax + sceareaw + 8, sceareay);
    m_ChapterSelectListBox.setSize(288, 80);
    m_ChapterSelectListBox.setListModel(&m_ChapterSelectListModel);
    g_GameMaster.GetSkinManager()->ApplyListBoxSkin(&m_ChapterSelectListBox);
    m_ChapterSelectListBox.addActionListener(this);
    add(&m_ChapterSelectListBox);

    m_TextExplanation.setPosition(sceareax, sceareay + sceareah + 8);
    m_TextExplanation.setSize(576, 240);
    m_TextExplanation.SetFontSize(16);
    m_TextExplanation.setForegroundColor(g_Ini.GetFontColor());
    //m_TextExplanation.setBackgroundColor(g_Ini.GetPaneColor());
    m_TextExplanation.setBackgroundColor(g_GameMaster.GetSkinManager()->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE));
    add(&m_TextExplanation);

    m_ButtonCancel.setCaption("");
    m_ButtonCancel.setSize(56, 24);
    m_ButtonCancel.setPosition(524, 416);
    g_GameMaster.GetSkinManager()->ApplyButtonEnabled(&m_ButtonCancel);
    m_ButtonCancel.addActionListener(this);
    add(&m_ButtonCancel);

    m_ButtonOk.setCaption("Jn");
    m_ButtonOk.setSize(56, 24);
    m_ButtonOk.setPosition(428, 416);
    g_GameMaster.GetSkinManager()->ApplyButtonDisabled(&m_ButtonOk);
    m_ButtonOk.addActionListener(this);
    m_ButtonOk.setEnabled(false);
    add(&m_ButtonOk);

    char caption[2] = {'\0', '\0'};
    const char *group = "g";    // ǂނȂ̂ŁB
    int rbx = 288;
    m_RadioButtonList.reserve(TFarenGameMaster::GAMELEVEL_MAX);
    for(int i=0; i<TFarenGameMaster::GAMELEVEL_MAX; ++i){
        gcn::RadioButton* rb = new gcn::RadioButton();
        caption[0] = '1' + i;
        rb->setSize(28, 16);
        rb->setPosition(rbx, 116);
        rb->setCaption(caption);
        //rb->setBackgroundColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_RADIOBUTTONBG));
        //rb->setForegroundColor(g_Ini.GetFontColor());
        g_GameMaster.GetSkinManager()->ApplyRadioButtonSkin(rb);
        rb->setGroup(group);
        rb->setVisible(false);
        add(rb);
        m_RadioButtonList.push_back(rb);
        rbx += 48;
    }
    // Q[x̏I=3
    m_RadioButtonList[2]->setSelected(true);

    setSize(588, 448);
}
//---------------------------------------------------------------------------
TSceSelectPane::~TSceSelectPane()
{
    for(unsigned u=0; u<m_RadioButtonList.size(); ++u){
        remove(m_RadioButtonList[u]);
        delete m_RadioButtonList[u];
    }
}
//---------------------------------------------------------------------------
void TSceSelectPane::Init()
{
    m_SceList.clear();
    std::string dirstr = g_GameMaster.GetBasePath() + "/Scenario";
    if(!boost::filesystem::exists(dirstr)){
        g_Logger.ErrorLog((boost::format("Scenario directory(%s) not found.") % dirstr).str());
    }
    boost::filesystem::path dirpath(dirstr);
    boost::filesystem::directory_iterator end;
    for(boost::filesystem::directory_iterator itr(dirpath); itr!=end; itr++){
        if(boost::filesystem::is_directory(*itr)){
            TFarenScenario sce;
            if(sce.LoadFromDirectory(itr->string().c_str())){
                m_SceList.push_back(sce);
                m_SceDirList.push_back(itr->leaf());    // boost 1.35.0
                //m_SceDirList.push_back(itr->filename());  // boost 1.36.0
            }
        }
    }
    m_SceSelectListModel.SetSceList(&m_SceList);
}
//---------------------------------------------------------------------------
int TSceSelectPane::GetSelectedLevel() const
{
    for(unsigned u=0; u<m_RadioButtonList.size(); ++u){
        if(m_RadioButtonList[u]->isSelected()){
            return u + 1;
        }
    }
    OFERRLOG("Q[xIB肦Ȃ͂B");
    return 1;
}
//---------------------------------------------------------------------------
void TSceSelectPane::action(const gcn::ActionEvent& actionEvent)
{
    if(actionEvent.getSource() == &m_ButtonOk){
        int sceindex = m_SceSelectListBox.getSelected();
        int chapindex = m_ChapterSelectListBox.getSelected();
        if(!g_GameMaster.LoadNewGame(m_SceDirList[sceindex], chapindex)){
            TGcnMsgDlg::Show("Error", "ViIf[^̓ǂݍ݂Ɏs܂B" + g_GameMaster.GetLastError(), TGcnMsgDlg::BUTTON_OK);
            g_Logger.ErrorLog(g_GameMaster.GetLastError().c_str());
            return;
        }
        //boost::shared_ptr<TMasterSelectWindow> pw(new TMasterSelectWindow());
        //g_GuiContainerList.Add(pw);
        //gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
        //top->add(pw.get());
        //top->moveToTop(pw.get()); // v́H
        //g_GuiContainerList.MoveWidgetToCenter(pw.get());
        g_GameMaster.SetMasterSelectVisible(true);
        //setVisible(false);
    }
    else if(actionEvent.getSource() == &m_ButtonCancel){
        g_GameMaster.SetSceSelectVisible(false);
    }
    else if(actionEvent.getSource() == &m_SceSelectListBox){
        int sceindex = m_SceSelectListBox.getSelected();
        if(sceindex >= 0){
            if(!m_SceList.empty()){
                m_ChapterSelectListModel.SetSce(&m_SceList[sceindex]);
                m_ChapterSelectListBox.setSelected(-1);
                ClearChaptorSelection();
            }
        }
    }
    else if(actionEvent.getSource() == &m_ChapterSelectListBox){
        int sceindex = m_SceSelectListBox.getSelected();
        int chapindex = m_ChapterSelectListBox.getSelected();
        if(sceindex >= 0 && chapindex >= 0){
            m_TextExplanation.SetText(m_SceList[sceindex].GetChapter(chapindex).GetExplanation());
            g_GameMaster.GetSkinManager()->ApplyButtonEnabled(&m_ButtonOk);
            for(unsigned u=0; u<m_RadioButtonList.size(); ++u){
                m_RadioButtonList[u]->setVisible(true);
            }
        }
        else{
            ClearChaptorSelection();
        }
    }
}
//---------------------------------------------------------------------------
void TSceSelectPane::draw(gcn::Graphics *g)
{
    gcn::Container::draw(g);
    if(m_ButtonOk.isEnabled()){
        g->setColor(gcn::Color(255,255,255));
        g->drawText("Q[x", 272, 96);
    }
}
//---------------------------------------------------------------------------
void TSceSelectPane::ClearChaptorSelection()
{
    m_TextExplanation.SetText("");
    g_GameMaster.GetSkinManager()->ApplyButtonDisabled(&m_ButtonOk);
    for(unsigned u=0; u<m_RadioButtonList.size(); ++u){
        m_RadioButtonList[u]->setVisible(false);
    }
}
//---------------------------------------------------------------------------
TSceSelectWindow::TSceSelectWindow()
{
    setOpaque(false);
    add(&m_Pane);
    g_GameMaster.GetSkinManager()->ApplyWindowSkin(this);
    resizeToContent();
}
//---------------------------------------------------------------------------
