//---------------------------------------------------------------------------
#include "scriptapi.h"
#include "widgetset.h"
#include "gui.h"
#include "ofini.h"
#include "msgdlg.h"
#include "logger.h"
#include "ofimage.h"
#include "ofmixer.h"
#include "gamemaster.h"
namespace
{
    template<typename T>
    struct TLayerHolderLess
    {
        bool operator()(const T& lhs, const T& rhs){
            return lhs.layer < rhs.layer;
        }
    };

//    template<typename T>
//    struct TLayerHolderEqual
//    {
//        bool operator()(const T& lhs, const T& rhs){
//            return lhs.layer == rhs.layer;
//        }
//    };
}
//---------------------------------------------------------------------------
TEventPane::TEventPane()
{
}
//---------------------------------------------------------------------------
TEventPane::~TEventPane()
{
}
//---------------------------------------------------------------------------
void TEventPane::draw(gcn::Graphics *g)
{
    typedef void (TOpenFarenImage::*TDrawFunc)(gcn::Graphics* g, int x, int y, int index) const;
    static const TDrawFunc drawfunc[IMAGE_FUNCMAX] = {
        &TOpenFarenImage::DrawEvent,
        &TOpenFarenImage::DrawFace,
        &TOpenFarenImage::DrawCharaChip};
    typedef void (TOpenFarenImage::*TDrawFuncStretch)(gcn::Graphics* g, int x, int y, int index, int xzoom, int yzoom) const;
    static const TDrawFuncStretch drawfunc_stretch[IMAGE_FUNCMAX] = {
        &TOpenFarenImage::DrawEventStretch,
        &TOpenFarenImage::DrawFaceStretch,
        &TOpenFarenImage::DrawCharaChipStretch};

    g->setColor(gcn::Color(0, 0, 0));
    gcn::Rectangle r = getDimension();
    r.x = 0;
    r.y = 0;
    g->fillRectangle(r);

    g->setColor(gcn::Color(255,255,255));
    for(std::vector<TEventString>::const_iterator itr = m_EventStringList.begin(); itr != m_EventStringList.end(); ++itr){
        g->drawText(itr->str, itr->x, itr->y);
    }
    TOpenFarenImage* image = g_GameMaster.GetImage();
    for(std::vector<TEventImage>::const_iterator itr = m_EventImageList.begin(); itr != m_EventImageList.end(); ++itr){
        if(itr->xzoom == 256 && itr->yzoom == 256){
            (image->*drawfunc[itr->imagetype])(g, itr->x, itr->y, itr->index);
        }
        else{
            (image->*drawfunc_stretch[itr->imagetype])(g, itr->x, itr->y, itr->index, itr->xzoom, itr->yzoom);
        }
    }
}
//---------------------------------------------------------------------------
void TEventPane::AddString(int x, int y, int layer, const std::string& str)
{
    TEventString es;
    es.x = x;
    es.y = y;
    es.layer = layer;
    es.str = str;
    AddString(es);
//    m_EventStringList.push_back(es);
}
//---------------------------------------------------------------------------
void TEventPane::ClearString(int layer)
{
    if(layer == -1){
        m_EventStringList.clear();
    }
    else{
        TEventString es;
        es.layer = layer;
        TEventStringList::iterator itr_l = std::lower_bound(m_EventStringList.begin(), m_EventStringList.end(), es, TLayerHolderLess<TEventString>());
        if(itr_l != m_EventStringList.end()){
            TEventStringList::iterator itr_u = std::upper_bound(m_EventStringList.begin(), m_EventStringList.end(), es, TLayerHolderLess<TEventString>());
            m_EventStringList.erase(itr_l, itr_u);
        }
    }
}
//---------------------------------------------------------------------------
void TEventPane::AddImage(int x, int y, int layer, TImageType imgtype, int index, int xzoom, int yzoom)
{
    TEventImage ei;
    ei.x = x;
    ei.y = y;
    ei.layer = layer;
    ei.imagetype = imgtype;
    ei.index = index;
    ei.xzoom = xzoom;
    ei.yzoom = yzoom;
    AddImage(ei);
//    m_EventImageList.push_back(ei);
}
//---------------------------------------------------------------------------
void TEventPane::ClearImage(int layer)
{
    if(layer == -1){
        m_EventImageList.clear();
    }
    else{
        TEventImage ei;
        ei.layer = layer;
        TEventImageList::iterator itr_l = std::lower_bound(m_EventImageList.begin(), m_EventImageList.end(), ei, TLayerHolderLess<TEventImage>());
        if(itr_l != m_EventImageList.end()){
            TEventImageList::iterator itr_u = std::upper_bound(m_EventImageList.begin(), m_EventImageList.end(), ei, TLayerHolderLess<TEventImage>());
            m_EventImageList.erase(itr_l, itr_u);
        }
    }
}
//---------------------------------------------------------------------------
void TEventPane::AddString(const TEventString& es)
{
    TEventStringList::iterator itr = std::lower_bound(m_EventStringList.begin(), m_EventStringList.end(), es, TLayerHolderLess<TEventString>());
    m_EventStringList.insert(itr, es);
}
//---------------------------------------------------------------------------
void TEventPane::AddImage(const TEventImage& ei)
{
    TEventImageList::iterator itr = std::lower_bound(m_EventImageList.begin(), m_EventImageList.end(), ei, TLayerHolderLess<TEventImage>());
    m_EventImageList.insert(itr, ei);
}
//---------------------------------------------------------------------------
TEventWindow::TEventWindow()
{
    add(&m_EventPane);
    setOpaque(false);
}
//---------------------------------------------------------------------------
TEventWindow::~TEventWindow()
{
}
//---------------------------------------------------------------------------
//TScriptAPI::TScriptAPI() :
//    m_EventWindow(NULL)
TScriptAPI::TScriptAPI(TGcnWidgetSet *wset) :
    m_WidgetSet(wset)
{
//    m_EventWindow.add(&m_EventPane);
//    m_EventWindow.setOpaque(false);
//    gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
//    top->add(&m_EventWindow);
//    top->moveToTop(&m_EventWindow);
    g_GuiContainerList.GetGui()->getTop()->addMouseListener(this);
}
//---------------------------------------------------------------------------
TScriptAPI::~TScriptAPI()
{
/*    for(unsigned u=0; u<m_EventWindowList.size(); ++u){
        //g_GuiContainerList.Remove(m_EventWindowList[u]);
        m_WidgetSet->Remove(m_EventWindowList[u]);
    }
    m_EventWindowList.clear();*/
    g_GuiContainerList.GetGui()->getTop()->removeMouseListener(this);
}
//---------------------------------------------------------------------------
/*void TScriptAPI::ShowMessageTimer(const std::string& title, const std::string& msg, bool timeout)
{
    int to = timeout ? g_Ini.GetMessageTimeoutFrame() : 0;
    TGcnMsgDlg::Show(title, msg, TGcnMsgDlg::BUTTON_OK, to);
}*/
//---------------------------------------------------------------------------
int TScriptAPI::NewWindow()
{
    gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
    boost::shared_ptr<TEventWindow> pw(new TEventWindow);
    //g_GuiContainerList.Add(pw);
    m_WidgetSet->Add(pw);
    top->add(pw.get());
    pw->addMouseListener(this);
    int handle = m_EventWindowList.size();
    m_EventWindowList.push_back(pw.get());
    m_ScriptEvent.resize(m_EventWindowList.size());
    return handle;
}
//---------------------------------------------------------------------------
TEventWindow* TScriptAPI::GetWindow(int handle)
{
    if(handle >= 0 && handle <= static_cast<signed>(m_EventWindowList.size())){
        return m_EventWindowList[handle];
    }
    else{
        return NULL;
    }
}
//---------------------------------------------------------------------------
void TScriptAPI::SetWindowSize(int handle, int w, int h)
{
    m_EventWindowList[handle]->GetEventPane().setSize(w, h);
    m_EventWindowList[handle]->resizeToContent();
}
//---------------------------------------------------------------------------
void TScriptAPI::SetWindowPosition(int handle, int x, int y)
{
    TEventWindow* win = m_EventWindowList[handle];
    gcn::Widget* top = g_GuiContainerList.GetGui()->getTop();
    if(x == -1){
        x = (top->getWidth() - win->getWidth()) / 2;
    }
    if(y == -1){
        y = (top->getHeight() - win->getHeight()) / 2;
    }
    m_EventWindowList[handle]->setPosition(x, y);
}
//---------------------------------------------------------------------------
void TScriptAPI::ShowWindow(int handle)
{
    m_EventWindowList[handle]->setVisible(true);
    gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
    top->moveToTop(m_EventWindowList[handle]);
}
//---------------------------------------------------------------------------
void TScriptAPI::HideWindow(int handle)
{
    m_EventWindowList[handle]->setVisible(false);
}
//---------------------------------------------------------------------------
void TScriptAPI::HideAllWindow()
{
    for(unsigned u=0; u<m_EventWindowList.size(); ++u){
        m_EventWindowList[u]->setVisible(false);
    }
}
//---------------------------------------------------------------------------
/*void TScriptAPI::ShowDialog(int x, int y, const std::string& msg, int btn)
{
    TGcnMsgDlg::Show(title, msg, btn);
}*/
//---------------------------------------------------------------------------
int TScriptAPI::LoadEventImage(const std::string& imgfile)
{
    //return g_GameMaster.GetImage()->LoadEvent(g_GameMaster.GetScenarioPath() + "/Picture/" + imgfile);
    std::string path;
    if(g_GameMaster.FindEventImagePath(imgfile, path)){
        return g_GameMaster.GetImage()->LoadEvent(path);
    }
    return -1;
}
//---------------------------------------------------------------------------
void TScriptAPI::AddImage(int handle, int x, int y, int layer, TImageType imgtype, int index)
{
    TEventPane& pane = m_EventWindowList[handle]->GetEventPane();
    // ʂȋC邪x,yɃZ^Ow肷邱Ƃ͏Ȃƌł̏ŁB
    if(x == -1){
        int imgw, imgh;
        g_GameMaster.GetImage()->GetEventSize(index, imgw, imgh);
        x = (pane.getWidth() - imgw) / 2;
    }
    if(y == -1){
        int imgw, imgh;
        g_GameMaster.GetImage()->GetEventSize(index, imgw, imgh);
        y = (pane.getHeight() - imgh) / 2;
    }
    pane.AddImage(x, y, layer, imgtype, index);
}
//---------------------------------------------------------------------------
void TScriptAPI::ClearImage(int handle, int layer)
{
    m_EventWindowList[handle]->GetEventPane().ClearImage(layer);
}
//---------------------------------------------------------------------------
void TScriptAPI::DrawCharaFaceOrChip(int handle, int x, int y, int layer, int faceindex, int chipindex)
{
    TEventPane& pane = m_EventWindowList[handle]->GetEventPane();
    if(faceindex != -1){
        pane.AddImage(x, y, layer, IMAGE_FACE, faceindex);
    }
    else{
        pane.AddImage(x, y, layer, IMAGE_CHARACHIP, chipindex, 768, 768);
    }
}
//---------------------------------------------------------------------------
void TScriptAPI::AddString(int handle, int x, int y, int layer, const std::string& str)
{
    TEventPane& pane = m_EventWindowList[handle]->GetEventPane();
    if(x == -1){
        x = (pane.getWidth() - str.size()*TGuiContainerList::FONT_SIZE_NORMAL/2) / 2;
    }
    if(y == -1){
        y = (pane.getHeight() - TGuiContainerList::FONT_SIZE_NORMAL) / 2;
    }
    pane.AddString(x, y, layer, str);
}
//---------------------------------------------------------------------------
void TScriptAPI::ClearString(int handle, int layer)
{
    m_EventWindowList[handle]->GetEventPane().ClearString(layer);
}
//---------------------------------------------------------------------------
bool TScriptAPI::PopEvent(int handle, TScriptEvent& ev)
{
    if(!m_ScriptEvent[handle].empty()){
        ev = m_ScriptEvent[handle].front();
        m_ScriptEvent[handle].pop();
        return true;
    }
    return false;
}
//---------------------------------------------------------------------------
bool TScriptAPI::PopEvent(TScriptEvent& ev)
{
/*    if(g_GuiContainerList.MousePressedOnThisFrame()){
        ev.x = 0;
        ev.y = 0;
        ev.eventtype = EVENT_MOUSELPRESSED;
        return true;
    }*/
    if(!m_GlobalScriptEvent.empty()){
        ev = m_GlobalScriptEvent.front();
        m_GlobalScriptEvent.pop();
        return true;
    }
    return false;
}
//---------------------------------------------------------------------------
void TScriptAPI::ClearEvent()
{
    unsigned sz = m_ScriptEvent.size();
    m_ScriptEvent.clear();
    m_ScriptEvent.resize(sz);
    while(!m_GlobalScriptEvent.empty()){
        m_GlobalScriptEvent.pop();
    }
}
//---------------------------------------------------------------------------
void TScriptAPI::mousePressed(gcn::MouseEvent &mouseEvent)
{
    int index = -1;
    for(std::vector<TEventWindow*>::iterator itr = m_EventWindowList.begin(); itr != m_EventWindowList.end(); ++itr){
        if(&(*itr)->GetEventPane() == mouseEvent.getSource()){
            index = std::distance(m_EventWindowList.begin(), itr);
            break;
        }
    };
    TScriptEvent evt;
    evt.x = mouseEvent.getX();
    evt.y = mouseEvent.getY();
    evt.eventtype = (mouseEvent.getButton() == gcn::MouseEvent::LEFT) ? EVENT_MOUSELPRESSED : EVENT_MOUSERPRESSED;
    if(index != -1){
        m_ScriptEvent[index].push(evt);
    }
    else{
        m_GlobalScriptEvent.push(evt);
    }
}
//---------------------------------------------------------------------------
int TScriptAPI::GetTick()
{
    return SDL_GetTicks();
}
//---------------------------------------------------------------------------
bool TScriptAPI::PlayMusic(const std::string& musicfile)
{
    std::string dirpath;
    if(!g_GameMaster.FindPlacedMusicSoundFile(musicfile, dirpath)){
        g_Logger.ErrorLog((boost::format("y %s ܂łB") % musicfile).str());
        return false;
    }
    TOpenFarenMixer *mixer = g_GameMaster.GetMixer();
    int music = mixer->LoadMusic(dirpath + "/" + musicfile);
    if(music == -1){
        g_Logger.ErrorLog((boost::format("y %s ̃[hɎs܂B") % musicfile).str());
        return false;
    }
    mixer->PlayMusicWithFadeout(music);
    return true;
}
//---------------------------------------------------------------------------
