//---------------------------------------------------------------------------
#ifndef strategyifH
#define strategyifH

#include <string>
#include <vector>
#include <queue>
//#include "ofdata.h"
#include <guichan.hpp>
//#include "eventeatcontainer.h"
//---------------------------------------------------------------------------
struct TEventString
{
    int x;
    int y;
    int layer;
    std::string str;
};
typedef std::vector<TEventString> TEventStringList;
//---------------------------------------------------------------------------
enum TImageType{
    IMAGE_EVENT,
    IMAGE_FACE,
    IMAGE_CHARACHIP,
    IMAGE_FUNCMAX
};
//---------------------------------------------------------------------------
struct TEventImage
{
    int x;
    int y;
    int layer;
    TImageType imagetype;
    int index;
    int xzoom, yzoom;
};
typedef std::vector<TEventImage> TEventImageList;
//---------------------------------------------------------------------------
class TEventPane : public gcn::Widget
{
public:
    TEventPane();
    virtual ~TEventPane();
    void AddString(int x, int y, int layer, const std::string& str);
    void ClearString(int layer);
    void AddImage(int x, int y, int layer, TImageType imgtype, int index, int xzoom=256, int yzoom=256);
    void ClearImage(int layer);
    virtual void draw(gcn::Graphics *g);
private:
    TEventStringList m_EventStringList;
    TEventImageList m_EventImageList;

    void AddString(const TEventString& es);
    void AddImage(const TEventImage& ei);
};
//---------------------------------------------------------------------------
class TEventWindow : public gcn::Window
{
private:
    TEventPane m_EventPane;
public:
    TEventWindow();
    virtual ~TEventWindow();
    TEventPane& GetEventPane() { return m_EventPane; }
};
//---------------------------------------------------------------------------
enum TScriptEventType{
    EVENT_MOUSELPRESSED,
    EVENT_MOUSERPRESSED,
    EVENT_MOUSEMOVED,
    SCRIPTEVENT_MAX
};
//---------------------------------------------------------------------------
struct TScriptEvent
{
    int x;
    int y;
    TScriptEventType eventtype;
};
typedef std::queue<TScriptEvent> TScriptEventQueue;
//---------------------------------------------------------------------------
class TGcnWidgetSet;
class TScriptAPI : gcn::MouseListener
{
public:
    TScriptAPI(TGcnWidgetSet *wset);
    ~TScriptAPI();
//    void ShowMessageTimer(const std::string& title, const std::string& msg, bool timeout);
    int NewWindow();
    TEventWindow* GetWindow(int handle);
    void SetWindowSize(int handle, int w, int h);
    void SetWindowPosition(int handle, int x, int y);
    void ShowWindow(int handle);
    void HideWindow(int handle);
    void HideAllWindow();
    bool IsValidWindow(int handle) const { return handle >= 0 && handle < m_EventWindowList.size(); }
    bool IsVisibleWindow(int handle) const { return m_EventWindowList[handle]->isVisible(); }
    //void ShowDialog(int x, int y, const std::string& msg, int btn);
    //bool IsVisibleDialog() const { return TGcnMsgDlg::IsVisible(); }
    //int GetDialogSelected() const { return TGcnMsgDlg::IsVisible(); }
    int LoadEventImage(const std::string& imgfile);
    void AddImage(int handle, int x, int y, int layer, TImageType imgtype, int index);
    void ClearImage(int handle, int layer);
    void DrawCharaFaceOrChip(int handle, int x, int y, int layer, int faceindex, int chipindex);
    void AddString(int handle, int x, int y, int layer, const std::string& str);
    void ClearString(int handle, int layer);
    bool PopEvent(int handle, TScriptEvent& ev);
    bool PopEvent(TScriptEvent& ev);
    void ClearEvent();
    int GetTick();
    bool PlayMusic(const std::string& musicfile);
    virtual void mousePressed(gcn::MouseEvent &mouseEvent);
private:
    TGcnWidgetSet *m_WidgetSet;
    std::vector<TEventWindow*> m_EventWindowList;
    std::vector<TScriptEventQueue> m_ScriptEvent;
    TScriptEventQueue m_GlobalScriptEvent;
};
//---------------------------------------------------------------------------
#endif
