//---------------------------------------------------------------------------
#ifndef sysmenuH
#define sysmenuH

#include <guichan.hpp>
#include "SDL.h"    // SDLKey
#include "gcnwindow.h"

//---------------------------------------------------------------------------
class TStartMenuPane : public gcn::Container
{
private:
    gcn::Button m_ButtonNew;
    gcn::Button m_ButtonLoad;
    gcn::Button m_ButtonConfig;
public:
    TStartMenuPane();
    virtual ~TStartMenuPane();
    gcn::Button& GetButtonNew() { return m_ButtonNew; }
    gcn::Button& GetButtonLoad() { return m_ButtonLoad; }
    gcn::Button& GetButtonConfig() { return m_ButtonConfig; }
};
//---------------------------------------------------------------------------
class TStartMenuWindow : public TGcnWindow, gcn::ActionListener
{
private:
    TStartMenuPane m_StartMenuPane;
public:
    TStartMenuWindow();
    virtual ~TStartMenuWindow();
    virtual void action(const gcn::ActionEvent& actionEvent);
};
//---------------------------------------------------------------------------
class TConfigWindowPane : public gcn::Container, gcn::ActionListener
{
public:
    TConfigWindowPane();
    virtual ~TConfigWindowPane();
    virtual void draw(gcn::Graphics *g);
    virtual void action(const gcn::ActionEvent& actionEvent);
    void InitUI();
    void UIToIni() const;
private:
    gcn::Label m_LabelGuidance;
    gcn::Button m_ButtonWidthDec;
    gcn::Button m_ButtonWidthInc;
    gcn::Button m_ButtonW800;
    gcn::Button m_ButtonW1024;
    gcn::Button m_ButtonW1280;
    gcn::Button m_ButtonHeightDec;
    gcn::Button m_ButtonHeightInc;
    gcn::Button m_ButtonH640;
    gcn::Button m_ButtonH800;
    gcn::Button m_ButtonH1024;
    gcn::CheckBox m_CheckBoxDisableSwapInterval;
    int m_Width;
    int m_Height;
    void UpdateUI();
};
//---------------------------------------------------------------------------
class TSelectPane : public gcn::Widget, gcn::MouseListener
{
public:
    TSelectPane();
    virtual ~TSelectPane() {}
    virtual void draw(gcn::Graphics *g);
    virtual void mousePressed(gcn::MouseEvent &mouseEvent);
    void ClearItem();
    void AddItem(const std::string& str);
    int GetSelected() const;
    void SetSelected(int index);
private:
    int m_Selected;
    std::vector<std::string> m_Items;
};
//---------------------------------------------------------------------------
class TConfigBattlePane : public gcn::Container, gcn::ActionListener
{
public:
    TConfigBattlePane();
    virtual ~TConfigBattlePane();
    virtual void action(const gcn::ActionEvent& actionEvent);
    void InitUI();
    void UIToIni() const;
private:
    gcn::Label m_LabelSpeed1;
    gcn::Label m_LabelSpeed2;
    TSelectPane m_SelectPaneSpeed;
    gcn::CheckBox m_CheckBoxWatch;
    gcn::CheckBox m_CheckBoxOnlyLarge;
    gcn::CheckBox m_CheckBoxOnlyMaster;
    gcn::CheckBox m_CheckBoxNoNeutral;
    gcn::CheckBox m_CheckBoxMaxSpeed;

    void UpdateUI();
};
//---------------------------------------------------------------------------
class TColorPane : public gcn::Widget
{
public:
    TColorPane() {}
    virtual ~TColorPane() {}
    virtual void draw(gcn::Graphics *g);
};
//---------------------------------------------------------------------------
class TShortcutListModel : public gcn::ListModel
{
public:
    TShortcutListModel();
    virtual ~TShortcutListModel() {}
    virtual std::string getElementAt(int i);
    virtual int getNumberOfElements();
};
//---------------------------------------------------------------------------
class TConfigShortcutPane : public gcn::Container, gcn::ActionListener
{
public:
    TConfigShortcutPane();
    virtual ~TConfigShortcutPane();
    virtual void action(const gcn::ActionEvent& actionEvent);
    virtual void logic();
    std::string KeysymToString(SDLKey key);
private:
    bool m_WaitKey;
    gcn::Button m_ButtonChange;
    gcn::Button m_ButtonCancel;
    TColorPane m_ColorPaneLabelBack;
    gcn::Label m_Label;
    TShortcutListModel m_ShortcutListModel;
    gcn::ListBox m_ShortcutListBox;
};
//---------------------------------------------------------------------------
class TConfigScreenPane : public gcn::Container, gcn::ActionListener
{
public:
    TConfigScreenPane();
    virtual ~TConfigScreenPane();
    virtual void draw(gcn::Graphics *g);
    virtual void action(const gcn::ActionEvent& actionEvent);
    void InitUI();
    void UIToIni() const;
private:
    gcn::Label m_LabelGuidance;
    gcn::Button m_ButtonLeftInc;
    gcn::Button m_ButtonLeftDec;
    gcn::Button m_ButtonRightInc;
    gcn::Button m_ButtonRightDec;
    gcn::Button m_ButtonTopInc;
    gcn::Button m_ButtonTopDec;
    gcn::Button m_ButtonBottomInc;
    gcn::Button m_ButtonBottomDec;
    int m_SlideL;
    int m_SlideR;
    int m_SlideT;
    int m_SlideB;
};
//---------------------------------------------------------------------------
class TConfigMusicSoundPane : public gcn::Container
{
public:
    TConfigMusicSoundPane();
    virtual ~TConfigMusicSoundPane();
    void InitUI();
    void UIToIni() const;
private:
    gcn::Label m_LabelGuidance;
    gcn::CheckBox m_CheckBoxMusic;
    gcn::CheckBox m_CheckBoxSound;

    void UpdateUI();
};
//---------------------------------------------------------------------------
class TConfigPane : public gcn::Container
{
public:
    TConfigPane();
    virtual ~TConfigPane();
    virtual void logic();
    gcn::Button& GetButtonOk() { return m_ButtonOk; }
    gcn::Button& GetButtonCancel() { return m_ButtonCancel; }
    void InitUI();
    void UIToIni() const;
private:
    TConfigBattlePane m_BattlePane;
    TConfigShortcutPane m_ShortcutPane;
    TConfigWindowPane m_WindowPane;
    TConfigScreenPane m_ScreenPane;
    TConfigMusicSoundPane m_MusicSoundPane;
    gcn::TabbedArea m_TabbedArea;
    gcn::Button m_ButtonOk;
    gcn::Button m_ButtonCancel;
};
//---------------------------------------------------------------------------
class TConfigWindow : public TGcnWindow, gcn::ActionListener
{
public:
    TConfigWindow();
    virtual ~TConfigWindow();
    void InitUI();
    virtual void action(const gcn::ActionEvent& actionEvent);
private:
    TConfigPane m_ConfigPane;
};
//---------------------------------------------------------------------------
enum TSysMenuMode{
    SYSMENU_WORLD,
    SYSMENU_BATTLE
};
//---------------------------------------------------------------------------
class TSysMenuPane : public gcn::Container
{
private:
    gcn::Button m_ButtonSave;
    gcn::Button m_ButtonLoad;
    gcn::Button m_ButtonConfig;
public:
    TSysMenuPane();
    virtual ~TSysMenuPane();
    gcn::Button& GetButtonSave() { return m_ButtonSave; }
    gcn::Button& GetButtonLoad() { return m_ButtonLoad; }
    gcn::Button& GetButtonConfig() { return m_ButtonConfig; }
    void SetMode(TSysMenuMode mode);
};
//---------------------------------------------------------------------------
class TSysMenuWindow : public TGcnWindow, gcn::ActionListener
{
private:
    TSysMenuPane m_SysMenuPane;
public:
    TSysMenuWindow();
    virtual ~TSysMenuWindow();
    virtual void action(const gcn::ActionEvent& actionEvent);
    void SetMode(TSysMenuMode mode);
};
//---------------------------------------------------------------------------
#endif
