//---------------------------------------------------------------------------
#include "worldmap.h"
#include "battlemap.h"
#include "msgdlg.h"
#include "gui.h"
#include "ofini.h"
#include "gamemaster.h"
#include "gcnskinmanager.h"
#include "widgetset.h"
#include "ofimage.h"
#include "ofmixer.h"
#include "ofgame.h"
#include "gridimpl.h"
#include "drawutil.h"
#include "logger.h"
#include "boostformat.h"
#include "porting.h"

namespace
{
    const char* LUAFUNC_SYSSTART = "on_system_start";
    const char* LUAFUNC_PROPOSEALLIANCE = "on_proposed_alliance";
    const char* XMLELM_LUAEVENTTOP = "LuaEvent";
}
//---------------------------------------------------------------------------
TWorldMenuPane::TWorldMenuPane()
{
    m_ButtonAbortWatch.setCaption("ϐ풆~");
    m_ButtonAbortWatch.setSize(96, 24);
    m_ButtonAbortWatch.setPosition(8, 8);
    g_GameMaster.GetSkinManager()->ApplyButtonEnabled(&m_ButtonAbortWatch);
    add(&m_ButtonAbortWatch);

    //setBaseColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_WINDOW));
    g_GameMaster.GetSkinManager()->ApplyGrayPaneSkin(this);
    setSize(112, 38);
}
//---------------------------------------------------------------------------
TWorldMenuWindow::TWorldMenuWindow()
{
    add(&m_MenuPane);
    setOpaque(false);
    //setBaseColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_WINDOW));
    g_GameMaster.GetSkinManager()->ApplyWindowSkin(this);
    resizeToContent();
}
//---------------------------------------------------------------------------
TWorldMapPane::TWorldMapPane() :
    m_Tick(0)
//    m_BrightArea(-1)
{
}
//---------------------------------------------------------------------------
TWorldMapPane::~TWorldMapPane()
{
}
//---------------------------------------------------------------------------
void TWorldMapPane::UpdateAreaCount()
{
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    int areacount = game->GetAreaInstanceCount();
    m_BrightAreaList.resize(areacount);
    std::fill(m_BrightAreaList.begin(), m_BrightAreaList.end(), 0);

    m_CircleAreaList.resize(areacount);
    std::fill(m_CircleAreaList.begin(), m_CircleAreaList.end(), 0);
}
//---------------------------------------------------------------------------
void TWorldMapPane::AddBrightEdge(int area1, int area2)
{
    //m_BrightEdgeTick = 0;
    m_BrightAreaTick = 0;
    m_BrightEdgeList.push_back(std::pair<int, int>(area1, area2));
    m_BrightEdgeList.push_back(std::pair<int, int>(area2, area1));
}
//---------------------------------------------------------------------------
void TWorldMapPane::ClearBrightEdge()
{
    m_BrightEdgeList.clear();
}
//---------------------------------------------------------------------------
//void TWorldMapPane::SetBrightArea(int area)
//{
//    m_BrightAreaTick = 0;
//    //m_BrightArea = area;
//    AddBrightArea(area, 0xffff0000);
//}
//---------------------------------------------------------------------------
void TWorldMapPane::AddBrightArea(int area, unsigned color)
{
    m_BrightAreaTick = 0;
    m_BrightAreaList[area] = color;
}
//---------------------------------------------------------------------------
void TWorldMapPane::ClearBrightArea()
{
    //m_BrightArea = -1;
    std::fill(m_BrightAreaList.begin(), m_BrightAreaList.end(), 0);
}
//---------------------------------------------------------------------------
void TWorldMapPane::AddCircleArea(int area, unsigned color)
{
    m_CircleAreaList[area] = color;
}
//---------------------------------------------------------------------------
void TWorldMapPane::ClearCircleArea()
{
    std::fill(m_CircleAreaList.begin(), m_CircleAreaList.end(), 0);
}
//---------------------------------------------------------------------------
void TWorldMapPane::AddDepartureChara(int x, int y, int ctype)
{
    m_DepatureCharaList.push_back(boost::tuple<int, int, int>(x, y, ctype));
}
//---------------------------------------------------------------------------
void TWorldMapPane::SetDepartureCharaPosition(int index, int x, int y)
{
    boost::tuple<int, int, int>& e = m_DepatureCharaList[index];
    e.get<0>() = x;
    e.get<1>() = y;
}
//---------------------------------------------------------------------------
void TWorldMapPane::ClearDepartureChara()
{
    m_DepatureCharaList.clear();
}
//---------------------------------------------------------------------------
void TWorldMapPane::logic()
{
    TGcnImagePaneLarge::logic();
    ++m_Tick;
    //++m_BrightEdgeTick;
    ++m_BrightAreaTick;
}
//---------------------------------------------------------------------------
void TWorldMapPane::draw(gcn::Graphics *g)
{
    const int fps = g_Ini.GetIntValue(TOpenFarenIni::WINDOW_FPS);
    int flaganim = m_Tick/(30*fps/60);

    int absx, absy;
    getAbsolutePosition(absx, absy);

    TGcnImagePaneLarge::draw(g);
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TOpenFarenImage* image = g_GameMaster.GetImage();
    // ӕ`B
    for(int i = 0; i < game->GetAreaInstanceCount(); i++){
        const TAreaInstance& area = game->GetAreaInstance(i);
        if(area.m_ignore){
            continue;
        }
        for(unsigned u = 0; u < area.m_neighbor.size(); u++){
            const TAreaInstance& narea = game->GetAreaInstance(area.m_neighbor[u]);
            if(narea.m_ignore){
                continue;
            }
            int x1 = absx + area.m_location.x + 6;
            int y1 = absy + area.m_location.y + 30;
            int x2 = absx + narea.m_location.x + 6;
            int y2 = absy + narea.m_location.y + 30;
            gcn::Color c = g_Ini.GetWorldMapEdgeColor();
            // ӂɎw肳ĂFςB
            if(std::find(m_BrightEdgeList.begin(), m_BrightEdgeList.end(), std::pair<int, int>(i, area.m_neighbor[u])) != m_BrightEdgeList.end()){
                //int bright = 191 + (g_GuiContainerList.GetSin(m_BrightEdgeTick % g_GuiContainerList.GetTrigonometricMax()) >> 2);
                int bright = 191 + (g_GuiContainerList.GetSin(m_BrightAreaTick % g_GuiContainerList.GetTrigonometricMax()) >> 2);
                c = c + gcn::Color(bright, bright, bright);
            }
            g->setColor(gcn::Color(0,0,0,128));
            DrawLine(x1, y1, x2, y2, 6);
            g->setColor(c);
            DrawLine(x1, y1, x2, y2, 4);
//            if(abs(x1 - x2) > abs(y1 - y2)){
//                g->drawLine(x1, y1-1, x2, y2-1);
//                g->drawLine(x1, y1+1, x2, y2+1);
//                g->drawLine(x1, y1, x2, y2);
//                DrawLine(x1, y1-1, x2, y2-1, 1);
//                DrawLine(x1, y1+1, x2, y2+1, 1);
//            }
//            else{
//                g->drawLine(x1-1, y1, x2-1, y2);
//                g->drawLine(x1+1, y1, x2+1, y2);
//                g->drawLine(x1, y1, x2, y2);
//                DrawLine(x1-1, y1, x2-1, y2, 1);
//                DrawLine(x1+1, y1, x2+1, y2, 1);
//            }
        }
    }
    // `B
    for(int i = 0; i < game->GetAreaInstanceCount(); i++){
        const TAreaInstance& area = game->GetAreaInstance(i);
        if(area.m_ignore){
            continue;
        }
        if(area.m_master != -1){
            int index = game->GetMasterInstance(area.m_master).m_type;
            //image->DrawFlag(g, area.m_location.x, area.m_location.y, data->GetMaster(index).m_flagindex, m_Tick/30);
            int flagindex = data->GetMaster(index).m_flagindex;
            if(flagindex != -1){
                image->DrawFlag(g, area.m_location.x, area.m_location.y, flagindex, flaganim);
            }
        }
        else{
            //image->DrawFlag(g, area.m_location.x, area.m_location.y, data->GetNoMasterFlagIndex(), m_Tick/30);
            image->DrawFlag(g, area.m_location.x, area.m_location.y, data->GetNoMasterFlagIndex(), flaganim);
        }
        // GA̕`B

        int namew = area.m_name.size() * m_FontSize / 2;
        int namex = area.m_location.x - namew/2;
        int namey = area.m_location.y + 33;
        image->DrawAreaName(g, namex, namey, area.m_nameimage);
    }
    // GA\`B
    int w = 32 + (g_GuiContainerList.GetSin(m_BrightAreaTick % g_GuiContainerList.GetTrigonometricMax()) >> 5);
    unsigned areas = m_BrightAreaList.size();
    for(unsigned u = 0; u < areas; ++u){
        unsigned c = m_BrightAreaList[u];
        if(c){
            const TAreaInstance& area = game->GetAreaInstance(u);
            g->setColor(gcn::Color(((c>>16) & 0xff), ((c>>8) & 0xff), (c & 0xff), ((c>>24) & 0xff)));
            DrawCircle(absx + area.m_location.x + 6, absy + area.m_location.y + 30, w, 4.0f);
        }
        c = m_CircleAreaList[u];
        if(c){
            const TAreaInstance& area = game->GetAreaInstance(u);
            g->setColor(gcn::Color(((c>>16) & 0xff), ((c>>8) & 0xff), (c & 0xff), ((c>>24) & 0xff)));
            DrawCircle(absx + area.m_location.x + 6, absy + area.m_location.y + 30, 32, 4.0f);
        }
    }
    // owL`B
    for(TDepatureCharaList::const_iterator itr = m_DepatureCharaList.begin(); itr != m_DepatureCharaList.end(); ++itr){
        image->DrawCharaChip(g, itr->get<0>(), itr->get<1>(), itr->get<2>());
    }
}
//---------------------------------------------------------------------------
TWorldMapWindow::TWorldMapWindow()
{
    //setBaseColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_WINDOW));
    g_GameMaster.GetSkinManager()->ApplyWindowSkin(this);
    setOpaque(false);
    setTitleBarHeight(24);
}
//---------------------------------------------------------------------------
TWorldMapWindow::~TWorldMapWindow()
{
}
//---------------------------------------------------------------------------
bool TWorldMapWindow::Show()
{
    const TOpenFarenImage *image = g_GameMaster.GetImage();
    m_WorldMapPane.SetImage(image->GetFieldMapImage(), image->GetFieldMapImageXCount());
    const TOpenFarenData* data = g_GameMaster.GetData();
    int fw, fh;
    data->GetFieldMapSize(&fw, &fh);
    //m_WorldMapPane.SetRegion(0, 0, fw, fh);
    m_WorldMapPane.setSize(fw, fh);

    const gcn::Widget* top = g_GuiContainerList.GetGui()->getTop();
    int wmargin = 128;
    int hmargin = 32;
    int w = top->getWidth() - wmargin;
    int h = top->getHeight() - hmargin;
    m_ScrollArea.setSize(w, h);
    m_ScrollArea.setScrollbarWidth(g_Ini.GetIntValue(TOpenFarenIni::SCROLLBAR_WIDTH));
    m_ScrollArea.setContent(&m_WorldMapPane);
    m_ScrollArea.setOpaque(false);
    add(&m_ScrollArea);
    resizeToContent();
    //int x = (top->getWidth() - getWidth()) / 2;
    int x = top->getWidth() - getWidth();
    int y = (top->getHeight() - getHeight()) / 2;
    setPosition(x, y);
    return true;
}
//---------------------------------------------------------------------------
void TWorldMapWindow::GetScrollAmount(int& x, int& y)
{
    x = m_ScrollArea.getHorizontalScrollAmount();
    y = m_ScrollArea.getVerticalScrollAmount();
}
//---------------------------------------------------------------------------
void TWorldMapWindow::SetScrollAmount(int x, int y)
{
    m_ScrollArea.setScrollAmount(x, y);
}
//---------------------------------------------------------------------------
void TWorldMapWindow::SetScrollCenter(int x, int y)
{
    //int panew = m_WorldMapPane.getWidth();
    //int paneh = m_WorldMapPane.getHeight();
    int ax = x - m_ScrollArea.getWidth()/2;
    int ay = y - m_ScrollArea.getHeight()/2;
    if(ax < 0){
        ax = 0;
    }
    if(ax > m_ScrollArea.getHorizontalMaxScroll()){
        ax = m_ScrollArea.getHorizontalMaxScroll();
    }
    if(ay < 0){
        ay = 0;
    }
    if(ay > m_ScrollArea.getVerticalMaxScroll()){
        ay = m_ScrollArea.getVerticalMaxScroll();
    }
    m_ScrollArea.setScrollAmount(ax, ay);
}
//---------------------------------------------------------------------------
void TWorldMapWindow::draw(gcn::Graphics *g)
{
    static const gcn::Color white(255, 255, 255);
    gcn::Color dark = g_GameMaster.GetSkinManager()->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE);

    TGcnWindow::draw(g);
    //g->setColor(g_Ini.GetPaneColor());
    g->setColor(dark);
    g->fillRectangle(gcn::Rectangle(4, 3, 204, 19));
    g->fillRectangle(gcn::Rectangle(getWidth() - 160, 3, 144, 19));

    const TOpenFarenData* data = g_GameMaster.GetData();
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    g->setColor(white);
    int curmaster = game->GetCurrentMasterIndex();
    if(curmaster != -1){
        const std::string& master = data->GetMaster(game->GetMasterInstance(curmaster).m_type).m_name;
        g->drawText(master.c_str(), 12, 5);
        g->drawText(master.c_str(), 13, 5);
    }

    std::string turn = (boost::format("%4d^[") % game->GetTurn()).str();
    g->drawText(turn.c_str(), getWidth() - 144, 5);
    g->drawText(turn.c_str(), getWidth() - 143, 5);

    // KC_X\B
    if(!m_Guidance.empty()){
        //g->setColor(g_Ini.GetPaneColor());
        g->setColor(dark);
        int w = m_Guidance.size() * m_FontSize/2;
        g->fillRectangle(gcn::Rectangle(getWidth()/2 - w/2 - 12, 2, w+24, 19));

        g->setColor(white);
        g->drawText(m_Guidance.c_str(), getWidth()/2 - w/2, 4);
        g->drawText(m_Guidance.c_str(), getWidth()/2 - w/2 + 1, 4);
    }
}
//---------------------------------------------------------------------------
void TWorldMapWindow::logic()
{
    gcn::Window::logic();
    // ŌĂԂ̂œK͕sB
    if(m_Logic != NULL){
        m_Logic->DoLogic();
    }
}
//---------------------------------------------------------------------------
TWorldMapLogic::TWorldMapLogic(TGcnWidgetSet *wset) :
    m_WidgetSet(wset),
    m_CharaListLastClickedTick(0),
    m_CharaListLastClickedIndex(-1),
    m_EmployListLastClickedTick(0),
    m_MousePointCount(0),
    m_LastPointedArea(-1)
{
    boost::shared_ptr<TWorldMenuWindow> wmw(new TWorldMenuWindow());
    m_WidgetSet->Add(wmw);
    m_WorldMenuWindow = wmw.get();
    m_WorldMenuWindow->setVisible(false);
    m_WorldMenuWindow->GetButtonAbortWatch().addActionListener(this);
    m_WorldMenuWindow->setPosition(4, 48);

    boost::shared_ptr<TGcnHintPane> hintp(new TGcnHintPane());
    m_WidgetSet->Add(hintp);
    m_HintPane = hintp.get();

    gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
    top->add(m_WorldMenuWindow);
    top->add(m_HintPane);
}
//---------------------------------------------------------------------------
TWorldMapLogic::~TWorldMapLogic()
{
}
//---------------------------------------------------------------------------
void TWorldMapLogic::mousePressed(gcn::MouseEvent &mouseEvent)
{
    if(mouseEvent.getSource() == &m_WorldMapWindow->GetWorldMapPane()){
        if(m_LogicStatus == LOGIC_PLAYERMOVE){
            // ړ̃GAI҂̎̓
            if(mouseEvent.getButton() == gcn::MouseEvent::LEFT){
                int area_to = GetAreaAt(mouseEvent.getX(), mouseEvent.getY());
                if(area_to != -1){
                    int area_from = m_CharaListWindow->GetCharaListPane().GetSelectedArea();
                    if(ShowMoveCharaWindow(area_from, area_to)){
                        m_WorldMapWindow->ClearGuidance();
                        m_MoveFromArea = area_from;
                        m_MoveToArea = area_to;
                        m_LogicStatus = LOGIC_PLAYERMOVEWINDOW;
                    }
                }
            }
            else{
                // ړGAILZ
                m_WorldMapWindow->ClearGuidance();
                m_LogicStatus = LOGIC_PLAYERSTRATEGY;
            }
            mouseEvent.consume();
        }
        else if(m_LogicStatus == LOGIC_PLAYERMOVEWINDOW){
            if(mouseEvent.getButton() != gcn::MouseEvent::LEFT){
                // ړLZ
                m_CharaMoveWindow->setVisible(false);
                m_WorldMapWindow->GetWorldMapPane().ClearBrightEdge();
                RecoveryAreaSelection();
                m_LogicStatus = LOGIC_PLAYERSTRATEGY;
            }
        }
        else if(m_LogicStatus == LOGIC_PLAYERWAR){
            // 푈I҂̏B
            if(mouseEvent.getButton() == gcn::MouseEvent::LEFT){
                const TOpenFarenGame* game = g_GameMaster.GetGame();
                int area_to = GetAreaAt(mouseEvent.getX(), mouseEvent.getY());
                if(area_to != -1 && game->IsDepaturableArea(area_to)){
                    //m_WorldMapWindow->ClearGuidance();
                    m_WorldMapWindow->SetGuidance("owGAowLIŉ");
                    //m_WorldMapWindow->GetWorldMapPane().SetBrightArea(area_to);
                    m_WorldMapWindow->GetWorldMapPane().AddBrightArea(area_to, 0xffff0000);
                    ShowWarCharaWindow(area_to);
                    m_WarTargetArea = area_to;
                    UpdateWarAreaSelection();
                    m_LogicStatus = LOGIC_PLAYERWARWINDOW;
                }
            }
            else{
                // 푈GAILZ
                m_WorldMapWindow->ClearGuidance();
                RecoveryAreaSelection();
                m_LogicStatus = LOGIC_PLAYERSTRATEGY;
            }
        }
        else if(m_LogicStatus == LOGIC_PLAYERWARWINDOW){
            // 푈EBhEoĂ鎞̏B
            if(mouseEvent.getButton() == gcn::MouseEvent::LEFT){
                // LGA푈̗אڃGAȂ푈EBhE̍yCɐݒ肷B
                int area_from = GetAreaAt(mouseEvent.getX(), mouseEvent.getY());
                const TOpenFarenGame* game = g_GameMaster.GetGame();
                if(area_from != -1 && game->GetAreaInstance(area_from).m_master == game->GetCurrentMasterIndex()){
                    const std::vector<int>& neighbor = game->GetAreaInstance(m_WarTargetArea).m_neighbor;
                    if(std::find(neighbor.begin(), neighbor.end(), area_from) != neighbor.end()){
                        m_WarCharaWindow->SetLeftArea(area_from);
                        UpdateWarAreaSelection();
                    }
                }
            }
            else{
                // 푈LZB
                m_WarCharaWindow->Hide();
                //m_WorldMapWindow->GetWorldMapPane().ClearBrightArea();
                RecoveryAreaSelection();
                m_WorldMapWindow->GetWorldMapPane().ClearBrightEdge();
                m_WorldMapWindow->ClearGuidance();
                RecoveryAreaSelection();
                m_LogicStatus = LOGIC_PLAYERSTRATEGY;
            }
            mouseEvent.consume();
        }
        else if(m_LogicStatus == LOGIC_PLAYERALLIANCE){
            // I҂̏B
            if(mouseEvent.getButton() == gcn::MouseEvent::LEFT){
                const TOpenFarenData* data = g_GameMaster.GetData();
                const TOpenFarenGame* game = g_GameMaster.GetGame();
                int area_to = GetAreaAt(mouseEvent.getX(), mouseEvent.getY());
                if(area_to != -1){
                    const TAreaInstance& area = game->GetAreaInstance(area_to);
                    if(area.m_master != -1 && area.m_master != game->GetCurrentMasterIndex()){
                        m_WorldMapWindow->ClearGuidance();
                        const TMasterInstance& masterins1 = game->GetMasterInstance(game->GetCurrentMasterIndex());
                        const TMasterInstance& masterins2 = game->GetMasterInstance(area.m_master);
                        const TMaster& master1 = data->GetMaster(masterins1.m_type);
                        const TMaster& master2 = data->GetMaster(masterins2.m_type);
                        int turn = TOpenFarenGame::DEFAULT_ALLIANCETURN;
                        int alindex = game->FindAlliance(game->GetCurrentMasterIndex(), area.m_master);
                        if(alindex != -1){
                            const TAlliance& al = game->GetAlliance(alindex);
                            if(al.turn == -1 || al.turn == TOpenFarenGame::DEFAULT_ALLIANCETURN){
                                std::string msg;
                                if(al.turn == -1){
                                    msg = (boost::format("%sƂ͊ɖ̓ł܂B") % master2.m_name).str();
                                }
                                else{
                                    msg = (boost::format("%sƂ͊%d^[̓ł܂B") % master2.m_name % al.turn).str();
                                }
                                TGcnMsgDlg::Show("Information", msg, TGcnMsgDlg::BUTTON_OK);
                                m_LogicStatus = LOGIC_PLAYERSTRATEGY;
                                mouseEvent.consume();
                                return;
                            }
                            turn -= al.turn;
                        }
                        int cost = game->EvalAllianceCost(game->GetCurrentMasterIndex(), area.m_master, turn);
                        if(cost > masterins1.m_money){
                            std::string msg;
                            if(turn == TOpenFarenGame::DEFAULT_ALLIANCETURN){
                                msg = (boost::format("%sƓԂɂ%d%sKvłA܂B") % master2.m_name % cost % (data->GetMoneyUnit())).str();
                            }
                            else{
                                msg = (boost::format("%sƂ̓ɂ%d%sKvłA܂B") % master2.m_name % cost % (data->GetMoneyUnit())).str();
                            }
                            TGcnMsgDlg::Show("Information", msg, TGcnMsgDlg::BUTTON_OK);
                            m_LogicStatus = LOGIC_PLAYERSTRATEGY;
                            mouseEvent.consume();
                            return;
                        }
                        m_ProposedAllianceMaster = area.m_master;
                        m_ProposedAllianceTurn = turn;
                        m_ProposedAllianceCost = cost;
                        std::string msg;
                        if(turn == TOpenFarenGame::DEFAULT_ALLIANCETURN){
                            msg = (boost::format("%s%d%s%d^[̓\݂܂H") % master2.m_name % cost % (data->GetMoneyUnit()) % turn).str();
                        }
                        else{
                            msg = (boost::format("%s%d%s%d^[̓\݂܂H") % master2.m_name % cost % (data->GetMoneyUnit()) % turn).str();
                        }
                        TGcnMsgDlg::Show("mF", msg, TGcnMsgDlg::BUTTON_YES | TGcnMsgDlg::BUTTON_NO);
                        m_LogicStatus = LOGIC_CONFIRM_ALLIANCE;
                    }
                }
            }
            else{
                // ILZ
                m_WorldMapWindow->ClearGuidance();
                m_LogicStatus = LOGIC_PLAYERSTRATEGY;
            }
            mouseEvent.consume();
        }
        else if(m_LogicStatus == LOGIC_PLAYERSTRATEGY){
            // ɉɓĂȂ̓
            if(mouseEvent.getButton() == gcn::MouseEvent::LEFT){
                int ai = GetAreaAt(mouseEvent.getX(), mouseEvent.getY());
                if(ai != -1){
                    const TOpenFarenGame* game = g_GameMaster.GetGame();
                    //m_CharaListWindow->SetSelectedArea(ai);
                    AddAreaSelection(ai);
                    UpdateAreaPointed(ai);
                    m_CharaListWindow->Show(game->GetAreaInstance(ai).m_master == game->GetCurrentMasterIndex());
                    g_GuiContainerList.GetGui()->getTop()->moveToTop(m_CharaListWindow);
                    mouseEvent.consume();
                }
            }
        }
    }
    else if(mouseEvent.getSource() == &m_CharaListWindow->GetCharaListPane()){
        // GAEBhEŃL_uNbN̏
        int tick = g_GuiContainerList.GetTick();
        if(tick > m_CharaListLastClickedTick + g_Ini.GetDoubleClickFrame()){
            m_CharaListLastClickedTick = tick;
            m_CharaListLastClickedIndex = m_CharaListWindow->GetCharaListPane().GetSelected();
            return;
        }
        //g_Logger.DebugLog("show");
        int ai = m_CharaListWindow->GetCharaListPane().GetSelectedArea();
        int i = m_CharaListWindow->GetCharaListPane().GetSelected();
        if(ai != -1 && i != -1 && i == m_CharaListLastClickedIndex){
            m_CharaListLastClickedTick = 0;
            const TOpenFarenGame* game = g_GameMaster.GetGame();
            const TAreaInstance& area = game->GetAreaInstance(ai);
            //m_CharaStatusWindow->GetCharaStatusPane().SetCharaStatus(area.m_charalist[i]);
            //g_Logger.DebugLog("umu");
            m_CharaStatusFullWindow->Show(area.m_charalist[i]);
            g_GuiContainerList.MoveWidgetToCenter(m_CharaStatusFullWindow);
            g_GuiContainerList.GetGui()->getTop()->moveToTop(m_CharaStatusFullWindow);
            mouseEvent.consume();
        }
        else{
            m_CharaListLastClickedTick = tick;
            m_CharaListLastClickedIndex = i;
        }
    }
    else if(mouseEvent.getSource() == &m_EmployWindow->GetEmployListBox()){
        // ٗpEBhEŃL_uNbN̏
        int tick = g_GuiContainerList.GetTick();
        if(tick > m_EmployListLastClickedTick + g_Ini.GetDoubleClickFrame()){
            m_EmployListLastClickedTick = tick;
            return;
        }
        m_EmployListLastClickedTick = 0;
        int ctype = m_EmployWindow->GetSelectedEmployType();
        if(ctype != -1){
            // vC[̓NEłقȂOƂB
            const TOpenFarenGame* game = g_GameMaster.GetGame();
            TCharacterInstance c = game->CreateCharaInstance(ctype, 0);
            m_CharaStatusFullWindow->Show(c);
            g_GuiContainerList.MoveWidgetToCenter(m_CharaStatusFullWindow);
            g_GuiContainerList.GetGui()->getTop()->moveToTop(m_CharaStatusFullWindow);
            mouseEvent.consume();
        }
    }
}
//---------------------------------------------------------------------------
void TWorldMapLogic::mouseMoved(gcn::MouseEvent &mouseEvent)
{
    if(mouseEvent.getSource() == &m_CharaListWindow->GetCharaListPane()){
        // GAEBhEŃL̏}EXŃ|CgƂ̏B
        int ai = m_CharaListWindow->GetCharaListPane().GetSelectedArea();
        int i = m_CharaListWindow->GetCharaListPane().GetBrowsed();
        if(ai != -1 && i != -1){
            const TOpenFarenGame* game = g_GameMaster.GetGame();
            const TAreaInstance& area = game->GetAreaInstance(ai);
            m_CharaStatusWindow->Show(area.m_charalist[i]);
            g_GuiContainerList.GetGui()->getTop()->moveToTop(m_CharaStatusWindow);
            mouseEvent.consume();
        }
    }
    else if(mouseEvent.getSource() == &m_WorldMapWindow->GetWorldMapPane()){
        if(LOGIC_PLAYERSTRATEGY <= m_LogicStatus && m_LogicStatus <= LOGIC_PLAYEREMPLOYHERO_NEXT){
            UpdateAreaPowerHelp(mouseEvent.getX(), mouseEvent.getY(), true);
            UpdateAreaPointed(GetAreaAt(mouseEvent.getX(), mouseEvent.getY()));
        }
    }
    else{
        UpdateAreaPowerHelp(0, 0, false);
    }
}
//---------------------------------------------------------------------------
void TWorldMapLogic::mouseExited(gcn::MouseEvent &mouseEvent)
{
    if(mouseEvent.getSource() == &m_WorldMapWindow->GetWorldMapPane()){
        UpdateAreaPowerHelp(0, 0, false);
        UpdateAreaPointed(-1);
    }
}
//---------------------------------------------------------------------------
void TWorldMapLogic::valueChanged(const gcn::SelectionEvent &event)
{
    if(event.getSource() == &m_CharaListWindow->GetAreaOpeDropDown()){
        int ai = m_CharaListWindow->GetCharaListPane().GetSelectedArea();
        if(ai == -1){
            OFERRLOG("ҋ@sύX삪sꂽɂւ炸GAIH肦Ȃ͂B");
            return;
        }
        int selected = m_CharaListWindow->GetAreaOpeDropDown().getSelected();
        if(selected != -1){
            TOpenFarenGame* game = g_GameMaster.GetGame();
            game->SetAreaOperation(ai, static_cast<TAreaOperation>(selected));
        }
    }
}
//---------------------------------------------------------------------------
void TWorldMapLogic::action(const gcn::ActionEvent& actionEvent)
{
    // Lڍו\B{^ƂOŁAłH
    m_CharaStatusFullWindow->setVisible(false);
    if(actionEvent.getSource() == &m_CharaListWindow->GetButtonEmploy() && m_LogicStatus == LOGIC_PLAYERSTRATEGY){
        // GAEBhEŌٗp{^Ƃ̏B
        int ai = m_CharaListWindow->GetCharaListPane().GetSelectedArea();
        if(ai == -1){
            TGcnMsgDlg::Show("Information", "ٗpsGAIĉB", TGcnMsgDlg::BUTTON_OK);
            return;
        }
        const TOpenFarenGame* game = g_GameMaster.GetGame();
        const TAreaInstance& area = game->GetAreaInstance(ai);
        if(area.m_master != game->GetCurrentMasterIndex()){
            TGcnMsgDlg::Show("Information", "IGALGAł͂܂B", TGcnMsgDlg::BUTTON_OK);
            return;
        }
        int ci = m_CharaListWindow->GetCharaListPane().GetSelected();
        if(ci == -1){
            TGcnMsgDlg::Show("Information", "ٗpsLIĉB", TGcnMsgDlg::BUTTON_OK);
            return;
        }
        const TOpenFarenData* data = g_GameMaster.GetData();
        if(area.m_charalist.size() >= TOpenFarenGame::AREACHARA_MAX){
            const int areacharamax = TOpenFarenGame::AREACHARA_MAX;
            std::string msg = (boost::format("GA̕%dɒBĂ邽ߌٗpł܂B") % areacharamax).str();
            TGcnMsgDlg::Show("Information", msg, TGcnMsgDlg::BUTTON_OK);
            return;
        }
        // ٗpSLIŒB
        m_CharaListWindow->GetCharaListPane().LockSelected(true);
        // ٗp\L擾B
        const TBattleCharacter& charadata = data->GetCharacter(area.m_charalist[ci].m_type);
        const TMasterInstance& mi = game->GetMasterInstance(game->GetCurrentMasterIndex());
        std::vector<int> employable;
        game->GetEmployable(ai, ci, employable);
        m_EmployWindow->Show(employable, charadata.m_race, mi.m_money);
        g_GuiContainerList.MoveWidgetToCenter(m_EmployWindow);
        g_GuiContainerList.GetGui()->getTop()->moveToTop(m_EmployWindow);
        m_LogicStatus = LOGIC_PLAYEREMPLOY;
    }
    else if(actionEvent.getSource() == &m_CharaListWindow->GetButtonDismiss() && m_LogicStatus == LOGIC_PLAYERSTRATEGY){
        // GAEBhEŉك{^Ƃ̏B
        int ai = m_CharaListWindow->GetCharaListPane().GetSelectedArea();
        if(ai == -1){
            TGcnMsgDlg::Show("Information", "قsGAIĉB", TGcnMsgDlg::BUTTON_OK);
            return;
        }
        const TOpenFarenGame* game = g_GameMaster.GetGame();
        const TAreaInstance& area = game->GetAreaInstance(ai);
        int ci = m_CharaListWindow->GetCharaListPane().GetSelected();
        if(ci == -1){
            TGcnMsgDlg::Show("Information", "قLIĉB", TGcnMsgDlg::BUTTON_OK);
            return;
        }
        const TOpenFarenData* data = g_GameMaster.GetData();
        const TMaster& masterdata = data->GetMaster(game->GetMasterInstance(game->GetCurrentMasterIndex()).m_type);
        if(masterdata.m_facecharaindex == area.m_charalist[ci].m_type){
            TGcnMsgDlg::Show("Information", "}X^[͉قł܂B", TGcnMsgDlg::BUTTON_OK);
            return;
        }
        m_DismissIndex.first = ai;
        m_DismissIndex.second = ci;
        const TBattleCharacter& charadata = data->GetCharacter(area.m_charalist[ci].m_type);
        std::string msg = (boost::format("%sق܂B") % charadata.m_name).str();
        TGcnMsgDlg::Show("mF", msg, TGcnMsgDlg::BUTTON_OK | TGcnMsgDlg::BUTTON_CANCEL);
        m_LogicStatus = LOGIC_CONFIRM_MESSAGE;
    }
    else if(actionEvent.getSource() == &m_CharaListWindow->GetButtonMove() && m_LogicStatus == LOGIC_PLAYERSTRATEGY){
        // ړI[hɈڍsB
        int ai = m_CharaListWindow->GetCharaListPane().GetSelectedArea();
        if(ai != -1){
            m_CharaListWindow->GetCharaListPane().ClearState();
            m_WorldMapWindow->SetGuidance("ړ̃GAIŉ");
            m_LogicStatus = LOGIC_PLAYERMOVE;
        }
    }
    else if(actionEvent.getSource() == &m_MasterWindow->GetButtonAreaList() && m_LogicStatus == LOGIC_PLAYERSTRATEGY){
        // }X^[EBhEŃGA{^Ƃ̏B
        m_AreaGridWindow->Show();
        m_LogicStatus = LOGIC_PLAYERAREALIST;
    }
    else if(actionEvent.getSource() == &m_MasterWindow->GetButtonMasterList() && m_LogicStatus == LOGIC_PLAYERSTRATEGY){
        // }X^[EBhEŃGA{^Ƃ̏B
        m_MasterGridWindow->Show();
        m_LogicStatus = LOGIC_PLAYERMASTERLIST;
    }
    else if(actionEvent.getSource() == &m_MasterWindow->GetButtonHeroList() && m_LogicStatus == LOGIC_PLAYERSTRATEGY){
        // }X^[EBhEŐlޏ{^Ƃ̏B
        m_HeroGridWindow->Show();
        m_LogicStatus = LOGIC_PLAYERHEROLIST;
    }
    else if(actionEvent.getSource() == &m_MasterWindow->GetButtonPowerMap() && m_LogicStatus == LOGIC_PLAYERSTRATEGY){
        // }X^[EBhEŐ͒n}{^Ƃ̏B
        m_PowerMapWindow->Show();
        m_LogicStatus = LOGIC_PLAYERPOWERMAP;
    }
    else if(actionEvent.getSource() == &m_MasterWindow->GetButtonAlliance() && m_LogicStatus == LOGIC_PLAYERSTRATEGY){
        // I[hɈڍsB
        m_WorldMapWindow->SetGuidance("\ސ͂Iŉ");
        m_LogicStatus = LOGIC_PLAYERALLIANCE;
    }
    else if(actionEvent.getSource() == &m_MasterWindow->GetButtonWar() && m_LogicStatus == LOGIC_PLAYERSTRATEGY){
        // 푈I[hɈڍsB
        m_CharaListWindow->GetCharaListPane().ClearState();
        m_WorldMapWindow->SetGuidance("푈̃GAIŉ");
        m_WarTargetArea = -1;
        UpdateWarAreaSelection();
        m_LogicStatus = LOGIC_PLAYERWAR;
    }
    else if(actionEvent.getSource() == &m_MasterWindow->GetButtonTurnEnd() && m_LogicStatus == LOGIC_PLAYERSTRATEGY){
        //if(TGcnMsgDlg::Show("mF", "^[I܂H", TGcnMsgDlg::BUTTON_YES | TGcnMsgDlg::BUTTON_NO) == TGcnMsgDlg::BUTTON_YES){
        //    NextMaster();
        //}
        if(!m_LuaWorld.CallFunction("OnTurnEndButton")){
            m_LogicStatus = LOGIC_ERROR;
            return;
        }
        if(m_LuaWorld.CanResume()){
            m_LogicStatus = LOGIC_SCRIPT;
            m_ScriptStatus = SCRIPT_PLAYERUI;
            return;
        }
    }
    else if(actionEvent.getSource() == &m_EmployWindow->GetButtonOk()){
        // ٗpEBhEOK{^Ƃ̏B
        m_EmployWindow->setVisible(false);
        int ai = m_CharaListWindow->GetCharaListPane().GetSelectedArea();
        int ci = m_CharaListWindow->GetCharaListPane().GetSelected();
        if(ai == -1 || ci == -1){
            g_Logger.ErrorLog("ٗp삪sꂽɂւ炸ٗpGA܂͌ٗpSLIivÕoOHj");
            return;
        }
        int ctype = m_EmployWindow->GetSelectedEmployType();
        if(ctype == -1){
            g_Logger.ErrorLog("ٗp삪sꂽɂւ炸ٗpLIivÕoOHj");
            return;
        }
        // ٗpSLIŒB
        m_CharaListWindow->GetCharaListPane().LockSelected(false);
        // ٗpČٗpSL̑IB
        g_GameMaster.GetGame()->EmployCharacter(ai, ci, ctype);
        //m_CharaListWindow->GetCharaListPane().SetSelected(-1);
        m_CharaListWindow->GetCharaListPane().ClearState();
        m_LogicStatus = LOGIC_PLAYERSTRATEGY;
    }
    else if(actionEvent.getSource() == &m_EmployWindow->GetButtonCancel()){
        // ٗpEBhECancel{^Ƃ̏B
        m_EmployWindow->setVisible(false);
        // ٗpSLIŒB
        m_CharaListWindow->GetCharaListPane().LockSelected(false);
        m_LogicStatus = LOGIC_PLAYERSTRATEGY;
    }
    else if(actionEvent.getSource() == &m_CharaMoveWindow->GetButtonOk()){
        // LړEBhEŊm{^Ƃ̏B
        m_CharaMoveWindow->setVisible(false);
        std::vector<int> indice1, indice2;
        m_CharaMoveWindow->GetLeftToRight(indice1);
        m_CharaMoveWindow->GetRightToLeft(indice2);
        if(!g_GameMaster.GetGame()->MoveCharacter(m_MoveFromArea, m_MoveToArea, indice1, indice2)){
            TGcnMsgDlg::Show("Error", "L̈ړɎs܂ivÕoOHj", TGcnMsgDlg::BUTTON_OK);
        }
        m_WorldMapWindow->GetWorldMapPane().ClearBrightEdge();
        RecoveryAreaSelection();
        m_LogicStatus = LOGIC_PLAYERSTRATEGY;
    }
    else if(actionEvent.getSource() == &m_CharaMoveWindow->GetButtonCancel()){
        // LړEBhEŎ{^Ƃ̏B
        m_CharaMoveWindow->setVisible(false);
        m_WorldMapWindow->GetWorldMapPane().ClearBrightEdge();
        RecoveryAreaSelection();
        m_LogicStatus = LOGIC_PLAYERSTRATEGY;
    }
    else if(actionEvent.getSource() == &m_WarCharaWindow->GetButtonOk()){
        // owEBhEŊm{^Ƃ̏B
        const TOpenFarenGame* game = g_GameMaster.GetGame();
        m_WarCharaWindow->GetLeftToRight(m_DepatureCharaIndice);
        m_WarCharaWindow->Hide();
        if(m_DepatureCharaIndice.empty()){
            g_Logger.ErrorLog("푈삪sꂽɂւ炸푈LIivÕoOHj");
            return;
        }
        // [h}bv[hł̃EBhEBB
        m_CharaListWindow->setVisible(false);
        m_MasterWindow->setVisible(false);
        m_CharaStatusWindow->setVisible(false);
        //g_GameMaster.SetSysMenuVisible(false);
        g_GameMaster.SetSysMenuMode(SYSMENU_BATTLE);

        m_WorldMapWindow->GetWorldMapPane().ClearBrightArea();
        m_WorldMapWindow->GetWorldMapPane().ClearBrightEdge();
        m_WorldMapWindow->ClearGuidance();
        Departure(m_WarTargetArea, m_DepatureCharaIndice);
    }
    else if(actionEvent.getSource() == &m_WarCharaWindow->GetButtonCancel()){
        // 푈LZB
        m_WarCharaWindow->Hide();
        m_WorldMapWindow->GetWorldMapPane().ClearBrightArea();
        m_WorldMapWindow->GetWorldMapPane().ClearBrightEdge();
        m_WorldMapWindow->ClearGuidance();
        RecoveryAreaSelection();
        m_LogicStatus = LOGIC_PLAYERSTRATEGY;
    }
    else if(actionEvent.getSource() == &m_EmployHeroWindow->GetButtonDetail()){
        // lޓopEBhEŏڍ׃{^Ƃ̏B
        int areaindex, charatype;
        m_EmployHeroWindow->GetHero(areaindex, charatype);
        const TOpenFarenGame* game = g_GameMaster.GetGame();
        TCharacterInstance c = game->GetWanderingChara(charatype);
        m_CharaStatusFullWindow->Show(c);
        g_GuiContainerList.MoveWidgetToCenter(m_CharaStatusFullWindow);
        g_GuiContainerList.GetGui()->getTop()->moveToTop(m_CharaStatusFullWindow);
    }
    else if(actionEvent.getSource() == &m_EmployHeroWindow->GetButtonEmploy()){
        // lޓopEBhEŌق{^Ƃ̏B
        TOpenFarenGame* game = g_GameMaster.GetGame();
        m_EmployHeroWindow->setVisible(false);
        // op菈B
        int areaindex, charatype;
        m_EmployHeroWindow->GetHero(areaindex, charatype);
        const TOpenFarenData* data = g_GameMaster.GetData();
        const TBattleCharacter& charadata = data->GetCharacter(charatype);
        const TMasterInstance& master = game->GetMasterInstance(game->GetCurrentMasterIndex());
        if(master.m_money < charadata.m_employcost){
            // sȂ烁bZ[W\ցB
            TGcnMsgDlg::Show("Information", "R܂", TGcnMsgDlg::BUTTON_OK, g_Ini.GetMessageTimeoutFrame());
            m_LogicStatus_AfterMessage = LOGIC_PLAYEREMPLOYHERO_NEXT;
            m_LogicStatus = LOGIC_SHOWING_MESSAGE;
            return;
        }
        if(!game->JudgeEmployWanderingChara(game->GetCurrentMasterIndex(), charatype)){
            // opsȂ烁bZ[W\ցB
            TGcnMsgDlg::Show("Information", "Uf܂", TGcnMsgDlg::BUTTON_OK, g_Ini.GetMessageTimeoutFrame());
            m_LogicStatus_AfterMessage = LOGIC_PLAYEREMPLOYHERO_NEXT;
            m_LogicStatus = LOGIC_SHOWING_MESSAGE;
            return;
        }
        // ٗpB
        game->EmployWanderingChara(areaindex, charatype);

        // ܂opłlނ邩B
        PlayerNextEmployHero();
    }
    else if(actionEvent.getSource() == &m_EmployHeroWindow->GetButtonCancel()){
        // lޓopEBhEŌقȂ{^Ƃ̏B
        m_EmployHeroWindow->setVisible(false);
        // ܂opłlނ邩B
        PlayerNextEmployHero();
    }
    else if(actionEvent.getSource() == &m_AreaGridWindow->GetButtonClose()){
        // GAEBhEŏI{^Ƃ̏B
        m_AreaGridWindow->setVisible(false);
        m_LogicStatus = LOGIC_PLAYERSTRATEGY;
    }
    else if(actionEvent.getSource() == &m_MasterGridWindow->GetButtonClose()){
        // }X^[EBhEŏI{^Ƃ̏B
        m_MasterGridWindow->setVisible(false);
        m_LogicStatus = LOGIC_PLAYERSTRATEGY;
    }
    else if(actionEvent.getSource() == &m_HeroGridWindow->GetButtonClose()){
        // lޏEBhEŏI{^Ƃ̏B
        m_HeroGridWindow->setVisible(false);
        m_LogicStatus = LOGIC_PLAYERSTRATEGY;
    }
    else if(actionEvent.getSource() == &m_WorldMenuWindow->GetButtonAbortWatch()){
        // ϐ풆~{^̏B
        m_BattleMapLogic->SetNoWatch();
        m_WorldMapWindow->setVisible(true);
        m_WorldMenuWindow->setVisible(false);
    }
}
//---------------------------------------------------------------------------
void TWorldMapLogic::Start(bool isNewGame, bool isWatch, const TiXmlElement *e)
{
    boost::shared_ptr<TWorldMapWindow> pw(new TWorldMapWindow());
    m_WidgetSet->Add(pw);
    m_WorldMapWindow = pw.get();
    m_WorldMapWindow->GetWorldMapPane().addMouseListener(this);
    m_WorldMapWindow->SetWorldMapLogic(this);   // {Listener`̕ǂȂB
    m_WorldMapWindow->Show();

    boost::shared_ptr<TCharaListWindow> pclw(new TCharaListWindow());
    m_WidgetSet->Add(pclw);
    m_CharaListWindow = pclw.get();
    m_CharaListWindow->GetButtonMove().addActionListener(this);
    m_CharaListWindow->GetButtonEmploy().addActionListener(this);
    m_CharaListWindow->GetButtonDismiss().addActionListener(this);
    m_CharaListWindow->GetCharaListPane().addMouseListener(this);
    m_CharaListWindow->GetAreaOpeDropDown().addSelectionListener(this);

    boost::shared_ptr<TMasterWindow> mw(new TMasterWindow());
    m_WidgetSet->Add(mw);
    m_MasterWindow = mw.get();
    m_MasterWindow->GetButtonAreaList().addActionListener(this);
    m_MasterWindow->GetButtonMasterList().addActionListener(this);
    m_MasterWindow->GetButtonHeroList().addActionListener(this);
    m_MasterWindow->GetButtonPowerMap().addActionListener(this);
    m_MasterWindow->GetButtonAlliance().addActionListener(this);
    m_MasterWindow->GetButtonWar().addActionListener(this);
    m_MasterWindow->GetButtonTurnEnd().addActionListener(this);

    boost::shared_ptr<TCharaStatusWindow> pcsw(new TCharaStatusWindow());
    m_WidgetSet->Add(pcsw);
    m_CharaStatusWindow = pcsw.get();

    boost::shared_ptr<TCharaStatusFullWindow> pcsfw(new TCharaStatusFullWindow());
    m_WidgetSet->Add(pcsfw);
    m_CharaStatusFullWindow = pcsfw.get();

    boost::shared_ptr<TEmployWindow> pew(new TEmployWindow());
    m_WidgetSet->Add(pew);
    m_EmployWindow = pew.get();
    m_EmployWindow->GetButtonOk().addActionListener(this);
    m_EmployWindow->GetButtonCancel().addActionListener(this);
    m_EmployWindow->GetEmployListBox().addMouseListener(this);

    boost::shared_ptr<TCharaMoveWindow> pcmw(new TCharaMoveWindow());
    m_WidgetSet->Add(pcmw);
    m_CharaMoveWindow = pcmw.get();
    m_CharaMoveWindow->GetButtonOk().addActionListener(this);
    m_CharaMoveWindow->GetButtonCancel().addActionListener(this);

    boost::shared_ptr<TWarCharaWindow> pwcw(new TWarCharaWindow());
    m_WidgetSet->Add(pwcw);
    m_WarCharaWindow = pwcw.get();
    m_WarCharaWindow->GetButtonOk().addActionListener(this);
    m_WarCharaWindow->GetButtonCancel().addActionListener(this);

    boost::shared_ptr<TEmployHeroWindow> ehw(new TEmployHeroWindow());
    m_WidgetSet->Add(ehw);
    m_EmployHeroWindow = ehw.get();
    m_EmployHeroWindow->GetButtonDetail().addActionListener(this);
    m_EmployHeroWindow->GetButtonEmploy().addActionListener(this);
    m_EmployHeroWindow->GetButtonCancel().addActionListener(this);

    boost::shared_ptr<TAreaGridWindow> agw(new TAreaGridWindow());
    m_WidgetSet->Add(agw);
    m_AreaGridWindow = agw.get();
    m_AreaGridWindow->setVisible(false);
    m_AreaGridWindow->GetButtonClose().addActionListener(this);

    boost::shared_ptr<TMasterGridWindow> mgw(new TMasterGridWindow());
    m_WidgetSet->Add(mgw);
    m_MasterGridWindow = mgw.get();
    m_MasterGridWindow->setVisible(false);
    m_MasterGridWindow->GetButtonClose().addActionListener(this);

    boost::shared_ptr<THeroGridWindow> hgw(new THeroGridWindow());
    m_WidgetSet->Add(hgw);
    m_HeroGridWindow = hgw.get();
    m_HeroGridWindow->setVisible(false);
    m_HeroGridWindow->GetButtonClose().addActionListener(this);

    boost::shared_ptr<TPowerMapWindow> pmw(new TPowerMapWindow());
    m_WidgetSet->Add(pmw);
    m_PowerMapWindow = pmw.get();
    m_PowerMapWindow->setVisible(false);

    gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
    top->add(m_WorldMapWindow);
    top->moveToTop(m_WorldMapWindow);
    top->add(m_CharaListWindow);
    top->add(m_MasterWindow);
    top->add(m_CharaStatusWindow);
    top->add(m_CharaStatusFullWindow);
    top->add(m_EmployWindow);
    top->add(m_CharaMoveWindow);
    top->add(m_WarCharaWindow);
    top->add(m_EmployHeroWindow);
    top->add(m_AreaGridWindow);
    top->add(m_MasterGridWindow);
    top->add(m_HeroGridWindow);
    top->add(m_PowerMapWindow);

    m_WorldMapWindow->GetWorldMapPane().UpdateAreaCount();

    boost::shared_ptr<TScriptAPI> sapi(new TScriptAPI(m_WidgetSet));
    m_ScriptAPI = sapi;

    boost::shared_ptr<TBattleMapLogic> blogic(new TBattleMapLogic(m_WidgetSet));
    m_BattleMapLogic = blogic;
    m_BattleMapLogic->SetExternalWindow(m_CharaStatusWindow, m_CharaStatusFullWindow);
    m_BattleMapLogic->SetScriptAPI(m_ScriptAPI.get());

    m_IsWatchMode = isWatch;

    std::string eventfile;
    if(!g_GameMaster.FindScriptPath("EventScript.lua", eventfile)){
        g_Logger.ErrorLog((boost::format("CxgXNvǧɎs܂B")).str());
        m_LogicStatus = LOGIC_ERROR;
        return;
    }
    if(!m_LuaEvent.LoadLua(eventfile.c_str())){
        g_Logger.ErrorLog((boost::format("CxgXNvg %s ̃[hɎs܂B") % eventfile).str());
        m_LogicStatus = LOGIC_ERROR;
        return;
    }
    m_LuaEvent.SetIf(m_BattleMapLogic.get(), m_ScriptAPI.get(), this);
    if(!m_LuaEvent.CallFunction(LUAFUNC_SYSSTART)){
        g_Logger.ErrorLog((boost::format("CxgXNvg %s ̏(%s)Ɏs܂B") % eventfile % LUAFUNC_SYSSTART).str());
        m_LogicStatus = LOGIC_ERROR;
        return;
    }
    m_LuaEvent.RegistInitVariable();

    std::string worldfile;
    if(!g_GameMaster.FindScriptPath("world.lua", worldfile)){
        g_Logger.ErrorLog("world.luǎɎs܂B");
        m_LogicStatus = LOGIC_ERROR;
        return;
    }
    if(!m_LuaWorld.LoadLua(worldfile.c_str())){
        g_Logger.ErrorLog("world.lua ̃[hɎs܂B");
        m_LogicStatus = LOGIC_ERROR;
        return;
    }
    m_LuaWorld.SetIf(m_BattleMapLogic.get(), m_ScriptAPI.get(), this);

    std::string strategyfile;
    if(!g_GameMaster.FindScriptPath("strategy.lua", strategyfile)){
        g_Logger.ErrorLog("strategy.luǎɎs܂B");
        m_LogicStatus = LOGIC_ERROR;
        return;
    }
//    if(!m_LuaStrategy.LoadLua(strategyfile.c_str())){
//        g_Logger.ErrorLog("strategy.lua ̃[hɎs܂B");
//        m_LogicStatus = LOGIC_ERROR;
//        return;
//    }
//    m_LuaStrategy.SetIf(m_BattleMapLogic.get(), NULL, this);
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    int mastercount = game->GetMasterInstanceCount();
    m_LuaStrategy.resize(mastercount);
    for(int i=0; i<mastercount; ++i){
        if(!m_LuaStrategy[i].LoadLua(strategyfile.c_str())){
            g_Logger.ErrorLog("strategy.lua ̃[hɎs܂B");
            m_LogicStatus = LOGIC_ERROR;
            return;
        }
        m_LuaStrategy[i].SetIf(m_BattleMapLogic.get(), NULL, this);
    }

    if(isNewGame){
        NextMaster();
    }
    else{
        if(!Load(*e)){
            g_Logger.ErrorLog("Lua ϐ̃[hɎs܂B");
            m_LogicStatus = LOGIC_ERROR;
            return;
        }
        StartMasterPhase();
    }
}
//---------------------------------------------------------------------------
void TWorldMapLogic::FocusArea(int areaindex)
{
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TAreaInstance& area = game->GetAreaInstance(areaindex);
    m_WorldMapWindow->SetScrollCenter(area.m_location.x, area.m_location.y);
}
//---------------------------------------------------------------------------
void TWorldMapLogic::GetTopSize(int &w, int& h) const
{
    const gcn::Widget* top = g_GuiContainerList.GetGui()->getTop();
    w = top->getWidth();
    h = top->getHeight();
}
//---------------------------------------------------------------------------
void TWorldMapLogic::DoLogic()
{
    UpdateAreaPowerHelp(-1, 0, m_LastPointedArea != -1);

    if(m_LogicStatus == LOGIC_COMSTRATEGY){
        const TOpenFarenGame* game = g_GameMaster.GetGame();
        int mi = game->GetCurrentMasterIndex();
        m_LuaStrategy[mi].Resume();
        // AI헪삪IĂȂꍇ͑JڂȂB
        if(!m_LuaStrategy[mi].CanResume()){
            if(m_LogicStatus == LOGIC_COMSTRATEGY){
                //NextMaster();
                // LuaAPIEndTurn()Ă΂͂AĂ΂ĂȂΎőJڂB璷ȃR[hB
                m_LogicStatus = LOGIC_NEXTMASTER;
            }
        }
    }
    else if(m_LogicStatus == LOGIC_PLAYERMOVEWINDOW){
        const TCharacterInstance* chara = m_CharaMoveWindow->GetBrowsedCharacter();
        if(chara != NULL){
            m_CharaStatusWindow->Show(*chara);
            g_GuiContainerList.GetGui()->getTop()->moveToTop(m_CharaStatusWindow);
        }
    }
    else if(m_LogicStatus == LOGIC_DEPARTURE){
        TOpenFarenGame* game = g_GameMaster.GetGame();
        const TAreaInstance& area_target = game->GetAreaInstance(m_WarTargetArea);
        if(!m_DepatureFrame){
            g_GameMaster.GetMixer()->PlayPermanentChunk(TOpenFarenMixer::CHUNK_DEPARTURE);
        }
        if(m_DepatureFrame == m_DepatureFrameMax){
            // owL\NAB
            m_WorldMapWindow->GetWorldMapPane().ClearDepartureChara();
            // ̃L`FbNB
            if(!game->GetAreaInstance(m_WarTargetArea).m_charalist.size()){
                // ̃LȂΐ퓬ɓ炸B
                game->MoveCharaAfterNoBattle(m_DepatureCharaIndice, m_WarTargetArea, game->GetCurrentMasterIndex());
                // bZ[W\B
                const TMasterInstance& masins = game->GetMasterInstance(game->GetCurrentMasterIndex());
                const TOpenFarenData* data = g_GameMaster.GetData();
                std::string msg = (boost::format("%sȐł") % data->GetMaster(masins.m_type).m_name).str();
                TGcnMsgDlg::Show("Information", msg, TGcnMsgDlg::BUTTON_OK, g_Ini.GetMessageTimeoutFrame());
                m_LogicStatus_AfterMessage = LOGIC_AFTER_WAR;
                m_LogicStatus = LOGIC_SHOWING_MESSAGE;
                return;
            }
            // [h}bv[hł̃EBhEBB
            //m_WorldMapWindow->setVisible(false);
            // 퓬Jn
            // LXg
            std::vector<TCharacterInstanceList*> charalist;
            // owGÃLCX^XRs[ăXgB
            m_SaveCharaInstance = game->GetAreaInstance(m_WarTargetArea).m_charalist;
            charalist.push_back(&m_SaveCharaInstance);
            // owGÃLCX^XRs[ăXgB
            m_DepatureCharaInstance.clear();
            for(unsigned u=0; u<m_DepatureCharaIndice.size(); ++u){
                int areaindex = m_DepatureCharaIndice[u].first;
                const TAreaInstance& area = game->GetAreaInstance(areaindex);
                m_DepatureCharaInstance.push_back(area.m_charalist[m_DepatureCharaIndice[u].second]);
            }
            charalist.push_back(&m_DepatureCharaInstance);

            std::vector<int> masterindex(2);
            masterindex[0] = area_target.m_master;
            masterindex[1] = game->GetCurrentMasterIndex();

            // owGAB
            std::vector<std::vector<int> > neighborarea(2);
            neighborarea[0].resize(m_SaveCharaInstance.size());
            neighborarea[0].assign(m_SaveCharaInstance.size(), -1); // ͑S-1
            std::transform(m_DepatureCharaIndice.begin(), m_DepatureCharaIndice.end(), std::back_inserter(neighborarea[1]), Select1st<std::pair<int, int> >());

            bool nowatch = m_BattleMapLogic->Start(game->GetBattleMap(m_WarTargetArea), charalist, masterindex, m_WarTargetArea, neighborarea);
            m_WorldMapWindow->setVisible(nowatch);
            if(!nowatch && m_BattleMapLogic->CanNoWatch()){
                m_WorldMenuWindow->setVisible(true);
            }
            m_LogicStatus = LOGIC_WAR;
            return;
        }
        ++m_DepatureFrame;
        //const int framemax = g_Ini.GetDepartureAnimationFrame();
        const int framemax = m_DepatureStepMax;
        // eGȀowLBۂɎĝ͈ꕔ蔲B
        std::vector<int> characount(game->GetAreaInstanceCount());
        for(int i = m_DepatureCharaIndice.size() - 1; i >= 0; --i){
            int areaindex = m_DepatureCharaIndice[i].first;
            const TAreaInstance& area = game->GetAreaInstance(areaindex);
            int frame = m_DepatureFrame - DEPATUREFRAME_PERCHARA * characount[areaindex];
            //int frame = m_DepatureFrame - characount[areaindex];
            // owGÃL𐔂B
            ++characount[areaindex];
            if(frame < 0){
                frame = 0;
            }else if(frame > framemax){
                frame = framemax;
            }
            int x = (area.m_location.x * (framemax - frame) + area_target.m_location.x * frame) / framemax;
            int y = (area.m_location.y * (framemax - frame) + area_target.m_location.y * frame) / framemax;
            m_WorldMapWindow->GetWorldMapPane().SetDepartureCharaPosition(i, x, y);
        }
    }
    else if(m_LogicStatus == LOGIC_SCRIPT){
        if(!m_LuaWorld.Resume()){
            m_LogicStatus = LOGIC_ERROR;
            return;
        }
        if(!m_LuaWorld.CanResume()){
            if(m_ScriptStatus == SCRIPT_MASTERDEAD ||
               m_ScriptStatus == SCRIPT_GAMEOVER){
                m_LogicStatus = LOGIC_AFTER_WAR;
            }
            else if(m_ScriptStatus == SCRIPT_PLAYERUI && m_LogicStatus == LOGIC_SCRIPT){
                // Luaɂm_LogicStatusύXĂȂ΃[UԂɖ߂B
                m_LogicStatus = LOGIC_PLAYERSTRATEGY;
            }
        }
        else{
            if(m_LogicStatus != LOGIC_SCRIPT &&
               m_ScriptStatus == SCRIPT_PLAYERUI){
                g_Logger.WarningLog("m_LogicStatusXNvgŕύX̂ɂ̃[gɓʖڂƎv񂾂ǁBvOXNvgȂƂȂ̂ł́H");
            }
        }
    }
    else if(m_LogicStatus == LOGIC_EVENTSCRIPT){
        if(!m_LuaEvent.Resume()){
            m_LogicStatus = LOGIC_ERROR;
            return;
        }
        if(!m_LuaEvent.CanResume()){
            const TOpenFarenGame* game = g_GameMaster.GetGame();
            if(game->IsEnding()){
                // GfBOXNvg̎sIvZXƏIB
                SDL_Event ev;
                memset(&ev, 0, sizeof ev);
                ev.type = SDL_QUIT;
                SDL_PushEvent(&ev);
                m_LogicStatus = LOGIC_IDLE;
                return;
            }
            m_ScriptAPI->HideAllWindow();
            StartMasterPhase();
        }
    }
    else if(m_LogicStatus == LOGIC_WAR){
        if(m_BattleMapLogic->IsEnd()){
            DoBattleEndLogic();
        }
    }
    else if(m_LogicStatus == LOGIC_AFTER_WAR){
        TOpenFarenGame* game = g_GameMaster.GetGame();
        game->UpdateMasterExists();
        if(!m_IsWatchMode){
            if(game->JudgePlayerGameOver()){
                m_IsWatchMode = true;
                if(!m_LuaWorld.CallFunction("GameOver")){
                    m_LogicStatus = LOGIC_ERROR;
                    return;
                }
                if(m_LuaWorld.CanResume()){
                    m_LogicStatus = LOGIC_SCRIPT;
                    m_ScriptStatus = SCRIPT_GAMEOVER;
                }
                return;
            }
        }
        const TMasterInstance& master = game->GetMasterInstance(game->GetCurrentMasterIndex());
        if(!master.m_exists){
            //NextMaster();
            m_LogicStatus = LOGIC_NEXTMASTER;
        }
        else{
            if(master.m_player){
                // [h}bv[hł̃EBhE߂B
                m_CharaListWindow->setVisible(true);
                m_MasterWindow->setVisible(true);
                m_CharaStatusWindow->setVisible(true);
                g_GameMaster.SetSysMenuMode(SYSMENU_WORLD);
                if(g_Ini.GetBoolValue(TOpenFarenIni::MUSIC_ENABLED)){
                    g_GameMaster.GetMixer()->FadeOutMusic();
                    const TOpenFarenData* data = g_GameMaster.GetData();
                    const TMaster& masterdata = data->GetMaster(master.m_type);
                    if(masterdata.m_music != -1){
                        g_GameMaster.GetMixer()->PlayMusic(masterdata.m_music);
                    }
                }
                m_LogicStatus = LOGIC_PLAYERSTRATEGY;
            }
            else{
                m_LogicStatus = LOGIC_COMSTRATEGY;
            }
        }
    }
    else if(m_LogicStatus == LOGIC_NEXTMASTER){
        NextMaster();
    }
    else if(m_LogicStatus == LOGIC_PLAYEREMPLOYHERO_NEXT){
        // ܂opłlނ邩B
        PlayerNextEmployHero();
    }
    else if(m_LogicStatus == LOGIC_SHOWING_MESSAGE){
        if(!TGcnMsgDlg::IsVisible()){
            m_LogicStatus = m_LogicStatus_AfterMessage;
        }
    }
    else if(m_LogicStatus == LOGIC_CONFIRM_MESSAGE){
        if(!TGcnMsgDlg::IsVisible()){
            int button = TGcnMsgDlg::GetPressedButtonStatic();
            if(button == TGcnMsgDlg::BUTTON_YES || button == TGcnMsgDlg::BUTTON_OK){
                TOpenFarenGame* game = g_GameMaster.GetGame();
                game->DismissCharacter(m_DismissIndex.first, m_DismissIndex.second);
                // ّΏۃL̑IB
                m_CharaListWindow->GetCharaListPane().ClearState();
            }
            m_LogicStatus = LOGIC_PLAYERSTRATEGY;
        }
    }
    else if(m_LogicStatus == LOGIC_CONFIRM_ALLIANCE){
        if(!TGcnMsgDlg::IsVisible()){
            int button = TGcnMsgDlg::GetPressedButtonStatic();
            bool ok = (button == TGcnMsgDlg::BUTTON_YES || button == TGcnMsgDlg::BUTTON_OK);
            TOpenFarenGame* game = g_GameMaster.GetGame();
            if(game->GetMasterInstance(game->GetCurrentMasterIndex()).m_player){
                // vC[COMւ̐\̏ꍇB
                if(ok){
                    bool agree;
                    const TMasterInstance& masterins2 = game->GetMasterInstance(m_ProposedAllianceMaster);
                    if(masterins2.m_player){
                        // }X^[vC[Ȃ疳ɒB
                        agree = true;
                    }
                    else{
                        // }X^[COMȂvl[`ɖ₢킹B
                        if(!QueryAllianceToCom(game->GetCurrentMasterIndex(), m_ProposedAllianceMaster, agree)){
                            m_LogicStatus = LOGIC_ERROR;
                            return;
                        }
                    }
                    const TOpenFarenData* data = g_GameMaster.GetData();
                    const TMaster& master2 = data->GetMaster(masterins2.m_type);
                    std::string msg;
                    if(agree){
                        game->ConcludeAlliance(game->GetCurrentMasterIndex(), m_ProposedAllianceMaster, m_ProposedAllianceTurn, m_ProposedAllianceCost);
                        msg = (boost::format("%sƓт܂B") % master2.m_name).str();
                    }
                    else{
                        msg = (boost::format("%sɓf܂B") % master2.m_name).str();
                    }
                    TGcnMsgDlg::Show("Information", msg, TGcnMsgDlg::BUTTON_OK);
                }
                m_LogicStatus = LOGIC_PLAYERSTRATEGY;
            }
            else{
                // COMvC[ւ̐\̏ꍇB
                if(ok){
                    game->ConcludeAlliance(game->GetCurrentMasterIndex(), m_ProposedAllianceMaster, m_ProposedAllianceTurn, m_ProposedAllianceCost);
                }
                m_LogicStatus = LOGIC_COMSTRATEGY;
            }
        }
    }
    else if(m_LogicStatus == LOGIC_PLAYERPOWERMAP){
        if(!m_PowerMapWindow->isVisible()){
            m_LogicStatus = LOGIC_PLAYERSTRATEGY;
        }
    }
}
//---------------------------------------------------------------------------
void TWorldMapLogic::Save(TiXmlElement& e) const
{
    TiXmlElement e_lua(XMLELM_LUAEVENTTOP);
    m_LuaEvent.SaveToXML(e_lua);
    e.InsertEndChild(e_lua);
}
//---------------------------------------------------------------------------
bool TWorldMapLogic::Load(const TiXmlElement& e)
{
    const TiXmlElement *e_lua = e.FirstChildElement(XMLELM_LUAEVENTTOP);
    if(e_lua == NULL){
        g_Logger.ErrorLog((boost::format("%s not found.") % XMLELM_LUAEVENTTOP).str());
        return false;
    }
    m_LuaEvent.LoadFromXML(*e_lua);
    return true;
}
//---------------------------------------------------------------------------
void TWorldMapLogic::EndTurn()
{
    m_WorldMapWindow->GetWorldMapPane().ClearBrightArea();
    m_CharaListWindow->setVisible(false);
    m_MasterWindow->setVisible(false);
    m_CharaStatusWindow->setVisible(false);
    g_GameMaster.SetSysMenuVisible(false);
    if(g_Ini.GetBoolValue(TOpenFarenIni::MUSIC_ENABLED)){
        g_GameMaster.GetMixer()->FadeOutMusic();
    }
    m_LogicStatus = LOGIC_NEXTMASTER;
}
//---------------------------------------------------------------------------
void TWorldMapLogic::Departure(int area_to, const std::vector<std::pair<int, int> >& departurecharalist)
{
    UpdateAreaPowerHelp(0, 0, false);
    m_WarTargetArea = area_to;
    m_DepatureCharaIndice = departurecharalist;
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    // owGAɃXN[
    const TAreaInstance& area_t = game->GetAreaInstance(m_WarTargetArea);
    m_WorldMapWindow->SetScrollCenter(area_t.m_location.x, area_t.m_location.y);
    // ő̏owGA|owGAԋB
    int maxdist = 0;
    // eGȀowLBۂɎĝ͈ꕔ蔲B
    std::vector<int> characount(game->GetAreaInstanceCount());
    // owAj[Vo^Bo^ɕ`悷̂ŋtœo^B
    for(int i = m_DepatureCharaIndice.size() - 1; i >= 0; --i){
        const TAreaInstance& area = game->GetAreaInstance(m_DepatureCharaIndice[i].first);
        m_WorldMapWindow->GetWorldMapPane().AddDepartureChara(area.m_location.x, area.m_location.y, area.m_charalist[m_DepatureCharaIndice[i].second].m_type);
        ++characount[m_DepatureCharaIndice[i].first];
        // owGA|owGAԋB
        int distx = abs(area_t.m_location.x - area.m_location.x);
        int disty = abs(area_t.m_location.y - area.m_location.y);
        int dist = max(distx, disty);
        if(maxdist < dist){
            maxdist = dist;
        }
    }
    // eGA̒ōłowL̑GȀowL߂B
    int maxchara = *std::max_element(characount.begin(), characount.end());
    m_DepatureStepMax = (maxdist + DEPATURE_PIXEL_PERFRAME - 1) / DEPATURE_PIXEL_PERFRAME;
    //m_DepatureFrameMax = g_Ini.GetDepartureAnimationFrame() + maxchara*DEPATUREFRAME_PERCHARA;
    m_DepatureFrameMax = m_DepatureStepMax + maxchara*DEPATUREFRAME_PERCHARA;
    m_DepatureFrame = 0;
    m_LogicStatus = LOGIC_DEPARTURE;
}
//---------------------------------------------------------------------------
// ߂l͓̉ۂ肳ꂽǂBfalsȅꍇȂ̂Luayieldresume҂KvB
bool TWorldMapLogic::ProposeAlliance(int masterindex, int turn, int cost)
{
    const TOpenFarenData* data = g_GameMaster.GetData();
    TOpenFarenGame* game = g_GameMaster.GetGame();
    int curmaster = game->GetCurrentMasterIndex();
    const TMasterInstance& masterins = game->GetMasterInstance(masterindex);
    const TMasterInstance& curmasterins = game->GetMasterInstance(curmaster);
    const TMaster& master1 = data->GetMaster(curmasterins.m_type);
    const TMaster& master2 = data->GetMaster(masterins.m_type);
    if(masterins.m_player){
        m_ProposedAllianceMaster = masterindex;
        m_ProposedAllianceTurn = turn;
        m_ProposedAllianceCost = cost;
        std::string msg;
        if(turn == TOpenFarenGame::DEFAULT_ALLIANCETURN){
            msg = (boost::format("%s%d%s%d^[̓\ŗ܂B\nт܂H") % master1.m_name % cost % (data->GetMoneyUnit()) % turn).str();
        }
        else{
            msg = (boost::format("%s%d%s%d^[̓\ŗ܂B\n܂H") % master1.m_name % cost % (data->GetMoneyUnit()) % turn).str();
        }
        TGcnMsgDlg::Show("mF", msg, TGcnMsgDlg::BUTTON_YES | TGcnMsgDlg::BUTTON_NO);
        m_LogicStatus = LOGIC_CONFIRM_ALLIANCE;
        return false;
    }
    else
    {
/*
        std::vector<std::string> args;
        args.push_back(master1.m_id);
        args.push_back(master2.m_id);
        if(!m_LuaStrategy[masterindex].CallFunction(LUAFUNC_PROPOSEALLIANCE, args)){
            m_LogicStatus = LOGIC_ERROR;
            return false;
        }
        if(m_LuaStrategy[masterindex].GetLastFunctionValue()){
            game->ConcludeAlliance(curmaster, masterindex, turn, cost);

            if(turn == TOpenFarenGame::DEFAULT_ALLIANCETURN){
                std::string msg = (boost::format("%s%sƓт܂B") % master1.m_name % master2.m_name).str();
                TGcnMsgDlg::Show("Information", msg, TGcnMsgDlg::BUTTON_OK, g_Ini.GetMessageTimeoutFrame());
                m_LogicStatus_AfterMessage = LOGIC_PLAYEREMPLOYHERO_NEXT;
                m_LogicStatus_AfterMessage = LOGIC_COMSTRATEGY;
                m_LogicStatus = LOGIC_SHOWING_MESSAGE;
                return false;
            }
        }
*/
        bool agree;
        if(!QueryAllianceToCom(curmaster, masterindex, agree)){
            m_LogicStatus = LOGIC_ERROR;
            return false;
        }
        if(agree){
            game->ConcludeAlliance(curmaster, masterindex, turn, cost);

            if(turn == TOpenFarenGame::DEFAULT_ALLIANCETURN){
                std::string msg = (boost::format("%s%sƓт܂B") % master1.m_name % master2.m_name).str();
                TGcnMsgDlg::Show("Information", msg, TGcnMsgDlg::BUTTON_OK, g_Ini.GetMessageTimeoutFrame());
                m_LogicStatus_AfterMessage = LOGIC_PLAYEREMPLOYHERO_NEXT;
                m_LogicStatus_AfterMessage = LOGIC_COMSTRATEGY;
                m_LogicStatus = LOGIC_SHOWING_MESSAGE;
                return false;
            }
        }
        return true;
    }
}
//---------------------------------------------------------------------------
void TWorldMapLogic::ApplyConfig()
{
    m_BattleMapLogic->ApplyConfig();
    if(g_Ini.GetBoolValue(TOpenFarenIni::MUSIC_ENABLED)){
        // yĂȂɉyIvVONɂȂ炷B
        if(!g_GameMaster.GetMixer()->IsPlayingMusic()){
            if(m_LogicStatus != LOGIC_WAR){
                TOpenFarenGame* game = g_GameMaster.GetGame();
                const TMasterInstance& master = game->GetMasterInstance(game->GetCurrentMasterIndex());
                if(master.m_player){
                    const TOpenFarenData* data = g_GameMaster.GetData();
                    const TMaster& masterdata = data->GetMaster(master.m_type);
                    if(masterdata.m_music != -1){
                        g_GameMaster.GetMixer()->PlayMusic(masterdata.m_music);
                    }
                }
            }
        }
    }
    else{
        g_GameMaster.GetMixer()->FadeOutMusic();
    }
}
//---------------------------------------------------------------------------
void TWorldMapLogic::DoBattleEndLogic()
{
    // [h}bv[hł̃EBhEꕔ߂B
    m_WorldMapWindow->setVisible(true);
    m_WorldMenuWindow->setVisible(false);

    TOpenFarenGame* game = g_GameMaster.GetGame();
    int winmaster = m_BattleMapLogic->GetWinMaster();
g_Logger.DebugLog((boost::format("WinMaster %d") % winmaster).str());

/*
    // 퓬ɎQāASlނID𒊏oBQpB
    const TOpenFarenData* data = g_GameMaster.GetData();
    // ow
    std::set<int> deadid_dep;
    for(int i = m_DepatureCharaInstance.size() - 1; i >= 0; --i){
        if(!m_DepatureCharaInstance[i].m_hp){
            int ctype = m_DepatureCharaInstance[i].m_type;
            if(data->GetCharacter(ctype).m_hero)){
                deadid_dep.insert(ctype);
            }
        }
    }
    // 
    std::set<int> deadid_sav;
    for(int i=0; i<m_SaveCharaInstance.size(); ++i){
        if(!m_SaveCharaInstance[i].m_hp){
            int ctype = m_SaveCharaInstance[i].m_type;
            if(data->GetCharacter(ctype).m_hero)){
                deadid_sav.insert(ctype);
            }
        }
    }
    // 퓬Q}X^[̗̓y玀SĂȂSlނID𒊏oB̊yB
    // MoveCharaAfterBattleDepWin()Ŕ肷͖̂ʓ|̂łCȂB
    // ɁAMoveCharaAfterBattleDepWin()ƃLXg퓬̕ɂȂĂ܂
    // łȂƐlނ肵Ă܂B
    TCharacterInstanceList hero_dep;
    TCharacterInstanceList hero_sav;
    int depmaster = game->GetCurrentMasterIndex();
    int savmaster = game->GetAreaInstance(m_WarTargetArea).m_master;
    int areacount = game->GetAreaInstanceCount();
    for(int ai=0; ai<areacount; ++ai){
        const TArea& area = game->GetAreaInstance(ai);
        if(area.m_master == -1){
            continue;
        }
        if(area.m_master == depmaster){
            for(int i=0; i<area.m_charalist.size(); ++i){
                const TCharacterInstance& chara = area.m_charalist[i];
                if(deadid_dep.find(chara.m_type) == deadid_dep.end()){
                    hero_dep.push_back(chara);  // łȂΒǉB
                }
            }
        }
        else if(area.m_master == savmaster){
            for(int i=0; i<area.m_charalist.size(); ++i){
                const TCharacterInstance& chara = area.m_charalist[i];
                if(deadid_sav.find(chara.m_type) == deadid_sav.end()){
                    hero_sav.push_back(chara);  // łȂΒǉB
                }
            }
        }
    }
*/

    // 퓬̃LړȂǁB
    TCharacterInstanceList hero_sav;    // L̐clނɑޔB}X^[SpBSĂȂ͓ɎgȂB
    game->MoveCharaAfterBattleDepWin(m_DepatureCharaIndice, m_DepatureCharaInstance, m_SaveCharaInstance, hero_sav, m_WarTargetArea, game->GetCurrentMasterIndex(), winmaster != 0);

    // n`XVB
    game->UpdateAreaAndBMapAfterBattle(m_WarTargetArea);

    std::vector<int> lostmaster;
    game->JudgeLostMaster(lostmaster);

    if(!lostmaster.empty()){
        // lޕQB}X^[Sꍇ̂ŗB
        for(int i=0; i<lostmaster.size(); ++i){
            if(lostmaster[i] == winmaster){
                // U}X^[SꍇAU}X^[̐lނQB
                //TCharacterInstanceList dummy;
                //game->WanderAfterWar(lostmaster[i], dummy);
            }
            else{
                // }X^[SꍇA}X^[̐lނQB
                //game->WanderAfterWar(lostmaster[i], hero_sav);
                for(int i=0; i<hero_sav.size(); ++i){
                    game->AddWanderChara(hero_sav[i]);
                }
            }
        }

        if(!m_LuaWorld.CallFunction("MasterDead")){
            m_LogicStatus = LOGIC_ERROR;
            return;
        }
        if(m_LuaWorld.CanResume()){
            m_LogicStatus = LOGIC_SCRIPT;
            m_ScriptStatus = SCRIPT_MASTERDEAD;
        }
        else{
            m_LogicStatus = LOGIC_AFTER_WAR;
        }
    }
    else{
        m_LogicStatus = LOGIC_AFTER_WAR;
    }
}
//---------------------------------------------------------------------------
void TWorldMapLogic::NextMaster()
{
    bool newturn = false;
    TOpenFarenGame* game = g_GameMaster.GetGame();
    if(game->JudgeOvercome()){
        game->StartEnding();
    }
    else{
        newturn = game->ActivateNextMaster();
    }

    if(newturn || game->IsEnding()){
        if(!m_LuaEvent.CallFunction("global_script")){
            m_LogicStatus = LOGIC_ERROR;
            return;
        }
    }
    if(m_LuaEvent.CanResume()){
        if(g_Ini.GetBoolValue(TOpenFarenIni::MUSIC_ENABLED)){
            g_GameMaster.GetMixer()->PlayPermanentMusicWithFadeout(TOpenFarenMixer::MUSIC_EVENT);
        }
        m_LogicStatus = LOGIC_EVENTSCRIPT;
        return;
    }
    StartMasterPhase();
}
//---------------------------------------------------------------------------
// }X^[̃tFCYɓB
void TWorldMapLogic::StartMasterPhase()
{
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TMasterInstance& master = game->GetMasterInstance(game->GetCurrentMasterIndex());
    if(master.m_player){
        //m_EmployHeroWindow->Show(areaindex, charatype, master.m_money);
        //m_LogicStatus = LOGIC_PLAYEREMPLOYHERO;
        PlayerNextEmployHero();
    }
    else{
        // {͐lނ̓opCOMɂfB͏ȂB
        StartMasterStrategy();
    }
}
//---------------------------------------------------------------------------
// lޓop̌̃}X^[헪JnB
void TWorldMapLogic::StartMasterStrategy()
{
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TMasterInstance& masterins = game->GetMasterInstance(game->GetCurrentMasterIndex());
    if(masterins.m_player){
OFDBGLOG((boost::format("PLAYER")).str());
        m_LogicStatus = LOGIC_PLAYERSTRATEGY;

        int basearea = game->GetMasterBaseArea(game->GetCurrentMasterIndex());
        const TOpenFarenData* data = g_GameMaster.GetData();
        const TMaster& master = data->GetMaster(masterins.m_type);
        if(basearea == -1){
            g_Logger.ErrorLog((boost::format("}X^[ %s ̏݃GǍɎsiScenarioX.xmlŃ}X^[zuĂȂHj") % master.m_name).str());
            m_LogicStatus = LOGIC_ERROR;
            return;
        }
        const TAreaInstance& area = game->GetAreaInstance(basearea);
        m_WorldMapWindow->SetScrollCenter(area.m_location.x, area.m_location.y);    // TODO
        m_CharaListWindow->setPosition(4, 48);
        //m_CharaListWindow->SetSelectedArea(basearea);
        AddAreaSelection(basearea);
        m_CharaListWindow->Show(true);

        m_MasterWindow->setPosition(4, m_CharaListWindow->getY() + m_CharaListWindow->getHeight() + 8);
        m_MasterWindow->SetMasterInstance(game->GetCurrentMasterIndex());
        m_MasterWindow->Show();

        g_GameMaster.SetSysMenuMode(SYSMENU_WORLD);
        g_GameMaster.SetSysMenuVisible(true);

        std::string msg;
        int curmaster = game->GetCurrentMasterIndex();
        int mascount = game->GetMasterInstanceCount();
        for(int i=0; i<mascount; ++i){
            int alindex = game->FindAlliance(curmaster, i);
            if(alindex != -1){
                const TAlliance& al = game->GetAlliance(alindex);
                if(!al.turn){
                    const TOpenFarenData* data = g_GameMaster.GetData();
                    const TMasterInstance& masterins2 = game->GetMasterInstance(i);
                    const TMaster& master1 = data->GetMaster(masterins.m_type);
                    const TMaster& master2 = data->GetMaster(masterins2.m_type);
                    if(!msg.empty()){
                        msg += '\n';
                    }
                    msg += (boost::format("%s%s̓؂܂B") % master1.m_name % master2.m_name).str();
                }
            }
        }
        if(!msg.empty()){
            TGcnMsgDlg::Show("Information", msg, TGcnMsgDlg::BUTTON_OK, g_Ini.GetMessageTimeoutFrame());
        }

        if(g_Ini.GetBoolValue(TOpenFarenIni::MUSIC_ENABLED)){
            g_GameMaster.GetMixer()->FadeOutMusic();
            if(master.m_music != -1){
                g_GameMaster.GetMixer()->PlayMusic(master.m_music);
            }
        }
    }
    else{
OFDBGLOG((boost::format("COM")).str());
        int mi = game->GetCurrentMasterIndex();
        m_LogicStatus = LOGIC_COMSTRATEGY;
        if(!m_LuaStrategy[mi].CallFunction("on_myturn")){
            m_LogicStatus = LOGIC_ERROR;
        }
        // Resumes̏ꍇAm_LogicStatus͑Jڍς݂ƂBłȂDoLogicResumeG[B
OFDBGLOG((boost::format("%d") % m_LogicStatus).str());
    }
}
//---------------------------------------------------------------------------
// vC[}X^[̃tFCYJnAɌقlނΐlޓopEBhE\BłȂ΃}X^[헪JnB
void TWorldMapLogic::PlayerNextEmployHero()
{
    TOpenFarenGame* game = g_GameMaster.GetGame();
    int areaindex, charatype;
    if(game->PopFoundWanderingChara(areaindex, charatype)){
        // ܂lނ΂܂lޓopEBhE\B
        const TMasterInstance& master = game->GetMasterInstance(game->GetCurrentMasterIndex());
        m_EmployHeroWindow->Show(areaindex, charatype, master.m_money);
        m_LogicStatus = LOGIC_PLAYEREMPLOYHERO;
    }
    else{
        // lނ΃}X^[헪JnցB
        StartMasterStrategy();
    }
}
//---------------------------------------------------------------------------
// [h}bv̍WʏłǂɈʒu邩肷B
void TWorldMapLogic::GetWorldMapPointIs(int x, int y, bool& isLeft, bool& isTop) const
{
    const gcn::Widget* top = g_GuiContainerList.GetGui()->getTop();
    int scx, scy;
    m_WorldMapWindow->GetScrollAmount(scx, scy);
    isLeft = m_WorldMapWindow->getX() + m_WorldMapWindow->getPadding() + x - scx < top->getWidth() / 2;
    isTop = m_WorldMapWindow->getY() + m_WorldMapWindow->getTitleBarHeight() + y - scy < top->getHeight() / 2;
}
//---------------------------------------------------------------------------
bool TWorldMapLogic::ShowMoveCharaWindow(int area_from, int area_to)
{
    // }X^[אڃGAȂړEBhEoB
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TAreaInstance& areaf = game->GetAreaInstance(area_from);
    const TAreaInstance& areat = game->GetAreaInstance(area_to);
    if(areaf.m_master != areat.m_master ||
       std::find(areaf.m_neighbor.begin(), areaf.m_neighbor.end(), area_to) == areaf.m_neighbor.end()){
        return false;
    }
    m_WorldMapWindow->GetWorldMapPane().AddBrightEdge(area_from, area_to);
    m_CharaMoveWindow->Show(area_from, areaf.m_charalist, area_to, areat.m_charalist);
    gcn::Widget* top = g_GuiContainerList.GetGui()->getTop();
    top->moveToTop(m_CharaMoveWindow);
    // LړEBhËʒuB
    int centerx = (areaf.m_location.x + areat.m_location.x) / 2;
    int centery = (areaf.m_location.y + areat.m_location.y) / 2;
    bool isLeft, isTop;
    GetWorldMapPointIs(centerx, centery, isLeft, isTop);
    // YW͒SƂB
    int y = (top->getHeight() - m_CharaMoveWindow->getHeight()) / 2;
    // XW͓KɍEɊ񂹂B
    int x;
    const int marginx = 64;
    if(isLeft){
        // E
        x = (top->getWidth() / 2) + marginx;
    }
    else{
        // 
        x = (top->getWidth() / 2) - m_CharaMoveWindow->getWidth() - marginx;
    }
/*
    int x = (top->getWidth() - m_CharaMoveWindow->getWidth()) / 2;
    int y;
    const int marginy = 64;
    int scx, scy;
    m_WorldMapWindow->GetScrollAmount(scx, scy);
    // S_ʂ̏㕔Ȃ
    if(m_WorldMapWindow->getY() + m_WorldMapWindow->getTitleBarHeight() + centery - scy > top->getHeight() / 2){
        y = marginy;
    }
    else{
        y = top->getHeight() - marginy;
    }
*/
    m_CharaMoveWindow->setPosition(x, y);
    //m_WorldMapWindow->SetScrollCenter(centerx, centery);
    return true;
}
//---------------------------------------------------------------------------
void TWorldMapLogic::ShowWarCharaWindow(int area_to)
{
    m_WarTargetArea = area_to;
    m_WarCharaWindow->Show(area_to);
    gcn::Widget* top = g_GuiContainerList.GetGui()->getTop();
    top->moveToTop(m_WarCharaWindow);
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TAreaInstance& area = game->GetAreaInstance(area_to);
    for(unsigned u=0; u<area.m_neighbor.size(); ++u){
        if(game->GetAreaInstance(area.m_neighbor[u]).m_master == game->GetCurrentMasterIndex()){
            m_WorldMapWindow->GetWorldMapPane().AddBrightEdge(area_to, area.m_neighbor[u]);
        }
    }
    int centerx = area.m_location.x;
    int centery = area.m_location.y;
    bool isLeft, isTop;
    GetWorldMapPointIs(centerx, centery, isLeft, isTop);
    // YW͒SƂB
    int y = (top->getHeight() - m_WarCharaWindow->getHeight()) / 2;
    // XW͓KɍEɊ񂹂B
    int x;
    const int marginx = 64;
    if(isLeft){
        // E
        x = (top->getWidth() / 2) + marginx;
    }
    else{
        // 
        x = (top->getWidth() / 2) - m_WarCharaWindow->getWidth() - marginx;
    }
    m_WarCharaWindow->setPosition(x, y);
}
//---------------------------------------------------------------------------
int TWorldMapLogic::GetAreaAt(int x, int y) const
{
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    for(int i = 0; i < game->GetAreaInstanceCount(); i++){
        const TAreaInstance& area = game->GetAreaInstance(i);
        if(area.m_ignore){
            continue;
        }
        //if(area.m_location.x - 4 <= x && area.m_location.x + 28 > x &&
        //   area.m_location.y <= y && area.m_location.y + 32 > y){
        //    return i;
        //}
        // TWorldMapPane::draw()ł x+6, y+30 ~W̒SƂĂB
        int ax = area.m_location.x + 6;
        int ay = area.m_location.y + 30;
        if(ax - 24 < x && x < ax + 24 &&
           ay - 24 < y && y < ay + 24 ){
            return i;
        }
    }
    return -1;
}
//---------------------------------------------------------------------------
// 邩ǂAIւ̖₢킹B
bool TWorldMapLogic::QueryAllianceToCom(int masterindex1, int masterindex2, bool& agree)
{
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TMasterInstance& masterins1 = game->GetMasterInstance(masterindex1);
    const TMasterInstance& masterins2 = game->GetMasterInstance(masterindex2);
    const TMaster& master1 = data->GetMaster(masterins1.m_type);
    const TMaster& master2 = data->GetMaster(masterins2.m_type);
    std::vector<std::string> args;
    args.push_back(master1.m_id);
    args.push_back(master2.m_id);
    if(!m_LuaStrategy[masterindex2].CallFunction(LUAFUNC_PROPOSEALLIANCE, args)){
        return false;
    }
    agree = m_LuaStrategy[masterindex2].GetLastFunctionValue();
    return true;
}
//---------------------------------------------------------------------------
// ͎w\XVB
void TWorldMapLogic::UpdateAreaPowerHelp(int mousex, int mousey, bool pointed)
{
    if(!pointed){
        m_HintPane->GoHidden();
        m_LastPointedArea = -1;
        m_MousePointCount = 0;
        return;
    }
    int ai;
    if(mousex != -1){
        ai = GetAreaAt(mousex, mousey);
        if(ai == -1){
            m_HintPane->GoHidden();
            m_LastPointedArea = -1;
            m_MousePointCount = 0;
            return;
        }
    }
    else{
        ai = m_LastPointedArea;
    }

    if(m_LastPointedArea == ai){
        ++m_MousePointCount;
    }
    else{
        m_LastPointedArea = ai;
        m_MousePointCount = 1;
        return;
    }
    int fps = g_Ini.GetIntValue(TOpenFarenIni::WINDOW_FPS);
    if(m_MousePointCount * 60 / fps > 2){
        const TOpenFarenGame* game = g_GameMaster.GetGame();
        const TAreaInstance& area = game->GetAreaInstance(ai);
        int power = 0;
        for(unsigned u=0; u<area.m_charalist.size(); ++u){
            power += area.m_charalist[u].m_power;
        }
        power /= 1000;
        std::string cap;
        if(area.m_master != -1){
            const TMasterInstance& masterins = game->GetMasterInstance(area.m_master);
            const TOpenFarenData* data = g_GameMaster.GetData();
            const TMaster& masterdata = data->GetMaster(masterins.m_type);
            cap = (boost::format("%s\n(͎w %d)") % masterdata.m_name % power).str();
        }
        else{
            cap = (boost::format("n\n(͎w %d)") % power).str();
        }

        m_HintPane->SetCaption(cap);
        int ax, ay;
        m_WorldMapWindow->GetWorldMapPane().getAbsolutePosition(ax, ay);
        ax += area.m_location.x;
        ay += area.m_location.y + 40;
        int rw, rh;
        g_Ini.GetRealWindowSize(rw, rh);
        if(rw < ax + m_HintPane->getWidth()){
            ax -= m_HintPane->getWidth();
        }
        if(rh < ay + m_HintPane->getHeight()){
            ay -= m_HintPane->getHeight() + 40;
        }
        m_HintPane->setPosition(ax, ay);
        m_HintPane->GoShown();
        g_GuiContainerList.GetGui()->getTop()->moveToTop(m_HintPane);
    }
}
//---------------------------------------------------------------------------
// GAIB
void TWorldMapLogic::AddAreaSelection(int areaindex)
{
    m_WorldMapWindow->GetWorldMapPane().ClearBrightArea();
    m_WorldMapWindow->GetWorldMapPane().AddBrightArea(areaindex, 0xffffff00);
    m_CharaListWindow->SetSelectedArea(areaindex);
}
//---------------------------------------------------------------------------
// ʕ\GAIԂɖ߂B
void TWorldMapLogic::RecoveryAreaSelection()
{
    AddAreaSelection(m_CharaListWindow->GetCharaListPane().GetSelectedArea());
}
//---------------------------------------------------------------------------
// ʕ\owGAIԂɂB
void TWorldMapLogic::UpdateWarAreaSelection()
{
    m_WorldMapWindow->GetWorldMapPane().ClearBrightArea();
    if(m_WarTargetArea != -1){
        m_WorldMapWindow->GetWorldMapPane().AddBrightArea(m_WarTargetArea, 0xffff0000);
    }
    if(m_WarCharaWindow->GetLeftArea() != -1){
        m_WorldMapWindow->GetWorldMapPane().AddBrightArea(m_WarCharaWindow->GetLeftArea(), 0xff0000ff);
    }
}
//---------------------------------------------------------------------------
// ʕ\GA|CgԂɂB
void TWorldMapLogic::UpdateAreaPointed(int areaindex)
{
    m_WorldMapWindow->GetWorldMapPane().ClearCircleArea();
    if(areaindex == -1){
        return;
    }
    int selected = m_CharaListWindow->GetCharaListPane().GetSelectedArea();
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    int master = game->GetAreaInstance(areaindex).m_master;
    if(master != -1){
        unsigned color;
        if(master == game->GetCurrentMasterIndex()){
            color = 0xff00ffff;
        }
        else{
            color = 0xff00ff00;
        }
        int areacount = game->GetAreaInstanceCount();
        for(int ai=0; ai<areacount; ++ai){
            // GAI~\͉~\ȂB
            if(ai == selected && m_LogicStatus == LOGIC_PLAYERSTRATEGY){
                continue;
            }
            const TAreaInstance& area = game->GetAreaInstance(ai);
            if(area.m_master != master){
                continue;
            }
            m_WorldMapWindow->GetWorldMapPane().AddCircleArea(ai, color);
        }
    }
    else{
        // nу|CgB
        // GAI~\͉~\ȂB
        if(areaindex != selected || m_LogicStatus != LOGIC_PLAYERSTRATEGY){
            m_WorldMapWindow->GetWorldMapPane().AddCircleArea(areaindex, 0xff00ff00);
        }
    }
}
//---------------------------------------------------------------------------
