//---------------------------------------------------------------------------
#ifndef worldmapH
#define worldmapH

#include <vector>
//#include <boost/shared_ptr.hpp>
#include <boost/tuple/tuple.hpp>
#include <guichan.hpp>
#include "imagepanelarge.h"
#include "areaope.h"
#include "charastatus.h"
#include "hintpane.h"
#include "gcnwindow.h"
#include "oflua.h"
#include "scriptapi.h"
//---------------------------------------------------------------------------
class TWorldMenuPane : public gcn::Container
{
private:
    gcn::Button m_ButtonAbortWatch;
public:
    TWorldMenuPane();
    virtual ~TWorldMenuPane() {}
    gcn::Button& GetButtonAbortWatch() { return m_ButtonAbortWatch; }
};
//---------------------------------------------------------------------------
class TWorldMenuWindow : public TGcnWindow
{
private:
    TWorldMenuPane m_MenuPane;
public:
    TWorldMenuWindow();
    virtual ~TWorldMenuWindow() {}
    gcn::Button& GetButtonAbortWatch() { return m_MenuPane.GetButtonAbortWatch(); }
};
//---------------------------------------------------------------------------
class TWorldMapPane : public TGcnImagePaneLarge
{
private:
    int m_Tick;
    std::vector<std::pair<int, int> > m_BrightEdgeList;
    //int m_BrightEdgeTick;
    //int m_BrightArea;
    std::vector<unsigned> m_BrightAreaList;
    int m_BrightAreaTick;
    std::vector<unsigned> m_CircleAreaList;
    // owL̃XgBvfx,y,TBattleCharacter̃CfbNXB
    typedef std::vector<boost::tuple<int, int, int> > TDepatureCharaList;
    TDepatureCharaList m_DepatureCharaList;
    static const int m_FontSize = 16;
public:
    TWorldMapPane();
    virtual ~TWorldMapPane();
    void UpdateAreaCount();
    void AddBrightEdge(int area1, int area2);
    void ClearBrightEdge();
    //void SetBrightArea(int area);
    void AddBrightArea(int area, unsigned color);
    void ClearBrightArea();
    void AddCircleArea(int area, unsigned color);
    void ClearCircleArea();
    void AddDepartureChara(int x, int y, int ctype);
    void SetDepartureCharaPosition(int index, int x, int y);
    void ClearDepartureChara();
    virtual void logic();
    virtual void draw(gcn::Graphics *g);
};
//---------------------------------------------------------------------------
class TWorldMapLogic;
class TWorldMapWindow : public TGcnWindow
{
private:
    //gcn::Image* m_image;
    gcn::ScrollArea m_ScrollArea;
    TWorldMapPane m_WorldMapPane;
    TWorldMapLogic *m_Logic;
    std::string m_Guidance;
    static const int m_FontSize = 16;
public:
    TWorldMapWindow();
    virtual ~TWorldMapWindow();
    bool Show();
    TWorldMapPane& GetWorldMapPane() { return m_WorldMapPane; }
    void SetWorldMapLogic(TWorldMapLogic *l){ m_Logic = l; }
    void GetScrollAmount(int& x, int& y);
    void SetScrollAmount(int x, int y);
    void SetScrollCenter(int x, int y);
    void SetGuidance(const std::string& str) { m_Guidance = str; }
    void ClearGuidance() { m_Guidance.clear(); }
    virtual void draw(gcn::Graphics *g);
    virtual void logic();
};
//---------------------------------------------------------------------------
class TGcnWidgetSet;
class TBattleMapLogic;
class TAreaGridWindow;
class TMasterGridWindow;
class THeroGridWindow;
class TPowerMapWindow;
class TWorldMapLogic : public gcn::MouseListener, public gcn::ActionListener, public gcn::SelectionListener
{
public:
    TWorldMapLogic(TGcnWidgetSet *wlist);
    virtual ~TWorldMapLogic();
    virtual void mousePressed(gcn::MouseEvent &mouseEvent);
    virtual void mouseMoved(gcn::MouseEvent &mouseEvent);
    virtual void mouseExited(gcn::MouseEvent &mouseEvent);
    virtual void valueChanged(const gcn::SelectionEvent &event);
    virtual void action(const gcn::ActionEvent& actionEvent);
    void Start(bool isNewGame, bool isWatch, const TiXmlElement *e);
    void FocusArea(int areaindex);
    void GetTopSize(int &w, int& h) const;
    void DoLogic();
    void Save(TiXmlElement& e) const;
    bool Load(const TiXmlElement& e);
    void EndTurn();
    void Departure(int area_to, const std::vector<std::pair<int, int> >& departurecharalist);
    bool ProposeAlliance(int masterindex, int turn, int cost);
    void ApplyConfig();
private:
    TGcnWidgetSet *m_WidgetSet;
    TWorldMapWindow *m_WorldMapWindow;
    TCharaListWindow *m_CharaListWindow;
    TMasterWindow *m_MasterWindow;
    TCharaStatusWindow *m_CharaStatusWindow;
    TCharaStatusFullWindow *m_CharaStatusFullWindow;
    TEmployWindow *m_EmployWindow;
    TCharaMoveWindow *m_CharaMoveWindow;
    TWarCharaWindow *m_WarCharaWindow;
    TEmployHeroWindow *m_EmployHeroWindow;
    TAreaGridWindow *m_AreaGridWindow;
    TMasterGridWindow *m_MasterGridWindow;
    THeroGridWindow *m_HeroGridWindow;
    TPowerMapWindow *m_PowerMapWindow;
    TWorldMenuWindow *m_WorldMenuWindow;

    // CxgXNvg
    TOpenFarenLuaManager m_LuaEvent;
    // Q[I[o[̃XNvg
    TOpenFarenLuaManager m_LuaWorld;
    // 헪AIXNvg
    std::vector<TOpenFarenLuaManager> m_LuaStrategy;

    boost::shared_ptr<TBattleMapLogic> m_BattleMapLogic;
    boost::shared_ptr<TScriptAPI> m_ScriptAPI;

    enum TLogicStatus{
        LOGIC_PLAYERSTRATEGY = 1,
        LOGIC_PLAYERMOVE,
        LOGIC_PLAYERMOVEWINDOW,
        LOGIC_PLAYEREMPLOY,
        LOGIC_PLAYERAREALIST,
        LOGIC_PLAYERMASTERLIST,
        LOGIC_PLAYERHEROLIST,
        LOGIC_PLAYERPOWERMAP,
        LOGIC_PLAYERWAR,
        LOGIC_PLAYERWARWINDOW,
        LOGIC_PLAYERALLIANCE,
        LOGIC_PLAYEREMPLOYHERO,
        LOGIC_PLAYEREMPLOYHERO_NEXT,
        LOGIC_COMSTRATEGY,
        LOGIC_DEPARTURE,
        LOGIC_WAR,
        LOGIC_AFTER_WAR,
        LOGIC_NEXTMASTER,
        LOGIC_SCRIPT,
        LOGIC_EVENTSCRIPT,
        LOGIC_SHOWING_MESSAGE,
        LOGIC_CONFIRM_MESSAGE,
        LOGIC_CONFIRM_ALLIANCE,
        LOGIC_IDLE,
        LOGIC_ERROR
    }m_LogicStatus;
    TLogicStatus m_LogicStatus_AfterMessage;

    enum TScriptStatus{
        SCRIPT_MASTERDEAD = 1,
        SCRIPT_GAMEOVER,
        SCRIPT_PLAYERUI
    }m_ScriptStatus;

    bool m_IsWatchMode;
    int m_CharaListLastClickedTick;
    int m_CharaListLastClickedIndex;
    int m_EmployListLastClickedTick;
    //int m_LastSelectedArea;
    int m_MoveFromArea;
    int m_MoveToArea;
    // c[`bv͎̐w\pB
    int m_MousePointCount;
    int m_LastPointedArea;
    TGcnHintPane *m_HintPane;

    // ٗpB蔲BfirstGAԍsecondLԍB
    std::pair<int, int> m_DismissIndex;

    // owpB
    int m_WarTargetArea;
    int m_DepatureFrame;
    int m_DepatureStepMax;
    int m_DepatureFrameMax;
    std::vector<std::pair<int, int> > m_DepatureCharaIndice;
    TCharacterInstanceList m_DepatureCharaInstance;
    TCharacterInstanceList m_SaveCharaInstance;

    // vC[ɑ΂铯ďB
    int m_ProposedAllianceMaster;
    int m_ProposedAllianceTurn;
    int m_ProposedAllianceCost;

    static const int DEPATUREFRAME_PERCHARA = 3;
    static const int DEPATURE_PIXEL_PERFRAME = 4;

    void DoBattleEndLogic();
    void NextMaster();
    void StartMasterPhase();
    void StartMasterStrategy();
    void PlayerNextEmployHero();
    void GetWorldMapPointIs(int x, int y, bool& isLeft, bool& isTop) const;
    bool ShowMoveCharaWindow(int area_from, int area_to);
    void ShowWarCharaWindow(int area_to);
    int GetAreaAt(int x, int y) const;
    bool QueryAllianceToCom(int masterindex1, int masterindex2, bool& agree);
    void UpdateAreaPowerHelp(int mousex, int mousey, bool pointed);
    void AddAreaSelection(int areaindex);
    void RecoveryAreaSelection();
    void UpdateWarAreaSelection();
    void UpdateAreaPointed(int areaindex);
};
//---------------------------------------------------------------------------
#endif
