//---------------------------------------------------------------------------
#include "battlemap.h"
#include "bmap.h"
#include "msgdlg.h"
#include "gui.h"
#include "widgetset.h"
#include "ofini.h"
#include "ofimage.h"
#include "ofmixer.h"
#include "drawutil.h"
#include "gamemaster.h"
#include "gcnskinmanager.h"
#include "charaplace.h"
#include "battlecommand.h"
#include "magicwindow.h"
#include "battlecharalist.h"
#include "logger.h"
#include "boostformat.h"
#include "boostlexicalcast.h"
#include "mtrand.h"
#include "neighbor.h"
#include "formula.h"
#include "porting.h"
#include <cstring>

#define LOGIC_TO_PHY_X(x, y) (x * TOpenFarenImage::MAPCHIP_WIDTH + (y%2)*(TOpenFarenImage::MAPCHIP_WIDTH/2) - TOpenFarenImage::MAPCHIP_WIDTH/2)
const int STEP_MAX = 256;
//---------------------------------------------------------------------------
//i}EXJ[\́jW->_W
void XY_PhyToMap(const TBattleMap& bmap, int PhyX, int PhyY, int *MapX, int *MapY)
{
    *MapY = PhyY/TOpenFarenImage::MAPCHIP_HEIGHT;
    if(!((*MapY)%2))
        PhyX += TOpenFarenImage::MAPCHIP_WIDTH/2;
    *MapX = PhyX/TOpenFarenImage::MAPCHIP_WIDTH;
    if(*MapX < 0)
        *MapX = 0;
    if(bmap.GetWidth() <= *MapX)
        *MapX = bmap.GetWidth()-1;
    if(*MapY < 0)
        *MapY = 0;
    if(bmap.GetHeight() <= *MapY)
        *MapY = bmap.GetHeight()-1;
}
//---------------------------------------------------------------------------
void DrawForceBrave(gcn::Graphics *g, int basecenterx, int basey, int leftforce, int leftbrave, int rightforce, int rightbrave, TGcnCharDrawer *chardrawersmall)
{
    const TGcnSkinManager *skinmng = g_GameMaster.GetSkinManager();
    gcn::Color dark = skinmng->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE);
    static const gcn::Color white(255, 255, 255);

    int centerx = basecenterx;

    chardrawersmall->DrawString(g, centerx - 15, basey + 2,  "Force", white);
    chardrawersmall->DrawString(g, centerx - 15, basey + 16, "Brave", white);

    int lforce = min(leftforce/16000, 112);
    lforce = max(1, lforce);
    gcn::Rectangle lforcerect_d(centerx - 20 - lforce,     basey + 4, lforce, 8);
    gcn::Rectangle lforcerect_f(centerx - 20 - lforce + 1, basey + 5, lforce - 2, 6);
    g->setColor(gcn::Color(255,0,0));
    g->fillRectangle(lforcerect_f);
    g->setColor(gcn::Color(0,0,0));
    g_DrawUtil.DrawRectangle(g, lforcerect_d);

    int lbrave = leftbrave;
    lbrave = max(1, lbrave);
    gcn::Rectangle lbraverect(centerx - 20 - lbrave, basey + 18, lbrave, 8);
    g->setColor(gcn::Color(255,0,0));
    g->fillRectangle(lbraverect);
    g->setColor(gcn::Color(0,0,0));
    g_DrawUtil.DrawRectangle(g, lbraverect);

    int rforce = min(rightforce/16000, 112);
    rforce = max(1, rforce);
    gcn::Rectangle rforcerect(centerx + 20, basey + 4, rforce, 8);
    g->setColor(gcn::Color(255,0,0));
    g->fillRectangle(rforcerect);
    g->setColor(gcn::Color(0,0,0));
    g_DrawUtil.DrawRectangle(g, rforcerect);

    int rbrave = rightbrave;
    rbrave = max(1, rbrave);
    gcn::Rectangle rbraverect(centerx + 20, basey + 18, rbrave, 8);
    g->setColor(gcn::Color(255,0,0));
    g->fillRectangle(rbraverect);
    g->setColor(gcn::Color(0,0,0));
    g_DrawUtil.DrawRectangle(g, rbraverect);
}
//---------------------------------------------------------------------------
TBattleMapPane::TBattleMapPane()
    : m_Tick(0),
    m_ShadeLayer(NULL)
{
}
//---------------------------------------------------------------------------
TBattleMapPane::~TBattleMapPane()
{
}
//---------------------------------------------------------------------------
void TBattleMapPane::SetMap(TBattleMap *bmap)
{
    m_map = bmap;
    int w = bmap->GetWidth() * TOpenFarenImage::MAPCHIP_WIDTH - TOpenFarenImage::MAPCHIP_WIDTH/2;
    int h = bmap->GetHeight() * TOpenFarenImage::MAPCHIP_HEIGHT;
    setSize(w, h);
    ClearBrightChip();
}
//---------------------------------------------------------------------------
int TBattleMapPane::AddShowingChara(const TShowingChara& c)
{
    int i = m_ShowingCharaList.size();
    m_ShowingCharaList.push_back(c);
    return i;
}
//---------------------------------------------------------------------------
/*void TBattleMapPane::SetShowingChara(int i, const TShowingChara& c)
{
    m_ShowingCharaList[i] = c;
}
//---------------------------------------------------------------------------
void TBattleMapPane::RemoveShowingChara(int x, int y)
{
    for(unsigned u=0; u<m_ShowingCharaList.size(); ++u){
        if(m_ShowingCharaList[u].m_location1.x == x && m_ShowingCharaList[u].m_location1.y == y){
            m_ShowingCharaList.erase(m_ShowingCharaList.begin() + u);
            break;
        }
    }
}*/
//---------------------------------------------------------------------------
void TBattleMapPane::AddChipFrame(int x, int y)
{
    m_ChipFrameList.push_back(gcn::Rectangle(LOGIC_TO_PHY_X(x, y), y * TOpenFarenImage::MAPCHIP_HEIGHT, m_ChipWidth-1, m_ChipHeight-1));
}
//---------------------------------------------------------------------------
void TBattleMapPane::SetBrightChip(int x, int y)
{
    m_BrightChip.x = x;
    m_BrightChip.y = y;
    m_BrightStep = 0;
}
//---------------------------------------------------------------------------
void TBattleMapPane::ClearBrightChip()
{
    m_BrightChip.x = -1;
}
//---------------------------------------------------------------------------
void TBattleMapPane::AddDamageString(int chipx, int chipy, const std::string& str, const gcn::Color& c)
{
    TDamageString ds;
    ds.m_location.x = LOGIC_TO_PHY_X(chipx, chipy);
    ds.m_location.y = chipy * TOpenFarenImage::MAPCHIP_HEIGHT + (TOpenFarenImage::MAPCHIP_HEIGHT>>2);
    ds.m_caption = str;
    ds.m_color = c;
    ds.m_tick = 0;
    m_DamageStringList.push_back(ds);
}
//---------------------------------------------------------------------------
void TBattleMapPane::AddRadiusChip(int chipx, int chipy, const gcn::Color& c)
{
    TRadiusChip rc;
    rc.m_location.x = LOGIC_TO_PHY_X(chipx, chipy);
    rc.m_location.y = chipy * TOpenFarenImage::MAPCHIP_HEIGHT;
    rc.m_color = c;
    m_RadiusChipList.push_back(rc);
}
//---------------------------------------------------------------------------
void TBattleMapPane::ClearRadiusChip()
{
    m_RadiusChipList.clear();
}
//---------------------------------------------------------------------------
void TBattleMapPane::AddEffectImage(int x, int y, int index, int anim, int dir, int width, int height)
{
    TEffectImage ei;
    ei.m_location.x = x;
    ei.m_location.y = y;
    ei.m_index = index;
    ei.m_anim = anim;
    ei.m_direction;
    ei.m_width = width;
    ei.m_height = height;
    ei.m_direction = dir;
    m_EffectImageList.push_back(ei);
//g_Logger.DebugLog((boost::format("ei=%d %d") % ei.m_location.x % ei.m_location.y).str());
}
//---------------------------------------------------------------------------
void TBattleMapPane::logic()
{
    ++m_Tick;
    ++m_BrightStep;
    if(!m_DamageStringList.empty()){
        for(std::vector<TDamageString>::reverse_iterator ritr = m_DamageStringList.rbegin(); ritr != m_DamageStringList.rend(); ++ritr){
            if(ritr->m_tick >= DAMAGETICK_MAX * m_AnimationSpeed){
                m_DamageStringList.erase((++ritr).base());
                if(ritr == m_DamageStringList.rend()){
                    break;
                }
            }
            else{
                ++(ritr->m_tick);
            }
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapPane::draw(gcn::Graphics *g)
{
    static const int damage_y[DAMAGETICK_MAX] = {0, -4, -7, -9, -10, -10, -9, -7, -4};
    //const TOpenFarenGame* game = g_GameMaster.GetGame();
    //const TOpenFarenData* data = g_GameMaster.GetData();
    const TOpenFarenImage* image = g_GameMaster.GetImage();
    int w = m_map->GetWidth();
    int h = m_map->GetHeight();
            //static bool b = false;
    for(int y=0; y<h; ++y){
        for(int x=0; x<w; ++x){
            int chip = m_map->GetMapChip(x, y);
            int obj = m_map->GetMapObj(x, y);
            int chipx = x * TOpenFarenImage::MAPCHIP_WIDTH + (y%2)*(TOpenFarenImage::MAPCHIP_WIDTH/2) - TOpenFarenImage::MAPCHIP_WIDTH/2;
            int chipy = y * TOpenFarenImage::MAPCHIP_HEIGHT;
            image->DrawMapChip(g, chipx, chipy, chip);
            if(obj){
                image->DrawMapObj(g, chipx, chipy, obj);
            }
            //if(!b){
            //    g_Logger.DebugLog((boost::format("%d, %d, %d") % x % y % obj).str());
            //}
        }
    }
                //b = true;

    if(m_BrightChip.x != -1){
        int i = 191 + (g_GuiContainerList.GetSin(m_BrightStep % g_GuiContainerList.GetTrigonometricMax()) >> 2);
//g_Logger.DebugLog((boost::format("i %d") % i).str());
        g->setColor(gcn::Color(255, i, i, 128));
        int chipx = LOGIC_TO_PHY_X(m_BrightChip.x, m_BrightChip.y);
        int chipy = m_BrightChip.y * TOpenFarenImage::MAPCHIP_HEIGHT;
        gcn::Rectangle r(chipx, chipy, m_ChipWidth, m_ChipHeight);
        g->fillRectangle(r);
    }

    // L`
    const TOpenFarenData* data = g_GameMaster.GetData();
    for(unsigned u=0; u<m_ShowingCharaList.size(); ++u){
        const TShowingChara& chara = m_ShowingCharaList[u];
        const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
        int chipx = LOGIC_TO_PHY_X(chara.m_location1.x, chara.m_location1.y);
        int chipy = chara.m_location1.y * TOpenFarenImage::MAPCHIP_HEIGHT;
        if(chara.m_step){
            chipx = (chipx * (STEP_MAX - chara.m_step) + LOGIC_TO_PHY_X(chara.m_location2.x, chara.m_location2.y) * chara.m_step)/STEP_MAX;
            chipy = (chipy * (STEP_MAX - chara.m_step) + (chara.m_location2.y * TOpenFarenImage::MAPCHIP_HEIGHT) * chara.m_step)/STEP_MAX;
        }
        image->DrawCharaChip(g, chipx, chipy, charadata.m_chipindex);
        //g->drawImage(image->GetChipImage(static_cast<TOpenFarenImage::TChipImageIndex>(chara.m_markchip)), chipx, chipy);
        image->DrawChip(g, chipx, chipy, static_cast<TOpenFarenImage::TChipImageIndex>(chara.m_markchip));
    }

    // KEZƖ@̕`
    for(std::vector<TEffectImage>::const_iterator itr = m_EffectImageList.begin(); itr != m_EffectImageList.end(); ++itr){
//g_Logger.DebugLog((boost::format("%d %d %d %d %d %d %d") % itr->m_location.x % itr->m_location.y % itr->m_index % itr->m_width % itr->m_height % itr->m_anim % itr->m_direction).str());
        image->DrawEffect(g, itr->m_location.x, itr->m_location.y, itr->m_index, itr->m_width, itr->m_height, itr->m_anim, itr->m_direction);
    }

    // zu̘gB
    for(unsigned u=0; u<m_ChipFrameList.size(); ++u){
        DrawChipFrame(g, m_ChipFrameList[u]);
    }

    // _[W^񕜐̕`
    for(std::vector<TDamageString>::const_iterator itr = m_DamageStringList.begin(); itr != m_DamageStringList.end(); ++itr){
        int y = damage_y[itr->m_tick / m_AnimationSpeed];
        g->setColor(gcn::Color(0, 0, 0));
        g->drawText(itr->m_caption, itr->m_location.x + 1, itr->m_location.y + y);
        g->setColor(itr->m_color);
        g->drawText(itr->m_caption, itr->m_location.x    , itr->m_location.y + y);
    }

    // e̕`
    if(m_ShadeLayer != NULL){
        g->setColor(g_Ini.GetBattleMapShadeColor());
        for(int y=0; y<h; ++y){
            for(int x=0; x<w; ++x){
                if(m_ShadeLayer->GetValue(x, y)){
                    int chipx = LOGIC_TO_PHY_X(x, y);
                    int chipy = y * TOpenFarenImage::MAPCHIP_HEIGHT;
                    gcn::Rectangle r(chipx, chipy, m_ChipWidth, m_ChipHeight);
                    g->fillRectangle(r);
                }
            }
        }
    }

    // ʔ͈͂̕`
    for(std::vector<TRadiusChip>::const_iterator itr = m_RadiusChipList.begin(); itr != m_RadiusChipList.end(); ++itr){
        g->setColor(itr->m_color);
        gcn::Rectangle r(itr->m_location.x, itr->m_location.y, TOpenFarenImage::MAPCHIP_WIDTH, TOpenFarenImage::MAPCHIP_HEIGHT);
        g->fillRectangle(r);
    }
}
//---------------------------------------------------------------------------
TBattleTopPane::TBattleTopPane()
{
    m_CharDrawerSmall = g_GuiContainerList.GetDrawer(TGuiContainerList::FONT_SIZE_SMALL);
}
//---------------------------------------------------------------------------
TBattleTopPane::~TBattleTopPane()
{
}
//---------------------------------------------------------------------------
void TBattleTopPane::SetString(const std::string& areaname, const std::string& lstr, const std::string& rstr)
{
    m_AreaName = areaname;
    m_LeftName = lstr;
    m_RightName = rstr;
}
//---------------------------------------------------------------------------
void TBattleTopPane::SetProperty(int lforce, int lbrave, int rforce, int rbrave, int turn)
{
    m_LeftForce = lforce;
    m_LeftBrave = lbrave;
    m_RightForce = rforce;
    m_RightBrave = rbrave;
    m_Turn = turn;
}
//---------------------------------------------------------------------------
void TBattleTopPane::draw(gcn::Graphics *g)
{
    const TGcnSkinManager *skinmng = g_GameMaster.GetSkinManager();
    gcn::Color dark = skinmng->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE);
    static const gcn::Color white(255, 255, 255);

    g->setColor(gcn::Color(64,64,64));
    gcn::Rectangle r(0, 0, getWidth(), getHeight());
    g->fillRectangle(r);

    gcn::Rectangle r_areaname(8, 4, 160, 20);
    gcn::Rectangle r_lname(176, 4, 160, 20);
    gcn::Rectangle r_rname(getWidth() - 276, 4, 160, 20);
    gcn::Rectangle r_turn(getWidth() - 104, 4, 96, 20);

    g->setColor(dark);
    g->fillRectangle(r_areaname);
    g->fillRectangle(r_rname);
    g->fillRectangle(r_lname);
    g->fillRectangle(r_turn);
    g->setColor(white);
    g->drawText(m_AreaName, r_areaname.x + 8, r_areaname.y + 2);
    g->drawText(m_AreaName, r_areaname.x + 9, r_areaname.y + 2);
    g->drawText(m_LeftName, r_lname.x + 8, r_lname.y + 2);
    g->drawText(m_LeftName, r_lname.x + 9, r_lname.y + 2);
    g->drawText(m_RightName, r_rname.x + 8, r_rname.y + 2);
    g->drawText(m_RightName, r_rname.x + 9, r_rname.y + 2);
    std::string strturn = (boost::format("%2d^[") % m_Turn).str();
    g->drawText(strturn, r_turn.x + 8, r_turn.y + 2);
    g->drawText(strturn, r_turn.x + 9, r_turn.y + 2);

    DrawForceBrave(g, getWidth()/2 + 32, 0, m_LeftForce, m_LeftBrave, m_RightForce, m_RightBrave, m_CharDrawerSmall);
/*
    int centerx = getWidth()/2 + 32;

    m_CharDrawer->DrawString(g, centerx - 15, 2, "Force", white);
    m_CharDrawer->DrawString(g, centerx - 15, 16, "Brave", white);

    int lforce = min(m_LeftForce/8000, 112);
    lforce = max(1, lforce);
    gcn::Rectangle lforcerect_d(centerx - 20 - lforce, 4, lforce, 8);
    gcn::Rectangle lforcerect_f(centerx - 20 - lforce + 1, 5, lforce - 2, 6);
    g->setColor(gcn::Color(255,0,0));
    g->fillRectangle(lforcerect_f);
    g->setColor(gcn::Color(0,0,0));
    //g->drawRectangle(lforcerect_d);
    g_DrawUtil.DrawRectangle(g, lforcerect_d);

    int lbrave = m_LeftBrave;
    lbrave = max(1, lbrave);
    gcn::Rectangle lbraverect(centerx - 20 - lbrave, 18, lbrave, 8);
    g->setColor(gcn::Color(255,0,0));
    g->fillRectangle(lbraverect);
    g->setColor(gcn::Color(0,0,0));
    //g->drawRectangle(lbraverect);
    g_DrawUtil.DrawRectangle(g, lbraverect);

    int rforce = min(m_RightForce/8000, 112);
    rforce = max(1, rforce);
    gcn::Rectangle rforcerect(centerx + 20, 4, rforce, 8);
    g->setColor(gcn::Color(255,0,0));
    g->fillRectangle(rforcerect);
    g->setColor(gcn::Color(0,0,0));
    //g->drawRectangle(rforcerect);
    g_DrawUtil.DrawRectangle(g, rforcerect);

    int rbrave = m_RightBrave;
    rbrave = max(1, rbrave);
    gcn::Rectangle rbraverect(centerx + 20, 18, rbrave, 8);
    g->setColor(gcn::Color(255,0,0));
    g->fillRectangle(rbraverect);
    g->setColor(gcn::Color(0,0,0));
    //g->drawRectangle(rbraverect);
    g_DrawUtil.DrawRectangle(g, rbraverect);
*/
}
//---------------------------------------------------------------------------
TBattleBottomPane::TBattleBottomPane() :
    m_PointedMoveType(-1)
{
    m_PointedChip.x = -1;
    m_CharDrawer = g_GuiContainerList.GetDrawer(m_FontSize);
}
//---------------------------------------------------------------------------
TBattleBottomPane::~TBattleBottomPane()
{
}
//---------------------------------------------------------------------------
void TBattleBottomPane::draw(gcn::Graphics *g)
{
    const TGcnSkinManager *skinmng = g_GameMaster.GetSkinManager();
    gcn::Color dark = skinmng->GetSkin()->GetColorValue(TOpenFarenSkin::SKINCOLOR_DARKPANE);

    g->setColor(gcn::Color(64,64,64));
    gcn::Rectangle r(0, 0, getWidth(), getHeight());
    g->fillRectangle(r);

    if(!m_Message.empty()){
        int w = m_Message.size() * 8 + 32;
        int x = (getWidth() - w) / 2;
        gcn::Rectangle r(x, 4, w, 24);
        g->setColor(dark);
        g->fillRectangle(r);
        g->setColor(gcn::Color(255,255,255));
        g->drawText(m_Message, x + 16, 8);
        g->drawText(m_Message, x + 17, 8);
    }
    if(m_OperatingChara.m_hp){
        const int x = getWidth() - 152;
        const int y = 2;
        const TOpenFarenData* data = g_GameMaster.GetData();
        const TOpenFarenImage* image = g_GameMaster.GetImage();
        const TBattleCharacter& charadata = data->GetCharacter(m_OperatingChara.m_type);
        g->setColor(dark);
        g->fillRectangle(gcn::Rectangle(x, y, 152, 44));
        image->DrawCharaChip(g, x + 2, y + 6, charadata.m_chipindex);
        image->DrawChip(g, x + 2, y + 6, static_cast<TOpenFarenImage::TChipImageIndex>(m_OperatingCharaMark));
        DrawSimpleCharaStatus(g, x + 40, y + 2, &m_OperatingChara);
    }
    if(m_PointedChip.x != -1){
        // |Cgn`̏`B
        const TOpenFarenData* data = g_GameMaster.GetData();
        const TOpenFarenImage* image = g_GameMaster.GetImage();
        int gndtype = m_Map->GetMapType(m_PointedChip.x, m_PointedChip.y);
        int gndcount = data->GetMapGroundCount();
        std::string name;
        if(gndtype < gndcount){
            name = data->GetMapGroundName(gndtype);
        }
        else{
            name = data->GetMapObjectName(gndtype - gndcount);
        }
        const int x = 4;
        const int y = 2;
        g->setColor(dark);
        g->fillRectangle(gcn::Rectangle(x, y, 128, 44));
        image->DrawMapChip(g, x + 4, y + 4, m_Map->GetMapChip(m_PointedChip.x, m_PointedChip.y));
        int obj = m_Map->GetMapObj(m_PointedChip.x, m_PointedChip.y);
        if(obj){
            image->DrawMapObj(g, x + 4, y + 4, obj);
        }
        gcn::Color c(255,255,255);
        m_CharDrawer->DrawString(g, x + 40, y + 2, name.c_str(), c);
        // L|CgĂꍇAZʕ␳`B
        if(m_PointedMoveType != -1){
            //const TBattleCharacter& charadata = data->GetCharacter(m_PointedCharaType);
            const TMoveType& mt = data->GetMoveType(m_PointedMoveType);
            std::string mvcost = (boost::format("Kvړ %3d") % mt.m_cost[gndtype]).str();
            m_CharDrawer->DrawString(g, x + 40, y + 16, mvcost.c_str(), c);
            std::string skill = (boost::format("ZʏC %+4d%%") % mt.m_skill[gndtype]).str();
            m_CharDrawer->DrawString(g, x + 40, y + 30, skill.c_str(), c);
        }
    }
}
//---------------------------------------------------------------------------
void TBattleBottomPane::SetPointedChip(int x, int y, int movetype)
{
    m_PointedChip.x = x;
    m_PointedChip.y = y;
    m_PointedMoveType = movetype;
}
//---------------------------------------------------------------------------
void TBattleBottomPane::ClearPointedChip()
{
    m_PointedChip.x = -1;
    m_PointedMoveType = -1;
}
//---------------------------------------------------------------------------
void TBattleBottomPane::SetOperatingChara(const TCharacterInstance& chara, int mark)
{
    m_OperatingChara = chara;
    m_OperatingCharaMark = mark;
}
//---------------------------------------------------------------------------
void TBattleBottomPane::ClearOperatingChara()
{
    m_OperatingChara.m_hp = 0;
}
//---------------------------------------------------------------------------
TBattleMapWindow::TBattleMapWindow()
{
    //setBaseColor(g_Ini.GetColorValue(TOpenFarenIni::GUICOLOR_WINDOW));
    setOpaque(false);
    //setTitleBarHeight(24);
    add(&m_ScrollArea);
    add(&m_BattleTopPane);
    add(&m_BattleBottomPane);
}
//---------------------------------------------------------------------------
TBattleMapWindow::~TBattleMapWindow()
{
}
//---------------------------------------------------------------------------
bool TBattleMapWindow::Show()
{
    const gcn::Widget* top = g_GuiContainerList.GetGui()->getTop();
    const int wmargin = 128;
    const int hmargin = 32;
    const int bottommargin = 48;
    const int topmargin = 32;
    int w = top->getWidth() - wmargin;
    int h = top->getHeight() - hmargin - topmargin - bottommargin;
    m_ScrollArea.setScrollbarWidth(g_Ini.GetIntValue(TOpenFarenIni::SCROLLBAR_WIDTH));
    m_ScrollArea.setContent(&m_BattleMapPane);
    w = min(w, m_BattleMapPane.getWidth());
    h = min(h, m_BattleMapPane.getHeight());
    m_ScrollArea.setPosition(0, topmargin);
    m_ScrollArea.setSize(w, h);
    //add(&m_ScrollArea);

    m_BattleTopPane.setPosition(0, 0);
    m_BattleTopPane.setSize(w, topmargin);
    //add(&m_BattleTopPane);

    m_BattleBottomPane.setPosition(0, topmargin + h);
    m_BattleBottomPane.setSize(w, bottommargin);
    //add(&m_BattleBottomPane);

    resizeToContent();
    //g_GuiContainerList.MoveWidgetToCenter(this);
    int x = top->getWidth() - getWidth();
    int y = (top->getHeight() - getHeight()) / 2;
    setPosition(x, y);
    setVisible(true);
    return true;
}
//---------------------------------------------------------------------------
void TBattleMapWindow::GetScrollAmount(int& x, int& y)
{
    x = m_ScrollArea.getHorizontalScrollAmount();
    y = m_ScrollArea.getVerticalScrollAmount();
}
//---------------------------------------------------------------------------
void TBattleMapWindow::SetScrollAmount(int x, int y)
{
    m_ScrollArea.setScrollAmount(x, y);
}
//---------------------------------------------------------------------------
void TBattleMapWindow::SetScrollCenter(int x, int y)
{
    const int margin = 96;
    int scw = m_ScrollArea.getWidth();
    int sch = m_ScrollArea.getHeight();
    int rax = m_ScrollArea.getHorizontalScrollAmount();
    int ray = m_ScrollArea.getVerticalScrollAmount();
    int nowx = rax + scw/2;
    int nowy = ray + sch/2;
    // ׂXN[̗}~B
    if(abs(nowx - x) < margin && abs(nowy - y) < margin){
        return;
    }

    int ax = x - scw/2;
    int ay = y - sch/2;
    if(ax < 0){
        ax = 0;
    }
    if(ax > m_ScrollArea.getHorizontalMaxScroll()){
        ax = m_ScrollArea.getHorizontalMaxScroll();
    }
    if(ay < 0){
        ay = 0;
    }
    if(ay > m_ScrollArea.getVerticalMaxScroll()){
        ay = m_ScrollArea.getVerticalMaxScroll();
    }
    m_ScrollArea.setScrollAmount(ax, ay);
}
//---------------------------------------------------------------------------
void TBattleMapWindow::SetScrollCenterOfChip(int chipx, int chipy)
{
    int x = LOGIC_TO_PHY_X(chipx, chipy);
    int y = chipy * TOpenFarenImage::MAPCHIP_HEIGHT;
    SetScrollCenter(x + TOpenFarenImage::MAPCHIP_WIDTH/2, y + TOpenFarenImage::MAPCHIP_HEIGHT/2);
}
//---------------------------------------------------------------------------
void TBattleMapWindow::SetScrollCenterOfChips(const std::vector<point_t>& plist)
{
    int x = 0, y = 0;
    size_t sz = plist.size();
    for(unsigned u=0; u<sz; ++u){
        x += LOGIC_TO_PHY_X(plist[u].x, plist[u].y);
        y += plist[u].y * TOpenFarenImage::MAPCHIP_HEIGHT;
    }
    SetScrollCenter(x/sz + TOpenFarenImage::MAPCHIP_WIDTH/2, y/sz + TOpenFarenImage::MAPCHIP_HEIGHT/2);
}
//---------------------------------------------------------------------------
void TBattleMapWindow::draw(gcn::Graphics *g)
{
    TGcnWindow::draw(g);
}
//---------------------------------------------------------------------------
void TBattleMapWindow::logic()
{
    gcn::Window::logic();
    // ŌĂԂ̂œK͕sB
    if(m_Logic != NULL){
        m_Logic->DoLogic();
    }
}
//---------------------------------------------------------------------------
TBattleMapLogic::TBattleMapLogic(TGcnWidgetSet *wset) :
    m_WidgetSet(wset),
    m_BattleMapWindow(NULL),
    //m_CharaPlaceWindow(NULL),
    //m_BattleCommandWindow(NULL),
    //m_MagicWindow(NULL),
    //m_AnimationPane(NULL),
    m_ShadeLayer(0, 0),
    m_CharaLayer(0, 0),
    m_MasterLayer(0, 0),
    m_MoveLayer(0, 0),
    m_DummyLayer(0, 0),
    m_MoveFromLayer(0, 0),
    m_LogicStatus(LOGIC_END)
{
    gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());

    {
        boost::shared_ptr<TBattleCommandWindow> pw(new TBattleCommandWindow());
        m_WidgetSet->Add(pw);
        m_BattleCommandWindow = pw.get();
        m_BattleCommandWindow->GetWaitButton().addActionListener(this);
        m_BattleCommandWindow->GetMoveButton().addActionListener(this);
        m_BattleCommandWindow->GetAttackButton().addActionListener(this);
        m_BattleCommandWindow->GetMagicButton().addActionListener(this);
        m_BattleCommandWindow->GetSpecialButton().addActionListener(this);
        m_BattleCommandWindow->GetChargeButton().addActionListener(this);
        m_BattleCommandWindow->GetRedoButton().addActionListener(this);
        m_BattleCommandWindow->GetRedo2Button().addActionListener(this);
        m_BattleCommandWindow->GetRunAwayButton().addActionListener(this);
        m_BattleCommandWindow->setVisible(false);
        top->add(m_BattleCommandWindow);
    }
    {
        boost::shared_ptr<TMagicWindow> pw(new TMagicWindow());
        m_WidgetSet->Add(pw);
        m_MagicWindow = pw.get();
        m_MagicWindow->SetActionListener(this);
        m_MagicWindow->setVisible(false);
        top->add(m_MagicWindow);
    }
    {
        boost::shared_ptr<TAnimationPane> pw(new TAnimationPane());
        m_WidgetSet->Add(pw);
        m_AnimationPane = pw.get();
        m_AnimationPane->setVisible(false);
        top->add(m_AnimationPane);
    }
    {
        boost::shared_ptr<TBattleCharaListWindow> pw(new TBattleCharaListWindow());
        m_WidgetSet->Add(pw);
        m_BattleCharaListWindow = pw.get();
        m_BattleCharaListWindow->setVisible(false);
        top->add(m_BattleCharaListWindow);
    }
    {
        boost::shared_ptr<TCharaPlaceWindow> pw(new TCharaPlaceWindow());
        m_WidgetSet->Add(pw);
        m_CharaPlaceWindow = pw.get();
        m_CharaPlaceWindow->GetCharaListPane().addMouseListener(this);
        m_CharaPlaceWindow->setVisible(false);
        top->add(m_CharaPlaceWindow);
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::SetExternalWindow(TCharaStatusWindow *csw, TCharaStatusFullWindow *csfw)
{
    m_CharaStatusWindow = csw;
    m_CharaStatusFullWindow = csfw;
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::Start(TBattleMap *bmap, const std::vector<TCharacterInstanceList*>& charalist, const std::vector<int>& masterindex, int areaindex, const std::vector<std::vector<int> >& neighborarea)
{
    m_ShadeLayer.Resize(0, 0);
    m_ShadeLayer.Resize(bmap->GetWidth(), bmap->GetHeight());
    m_MasterLayer.Resize(bmap->GetWidth(), bmap->GetHeight());
    m_DummyLayer.Resize(bmap->GetWidth(), bmap->GetHeight());
    m_MasterLayer.Init(-1);
    m_CharaLayer.Resize(bmap->GetWidth(), bmap->GetHeight());
    m_MoveLayer.Resize(0, 0);
    m_MoveLayer.Resize(bmap->GetWidth(), bmap->GetHeight());
    m_MoveFromLayer.Resize(bmap->GetWidth(), bmap->GetHeight());

    gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
    if(m_BattleMapWindow == NULL){
        boost::shared_ptr<TBattleMapWindow> pw(new TBattleMapWindow());
        m_WidgetSet->Add(pw);
        m_BattleMapWindow = pw.get();
        m_BattleMapWindow->GetBattleMapPane().addMouseListener(this);
        m_BattleMapWindow->GetBattleMapPane().SetShadeLayer(&m_ShadeLayer);
        m_BattleMapWindow->SetBattleMapLogic(this);   // {Listener`̕ǂȂB
        m_BattleMapWindow->setVisible(false);

        top->add(m_BattleMapWindow);
    }
    m_BattleMapWindow->GetBattleMapPane().SetMap(bmap);
    m_BattleMapWindow->GetBattleBottomPane().SetMap(bmap);
    bmap->GenerateTempData(g_GameMaster.GetData());
    m_Map = bmap;
    m_CharaList = charalist;
    //m_PlayerFlag = playerflag;
    m_MasterIndex = masterindex;
    m_AreaIndex = areaindex;
    m_NeighborArea = neighborarea;
    UpdateOnCharaChange();
    /*
    UpdateCommandableRace();
    UpdateAutoSpecialList();
    UpdateShowingChara();
    */
    // ̓\擾B
    std::vector<int> areaindexlist;
    std::vector<int> runable;
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    m_SaveRunAwayCount = game->GetRunnableArea(m_AreaIndex, areaindexlist, runable);
    m_RunAwayList.clear();
    // @gp񐔏B
    const TOpenFarenData* data = g_GameMaster.GetData();
    m_MagicUseCount.clear();
    m_MagicUseCount.resize(masterindex.size());
    for(int i=0; i<masterindex.size(); ++i){
        m_MagicUseCount[i].resize(data->GetMagicDataCount());
    }
    // Special疂@ւ̋te[u쐬BB
    m_SpecialMagicMap.clear();
    int mgcount = data->GetMagicGroupCount();
    for(int mg = 0; mg < mgcount; ++mg){
        const TMagicGroup& group = data->GetMagicGroup(mg);
        for(unsigned u=0; u<group.m_magiclist.size(); ++u){
            int mdindex = group.m_magiclist[u];
            const TMagicData& magicdata = data->GetMagicData(mdindex);
            for(unsigned rank=0; rank<magicdata.m_magics.size(); ++rank){
                const TMagic& magic = magicdata.m_magics[rank];
                // gp񐔐
                if(magicdata.m_limit){
                    m_SpecialMagicMap[magic.m_special] = mdindex;
                }
            }
        }
    }

    // SXgB
    m_DeadList.clear();
    m_DeadList.resize(masterindex.size());

    // ⏕pϐB
    m_LastSpecialTable.resize(masterindex.size());
    m_LastSpecial.resize(masterindex.size());
    m_LastSpecial.assign(masterindex.size(), -1);

    // vl[`ݒ
    std::string comfile;
    if(!g_GameMaster.FindScriptPath("battlecom.lua", comfile)){
        g_Logger.ErrorLog((boost::format("COMXNvǧɎs܂B")).str());
        m_LogicStatus = LOGIC_ERROR;
        return false;
    }
    InitMoveTypeCostSkill();
    m_LuaBattleCom.resize(m_CharaList.size());
    for(unsigned u=0; u<m_CharaList.size(); ++u){
        if(!m_LuaBattleCom[u].LoadLua(comfile.c_str())){
            g_Logger.ErrorLog((boost::format("COMXNvg %s ̃[hɎs܂B") % comfile).str());
            m_LogicStatus = LOGIC_ERROR;
            return false;
        }
        m_LuaBattleCom[u].SetIf(this, m_ScriptAPI, NULL);
        m_LuaBattleCom[u].SetBattleMasterIndex(u);
    }

    // 퓬Cxgݒ
    std::string eventfile;
    if(!g_GameMaster.FindScriptPath("battle.lua", eventfile)){
        g_Logger.ErrorLog((boost::format("퓬CxgXNvǧɎs܂B")).str());
        m_LogicStatus = LOGIC_ERROR;
        return false;
    }
    if(!m_LuaBattleEvent.LoadLua(eventfile.c_str())){
        g_Logger.ErrorLog((boost::format("퓬Cxg %s ̃[hɎs܂B") % eventfile).str());
        m_LogicStatus = LOGIC_ERROR;
        return false;
    }
    //m_LuaBattleEvent.SetIf(g_GameMaster.GetGame(), this);
    m_LuaBattleEvent.SetIf(this, m_ScriptAPI, NULL);

    g_GuiContainerList.GetGui()->addGlobalKeyListener(this);

    // zu[hŊJnB
    for(unsigned u=0; u<m_CharaList.size(); ++u){
        for(unsigned chr=0; chr<m_CharaList[u]->size(); ++chr){
            (*m_CharaList[u])[chr].m_location.x = -1;
        }
    }
    m_PlaceMasterIndex = 0;
    m_ShownCharaMaster = -1;
    m_EffectTarget.x = -1;
    m_Turn = 1;
    m_TurnLoop = 0;    // ^[P񑽂s|Cg𑝂₷B
    m_LogicStatus = LOGIC_PLACE;

    for(unsigned u=0; u<m_CharaList.size(); ++u){
        m_LuaBattleCom[u].CallFunction("on_battle_start");
    }

    m_AnimationSpeed = g_Ini.GetIntValue(TOpenFarenIni::BATTLE_ANIMSPEED);
    if(m_AnimationSpeed < TFarenGameMaster::BATTLESPEED_MIN){
        m_AnimationSpeed = TFarenGameMaster::BATTLESPEED_MIN;
    }
    else if(m_AnimationSpeed > TFarenGameMaster::BATTLESPEED_MAX){
        m_AnimationSpeed = TFarenGameMaster::BATTLESPEED_MAX;
    }

    // ϐ픻B
    m_NoWatch = false;
    if(CanNoWatch()){
        bool watch = false;
        if(g_Ini.GetBoolValue(TOpenFarenIni::WATCH_USE)){
            // R
            if(!g_Ini.GetBoolValue(TOpenFarenIni::WATCH_NONEUTRAL) ||
               std::find(m_MasterIndex.begin(), m_MasterIndex.end(), -1) == m_MasterIndex.end()){
                // }X^[݂̑Ɛ͔
                bool masterexists = false;
                int power = 0;
                int sz = m_CharaList.size();
                for(int i=0; i<sz; ++i){
                    unsigned cnt = (*m_CharaList[i]).size();
                    for(unsigned chr=0; chr<cnt; ++chr){
                        const TCharacterInstance & ci = (*m_CharaList[i])[chr];
                        power += ci.m_power;
                        const TBattleCharacter& charadata = data->GetCharacter(ci.m_type);
                        masterexists |= charadata.m_master;
                    }
                }
                if(!g_Ini.GetBoolValue(TOpenFarenIni::WATCH_ONLYLARGE) || power > 5000){
                    watch = true;
                }
                if(!g_Ini.GetBoolValue(TOpenFarenIni::WATCH_ONLYMASTER) || masterexists){
                    watch = true;
                }
            }
        }
        m_NoWatch = !watch;
        //if(g_Ini.GetBoolValue(TOpenFarenIni::WATCH_MAXSPEED)){
        //    m_AnimationSpeed = TFarenGameMaster::BATTLESPEED_MIN;
        //}
    }

/*
    m_BattleMapWindow->GetBattleMapPane().SetAnimationSpeed(m_AnimationSpeed);

    m_AnimationPerChip = m_AnimationSpeed;
    static const int mvframe[5] = {1, 2, 3, 4, 8};
    m_MoveFramePerChip = mvframe[m_AnimationSpeed - 1];
    static const int atkframe[5] = {2, 4, 8, 16, 32};
    m_AttackFrameMax = atkframe[m_AnimationSpeed - 1];
*/
    UpdateBattleSpeed();

    if(m_NoWatch){
        g_Logger.DebugLog("ϐȂ");
        ShowBattleCharaList(false);
    }
    else{
        PlayBattleMusic();
        m_BattleMapWindow->Show();
        UpdateSideName();
        UpdateForceBrave();
    }
g_Logger.DebugLog("Start");
    return m_NoWatch;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::DoLogic()
{
    //if(m_NoWatch){
    //    OFDBGLOG((boost::format("m_LogicStatus=%d") % m_LogicStatus).str());
    //}
    if(m_LogicStatus == LOGIC_PLAYERCOMMAND_BEFOREMOVE || m_LogicStatus == LOGIC_PLAYERCOMMAND_AFTERMOVE){
        if(g_GameMaster.IsShowingConfig()){
            return;
        }
        SDLKey key;
        if(g_GameMaster.GetKeyQueue(key)){
            g_GameMaster.PopKeyQueue();
            TOpenFarenIni::TShortcutValue scval;
            if(g_Ini.FindShortcutValue(key, scval)){
                gcn::Widget *srcs[TOpenFarenIni::SHORTCUTVAL_MAX] = {
                    &m_BattleCommandWindow->GetWaitButton(),
                    &m_BattleCommandWindow->GetMoveButton(),
                    &m_BattleCommandWindow->GetAttackButton(),
                    &m_BattleCommandWindow->GetChargeButton(),
                    &m_BattleCommandWindow->GetSpecialButton(),
                    &m_BattleCommandWindow->GetMagicButton(),
                    &m_BattleCommandWindow->GetRunAwayButton()
                };
                gcn::ActionEvent evt(srcs[scval], "");
                action(evt);
            }
        }
    }
    else if(m_LogicStatus == LOGIC_PLACE){
        if(IsPlayerMaster(m_PlaceMasterIndex)){
            // vC[̏zuɓB
            m_PlayerPlacedCount = 0;
            PlayerNextPlace();
        }
        else{
            // AȈzu
            // {LuaĂяoĂ̌ʂŔzuB
            GetPlaceList(m_NeighborArea[m_PlaceMasterIndex], *m_CharaList[m_PlaceMasterIndex], 0, m_PlaceXYList, m_PlaceCharaList);
            int offset = 0;
            for(;;){
                if(m_PlaceCharaList.size() > m_PlaceXYList.size()){
                    g_Logger.ErrorLog((boost::format("zuW܂i%djBmaster=%d charas=%d") % m_PlaceXYList.size() % m_PlaceMasterIndex % m_PlaceCharaList.size()).str());
                    m_LogicStatus = LOGIC_ERROR;
                    return;
                }
                for(unsigned u=0; u<m_PlaceCharaList.size(); ++u){
                    //m_PlaceCharaList[u].m_location = m_PlaceXYList[u];
                    (*m_CharaList[m_PlaceMasterIndex])[offset + u].m_location = m_PlaceXYList[u];
                    m_MasterLayer.GetValue(m_PlaceXYList[u].x, m_PlaceXYList[u].y) = m_PlaceMasterIndex;
                    m_CharaLayer.GetValue(m_PlaceXYList[u].x, m_PlaceXYList[u].y) = offset + u;
                }
                offset += m_PlaceCharaList.size();
                if(offset == (*m_CharaList[m_PlaceMasterIndex]).size()){
                    break;
                }
                GetPlaceList(m_NeighborArea[m_PlaceMasterIndex], *m_CharaList[m_PlaceMasterIndex], offset, m_PlaceXYList, m_PlaceCharaList);
            }
            UpdateShowingChara();
            // ̃}X^[̔zuցB
            ++m_PlaceMasterIndex;
            if(m_PlaceMasterIndex >= m_CharaList.size()){
                // S}X^[̔zuB
                m_LogicStatus = LOGIC_PLACECOMPLETE;
            }
        }
    }
    else if(m_LogicStatus == LOGIC_PLACECOMPLETE){
        for(unsigned master=0; master<m_CharaList.size(); ++master){
            for(unsigned u=0; u<m_CharaList[master]->size(); ++u){
                (*m_CharaList[master])[u].m_actionpoint = 0;
            }
        }
        //m_Turn = 1;
        //m_TurnLoop = 0;    // ^[P񑽂s|Cg𑝂₷B
        m_ShadeLayer.Init();
        m_LogicStatus = LOGIC_NEXTCHARA;

#if 0
        // Jp
        TCharacterInstance& c = (*m_CharaList[0])[0];
        m_MasterLayer.GetValue(c.m_location.x, c.m_location.y) = -1;
        c.m_location = (*m_CharaList[1])[0].m_location;
        c.m_location.x -= 2;
        m_MasterLayer.GetValue(c.m_location.x, c.m_location.y) = 0;
        m_CharaLayer.GetValue(c.m_location.x, c.m_location.y) = 0;
        UpdateShowingChara();
        //(*m_CharaList[1])[0].m_hp = 1;
#endif
    }
    else if(m_LogicStatus == LOGIC_MOVING){
        ++m_MoveFrame;
        TCharacterInstance& c = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
        if(m_MoveFrame == m_MoveFrameMax){
            DoMoveEndLogic();
            return;
        }
        int backcount = (m_MoveFrameMax - m_MoveFrame - 1) / m_MoveFramePerChip;
        //int movestep = m_MoveFrame % m_MoveFramePerChip;
        //const TCharacterInstance& c = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
        point_t p_to = m_MoveTo;
        for(int i=0; i<backcount; ++i){
            p_to = m_MoveFromLayer.GetValue(p_to.x, p_to.y);
        }
        TShowingChara showchara;
        std::memset(&showchara, 0, sizeof showchara);
        showchara.m_type = c.m_type;
        showchara.m_location1 = m_MoveFromLayer.GetValue(p_to.x, p_to.y);
        showchara.m_location2 = p_to;
        showchara.m_markchip = (m_ActiveMaster != 0);
        showchara.m_step = (m_MoveFrame % m_MoveFramePerChip) * STEP_MAX / m_MoveFramePerChip;
        m_BattleMapWindow->GetBattleMapPane().SetShowingChara(c.m_showingindex, showchara);
    }
    else if(m_LogicStatus == LOGIC_NEXTCHARA){
        CallBattleProgress();
        UpdateSideName();
        UpdateForceBrave();
        // rŊϐ풆~̂߁ALs̐ߖڂőxXVB
        UpdateBattleSpeed();
        // KEZNAB
        m_BattleMapWindow->GetBattleBottomPane().ClearMessage();
        // lގS̃bZ[Woꍇ͂܂̏ɑ҂B
        if(DoJudgeHeroLogic()){
            return;
        }
        // KEZXg`FbNB
        if(DoAutoSpecialLogic()){
            return;
        }
        // Lɂ퓬IB
        if(DoJudgeGameLogic()){
            return;
        }
        // ̃L̍sA퓬IA^[iŝꂩɂȂB
        bool battleend, progressturn;
        ActivateNextChara(battleend, progressturn);
        if(battleend){
            // 퓬I
            HideCharaDetail();
            HideBattleCharaList();
            m_WinMaster = 0;    // ̏
            std::string msg = (boost::format("%d^[o߂܂") % g_Ini.GetIntValue(TOpenFarenIni::BATTLE_TURNMAX)).str();
            TGcnMsgDlg::Show("Information", msg, TGcnMsgDlg::BUTTON_OK, g_Ini.GetMessageTimeoutFrame());
            m_LogicStatus = LOGIC_TIMEOUT_MSG;
        }
        else{
            if(progressturn){
                // ^[is
                m_TurnProgressFrame = -1;                       // \錻ۂȂɏB
                m_TurnProgressStage = TURNPROGRESS_BADSTATUS;
                m_LogicStatus = LOGIC_PROGRESSTURN;
            }
            else{
                // ̃L̍sB
                const TCharacterInstance& c = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
                // ړ\͈͎擾B
                GetMovable(m_ActiveMaster, c);
                m_KillList.clear();
                m_BattleMapWindow->SetScrollCenterOfChip(c.m_location.x, c.m_location.y);
                // COMB
                m_ActiveCharaIsPlayer = IsCommandableChara(m_ActiveMaster, m_ActiveChara);
                if(m_ActiveCharaIsPlayer){
                    // \Lłꍇ͓B
                    m_LuaBattleCom[m_ActiveMaster].CallFunction("check_wish_runaway");
                    if(m_LuaBattleCom[m_ActiveMaster].GetLastFunctionValue()){
                        m_ActiveCharaIsPlayer = false;
                        return;
                    }
                    // x\B
                    m_BattleMapWindow->GetBattleMapPane().SetBrightChip(c.m_location.x, c.m_location.y);
                    UpdateActionCommandWindow(false);
                    ShowBattleCommand();
                    m_BattleMapWindow->GetBattleBottomPane().SetOperatingChara(c, m_ActiveMaster != 0);
                    const TOpenFarenData* data = g_GameMaster.GetData();
                    const TBattleCharacter& charadata = data->GetCharacter(c.m_type);
                    m_BattleMapWindow->GetBattleBottomPane().SetMessage((boost::format("%sɖ߂") % charadata.m_name).str());
                    m_LogicStatus = LOGIC_PLAYERCOMMAND_BEFOREMOVE;
                }
                else{
                    // COM
                    // TODO:sϐ탂[h̎͂LuaR[΂Pt[k܂H
                    m_LogicStatus = LOGIC_COMTHINK;
                }
                if(m_NoWatch){
                    while(m_LogicStatus != LOGIC_NEXTCHARA && m_LogicStatus != LOGIC_END && m_LogicStatus != LOGIC_ERROR){
                        DoLogic();
                    }
                }
            }
        }
    }
    else if(m_LogicStatus == LOGIC_PROGRESSTURN){
        bool end = false;
        if(m_NoWatch){
            for(int i=0; i<256; ++i){
                end = ProgressTurn();
                if(end){
                    break;
                }
            }
            if(!end){
                OFERRLOG("256t[^[isɂ킯ȂBvÕoOH");
            }
        }
        else{
            end = ProgressTurn();
        }
        if(end){
            OFDBGLOG("^[isI");
            m_BattleMapWindow->GetBattleBottomPane().ClearMessage();
            m_LogicStatus = LOGIC_NEXTCHARA;
        }
    }
    else if(m_LogicStatus == LOGIC_ATTACKING || m_LogicStatus == LOGIC_CHARGING){
        if(DoAttackingLogic()){
            m_LogicStatus = LOGIC_NEXTCHARA;
        }
    }
    else if(m_LogicStatus == LOGIC_EFFECT){
        DoEffectLogic();
    }
    else if(m_LogicStatus == LOGIC_WAITBEFORENEXTCHARA){
        if(!m_WaitBeforeNextCharaFrame){
            m_LogicStatus = LOGIC_NEXTCHARA;
        }
        else{
            --m_WaitBeforeNextCharaFrame;
        }
    }
    else if(m_LogicStatus == LOGIC_COMTHINK){
        m_LuaBattleCom[m_ActiveMaster].CallFunction("on_chara_action");
    }
    else if(m_LogicStatus == LOGIC_SCRIPT){
        if(!m_LuaBattleEvent.Resume()){
            m_LogicStatus = LOGIC_ERROR;
            return;
        }
        if(!m_LuaBattleEvent.CanResume()){
            if(m_ScriptStatus == SCRIPT_BEFORE_NEXTCHARA){
                m_LogicStatus = LOGIC_NEXTCHARA;
            }
            else if(m_ScriptStatus == SCRIPT_AFTER_SPECIAL){
                m_WaitBeforeNextCharaFrame = 10;    // KB
                m_LogicStatus = LOGIC_NEXTCHARA;
            }
            else if(m_ScriptStatus == SCRIPT_BEFORE_SPECIAL){
                if(m_LuaBattleEvent.GetLastFunctionValue()){
                    DoSelectedSpecialLogic(m_SelectedSpecial, false);
                }
                else{
                    m_LogicStatus = LOGIC_NEXTCHARA;
                }
            }
        }
    }
    else if(m_LogicStatus == LOGIC_HERODEAD_MSG){
        if(!TGcnMsgDlg::IsVisible()){
            m_LogicStatus = LOGIC_NEXTCHARA;
        }
    }
    else if(m_LogicStatus == LOGIC_TIMEOUT_MSG){
        if(!TGcnMsgDlg::IsVisible()){
            TGcnMsgDlg::Show("Information", "UR͑ދp܂", TGcnMsgDlg::BUTTON_OK, g_Ini.GetMessageTimeoutFrame());
            m_LogicStatus = LOGIC_TIMEOUT_RETREAT;
        }
    }
    else if(m_LogicStatus == LOGIC_TIMEOUT_RETREAT || m_LogicStatus == LOGIC_BATTLEEND_MSG){
        // ^[ɒBƂŏI
        if(!TGcnMsgDlg::IsVisible()){
            m_Map->ClearTempData();
            g_GuiContainerList.GetGui()->removeGlobalKeyListener(this);
            m_LogicStatus = LOGIC_END;
            m_BattleMapWindow->setVisible(false);
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::mousePressed(gcn::MouseEvent &mouseEvent)
{
    if(mouseEvent.getSource() == &m_BattleMapWindow->GetBattleMapPane()){
        int x, y;
        XY_PhyToMap(*m_Map, mouseEvent.getX(), mouseEvent.getY(), &x, &y);
        if(m_LogicStatus == LOGIC_PLAYERPLACE){
            if(mouseEvent.getButton() == gcn::MouseEvent::LEFT){
                if(m_ShadeLayer.GetValue(x, y)){
                    return;
                }
                // ɑ̔zuς݃Lꍇ͔zuLZB
                int masterindex = m_MasterLayer.GetValue(x, y);
                if(masterindex == m_PlaceMasterIndex){
                    int charaindex = m_CharaLayer.GetValue(x, y);
                    m_PlacedFlag[charaindex - m_PlayerPlacedCount] = 0;
                    m_MasterLayer.GetValue(x, y) = -1;
                    TCharacterInstance& c = (*m_CharaList[m_PlaceMasterIndex])[charaindex];
                    c.m_location.x = -1;
                    //m_BattleMapWindow->GetBattleMapPane().RemoveShowingChara(c.m_showingindex);
                }
                int i = m_CharaPlaceWindow->GetCharaListPane().GetSelected();
                if(i != -1 && !m_PlacedFlag[i]){
                    m_PlacedFlag[i] = 1;
                    TCharacterInstance& c = (*m_CharaList[m_PlaceMasterIndex])[m_PlayerPlacedCount + i];
                    c.m_location.x = x;
                    c.m_location.y = y;
                    m_MasterLayer.GetValue(x, y) = m_PlaceMasterIndex;
                    m_CharaLayer.GetValue(x, y) = m_PlayerPlacedCount + i;
                    // L\ǉ
                    TShowingChara showchara;
                    std::memset(&showchara, 0, sizeof showchara);
                    showchara.m_type = c.m_type;
                    showchara.m_location1 = c.m_location;
                    // zuL
                    std::vector<int>::iterator itr = std::find(m_PlacedFlag.begin(), m_PlacedFlag.end(), 0);
                    if(itr == m_PlacedFlag.end()){
                        // 1GÃLzu
                        m_BattleMapWindow->GetBattleMapPane().ClearChipFrame();
                        m_PlayerPlacedCount += m_PlacedFlag.size();
                        if(m_PlayerPlacedCount == m_CharaList[m_PlaceMasterIndex]->size()){
                            // SLzu玟̃}X^[̔zuցB
                            m_CharaPlaceWindow->setVisible(false);
                            ++m_PlaceMasterIndex;
                            if(m_PlaceMasterIndex >= m_CharaList.size()){
                                // S}X^[̔zuB
                                //m_ShadeLayer.Init();  // ł̂ȂH폜B2010/03/17
                                m_LogicStatus = LOGIC_PLACECOMPLETE;
                            }
                            else{
                                // ̃}X^[̔zuցB
                                m_LogicStatus = LOGIC_PLACE;
                            }
                        }
                        else{
                            // ܂̃GÃLzuB
                            PlayerNextPlace();
                        }
                    }
                    else{
                        // zuLcĂꍇB
                        // ̖zuLIԂɂB
                        m_CharaPlaceWindow->GetCharaListPane().SetSelected(std::distance(m_PlacedFlag.begin(), itr));
                    }
                }
                UpdateShowingChara();
                mouseEvent.consume();
            }
            else{
                // ENbNŔzuLZ
                for(unsigned u=0; u<m_PlaceCharaList.size(); ++u){
                    TCharacterInstance& c = (*m_CharaList[m_PlaceMasterIndex])[m_PlayerPlacedCount + u];
                    if(c.m_location.x == x && c.m_location.y == y && m_PlacedFlag[u]){
                        m_PlacedFlag[u] = 0;
                        c.m_location.x = -1;
                        m_MasterLayer.GetValue(x, y) = -1;
                        UpdateShowingChara();
                        mouseEvent.consume();
                        return;
                    }
                }
            }
        }
        else if(m_LogicStatus == LOGIC_PLAYERMOVE){
            if(mouseEvent.getButton() == gcn::MouseEvent::LEFT){
                if(!m_ShadeLayer.GetValue(x, y)){
                    m_ShadeLayer.Init();
                    const TCharacterInstance& c = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
                    if(c.m_location != point_t(x, y)){
                        DoSelectedMoveLogic(x, y);
                        mouseEvent.consume();
                    }
                }
            }
            else{
                // ړLZ
                m_ShadeLayer.Init();
                ShowBattleCommand();
                m_LogicStatus = LOGIC_PLAYERCOMMAND_BEFOREMOVE;
                mouseEvent.consume();
            }
        }
        else if(m_LogicStatus == LOGIC_PLAYERCOMMAND_AFTERMOVE){
            if(mouseEvent.getButton() != gcn::MouseEvent::LEFT){
                // ړLZ
                TCharacterInstance& c = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
                m_MasterLayer.GetValue(c.m_location.x, c.m_location.y) = -1;
                c.m_location = m_MoveFrom;
                UpdateShowingChara(m_ActiveMaster, m_ActiveChara);
                m_BattleMapWindow->GetBattleMapPane().SetBrightChip(c.m_location.x, c.m_location.y);
                // LCfbNX͈ړÔ܂܂Ȃ̂œɍXVȂB
                m_MasterLayer.GetValue(c.m_location.x, c.m_location.y) = m_ActiveMaster;
                m_Moved = false;
                UpdateActionCommandWindow(false);
                ShowBattleCommand();
                const TOpenFarenData* data = g_GameMaster.GetData();
                const TBattleCharacter& charadata = data->GetCharacter(c.m_type);
                m_BattleMapWindow->GetBattleBottomPane().SetMessage((boost::format("%sɖ߂") % charadata.m_name).str());
                m_LogicStatus = LOGIC_PLAYERCOMMAND_BEFOREMOVE;
                mouseEvent.consume();
            }
        }
        else if(m_LogicStatus == LOGIC_PLAYERATTACK || m_LogicStatus == LOGIC_PLAYERCHARGE){
            if(mouseEvent.getButton() == gcn::MouseEvent::LEFT){
                if(!m_ShadeLayer.GetValue(x, y)){
                    if(DoSelectedAttackLogic(x, y, m_LogicStatus == LOGIC_PLAYERCHARGE)){
                        m_BattleMapWindow->GetBattleMapPane().ClearBrightChip();
                        m_ShadeLayer.Init();
                        m_BattleMapWindow->GetBattleBottomPane().ClearOperatingChara();
                        mouseEvent.consume();
                    }
                }
            }
            else{
                // ULZ
                m_ShadeLayer.Init();
                ShowBattleCommand();
                if(m_Moved){
                    m_LogicStatus = LOGIC_PLAYERCOMMAND_AFTERMOVE;
                }
                else{
                    m_LogicStatus = LOGIC_PLAYERCOMMAND_BEFOREMOVE;
                }
                mouseEvent.consume();
            }
        }
        else if(m_LogicStatus == LOGIC_PLAYERSPECIAL){
            if(mouseEvent.getButton() == gcn::MouseEvent::LEFT){
                //point_t p(x, y);
                //m_EffectTo.push_back(p);
                const TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
                const TOpenFarenData* data = g_GameMaster.GetData();
                const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
                if(!m_ShadeLayer.GetValue(x, y) && CanEffect(chara.m_location, point_t(x, y), m_ActiveMaster, m_SelectedSpecial)){
                    // KEZs
                    m_ShadeLayer.Init();
                    m_BattleMapWindow->GetBattleMapPane().ClearBrightChip();
                    m_BattleMapWindow->GetBattleMapPane().ClearRadiusChip();
                    m_BattleMapWindow->GetBattleBottomPane().ClearOperatingChara();
                    // gp񐔐@tāAΉ񐔂𑝂₷B3ӏi[U2ӏACOM1ӏjB
                    std::map<int, int>::const_iterator itr = m_SpecialMagicMap.find(m_SelectedSpecial);
                    if(itr != m_SpecialMagicMap.end()){
                        ++m_MagicUseCount[m_ActiveMaster][itr->second];
                    }
                    const TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
                    GenerateEffectPointList(chara.m_location, point_t(x, y), m_SelectedSpecial);
                    //m_EffectTarget = point_t(x, y);
                    //m_EffectCount = 0;
                    //m_EffectFrame = 0;
                    //m_LogicStatus = LOGIC_EFFECT;
                    TransitEffectLogic(x, y);
                    mouseEvent.consume();
                }
            }
            else{
                // KEZLZ
                m_ShadeLayer.Init();
                m_BattleMapWindow->GetBattleMapPane().ClearRadiusChip();
                m_BattleMapWindow->GetBattleBottomPane().ClearMessage();
                ShowBattleCommand();
                if(m_Moved){
                    m_LogicStatus = LOGIC_PLAYERCOMMAND_AFTERMOVE;
                }
                else{
                    m_LogicStatus = LOGIC_PLAYERCOMMAND_BEFOREMOVE;
                }
                mouseEvent.consume();
            }
        }
        if(!mouseEvent.isConsumed()){
            int masterindex = m_MasterLayer.GetValue(x, y);
            if(masterindex != -1){
                int charaindex = m_CharaLayer.GetValue(x, y);
                //m_CharaStatusFullWindow->Show((*m_CharaList[masterindex])[charaindex]);
                //g_GuiContainerList.MoveWidgetToCenter(m_CharaStatusFullWindow);
                //g_GuiContainerList.GetGui()->getTop()->moveToTop(m_CharaStatusFullWindow);
                ShowCharaDetail(masterindex, charaindex);
                mouseEvent.consume();
            }
        }
    }
    else if(mouseEvent.getSource() == &m_CharaPlaceWindow->GetCharaListPane()){
        int i = m_CharaPlaceWindow->GetCharaListPane().GetCharaIndexAt(mouseEvent.getX(), mouseEvent.getY());
        if(i != -1 && !m_PlacedFlag[i]){
            m_CharaPlaceWindow->GetCharaListPane().SetSelected(i);
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::mouseMoved(gcn::MouseEvent &mouseEvent)
{
    if(mouseEvent.getSource() == &m_BattleMapWindow->GetBattleMapPane()){
        int x, y;
        XY_PhyToMap(*m_Map, mouseEvent.getX(), mouseEvent.getY(), &x, &y);
        if(!IsValidXY(x, y)){
            m_BattleMapWindow->GetBattleBottomPane().ClearPointedChip();
            return;
        }
        // E̒n`\XVB
        int movetype = -1;
        if(m_LogicStatus >= LOGIC_PLAYERCOMMAND_BEFOREMOVE &&
           m_LogicStatus <= LOGIC_PLAYERMAGIC){
            const TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
            movetype = chara.m_movetype;
        }
        m_BattleMapWindow->GetBattleBottomPane().SetPointedChip(x, y, movetype);
        // KEŽʔ͈͂\B
        if(m_LogicStatus == LOGIC_PLAYERSPECIAL){
            const TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
            const TOpenFarenData* data = g_GameMaster.GetData();
            const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
            if(!m_ShadeLayer.GetValue(x, y) && CanEffect(chara.m_location, point_t(x, y), m_ActiveMaster, m_SelectedSpecial)){
                const TSpecial& spe = data->GetSpecial(m_SelectedSpecial);
                const TAttribute& attr = data->GetAttribute(spe.m_attribute);
                gcn::Color c;
                if(attr.m_type == ATTRTYPE_BOOST){
                    c = gcn::Color(0, 0, 255, 128);
                }
                else{
                    c = gcn::Color(255, 0, 0, 128);
                }
                if(spe.m_rangetype == RANGE_STREAM){
                    int dir = GetStreamDirection(chara.m_location.x, chara.m_location.y, x, y);
                    SetStreamChip(chara.m_location.x, chara.m_location.y, spe.m_maxradius, dir, c);
                }
                else{
                    SetRadiusChip(x, y, spe.m_maxradius, c);
                }
            }
            else{
                m_BattleMapWindow->GetBattleMapPane().ClearRadiusChip();
            }
        }
        int masterindex = m_MasterLayer.GetValue(x, y);
        if(masterindex != -1){
            int charaindex = m_CharaLayer.GetValue(x, y);
            const TCharacterInstance& chara = (*m_CharaList[masterindex])[charaindex];
            m_CharaStatusWindow->Show(chara);
            g_GuiContainerList.GetGui()->getTop()->moveToTop(m_CharaStatusWindow);
        }
        else{
            m_CharaStatusWindow->setVisible(false);
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::keyPressed(gcn::KeyEvent &keyEvent)
{
    if(keyEvent.getKey() == gcn::Key::F1){
        ShowBattleCharaList(true);
    }
    else if(keyEvent.getKey() == gcn::Key::F2){
        // BR}hF݂̃L̑COMɂ܂
        if(m_LogicStatus == LOGIC_PLAYERCOMMAND_BEFOREMOVE){
            HideBattleCommand();
            m_ActiveCharaIsPlayer = false;
            m_LogicStatus = LOGIC_COMTHINK;
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::action(const gcn::ActionEvent& actionEvent)
{
    gcn::Widget *src = actionEvent.getSource();
    m_BattleMapWindow->GetBattleBottomPane().ClearMessage();
    if(src == &m_BattleCommandWindow->GetWaitButton()){
        // ҋ@
        if(!m_BattleCommandWindow->GetWaitButton().isEnabled()){
            return;
        }
        //m_BattleCommandWindow->setVisible(false);
        HideBattleCommand();
        m_BattleMapWindow->GetBattleBottomPane().ClearOperatingChara();
        DoSelectedWaitLogic();
    }
    else if(src == &m_BattleCommandWindow->GetMoveButton()){
        // ړ
        if(!m_BattleCommandWindow->GetMoveButton().isEnabled()){
            return;
        }
        //m_BattleCommandWindow->setVisible(false);
        HideBattleCommand();
        // ړ\CeCɔ]Rs[
        std::transform(m_MoveLayer.begin(), m_MoveLayer.end(), m_ShadeLayer.begin(), std::logical_not<int>());
        m_LogicStatus = LOGIC_PLAYERMOVE;
    }
    else if(src == &m_BattleCommandWindow->GetAttackButton()){
        // U
        if(!m_BattleCommandWindow->GetAttackButton().isEnabled()){
            return;
        }
        //m_BattleCommandWindow->setVisible(false);
        HideBattleCommand();
        TCharacterInstance& c = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
        HighlightCircle(c.m_location.x, c.m_location.y);
        m_LogicStatus = LOGIC_PLAYERATTACK;
    }
    else if(src == &m_BattleCommandWindow->GetChargeButton()){
        // ˌ
        if(!m_BattleCommandWindow->GetChargeButton().isEnabled()){
            return;
        }
        //m_BattleCommandWindow->setVisible(false);
        HideBattleCommand();
        const TCharacterInstance& c = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
        HighlightCircle(c.m_location.x, c.m_location.y);
        m_LogicStatus = LOGIC_PLAYERCHARGE;
    }
    else if(src == &m_BattleCommandWindow->GetSpecialButton()){
        // KEZ
        if(!m_BattleCommandWindow->GetSpecialButton().isEnabled()){
            return;
        }
        //m_BattleCommandWindow->setVisible(false);
        HideBattleCommand();
        const TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
        const TOpenFarenData* data = g_GameMaster.GetData();
        const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
        DoSelectedSpecialLogic(charadata.m_specialindex, true);
    }
    else if(src == &m_BattleCommandWindow->GetMagicButton()){
        // @
        if(!m_BattleCommandWindow->GetMagicButton().isEnabled()){
            return;
        }
        //m_BattleCommandWindow->setVisible(false);
        HideBattleCommand();
        const TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
        m_MagicWindow->Show(chara, &m_MagicUseCount[m_ActiveMaster]);
        m_LogicStatus = LOGIC_PLAYERMAGIC;
    }
    else if(src == &m_BattleCommandWindow->GetRedoButton()){
        // Ďs
        std::map<int, int>::const_iterator citr_lastspecial = m_LastSpecialTable[m_ActiveMaster].find(m_ActiveChara);
        if(citr_lastspecial != m_LastSpecialTable[m_ActiveMaster].end()){
            HideBattleCommand();
            DoSelectedSpecialLogic(citr_lastspecial->second, true);
        }
        else{
            OFERRLOG("Ďs{^ꂽ̂ɉ̂ĎssȂ");
        }
    }
    else if(src == &m_BattleCommandWindow->GetRedo2Button()){
        // Ďs wcʔ
        if(m_LastSpecial[m_ActiveMaster] != -1){
            // l̖@ZgłȂ悤ɃK[hcc͂ł͂ȂB
            // 炩̃oOłłĂ܂ĂZɂȂ邾ŗ͂Ȃ낤B
            HideBattleCommand();
            DoSelectedSpecialLogic(m_LastSpecial[m_ActiveMaster], true);
        }
        else{
            OFERRLOG("Ďs{^ꂽ̂ɉ̂wcĎssݒ肳ĂȂ");
        }
    }
    else if(src == &m_BattleCommandWindow->GetRunAwayButton()){
        // 
        if(!m_BattleCommandWindow->GetRunAwayButton().isEnabled()){
            return;
        }
        //m_BattleCommandWindow->setVisible(false);
        HideBattleCommand();
        m_BattleMapWindow->GetBattleMapPane().ClearBrightChip();
        m_BattleMapWindow->GetBattleBottomPane().ClearOperatingChara();
        DoSelectedRunAwayLogic();
    }
    else if(m_MagicWindow->IsMyAction(src)){
        m_MagicWindow->setVisible(false);
        int speindex = boost::lexical_cast<int>(actionEvent.getId());
        if(speindex == -1){
            ShowBattleCommand();
            m_LogicStatus = LOGIC_PLAYERCOMMAND_BEFOREMOVE;
            return;
        }
        DoSelectedSpecialLogic(speindex, true);
    }
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::CanNoWatch() const
{
    bool playerExists = false;
    for(int i=0; i<m_MasterIndex.size(); ++i){
        playerExists |= IsPlayerMaster(i);
    }
    return !playerExists;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::SetNoWatch()
{
    m_BattleMapWindow->setVisible(false);
    ShowBattleCharaList(false);
    m_NoWatch = true;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::ApplyConfig()
{
    // R}h͑҂̎Ȃ瑦Aj[VxXVB
    if(m_LogicStatus == LOGIC_PLAYERCOMMAND_BEFOREMOVE ||
       m_LogicStatus == LOGIC_PLAYERCOMMAND_AFTERMOVE){
        UpdateBattleSpeed();
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetMapSize(int& w, int& h) const
{
    w = m_Map->GetWidth();
    h = m_Map->GetHeight();
}
//---------------------------------------------------------------------------
int TBattleMapLogic::GetMapHeight(int x, int y) const
{
    return m_Map->GetMapHeight(x, y);
}
//---------------------------------------------------------------------------
const TBattleMap* TBattleMapLogic::GetBattleMap() const
{
    return m_Map;
}
//---------------------------------------------------------------------------
// ̈ړ^Cṽ݂}bvŏRXg̍v𓾂BقǗLȈړ^CvƍlB
int TBattleMapLogic::GetMoveTypeCost(int movetype) const
{
    const TOpenFarenData *data = g_GameMaster.GetData();
    if(movetype < 0 || movetype >= data->GetMoveTypeCount()){
        OFERRLOG((boost::format("movetype %d out of range") % movetype).str());
        return -1;
    }
    return m_MoveTypeCost[movetype];
}
//---------------------------------------------------------------------------
// ̈ړ^Cṽ݂}bvœn`ʂ̍v𓾂B傫قǗLȈړ^CvƍlB
int TBattleMapLogic::GetMoveTypeSkill(int movetype) const
{
    const TOpenFarenData *data = g_GameMaster.GetData();
    if(movetype < 0 || movetype >= data->GetMoveTypeCount()){
        OFERRLOG((boost::format("movetype %d out of range") % movetype).str());
        return -1;
    }
    return m_MoveTypeSkill[movetype];
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetMovableLayer(TValueLayer<int>& layer) const
{
    layer = m_MoveLayer;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetMovableLayerFrom(int master, const TCharacterInstance& chara, int x, int y, int val, TValueLayer<int>& movelayer, TValueLayer<point_t>& movefromlayer, bool loose) const
{
    GetMovableFrom(master, chara, x, y, val, movelayer, movefromlayer, loose);
}
//---------------------------------------------------------------------------
int TBattleMapLogic::FindMasterIndex(int mastertype) const
{
    const TOpenFarenGame *game = g_GameMaster.GetGame();
    for(unsigned u=0; u<m_MasterIndex.size(); ++u){
        if(m_MasterIndex[u] != -1 && game->GetMasterInstance(m_MasterIndex[u]).m_type == mastertype){
            return u;
        }
    }
    return -1;
}
//---------------------------------------------------------------------------
// AĨ}X^[ԍ擾̂ɎgpB
int TBattleMapLogic::GetMyMaster() const
{
    return m_ActiveMaster;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetActiveChara(int& master, int& charaindex) const
{
    master = m_ActiveMaster;
    charaindex = m_ActiveChara;
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::GetCharaOn(int x, int y, int& master, int& charaindex) const
{
    if(m_MasterLayer.GetValue(x, y) == -1){
        return false;
    }
    master = m_MasterLayer.GetValue(x, y);
    charaindex = m_CharaLayer.GetValue(x, y);
    return true;
}
//---------------------------------------------------------------------------
// Q̃L擾BSĂĂ擾B
bool TBattleMapLogic::GetChara(int master, int charatype, int& charaindex) const
{
    for(unsigned u=0; u<m_CharaList[master]->size(); ++u){
        if((*m_CharaList[master])[u].m_type == charatype){
            charaindex = u;
            return true;
        }
    }
    return false;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetCharaStatus(int master, int charaindex, TCharacterInstance& chara) const
{
    chara = (*m_CharaList[master])[charaindex];
}
//---------------------------------------------------------------------------
TCharacterInstance TBattleMapLogic::CreateCharaInstance(int charatype, int rank) const
{
    return g_GameMaster.GetGame()->CreateCharaInstance(charatype, rank);
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetActionList(int movex, int movey, TActionList& actlist, std::vector<point_t>& targetlist) const
{
    const TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
    // z
    bool dream = HasSystemEffect(chara, SYSTEM_DREAM);
    // U
    std::vector<point_t> atklist;
    if(dream){
        // z͖̎ɍU
        GetMySideNextList(m_ActiveMaster, m_ActiveChara, movex, movey, atklist);
    }
    else{
        GetEnemyNextList(m_ActiveMaster, movex, movey, atklist);
    }
    //OFDBGLOG((boost::format("move=(%d, %d) atklist=%d") % movex % movey % atklist.size()).str());
    for(unsigned u=0; u<atklist.size(); ++u){
        TAction act;
        act.acttype = ACTION_ATTACK;
        act.location = atklist[u];
        act.special = 0;
        actlist.push_back(act);
    }
    if(dream){
        return;
    }
    //OFDBGLOG((boost::format("Attack actlist=%d") % actlist.size()).str());
    // @ƕKEZoB
    std::vector<int> speciallist;
    if(chara.m_location == point_t(movex, movey)){
        // ړȂꍇ
        // ˌ
        for(unsigned u=0; u<atklist.size(); ++u){
            if(!m_Map->GetMapObjType(atklist[u].x, atklist[u].y)){
                TAction act;
                act.acttype = ACTION_CHARGE;
                act.location = atklist[u];
                act.special = 0;
                actlist.push_back(act);
            }
        }
        // @
        int mgcount = data->GetMagicGroupCount();
        for(int mg = 0; mg < mgcount; ++mg){
            const TMagicGroup& group = data->GetMagicGroup(mg);
            for(unsigned u=0; u<group.m_magiclist.size(); ++u){
                int mdindex = group.m_magiclist[u];
                const TMagicData& magicdata = data->GetMagicData(mdindex);
                // gp񐔐
                if(magicdata.m_limit && m_MagicUseCount[m_ActiveMaster][mdindex] >= magicdata.m_limit){
                    continue;
                }
                // gp\N`FbN
                for(unsigned rank=0; rank<magicdata.m_magics.size(); ++rank){
                    const TMagic& magic = magicdata.m_magics[rank];
                    if(magic.m_needrank <= charadata.m_magicrank[mg]){
                        speciallist.push_back(magic.m_special);
                    }
                }
            }
        }
    }
    // KEZ
    if(charadata.m_specialindex != -1){
        const TSpecial& spe = data->GetSpecial(charadata.m_specialindex);
        if(spe.m_consumemp <= chara.m_mp){
            speciallist.push_back(charadata.m_specialindex);
        }
    }
    // @ƕKEZ̎gp\B
    int w = m_Map->GetWidth();
    int h = m_Map->GetHeight();
    TValueLayer<int> layer(m_Map->GetWidth(), m_Map->GetHeight());
    for(unsigned u=0; u<speciallist.size(); ++u){
        const TSpecial& spe = data->GetSpecial(speciallist[u]);
        //OFDBGLOG((boost::format("check spe=%s") % spe.m_id).str());
        if(spe.m_consumemp <= chara.m_mp){
            if(spe.m_rangetype == RANGE_NORMAL || spe.m_rangetype == RANGE_STREAM){
                GetRange(layer, movex, movey, spe.m_rangetype, spe.m_maxrange, spe.m_maxradius, 0);
                for(int y=0; y<h; ++y){
                    for(int x=0; x<w; ++x){
                        if(layer.GetValue(x, y)){
                            std::vector<point_t> allplist, plist;
                            GetEffectPointList(point_t(movex, movey), point_t(x, y), spe.m_maxradius, spe.m_rangetype, spe.m_rangeheight, allplist);
                            std::deque<bool> evaltiming(allplist.size(), true);
                            GetEffectTargetList(allplist, evaltiming, m_ActiveMaster, speciallist[u], plist);
                            if(!plist.empty()){
//if(spe.m_id == "|")
//g_Logger.DebugLog((boost::format("GetActionList move=(%d, %d) spe=%s xy=(%d, %d) plist=%d") % movex % movey % spe.m_name % x % y % plist.size()).str());
//    if(spe.m_name=="EH[^K[hP" && plist.size() >= 4){
//        g_Logger.DebugLog((boost::format("GetActionList move=(%d, %d) spe=%s xy=(%d, %d) plist=%d") % movex % movey % spe.m_name % x % y % plist.size()).str());
//    }

                                TAction act;
                                act.acttype = ACTION_SPECIAL;
                                act.location = point_t(x, y);
                                act.special = speciallist[u];
                                actlist.push_back(act);
                            }
                        }
                    }
                }
            }
            else{
                std::vector<point_t> plist;
                GetEffectPointList(chara.m_location, chara.m_location, spe.m_maxradius, spe.m_rangetype, spe.m_rangeheight, plist);
                if(!plist.empty()){
                    TAction act;
                    act.acttype = ACTION_SPECIAL;
                    act.location = chara.m_location;
                    act.special = speciallist[u];
                    actlist.push_back(act);
                }
            }
        }
    }
    //OFDBGLOG((boost::format("actlist=%d") % actlist.size()).str());
    if(CanRunAway(m_ActiveMaster, m_ActiveChara, movex, movey)){
        TAction act;
        act.acttype = ACTION_RUNAWAY;
        actlist.push_back(act);
    }
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::GetLastTarget(int& x, int& y) const
{
    if(m_EffectTarget.x == -1){
        return false;
    }
    x = m_EffectTarget.x;
    y = m_EffectTarget.y;
    return true;
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::EvalAction(int movex, int movey, TActionType acttype, int special, int targetx, int targety, std::vector<TEvalResult>& evallist) const
{
    // 2010/01/01 ԔɂAPIgps͈ӖȂ̂ō폜B
    //if(m_LogicStatus != LOGIC_COMTHINK){
    //    g_Logger.ErrorLog((boost::format("TBattleMapLogic::EvalAction error:not com think status")).str());
    //    return false;
    //}
    const TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
    if(chara.m_location != point_t(movex, movey)){
        if(!m_MoveLayer.GetValue(movex, movey)){
            g_Logger.ErrorLog((boost::format("TBattleMapLogic::EvalAction error:ړłȂW(%d,%d)w肳܂B") % movex % movey).str());
            return false;
        }
        if(acttype == ACTION_CHARGE || (acttype == ACTION_SPECIAL && special != charadata.m_specialindex)){
            g_Logger.ErrorLog((boost::format("TBattleMapLogic::EvalAction error:ړ͕s\ȑiˌ܂͖@jw肳܂B")).str());
            return false;
        }
    }
    if(acttype == ACTION_ATTACK || acttype == ACTION_CHARGE){
        if(acttype == ACTION_CHARGE && m_Map->GetMapObjType(targetx, targety)){
            g_Logger.ErrorLog((boost::format("TBattleMapLogic::EvalAction error:ˌłȂW(%d,%d)w肳܂B") % targetx % targety).str());
            return false;
        }
        int defmaster = m_MasterLayer.GetValue(targetx, targety);
        if(defmaster != -1){
            int defcharaindex = m_CharaLayer.GetValue(targetx, targety);
            EvalAttackList(m_ActiveMaster, m_ActiveChara, defmaster, defcharaindex, evallist, acttype == ACTION_CHARGE);
        }
        else{
            TEvalResult eval;
            eval.location = point_t(targetx, targety);
            eval.systemeffect = SYSTEM_CRASHWALL;
            evallist.push_back(eval);
        }
    }
    else if(acttype == ACTION_SPECIAL){
        const TSpecial& spe = data->GetSpecial(special);
        std::vector<point_t> eplist;
        std::vector<point_t> targetlist;
        if(spe.m_rangetype == RANGE_NORMAL || spe.m_rangetype == RANGE_STREAM){
            GetEffectPointList(point_t(movex, movey), point_t(targetx, targety), spe.m_maxradius, spe.m_rangetype, spe.m_rangeheight, eplist);
//g_Logger.DebugLog((boost::format("TBattleMapLogic::EvalAction move=(%d,%d) tar=(%d,%d) eplist=%d") % movex % movey % targetx % targety % eplist.size()).str());
        }
        else{
            GetEffectPointList(point_t(movex, movey), point_t(movex, movey), spe.m_maxradius, spe.m_rangetype, spe.m_rangeheight, eplist);
        }
        std::deque<bool> evaltiming(eplist.size(), true);
        GetEffectTargetList(eplist, evaltiming, m_ActiveMaster, special, targetlist);
        std::vector<TEffectResult> resultlist;
        EvalSpecial(m_ActiveMaster, chara, targetlist, special, resultlist, 0, false);
        for(unsigned u=0; u<resultlist.size(); ++u){
            TEvalResult eval;
            eval.location = resultlist[u].m_location;
            eval.masterindex = m_MasterLayer.GetValue(eval.location.x, eval.location.y);
            eval.charaindex = m_CharaLayer.GetValue(eval.location.x, eval.location.y);
            eval.value = resultlist[u].m_value;
            eval.revivecharatype = resultlist[u].m_revivecharatype;
            eval.summoncharatype = resultlist[u].m_summoncharatype;
            eval.systemeffect = resultlist[u].m_systemeffect;
            if(resultlist[u].m_systemeffect == SYSTEM_CHANGE_RESISTANCE){
                const TSpecial& spe = data->GetSpecial(resultlist[u].m_special);
                eval.resattr = spe.m_resistanceattr;
                eval.restype = spe.m_resistancetype;
            }
            else if(resultlist[u].m_systemeffect == SYSTEM_CHANGE_MOVETYPE){
                const TSpecial& spe = data->GetSpecial(resultlist[u].m_special);
                eval.movetype = spe.m_movetype;
            }
            evallist.push_back(eval);
        }
    }
    return true;
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::SetAction(int movex, int movey, TActionType acttype, int special, int targetx, int targety)
{
//g_Logger.DebugLog((boost::format("TBattleMapLogic::SetAction")).str());
    // 2010/01/01 ԔɂAPIgps͈ӖȂ̂ō폜B
    //if(m_LogicStatus != LOGIC_COMTHINK){
    //    OFERRLOG((boost::format("not com think status")).str());
    //    return false;
    //}
    m_ComActionType = acttype;
    m_ComTarget.x = targetx;
    m_ComTarget.y = targety;
    const TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
    if(chara.m_location == point_t(movex, movey)){
        if(acttype == ACTION_WAIT){
            DoSelectedWaitLogic();
        }
        else if(acttype == ACTION_ATTACK || acttype == ACTION_CHARGE){
            if(acttype == ACTION_CHARGE && m_Map->GetMapObjType(targetx, targety)){
                OFERRLOG((boost::format("ˌłȂW(%d,%d)w肳܂B") % targetx % targety).str());
                return false;
            }
            DoSelectedAttackLogic(targetx, targety, acttype == ACTION_CHARGE);
        }
        else if(acttype == ACTION_SPECIAL){
            // gp񐔐@tāAΉ񐔂𑝂₷B3ӏi[U2ӏACOM1ӏjB
            std::map<int, int>::const_iterator itr = m_SpecialMagicMap.find(special);
            if(itr != m_SpecialMagicMap.end()){
                ++m_MagicUseCount[m_ActiveMaster][itr->second];
            }
            DoSelectedSpecialLogic(special, true);
        }
        else if(acttype == ACTION_RUNAWAY){
            DoSelectedRunAwayLogic();
        }
    }
    else{
        if(!m_MoveLayer.GetValue(movex, movey)){
            g_Logger.ErrorLog((boost::format("TBattleMapLogic::SetAction error:ړłȂW(%d,%d)w肳܂B") % movex % movey).str());
            return false;
        }
        if(acttype == ACTION_CHARGE || (acttype == ACTION_SPECIAL && special != charadata.m_specialindex)){
            g_Logger.ErrorLog((boost::format("TBattleMapLogic::SetAction error:ړ͕s\ȑiˌ܂͖@jw肳܂B")).str());
            return false;
        }
//g_Logger.DebugLog((boost::format("TBattleMapLogic::SetAction acttype=%d") % acttype).str());
        m_ComSpecial = special;
        DoSelectedMoveLogic(movex, movey);
    }
    return true;
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::GetAutoRunAwayPoint(point_t& loc) const
{
    const TOpenFarenData* data = g_GameMaster.GetData();

    int w = m_Map->GetWidth();
    int h = m_Map->GetHeight();
    TValueLayer<int> layer(w, h);
    for(int master=0; master<m_CharaList.size(); ++master){
        if(master != m_ActiveMaster){
            for(unsigned u=0; u<m_CharaList[master]->size(); ++u){
                const TCharacterInstance& chara = (*m_CharaList[master])[u];
                const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
                int movable = charadata.m_movable * 2;
                if(movable > TNeighbor::NEIGHBOR_MAX){
                    movable = TNeighbor::NEIGHBOR_MAX;
                }
                int from = TNeighbor::GetNeighborIndex(0);
                int to = TNeighbor::GetNeighborIndex(movable + 1);
                int mode = chara.m_location.y%2;
                for(int i=from; i<to; ++i){
                    point_t p = TNeighbor::GetNeighbor(mode, i);
                    int nextx = chara.m_location.x + p.x;
                    int nexty = chara.m_location.y + p.y;
                    if(IsValidXY(nextx, nexty)){
                        ++layer.GetValue(nextx, nexty);
                    }
                }
            }
        }
    }

    bool ok = false;
    loc = (*m_CharaList[m_ActiveMaster])[m_ActiveChara].m_location;
    int minp = INT_MAX;
    for(int y=0; y<h; ++y){
        for(int x=0; x<w; ++x){
            if(m_MoveLayer.GetValue(x, y)){
                if(m_MoveLayer.GetValue(x, y) && !InEnemyZoc(m_ActiveMaster, x, y) && minp > layer.GetValue(x, y)){
                    minp = layer.GetValue(x, y);
                    loc.x = x;
                    loc.y = y;
                    ok = true;
                }
            }
        }
    }
    return ok;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::SetCharaBrave(int master, int charaindex, int brave)
{
    if(master < 0 || master >= m_CharaList.size() ||
       charaindex < 0 || charaindex >= (*m_CharaList[master]).size()){
        OFERRLOG((boost::format("ȃp[^ master=%d charaindex=%d") % master % charaindex).str());
        return;
    }
    if(brave < 0){
        brave = 0;
    }
    else if(brave > BRAVE_MAX){
        brave = BRAVE_MAX;
    }
    (*m_CharaList[master])[charaindex].m_brave = brave;
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::ChangeCharaType(int master, int charaindex, int charatype)
{
    int rank = (*m_CharaList[master])[charaindex].m_rank;
    int exp = (*m_CharaList[master])[charaindex].m_experience;
    int hp = (*m_CharaList[master])[charaindex].m_hp;
    int mp = (*m_CharaList[master])[charaindex].m_mp;
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    (*m_CharaList[master])[charaindex] = game->CreateCharaInstance(charatype, rank);
    (*m_CharaList[master])[charaindex].m_experience = exp;
    (*m_CharaList[master])[charaindex].m_hp = hp;
    (*m_CharaList[master])[charaindex].m_mp = mp;
    return true;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::SetMapChip(int x, int y, int chip)
{
    int oldtype = m_Map->GetMapType(x, y);
    m_Map->SetMapChip(g_GameMaster.GetData(), x, y, chip);
    UpdateMoveTypeCostSkill(x, y, oldtype);
}
//---------------------------------------------------------------------------
void TBattleMapLogic::RemoveMapObj(int x, int y)
{
    int oldtype = m_Map->GetMapType(x, y);
    m_Map->RemoveMapObj(g_GameMaster.GetData(), x, y);
    UpdateMoveTypeCostSkill(x, y, oldtype);
}
//---------------------------------------------------------------------------
void TBattleMapLogic::RemoveMapObjSpecify(int x, int y, TMapObject objtype)
{
    int oldtype = m_Map->GetMapType(x, y);
    m_Map->RemoveMapObjSpecify(g_GameMaster.GetData(), x, y, objtype);
    UpdateMoveTypeCostSkill(x, y, oldtype);
}
//---------------------------------------------------------------------------
void TBattleMapLogic::DoSelectedWaitLogic()
{
    m_BattleMapWindow->GetBattleMapPane().ClearBrightChip();
    // 㕨
    UpdateMapObjStepped();
    // ol擾
    AddExperience((*m_CharaList[m_ActiveMaster])[m_ActiveChara], false, m_KillList);
    // ҋ@ꍇ͎̍s悤ȋĈőĂ݂B
    if(m_LogicStatus == LOGIC_PLAYERCOMMAND_BEFOREMOVE ||
       m_LogicStatus == LOGIC_COMTHINK){
        // ړҋ@ꍇ
        (*m_CharaList[m_ActiveMaster])[m_ActiveChara].m_actionpoint += 200;
    }
    else{
        // ړđҋ@ꍇ
        (*m_CharaList[m_ActiveMaster])[m_ActiveChara].m_actionpoint += 100;
    }
    UpdateShowingChara();   // NX`FWꍇ̂߁B
    m_LogicStatus = LOGIC_NEXTCHARA;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::DoSelectedMoveLogic(int x, int y)
{
    m_BattleMapWindow->GetBattleMapPane().ClearBrightChip();
    const TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
    point_t p(x, y);
    int dist = 0;
    for(;;){
        if(p == chara.m_location){
            break;
        }
        p = m_MoveFromLayer.GetValue(p.x, p.y);
        ++dist;
    }
    m_MoveTo.x = x;
    m_MoveTo.y = y;
    m_Moved = true;
    m_MoveFrameMax = dist * m_MoveFramePerChip;
    m_MoveFrame = 0;
    if(m_NoWatch){
        DoMoveEndLogic();
    }
    else{
        m_LogicStatus = LOGIC_MOVING;
    }
//g_Logger.DebugLog((boost::format("DoSelectedMoveLogic from=(%d,%d) to=(%d,%d) dist=%d") % chara.m_location.x % chara.m_location.y % m_MoveTo.x % m_MoveTo.y % dist).str());
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::DoSelectedAttackLogic(int x, int y, bool isCharge)
{
    // 㕨
    UpdateMapObjStepped();
    // L̗L擾B
    int masterindex = m_MasterLayer.GetValue(x, y);
    if(masterindex != -1 && masterindex != m_ActiveMaster){
        if(isCharge && m_Map->GetMapObjType(x, y)){
            return false;
        }
        m_AttackTo.x = x;
        m_AttackTo.y = y;
        m_DefenseMaster = masterindex;
        m_DefenseChara = m_CharaLayer.GetValue(x, y);
        //m_AttackIndex = 0;
        m_AttackIndex = FindNextAttackIndex((*m_CharaList[m_ActiveMaster])[m_ActiveChara], (*m_CharaList[m_DefenseMaster])[m_DefenseChara], 0);
        m_AttackFrame = 0;
        if(isCharge){
            m_LogicStatus = LOGIC_CHARGING;
        }
        else{
            m_LogicStatus = LOGIC_ATTACKING;
        }
        if(m_NoWatch){
            // ϐ펞͑Ɍʂ߂B
            int i;
            for(i=0; i<256; ++i){
                if(DoAttackingLogic()){
                    break;
                }
            }
            if(i == 256){
                OFERRLOG("256[v킯ȂBvOɃoOH");
            }
            m_LogicStatus = LOGIC_NEXTCHARA;
        }
        return true;
    }
    else if(!isCharge && m_ActiveMaster && m_Map->GetMapObjType(x, y) == MAPOBJ_WALL){
        // ڕWǂ̏ꍇ͕ǂɓSpecialsB
        const TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
        const TOpenFarenData* data = g_GameMaster.GetData();
        m_SelectedSpecial = data->GetAttackWallSpecial();
        GenerateEffectPointList(chara.m_location, point_t(x, y), m_SelectedSpecial);
        //m_EffectTarget = point_t(x, y);
        //m_EffectCount = 0;
        //m_EffectFrame = 0;
        m_SpecialCount = 1;
        //m_LogicStatus = LOGIC_EFFECT;
        TransitEffectLogic(x, y);
        return true;
    }
    return false;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::DoSelectedSpecialLogic(int specialindex, bool isBeforeScript)
{
    // 㕨
    UpdateMapObjStepped();
    const TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
    const TSpecial& spe = data->GetSpecial(specialindex);
g_Logger.DebugLog((boost::format("selected=%s before=%d") % spe.m_name % isBeforeScript).str());
    m_SelectedSpecial = specialindex;
    // 񐔐ݒiႦ΃AeĩuX2j
    if(specialindex == charadata.m_specialindex){
        m_SpecialCount = charadata.m_specialcount;
    }
    else{
        m_SpecialCount = 1;
    }
    if(isBeforeScript){
g_Logger.DebugLog((boost::format("%s") % spe.m_script_before).str());
        if(!spe.m_script_before.empty()){
            m_LuaBattleEvent.CallFunction(spe.m_script_before.c_str());
            // yieldŔꍇSpecialOXNvgsԂɑJڂB
            if(m_LuaBattleEvent.CanResume()){
                m_LogicStatus = LOGIC_SCRIPT;
                m_ScriptStatus = SCRIPT_BEFORE_SPECIAL;
                return;
            }
            if(!m_LuaBattleEvent.GetLastFunctionValue()){
g_Logger.DebugLog((boost::format("%s returned false(?)") % spe.m_script_before).str());
                m_LogicStatus = LOGIC_NEXTCHARA;
                return;
            }
        }
    }
    m_BattleMapWindow->GetBattleBottomPane().SetMessage(spe.m_name);
    TValueLayer<int> *layer;
    if(m_ActiveCharaIsPlayer){
        layer = &m_ShadeLayer;
    }
    else{
        layer = &m_DummyLayer;
    }
    bool needselect = GetRange(*layer, chara.m_location.x, chara.m_location.y, spe.m_rangetype, spe.m_maxrange, spe.m_maxradius, 1);
    // COM̏ꍇ̕WIIB
    if(needselect && !m_ActiveCharaIsPlayer){
        GenerateEffectPointList(chara.m_location, m_ComTarget, specialindex);
        //m_EffectTarget = m_ComTarget;
        //m_EffectCount = 0;
        //m_EffectFrame = 0;
        //m_LogicStatus = LOGIC_EFFECT;
        TransitEffectLogic(m_ComTarget.x, m_ComTarget.y);
        return;
    }
    if(needselect){
        // IցB
        m_LogicStatus = LOGIC_PLAYERSPECIAL;
    }
    else{
        // I̕KvꍇB
        // gp񐔐@tāAΉ񐔂𑝂₷B3ӏi[U2ӏACOM1ӏjB
        std::map<int, int>::const_iterator itr = m_SpecialMagicMap.find(specialindex);
        if(itr != m_SpecialMagicMap.end()){
            ++m_MagicUseCount[m_ActiveMaster][itr->second];
        }
        // ʔ͈͂߂ČʏցB
        GenerateEffectPointList(chara.m_location, chara.m_location, specialindex);
        //m_EffectCount = 0;
        //m_EffectFrame = 0;
        //m_LogicStatus = LOGIC_EFFECT;
        TransitEffectLogic(0, 0);
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::DoSelectedRunAwayLogic()
{
    // 㕨
    UpdateMapObjStepped();
    TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
    AddExperience(chara, false, m_KillList);
    m_RunAwayList.push_back(std::pair<int, int>(m_ActiveMaster, m_ActiveChara));
    chara.m_try_runaway = true;
    m_LogicStatus = LOGIC_NEXTCHARA;
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::DoAttackingLogic()
{
    TCharacterInstance *atkchara, *defchara;
//g_Logger.DebugLog((boost::format("i=%d frame=%d") % m_AttackIndex % m_AttackFrame).str());
    int atkcharamark;   // L̃}[N\ێ̂߁BȂB
    if(!(m_AttackIndex % 2)){
        // ͍Ud|̍UB
        atkchara = &(*m_CharaList[m_ActiveMaster])[m_ActiveChara];
        defchara = &(*m_CharaList[m_DefenseMaster])[m_DefenseChara];
        atkcharamark = (m_ActiveMaster != 0);
    }
    else{
        // ͍Uꂽ̍UB
        atkchara = &(*m_CharaList[m_DefenseMaster])[m_DefenseChara];
        defchara = &(*m_CharaList[m_ActiveMaster])[m_ActiveChara];
        atkcharamark = (m_DefenseMaster != 0);
    }
    if(!m_AttackFrame){
        int atkindex;
        if(m_LogicStatus == LOGIC_ATTACKING){
            atkindex = m_AttackIndex / 2;
        }
        else{
            atkindex = 0;   // ˌ͏1ڂ̍UpB
        }
        m_LastAttackResultList.clear();
        EvalAttackDamage(*atkchara, atkindex, *defchara, m_LastAttackResultList);
        if(g_Ini.GetBoolValue(TOpenFarenIni::SOUND_ENABLED) && !m_NoWatch){
            if(!m_LastAttackResultList.empty() && m_LastAttackResultList[0].m_hit){
                g_GameMaster.GetMixer()->PlayPermanentChunk(TOpenFarenMixer::CHUNK_ATTACK);
            }
        }
    }
    if(m_AttackFrame == m_AttackFrameMax / 2){
        bool existsDeadChara;
        ApplyEffectResult(atkchara, m_LastAttackResultList, existsDeadChara);
        SetEffectString(m_LastAttackResultList);
        if(existsDeadChara){
            if(g_Ini.GetBoolValue(TOpenFarenIni::SOUND_ENABLED) && !m_NoWatch){
                g_GameMaster.GetMixer()->PlayPermanentChunk(TOpenFarenMixer::CHUNK_DEAD);
            }
        }
        OFDBGLOG("after ApplyEffectResult 1");
    }
    if(m_AttackFrame >= m_AttackFrameMax){
        // ǂ炩SĂΏIBizUz͍̎UʂƂj
        if(!atkchara->m_hp || !defchara->m_hp){
            if(m_LogicStatus == LOGIC_CHARGING){
                // ˌœˌꍇWXVB
                TCharacterInstance& activeChara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
                if(activeChara.m_hp){
                    m_MasterLayer.GetValue(activeChara.m_location.x, activeChara.m_location.y) = -1;
                    m_MasterLayer.GetValue(m_AttackTo.x, m_AttackTo.y) = m_ActiveMaster;
                    m_CharaLayer.GetValue(m_AttackTo.x, m_AttackTo.y) = m_ActiveChara;
                    activeChara.m_location = m_AttackTo;
                }
            }
            if(atkchara->m_hp){
                AddExperience(*atkchara, (&(*m_CharaList[m_ActiveMaster])[m_ActiveChara] == atkchara), m_KillList);
            }
            else{
                AddExperience(*defchara, (&(*m_CharaList[m_ActiveMaster])[m_ActiveChara] == defchara), m_KillList);
            }
            UpdateShowingChara();   // NX`FWꍇ̂߁B
            return true;
        }
        m_AttackFrame = 0;
        if(m_LogicStatus == LOGIC_ATTACKING){
            // ̍U邩ǂ߂B
            ++m_AttackIndex;
            m_AttackIndex = FindNextAttackIndex((*m_CharaList[m_ActiveMaster])[m_ActiveChara], (*m_CharaList[m_DefenseMaster])[m_DefenseChara], m_AttackIndex);
        }
        else{
            // ˌ̏ꍇ͏񐔂܂łB
            // UꂽUłȂԂ̏ꍇAUꂽ̍UƂ΂B
            if((m_AttackIndex + 1) % 2){
                if(!CanAttack((*m_CharaList[m_DefenseMaster])[m_DefenseChara])){
                    ++m_AttackIndex;
                }
            }
            if(m_AttackIndex >= CHARGE_MAX){
                m_AttackIndex = -1;
            }
            else{
                ++m_AttackIndex;
            }
        }
        // ǂɂłUΏIB
        if(m_AttackIndex == -1){
            AddExperience((*m_CharaList[m_ActiveMaster])[m_ActiveChara], true, m_KillList);
            UpdateShowingChara();   // NX`FWꍇ̂߁B
            return true;
        }
        return false;
    }
    ++m_AttackFrame;
    TShowingChara showchara;
    std::memset(&showchara, 0, sizeof showchara);
    showchara.m_type = atkchara->m_type;
    showchara.m_location1 = atkchara->m_location;
    showchara.m_location2 = defchara->m_location;
    showchara.m_markchip = atkcharamark;
    if(m_AttackFrame < m_AttackFrameMax/2){
        showchara.m_step = m_AttackFrame * STEP_MAX / (m_AttackFrameMax/2);
    }
    else if(m_AttackFrame < m_AttackFrameMax){
        int fmax = m_AttackFrameMax/2;
        showchara.m_step = (fmax - m_AttackFrame % fmax) * STEP_MAX / fmax;
    }
    else{
        showchara.m_step = 0;
    }
    m_BattleMapWindow->GetBattleMapPane().SetShowingChara(atkchara->m_showingindex, showchara);
//g_Logger.DebugLog("attacking end");
    return false;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::DoEffectLogic()
{
    TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
    const TSpecial& spe = data->GetSpecial(m_SelectedSpecial);
    if(!m_EffectFrame && spe.m_sound != -1){
        if(g_Ini.GetBoolValue(TOpenFarenIni::SOUND_ENABLED) && !m_NoWatch){
            g_GameMaster.GetMixer()->PlayChunk(spe.m_sound);
        }
    }
    std::deque<bool> evaltiming;
    bool end;
    ShowSpecial(chara, m_EffectTo, m_SelectedSpecial, m_EffectFrame, evaltiming, end);
    if(!spe.m_script_before.empty()){
        OFDBGLOG((boost::format("ecnt=%d") % m_EffectTo.size()).str());
    }
OFDBGLOG((boost::format("f=%d ecnt=%d scnt=%d") % m_EffectFrame % m_EffectCount % m_SpecialCount).str());
    if(end){
        //UpdateShowingChara();
        ++m_EffectCount;
        if(m_EffectCount < m_SpecialCount){
            m_EffectFrame = 0;
        }
        else{
            (*m_CharaList[m_ActiveMaster])[m_ActiveChara].m_mp -= spe.m_consumemp;
            AddExperience((*m_CharaList[m_ActiveMaster])[m_ActiveChara], true, m_KillList);
            bool scriptresume = false;
            // KEZ̃XNvgꍇ͎sB
g_Logger.DebugLog((boost::format("SPE=%s after=%s") % spe.m_name % spe.m_script_after).str());
            if(!spe.m_script_after.empty()){
                m_LuaBattleEvent.CallFunction(spe.m_script_after.c_str());
                // yieldŔꍇSpecialOXNvgsԂɑJڂB
                if(m_LuaBattleEvent.CanResume()){
                    m_LogicStatus = LOGIC_SCRIPT;
                    m_ScriptStatus = SCRIPT_AFTER_SPECIAL;
                    scriptresume = true;
                }
            }
            if(!scriptresume){
                if(m_NoWatch){
                    m_LogicStatus = LOGIC_NEXTCHARA;
                }
                else{
                    m_WaitBeforeNextCharaFrame = 10;    // KB
                    m_LogicStatus = LOGIC_WAITBEFORENEXTCHARA;
                }
            }
        }
    }
    else{
        if(std::find(evaltiming.begin(), evaltiming.end(), true) != evaltiming.end()){
            //const TAttribute& attr = data->GetAttribute(spe.m_attribute);
            std::vector<point_t> targetlist;
            GetEffectTargetList(m_EffectTo, evaltiming, m_ActiveMaster, m_SelectedSpecial, targetlist);
            std::vector<TEffectResult> resultlist;
            EvalSpecial(m_ActiveMaster, chara, targetlist, m_SelectedSpecial, resultlist, 0, false);
            bool existsDeadChara;
            ApplyEffectResult(&chara, resultlist, existsDeadChara);
            SetEffectString(resultlist);
            if(existsDeadChara){
                if(g_Ini.GetBoolValue(TOpenFarenIni::SOUND_ENABLED) && !m_NoWatch){
                    g_GameMaster.GetMixer()->PlayPermanentChunk(TOpenFarenMixer::CHUNK_DEAD);
                }
            }
            OFDBGLOG("after ApplyEffectResult 2");
        }
        ++m_EffectFrame;
    }
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::DoJudgeGameLogic()
{
    int winmaster;
    if(JudgeGame(winmaster)){
        HideCharaDetail();
        HideBattleCharaList();
        std::string msg;
        if(winmaster == -1){
            // 傤ǗRłőSSꍇ͋NB̏ꍇ͎̏ƂB
            winmaster = 0;
        }
        if(m_MasterIndex[winmaster] != -1){
            const TOpenFarenGame* game = g_GameMaster.GetGame();
            const TOpenFarenData* data = g_GameMaster.GetData();
            const TMasterInstance& master = game->GetMasterInstance(m_MasterIndex[winmaster]);
            const TMaster& masterdata = data->GetMaster(master.m_type);
            msg = (boost::format("%sȐł") % masterdata.m_name).str();
        }
        else{
            msg = "Ȑł";
        }
        if(!m_NoWatch){
            TGcnMsgDlg::Show("Information", msg, TGcnMsgDlg::BUTTON_OK, g_Ini.GetMessageTimeoutFrame());
        }
        m_WinMaster = winmaster;
        m_LogicStatus = LOGIC_BATTLEEND_MSG;
        return true;
    }
    return false;
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::DoJudgeHeroLogic()
{
    std::string msg;
    for(std::vector<const TCharacterInstance*>::const_iterator itr = m_KillList.begin(); itr != m_KillList.end(); ++itr){
        const TOpenFarenData* data = g_GameMaster.GetData();
        const TBattleCharacter& charadata = data->GetCharacter((*itr)->m_type);
        if(charadata.m_deadmessage || charadata.m_master){
            if(!msg.empty()){
                msg += '\n';
            }
            msg += (boost::format("%s񂾁I") % charadata.m_name).str();
        }
    }
    m_KillList.clear();
    if(!msg.empty()){
        TGcnMsgDlg::Show("Information", msg, TGcnMsgDlg::BUTTON_OK, g_Ini.GetMessageTimeoutFrame());
        m_LogicStatus = LOGIC_HERODEAD_MSG;
        return true;
    }
    return false;
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::DoAutoSpecialLogic()
{
    if(m_ActivatedAutoSpecialList.empty()){
        return false;
    }
g_Logger.DebugLog("DoAutoSpecialLogic");
    //int master = m_ActivatedAutoSpecialList[0].first;
    //int charaindex = m_ActivatedAutoSpecialList[0].second;
    TAutoSpecialEntry entry = m_ActivatedAutoSpecialList[0];
    m_ActivatedAutoSpecialList.erase(m_ActivatedAutoSpecialList.begin());
    if(!entry.m_function.empty()){
        m_LuaBattleEvent.CallFunction(entry.m_function.c_str());
        // yieldŔꍇ̓XNvgsԂɑJڂB
        if(m_LuaBattleEvent.CanResume()){
            m_LogicStatus = LOGIC_SCRIPT;
            m_ScriptStatus = SCRIPT_BEFORE_NEXTCHARA;
g_Logger.DebugLog("DoAutoSpecialLogic yield");
            return true;
        }
g_Logger.DebugLog("DoAutoSpecialLogic end");
        return false;
    }
    //int master = entry.m_master;
    //int charaindex = entry.m_charaindex;
    //const TCharacterInstance& atkchara = (*m_CharaList[master])[charaindex];
    m_ActiveMaster = entry.m_master;
    m_ActiveChara = entry.m_charaindex;
    const TCharacterInstance& atkchara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& atkcharadata = data->GetCharacter(atkchara.m_type);
    DoSelectedSpecialLogic(atkcharadata.m_autospecialindex, true);
    return true;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::DoMoveEndLogic()
{
    TCharacterInstance& c = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
    // WXVB
    m_MasterLayer.GetValue(c.m_location.x, c.m_location.y) = -1;
    m_MasterLayer.GetValue(m_MoveTo.x, m_MoveTo.y) = m_ActiveMaster;
    m_CharaLayer.GetValue(m_MoveTo.x, m_MoveTo.y) = m_ActiveChara;
    m_MoveFrom = c.m_location;
    c.m_location = m_MoveTo;
    // \XVB
    UpdateShowingChara(m_ActiveMaster, m_ActiveChara);
    if(m_ActiveCharaIsPlayer){
        m_BattleMapWindow->GetBattleMapPane().SetBrightChip(c.m_location.x, c.m_location.y);
        m_BattleMapWindow->SetScrollCenterOfChip(c.m_location.x, c.m_location.y);
        UpdateActionCommandWindow(true);
        ShowBattleCommand();
        m_LogicStatus = LOGIC_PLAYERCOMMAND_AFTERMOVE;
    }
    else{
        // COM̏ꍇB
        if(m_ComActionType == ACTION_WAIT){
            DoSelectedWaitLogic();
        }
        else if(m_ComActionType == ACTION_ATTACK){
            DoSelectedAttackLogic(m_ComTarget.x, m_ComTarget.y, false);
        }
        else if(m_ComActionType == ACTION_SPECIAL){
            DoSelectedSpecialLogic(m_ComSpecial, true);
        }
        else if(m_ComActionType == ACTION_RUNAWAY){
            DoSelectedRunAwayLogic();
        }
        else{
            g_Logger.ErrorLog("AI routine error.");
            m_LogicStatus = LOGIC_ERROR;
        }
    }
}
//---------------------------------------------------------------------------
// m_SpecialCount ɂĂ͂̊֐̊OŐݒ肷B
void TBattleMapLogic::TransitEffectLogic(int targetx, int targety)
{
    // ⏕pFLŌɎs@EKEZLB
    m_LastSpecialTable[m_ActiveMaster][m_ActiveChara] = m_SelectedSpecial;
    m_LastSpecial[m_ActiveMaster] = m_SelectedSpecial;
    // ԑJڗp
    m_EffectTarget = point_t(targetx, targety);
    m_EffectCount = 0;
    m_EffectFrame = 0;
    m_LogicStatus = LOGIC_EFFECT;
}
//---------------------------------------------------------------------------
int TBattleMapLogic::FindNextAttackIndex(const TCharacterInstance& atkchara, const TCharacterInstance& defchara, int totalindex) const
{
    // ̍U邩ǂ߂B
    std::vector<TAttack> atkatklist, defatklist;
    GetAttackList(atkchara, atkatklist);
    GetAttackList(defchara, defatklist);
    for(;;){
        int i = totalindex/2;
        if(!(totalindex%2)){
            if(i < atkatklist.size() && atkatklist[i].m_special != -1){
                break;
            }
        }
        else{
            if(i < defatklist.size() && defatklist[i].m_special != -1){
                break;
            }
        }
        if(i >= atkatklist.size() && i >= defatklist.size()){
            totalindex = -1;
            break;
        }
        ++totalindex;
    }
    return totalindex;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetPlaceList(const std::vector<int>& neighbor, const TCharacterInstanceList& charalist, int offset, std::vector<point_t>& placelist, TCharacterInstanceList& target)
{
    target.clear();
    if(neighbor[0] == -1){
        point_t center = m_Map->GetCenterXY();
        // ̏zuWB
        placelist.clear();
//        placelist.push_back(center);
//        int citymax = m_Map->GetCityCount();
//        for(unsigned place = 0; place < citymax; ++place){
//            if(placelist.size() >= SAVESIDE_PLACEMAX){
//                break;
//            }
//            placelist.push_back(m_Map->GetCityXY(place));
//        }
//        int wallmax = m_Map->GetWallCount();
//        for(unsigned place = 0; place < wallmax; ++place){
//            if(placelist.size() >= SAVESIDE_PLACEMAX){
//                break;
//            }
//            placelist.push_back(m_Map->GetWallXY(place));
//        }
        // SXEǁETĉB
        int search_from = TNeighbor::GetNeighborIndex(0);
        int search_to = TNeighbor::GetNeighborIndex(TNeighbor::NEIGHBOR_MAX - 1);
        int mode = center.y%2;
        for(int i=search_from; i<search_to; ++i){
            point_t p = TNeighbor::GetNeighbor(mode, i);
            int nextx = center.x + p.x;
            int nexty = center.y + p.y;
            if(IsValidXY(nextx, nexty)){
                int obj = m_Map->GetMapObjType(nextx, nexty);
                if(obj == MAPOBJ_CITY || obj == MAPOBJ_WALL || obj == MAPOBJ_CASTLE){
                    placelist.push_back(point_t(nextx, nexty));
                    if(placelist.size() >= SAVESIDE_PLACEMAX){
                        break;
                    }
                }
            }
        }
        // SꍇtĂiʏAS͏Ȃ̂łsvȏHjB
        if(std::find(placelist.begin(), placelist.end(), center) == placelist.end()){
            placelist.push_back(center);
        }
        // WȂꍇ͒SKɒoB
        if(placelist.size() < SAVESIDE_PLACEMIN){
            int from = TNeighbor::GetNeighborIndex(0);
            int to = TNeighbor::GetNeighborIndex(10);
            //int mode = center.y%2;
            for(int i=from; i<to; ++i){
                point_t p = TNeighbor::GetNeighbor(mode, i);
                int nextx = center.x + p.x;
                int nexty = center.y + p.y;
                if(IsValidXY(nextx, nexty)){
                    point_t np(nextx, nexty);
                    if(std::find(placelist.begin(), placelist.end(), np) == placelist.end()){
                        placelist.push_back(np);
                        if(placelist.size() >= SAVESIDE_PLACEMIN){
                            break;
                        }
                    }
                }
            }
        }
        // SLzuΏہB
        target = charalist;
    }
    else{
        // ȔzuWB
        if(!m_Map->GetNeighborPlace(neighbor[offset], placelist)){
            g_Logger.ErrorLog("UzuW̌Ɏs");
            return;
        }
        // zuς݂̏ꏊ͔zusɂizuWdȂĂȂ΂̏͗vȂdȂĂꍇɔājB
        for(int i = placelist.size() - 1; i >= 0; --i){
            if(m_MasterLayer.GetValue(placelist[i].x, placelist[i].y) != -1){
                placelist.erase(placelist.begin() + i);
            }
        }
        int ai = neighbor[offset];
        for(unsigned u = offset; u < neighbor.size(); ++u){
//g_Logger.DebugLog((boost::format("n=%d")%neighbor[u]).str());
            if(neighbor[u] != ai){
                break;
            }
            target.push_back(charalist[u]);
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::PlayerNextPlace()
{
    m_ShadeLayer.Init(1);
    GetPlaceList(m_NeighborArea[m_PlaceMasterIndex], *m_CharaList[m_PlaceMasterIndex], m_PlayerPlacedCount, m_PlaceXYList, m_PlaceCharaList);
    for(unsigned u=0; u<m_PlaceXYList.size(); ++u){
        m_BattleMapWindow->GetBattleMapPane().AddChipFrame(m_PlaceXYList[u].x, m_PlaceXYList[u].y);
        m_ShadeLayer.GetValue(m_PlaceXYList[u].x, m_PlaceXYList[u].y) = 0;
    }
    m_PlacedFlag.clear();
    m_PlacedFlag.resize(m_PlaceCharaList.size());
    // zuLXgEBhE\
    m_CharaPlaceWindow->GetCharaListPane().SetCharaList(&m_PlaceCharaList, &m_PlacedFlag);
    gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
    m_CharaPlaceWindow->Show();
    top->moveToTop(m_CharaPlaceWindow);
    // zuW𒆐SɃXN[
    m_BattleMapWindow->SetScrollCenterOfChips(m_PlaceXYList);
    m_LogicStatus = LOGIC_PLAYERPLACE;
}
//---------------------------------------------------------------------------
// Ŏ̃L̍sA퓬IA^[iŝꂩԂB
void TBattleMapLogic::ActivateNextChara(bool& battleend, bool& progressturn)
{
    battleend = false;
    progressturn = false;
    for(;;){
        for(unsigned master=0; master<m_CharaList.size(); ++master){
            for(unsigned u=0; u<m_CharaList[master]->size(); ++u){
//g_Logger.DebugLog((boost::format("%u:actp=%d") % u % (*m_CharaList[master])[u].m_actionpoint).str());
                TCharacterInstance& chara = (*m_CharaList[master])[u];
                if(!chara.m_hp || chara.m_try_runaway || chara.m_runaway || chara.m_elementstatus == TCharacterInstance::ELEMENT_SUMMON){
                    continue;
                }
                if(chara.m_actionpoint >= ACTIONPOINT_OK){
                    // s|CgB
                    chara.m_actionpoint -= ACTIONPOINT_OK;
                    // ჂƐΉƐ͍ssB
                    if(!CanAction(chara)){
                        continue;
                    }
                    // ̃L̍sԂB
                    m_ActiveMaster = master;
                    m_ActiveChara = u;
                    m_Moved = false;
                    return;
                }
            }
        }
        if(m_TurnLoop >= TURNLOOP_MAX){
            if(m_Turn >= g_Ini.GetIntValue(TOpenFarenIni::BATTLE_TURNMAX)){
                // 퓬IԂB
                battleend = true;
                return;
            }
            // ^[isԂB
            ++m_Turn;
            m_TurnLoop = 0;
            progressturn = true;
            return;
        }
        ++m_TurnLoop;
        const TOpenFarenData* data = g_GameMaster.GetData();
        for(unsigned master=0; master<m_CharaList.size(); ++master){
            for(unsigned u=0; u<m_CharaList[master]->size(); ++u){
                TCharacterInstance& chara = (*m_CharaList[master])[u];
                const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
                if(!chara.m_hp || chara.m_runaway || chara.m_elementstatus == TCharacterInstance::ELEMENT_SUMMON){
                    continue;
                }
                (*m_CharaList[master])[u].m_actionpoint += charadata.m_speed*0.8 + 50 + (mtrand() / (MTRAND_MAX / 10 + 1));
            }
        }
    }
}
//---------------------------------------------------------------------------
// ߂ltrueȂ^[is̏IB
bool TBattleMapLogic::ProgressTurn()
{
    // obhXe[^X̉EŁE񕜂Ȃǂ鎞ԁBKB
    static const int SHOWPROGRESS_TICK = 10;

    // ̌ۂ\ȂJE^CNgđ҂B
    if(m_TurnProgressFrame >= 0 && m_TurnProgressFrame < m_TurnProgressFrameMax * m_AnimationSpeed){
        ++m_TurnProgressFrame;
        return false;
    }
    // SĂ̔肪IĂꍇIB
    if(m_TurnProgressStage == TURNPROGRESS_MAX){
        return true;
    }

    if(m_TurnProgressStage == TURNPROGRESS_BADSTATUS){
        for(unsigned master=0; master<m_CharaList.size(); ++master){
            for(unsigned u=0; u<m_CharaList[master]->size(); ++u){
                TCharacterInstance& chara = (*m_CharaList[master])[u];
                if(!chara.m_hp){
                    continue;
                }
                // _ŃobhXe[^XB
                for(int sesi = chara.m_seslist.size() - 1; sesi >= 0; --sesi){
                    if(chara.m_seslist[sesi].m_type == SYSTEM_SLEEP ||
                       chara.m_seslist[sesi].m_type == SYSTEM_DREAM){
                        if(chara.m_seslist[sesi].m_tick < BADSTATUS_TICKMAX){
                            continue;
                        }
                    }
                    int rate = 0;
                    if(chara.m_seslist[sesi].m_type == SYSTEM_SLEEP){
                        rate = 33;
                    }
                    else if(chara.m_seslist[sesi].m_type == SYSTEM_PARALYZE){
                        rate = 20;
                    }
                    else if(chara.m_seslist[sesi].m_type == SYSTEM_DREAM){
                        rate = 20;
                    }
                    // VXeʃXg폜B
                    if(rate && mtrand_n(100) < rate){
                        const TOpenFarenData* data = g_GameMaster.GetData();
                        const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
                        TSystemEffect syseff = chara.m_seslist[sesi].m_type;
                        std::string msg = (boost::format("%s%s񕜂") % charadata.m_name % data->GetSystemEffectName(syseff)).str();
                        m_BattleMapWindow->GetBattleBottomPane().SetMessage(msg);
                        // 폜
                        chara.m_seslist.erase(chara.m_seslist.begin() + sesi);
                        m_TurnProgressFrame = 0;
                        m_TurnProgressFrameMax = SHOWPROGRESS_TICK;
                        return false;
                    }
                }
            }
        }
        // SẴobhXe[^X`FbN玟̔菈ցB
        m_TurnProgressStage = TURNPROGRESS_HPRECOVER;
    }
    const TOpenFarenData* data = g_GameMaster.GetData();
    // ŁAHP/MP񕜂̕\pB
    std::vector<point_t> plist;
    std::vector<TEffectResult> results;
    std::string bottommsg;
    if(m_TurnProgressStage == TURNPROGRESS_HPRECOVER){
        // HP
        for(unsigned master=0; master<m_CharaList.size(); ++master){
            for(unsigned u=0; u<m_CharaList[master]->size(); ++u){
                const TCharacterInstance& chara = (*m_CharaList[master])[u];
                if(!chara.m_hp || chara.m_location.x == -1){
                    continue;
                }
                const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
                if(charadata.m_hprecover + chara.m_hprecover_plus){
                    TEffectResult res;
                    res.m_location = chara.m_location;
                    res.m_hit = true;
                    res.m_systemeffect = SYSTEM_HPRECOVER;
                    res.m_value = charadata.m_hprecover + chara.m_hprecover_plus;
                    res.m_special = -1;
                    plist.push_back(chara.m_location);
                    results.push_back(res);
                }
            }
        }
        bottommsg = "HP񕜂";
        // ʂɊւ炸̏Ԃɐi߂B
        m_TurnProgressStage = TURNPROGRESS_MPRECOVER;
    }
    else if(m_TurnProgressStage == TURNPROGRESS_MPRECOVER){
        // MP
        for(unsigned master=0; master<m_CharaList.size(); ++master){
            for(unsigned u=0; u<m_CharaList[master]->size(); ++u){
                const TCharacterInstance& chara = (*m_CharaList[master])[u];
                if(!chara.m_hp || chara.m_location.x == -1){
                    continue;
                }
                const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
                if(charadata.m_mprecover + chara.m_mprecover_plus){
                    TEffectResult res;
                    res.m_location = chara.m_location;
                    res.m_hit = true;
                    res.m_systemeffect = SYSTEM_MPRECOVER;
                    res.m_value = charadata.m_mprecover + chara.m_mprecover_plus;
                    res.m_special = -1;
                    plist.push_back(chara.m_location);
                    results.push_back(res);
                }
            }
        }
        bottommsg = "MP񕜂";
        // ʂɊւ炸̏Ԃɐi߂B
        m_TurnProgressStage = TURNPROGRESS_POISON;
    }
    else if(m_TurnProgressStage == TURNPROGRESS_POISON){
        // łɂHP팸B
        for(unsigned master=0; master<m_CharaList.size(); ++master){
            for(unsigned u=0; u<m_CharaList[master]->size(); ++u){
                const TCharacterInstance& chara = (*m_CharaList[master])[u];
                if(!chara.m_hp || chara.m_location.x == -1){
                    continue;
                }
                for(unsigned sesi = 0; sesi < chara.m_seslist.size(); ++sesi){
                    if(chara.m_seslist[sesi].m_type == SYSTEM_POISON){
                        TEffectResult res;
                        res.m_location = chara.m_location;
                        res.m_hit = true;
                        res.m_systemeffect = SYSTEM_NOEFFECT;   // łłɂ_[W͒ʏ\ŁB
                        // IWi̎sȂ̂łƂ肠B
                        res.m_value = (chara.m_hpmax_rank / 20) + mtrand_n(10);
                        res.m_special = -1;
                        plist.push_back(chara.m_location);
                        results.push_back(res);
                        // ł̓Xgɕ͂Ȃ̂ŔB
                        break;
                    }
                }
            }
        }
        bottommsg = "łɂ_[W";
        // ʂɊւ炸̏Ԃɐi߂B
        m_TurnProgressStage = TURNPROGRESS_RUNAWAY;
    }
    else if(m_TurnProgressStage == TURNPROGRESS_RUNAWAY){
        // B
        bool runexists = false;
        for(unsigned u=0; u<m_RunAwayList.size(); ++u){
            int master = m_RunAwayList[u].first;
            int charaindex = m_RunAwayList[u].second;
            TCharacterInstance& chara = (*m_CharaList[master])[charaindex];
            chara.m_try_runaway = false;
            if(!chara.m_hp){
                // ɎSĂ瓦tONÂ݁B
                continue;
            }
            if(master){
                // U͖ŁB
                m_MasterLayer.GetValue(chara.m_location.x, chara.m_location.y) = -1;
                chara.m_runaway = true;
                chara.m_location.x = -1;
                runexists = true;
                continue;
            }
            // ͓悪ΕsB
            if(m_SaveRunAwayCount > 0){
                m_MasterLayer.GetValue(chara.m_location.x, chara.m_location.y) = -1;
                chara.m_runaway = true;
                chara.m_location.x = -1;
                runexists = true;
                --m_SaveRunAwayCount;  // g
            }
        }
        m_RunAwayList.clear();
        if(runexists){
            UpdateShowingChara();
            // bZ[W\B
            m_TurnProgressFrame = 0;
            m_TurnProgressFrameMax = SHOWPROGRESS_TICK;
            m_BattleMapWindow->GetBattleBottomPane().SetMessage("ɐ");
        }
        // ʂɊւ炸̏Ԃɐi߂B
        m_TurnProgressStage = TURNPROGRESS_MAX; // ͖̏iIjB
    }
    if(!results.empty()){
        // \錻ۂΓKpƕ\sB
        bool existsDeadChara;
        ApplyEffectResult(NULL, results, existsDeadChara);
        SetEffectString(results);
        if(existsDeadChara){
            if(g_Ini.GetBoolValue(TOpenFarenIni::SOUND_ENABLED) && !m_NoWatch){
                g_GameMaster.GetMixer()->PlayPermanentChunk(TOpenFarenMixer::CHUNK_DEAD);
            }
        }
        m_TurnProgressFrame = 0;
        m_TurnProgressFrameMax = SHOWPROGRESS_TICK;
        m_BattleMapWindow->GetBattleBottomPane().SetMessage(bottommsg);
        OFDBGLOG((boost::format("after ApplyEffectResult 3 state=%d") % m_TurnProgressStage).str());
        return false;
    }
    // ɕ\錻ۂꍇA̔(false)B܂͏I(true)B
    return m_TurnProgressStage == TURNPROGRESS_MAX;
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::IsCommandableChara(int master, int charaindex)
{
    const TCharacterInstance& chara = (*m_CharaList[master])[charaindex];
    // z͖ɑsB
    if(HasSystemEffect(chara, SYSTEM_DREAM)){
        return false;
    }
    if(IsPlayerMaster(master)){
        // }X^[vC[̏ꍇB
        // Gg͖true
        if(chara.m_elementstatus == TCharacterInstance::ELEMENT_SUMMONED){
            return true;
        }
        const TOpenFarenData* data = g_GameMaster.GetData();
        const TOpenFarenGame* game = g_GameMaster.GetGame();
        const TMasterInstance& masterins = game->GetMasterInstance(m_MasterIndex[master]);
        const TMaster& masterdata = data->GetMaster(masterins.m_type);
        const TBattleCharacter& masterchara = data->GetCharacter(masterdata.m_facecharaindex);
        const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
g_Logger.DebugLog((boost::format("race = %d, %d") % data->GetRace(charadata.m_race).m_name.c_str() % data->GetRace(masterchara.m_race).m_name.c_str()).str());
        if(charadata.m_race == masterchara.m_race){
            // }X^[Ɠ푰͖ɖ߉\B
            return true;
        }
        else{
            // ߉\Ȏ푰B
            return m_CommandableRace[master][charadata.m_race];
        }
    }
    return false;
}
//---------------------------------------------------------------------------
// ߉\Ȏ푰tOXVB
void TBattleMapLogic::UpdateCommandableRace()
{
    const TOpenFarenData* data = g_GameMaster.GetData();
    m_CommandableRace.resize(m_MasterIndex.size());
    for(unsigned master=0; master<m_CharaList.size(); ++master){
        // UtONAB
        m_CommandableRace[master].clear();
        m_CommandableRace[master].resize(data->GetRaceCount());
        // SL̖߉\푰XgHătO𗧂ĂB
        for(unsigned u=0; u<m_CharaList[master]->size(); ++u){
            const TBattleCharacter& charadata = data->GetCharacter((*m_CharaList[master])[u].m_type);
            for(int i=0; i<charadata.m_commandrace.size(); ++i){
                m_CommandableRace[master][charadata.m_commandrace[i]] = 1;
            }
        }
    }
}
//---------------------------------------------------------------------------
// KEZXgXVB
void TBattleMapLogic::UpdateAutoSpecialList()
{
g_Logger.DebugLog((boost::format("UpdateAutoSpecialList")).str());
    const TOpenFarenData* data = g_GameMaster.GetData();
    m_AutoSpecialList.clear();
    for(unsigned master=0; master<m_CharaList.size(); ++master){
        for(unsigned u=0; u<m_CharaList[master]->size(); ++u){
            const TBattleCharacter& charadata = data->GetCharacter((*m_CharaList[master])[u].m_type);
            if(!charadata.m_autoscript.empty()){
                TAutoSpecialEntry entry;
                entry.m_event = charadata.m_autoscripttype;
                entry.m_master = master;
                entry.m_charaindex = u;
                entry.m_function = charadata.m_autoscript;
                m_AutoSpecialList.push_back(entry);
            }
            if(charadata.m_autospecialindex != -1){
                TAutoSpecialEntry entry;
                entry.m_event = charadata.m_autospecialtype;
                entry.m_master = master;
                entry.m_charaindex = u;
                m_AutoSpecialList.push_back(entry);
            }
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::ActivateAutoSpecial(int master, int charaindex)
{
g_Logger.DebugLog((boost::format("ActivateAutoSpecial")).str());
    const TOpenFarenData* data = g_GameMaster.GetData();
    for(unsigned u = 0; u < m_AutoSpecialList.size(); ++u){
        const TAutoSpecialEntry& entry = m_AutoSpecialList[u];
g_Logger.DebugLog((boost::format("ActivateAutoSpecial %d %d") % entry.m_master % entry.m_charaindex).str());
        const TCharacterInstance& chara = (*m_CharaList[entry.m_master])[entry.m_charaindex];
        const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
        if(entry.m_function.empty()){
            const TSpecial& spe = data->GetSpecial(charadata.m_autospecialindex);
            if(chara.m_mp < spe.m_consumemp){
                continue;
            }
        }
        if(entry.m_event == SPECIALEVENT_MEDEAD){
            // SB
            if(master == entry.m_master && charaindex == entry.m_charaindex){
                m_ActivatedAutoSpecialList.push_back(entry);
            }
        }
        else if(entry.m_event == SPECIALEVENT_MYSIDEDEAD){
            // ȊO̖SB
            if(master == entry.m_master && charaindex != entry.m_charaindex){
                m_ActivatedAutoSpecialList.push_back(entry);
            }
        }
        else if(entry.m_event == SPECIALEVENT_ENEMYDEAD){
            // GSB
            if(master != entry.m_master){
                m_ActivatedAutoSpecialList.push_back(entry);
            }
        }
    }
g_Logger.DebugLog((boost::format("ActivateAutoSpecial end")).str());
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::IsPlayerMaster(int master) const
{
    if(m_MasterIndex[master] == -1){
        return false;
    }
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    return game->GetMasterInstance(m_MasterIndex[master]).m_player;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetMovable(int master, const TCharacterInstance& chara)
{
    m_MoveLayer.Init();
    m_MoveFromLayer.Init();
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
    // TODO:ړ͂̌Ws
    GetMovableFrom(master, chara, chara.m_location.x, chara.m_location.y, (charadata.m_movable + chara.m_movable_plus)*MOVECOST_FACTOR, m_MoveLayer, m_MoveFromLayer, false);
}
//---------------------------------------------------------------------------
// loose=truȅꍇA傴ςȌoHT̂ߓG̕ǂʉ߂ĒTB
void TBattleMapLogic::GetMovableFrom(int master, const TCharacterInstance& chara, int x, int y, int val, TValueLayer<int>& movelayer, TValueLayer<point_t>& movefromlayer, bool loose) const
{
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
    const TMoveType& movedata = data->GetMoveType(chara.m_movetype);
    const TBattleMap *battleMap = m_Map;
    //int objcost = (charadata.m_movable + chara.m_movable_plus)*MOVECOST_FACTOR; // G̏㕨̈ړRXgB
    int layerw = battleMap->GetWidth();
    int layerh = battleMap->GetHeight();
    int height = battleMap->GetMapHeight(x, y);
    int costsize = movedata.m_cost.size();
    //int val = movelayer.GetValue(x, y);
    for(int i=0; i<6; ++i){
        int nextx = x + TNeighbor::NextDX[y%2][i];
        int nexty = y + TNeighbor::NextDY[i];
        if(nexty < 0 || nexty >= layerh){
            continue;
        }
        if(nexty % 2){
            if(nextx < 0 || nextx >= layerw - 1){
                continue;
            }
        }
        else{
            if(nextx < 1 || nextx >= layerw){
                continue;
            }
        }
        if(m_MasterLayer.GetValue(nextx, nexty) != -1){
            continue;
        }
        int maptype = battleMap->GetMapType(nextx, nexty);
        if(maptype >= costsize){
            g_Logger.ErrorLog((boost::format("map type out of range (%d >= %d)") % battleMap->GetMapType(nextx, nexty) % movedata.m_cost.size()).str());
            continue;
        }
        // TODO:ړ͂̌Ws
        const int factor = 1;
        int h = battleMap->GetMapHeight(nextx, nexty) - height;
        if(h < 0){
            h = 0;
        }
        int cost = movedata.m_cost[maptype] + h;
        cost *= factor;
        int mapobj = battleMap->GetMapObjType(nextx, nexty);
        if(mapobj){
            if(mapobj == MAPOBJ_WALL){
                if(master){
                    // U͕ǂʂȂB
                    if(loose){
                        // x傫ȒlƂĂB
                        cost = 10;
                    }
                    else{
                        continue;
                    }
                }
                //else{
                //    // ̕ǒʉ߃RXg͓HƓB
                //    cost = movedata.m_cost[data->GetMapGroundCount() + MAPOBJ_ROAD];
                //    cost *= factor;
                //    costdef = true;
                //}
            }
            else if(mapobj == MAPOBJ_CITY){
                if(master){
                    // U͏㕨i߂ȂB
                    if(loose){
                        // x傫ȒlƂĂB
                        cost = 9;
                    }
                    else{
                        if(cost < val - 1){
                            cost = val - 1;
                        }
                    }
                }
            }
        }
        if(movelayer.GetValue(nextx, nexty) < val - cost){
            movelayer.GetValue(nextx, nexty) = val - cost;
            movefromlayer.GetValue(nextx, nexty) = point_t(x, y);
            if(!InEnemyZoc(m_ActiveMaster, nextx, nexty)){
                GetMovableFrom(master, chara, nextx, nexty, val - cost, movelayer, movefromlayer, loose);
            }
        }
    }
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::InEnemyZoc(int masterindex, int x, int y) const
{
    return ExistsEnemyNext(masterindex, x, y, true);
}
//---------------------------------------------------------------------------
// w肵Wׂ̗̓GLXg擾B
void TBattleMapLogic::GetEnemyNextList(int masterindex, int x, int y, std::vector<point_t>& plist) const
{
    for(int i=0; i<6; ++i){
        int nextx = x + TNeighbor::NextDX[y%2][i];
        int nexty = y + TNeighbor::NextDY[i];
        if(!IsValidXY(nextx, nexty)){
            continue;
        }
        int masterindex2 = m_MasterLayer.GetValue(nextx, nexty);
        if(masterindex2 != -1){
            if(masterindex2 != masterindex){
                plist.push_back(point_t(nextx, nexty));
            }
        }
        else{
            // ow͏ǂɍU\B
            if(m_Map->GetMapObjType(nextx, nexty) == MAPOBJ_WALL){
                if(masterindex){
                    plist.push_back(point_t(nextx, nexty));
                }
            }
        }
    }
}
//---------------------------------------------------------------------------
// w肵Wׂ̗̎LXg擾B
void TBattleMapLogic::GetMySideNextList(int masterindex, int charaindex, int x, int y, std::vector<point_t>& plist) const
{
    for(int i=0; i<6; ++i){
        int nextx = x + TNeighbor::NextDX[y%2][i];
        int nexty = y + TNeighbor::NextDY[i];
        if(!IsValidXY(nextx, nexty)){
            continue;
        }
        int masterindex2 = m_MasterLayer.GetValue(nextx, nexty);
        if(masterindex2 != -1 && masterindex2 == masterindex && m_CharaLayer.GetValue(nextx, nexty) != charaindex){
            plist.push_back(point_t(nextx, nexty));
        }
    }
}
//---------------------------------------------------------------------------
// ǂɗאڂĂ邩B
bool TBattleMapLogic::HasWallNext(int x, int y) const
{
    for(int i=0; i<6; ++i){
        int nextx = x + TNeighbor::NextDX[y%2][i];
        int nexty = y + TNeighbor::NextDY[i];
        if(!IsValidXY(nextx, nexty)){
            continue;
        }
        if(m_Map->GetMapObjType(nextx, nexty) == MAPOBJ_WALL){
            return true;
        }
    }
    return false;
}
//---------------------------------------------------------------------------
// w肵Wׂ̗ɓGL邩B
bool TBattleMapLogic::ExistsEnemyNext(int masterindex, int x, int y, bool checkEnemyAttack) const
{
    for(int i=0; i<6; ++i){
        int nextx = x + TNeighbor::NextDX[y%2][i];
        int nexty = y + TNeighbor::NextDY[i];
        if(nextx < 0 || nexty < 0 || nextx >= m_Map->GetWidth() || nexty >= m_Map->GetHeight()){
            continue;
        }
        int masterindex2 = m_MasterLayer.GetValue(nextx, nexty);
        if(masterindex2 != -1 && masterindex2 != masterindex){
            int charaindex2 = m_CharaLayer.GetValue(nextx, nexty);
            const TCharacterInstance& chara2 = (*m_CharaList[masterindex2])[charaindex2];
            const TOpenFarenData* data = g_GameMaster.GetData();
            const TBattleCharacter& charadata = data->GetCharacter(chara2.m_type);
            // G̍U`FbNIvV`FbNB
            if(!checkEnemyAttack || HasAttack(charadata)){
                return true;
            }
        }
    }
    return false;
}
//---------------------------------------------------------------------------
// w肵Wт̋ߖTɓGL邩B
bool TBattleMapLogic::ExistsEnemyInRadius(int masterindex, int x, int y, int radius) const
{
    m_BattleMapWindow->GetBattleMapPane().ClearRadiusChip();
    int from = TNeighbor::GetNeighborIndex(0);
    int to = TNeighbor::GetNeighborIndex(radius + 1);
    int mode = y%2;
    for(int i=from; i<to; ++i){
        point_t p = TNeighbor::GetNeighbor(mode, i);
        int nextx = x + p.x;
        int nexty = y + p.y;
        if(IsValidXY(nextx, nexty)){
            if(m_MasterLayer.GetValue(nextx, nexty) != -1){
                return true;
            }
        }
    }
    return false;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::EvalAttackDamage(const TCharacterInstance& atkchara, int atkindex, const TCharacterInstance& defchara, std::vector<TEffectResult>& resultlist) const
{
    // _[WvZ
    std::vector<TAttack> atklist;
    GetAttackList(atkchara, atklist);
    if(atklist.size() <= atkindex){
        return;
    }
    const TAttack& atkdata = atklist[atkindex];

    std::vector<point_t> targetlist;
    targetlist.push_back(defchara.m_location);
    EvalSpecial(-1, atkchara, targetlist, atkdata.m_special, resultlist, atkdata.m_power, true);

#if 0
    atk = (atk * atkdata.m_power) / 100;
    // Attack=(U+30+random(8))*Ul/80
    double atk2 = (atk + 30 + mtrand_n(8)) * atk / 80;
    // Defenceh+30
    double def2 = def + 30;
    // ɁuvLꍇ͂Defence*2Auアv̏ꍇAttack*2B
    if(defchara.m_resistance[atkdata.m_attribute] == RES_STRONG){
        def2 *= 2;
    }
    else if(defchara.m_resistance[atkdata.m_attribute] == RES_WEAK){
        atk2 *= 2;
    }
    // _[W = Attack^0.7*Attack/(Defense+10+random(10))*(20+random(5))/35
    int tempdamage = std::pow(atk2, 0.7) * atk2 / (def2 + 10 + mtrand_n(10)) * (20 + mtrand_n(5)) / 35;
g_Logger.DebugLog((boost::format("at=%d dt=%d dmg=%d") % atk2 % def2 % tempdamage).str());

    // _[W1ȉȂ1A999ȏȂ999Bz΁AŃ}CiXɔ]B
    if(tempdamage < 0){
        tempdamage = 1;
    }
    else if(tempdamage > 999){
        tempdamage = 999;
    }
    /*if(defchara.m_resistance[atkdata.m_attribute] == RES_ABSORB){
        tempdamage = -tempdamage;
    }*/

    // VXe`̕tʁB
    const TAttribute& attr = data->GetAttribute(atkdata.m_attribute);
//g_Logger.DebugLog((boost::format("attr=%s") % attr.m_name).str());
    if(attr.m_systemeffect != SYSTEM_NONE){
        if(defchara.m_resistance[atkdata.m_attribute] == RES_WEAK){
            // u`ɎアvȂKtʂ炤B
            addeffect = attr.m_systemeffect;
        }
        else{
            // xB
            if(hitratio < mtrand_n(100)){
                addeffect = attr.m_systemeffect;
            }
        }
        // ŰzBzuzUzvĂȂꍇɐ^B
        atkabsorb = attr.m_systemeffect == SYSTEM_ABSORB && tempdamage > 0;
    }
    damage = tempdamage;
#endif
}
//---------------------------------------------------------------------------
// U񐔂ςĂꍇ̂Ŏۂ̎cUXg߂B
void TBattleMapLogic::GetAttackList(const TCharacterInstance& atkchara, std::vector<TAttack>& atklist) const
{
    atklist.clear();
    //if(HasSystemEffect(atkchara, SYSTEM_PARALYZE) || HasSystemEffect(atkchara, SYSTEM_SLEEP) || HasSystemEffect(atkchara, SYSTEM_STONE)){
    if(!CanAttack(atkchara)){
        return;
    }
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(atkchara.m_type);
    int atkcount = charadata.m_attacklist.size() + atkchara.m_attackcount_plus;
    if(atkcount < 0){
        atkcount = 0;
    }
    for(int i=0; i<atkcount; ++i){
        if(i < charadata.m_attacklist.size()){
            atklist.push_back(charadata.m_attacklist[i]);
        }
        else{
            // 1ڂ̍UɂȂB
            atklist.push_back(charadata.m_attacklist[0]);
        }
    }
}
//---------------------------------------------------------------------------
// U\ȏԂǂB
bool TBattleMapLogic::CanAttack(const TCharacterInstance& atkchara) const
{
    return !HasSystemEffect(atkchara, SYSTEM_PARALYZE) && !HasSystemEffect(atkchara, SYSTEM_SLEEP) && !HasSystemEffect(atkchara, SYSTEM_STONE);
}
//---------------------------------------------------------------------------
// vl[`p̍U]B
void TBattleMapLogic::EvalAttackList(int atkmaster, int atkcharaindex, int defmaster, int defcharaindex, TEvalResultList& evallist, bool isCharge) const
{
OFDBGLOG((boost::format("s")).str());
    const TCharacterInstance& atkchara = (*m_CharaList[atkmaster])[atkcharaindex];
    const TCharacterInstance& defchara = (*m_CharaList[defmaster])[defcharaindex];
    std::vector<TAttack> atkatklist, defatklist;
    GetAttackList(atkchara, atkatklist);
    GetAttackList(defchara, defatklist);
    TEffectResultList allresultlist;
    std::deque<bool> isatklist;
    int index = 0;
    for(;;){
        int atkindex;
        if(isCharge){
            atkindex = 0;   // ˌ͏1ڂ̍UpB
        }
        else{
            atkindex = index / 2;
        }
        if(!(index%2)){
            if(atkindex < atkatklist.size() && atkatklist[atkindex].m_special != -1){
                TEffectResultList resultlist;
                EvalAttackDamage(atkchara, atkindex, defchara, resultlist);
                std::copy(resultlist.begin(), resultlist.end(), std::back_inserter(allresultlist));
                for(unsigned u=0; u<resultlist.size(); ++u){
                    isatklist.push_back(true);
                }
            }
        }
        else{
            if(atkindex < defatklist.size() && defatklist[atkindex].m_special != -1){
                TEffectResultList resultlist;
                EvalAttackDamage(defchara, atkindex, atkchara, resultlist);
                std::copy(resultlist.begin(), resultlist.end(), std::back_inserter(allresultlist));
                for(unsigned u=0; u<resultlist.size(); ++u){
                    isatklist.push_back(false);
                }
            }
        }
        if(isCharge){
            if(index >= CHARGE_MAX){
                break;
            }
        }
        else{
            int i = index/2;
            if(i >= atkatklist.size() && i >= defatklist.size()){
                break;
            }
        }
        ++index;
    }
    for(unsigned u=0; u<allresultlist.size(); ++u){
        bool isatk = isatklist[u];
        TEvalResult eval;
        eval.location = defchara.m_location;
        eval.masterindex = isatk ? defmaster : atkmaster;
        eval.charaindex = isatk ? defcharaindex : atkcharaindex;
        eval.value = allresultlist[u].m_value;
        eval.revivecharatype = allresultlist[u].m_revivecharatype;
        eval.summoncharatype = allresultlist[u].m_summoncharatype;
        eval.systemeffect = allresultlist[u].m_systemeffect;
        evallist.push_back(eval);
//g_Logger.DebugLog((boost::format("EvalAttackList %s %d %d %d") % isCharge % eval.masterindex % eval.value % allresultlist[u].m_systemeffect).str());
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::EvalSpecial(int atkmaster, const TCharacterInstance& atkchara, const std::vector<point_t>& targetlist, int special, std::vector<TEffectResult>& resultlist, int power, bool usepower) const
{
    // vZ֐|C^̔zB
    typedef int (*TFormulaFunction)(const TBattleMap& btmap, const TCharacterInstance& atkchara, const TCharacterInstance& defchara, const TSpecial& spe, int power, bool isCure);
    TFormulaFunction Formulas[FORMULA_MAX] = {
        NULL,
        &FormulaFTAttack,
        &FormulaFTSpecial,
        &FormulaFTMagic,
        NULL,                   // 100%i100Ԃ֐ɂĂj
        &FormulaFTAttackHit,
        &FormulaFTAttackHit,    // KEZ̖
        &FormulaFTMagicHit};

    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(atkchara.m_type);
    //OFDBGLOG((boost::format("charadata=%s spe=%d") %charadata.m_id%special).str());
    const TSpecial& spe = data->GetSpecial(special);
    const TAttribute& attr = data->GetAttribute(spe.m_attribute);
    // ̏͒̂ŕʊ֐ŁB
    if(attr.m_systemeffect == SYSTEM_REVIVE){
        EvalRevive(atkmaster, charadata, targetlist, special, resultlist);
        return;
    }
    // p[w肳ꂽꍇiȔꍇj͂̒lgB
    int powerval;
    if(usepower){
        powerval = power;
    }
    else{
        powerval = spe.m_power;
    }
    for(std::vector<point_t>::const_iterator itr = targetlist.begin(); itr != targetlist.end(); ++itr){
        const TAttribute& attr = data->GetAttribute(spe.m_attribute);
        if(attr.m_systemeffect == SYSTEM_CRASHWALL){
            // ǔj
            if(m_Map->GetMapObjType(itr->x, itr->y) != MAPOBJ_WALL){
                continue;
            }
            TEffectResult result_wall;
            result_wall.m_special = special;
            result_wall.m_location = *itr;
            result_wall.m_hit = true;
            result_wall.m_value = 0;
            result_wall.m_systemeffect = attr.m_systemeffect;
            resultlist.push_back(result_wall);
            // W0̏ꍇ͏ǔj󂾂sLɃ_[W͖B
            if(!spe.m_power){
                continue;
            }
        }
        if(attr.m_systemeffect == SYSTEM_SUMMON){
            // 
            const TBattleCharacter& sumcharadata = data->GetCharacter(spe.m_summonchara);
            TEffectResult result_summon;
            result_summon.m_special = special;
            result_summon.m_location = *itr;
            result_summon.m_hit = true;
            result_summon.m_value = sumcharadata.m_hpmax;
            result_summon.m_systemeffect = attr.m_systemeffect;
            result_summon.m_summoncharatype = spe.m_summonchara;
            resultlist.push_back(result_summon);
            continue;
        }

        int master = m_MasterLayer.GetValue(itr->x, itr->y);
        if(master == -1){
            continue;
        }
        TCharacterInstance& defchara = (*m_CharaList[master])[m_CharaLayer.GetValue(itr->x, itr->y)];
        const TBattleCharacter& defcharadata = data->GetCharacter(defchara.m_type);
        // ΏۃLOړ^Cv̏ꍇ͌ʑΏۂƂȂB
        if(std::find(spe.m_excludemt.begin(), spe.m_excludemt.end(), defcharadata.m_movetype) != spe.m_excludemt.end()){
            continue;
        }
        TEffectResult result;
        result.m_location = *itr;
        result.m_special = special;

        bool isCure = (attr.m_systemeffect == SYSTEM_HPRECOVER || attr.m_systemeffect == SYSTEM_MPRECOVER);
        // ʒl
        if(spe.m_hitformula1 == FORMULA_ANYTIME){
            result.m_hit = true;
        }
        else{
            // vZ
            int hit1 = (*Formulas[spe.m_hitformula1])(*m_Map, atkchara, defchara, spe, spe.m_power, isCure);
            result.m_hit = mtrand_n(100) < hit1;
        }
        if(result.m_hit){
            if(attr.m_systemeffect >= SYSTEM_PLUS_PA && attr.m_systemeffect <= SYSTEM_PLUS_ATTACKCOUNT){
                result.m_value = powerval;
            }
            else{
                // p[AbvȊO͍UlƖhlŒl肷B
                result.m_value = (*Formulas[spe.m_powerformula])(*m_Map, atkchara, defchara, spe, powerval, isCure);
                if(defchara.m_resistance[spe.m_attribute] == RES_ABSORB){
                    result.m_value = -result.m_value;
                }
                // ̓_ŉ񕜁B
                if(HasSystemEffect(defchara, SYSTEM_SLEEP)){
                    if(attr.m_systemeffect <= SYSTEM_ABSORB &&
                       attr.m_systemeffect != SYSTEM_SLEEP &&
                       !mtrand_n(1)){
                        TEffectResult result_cs;
                        result_cs.m_special = special;
                        result_cs.m_location = defchara.m_location;
                        result_cs.m_hit = true;
                        result_cs.m_systemeffect = SYSTEM_CURE_SLEEP;
                        resultlist.push_back(result_cs);
                    }
                }
                // z͍sL̉񕜂ƂĒǉB
                if(attr.m_systemeffect == SYSTEM_ABSORB){
                    TEffectResult result_ab;
                    result_ab.m_special = special;
                    result_ab.m_location = atkchara.m_location;
                    result_ab.m_hit = true;
                    result_ab.m_value = result.m_value;
                    result_ab.m_systemeffect = SYSTEM_HPRECOVER;
                    resultlist.push_back(result_ab);
                }
            }
        }
        // ǉʖ
//g_Logger.DebugLog((boost::format("attr=%s systemeffect=%d value=%d") % attr.m_name % attr.m_systemeffect % result.m_value).str());
        if(attr.m_systemeffect > SYSTEM_NOEFFECT && attr.m_systemeffect != SYSTEM_CRASHWALL){
            if((defchara.m_resistance[spe.m_attribute] == RES_STRONG || defchara.m_resistance[spe.m_attribute] == RES_ABSORB) &&
               attr.m_systemeffect != SYSTEM_ABSORB){
                // ̏ꍇ͒ǉʂȂBz͕ʁiWIłĂz͍sjB
                result.m_systemeffect = SYSTEM_NOEFFECT;
            }
            else if(spe.m_hitformula2 == FORMULA_ANYTIME || defchara.m_resistance[spe.m_attribute] == RES_WEAK){
                result.m_systemeffect = attr.m_systemeffect;
            }
            else{
                // vZB
                bool effect_hit = false;
                if(spe.m_hitjudgetype == HITJUDGE_SINGLE){
                    effect_hit = true;
                }
                else if((spe.m_hitjudgetype == HITJUDGE_DOUBLE && result.m_hit) || spe.m_hitjudgetype == HITJUDGE_SEPARATE){
                    int hit2 = (*Formulas[spe.m_hitformula2])(*m_Map, atkchara, defchara, spe, powerval, isCure);
                    effect_hit = (mtrand_n(100) < hit2);
                }
                if(effect_hit){
                    result.m_systemeffect = attr.m_systemeffect;
                }
                else{
                    result.m_systemeffect = SYSTEM_NOEFFECT;
                }
//                int hit2 = (*Formulas[spe.m_hitformula2])(*m_Map, atkchara, defchara, spe, powerval, isCure);
//                if(mtrand_n(100) < hit2){
//                    result.m_systemeffect = attr.m_systemeffect;
//                }
//                else{
//                    result.m_systemeffect = SYSTEM_NOEFFECT;
//                }
            }
        }
        else{
            result.m_systemeffect = SYSTEM_NOEFFECT;
        }
//g_Logger.DebugLog((boost::format("effect=%d")%result.m_systemeffect).str());
        result.m_location = *itr;
        resultlist.push_back(result);
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::EvalRevive(int master, const TBattleCharacter& atkcharadata, const std::vector<point_t>& targetlist, int special, std::vector<TEffectResult>& resultlist) const
{
g_Logger.DebugLog((boost::format("EvalRevive")).str());
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TSpecial& spe = data->GetSpecial(special);
    const TRevive& rev = data->GetRevive(spe.m_revive);
    std::vector<point_t> plist;
    std::vector<std::pair<int, int> > indexlist;
    int charaindex = -1;
    // ʔ͈͂ƌʑΏۃL͕ʂƂlŔCӂ̍Wɕ\ɂB
    switch(spe.m_rangetype){
    case RANGE_LASTDEADMYSIDE:   // ŌɎS̍W
g_Logger.DebugLog((boost::format("EvalRevive 1")).str());
        GetLastDeadMySidePointList(master, plist, indexlist);
        break;
    case RANGE_LASTDEADENEMY:    // ŌɎSG̍W
g_Logger.DebugLog((boost::format("EvalRevive 2")).str());
        GetLastDeadEnemyPointList(master, plist, indexlist);
        break;
    case RANGE_ALLDEADMYSIDE:    // SS̍W
g_Logger.DebugLog((boost::format("EvalRevive 3")).str());
        GetAllDeadMySidePointList(master, plist, indexlist);
        break;
    case RANGE_ALLDEADENEMY:     // SGS̍W
g_Logger.DebugLog((boost::format("EvalRevive 4")).str());
        GetAllDeadEnemyPointList(master, plist, indexlist);
        break;
    default:
g_Logger.DebugLog((boost::format("EvalRevive 5")).str());
        // ̑̎˒^CvŌʔ͈͂߂ꍇAΏۂƂ݂ȂB
        GetAllDeadMySidePointList(master, plist, indexlist);
        /*if(rev.m_selectmethod == REVIVE_SAME){
            GetLastDeadMySidePointList(master, plist, indexlist);
        }
        else{
            GetLastDeadEnemyPointList(master, plist, indexlist);
        }*/
        break;
    }
    // ΏۂΉȂB
    if(indexlist.empty()){
g_Logger.DebugLog((boost::format("EvalRevive empty %d %d")%m_DeadList[1].size()%master).str());
        return;
    }
    //int revcount = 0;
    std::vector<point_t> revplist;    // ʂɐݒ肵W̃XgB
    for(std::vector<point_t>::const_iterator itr = targetlist.begin(); itr != targetlist.end(); ++itr){
        //if(revcount >= indexlist.size()){
        //    break;
        //}
        if(plist.empty()){
            break;
        }
        if(m_MasterLayer.GetValue(itr->x, itr->y) != -1){
            continue;
        }
        // RANGE_ALLDEADMYSIDE / RANGE_ALLDEADENEMYł͍Wd邱Ƃ̂ŁB
        if(std::find(revplist.begin(), revplist.end(), *itr) != revplist.end()){
            continue;
        }
        int revindex = 0;
        std::vector<point_t>::iterator itr_loc = std::find(plist.begin(), plist.end(), *itr);
        if(itr_loc != plist.end()){
            revindex = std::distance(plist.begin(), itr_loc);
        }
g_Logger.DebugLog((boost::format("EvalRevive 1")).str());
        TEffectResult result;
        result.m_location = *itr;
        result.m_hit = true;
        result.m_special = special;
        result.m_systemeffect = SYSTEM_REVIVE;
        result.m_revivecharamaster = indexlist[revindex].first;
        result.m_revivecharaindex = indexlist[revindex].second;
        if(rev.m_selectmethod == REVIVE_SAME){
g_Logger.DebugLog((boost::format("EvalRevive 2")).str());
            // ̂܂ܕB
            TCharacterInstance& revchara = (*m_CharaList[result.m_revivecharamaster])[result.m_revivecharaindex];
            result.m_revivecharatype = revchara.m_type;
            result.m_value = (revchara.m_hpmax_rank + 1) / 2;
        }
        else if(rev.m_selectmethod == REVIVE_EMPLOYABLE){
g_Logger.DebugLog((boost::format("EvalRevive 3")).str());
            // ٗp\LIB
            if(atkcharadata.m_employable.empty()){
                OFERRLOG("ΏۂɌٗp\Lw肳ꂽǁÃL͌ٗp\ȃL̂őŃNbV܂B");
            }
            int i = mtrand_n(atkcharadata.m_employable.size());
            result.m_revivecharatype = atkcharadata.m_employable[i];
            const TBattleCharacter& charadata = data->GetCharacter(result.m_revivecharatype);
            result.m_value = charadata.m_hpmax;
        }
        else if(rev.m_selectmethod == REVIVE_FROMLIST){
g_Logger.DebugLog((boost::format("EvalRevive 4")).str());
            // w胊XgIB
            int i = mtrand_n(rev.m_charalist.size());
            result.m_revivecharatype = rev.m_charalist[i];
            const TBattleCharacter& charadata = data->GetCharacter(result.m_revivecharatype);
            result.m_value = charadata.m_hpmax;
        }
g_Logger.DebugLog((boost::format("EvalRevive 5")).str());
        resultlist.push_back(result);
        //++revcount;
        plist.erase(plist.begin() + revindex);
        indexlist.erase(indexlist.begin() + revindex);
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::ShowSpecial(const TCharacterInstance& atkchara, const std::vector<point_t>& targetlist, int special, int frame, std::deque<bool>& evaltiming, bool& end) const
{
    m_BattleMapWindow->GetBattleMapPane().ClearEffectImage();
    evaltiming.clear();
    // ڕWW𒊏oB
    std::vector<point_t> targetlist2;
    std::deque<bool> dmyeval(targetlist.size(), true);
    GetEffectTargetList(targetlist, dmyeval, m_ActiveMaster, special, targetlist2);
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TSpecial& spe = data->GetSpecial(special);
    // ϐȗA摜ꍇ2t[ŏIB
    if(m_NoWatch || spe.m_imagefile.empty()){
        // ʂ̍ŒɊϐȗɂȂꍇA^C~OɂĂ͂̌ʂKpȂƂlB
        // ϐȗ悤Ȑ퓬ł͂̒x͌덷͈͓̔Ƃ邩H
        end = (frame >= 1);
        if(!end){
            evaltiming.assign(targetlist.size(), true);
        }
        return;
    }
    const TOpenFarenImage* image = g_GameMaster.GetImage();
    int imgw, imgh;
    image->GetEffectSize(spe.m_image, imgw, imgh);
    int x, y;
    int anim, animmax;
    int i;
    switch(spe.m_animtype){
    case ANIMATION_NONE:
        break;
    case ANIMATION_CHIP:
        end = (frame == m_AnimationSpeed * spe.m_animcount);
        if(end){
            return;
        }
        anim = frame/m_AnimationSpeed;
        animmax = imgw / spe.m_animwidth;
        if(anim < animmax){
            for(std::vector<point_t>::const_iterator itr = targetlist2.begin(); itr != targetlist2.end(); ++itr){
                x = LOGIC_TO_PHY_X(itr->x, itr->y);
                y = itr->y * TOpenFarenImage::MAPCHIP_HEIGHT;
                m_BattleMapWindow->GetBattleMapPane().AddEffectImage(x, y, spe.m_image, anim, 0, spe.m_animwidth, spe.m_animheight);
            }
        }
        if(frame == m_AnimationSpeed * spe.m_animcount - 1){
            evaltiming.assign(targetlist.size(), true);
        }
        break;
    case ANIMATION_CENTER:
        end = (frame == m_AnimationSpeed * spe.m_animcount);
        if(end){
            return;
        }
        anim = frame/m_AnimationSpeed;
        animmax = imgw / spe.m_animwidth;
        if(anim >= animmax){
            anim = animmax - 1;
        }
        x = LOGIC_TO_PHY_X(targetlist[0].x, targetlist[0].y) + TOpenFarenImage::MAPCHIP_WIDTH / 2 - spe.m_animwidth / 2;
        y = targetlist[0].y * TOpenFarenImage::MAPCHIP_HEIGHT + TOpenFarenImage::MAPCHIP_HEIGHT / 2 - spe.m_animheight / 2;
        m_BattleMapWindow->GetBattleMapPane().AddEffectImage(x, y, spe.m_image, anim, 0, spe.m_animwidth, spe.m_animheight);
        if(frame == m_AnimationSpeed * spe.m_animcount - 1){
            evaltiming.assign(targetlist.size(), true);
        }
        break;
    case ANIMATION_STREAM:
        end = (frame == m_AnimationSpeed * (spe.m_animcount + spe.m_maxradius));
        if(end){
            return;
        }
        i = 0;
        evaltiming.resize(targetlist.size());
        for(std::vector<point_t>::const_iterator itr = targetlist.begin(); itr != targetlist.end(); ++itr){
            x = LOGIC_TO_PHY_X(itr->x, itr->y);
            y = itr->y * TOpenFarenImage::MAPCHIP_HEIGHT;
            anim = frame/m_AnimationSpeed - i;
            if(anim >= 0 && anim < spe.m_animcount){
                m_BattleMapWindow->GetBattleMapPane().AddEffectImage(x, y, spe.m_image, anim, 0, spe.m_animwidth, spe.m_animheight);
            }
            // ꂼ̃`bṽAj[V̍ŌŃ_[WB
            evaltiming[i] = frame == m_AnimationSpeed * (spe.m_animcount + i) - 1;
            ++i;
        }
        break;
    case ANIMATION_SHOT:
        {
            int maxdist = GetAnimationDistanceMax(atkchara.m_location, targetlist);
            maxdist /= TOpenFarenImage::MAPCHIP_HEIGHT / m_AnimationPerChip;
            end = (frame == m_AnimationSpeed * maxdist);
            if(end){
                return;
            }
            evaltiming.resize(targetlist.size());
            int fromx = LOGIC_TO_PHY_X(atkchara.m_location.x, atkchara.m_location.y);
            int fromy = atkchara.m_location.y * TOpenFarenImage::MAPCHIP_HEIGHT;
            for(std::vector<point_t>::const_iterator itr = targetlist2.begin(); itr != targetlist2.end(); ++itr){
                int dist = GetAnimationDistance(atkchara.m_location, *itr);
                dist /= TOpenFarenImage::MAPCHIP_HEIGHT / m_AnimationPerChip;
                if(frame < m_AnimationSpeed * dist){
                    x = LOGIC_TO_PHY_X(itr->x, itr->y);
                    y = itr->y * TOpenFarenImage::MAPCHIP_HEIGHT;
                    anim = frame/m_AnimationSpeed;
                    int shotx = ((fromx * (dist - anim)) + (x * anim)) / dist;
                    int shoty = ((fromy * (dist - anim)) + (y * anim)) / dist;
                    int dir = GetAnimationDirection(point_t(fromx, fromy), point_t(x, y));
                    m_BattleMapWindow->GetBattleMapPane().AddEffectImage(shotx, shoty, spe.m_image, anim, dir, spe.m_animwidth, spe.m_animheight);
                    // ꂼ̖̍Ō̃Aj[VŃ_[WB
                    //evaltiming.push_back(frame == m_AnimationSpeed * dist - 1);
                    bool eval = frame == m_AnimationSpeed * dist - 1;
                    if(eval){
                        // Ԕtargetlisttargetlist2ŗvfႤ̂ŌăCfbNX߂B
                        int index = std::distance(targetlist.begin(), std::find(targetlist.begin(), targetlist.end(), *itr));
                        evaltiming[index] = true;
                    }
                }
            }
        }
        break;
    case ANIMATION_SCREENCENTER:
        end = (frame == m_AnimationSpeed * spe.m_animcount);
        if(end){
            m_AnimationPane->setVisible(false);
            return;
        }
        m_AnimationPane->SetAnimation(spe.m_image, spe.m_animwidth, spe.m_animheight, frame/m_AnimationSpeed);
        g_GuiContainerList.MoveWidgetToCenter(m_AnimationPane);
        gcn::Container* top = static_cast<gcn::Container*>(g_GuiContainerList.GetGui()->getTop());
        top->moveToTop(m_AnimationPane);
        m_AnimationPane->setVisible(true);
        if(frame == m_AnimationSpeed * spe.m_animcount - 1){
            evaltiming.assign(targetlist.size(), true);
        }
        break;
    }
}
//---------------------------------------------------------------------------
// ݂̏ԂŎgp\ȍs{^𒲂ׂăR}hEBhEXVB
void TBattleMapLogic::UpdateActionCommandWindow(bool moved)
{
    // \B
    std::vector<std::string> strs;
    strs.resize(TOpenFarenImage::BUTTONIMAGE_MAX);
    strs[TOpenFarenImage::BUTTONIMAGE_WAIT] = "ҋ@";
    strs[TOpenFarenImage::BUTTONIMAGE_MOVE] = "ړ";
    strs[TOpenFarenImage::BUTTONIMAGE_ATTACK]  = "U";
    strs[TOpenFarenImage::BUTTONIMAGE_MAGIC]   = "@";
    strs[TOpenFarenImage::BUTTONIMAGE_CHARGE]  = "ˌ";
    strs[TOpenFarenImage::BUTTONIMAGE_RUNAWAY] = "";

    const TCharacterInstance& c = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TBattleCharacter& charadata = data->GetCharacter(c.m_type);
    // ͂̓GƍU𒲂ׂB
    bool existsEnemy = ExistsEnemyNext(m_ActiveMaster, c.m_location.x, c.m_location.y, false);
    bool hasAttack = HasAttack(charadata);
    // KEZ͏MPŔB
    bool canspe = false;
    if(charadata.m_specialindex != -1){
        const TSpecial& spe = data->GetSpecial(charadata.m_specialindex);
        canspe = c.m_mp >= spe.m_consumemp;
        strs[TOpenFarenImage::BUTTONIMAGE_SPECIAL] = spe.m_name;
    }
    m_BattleCommandWindow->GetSpecialButton().setEnabled(canspe);
    // ړ͈ړƖ@sBჂ͈ړsB
    //m_BattleCommandWindow->GetMoveButton().setEnabled(!moved && (std::find(chara.m_seslist.begin(), chara.m_seslist.end(), SYSTEM_PARALYZE) != chara.m_seslist.end()));
    m_BattleCommandWindow->GetMoveButton().setEnabled(!moved && !HasSystemEffect(c, SYSTEM_PARALYZE));
    m_BattleCommandWindow->GetMagicButton().setEnabled(!moved && HasUsableMagic(c));

    // ǔB
    bool wallatk = m_ActiveMaster && HasWallNext(c.m_location.x, c.m_location.y);
    // U\ǂB
    m_BattleCommandWindow->GetAttackButton().setEnabled(hasAttack && (existsEnemy || wallatk));
    if(moved){
        // ͖ɓˌs\B
        m_BattleCommandWindow->GetChargeButton().setEnabled(false);
    }
    else{
        // UĂėׂɓGΓˌ\B
        m_BattleCommandWindow->GetChargeButton().setEnabled(hasAttack && existsEnemy);
    }

    // Ō̍s\ǂB
    bool canredo = false;
    std::map<int, int>::const_iterator citr_lastspecial = m_LastSpecialTable[m_ActiveMaster].find(m_ActiveChara);
    if(citr_lastspecial != m_LastSpecialTable[m_ActiveMaster].end()){
        const TSpecial& lastspe = data->GetSpecial(citr_lastspecial->second);
        strs[TOpenFarenImage::BUTTONIMAGE_REDO] = (boost::format("ĎsF%s") % lastspe.m_name).str();
        if(IsAllowedRedoAction(moved, ACTION_SPECIAL, citr_lastspecial->second)){
            canredo = true;
        }
    }
    else{
        strs[TOpenFarenImage::BUTTONIMAGE_REDO] = "ĎsFȂ";
    }
    m_BattleCommandWindow->GetRedoButton().setEnabled(canredo);

    // Ō̍s\ǂBwcʔ
    bool canredo2 = false;
    if(m_LastSpecial[m_ActiveMaster] != -1){
        int s = m_LastSpecial[m_ActiveMaster];
        const TSpecial& lastspe = data->GetSpecial(s);
        strs[TOpenFarenImage::BUTTONIMAGE_REDO2] = (boost::format("ĎsF%s") % lastspe.m_name).str();
        // l̖@ZgłȂ悤ɃK[hB
        if(IsUsableMagic(c, s) || s == charadata.m_specialindex){
            if(IsAllowedRedoAction(moved, ACTION_SPECIAL, s)){
                canredo2 = true;
            }
        }
    }
    else{
        strs[TOpenFarenImage::BUTTONIMAGE_REDO2] = "ĎsFȂ";
    }
    m_BattleCommandWindow->GetRedo2Button().setEnabled(canredo2);

    // \B
    m_BattleCommandWindow->GetRunAwayButton().setEnabled(CanRunAway(m_ActiveMaster, m_ActiveChara, c.m_location.x, c.m_location.y));
    m_BattleCommandWindow->SetCommandStrings(strs);
}
//---------------------------------------------------------------------------
void TBattleMapLogic::UpdateShowingChara(int masterindex, int charaindex)
{
    const TCharacterInstance& c = (*m_CharaList[masterindex])[charaindex];
    TShowingChara showchara;
    std::memset(&showchara, 0, sizeof showchara);
    showchara.m_type = c.m_type;
    showchara.m_location1 = c.m_location;
    showchara.m_markchip = (masterindex != 0);
    m_BattleMapWindow->GetBattleMapPane().SetShowingChara(c.m_showingindex, showchara);
}
//---------------------------------------------------------------------------
void TBattleMapLogic::UpdateShowingChara()
{
    m_BattleMapWindow->GetBattleMapPane().ClearShowingChara();
    for(unsigned u=0; u<m_CharaList.size(); ++u){
        for(unsigned chr=0; chr<m_CharaList[u]->size(); ++chr){
            if((*m_CharaList[u])[chr].m_hp && (*m_CharaList[u])[chr].m_location.x != -1){
                TCharacterInstance & ci = (*m_CharaList[u])[chr];
                TShowingChara c;
                std::memset(&c, 0, sizeof c);
                c.m_type = ci.m_type;
                c.m_location1 = ci.m_location;
                c.m_markchip = (u != 0);
                ci.m_showingindex = m_BattleMapWindow->GetBattleMapPane().AddShowingChara(c);
            }
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::UpdateForceBrave()
{
    int lforce=0, lbrave=0, rforce=0, rbrave=0;
    int rcnt, lcnt;

    // U
    lcnt = m_CharaList[1]->size();
    if(lcnt){
        for(unsigned chr=0; chr<lcnt; ++chr){
            const TCharacterInstance& ci = (*m_CharaList[1])[chr];
            if(ci.m_hp && !ci.m_runaway){
                lforce += (ci.m_power * ci.m_hp) / ci.m_hpmax_rank;
                lbrave += ci.m_brave;
            }
        }
        lbrave /= lcnt;
    }
    // 
    rcnt = m_CharaList[0]->size();
    if(rcnt){
        for(unsigned chr=0; chr<rcnt; ++chr){
            const TCharacterInstance& ci = (*m_CharaList[0])[chr];
            if(ci.m_hp && !ci.m_runaway){
                rforce += (ci.m_power * ci.m_hp) / ci.m_hpmax_rank;
                rbrave += ci.m_brave;
            }
        }
        rbrave /= rcnt;
    }
    m_BattleMapWindow->GetBattleTopPane().SetProperty(lforce, lbrave, rforce, rbrave, m_Turn);
    m_BattleCharaListWindow->GetPane().SetProperty(lforce, lbrave, rforce, rbrave, m_Turn);
    //OFDBGLOG((boost::format("FORCE: L %d  R %d") % lforce % rforce).str());
}
//---------------------------------------------------------------------------
void TBattleMapLogic::UpdateSideName()
{
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TMasterInstance& lmaster = game->GetMasterInstance(m_MasterIndex[1]);
    const TMaster& lmasterdata = data->GetMaster(lmaster.m_type);
    std::string rname;
    if(m_MasterIndex[0] != -1){
        const TMasterInstance& rmaster = game->GetMasterInstance(m_MasterIndex[0]);
        const TMaster& rmasterdata = data->GetMaster(rmaster.m_type);
        rname = rmasterdata.m_name + "R";
    }
    else{
        rname = "R";
    }
    const TAreaInstance& area = game->GetAreaInstance(m_AreaIndex);
    m_BattleMapWindow->GetBattleTopPane().SetString(area.m_name, lmasterdata.m_name + "R", rname);
}
//---------------------------------------------------------------------------
void TBattleMapLogic::HighlightCircle(int x, int y)
{
    m_ShadeLayer.Init(1);
    GetNormalRange(m_ShadeLayer, x, y, 1, 1);
}
//---------------------------------------------------------------------------
void TBattleMapLogic::ShowCharaDetail(int masterindex, int charaindex)
{
    m_CharaStatusFullWindow->Show((*m_CharaList[masterindex])[charaindex]);
    g_GuiContainerList.MoveWidgetToCenter(m_CharaStatusFullWindow);
    g_GuiContainerList.GetGui()->getTop()->moveToTop(m_CharaStatusFullWindow);
    m_ShownCharaMaster = masterindex;
    m_ShownChara = charaindex;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::HideCharaDetail()
{
    m_CharaStatusFullWindow->setVisible(false);
    m_ShownCharaMaster = -1;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::UpdateCharaDetail()
{
    if(m_ShownCharaMaster != -1 && m_CharaStatusFullWindow->isVisible()){
        const TCharacterInstance& charains = (*m_CharaList[m_ShownCharaMaster])[m_ShownChara];
        if(charains.m_hp){
            m_CharaStatusFullWindow->Show(charains);
        }
        else{
            HideCharaDetail();
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::ShowBattleCharaList(bool canHide)
{
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    for(int index=0; index<m_CharaList.size(); ++index){
/*
        const TCharacterInstanceList& src = *m_CharaList[index];
        int sz = src.size();
        std::vector<const TCharacterInstance*> charalist(sz);
        for(int i=0; i<sz; ++i){
            charalist[i] = &src[i];
        }
*/
        int mtype;
        if(m_MasterIndex[index] != -1){
            mtype = game->GetMasterInstance(m_MasterIndex[index]).m_type;
        }
        else{
            mtype = -1;
        }
        m_BattleCharaListWindow->GetPane().SetCharaList(index, mtype, m_CharaList[index]);
    }
    m_BattleCharaListWindow->SetCanHide(canHide);
    m_BattleCharaListWindow->setVisible(true);
    g_GuiContainerList.MoveWidgetToCenter(m_BattleCharaListWindow);
    g_GuiContainerList.GetGui()->getTop()->moveToTop(m_BattleCharaListWindow);
}
//---------------------------------------------------------------------------
void TBattleMapLogic::HideBattleCharaList()
{
    m_BattleCharaListWindow->GetPane().ClearCharaList();
    m_BattleCharaListWindow->setVisible(false);
}
//---------------------------------------------------------------------------
// ̑I𑀍s߂̑I\͈͂layerɎ擾B擾ꍇAtrueBKvȂ̂Ŏ擾ȂꍇAfalseB
bool TBattleMapLogic::GetRange(TValueLayer<int>& layer, int x, int y, int rangetype, int range, int radius, int reverse) const
{
    switch(rangetype){
    case 0:
        g_Logger.ErrorLog("TBattleMapLogic::GetRange() parameter error:undefined rangetype");
        return false;
    case RANGE_NORMAL:
        GetNormalRange(layer, x, y, range, reverse);
        return true;
    case RANGE_ME:
        return false;
    case RANGE_STREAM:
        GetStreamRange(layer, x, y, radius, reverse);
        return true;
    case RANGE_ALLMYSIDE:
    case RANGE_ALLENEMY:
    case RANGE_ALLWALL:
    case RANGE_LASTDEADMYSIDE:   // ŌɎS̍W
    case RANGE_LASTDEADENEMY:    // ŌɎSG̍W
    case RANGE_ALLDEADMYSIDE:    // SS̍W
    case RANGE_ALLDEADENEMY:     // SGS̍W
    case RANGE_LOWERTHAN:        // ፂx̒n`
        return false;
    }
    return false;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetNormalRange(TValueLayer<int>& layer, int x, int y, int range, int reverse) const
{
    layer.Init(reverse);
    if(range > TNeighbor::NEIGHBOR_MAX){
        range = TNeighbor::NEIGHBOR_MAX;
    }
    int from = TNeighbor::GetNeighborIndex(0);
    int to = TNeighbor::GetNeighborIndex(range + 1);
    int mode = y%2;
    for(int i=from; i<to; ++i){
        point_t p = TNeighbor::GetNeighbor(mode, i);
        int nextx = x + p.x;
        int nexty = y + p.y;
        if(nexty < 0 || nexty >= m_Map->GetHeight()){
            continue;
        }
        if(nexty % 2){
            if(nextx < 0 || nextx >= m_Map->GetWidth() - 1){
                continue;
            }
        }
        else{
            if(nextx < 1 || nextx >= m_Map->GetWidth()){
                continue;
            }
        }
        layer.GetValue(nextx, nexty) = !reverse;
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetStreamRange(TValueLayer<int>& layer, int x, int y, int radius, int reverse) const
{
    layer.Init(reverse);
    int mode = y%2;
    for(int dir=0; dir<6; ++dir){
        int nextx = x;
        int nexty = y;
        for(int i=0; i<=radius; ++i){
            nextx += TNeighbor::NextDX[nexty%2][dir];
            nexty += TNeighbor::NextDY[dir];
            if(IsValidXY(nextx, nexty)){
                layer.GetValue(nextx, nexty) = !reverse;
            }
        }
    }
}
//---------------------------------------------------------------------------
// }EXŃ|CgWw肵̕߂B
int TBattleMapLogic::GetStreamDirection(int fromx, int fromy, int x, int y) const
{
    int dx = x - fromx;
    int dy = fromy - y;
    if(!dy){
        if(dx > 0){
            return 2;
        }
        else{
            return 5;
        }
    }
    else{
        if(dy > 0){
            if(dx > 0){
                return 1;
            }else{
                return 0;
            }
        }
        else{
            if(dx > 0){
                return 3;
            }else{
                return 4;
            }
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GenerateEffectPointList(point_t from, point_t to, int specialindex)
{
    m_EffectTo.clear();
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TSpecial& spe = data->GetSpecial(specialindex);
    GetEffectPointList(from, to, spe.m_maxradius, spe.m_rangetype, spe.m_rangeheight, m_EffectTo);
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetEffectPointList(point_t from, point_t to, int radius, TRangeType rangetype, int rangeheight, std::vector<point_t>& plist) const
{
    std::vector<std::pair<int, int> > indexlist;
    switch(rangetype){
    case 0:
        break;
    case RANGE_NORMAL:
        GetNormalPointList(to.x, to.y, radius, plist);
        break;
    case RANGE_ME:
        GetNormalPointList(from.x, from.y, radius, plist);
        break;
    case RANGE_STREAM:
        GetStreamPointList(from.x, from.y, radius, GetStreamDirection(from.x, from.y, to.x, to.y), plist);
        break;
    case RANGE_ALLMYSIDE:
        GetAllMySidePointList(m_ActiveMaster, plist);
        break;
    case RANGE_ALLENEMY:
        GetAllEnemyPointList(m_ActiveMaster, plist);
        break;
    case RANGE_ALLWALL:
        GetWallPointList(plist);
        break;
    case RANGE_LASTDEADMYSIDE:   // ŌɎS̍W
        GetLastDeadMySidePointList(m_ActiveMaster, plist, indexlist);
        break;
    case RANGE_LASTDEADENEMY:    // ŌɎSG̍W
        GetLastDeadEnemyPointList(m_ActiveMaster, plist, indexlist);
        break;
    case RANGE_ALLDEADMYSIDE:    // SS̍W
        GetAllDeadMySidePointList(m_ActiveMaster, plist, indexlist);
        break;
    case RANGE_ALLDEADENEMY:     // SGS̍W
        GetAllDeadEnemyPointList(m_ActiveMaster, plist, indexlist);
        break;
    case RANGE_LOWERTHAN:        // ፂx̒n`
        GetLowerThanPointList(rangeheight, plist);
        break;
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetNormalPointList(int x, int y, int radius, std::vector<point_t>& plist) const
{
    int from = TNeighbor::GetNeighborIndex(0);
    int to = TNeighbor::GetNeighborIndex(radius + 1);
    int mode = y%2;
    for(int i=from; i<to; ++i){
        point_t p = TNeighbor::GetNeighbor(mode, i);
        int nextx = x + p.x;
        int nexty = y + p.y;
        if(IsValidXY(nextx, nexty)){
            plist.push_back(point_t(nextx, nexty));
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetStreamPointList(int x, int y, int radius, int dir, std::vector<point_t>& plist) const
{
    int nextx = x;
    int nexty = y;
    for(int i=0; i<=radius; ++i){
        nextx += TNeighbor::NextDX[nexty%2][dir];
        nexty += TNeighbor::NextDY[dir];
        if(IsValidXY(nextx, nexty)){
            plist.push_back(point_t(nextx, nexty));
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetAllMySidePointList(int master, std::vector<point_t>& plist) const
{
    for(unsigned u=0; u<(*m_CharaList[master]).size(); ++u){
        plist.push_back((*m_CharaList[master])[u].m_location);
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetAllEnemyPointList(int master, std::vector<point_t>& plist) const
{
    for(unsigned m=0; m<m_CharaList.size(); ++m){
        if(m != master){
            for(unsigned u=0; u<(*m_CharaList[m]).size(); ++u){
                plist.push_back((*m_CharaList[m])[u].m_location);
            }
        }
    }
}
//---------------------------------------------------------------------------
// SĂ̕ǂ̍W𒊏oB
void TBattleMapLogic::GetWallPointList(std::vector<point_t>& plist) const
{
//g_Logger.DebugLog((boost::format("GetWallRange")).str());
    int sz = m_Map->GetWallCount();
    for(int i=0; i<sz; ++i){
        point_t p = m_Map->GetWallXY(i);
        if(m_Map->GetMapObjType(p.x, p.y) == MAPOBJ_WALL){
            plist.push_back(p);
//g_Logger.DebugLog((boost::format("GetWallRange %d %d") % p.x % p.y).str());
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetLastDeadMySidePointList(int master, std::vector<point_t>& plist, std::vector<std::pair<int, int> >& indexlist) const
{
    if(!m_DeadList[master].empty()){
        int charaindex = *(m_DeadList[master].end() - 1);
        TCharacterInstance& chara = (*m_CharaList[master])[charaindex];
        plist.push_back(chara.m_location);
        indexlist.push_back(std::pair<int ,int>(master, charaindex));
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetLastDeadEnemyPointList(int master, std::vector<point_t>& plist, std::vector<std::pair<int, int> >& indexlist) const
{
    for(unsigned m = 0; m < m_DeadList.size(); ++m){
        if(m != master && !m_DeadList[m].empty()){
            int charaindex = *(m_DeadList[m].end() - 1);
            TCharacterInstance& chara = (*m_CharaList[m])[charaindex];
            plist.push_back(chara.m_location);
            indexlist.push_back(std::pair<int ,int>(m, charaindex));
            break;
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetAllDeadMySidePointList(int master, std::vector<point_t>& plist, std::vector<std::pair<int, int> >& indexlist) const
{
    // W̏d͂ł͍lSB
    for(unsigned u = 0; u < m_DeadList[master].size(); ++u){
        TCharacterInstance& chara = (*m_CharaList[master])[m_DeadList[master][u]];
        plist.push_back(chara.m_location);
        indexlist.push_back(std::pair<int ,int>(master, u));
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetAllDeadEnemyPointList(int master, std::vector<point_t>& plist, std::vector<std::pair<int, int> >& indexlist) const
{
    for(unsigned m = 0; m < m_DeadList.size(); ++m){
        if(m != master){
            // W̏d͂ł͍lSB
            for(unsigned u = 0; u < m_DeadList[m].size(); ++u){
                TCharacterInstance& chara = (*m_CharaList[m])[m_DeadList[m][u]];
                plist.push_back(chara.m_location);
                indexlist.push_back(std::pair<int ,int>(m, u));
            }
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::GetLowerThanPointList(int rangeheight, std::vector<point_t>& plist) const
{
    int w = m_Map->GetWidth();
    int h = m_Map->GetHeight();
    for(int y=0; y<h; ++y){
        for(int x=0; x<w; ++x){
            if(m_Map->GetMapHeight(x, y) < rangeheight){
                plist.push_back(point_t(x, y));
            }
        }
    }
}
//---------------------------------------------------------------------------
// ʑΏۂ̃LW𒊏oBmasterindex͏p҂̃}X^[B
void TBattleMapLogic::GetEffectTargetList(const std::vector<point_t>& src, const std::deque<bool>& evaltiming, int masterindex, int special, std::vector<point_t>& plist) const
{
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TSpecial& spe = data->GetSpecial(special);
    const TAttribute& attr = data->GetAttribute(spe.m_attribute);
    std::deque<bool>::const_iterator eitr = evaltiming.begin();
    int master;
    switch(attr.m_type){
    case ATTRTYPE_NONE:
        break;
    case ATTRTYPE_ATTACK:        // GɓZ
        for(std::vector<point_t>::const_iterator itr = src.begin(); itr != src.end(); ++itr){
            if(*eitr){
                master = m_MasterLayer.GetValue(itr->x, itr->y);
                if(master != -1 && master != masterindex){
                    if(spe.m_target == -1){
                        plist.push_back(*itr);
                    }
                    else{
                        // ^[Qbgw肳Ăꍇ̓L^[Qbg肷B
                        int charaindex = m_CharaLayer.GetValue(itr->x, itr->y);
                        const TCharacterInstance& chara = (*m_CharaList[master])[charaindex];
                        const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
                        if(spe.m_target == charadata.m_target){
                            plist.push_back(*itr);
                        }
                    }
                }
            }
            ++eitr;
        }
        break;
    case ATTRTYPE_BOOST:         // ɓZ
        for(std::vector<point_t>::const_iterator itr = src.begin(); itr != src.end(); ++itr){
            if(*eitr){
                master = m_MasterLayer.GetValue(itr->x, itr->y);
                if(master != -1 && master == masterindex){
                    if(spe.m_target == -1){
                        plist.push_back(*itr);
                    }
                    else{
                        // ^[Qbgw肳Ăꍇ̓L^[Qbg肷B
                        int charaindex = m_MasterLayer.GetValue(itr->x, itr->y);
                        const TCharacterInstance& chara = (*m_CharaList[master])[charaindex];
                        const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
                        if(spe.m_target == charadata.m_target){
                            plist.push_back(*itr);
                        }
                    }
                }
            }
            ++eitr;
        }
        break;
    case ATTRTYPE_CRASHWALL:     // ǂɓZ
        for(std::vector<point_t>::const_iterator itr = src.begin(); itr != src.end(); ++itr){
            if(*eitr){
                if(m_Map->GetMapObjType(itr->x, itr->y) == MAPOBJ_WALL){
                    plist.push_back(*itr);
                }
            }
            ++eitr;
        }
        break;
    case ATTRTYPE_SUMMON:        // GȂɓZ
        for(std::vector<point_t>::const_iterator itr = src.begin(); itr != src.end(); ++itr){
            if(*eitr){
                master = m_MasterLayer.GetValue(itr->x, itr->y);
                if(master == -1){
                    plist.push_back(*itr);
                }
            }
            ++eitr;
        }
        break;
    default:
        for(std::vector<point_t>::const_iterator itr = src.begin(); itr != src.end(); ++itr){
            if(*eitr){
                plist.push_back(*itr);
            }
            ++eitr;
        }
        break;
    }
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::CanEffect(point_t from, point_t to, int masterindex, int specialindex) const
{
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TSpecial& spe = data->GetSpecial(specialindex);
    //const TAttribute& attr = data->GetAttribute(spe.m_attribute);
    std::vector<point_t> plist;
    GetEffectPointList(from, to, spe.m_maxradius, spe.m_rangetype, 0, plist);
    if(plist.empty()){
        return false;
    }
    std::vector<point_t> targetlist;
    std::deque<bool> evaltiming(plist.size(), true);
    GetEffectTargetList(plist, evaltiming, masterindex, specialindex, targetlist);
    return !targetlist.empty();
}
//---------------------------------------------------------------------------
// A|^CṽAj[Ṽt[Ɏgp鋗߂B
int TBattleMapLogic::GetAnimationDistance(point_t from, point_t to) const
{
    int fromx = LOGIC_TO_PHY_X(from.x, from.y);
    int fromy = from.y * TOpenFarenImage::MAPCHIP_HEIGHT;
    int tox = LOGIC_TO_PHY_X(to.x, to.y);
    int toy = to.y * TOpenFarenImage::MAPCHIP_HEIGHT;
    int dx = fromx - tox;
    int dy = fromy - toy;
    return std::sqrt(static_cast<double>(dx * dx + dy * dy));    // IWiłsqrtŋ߂Ă݂܂B
}
//---------------------------------------------------------------------------
// A|^CṽAj[V̑t[Ɏgp鋗߂B
int TBattleMapLogic::GetAnimationDistanceMax(point_t from, const std::vector<point_t>& tolist) const
{
    int maxdist = 0;
    for(std::vector<point_t>::const_iterator itr = tolist.begin(); itr != tolist.end(); ++itr){
        int dist = GetAnimationDistance(from, *itr);
        if(maxdist < dist){
            maxdist = dist;
        }
    }
    return maxdist;
}
//---------------------------------------------------------------------------
// ΂Ȃǂ̕߂B
int TBattleMapLogic::GetAnimationDirection(point_t from, point_t to) const
{
    int dx = to.x - from.x;
    int dy = from.y - to.y; // lÂ炢̂ŉYW͔]B
    if(dx > 0){
        if(dy > 0){
            if(dy > 2*dx){
                return 0;
            }
            else if(dy < dx/2){
                return 2;
            }
            else{
                return 1;
            }
        }
        else{
            if(dy > -dx/2){
                return 2;
            }
            else if(dy < -2*dx){
                return 4;
            }
            else{
                return 3;
            }
        }
    }
    else{
        if(dy > 0){
            if(dy > -2*dx){
                return 0;
            }
            else if(dy < -dx/2){
                return 6;
            }
            else{
                return 7;
            }
        }
        else{
            if(dy < 2*dx){
                return 4;
            }
            else if(dy > dx/2){
                return 6;
            }
            else{
                return 5;
            }
        }
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::SetRadiusChip(int x, int y, int radius, const gcn::Color& c)
{
    m_BattleMapWindow->GetBattleMapPane().ClearRadiusChip();
    int from = TNeighbor::GetNeighborIndex(0);
    int to = TNeighbor::GetNeighborIndex(radius + 1);
    int mode = y%2;
    for(int i=from; i<to; ++i){
        point_t p = TNeighbor::GetNeighbor(mode, i);
        int nextx = x + p.x;
        int nexty = y + p.y;
        m_BattleMapWindow->GetBattleMapPane().AddRadiusChip(nextx, nexty, c);
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::SetStreamChip(int x, int y, int radius, int dir, const gcn::Color& c)
{
    m_BattleMapWindow->GetBattleMapPane().ClearRadiusChip();
    int nextx = x;
    int nexty = y;
    for(int i=0; i<=radius; ++i){
        nextx += TNeighbor::NextDX[nexty%2][dir];
        nexty += TNeighbor::NextDY[dir];
        m_BattleMapWindow->GetBattleMapPane().AddRadiusChip(nextx, nexty, c);
    }
}
//---------------------------------------------------------------------------
void TBattleMapLogic::SetEffectString(const std::vector<TEffectResult>& results)
{
    static const gcn::Color damagecolor[SYSTEM_EFFECTMAX] = {
        gcn::Color(255,255,255),    // ʏ
        gcn::Color(224,0,0),        // 
        gcn::Color(192,192,192),    // Ή
        gcn::Color(224,224,0),      // 
        gcn::Color(224,0,224),      // z
        gcn::Color(255,0,255),      // 
        gcn::Color(255,0,0),        // 
        gcn::Color(255,96,96),       // z
        gcn::Color(64,255,255),       // HP
        gcn::Color(64,128,255)        // MP
        };

    // ϐȗ͉ȂB
    if(m_NoWatch){
        return;
    }
    const TOpenFarenData* data = g_GameMaster.GetData();
    for(unsigned u = 0; u < results.size(); ++u){
//g_Logger.DebugLog((boost::format("SetEffectString systemeffect=%d") % results[u].m_systemeffect).str());
        const point_t p = results[u].m_location;
        if(results[u].m_hit){
            if(results[u].m_special != -1){
                // zNHP񕜂͐\ȂB
                if(results[u].m_systemeffect == SYSTEM_HPRECOVER){
                    const TSpecial& spe = data->GetSpecial(results[u].m_special);
                    const TSystemEffect se = data->GetAttribute(spe.m_attribute).m_systemeffect;
                    if(se == SYSTEM_ABSORB){
                        continue;
                    }
                }
            }
            if(results[u].m_systemeffect > SYSTEM_MPRECOVER){
//g_Logger.DebugLog((boost::format("SetEffectString continue")).str());
                continue;
            }
            std::string str = (boost::format("%3d") % results[u].m_value).str();
            m_BattleMapWindow->GetBattleMapPane().AddDamageString(p.x, p.y, str, damagecolor[results[u].m_systemeffect - 1]);
        }
        else{
            m_BattleMapWindow->GetBattleMapPane().AddDamageString(p.x, p.y, "Miss", gcn::Color(255,255,255));
        }
    }
//g_Logger.DebugLog((boost::format("SetEffectString end")).str());
}
//---------------------------------------------------------------------------
void TBattleMapLogic::ApplyEffectResult(TCharacterInstance* atkchara, const std::vector<TEffectResult>& results, bool& existsDeadChara)
{
    static int TCharacterInstance::* const params[SYSTEM_PLUS_ATTACKCOUNT - SYSTEM_PLUS_PA + 1] = {
        &TCharacterInstance::m_pa_plus,
        &TCharacterInstance::m_pd_plus,
        &TCharacterInstance::m_skill_plus,
        &TCharacterInstance::m_speed_plus,
        &TCharacterInstance::m_ma_plus,
        &TCharacterInstance::m_md_plus,
        &TCharacterInstance::m_hprecover_plus,
        &TCharacterInstance::m_mprecover_plus,
        &TCharacterInstance::m_attackcount_plus};

//g_Logger.DebugLog((boost::format("Apply %d") % results.size()).str());
    const TOpenFarenData* data = g_GameMaster.GetData();
    existsDeadChara = false;
    bool needupdate = false;
    for(unsigned u = 0; u < results.size(); ++u){
        if(!results[u].m_hit){
            continue;
        }
        const point_t& p = results[u].m_location;
//g_Logger.DebugLog((boost::format("Apply se%d") % results[u].m_systemeffect).str());
        if(results[u].m_systemeffect == SYSTEM_CRASHWALL){
            // ǂ邩ǂEvalSpecial()Ŕ肷B
            m_Map->RemoveMapObj(data, p.x, p.y);
            if(!results[u].m_value){
                continue;
            }
        }
        if(results[u].m_systemeffect == SYSTEM_SUMMON){
            // 
            ApplySummon(results[u], atkchara);
            needupdate = true;
            continue;
        }
        if(results[u].m_systemeffect == SYSTEM_REVIVE){
            // 
            ApplyRevive(results[u]);
            needupdate = true;
            continue;
        }

//g_Logger.DebugLog((boost::format("Apply 1")).str());
        int master = m_MasterLayer.GetValue(p.x, p.y);
        if(master == -1){
            if(results[u].m_special != -1){
                const TSpecial& spe = data->GetSpecial(results[u].m_special);
                OFERRLOG((boost::format("logic error:chara not found (%s) systemeffect=%d location=%d,%d") % spe.m_id % results[u].m_systemeffect % p.x % p.y).str());
            }
            else{
                OFERRLOG((boost::format("logic error:chara not found (no special) systemeffect=%d location=%d,%d") % results[u].m_systemeffect % p.x % p.y).str());
            }
            continue;
        }
        int charaindex = m_CharaLayer.GetValue(p.x, p.y);
        //TCharacterInstance& c = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
        TCharacterInstance& c = (*m_CharaList[master])[charaindex];
        //const TBattleCharacter& charadata = data->GetCharacter(c.m_type);
        // Ԃɒǉ镶񂪂鑮ȂAdΒǉB
        if(results[u].m_special != -1){
            const TSpecial& spe = data->GetSpecial(results[u].m_special);
            if(!spe.m_addstr.empty()){
                std::vector<int>::iterator itr = std::lower_bound(c.m_addstrspe.begin(), c.m_addstrspe.end(), results[u].m_special);
                if(itr == c.m_addstrspe.end() || *itr != results[u].m_special){
                    c.m_addstrspe.insert(itr, results[u].m_special);
                }
            }
        }

        // ʏU`z̃_[WB
        if(results[u].m_systemeffect <= SYSTEM_ABSORB){
            existsDeadChara |= ApplyHPChange(master, charaindex, -results[u].m_value);
            needupdate |= existsDeadChara;
        }

        if(results[u].m_systemeffect >= SYSTEM_POISON && results[u].m_systemeffect <= SYSTEM_DREAM){
            AddSystemEffect(c, results[u].m_systemeffect);
        }
        else if(results[u].m_systemeffect == SYSTEM_DEATH){
//g_Logger.DebugLog((boost::format("Apply death")).str());
            existsDeadChara |= ApplyHPChange(master, charaindex, -999);
            needupdate |= existsDeadChara;
        }
        else if(results[u].m_systemeffect == SYSTEM_HPRECOVER){
            existsDeadChara |= ApplyHPChange(master, charaindex, results[u].m_value);
            needupdate |= existsDeadChara;
        }
        else if(results[u].m_systemeffect == SYSTEM_MPRECOVER){
            c.m_mp += results[u].m_value;
            if(c.m_mp > c.m_mpmax_rank){
                c.m_mp = c.m_mpmax_rank;
            }
        }
        else if(results[u].m_systemeffect >= SYSTEM_PLUS_PA &&
                results[u].m_systemeffect <= SYSTEM_PLUS_ATTACKCOUNT){
            int *p = &(c.*params[results[u].m_systemeffect - SYSTEM_PLUS_PA]);
            if(results[u].m_value > 0){
                // 
                if(*p < results[u].m_value){
                    // 傫łꍇ͑傫B
                    *p = results[u].m_value;
                }
            }
            else{
                // 
                if(*p > results[u].m_value){
                    // 傫łꍇ͑傫B
                    *p = results[u].m_value;
                }
            }
        }
        else if(results[u].m_systemeffect == SYSTEM_CHANGE_RESISTANCE){
            if(results[u].m_special != -1){
                const TSpecial& spe = data->GetSpecial(results[u].m_special);
                c.m_resistance[spe.m_resistanceattr] = spe.m_resistancetype;
            }
            else{
                g_Logger.ErrorLog("TBattleMapLogic::ApplyEffectResult logic error:ChangeRegistance parameter not enough");
            }
        }
        else if(results[u].m_systemeffect == SYSTEM_CHANGE_MOVETYPE){
            if(results[u].m_special != -1){
                const TSpecial& spe = data->GetSpecial(results[u].m_special);
                c.m_movetype = spe.m_movetype;
            }
            else{
                g_Logger.ErrorLog("TBattleMapLogic::ApplyEffectResult logic error:ChangeMoveType parameter not enough");
            }
        }
        else if(results[u].m_systemeffect >= SYSTEM_CURE_POISON &&
                results[u].m_systemeffect <= SYSTEM_CURE_DREAM){
            // SYSTEM_POISON`SYSTEM_CURE_POISON`̕тƂB
            RemoveSystemEffect(c, static_cast<TSystemEffect>(results[u].m_systemeffect - SYSTEM_CURE_POISON + SYSTEM_POISON));
        }
        else if(results[u].m_systemeffect == SYSTEM_CURE_ALL){
            // ُԎ
            for(int i=SYSTEM_POISON; i<=SYSTEM_DREAM; ++i){
                RemoveSystemEffect(c, static_cast<TSystemEffect>(i));
            }
            // U
            if(c.m_attackcount_plus < 0){
                c.m_attackcount_plus = 0;
            }
        }
    }
    if(needupdate){
        UpdateOnCharaChange();
        //UpdateCommandableRace();
        //UpdateAutoSpecialList();
        //UpdateShowingChara();
    }
    UpdateCharaDetail();
    g_Logger.DebugLog((boost::format("Apply end")).str());
}
//---------------------------------------------------------------------------
void TBattleMapLogic::ApplySummon(const TEffectResult& result, TCharacterInstance* atkchara)
{
    TCharacterInstance& sumchara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];   // ͂Ƃ肵ȂCB
    // L쐬B
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    TCharacterInstance newchara = game->CreateCharaInstance(result.m_summoncharatype, 0);
    newchara.m_location = result.m_location;
    newchara.m_hp = result.m_value;
    // zuB
    m_MasterLayer.GetValue(result.m_location.x, result.m_location.y) = m_ActiveMaster;
    int newcharaindex = (*m_CharaList[m_ActiveMaster]).size();
    m_CharaLayer.GetValue(result.m_location.x, result.m_location.y) = newcharaindex;
    // ԐݒB
    atkchara->m_elementstatus = TCharacterInstance::ELEMENT_SUMMON;
    atkchara->m_elementindex = newcharaindex;
    newchara.m_elementstatus = TCharacterInstance::ELEMENT_SUMMONED;
    newchara.m_elementindex = m_ActiveChara;    // ͂Ƃ肵ȂCB
    (*m_CharaList[m_ActiveMaster]).push_back(newchara);
}
//---------------------------------------------------------------------------
void TBattleMapLogic::ApplyRevive(const TEffectResult& result)
{
    // SXg폜Bΐ͂B
    m_DeadList[result.m_revivecharamaster].erase(std::find(m_DeadList[result.m_revivecharamaster].begin(), m_DeadList[result.m_revivecharamaster].end(), result.m_revivecharaindex));
    TCharacterInstance& revchara = (*m_CharaList[result.m_revivecharamaster])[result.m_revivecharaindex];
    if(revchara.m_type == result.m_revivecharatype){
        revchara.m_hp = result.m_value;
        m_MasterLayer.GetValue(result.m_location.x, result.m_location.y) = result.m_revivecharamaster;
        m_CharaLayer.GetValue(result.m_location.x, result.m_location.y) = result.m_revivecharaindex;
    }
    else{
        const TOpenFarenGame* game = g_GameMaster.GetGame();
        TCharacterInstance newchara = game->CreateCharaInstance(result.m_revivecharatype, 0);
        newchara.m_location = result.m_location;
        m_MasterLayer.GetValue(result.m_location.x, result.m_location.y) = m_ActiveMaster;
        m_CharaLayer.GetValue(result.m_location.x, result.m_location.y) = (*m_CharaList[m_ActiveMaster]).size();
        (*m_CharaList[m_ActiveMaster]).push_back(newchara);
    }
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::ApplyHPChange(int masterindex, int charaindex, int dhp)
{
    bool needupdate = false;
    TCharacterInstance& c = (*m_CharaList[masterindex])[charaindex];
    c.m_hp += dhp;
    if(c.m_hp <= 0){
        c.m_hp = 0;
        m_MasterLayer.GetValue(c.m_location.x, c.m_location.y) = -1;
        m_KillList.push_back(&c);
        m_DeadList[masterindex].push_back(charaindex);
        // Gg̏ꍇ͏҂̏ԂNAB
        if(c.m_elementstatus == TCharacterInstance::ELEMENT_SUMMONED){
            (*m_CharaList[masterindex])[c.m_elementindex].m_elementstatus = TCharacterInstance::ELEMENT_NONE;
        }
        // KEZB
        ActivateAutoSpecial(masterindex, charaindex);
        needupdate = true;
    }
    else if(c.m_hp > c.m_hpmax_rank){
        c.m_hp = c.m_hpmax_rank;
    }
    return needupdate;
}
//---------------------------------------------------------------------------
void TBattleMapLogic::AddExperience(TCharacterInstance& c, bool actioned, std::vector<const TCharacterInstance*> killlist)
{
    int exp = EvalExperience(c, actioned, killlist);
    c.m_experience += exp;
{
const TOpenFarenData* data = g_GameMaster.GetData();
const TBattleCharacter& charadata = data->GetCharacter(c.m_type);
g_Logger.DebugLog((boost::format("AddExperience chara=%s %d") % charadata.m_name % exp).str());
}
    const TOpenFarenData* data = g_GameMaster.GetData();
    // jZ
    c.m_wincount += killlist.size();
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    if(game->CheckRankup(c)){
        UpdateOnCharaChange();
    }
    // Gg̏ꍇ͏҂ɂolB
    if(c.m_elementstatus == TCharacterInstance::ELEMENT_SUMMONED){
        AddExperience((*m_CharaList[m_ActiveMaster])[c.m_elementindex], actioned, killlist);
    }
}
//---------------------------------------------------------------------------
// L̎ށEۂω̍XVB
void TBattleMapLogic::UpdateOnCharaChange()
{
    UpdateCommandableRace();
    UpdateAutoSpecialList();
    UpdateShowingChara();
}
//---------------------------------------------------------------------------
// Gɓ܂ꂽ㕨菜B
void TBattleMapLogic::UpdateMapObjStepped()
{
    if(m_ActiveMaster){
        const TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
        if(m_Map->GetMapObjType(chara.m_location.x, chara.m_location.y) == MAPOBJ_CITY){
            m_Map->RemoveMapObj(g_GameMaster.GetData(), chara.m_location.x, chara.m_location.y);
        }
    }
}
//---------------------------------------------------------------------------
// 퓬BĂwinmasterɏ}X^[̃CfbNXtrueԂBҖ-1B
bool TBattleMapLogic::JudgeGame(int& winmaster) const
{
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TOpenFarenGame* game = g_GameMaster.GetGame();
    // cL̔B
    std::deque<bool> exists(m_CharaList.size(), false);
    for(int master=0; master<m_CharaList.size(); ++master){
        int masinsidx = m_MasterIndex[master];
        for(unsigned u=0; u<m_CharaList[master]->size(); ++u){
            const TCharacterInstance& chara = (*m_CharaList[master])[u];
            if(masinsidx != -1){
                int mastype = game->GetMasterInstance(masinsidx).m_type;
                const TMaster& masterdata = data->GetMaster(mastype);
                if(chara.m_type == masterdata.m_facecharaindex){
                    if(!chara.m_hp){
                        break;
                    }
                }
            }
            if(chara.m_hp && !(chara.m_runaway)){
                exists[master] = true;
                break;
            }
        }
    }
    // c}X^[̔B
    int cnt = std::count(exists.begin(), exists.end(), true);
    if(cnt <= 1){
        if(cnt == 1){
            winmaster = std::distance(exists.begin(), std::find(exists.begin(), exists.end(), true));
        }
        else{
            winmaster = -1;
        }
        return true;
    }
    return false;
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::CanRunAway(int master, int charaindex, int movex, int movey) const
{
    const TCharacterInstance& chara = (*m_CharaList[master])[charaindex];
    // ZOC̒ƓsB
    if(InEnemyZoc(master, movex, movey)){
        return false;
    }
    // UȂZOC̒łȂΓB
    if(master){
        return true;
    }
    return m_SaveRunAwayCount > 0;
}
//---------------------------------------------------------------------------
// 퓬̐ߖڂLuaR[obN֐ĂԁB
void TBattleMapLogic::CallBattleProgress()
{
    for(unsigned u=0; u<m_CharaList.size(); ++u){
        m_LuaBattleCom[u].CallFunction("on_battle_progress");
    }
}
//---------------------------------------------------------------------------
// rŊϐ풆~̂߁ALs̐ߖڂőxXVB
void TBattleMapLogic::UpdateBattleSpeed()
{
    if(m_NoWatch){
        m_AnimationSpeed = TFarenGameMaster::BATTLESPEED_MIN;
    }
    else{
        if(CanNoWatch() && g_Ini.GetBoolValue(TOpenFarenIni::WATCH_MAXSPEED)){
            m_AnimationSpeed = TFarenGameMaster::BATTLESPEED_MIN;
        }
        else{
            m_AnimationSpeed = g_Ini.GetIntValue(TOpenFarenIni::BATTLE_ANIMSPEED);
        }
    }
    m_BattleMapWindow->GetBattleMapPane().SetAnimationSpeed(m_AnimationSpeed);

    m_AnimationPerChip = m_AnimationSpeed;
    static const int mvframe[TFarenGameMaster::BATTLESPEED_MAX] = {1, 2, 3, 4, 8};
    m_MoveFramePerChip = mvframe[m_AnimationSpeed - 1];
    static const int atkframe[TFarenGameMaster::BATTLESPEED_MAX] = {2, 4, 8, 16, 32};
    m_AttackFrameMax = atkframe[m_AnimationSpeed - 1];
}
//---------------------------------------------------------------------------
void TBattleMapLogic::ShowBattleCommand()
{
    m_BattleCommandWindow->Show();
    g_GameMaster.ClearKeyQueue();
    //g_GameMaster.SetKeyQueueEnabled(true);
}
//---------------------------------------------------------------------------
void TBattleMapLogic::HideBattleCommand()
{
    m_BattleCommandWindow->setVisible(false);
    //g_GameMaster.SetKeyQueueEnabled(false);
}
//---------------------------------------------------------------------------
void TBattleMapLogic::PlayBattleMusic()
{
    if(!g_Ini.GetBoolValue(TOpenFarenIni::MUSIC_ENABLED)){
        return;
    }
    g_GameMaster.GetMixer()->FadeOutMusic();
    const TOpenFarenData* data = g_GameMaster.GetData();
    const TOpenFarenGame* game = g_GameMaster.GetGame();

    // }X^[퓬y̔B}X^[ꍇUDB
    // łɐ͂WvB
    int force = 0;
    int masterbattle = -1;
    for(unsigned master=0; master<m_CharaList.size(); ++master){
        if(m_MasterIndex[master] == -1){
            continue;
        }
        const TMasterInstance& masterins = game->GetMasterInstance(m_MasterIndex[master]);
        const TMaster& masterdata = data->GetMaster(masterins.m_type);
        for(unsigned chr=0; chr<m_CharaList[master]->size(); ++chr){
            TCharacterInstance & ci = (*m_CharaList[master])[chr];
            if(ci.m_type == masterdata.m_facecharaindex){
                masterbattle = masterdata.m_masterbattlemusic;
            }
            force += (ci.m_power * ci.m_hp) / ci.m_hpmax_rank;
        }
    }
    if(masterbattle != -1){
        g_GameMaster.GetMixer()->PlayMusic(masterbattle);
        return;
    }

    // wc퓬y̔B
    int sidebattle = -1;
    for(unsigned master=0; master<m_CharaList.size(); ++master){
        if(m_MasterIndex[master] == -1){
            continue;
        }
        const TMasterInstance& masterins = game->GetMasterInstance(m_MasterIndex[master]);
        const TMaster& masterdata = data->GetMaster(masterins.m_type);
        sidebattle = masterdata.m_battlemusic;
    }
    if(sidebattle != -1){
        g_GameMaster.GetMixer()->PlayMusic(sidebattle);
        return;
    }

    // ͂傫Ƃ̉y̔Bu͂傫v̔Ȃ̂œKɁB
    if(force >= 10000){
        g_GameMaster.GetMixer()->PlayPermanentMusic(TOpenFarenMixer::MUSIC_DECISIVE);
        return;
    }

    // GAy΍ĐB
    const TAreaInstance& area = game->GetAreaInstance(m_AreaIndex);
    if(area.m_music != -1){
        g_GameMaster.GetMixer()->PlayMusic(area.m_music);
    }
}
//---------------------------------------------------------------------------
// AIpBSĂ̈ړ^Cvɑ΂݂̃}bv̈ړRXgv߂B
void TBattleMapLogic::InitMoveTypeCostSkill()
{
    const TOpenFarenData* data = g_GameMaster.GetData();
    std::vector<int> costlist, skilllist;
    int mtcount = data->GetMoveTypeCount();
    costlist.resize(mtcount);
    skilllist.resize(mtcount);
    const TBattleMap* bmap = m_Map;
    int w = bmap->GetWidth();
    int h = bmap->GetHeight();
    for(int m=0; m<mtcount; ++m){
        int& cost = costlist[m];
        int& skill = skilllist[m];
        cost = 0;
        skill = 0;
        const std::vector<int>& costdata = data->GetMoveType(m).m_cost;
        const std::vector<int>& skilldata = data->GetMoveType(m).m_skill;
        for(int y=0; y<h; ++y){
            for(int x=0; x<w; ++x){
                int maptype = bmap->GetMapType(x, y);
                cost += costdata[maptype];
                skill += skilldata[maptype];
            }
        }
    }
    m_MoveTypeCost = costlist;
    m_MoveTypeSkill = skilllist;
}
//---------------------------------------------------------------------------
// InitMoveTypeCost()ŋ߂f[^n`ωƂɍXVBoldtype͕ωO̒n`^CvB
void TBattleMapLogic::UpdateMoveTypeCostSkill(int x, int y, int oldtype)
{
    const TOpenFarenData* data = g_GameMaster.GetData();
    int mtcount = data->GetMoveTypeCount();
    for(int m=0; m<mtcount; ++m){
        const std::vector<int>& costdata = data->GetMoveType(m).m_cost;
        const std::vector<int>& skilldata = data->GetMoveType(m).m_skill;
        int maptype = m_Map->GetMapType(x, y);
        m_MoveTypeCost[m] += costdata[maptype] - costdata[oldtype];
        m_MoveTypeSkill[m] += skilldata[maptype] - skilldata[oldtype];
    }
}
//---------------------------------------------------------------------------
// O̍sĎsR}hōĎsł邩𔻒B
bool TBattleMapLogic::IsAllowedRedoAction(bool moved, TActionType acttype, int special) const
{
    // @ƕKEZ̍ĎspB
    if(acttype != ACTION_SPECIAL){
        return false;
    }
    const TCharacterInstance& chara = (*m_CharaList[m_ActiveMaster])[m_ActiveChara];
    const TOpenFarenData* data = g_GameMaster.GetData();
    // gp񐔐L薂@`FbNB
    std::map<int, int>::const_iterator itr = m_SpecialMagicMap.find(special);
    if(itr != m_SpecialMagicMap.end()){
        int mdindex = itr->second;
        const TMagicData& magicdata = data->GetMagicData(mdindex);
        if(magicdata.m_limit && m_MagicUseCount[m_ActiveMaster][mdindex] >= magicdata.m_limit){
            return false;
        }
    }

    // MP`FbNB
    if(acttype == ACTION_SPECIAL){
        const TSpecial& spe = data->GetSpecial(special);
        if(spe.m_consumemp > chara.m_mp){
            return false;
        }
    }
    // Ăꍇ@͕sB
    const TBattleCharacter& charadata = data->GetCharacter(chara.m_type);
    //if(chara.m_location != point_t(movex, movey)){
    if(moved){
        if(special != charadata.m_specialindex){
            return false;
        }
    }
    return true;
}
//---------------------------------------------------------------------------
bool TBattleMapLogic::IsValidXY(int x, int y) const
{
    if(y < 0 || y >= m_Map->GetHeight()){
        return false;
    }
    if(y % 2){
        if(x < 0 || x >= m_Map->GetWidth() - 1){
            return false;
        }
    }
    else{
        if(x < 1 || x >= m_Map->GetWidth()){
            return false;
        }
    }
    return true;
}
//---------------------------------------------------------------------------
